<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.imageDatabase.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-mediaManager-api
 * @version     4.x
 */

/**
 * This class is a wrapper used to access image data
 * stored in database.
 */
class cmfImageDatabase
{
    /**
     * The MySQLi connection to the image database. 
     * @var mysqli
     */
    private $connection;

    /**
     * The name of the image table.
     * @var string
     */
    private $table;

    /**
     * Public constructor.
     * @param mysqli $connection The MySQLi connection to
     * the image database. 
     * @param string $table The name of the image table.
     */
    public function __construct(mysqli $connection, $table)
    {
        $this->connection = $connection;
        $this->table = $table;
    }

    /**
     * Add a new (or update an existing) image in the database.
     * @param int $id The ID of the image.
     * @param array $formats The formats information
     * about the image.
     * @return boolean True if the insert operation was
     * successfull, false otherwise.
     */
    public function add($id, $formats)
    {
        $sql = 'INSERT INTO `' . $this->table . '` (`id`, `formats`) VALUES(?,?) ON DUPLICATE KEY UPDATE `formats`=?';
        $stmt = $this->connection->prepare($sql);
        if (mysqli_error($this->connection))
        {
            throw new Exception('In ' . __CLASS__  . '::' . __FUNCTION__ . ', line ' . __LINE__ . '   : ' .  mysqli_error($this->connection));
        }    

        $serializedformats = serialize($formats);
        $stmt->bind_param('iss', $id, $serializedformats, $serializedformats);

        $result = $stmt->execute();
        if ($stmt->error)
        {
            throw new cmfMediaManagerDatabaseException('In ' . __CLASS__  . '::' . __FUNCTION__ . '  : ' . $stmt->error);
        }
        return $result;
    }

    /**
     * Get the formats information about a given image.
     * @param int $id The ID of the image.
     * @return array The format information, or 
     * null if failed.
     */
    public function getFormat($id)
    {
        $sql = 'SELECT `formats` FROM `' . $this->table  . '` WHERE `id`=?';

        $stmt = $this->connection->prepare($sql);
        if (mysqli_error($this->connection))
        {
            throw new Exception('In ' . __CLASS__  . '::' . __FUNCTION__ . ', line ' . __LINE__ . '   : ' .  mysqli_error($this->connection));
        }    


        $stmt->bind_param('i', $id);
        $stmt->execute();
        mysqli_stmt_bind_result($stmt, $formats);
        $stmt->fetch();

        if (null != $formats)
        {
            return unserialize($formats);
        }
        return null;
    }

    /**
     * Checks if an image exists in database.
     * @param int $id  The image ID.
     * @return boolean True if the image exists,
     * false otherwise.
     */
    public function exists($id)
    {
        $sql = 'SELECT count(*) FROM `' . $this->table  . '` WHERE `id`=?';

        $stmt = $this->connection->prepare($sql);
        if (mysqli_error($this->connection))
        {
            throw new Exception('In ' . __CLASS__  . '::' . __FUNCTION__ . ', line ' . __LINE__ . '   : ' .  mysqli_error($this->connection));
        }    


        $stmt->bind_param('i', $id);

        $stmt->execute();
        mysqli_stmt_bind_result($stmt, $id);
        $stmt->fetch();

        return (boolean)$id;
    }

    /**
     * Delete an image from the database.
     * @param int $id The image ID.
     * @return boolean True if the delete operation was
     * successfull, false otherwise.
     */
    public function delete($id)
    {
        $sql = 'DELETE FROM `' . $this->table  . '` WHERE `id`=? LIMIT 1';
        if ($stmt = $this->connection->prepare($sql))
        {
            $stmt->bind_param('i', $id);
            $stmt->execute();
            $result = $stmt->execute();
            if ($stmt->errno)
            {
                throw new InvalidArgumentException('In ' . __CLASS__  . '::' . __FUNCTION__ . '  : ' . $stmt->errno);
            }
            $stmt->close();
            return true;
        }
        else
        {
            if (mysqli_error($this->connection))
            {
                throw new Exception('In ' . __CLASS__  . '::' . __FUNCTION__ . ', line ' . __LINE__ . '   : ' .  mysqli_error($this->connection));
            }    
            $mysqli->close();
            return false;
        }
    }
}
