<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.controller.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * Abstract parent class of all CMF controllers.
 */
abstract class cmfController implements ArrayAccess
{
    /**
     * The generation mode for single generations.
     * @var int
     */
    const GEN_MODE_SINGLE = 0;

    /**
     * The generation mode for multiple generations.
     * @var int
     */
    const GEN_MODE_MULTIPLE = 1;

    /**
     * The WCM generation service.
     * @var cmfWcmGenerationServiceClient
     */
    private $generationService;

    /**
     * The path builder used when computing a WCM block in multiple mode.
     * @var Nstein_Path_Builder
     */
    private $pathBuilder;

    /**
     * The path of the CMF views directory.
     * @var string
     */
    private $viewDirectory;

    /**
     * The path of the CMF blocks directory.
     * @var string
     */
    private $blockDirectory;

    /**
     * The cache system for CMF blocks.
     * @var cmfCache
     */
    private $cmfBlockCache;

    /**
     * The request data.
     * @var array
     */
    private $request = array();

    /**
     * The public name of the controller, as it was
     * requested to the front dispatcher.
     * @var string
     */
    private $name;

    /**
     * The current context.
     * @var array
     */
    private $context;

    /**
     * The mode (normal or preview).
     * @see cmfFrontDispatcher
     * @var int
     */
    private $mode;

    /**
     * Array caching computed property 
     * (lazing mode computation is implemented).
     * @var array
     */
    private $cache = array();

    /**
     * The path of the view to display.
     * @var string
     */
    private $view = null;

    /**
     * Initialize the controller.
     * @param array $request The request data.
     * @param string $name The public name of this 
     * controller, as requested to the front dispatcher.
     * @param array $context The current context.
     * @param int $mode The mode (normal or preview). 
     * Default is cmfFrontDispatcher::MODE_NORMAL.
     * @return void
     */
    public function initialize(array $request, $name, array $context, $mode = cmfFrontDispatcher::MODE_NORMAL)
    {
        $this->request = $request;
        $this->name = $name;
        $this->context = $context;
        $this->mode = $mode;
    }

    /**
     * Inject the dependencies the controller needs
     * through an IoC container.
     * @param cmfContainer $container The container.
     * @return void
     */
    public function injectDependencies(cmfContainer $container)
    {
        $this->generationService = cmfToolbox::getArrayParameter($container, 'mvcGenerationService');
        $this->viewDirectory = cmfToolbox::getArrayParameter($container, 'mvcViewDirectory');
        $this->blockDirectory = cmfToolbox::getArrayParameter($container, 'mvcBlockDirectory');
        $this->cmfBlockCache = cmfToolbox::getArrayParameter($container, 'mvcCmfBlockCache');
        $this->pathBuilder = cmfToolbox::getArrayParameter($container, 'mvcPathBuilder');
    }

    /**
     * Get the WCM generation service.
     * @return cmfWcmGenerationServiceClient
     */
    public function getGenerationService()
    {
        return $this->generationService;
    }

    /**
     * Get the path of the CMF views directory.
     * @return string
     */
    public function getViewDirectory()
    {
        return $this->viewDirectory;
    }

    /**
     * Get the path of the CMF block directory.
     * @return string
     */
    public function getBlockDirectory()
    {
        return $this->blockDirectory;
    }

    /**
     * Get the cache system for CMF blocks.
     * @return cmfCache
     */
    public function getCmfBlockCache()
    {
        return $this->cmfBlockCache;
    }

    /**
     * Get the controller name, as requested to the
     * front dispatcher.
     * @return string
     */
    public function getControllerName()
    {
        return $this->name;
    }

    /**
     * Get the current context.
     * @return array
     */
    public function getContext()
    {
        return $this->context;
    }

    /**
     * Set the current context.
     * @param array $context The new context.
     * @return void
     */
    public function setContext(array $context)
    {
        $this->context = $context;
    }

    /**
     * Get the mode.
     * @return int
     */
    public function getMode()
    {
        return $this->mode;
    }

    /**
     * Get the request data.
     * @return array
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * Load an in-memory block.
     * @param string $block The block to load.
     * @return void
     */
    public function loadBlock($block)
    {
        echo eval('?>' . $block);
    }

    /**
     * Compute the filepath of a block.
     * @param string $blockName The internal block name.
     * @return string The absolute filepath of the block.
     */
    protected function computeBlockPath($blockName)
    {
        if (cmfToolbox::isAbsolutePath($blockName))
        {
            return $blockName;
        }

        return $this->blockDirectory . $blockName;
    }

    /**
     * Compute the file path where will be cached a given widget.
     * @param string $widgetName The name of the widget.
     * @param array $extraContext The extra context.
     * @return string The file path.
     */
    /*private function computeWcmWidgetFilePath($widgetName, array $extraContext = array())
      {
      return $this->widgetDirectory . $widgetName . DIRECTORY_SEPARATOR . md5(implode('_', $extraContext));
      }*/

    /**
     * Load a specific block, regenerate block if required (ttl has expired)
     * @param $blockName String Name of block to regenerate
     * @param $ttl Int Time to live
     * @param $genContentCode String Code of the generation content to execute
     * @return void
     */
    /*public function loadWcmWidget($widgetName, $guid, $ttl = 0, $extraContext = array())
      {
    // Enable timer for stats
    $time = microtime(true);
    $gen = false;

    $context = array_merge(
    $this->getContext(), 
    $extraContext
    );

    $filePath = $this->computeWcmWidgetFilePath($widgetName, $extraContext);

    // Regen the block if the preview mode is enabled or if the ttl of the
    // block has expired
    $cmfMode = $this->getMode();

    // Handle different widget mode based on the cmf Mode
    $widgetMode = $cmfMode == cmfFrontDispatcher::MODE_ICE ? 7 : 4;

    $hasExpired = false;
    if (cmfFrontDispatcher::MODE_PREVIEW <= $cmfMode || $hasExpired = ($ttl > 0 && file_exists($filePath) && (filemtime($filePath) + $ttl <= $_SERVER['REQUEST_TIME'])))
    {
    $result = $this->generationService->generateWidget(
    $widgetName, 
    $guid, 
    $widgetMode, 
    $context, 
    $cmfMode & cmfFrontDispatcher::MODE_PREVIEW
    );

    $gen = true;
    }

    // Preview mode
    if (cmfFrontDispatcher::MODE_PREVIEW <= $cmfMode)
    {
    $this->loadBlock($result);
    }
    // Expired content
    elseif ($hasExpired)
    {
    file_put_contents($filePath, $result);
    require($filePath);
    }
    // Load current block
    else
    {
    require($filePath);
    }

    // End timer and save result for stats
    cmsDebugBlocks::add(
    $widgetName,
    array(
    'time' => microtime(true) - $time,
    'cached' => $ttl,
    'regen' => $gen,
    'url' => ''
    )
    );
    }*/

    /**
     * Compute the internal name of a WCM block.
     * @param string $blockName The public name of the block.
     * @param int $genMode The generation mode (single or multiple).
     * @param array $context The context.
     * @return string The internal name of the block.
     */
    public function computeWcmBlockName($blockName, $genMode, array $context)
    {
        if (self::GEN_MODE_MULTIPLE === $genMode)
        {
            $blockName .=  '_' . $context['id'] . '.php';
            if (null !== $this->pathBuilder)
            {
                $pos = strrpos($blockName, DIRECTORY_SEPARATOR);
                $blockName  = substr($blockName, 0, $pos) . DIRECTORY_SEPARATOR . $this->pathBuilder->generatePath($context['id']) . substr($blockName, $pos);
            }
        }
        elseif (self::GEN_MODE_SINGLE === $genMode)
        {
            $blockName .= '.php';
        }
        return $blockName;
    }

    /**
     * Generate a WCM block.
     * @param string $genContentCode The code of the generation content.
     * @param int $cmfMode The CMF mode.
     * @param array $context The generation context.
     * @return string The result of the generation.
     * @exception Exception If an error occurs while
     * executing the generation.
     */
    public function generateWcmBlock($genContentCode, $cmfMode, array $context = array(), $genMode = self::GEN_MODE_SINGLE)
    {

        if (!$this->generationService->isLoggedIn())
        {
            if (!$this->generationService->login())
            {
                throw new Exception();
            }
        }

        $result = $this->generationService->executeGenerationContentByCode(
            $genContentCode, 
            $context, 
            $cmfMode,
            true
        );

        $content = null;
        if ($result)
        {
            if (self::GEN_MODE_MULTIPLE == $genMode)
            {
                if (isset($result[0]['content']))
                {
                    $content = $result[0]['content'];
                }
            }
            elseif (self::GEN_MODE_SINGLE == $genMode)
            {
                if (isset($result['content']))
                {
                    $content = $result['content'];
                }
            }
        }

        return $content;
    }

    /**
     * Load a specific block, regenerate block if required (if ttl has expired)
     * @param string $blockName The name of the WCM block.
     * @param int $ttl The time to live.
     * @param string $genContentCode The code of the generation content to 
     * execute if the block needs to be regenerated.
     * @param int $genMode The mode of the generation content
     * (single or multiple).
     * @param array $extraContext Some extra context that will be sent
     * to the generation. If keys in the extra context also exist in the
     * current controller context, the values from the extra context will
     * override the controller's context values.
     * @return boolean True if the block was loaded correctly, false if the
     * block doesn't exist.
     */
    public function loadWcmBlock($blockName, $ttl = 0, $genContentCode = null, $genMode = self::GEN_MODE_SINGLE, $extraContext = array())
    {
        // Start timer
        $time = microtime(true);
        $gen = false;

        $context = array_merge(
            $this->getContext(), 
            $extraContext
        );

        // Compute folder & block path
        $blockName = $this->computeWcmBlockName($blockName, $genMode, $context);
        $filePath = $this->computeBlockPath($blockName);

        // Regen the block if the preview mode is enabled or if the ttl of the
        // block has expired
        $cmfMode = $this->getMode();
        $hasExpired = false;
        $exists = file_exists($filePath);

        if (($cmfMode & cmfFrontDispatcher::MODE_PREVIEW) || $hasExpired = ($ttl > 0 && $exists && (filemtime($filePath) + $ttl <= $_SERVER['REQUEST_TIME'])))
        {
            $result = $this->generateWcmBlock($genContentCode, $cmfMode, $context, $genMode);
            $gen = true;
        }

        if (cmfFrontDispatcher::MODE_PREVIEW & $cmfMode)
        {
            // Preview mode.
            if (null !== $result)
            {
                $this->loadBlock($result);
            }
        }
        elseif ($hasExpired)
        {
            // Expired content.
            if (null !== $result)
            {
                file_put_contents($filePath, $result);
            }
            else
            {
                // Unable to generate, so refresh the block's TTLt.
                touch($filePath);
            }
            require($filePath);
        }
        elseif ($exists)
        {
            // Load current block.
            require($filePath);
        }
        else
        {
            // Block doesn't exist, return false.
            return false;
        }

        // End timer and save result for stats.
        cmsDebugBlocks::add(
            $blockName,
            array(
                'time' => microtime(true) - $time,
                'cached' => $ttl,
                'regen' => $gen,
                'url' => '?' . http_build_query(
                    array(
                        'controllerName' => 'debug',
                        'action' => 'loadWcmBlock',
                        'code' => $genContentCode,
                        'mode' => $genMode ,
                        'context' => $context
                    )
                )
            )
        );
        
        return true;
    }

    /**
     * Load a CMF block.
     * @param string $blockName The name of the CMF block.
     * @param int $ttl The time to live.
     * @param cmfCache $cache The cache system to use.
     * @return void
     */
    public function loadCmfBlock($blockName, $ttl = 0, cmfCache $cache = null)
    {
        $time = microtime(true);
        $gen  = false;

        $filePath = $this->computeBlockPath($blockName . '.php');

        if (!$ttl || ($this->getMode() & cmfFrontDispatcher::MODE_PREVIEW))
        {
            $gen = true;
            require($filePath);
        }
        else 
        {
            if (null === $cache)
            {
                $cache = $this->cmfBlockCache;
            }

            $content = $cache->get($filePath, $ttl);
            if (!$content)
            {
                $gen = true;
                ob_start();
                require($filePath);
                $content = ob_get_contents();
                ob_end_clean();
                $cache->set($filePath, $content, $ttl);
            }

            echo $content;
        }

        cmsDebugBlocks::add(
            $blockName . ' (Cmf Block)', array( 
                'time' => microtime(true) - $time,
                'cached' => $ttl,
                'regen' => $gen,
                'url' => '?' . http_build_query(
                    array(
                        'controllerName' => 'debug',
                        'action' => 'loadCmfBlock',
                        'blockName' => $blockName
                    )
                )
            )
        );               
    }

    /**
     * Execute an action and render the view.
     * It implements a hook mechanism allowing to execute 
     * code before and after an action is performed.
     * The before{$action} must be implemented (e.g. 
     * beforeSave) should be implemented to execute an 
     * action before. The after{$action} must be implemented 
     * (e.g. afterSave) should be implemented to execute an 
     * action after (take note the first character of the action 
     * is upperCase).
     * @param string $action The action to perform.
     * @return void
     */
    public final function execute($action)
    {
        $methodSuffix = ucfirst($action);

        if (!method_exists($this, 'on' . $methodSuffix))
        {
            $this->before();
            $this->on();
            $this->after();
        }
        else
        {
            // Check if the before{$action} method exists
            $methodName = 'before' . $methodSuffix;
            if (method_exists($this, $methodName))
            {
                $this->$methodName();
            }

            $methodName = 'on' . $methodSuffix;
            $this->$methodName();

            // Check if the after{$action} method exists
            $methodName = 'after' . $methodSuffix;
            if (method_exists($this, $methodName))
            {
                $this->$methodName();
            }
        }
    }

    /**
     * Load and display the appropriate view.
     * @return boolean True if the current view
     * could be rendered, false otherwise.
     */
    public function renderView()
    {
        $view = $this->getViewFilename();
        if ($view)
        {
            include($view);
            return true;
        }
        return false;
    }


    /**
     * Get the absolute filepath of the current view.
     * @return string The absolute filepath of the
     * current view, or null if the current view is
     * invalid. 
     */
    public function getViewFilename()
    {
        if (null !== $this->view)
        {
            // Append the CMF_VIEW_DIR constant if the view has a relative path
            $viewFilename = cmfToolbox::isAbsolutePath($this->view) 
                ? $this->view 
                : $this->viewDirectory . $this->view;
        }
        else
        {
            $viewFilename = $this->viewDirectory . $this->getControllerName() . '.php';
        }

        if (file_exists($viewFilename))
        {
            return $viewFilename;
        }

        return null;
    }

    /**
     * Get the current view.
     * @return string The view file.
     */
    public function getView()
    {
        return $this->view;
    }

    /**
     * Set the current view.
     * @param string $view The view file.
     * @return void
     */
    public function setView($view)
    {
        $this->view = $view;
    }

    /**
     * This method is always called at the beginning
     * of the dispatch operation.
     * @return void
     */
    public function always()
    {
    }

    /**
     * This method is called when executing an
     * unknown action.
     * @return void
     */
    public function on()
    {
    }

    /**
     * This method is called before executing
     * an unknown action.
     * @return void
     */
    public function before()
    {
    }

    /**
     * This method is called after executing an
     * unknown action.
     * @return void
     */
    public function after()
    {
    }

    /**
     * Implements ArrayAccess
     *
     * @param $key String Key name
     */
    public function offsetGet($key)
    {
        if (array_key_exists($key, $this->cache))
        {
            return $this->cache[$key];
        }
        elseif (property_exists($this, $key))
        {
            return $this->$key;
        }
        elseif (method_exists($this, $methodName = 'get' . ucfirst($key)))
        {
            $this->cache[$key] = $this->$methodName();
            return $this->cache[$key];
        }
        else
        {
            trigger_error('Undefined index: ' . $key, E_USER_NOTICE);
            return;
        }
    }

    /**
     * Implements ArrayAccess
     *
     * @param String $key Key name
     * @param String $value Value to assign
     */
    public function offsetSet($key, $value)
    {
        $this->cache[$key] = $value;
    }

    /**
     * Implements ArrayAccess
     *
     * @param String $key Key name
     */
    public function offsetUnset($key)
    {
        $this->cache[$key] = null;
    }

    /**
     * Implements ArrayAcces
     *
     * @param String $key Key name
     * @return Boolean True if the key exist false otherwise
     */
    public function offsetExists($key)
    {
        return (property_exists($this, $key) || array_key_exists($key, $this->cache) || method_exists($this, 'get' . ucfirst($key)));
    }
}
