<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.searchQuery.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * This is a search query.
 */
class cmfSearchQuery
{
    /**
     * The prefix for the applied facets session key.
     * @var string
     */
    const APPLIED_FACETS_PREFIX = 'facets_';

    /**
     * The cmdSearchConfig.
     * @var cmfSearchConfig
     */
    public $configuration;

    /**
     * The ID of the search.
     * @var string
     */
    public $searchId;

    /**
     * The current lucene query.
     * @var string
     */
    public $queryString;

    /**
     * The sort clause (uses the SQL order by syntax).
     * @var string
     */
    public $sortClause;

    /**
     * Public constructor. Should be provided with a query string and a sort clause.
     * 
     * @param cmfSearchConfig $config The search configuration.
     * @param string $searchId The prefix for the search ID.
     * @param string $queryString The lucene query string.
     * @param string $sortClause The sort clause (follows the SQL order by syntax).
     */
    public function __construct(cmfSearchConfig $config, $searchId, $queryString, $sortClause)
    {
        $this->configuration = $config;
        $this->searchId = $searchId;
        $this->queryString = $queryString;
        $this->sortClause = $sortClause;
    }

    /************************************
     * Applied facets handling methods. *
     ************************************/

    /**
     * Get the session key for the applied facet values.
     * @return string
     */
    private function getAppliedFacetSessionKey()
    {
        return self::APPLIED_FACETS_PREFIX . $this->searchId;
    }

    /**
     * Return the names of all facets currently applied to the search results.
     * @return array
     */
    public function getAppliedFacets()
    {
        cmfToolbox::ensureSessionStarted();
        return cmfToolbox::getArrayParameter($_SESSION, $this->getAppliedFacetSessionKey(), array());
    }

    /**
     * Return true if at least one facet value is currently applied on the search result;
     * false otherwise.
     * @return bool
     */
    public function areFacetsApplied()
    {
        $appliedFacets = $this->getAppliedFacets();
        return is_array($appliedFacets) && count($appliedFacets) > 0;
    }

    /**
     * Return the currently applied values for a given facet.
     * @param $facet The facet name.
     * @return array
     */
    public function getAppliedFacetValues($facet)
    {
        $facets = $this->getAppliedFacets();
        if (array_key_exists($facet, $facets) && is_array($facets[$facet]))
        {
            return array_keys($facets[$facet]);
        }
        return array();
    }

    /**
     * Return the captions of the currently applied values for a given facet.
     * @param $facet The facet name.
     * @return array
     */
    public function getAppliedFacetCaptions($facet)
    {
        $facets = $this->getAppliedFacets();
        if (array_key_exists($facet, $facets) && is_array($facets[$facet]))
        {
            return $facets[$facet];
        }
        return array();
    }

    /**
     * Return true if a given value is currently applied on the search result
     * for a given facet.
     * @param $facet The name of the facet.
     * @param $value The value of the facet.
     * @return bool
     */
    public function isFacetValueApplied($facet, $value)
    {
        $values = $this->getAppliedFacetValues($facet);
        return in_array($value, $values);
    }

    /**
     * Put a facet value to the facets currently applied to the search results.
     * If the facetInfo is not defined, the operation is canceled.
     * If the facet type is singlevalued, any existing value will be overwritten.
     * If the facet type is multivalued, the value will be added to any existing values.
     * If the facet type is hierarchical, the value will be appended to the list of parent values.
     * @param string $facet The facet name.
     * @param string $value The facet value.
     * @param string $caption The caption of the facet value.
     */
    public function putFacetValue($facet, $value, $caption)
    {
        $facets = $this->getAppliedFacets();

        $facetInfo = $this->configuration->getFacetDefinition($facet);
        if (null !== $facetInfo)
        {
            $values = cmfToolbox::getArrayParameter($facets, $facet, array());

            // Init new facet, delete previous values for single valued facet
            if (cmfFacetDefinition::FACET_TYPE_SINGLE_VALUE == $facetInfo->type)
            {
                $values = array();
            }

            $values[$value] = $caption;
            $facets[$facet] = $values;
        }

        $_SESSION[$this->getAppliedFacetSessionKey()] = $facets;
    }

    /**
     * Remove a facet value from the facets currently applied to the search results.
     * If the facetInfo is not defined, the operation is canceled.
     * If the facet type is singlevalued, the facet is removed.
     * If the facet type is multivalued, the value is removed from the facet values list.
     * If the facet type is hierarchical, the value is removed, along with its children, if any.
     * @param $facet The facet name.
     * @param $value The facet value.
     */
    public function removeFacetValue($facet, $value)
    {
        $facets = $this->getAppliedFacets();

        if (!empty($facets))
        {
            $facetInfo = $this->configuration->getFacetDefinition($facet);
            if (null !== $facetInfo)
            {
                if (isset($facets[$facet][$value]))
                {
                    if (cmfFacetDefinition::FACET_TYPE_HIERARCHICAL == $facetInfo->type)
                    {
                        // Remove facet value and all children values (for hierarchical facets)
                        $index = array_search($value, array_keys($facets[$facet]));
                        if (false !== $index)
                        {
                            $facets[$facet] = array_slice($facets[$facet], 0, $index, true);
                        }
                    }
                    else
                    {
                        unset($facets[$facet][$value]);
                    }

                    // Remove empty facet entries
                    if (empty($facets[$facet]))
                    {
                        unset($facets[$facet]);
                    }
                }
            }

            $_SESSION[$this->getAppliedFacetSessionKey()] = $facets;
        }
    }

    /**
     * Remove all the values of a facet from the facets currently applied to the search results.
     * @param $facet The facet name.
     */
    public function removeFacet($facet)
    {
        $facets = $this->getAppliedFacets();

        if (!empty($facets))
        {
            if (isset($facets[$facet]))
            {
                unset($facets[$facet]);
            }

            $_SESSION[$this->getAppliedFacetSessionKey()] = $facets;
        }
    }

    /**
     * Remove all the facets currently applied to the search results.
     */
    public function clearFacets()
    {
        cmfToolbox::ensureSessionStarted();
        $_SESSION[$this->getAppliedFacetSessionKey()] = array();
    }

    /**
     * Build a query fragments according to the facets currently applied 
     * to the search results.
     * @return string
     */
    public function buildFacetsQueryFragment()
    {
        $fragments = array();

        $facets = $this->getAppliedFacets();
        foreach ($facets as $facet => $values)
        {
            $facetInfo = $this->configuration->getFacetDefinition($facet);

            // Ignore facet values if facetInfo not defined
            if (null !== $facetInfo)
            {
                switch ($facetInfo->type)
                {
                    case cmfFacetDefinition::FACET_TYPE_MULTIPLE_VALUE:
                        // Use all values for criteria
                        $values = array_keys($values);
                        $fragments[] = sprintf('%s:("%s")', $facetInfo->searchIndex, implode('" AND "', $values));
                        break;

                    case cmfFacetDefinition::FACET_TYPE_HIERARCHICAL:
                        // Use only the deepest criteria (as others are its parents)
                        $keys = array_keys($values);
                        $fragments[] = sprintf('%s:"%s"', $facetInfo->searchIndex, $keys[count($keys) - 1]);
                        break;

                    case cmfFacetDefinition::FACET_TYPE_SINGLE_VALUE:
                    default:
                        // Use the first criteria (there shouldn't be more than one value anyway)
                        $keys = array_keys($values);
                        $fragments[] = sprintf('%s:"%s"', $facetInfo->searchIndex, $keys[0]);
                        break;
                }
            }
        }

        return implode(' AND ', $fragments);
    }
}
