<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.solrSearchPlugin.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The solr search plugin.
 */
class cmfSolrSearchPlugin extends cmfSearchPlugin
{
    /**
     * The Solr date format.
     */
    const SOLR_DATE_FORMAT = 'Y-m-d';

    /**
     * The Solr time format.
     */
    const SOLR_TIME_FORMAT = 'H:i:s';

    /**
     * The Solr date-time format.
     */
    const SOLR_DATETIME_FORMAT = 'Y-m-d\TH:i:s\Z';

    /**
     * Regexp matching a Lucene query term field.
     */
    private static $queryTermFieldRegExp = null;

    /**
     * Regexp matching a Lucene query term date.
     */
    private static $queryTermDateRegExp = null;

    /**
     * The Solr service URL.
     * @var string
     */
    protected $solrServiceURL = null;

    /**
     * The Solr service instance.
     * @var Apache_Solr_Service
     */
    protected $solrService = null;

    /**
     * The class name of the cache plugin.
     * @var string
     */
    protected $cachePluginClass = null;

    /**
     * The arguments for the cache plugin.
     * @var array
     */
    protected $cachePluginArguments = null;

    /**
     * The cache plugin instance.
     * @var cmfCache
     */
    protected $cachePlugin = null;

    /**
     * Construct the plugin. The expected parameters are :
     * 	   'indexDefinitions' => the array of cmfIndexDefinition
     *     'facetDefinitions' => the array of cmfFacetDefinition
     *     'solrWebService'   => the URL of the solr web service
     *     'cache'			  => the cache plugin informations
     * @param array $parameters The parameters for the plugin.
     */
    public function __construct(array $parameters)
    {
        parent::__construct($parameters);

        $this->indexDefinitions = cmfToolbox::getArrayParameter($parameters, 'indexDefinitions', null);
        if (null === $this->indexDefinitions)
        {
            throw new Exception('cmfSolrSearchPlugin : argument "indexDefinitions" is required.');
        }

        $solrServiceURL = cmfToolbox::getArrayParameter($parameters, 'solrWebService', null);
        $solrServiceURLParts = parse_url($solrServiceURL);
        $this->solrServiceURL = $solrServiceURL;
        $this->solrService = new Apache_Solr_Service($solrServiceURLParts['host'],
                $solrServiceURLParts['port'],
                $solrServiceURLParts['path']);

        $cachePluginData = cmfToolbox::getArrayParameter($parameters, 'cache', null);
        if ($cachePluginData)
        {
            $this->cachePluginClass = cmfToolbox::getArrayParameter($cachePluginData, 'class', null);
            $this->cachePluginArguments = cmfToolbox::getArrayParameter($cachePluginData, 'arguments', array());
        }
    }

    /**
     * Get the cache instance. If no instance has been created yet,
     * create it.
     * @return cmfCache The cache plugin instance.
     */
    protected function getCache()
    {
        if (null === $this->cachePlugin)
        {
            $class = $this->cachePluginClass;
            $this->cache = new $class($this->cachePluginArguments);
        }
        return $this->cache;
    }

    /**
     * Get the cache key for the native query.
     * @return string
     */
    protected function getNativeQueryCacheKey($searchId)
    {
        cmfToolbox::ensureSessionStarted();
        return $searchId . session_id();
    }

    /**
     * (non-PHPdoc)
     * @see cmf/core/search/plugins/cmfSearchPlugin#loadIndexDefinitions()
     */
    protected function loadIndexDefinitions()
    {
        return null;
    }

    /**
     * Initiates a new search given a unique ID, a query, and sorting
     * criteria.
     *
     * The $query parameter may specify:
     *
     * - a string, in which case its value is expected to conform to
     *   the Lucene query syntax; or
     *
     * - an assoc. array of name-value pairs, in which case each name
     *   is epxected to refer to the name of a search field.
     *
     * The assoc. array form of the query is equivalent to a Lucene
     * query in which each name-value pair forms a term and the terms
     * are joined by the "AND" operator, thus:
     *
     *   name1:value1 AND name2:value2 AND ...
     *
     * If non-null, the $sortingCriteria parameter is expected to
     * follow the SQL "order by" syntax with the default sort order
     * being "ASC", like so:
     *
     * name1 [ASC|DESC], name2 [ASC|DESC], ...
     *
     * @param string      $searchId        The unique ID to identify the search
     * @param string      $query           The Lucene query
     * @param string|null $sortingCriteria The sorting criteria (default is null)
     *
     * @return int|null The number of search results, or null on failure
     */
    public function initSearch($searchId, $query, $filterQuery = null, $sortingCriteria = null)
    {
        // escape lucene query caracters (some are missing probably)
        //$query = addcslashes($query, '*[]&|^():{}');

        $result = $this->initQuery($query, $filterQuery, $sortingCriteria);
        if (null === $result)
        {
            return null;
        }

        list($nativeQuery, $query) = $result;

        $this->getCache()->set($this->getNativeQueryCacheKey($searchId), $nativeQuery);

        try
        {
            $result = $this->getSolrService()->search($nativeQuery['query'], 0, 0, $nativeQuery['queryParams']);
        }
        catch (Exception $error)
        {
            $this->lastErrorMsg = $error->getMessage();
            return null;
        }

        return $result->response->numFound;
    }

    /**
     * Initializes a native query from a given Lucene-syntax query string.
     *
     * The returned native query is actually an assoc. array as follows:
     *
     *   'query'       => the Lucene query
     *   'queryParams' => the query parameters (eg. for sorting)
     *
     * @param string $queryString A Lucene query string
     * @param string|null $filterQuery A Lucene filter query string
     * @param string|null $sort A sort clause
     * @return array|null The native query and updated query string (in that order), or null on error
     */
    protected function initQuery($queryString, $filterQuery = null, $sort = null)
    {
        // Convert the query string into a native query
        $result = $this->parseQuery($queryString);
        if (null === $result)
        {
            return null;
        }

        list($solrQuery, $query) = $result;

        // Solr does not accept an empty query to mean "everything"
        if (!$solrQuery)
        {
            $solrQuery = '[* TO *]';
        }

        // Convert the filter query string into a native query
        $solrFilter = null;
        if (null !== $filterQuery)
        {
            $result = $this->parseQuery($filterQuery);
            if (null !== $result)
            {
                list($solrFilter, $filterQuery) = $result;
            }
        }

        if (null !== $sort)
        {
            $sort = strtolower($sort);
        }

        $nativeQuery = array();
        $nativeQuery['query'] = $solrQuery;
        $nativeQuery['queryParams'] = array(
                'sort' => $sort,
                'fq' => $solrFilter
                );

        return array($nativeQuery, $query);
    }

    /**
     * Parses a Lucene query or an assoc. array of search parameters
     * to generate a native query.
     *
     * If the $query parameter is null and $parameters is non-empty,
     * generates a Lucene query from any parameters in the $parameters
     * parameter referring to search indexes.
     *
     * @param string|null $queryString The Lucene query string
     *
     * @return array|null The native query and updated query (in that order), or null on error
     */
    protected function parseQuery($queryString)
    {
        // No conversion necessary since a Solr query *is* a Lucene query
        $nativeQuery = $queryString;

        // Noramlize query term fields
        $regExp = $this->getQueryTermFieldRegExp();
        if ('' !== $regExp)
        {
            $callback = array($this, 'normalizeQueryTermField_callback');
            $nativeQuery = preg_replace_callback($regExp, $callback, $nativeQuery);
        }

        // Noramlize query term dates
        $regExp = $this->getQueryTermDateRegExp();
        if ('' !== $regExp)
        {
            $callback = array($this, 'normalizeQueryTermDate_callback');
            $nativeQuery = preg_replace_callback($regExp, $callback, $nativeQuery);
        }

        return array($nativeQuery, $queryString);
    }

    /**
     * Gets a range of documents after a successful call to the
     * initSearch method.
     * @param string $searchId The search ID used in the call to the initSearch method
     * @param int $from The index of the first document to get.
     * @param int $to The index of the last document to get.
     * @return array|null An array of the requested range of documents,
     * formatted as SimpleXMLElements.
     */
    public function getDocumentRange($searchId, $from, $to)
    {
        $nativeQuery = $this->getCache()->get($this->getNativeQueryCacheKey($searchId));
        if ($nativeQuery === false)
        {
            throw new Exception(sprintf('cmfSolrSearchPlugin: Invalid search ID "%s"', $searchId));
        }

        try
        {
            $solrResponse = $this->getSolrService()->search(
                $nativeQuery['query'],
                $from,
                $to - $from + 1,
                $nativeQuery['queryParams']
            );
        }
        catch (Exception $error)
        {
            throw $error;
        }

        $result = array();

        foreach ($solrResponse->response->docs as $document)
        {
            $xml = simplexml_load_string($document->objectXml);
            $result[] = $xml;
        }

        return $result;
    }

    /**
     * Gets the raw facet values and their occurence count for a given
     * facet index.
     * @param string $searchId The reference search ID.
     * @param cmfFacetDefinition $facetInfo The facet definition.
     * @return array An array of raw facet value => occurence count.
     */
    public function getRawFacetValues($searchId, cmfFacetDefinition $facetInfo)
    {
        $facetValues = array();

        $nativeQuery = $this->getCache()->get($this->getNativeQueryCacheKey($searchId));
        if (false === $nativeQuery)
        {
            throw new Exception(sprintf('cmfSolrSearchPlugin : invalid search ID "%s"', $searchId));
        }

        $query = $nativeQuery['query'];

        $queryParams = $nativeQuery['queryParams'];
        $queryParams['facet'] = 'true';
        $queryParams['facet.enum.cache.minDf'] = 1;
        $queryParams['facet.field'] = array($facetInfo->index);
        $queryParams['facet.limit'] = $facetInfo->maxValues;
        $queryParams['facet.mincount'] = 1;
        $queryParams['facet.sort'] = $facetInfo->sort ? 'true' : 'false';

        $solrResponse = $this->getSolrService()->search($query, 0, 0, $queryParams);
        $rawValues = get_object_vars($solrResponse->facet_counts->facet_fields->{$facetInfo->index});

        foreach ($rawValues as $value => $count)
        {
            $facetValues[$value] = new cmfFacetValue($value, $value, $count);
        }

        return $facetValues;
    }

    /**
     * Get the count of occurences for a given raw facet value
     * @param string $searchId The search ID.
     * @param cmfFacetDefinition $facetInfo The definition of the facet.
     * @param string $rawFacetValue The raw facet value.
     * @return int The count of occurences.
     */
    public function getFacetValueCount($searchId, cmfFacetDefinition $facetInfo, $rawFacetValue)
    {
        $facetValues = $this->getRawFacetValues($searchId, $facetInfo->index);
        if (null !== $facetValues && array_key_exists($rawFacetValue, $facetValues))
        {
            return $facetValues[$rawFacetValue]->count;
        }
        return 0;
    }

    /**
     * Gets the Solr service instance.
     *
     * Throws an exception if the service times out.
     *
     * @return Apache_Solr_Service The Solr service instance
     */
    private function getSolrService()
    {
        return $this->solrService;
    }

    /**
     * Gets the regexp matching a Lucene query term field.
     *
     * @return string The regexp
     */
    private function getQueryTermFieldRegExp()
    {
        if (self::$queryTermFieldRegExp === null)
        {
            $fields = array();
            foreach ($this->getIndexDefinitions() as $definition)
            {
                $fields[] = $definition->name;
                foreach ($definition->aliases as $alias)
                {
                    $fields[] = $alias;
                }
            }
            if (!empty($fields))
            {
                self::$queryTermFieldRegExp = '/(' . implode('|', $fields) . ')\\s*:/i';
            }
            else
            {
                self::$queryTermFieldRegExp = '';
            }
        }

        return self::$queryTermFieldRegExp;
    }

    /**
     * Gets the regexp matching a Lucene query term date.
     *
     * @return string The regexp
     */
    private function getQueryTermDateRegExp()
    {
        if (self::$queryTermDateRegExp === null)
        {
            $fields = array();
            foreach ($this->getIndexDefinitions() as $name => $definition)
            {
                if (cmfIndexDefinition::DATE === $definition->type)
                {
                    $fields[] = $name;
                    foreach ($definition->aliases as $alias)
                    {
                        $fields[] = $alias;
                    }
                }
            }

            if (!empty($fields))
            {
                $fieldRegExp = '(' . implode('|', $fields) . ')\\s*:\\s*';
                $dateRegExp = '\[(\\S+)(?:\\s+TO\\s+(\\S+))?\]';

                self::$queryTermDateRegExp = '/' . $fieldRegExp . $dateRegExp . '/i';
            }
            else
            {
                self::$queryTermDateRegExp = '';
            }
        }

        return self::$queryTermDateRegExp;
    }

    /**
     * Callback for the preg_replace_callback function that normalizes
     * a Lucene query term field as defined by the regexp returned by
     * the getQueryTermFieldRegExp method.
     *
     * @param array $matches The current matches
     *
     * @return string The replacement string
     */
    private function normalizeQueryTermField_callback($matches)
    {
        return strtolower($matches[0]);
    }

    /**
     * Callback for the preg_replace_callback function that normalizes
     * a Lucene query term date as defined by the regexp returned by
     * the getQueryTermDateRegExp method.
     *
     * @param array $matches The current matches
     *
     * @return string The replacement string
     */
    private function normalizeQueryTermDate_callback($matches)
    {
        $field    = $matches[1];
        $fromDate = $matches[2];
        $toDate   = isset($matches[3]) ? $matches[3] : null;

        // Generate a "to" date if none
        if (!$toDate)
        {
            // If "from" date is relative, "to" date is "now",
            // otherwise "to" date is equal to "from" date
            if (is_numeric($fromDate) && intval($fromDate) <= 365)
            {
                $toDate = time();
            }
            else
            {
                $toDate = $fromDate;
            }
        }

        // Convert dates to timestamps
        $fromTime = $this->date2time($fromDate);
        $toTime   = $this->date2time($toDate);

        // Swap "from" and "to" dates if reversed
        if ($fromTime > $toTime)
        {
            list($toTime, $fromTime) = array($fromTime, $toTime);
        }

        // Timestamp => date-time
        $fromDate = self::escapeQuery(date(self::SOLR_DATETIME_FORMAT, $fromTime));
        $toDate = self::escapeQuery(date(self::SOLR_DATETIME_FORMAT, $toTime));

        return $field . ':[' . $fromDate . ' TO ' . $toDate . ']';
    }

    /**
     * Converts a given date to a Unix timestamp.
     *
     * @param mixed $date The date to convert
     *
     * @return int The corresponding timestamp
     */
    private function date2time($date)
    {
        $time = null;
        if (!is_numeric($date))
        {
            // Accept m10 for -10 and 2008x09x09 for 2008-09-09 to
            // work around a limitation in the Zend query parser
            $date = preg_replace('/[mx]([0-9]+)/i', '-$1', $date);

            // Alpha-numeric date format
            $time = strtotime($date);
        }
        elseif (intval($date) <= 365)
        {
            // Delta in days
            $time = strtotime($date . ' days');
        }
        else
        {
            // Unix timestamp
            $time = intval($date);
        }

        return $time;
    }
}
