<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.container.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The base class for dependency injection containers.
 * This class handle a level of cache for objects.
 * To declare an object in the container, simply
 * create a builder method for the object. The builder
 * method name must be "build_", followed by the
 * object's name (with all dots, slashes,
 * dashes and so on replaced with an underscore).
 * This method must require no parameter and must 
 * return the object. The builder method will be 
 * called the first time the object is requested, 
 * and the container will cache the resulting object.
 */
class cmfContainer implements ArrayAccess
{
    /**
     * The object cache.
     * @var array
     */
    private $cache = array();
    
    /**
     * Get the builder method name for a given object name.
     * @param string $offset The object name
     * @return string The builder method name.
     */
    private function getBuilderMethod($objectName)
    {
        return 'build' . ucfirst($objectName);
    }
    
    /**
     * Check if a builder method exists for a given
     * object name.
     * @param string $objectName The object name.
     * @return boolean True if a builder method exists
     * for the object name, false otherwise.
     */
    public function isObjectDefined($objectName)
    {
        return method_exists($this, $this->getBuilderMethod($objectName));
    }
    
    /**
     * Get an object from the container.
     * @param string $objectName The name of the object.
     * @return mixed The object.
     * @exception InvalidArgumentException If the provided
     * object name is undefined in the current container.
     */
    public function getObject($objectName)
    {
        // Check if the object has already been built.
        if (!isset($this->cache[$objectName]))
        {
            // Compute the builder method name.
            $builderMethod = $this->getBuilderMethod($objectName);
            if (!method_exists($this, $builderMethod))
            {
                // The method is undefined.
                throw new InvalidArgumentException(
                    'Undefined object: ' . $objectName
                );
            }
            
            // Call the builder method and cache the resulting object.
            $this->cache[$objectName] = $this->$builderMethod();
        }
        
        // Get the object from the cache.
        return $this->cache[$objectName];
    }
    
    /**
     * Set an object inside the container's cache.
     * Watch out, since this method allows to
     * override an object that's declared through
     * a builder method.
     * @param string $name The name of the object.
     * @param mixed $object The object.
     * @return void
     */
    public function setObject($name, $object)
    {
        $this->cache[$name] = $object;
    }
    
    /**
     * Remove an object from the cache. If the
     * object has no builder method (meaning it
     * was inserted through the setObject()
     * method), it will be simply discarded;
     * if the object has a builder method, the
     * cached instance will be cleared, and the
     * builder will be re-called the next time
     * the object is requested.
     * @param string $name The object name.
     * @return void
     */
    public function removeObject($name)
    {
        unset($this->cache[$name]);
    }
    
    /**
     * Clear the object cache. This means that
     * all objects inserted into the container
     * through the setObject() method will
     * be discarded.
     * @return void
     */
    public function clear()
    {
        $this->cache = array();
    }
    
    
    public function offsetExists($offset)
    {
        return $this->isObjectDefined($offset);
    }

    public function offsetGet($offset)
    {
        return $this->getObject($offset);
    }

    public function offsetSet($offset, $value)
    {
        $this->setObject($offset, $value);
    }

    public function offsetUnset($offset)
    {
        $this->removeObject($offset);
    }
}
