<?php
/**
 * Nstein application framework
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

/**
 * Nstein HTTP client.
 *
 * This implementation, based on the Zend_Http_Client class, allows for:
 *
 * - HTTP Digest Access Authentication; and
 *
 * - POST and PUT data files to be specified as file resources, thus avoiding huge files from being
 *   loaded into memory before being sent to the remote server.
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */
class Nstein_Http_Client extends Zend_Http_Client
{
    /**
     * Combines two URIs into one.
     *
     * Each URI can have one of the following values:
     *
     * - Zend_Uri_Http instance. In this case, any component that is not present in the first URI is
     *   taken from the second URI if possible.
     *
     * - URI string. The string is converted into a Zend_Uri_Http instance and the components
     *   selected as in the previous case.
     *
     * - Array of the following form:
     *
     *       <uri> [, <include?> [, <component> ...] ]
     *
     *   where:
     *
     *       <uri>       = <Zend_Uri_Http instance> | <URI string>
     *       <include?>  = true|false
     *       <component> = 'username' | 'password' | 'host' | 'port' | 'path' | 'query' | 'fragment'
     *
     *   The <include?> determines whether to include (true) or exclude (false) the components that
     *   follow. The reverse condition is applied to any unspecified components.
     *
     *   If a component is to be included implicitly or explicitly from both URIs, the component
     *   from the first URI takes precedence.
     *
     *   If a component is to be excluded implicitly or explicitly from both URIs, it is excluded
     *   from the combined URI.
     *
     * @param mixed $uri_1 First URI
     * @param mixed $uri_2 Second URI
     *
     * @return Zend_Uri_Http Combined URI
     */
    public static function combineURIs($uri_1, $uri_2)
    {
        // Extract inclusion/exclusion info for the first URI
        $include_1 = false;
        $components_1 = array();
        if (is_array($uri_1))
            list($uri_1, $include_1, $components_1) = $uri_1;

        $include_1 = (bool) $include_1;
        $components_1 = (array) $components_1;

        // Convert the first URI to a Zend_Uri_Http instance if necessary
        if (!($uri_1 instanceof Zend_Uri_Http))
            $uri_1 = Zend_Uri_Http::factory($uri_1);

        // Extract inclusion/exclusion info for the second URI
        $include_2 = false;
        $components_2 = array();
        if (is_array($uri_2))
            list($uri_2, $include_2, $components_2) = $uri_2;

        $include_2 = (bool) $include_2;
        $components_2 = (array) $components_2;

        // Convert the second URI to a Zend_Uri_Http instance if necessary
        if (!($uri_2 instanceof Zend_Uri_Http))
        {
            // The second URI may be relative
            if (preg_match('!https?://!', $uri_2))
            {
                $uri_2 = Zend_Uri_Http::factory($uri_2);
            }
            else
            {
                $uri_2_tail = $uri_2;
                $uri_2_head = Zend_Uri_Http::factory($uri_1->getScheme());
                $uri_2_head->setHost($uri_1->getHost());
                $uri_2_head->setPort($uri_1->getPort());
                $uri_2 = Zend_Uri_Http::factory((string) $uri_2_head . $uri_2_tail);
            }
        }

        // Instantiate the combined URI
        $uri = Zend_Uri_Http::factory($uri_1->getScheme());

        // Combined components
        $components = array(
            'username' => array('value' => null, 'sep' => null, 'substr' => null),
            'password' => array('value' => null, 'sep' => null, 'substr' => null),
            'host'     => array('value' => null, 'sep' => null, 'substr' => null),
            'port'     => array('value' => null, 'sep' => null, 'substr' => null),
            'path'     => array('value' => null, 'sep' => '/',  'substr' => true),
            'query'    => array('value' => null, 'sep' => '&',  'substr' => false),
            'fragment' => array('value' => null, 'sep' => null, 'substr' => null),
            );

        // Collect (and possibly combine) included components
        foreach ($components as $name => $data)
        {
            $getter = 'get' . ucfirst($name);

            // Get value from first URI
            $value_1 = ($include_1 xor !in_array($name, $components_1))
                ? call_user_func(array($uri_1, $getter)) : null;

            // Get value from second URI
            $value_2 = ($include_2 xor !in_array($name, $components_2))
                ? call_user_func(array($uri_2, $getter)) : null;

            // If both values are non-empty and we have a separator,
            // combine the values into one
            if ($value_1 && $value_2 && ($sep = $data['sep']) !== null)
            {
                if ($data['substr'] && substr($value_2, 0, strlen($value_1)) == $value_1)
                    $value_1 = $value_2;
                else
                    $value_1 = rtrim($value_1, $sep) . $sep . ltrim($value_2, $sep);
            }

            // Pick the first value over the second
            if ($value_1)
                $components[$name]['value'] = $value_1;
            else if ($value_2)
                $components[$name]['value'] = $value_2;
        }

        // Set each non-null component in the combined URI
        foreach ($components as $name => $data)
        {
            $value = $data['value'];
            if ($value !== null)
            {
                $setter = 'set' . ucfirst($name);
                call_user_func(array($uri, $setter), $value);
            }
        }

        return $uri;
    }

    /**
     * Current username (may be null; see {@link setCredentials()})
     *
     * @var string
     */
    private $username;

    /**
     * Current password (may be null; see {@link setCredentials()})
     *
     * @var string
     */
    private $password;

    /**
     * HTTP digest authentication information (may be null; see {@link request()})
     *
     * @var Nstein_Http_Digest
     */
    private $httpDigest;

    /**
     * Sets the user credentials for subsequent requests, clearing any previous HTTP digest
     * authentication information.
     *
     * @param string $username Username (may be null)
     * @param string $password User password (may be null)
     *
     * @return void
     */
    public function setCredentials($username, $password)
    {
        $this->username = $username;
        $this->password = $password;

        $this->httpDigest = null;
    }

    /**
     * @return string The current username, or null if none has been set
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * @return string The current password, or null if none has been set
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Sets a new HTTP digest.
     *
     * @param Nstein_Http_Digest $httpDigest The new HTTP digest
     */
    public function setHttpDigest(Nstein_Http_Digest $httpDigest)
    {
        $this->httpDigest = $httpDigest;
    }

    /**
     * Resets the HTTP digest.
     */
    public function resetHttpDigest()
    {
        $this->httpDigest = null;
    }

    /**
     * Gets the current HTTP digest.
     *
     * @return Nstein_Http_Digest The current HTTP authentication wrapper, or null if HTTP digest
     *                            authentication has not yet been attempted successfully.
     */
    public function getHttpDigest()
    {
        return $this->httpDigest;
    }

    /**
     * Sets a file to upload during a subsequent POST request.
     *
     * This method can be used in any of three ways:
     *
     * 1. $data is null (default):
     *
     *    a. $file is a string treated as the name of a local file which will be read and sent. Will
     *       try to guess the content type using mime_content_type().
     *
     *    b. $file is a file resource to be read and sent in chunks. Content-type header must be
     *       given explicitly by parameter $ctype.
     *
     * 2. $data is set - $file is sent as the file name, but $data is sent as the file contents and
     *    no file is read from the file system. In this case, you need to manually set the
     *    Content-Type ($ctype) or it will default to application/octet-stream.
     *
     * @param mixed  $file     File resource to upload, or name of file to upload, or name to save as
     * @param string $formname Name of form element to send as
     * @param string $data     Data to send (if null, $file is read and sent)
     * @param string $ctype    Content type to use (if $data is set and $ctype is
     *                         null, will be application/octet-stream)
     *
     * @return Zend_Http_Client
     * @throws Zend_Http_Client_Exception
     */
    public function setFileUpload($file, $formname, $data = null, $ctype = null)
    {
        if ($data === null)
        {
            if (!is_resource($file))
            {
                if (($data = @file_get_contents($file)) === false)
                {
                    /** @see Zend_Http_Client_Exception */
                    require_once 'Zend/Http/Client/Exception.php';
                    throw new Zend_Http_Client_Exception("Unable to read file '{$file}' for upload");
                }

                if (!$ctype)
                {
                    $ctype = $this->_detectFileMimeType($file);
                }

                $filename = basename($file);
            }
            else
            {
                // The "data" is the file resource that will eventually be read by the HTTP client
                // adapter's write method
                $data = $file;

                // We don't have a filename, so just use the form element name
                $filename = $formname;
            }
        }

        // Force enctype to multipart/form-data
        $this->setEncType(self::ENC_FORMDATA);

        $this->files[$formname] = array($filename, $ctype, $data);

        return $this;
    }

    /**
     * Sends the current HTTP request and returns the corresponding HTTP response.
     *
     * If the method receives a "401 Unauthorized" response from the remote server and user
     * credentials have been set using the {@link setCredentials()} method, retries the request with
     * the appropriate HTTP digest authentication "Authorization" header.
     *
     * @param string $method The request method
     *
     * @return Zend_Http_Response The corresponding HTTP response
     * @throws Zend_Http_Client_Exception
     */
    public function request($method = null)
    {
        // Get the request URI
        $uri = $this->getUri();
        
        // Set the "Authorization" header if possible
        if ($this->httpDigest !== null)
        {
            $authHeader = $this->httpDigest->computeAuthorizationHeader($method, $uri);
            if ($authHeader !== null)
            {
                $this->setHeaders(Nstein_Http_Response::H_AUTHORIZATION, $authHeader);
            }
        }

       // Let the base implementation perform the request
        $response = parent::request($method);

        // If the HTTP response status is "401 Unauthorized" and we have user credentials, try to
        // authenticate the user using the "HTTP Digest Access Authentication" protocol
        if ($response->getStatus() == Nstein_Http_Response::S_UNAUTHORIZED
            && $this->username !== null && $this->password !== null)
        {
            // Get the "WWW-Authenticate" header from the response
            $wwwAuthHdr = strval($response->getHeader(Nstein_Http_Response::H_WWW_AUTHENTICATE));

            // Initialize the HTTP digest authentication "wrapper"
            $this->httpDigest = new Nstein_Http_Digest($this->username, $this->password);

            // Compute the "Authorization" header
            $header = $this->httpDigest->computeAuthorizationHeader($method, $uri, $wwwAuthHdr);

            // Set the "Authorization" header and retry the request
            $this->setHeaders(Nstein_Http_Response::H_AUTHORIZATION, $header);

            // Retry the request
            $response = parent::request($method);
        }

        return $response;
    }

    /**
     * Prepare the request body (for POST and PUT requests)
     *
     * @return array Array of lines and/or resources
     * @throws Zend_Http_Client_Exception
     */
    protected function _prepareBody()
    {
        // According to RFC2616, a TRACE request should not have a body.
        if ($this->method == self::TRACE)
        {
            return array();
        }

        // If we have raw_post_data set, just use it as the body.
        if (isset($this->raw_post_data))
        {
            // Determine the content length depending on whether the body was given as a string or
            // a file resource
            if (!is_resource($this->raw_post_data))
            {
                $contentLength = strlen($this->raw_post_data);
            }
            else
            {
                $stat = fstat($this->raw_post_data);
                $contentLength = $stat['size'];
            }
            $this->setHeaders(self::CONTENT_LENGTH, $contentLength);

            return array($this->raw_post_data);
        }

        // Body is array of lines and/or file resources
        $body = array();

        // If we have files to upload, force enctype to multipart/form-data
        if (count ($this->files) > 0) $this->setEncType(self::ENC_FORMDATA);

        // If we have POST parameters or files, encode and add them to the body
        if (count($this->paramsPost) > 0 || count($this->files) > 0)
        {
            switch($this->enctype)
            {
            case self::ENC_FORMDATA:
                // Encode body as multipart/form-data
                $boundary = '---ZENDHTTPCLIENT-' . md5(microtime());
                $this->setHeaders(self::CONTENT_TYPE, self::ENC_FORMDATA . "; boundary={$boundary}");

                // Get POST parameters and encode them
                $params = $this->_getParametersRecursive($this->paramsPost);
                foreach ($params as $pp)
                {
                    $body = array_merge($body, self::encodeFormData($boundary, $pp[0], $pp[1]));
                }

                // Encode files
                foreach ($this->files as $name => $file)
                {
                    $fhead = array(self::CONTENT_TYPE => $file[1]);
                    $body = array_merge($body, self::encodeFormData(
                                            $boundary, $name, $file[2], $file[0], $fhead));
                }

                $body[] = '--' . $boundary . '--';
                break;

            case self::ENC_URLENCODED:
                // Encode body as application/x-www-form-urlencoded
                $this->setHeaders(self::CONTENT_TYPE, self::ENC_URLENCODED);
                $body[] = http_build_query($this->paramsPost, '', '&');
                break;

            default:
                /** @see Zend_Http_Client_Exception */
                require_once 'Zend/Http/Client/Exception.php';
                throw new Zend_Http_Client_Exception("Cannot handle content type '{$this->enctype}' automatically." .
                                                     " Please use Nstein_Http_Client::setRawData to send this kind of content.");
                break;
            }
        }

        // Set the Content-Length if we have a body or if request is POST/PUT
        if ($body || $this->method == self::POST || $this->method == self::PUT)
        {
            $contentLength = 0;
            foreach ($body as $bodyPart)
            {
                if (!is_resource($bodyPart))
                {
                    $contentLength += strlen($bodyPart);
                }
                else
                {
                    $stat = fstat($bodyPart);
                    $contentLength += $stat['size'];
                }
                $contentLength += 2; // "\r\n"
            }
            $this->setHeaders(self::CONTENT_LENGTH, $contentLength);
        }

        return $body;
    }

    /**
     * Encode data to a multipart/form-data part suitable for a POST request.
     *
     * This function returns the encoded data as an array, as follows:
     *
     * @param string $boundary
     * @param string $name
     * @param mixed $value
     * @param string $filename
     * @param array $headers Associative array of optional headers @example ("Content-Transfer-Encoding" => "binary")
     * @return array Array of lines and/or file resources
     */
    public static function encodeFormData($boundary, $name, $value,
                                          $filename = null, $headers = array())
    {
        $data = array('--' . $boundary);

        $header = 'Content-Disposition: form-data; name="' . $name . '"';
        if ($filename)
        {
            $header .= '; filename="' . $filename . '"';
        }
        $data[] = $header;

        foreach ($headers as $name => $value)
        {
            $data[] = $name . ': ' . $value;
        }
        $data[] = '';
        $data[] = $value;

        return $data;
    }
}