<?php
/**
 * Nstein MVC application framework
 *
 * @category   Nstein
 * @package    Nstein_MVC
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

require_once 'Zend/Http/Client/Adapter/Socket.php';

/**
 * This class fixes some problems with the Zend_Http_Client_Adapter_Socket class:
 *
 * - The Zend_Http_Client_Adapter_Socket class is too strict with the "chunked" transfer encoding
 *   when there is no chunk to read. The default adapter will throw an error, but this causes
 *   a problem when we access a URL and we aren't authentified. This implementation doesn't raise
 *   the error and sets the body to null.
 *
 * - The Zend_Http_Client does not handle large file uploads (via POST or PUT) efficiently, reading
 *   the whole content of the file in memory. This implementation (in conjuntion with the
 *   Nstein_Http_Client class) assumes the $body parameter of the write() method to be an array of
 *   lines and/or resources, permitting the content to be streamed to the remote server.
 */
class Nstein_Http_Client_Adapter_Socket extends Zend_Http_Client_Adapter_Socket
{
    /**
     * Read response from server
     *
     * @return string
     */
    public function read()
    {
        // First, read headers only
        $response = '';
        $gotStatus = false;
        while (($line = @fgets($this->socket)) !== false)
        {
            $gotStatus = $gotStatus || (strpos($line, 'HTTP') !== false);
            if ($gotStatus)
            {
                $response .= $line;
                if (rtrim($line) === '') break;
            }
        }

        $statusCode = Zend_Http_Response::extractCode($response);

        // Handle 100 and 101 responses internally by restarting the read again
        if ($statusCode == 100 || $statusCode == 101) return $this->read();

        /**
         * Responses to HEAD requests and 204 or 304 responses are not expected
         * to have a body - stop reading here
         */
        if ($statusCode == 304 || $statusCode == 204 ||
            $this->method == Zend_Http_Client::HEAD) return $response;

        // Check headers to see what kind of connection / transfer encoding we have
        $headers = Zend_Http_Response::extractHeaders($response);

        // If we got a 'transfer-encoding: chunked' header
        if (isset($headers['transfer-encoding']))
        {
            if ($headers['transfer-encoding'] == 'chunked')
            {
                do
                {
                    $line  = @fgets($this->socket);
                    $chunk = $line;

                    // Figure out the next chunk size
                    $chunksize = trim($line);
                    if (!ctype_xdigit($chunksize))
                    {
                        /**
                         * THIS IS ZEND INITIAL CODE
                         $this->close();
                         // TODO use Nstein-based exception?
                         require_once 'Zend/Http/Client/Adapter/Exception.php';
                         throw new Zend_Http_Client_Adapter_Exception('Invalid chunk size "' .
                         $chunksize . '" unable to read chunked body');
                        */

                        /**
                         * OUR FIX
                         */
                        return $response;
                        /*
                         * END OF OUR FIX
                         */
                    }

                    // Convert the hexadecimal value to plain integer
                    $chunksize = hexdec($chunksize);

                    // Read chunk
                    $left_to_read = $chunksize;
                    while ($left_to_read > 0)
                    {
                        $line = @fread($this->socket, $left_to_read);
                        $chunk .= $line;
                        $left_to_read -= strlen($line);

                        // Break if the connection ended prematurely
                        if (feof($this->socket)) break;
                    }

                    $chunk .= @fgets($this->socket);
                    $response .= $chunk;
                }
                while ($chunksize > 0);
            }
            else
            {
                // TODO use Nstein-based exception?
                throw new Zend_Http_Client_Adapter_Exception('Cannot handle "' .
                                                             $headers['transfer-encoding'] . '" transfer encoding');
            }

            // Else, if we got the content-length header, read this number of bytes
        }
        elseif (isset($headers['content-length']))
        {
            $left_to_read = $headers['content-length'];
            $chunk = '';
            while ($left_to_read > 0)
            {
                $chunk = @fread($this->socket, $left_to_read);
                $left_to_read -= strlen($chunk);
                $response .= $chunk;

                // Break if the connection ended prematurely
                if (feof($this->socket)) break;
            }

            // Fallback: just read the response until EOF
        }
        else
        {
            while (($buff = @fread($this->socket, 8192)) !== false)
            {
                $response .= $buff;
                if (feof($this->socket)) break;
            }

            $this->close();
        }

        // Close the connection if requested to do so by the server
        if (isset($headers['connection']) && $headers['connection'] == 'close')
        {
            $this->close();
        }

        return $response;
    }
    
     /**
     * Connect to the remote server
     *
     * @param string  $host
     * @param int     $port
     * @param boolean $secure
     * @param int     $timeout
     */
    public function connect($host, $port = 80, $secure = false)
    {
        try 
        {
            parent::connect($host, $port, $secure);
        }
        catch (Exception $ex)
        {
           $classNameEx = get_class($ex);
           throw new $classNameEx($ex->getMessage(), Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
        }
    }

    /**
     * Sends the request to the remote server.
     *
     * @param string        $method
     * @param Zend_Uri_Http $uri
     * @param string        $http_ver
     * @param array         $headers
     * @param array         $body
     *
     * @return string Request as string (not including any resources)
     */
    public function write($method, $uri, $http_ver = '1.1', $headers = array(), $body = array())
    {
        // Make sure we're properly connected
        if (!$this->socket)
        {
            require_once 'Zend/Http/Client/Adapter/Exception.php';
            throw new Zend_Http_Client_Adapter_Exception('Trying to write but we are not connected', Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
        }

        $host = $uri->getHost();
        $host = (strtolower($uri->getScheme()) == 'https' ? $this->config['ssltransport'] : 'tcp') . '://' . $host;
        if ($this->connected_to[0] != $host || $this->connected_to[1] != $uri->getPort())
        {
            require_once 'Zend/Http/Client/Adapter/Exception.php';
            throw new Zend_Http_Client_Adapter_Exception('Trying to write but we are connected to the wrong host');
        }

        // Save request method for later
        $this->method = $method;

        // Build request line and headers
        $path = $uri->getPath();
        if ($uri->getQuery()) $path .= '?' . $uri->getQuery();
        $request = "{$method} {$path} HTTP/{$http_ver}\r\n";
        foreach ($headers as $k => $v)
        {
            if (is_string($k)) $v = ucfirst($k) . ": $v";
            $request .= "$v\r\n";
        }

        // Empty line signals end of headers
        $request .= "\r\n";

        // Send request line and headers
        if (!@fwrite($this->socket, $request))
        {
            require_once 'Zend/Http/Client/Adapter/Exception.php';
            throw new Zend_Http_Client_Adapter_Exception('Error writing request to server', Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
        }

        // If the content type is 'multipart/form-data', the $body parameter is an array of lines
        // and/or file resources, so we'll need to append an end-of-line to each one
        $eol = '';
        if (isset($headers[Nstein_Http_Client::CONTENT_TYPE])
            && $headers[Nstein_Http_Client::CONTENT_TYPE] == Nstein_Http_Client::ENC_FORMDATA)
        {
            $eol = "\r\n";
        }

        // Send each line (or resource) to the remote server
        foreach ($body as $bodyPart)
        {
            // Line can be string or resource
            if (!is_resource($bodyPart))
            {
                $line = $bodyPart;

                // Possibly append an end-of-line
                if ($eol)
                {
                    $line .= $eol;
                }

                // Send the line
                if (!@fwrite($this->socket, $line))
                {
                    require_once 'Zend/Http/Client/Adapter/Exception.php';
                    throw new Zend_Http_Client_Adapter_Exception('Error writing request to server', Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
                }

                // Append to $request return value
                $request .= $line;
            }
            else
            {
                $resource = $bodyPart;

                // Ensure we're at the beginning of the file; this is necessary in case the request
                // is retried after a "401 Unauthorized" response
                fseek($resource, 0, SEEK_SET);

                // Send resource in chunks
                while (!feof($resource))
                {
                    // Read a chunk
                    $chunk = @fread($resource, 8192); // TODO use constant
                    if ($chunk === false)
                    {
                        require_once 'Zend/Http/Client/Adapter/Exception.php';
                        throw new Zend_Http_Client_Adapter_Exception(
                            'Error reading request body chunk from file resource');
                    }

                    // Send the chunk
                    if (!@fwrite($this->socket, $chunk))
                    {
                        require_once 'Zend/Http/Client/Adapter/Exception.php';
                        throw new Zend_Http_Client_Adapter_Exception('Error writing request to server', Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
                    }
                }

                // Possibly send an end-of-line
                if ($eol && !@fwrite($this->socket, $eol))
                {
                    require_once 'Zend/Http/Client/Adapter/Exception.php';
                    throw new Zend_Http_Client_Adapter_Exception('Error writing request to server', Nstein_REST_Response::S_SERVICE_UNAVAILABLE);
                }

                // Append placeholder to $request return value
                $request .= '<<RESOURCE>>';
            }
        }

        return $request;
    }
}