<?php
/**
 * Project:     WCM
 * File:        $Id: biz.channel.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * Definition of a channel
 */
class channel extends bizobject
{
    /**
     * site Id
     * @var int
     */
    public $siteId;

    /**
     * parent Id
     * @var int
     */
    public $parentId = null;

    /**
     * rank
     * @var int
     */
    public $rank;

    /**
     * title
     * @var string
     */
    public $title;

    /**
     * description
     * @var string
     */
    public $description;

    /**
     * template id
     * @var int
     */
    public $templateId;

    /**
     * image path
     * @var string
     */
    public $imagePath = '';

    /**
     * keywords
     * @var string
     */
    public $keywords;

    /**
     * management kind
     * @var string
     */
    public $managementKind;

    /**
     * native request used to get content
     * @var string
     */
    public $request;

    /**
     * publication Id
     * @var int
     */
    public $publicationId;


    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $config = wcmConfig::getInstance();

        $this->parentId = null;
        $this->rank = 0;
        $this->managementKind = 'date';
        $this->properties = array();
        $this->request = array('query' => $config['wcm.channel.query'], 'orderBy' => '', 'limit' => '');
    }

    /**
     * Unserialize some properties if needed
     * This method is called by nextEnum() and refresh()
     * and may be invoked after a bindArrayToObject() call
     */
    public function unserializeProperties()
    {
        parent::unserializeProperties();

        if ($this->id && is_string($this->request) && '' != $this->request)
        {
            $this->request = unserialize($this->request);
        }
    }

    /**
     * Return the parent channel of the object
     *
     * @return channel parent channel or null
     */
    public function getParentChannel()
    {
        $channel = wcmObjectFactory::create('channel', $this->parentId);
        return ($channel->id) ? $channel : null;
    }

    /**
     * Inserts or Updates object in database
     *
     * @return bool true on success, false on failure
     */
    protected function store()
    {
        if (!parent::store())
        {
            return false;
        }

        // Update cache of channels (of site)
        wcmCacheRegistry::get()->setElem('channel_of_site' . $this->siteId, $this->id, $this);
        return true;
    }

    /**
     * Deletes current object from database
     *
     * @return bool true on success or an error message (string)
     */
    public function delete()
    {
        $sid = $this->siteId;
        $id = $this->id;
        if (!parent::delete())
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->unsetElem('channel_of_site' . $sid, $id);

        return true;
    }

    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    function ofClause($of)
    {
        if (null == $of || !is_array($of))
        {
            return;
        }

        $sql = null;

        foreach ($of as $key => $value)
        {
            switch ($key)
            {
                case 'site':
                    if (null != $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'siteId=' . $value;
                    break;
                case 'id':
                    if (null !== $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'id=' . $value;
                    break;
            }
        }
        return $sql;
    }

    /**
    * Says whether the id of the instance is present in the relation array
    *
    * @param int $objectid Object ID
    * @param string $objectclass Object class name
    * @param array Array of relations to test
    *
    * @return bool true if the id is prensent
    */
    public function scanlist($objectid, $objectclass, $relations)
    {
        foreach ($relations as $key => $value)
        {
            $arr = explode('-', $value);
            $relclass = $arr[0];
            $relid = $arr[1];

            if ($objectid == $relid && $objectclass == $relclass)
            {
                return true;
            }
        }
        return false;
    }

    /**
    * Returns the first element of the channel's content
    *
    * @param string $date    Optional date used to compute channel's content (default is today's date)
    *
    * @return mixed the first element that's in the channel's content
    */
    public function getfirstelem($date = null)
    {
        //set default value for date parameter?
        if (null === $date)
        {
            $date = date('Y-m-d');
        }

        $mixedresults = $this->getContent(10, $date, true);

           return (isset($mixedresults['1']) ? $mixedresults['1'] : null);
    }

    /**
    * Returns content of the channel
    *
    * @param  int  $limit Optional maximum number of objects to return
    * @param  string $date  Optional date used to compute channel's content (default is today's date)
    * @param  bool $returnBizobject Return related bizobject when a bizrelation if found (default is false)
    * @param  bool $toXML TRUE when this method is invoked in the context of 'toXML()' (default is false)
    * @param  string $orderby_forced to override the actual order_by (in templates)
    *
    * @return array Array of bizobjects (or null if management method is invalid)
    */
    public function getContent($limit = null, $date = null, $returnBizobject = false, $toXML = false, $orderby_forced = null)
    {
        // set default value for date parameter?
        if (null === $date)
        {
            $date = date('Y-m-d');
        }

        //get query
        $search_query = $this->request;

        $query = (isset($search_query['query']) ? $search_query['query'] : '');
        //replace smart tag @channelId if any
        $query = str_replace('@channelId', $this->id, $query);

        $orderBy = isset($orderby_forced) ? $orderby_forced : (isset($search_query['orderBy']) ? $search_query['orderBy'] : null);
        $limit = ($limit) ? $limit : ((isset($search_query['limit']) && is_numeric($search_query['limit'])) ? $search_query['limit'] : 10);

        $results = array();
        if ('' != $query)
        {
            $config = wcmConfig::getInstance();
            $engine = $config['wcm.plugins.search.default'];
            $search = wcmSearch::getInstance($engine);
            $total = $search->search('quickSearch', $query, $orderBy);
            $results = $search->getSearchResults(0, $limit, false);
        }
        //get relations
        $relations = bizrelation::getBizobjectRelations($this, bizrelation::IS_COMPOSED_OF, null, $toXML, $date);

        //sort them in order
        $mixedresults = array();
        $forcedcontentids = array();
        foreach ($relations as $rel)
        {
            $destclass = $rel['destinationClass'];
            $destid = $rel['destinationId'];
            $forcedcontentids[] = $destclass . '-' . $destid;

            if ($returnBizobject)
            {
                $mixedresults[$rel['rank']] = wcmObjectFactory::create($destclass, $destid);
            }
            else
            {
                $mixedresults[$rel['rank']] = $rel;
            }
        }
        $query_results_counter = 0;
        $mixed_results_counter = 1;

        $limit = $limit + count($relations);

        for ($i = 1; $i <= $limit; $i++)
        {
            if (isset($results[$query_results_counter]) && $this->scanlist($results[$query_results_counter]->id, get_class($results[$query_results_counter]), $forcedcontentids))
            {
                $query_results_counter++;
            }
            else
            {
                if (!isset($mixedresults[$mixed_results_counter]) && isset($results[$query_results_counter]) && !($results[$query_results_counter]->id == $this->id && get_class($results[$query_results_counter]) == get_class($this)))
                {
                    $mixedresults[$mixed_results_counter] = $results[$query_results_counter];
                    $query_results_counter++;
                }
                elseif (!isset($mixedresults[$mixed_results_counter]))
                {
                    $mixedresults[$mixed_results_counter] = null;
                }

                $mixed_results_counter++;
            }

        }
        //sort using indexes (which are the ranks)
        ksort($mixedresults);

        array_splice($mixedresults, $limit);

        return $mixedresults;
    }

    /**
    * Returns an array of bizobjects for a specific date
    *
    * @param  int $limit Optional maximum number of objects to return
    * @param  string  $referenceDate Date used for content retrieval (or null for last available content)
    *
    * @return array Array of bizobjects
    */
    public function getContent_date($limit=0, $referenceDate = null)
    {
        // Prepare returned result
         $content = array();

        // Check referenceDate
        if (!$referenceDate)
        {
            $referenceDate = date('Y-m-d');
        }

        // Build the bizrelation object from the channel class and id
        $bizrelation = new bizrelation();
        $bizrelation->sourceClass = $this->getType();
        $bizrelation->sourceId = $this->id;
        $bizrelation->kind = bizrelation::IS_COMPOSED_OF;

        // Get last validityDate
        $referenceDate = $bizrelation->getLatestValidityDate($referenceDate);

        if ($referenceDate)
        {
            $bizrelation->beginEnumOnDate($lastDate, 0, $limit);
            while ($bizrelation->nextEnum())
            {
                $bizobject = wcmObjectFactory::create($bizrelation->destinationClass, $bizrelation->destinationId);
                if ($bizobject->id)
                {
                    $content[] = $bizobject;
                }
                else
                {
                    // related object does not exists anymore!
                    $bizrelation->delete();
                }
            }
            $bizrelation->endEnum();
        }

        return $content;
    }

    /**
     * Returns articles that match the query or that just
     * belongs to the channel if no query has been set
     *
     * @param int $offset Offset used for the search (0 by default)
     * @param int $limit Maximum number of articles returned (0, by default, for all)
     *
     * @return array An associative array of bizobjects (keys are ids)
     */
    public function getContent_query($offset=0, $limit=0)
    {
        $content = array();

        if ($this->request)
        {
            // Invoke bizsearch
        }
        else
        {
            $enum = new wcmEnumeration('article', 'channelId=' . $this->id, 'publicationDate DESC', $offset, $limit);
            foreach ($enum as $article)
            {
                $content[] = $article;
            }
        }
        return $content;
    }

    /**
     * Returns an array of all channel's ancestors
     * The array is sorted in ancestry order (e.g. parent, grand-parent, etc...)
     *
     * @return array An array of channel objects
     */
    public function getAncestors()
    {
        $ancestors = array();

        $channel = wcmObjectFactory::create('channel');
        $channel->parentId = $this->parentId;
        while ($channel->parentId)
        {
            if ($channel->refresh($channel->parentId))
            {
                $ancestors[] = clone($channel);
            }
        }
        return $ancestors;
    }

    /**
     * Returns the list of available method to organize content
     *
     * @return array An assoc array (method => method title)
     */
    public function getChannelOrganisationList()
    {
        $array = array('date' => _BIZ_DATE_MANAGEMENT,
                       'query' => 'Query based',
                       'mixed' => _BIZ_MIXED_MANAGEMENT);
        foreach ($array as $key => $value)
        {
            if (!method_exists($this, 'getContent_' . $key))
            {
                unset($array[$key]);
            }
        }

        return $array;
    }

    /**
     * Returns hierarchical array of channel's paths for current site
     *
     * @param array $channels Array to update (should be initialized)
     * @param int $excludeId Optional ID of menu to exclud forom the list
     * @param string $prefix Initial prefix (used for recursive call)
     * @param int $parentId Initial channel (or zero to take root channels, by default)
     * @param string $separator Separator tu use in path (default is ' :: ')
     *
     * @return array An assoc array of channels in hierarchical order (key is id, value is channel path)
     */
    static function getChannelHierarchy(array &$channels = null, $excludeId=0, $prefix=null, $parentId=0, $separator = ' :: ')
    {
        if (!$channels)
        {
            $channels = array();
        }
        $project = wcmProject::getInstance();
        $session = wcmSession::getInstance();

        // load channels of site from cache
        $channelList = wcmCacheRegistry::get()->fetch('channel_of_site' . $session->getSiteId());

        if (false === $channelList)
        {
            $channelList = array();
            $enum = new wcmEnumeration('channel', 'siteId = ' . $session->getSiteId(), 'parentId, rank');
            foreach ($enum as $channel)
            {
                $channelList[$channel->id] = $channel;
            }
            wcmCacheRegistry::get()->store('channel_of_site' . $session->getSiteId(), $channelList);
        }

        foreach ($channelList as $channel)
        {
            if ($channel->parentId == $parentId)
            {
                // Add menu (unless it is excluded)
                $path = ($prefix) ? $prefix . $separator . getConst($channel->title) : getConst($channel->title);
                if ($channel->id != $excludeId)
                {
                    $channels[$channel->id] = $path;
                }

                // Recursively add sub-menus
                channel::getChannelHierarchy($channels, $excludeId, $path, $channel->id, $separator);
            }
        }
        return $channels;
    }


    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid
     *
     */
    public function checkValidity()
    {
        $channelList = array();
        //get the channel list except the current one
        $channelList = channel::getChannelHierarchy($channelList, $this->id);

        if (!parent::checkValidity())
        {
            return false;
        }

        if ('' === trim($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_IS_MANDATORY;
            return false;
        }

        if (255 < strlen($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_TOO_LONG;
            return false;
        }

        if (in_array(strtolower(trim($this->title)), array_map('strtolower', $channelList)))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_ALREADY_EXISTS;
            return false;
        }

        if ($this->keywords && 255 < strlen($this->keywords))
        {
            $this->lastErrorMsg = _BIZ_ERROR_KEYWORDS_TOO_LONG;
            return false;
        }

        return true;
    }
}
