<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.import.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * Abstract class
 *
 * Generic import class that all import plugins extend
 */
abstract class wcmImport
{
    /**
    * Logger
    *
    * @var wcmLogger
    */
    protected $logger;

    /**
    * a generic associative array to transmit extra parameters to inherited classes
    * @var array
    */
    protected $parameters;

    /**
    * set this property to TRUE to cancel import process (checked within each process loop)
    * @var bool
    */
    protected $cancelled = false;

    /**
     * FRFR logFile
     * @var string
     *
     */
    protected $logFile = null;

    /**
     * Polling
     * @var bool
     */
    protected $polling = false;

    /**
     * Total number of items
     * @var int
     */
    protected $total;

    /**
     * Total number of items processed
     * @var int
     */
    protected $totalProcessed;

    static public function getPluginCache()
    {
        if (!file_exists(WCM_BACKOFFICE_CONF_DIR . '/plugins/import.xml'))
        {
            self::buildPluginCache();
        }
        return simplexml_load_file(WCM_BACKOFFICE_CONF_DIR . '/plugins/import.xml');
    }

    static public function buildPluginCache()
    {
        $xml = new DOMDocument();
        $xml->preserveWhiteSpace = false;
        $xml->formatOutput = true;
        $rootNode = $xml->createElement('plugins');
        $xml->appendChild($rootNode);

        $dir = new RecursiveDirectoryIterator(WCM_SYS_API_DIR . '/plugins/import/');
        foreach (new RecursiveIteratorIterator($dir, RecursiveIteratorIterator::SELF_FIRST) as $file)
        {
            if ($file->getFilename() !== 'plugin.xml')
            {
                continue;
            }
            $pluginDir = dirname($file->getPathname());
            $pluginDOM = DOMDocument::load($file->getPathname());
            $pluginXML = $xml->importNode($pluginDOM->documentElement, true);
            $pluginXML->appendChild($xml->createElement('path', $pluginDir));

            $xml->documentElement->appendChild($pluginXML);
        }

        $xml->save(WCM_BACKOFFICE_CONF_DIR . '/plugins/import.xml');
        unset($xml);
    }

    /**
    * Constructor
    *
    * @param array $parameters A list of parameters for the plugin
    */
    public function __construct(array $parameters)
    {
        $this->siteId = $parameters['siteId'];
        $this->parameters = is_array($parameters) ? $parameters : array();
        $this->logFile = isset($parameters['logFile']) ? $parameters['logFile'] : 'import_' . date('YmdHis');
        if (!isset($parameters['logger']) || !($parameters['logger'] instanceof wcmLogger))
        {
            $config = wcmConfig::getInstance();
            $this->logger = new wcmLogger(true, true, $config['wcm.logging.path']  . 'traces/' . $this->logFile, false);
        }
        else
        {
            $this->logger = $parameters['logger'];
        }
    }

    /**
     * Returns current logger
     * @return wcmLogger Current logger
     */
    public function getLogger()
    {
        return $this->logger;
    }

    /**
     * Set current logger
     * @param wcmLogger Logger to set
     */
    public function setLogger($logger)
    {
        $this->logger = $logger;
    }

    /*
    * Launch import process
    * @return bool TRUE on success
    */
    abstract public function process();

    /**
     * Import from an object
     * @param wcmObject $argObj Object to import
     * @param array $argParams Extra parameters
     * @return bool TRUE on success
     */
    public function importFromObj(wcmObject $argObj, array $argParams = array())
    {
        return true;
    }

    /**
     * Import from XML
     * @param string $argXML XML
     * @param array $argParams Extra parameters
     * @return bool TRUE on success
     */
    public function importFromXML($argXml, array $argParams = array())
    {
        return true;
    }

    /**
     * Enable polling
     */
    public function enablePolling()
    {
        $this->polling = true;
        $this->pollFile = $this->logFile . '_polling.log';
        $this->logger->logMessage('Enabled polling to :' . $this->pollFile);
        $config = wcmConfig::getInstance();
        touch($config['wcm.logging.path'] . 'traces/' . $this->pollFile);
    }

    /**
     * Write poll data to file
     * @param string $argPercent Data to write
     */
    public function writePollData($argPercent)
    {
        if ($this->polling)
        {
            $config = wcmConfig::getInstance();
            $fp = fopen($config['wcm.logging.path'] . 'traces/' . $this->pollFile, 'wb+');
            fwrite($fp, $argPercent);
            fclose($fp);
        }
    }

    /**
    * This method is used to notify import that processing should be cancel
    *
    * @param string $message An optional message to log before cancelling
    */
    protected function cancelImport($message = null)
    {
        if ($message)
        {
            $this->logger->logInfo($message);
        }
        $this->cancelled = true;
    }

    /**
     * Get total number of items
     * @return int Total numbers of items
     */
    abstract public function getTotal();
}
