<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.bizobject.php 9388 2010-04-06 19:50:22Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmBizobject class inherits from the {@link wcmSysobject} class
 * and represents business objects.
 */
abstract class wcmBizobject extends wcmSysobject
{
    /**
     * TRUE if bizobject must be generated
     * => This property is automatically set to true when checkin() is invoked
     * @var bool
     */
    public $mustGenerate = false;

    /**
     * An associative array of tags (name => category)
     * @var array
     */
    public $xmlTags;

    /**
     * Contains the semantic data
     * @var wcmSemanticData
     */
    public $semanticData;

    //
    // Original (import) properties (source)
    //

    /**
     * Source of the bizobject where it was imported from.
     * @var string
     */
    public $source;

    /**
     * Id in the source for this bizobject.
     * @var int
     */
    public $sourceId;

    /**
     * Version in the source for this bizobject.
     * @var string
     */
    public $sourceVersion;

    /**
     * Bizobject's permalinks
     * @var array
     */
    public $permalinks;

    /**
     * State of bizobject (in workflow)
     * @var string
     */
    public $workflowState = null;

    /**
     * bizobject's workflow
     * @var wcmWorkflow
     */
    protected $workflow;
    
    /**
     * The bizobject's workflow script.
     * @var wcmIWorkflowScript
     */
    protected $workflowScript = false;

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->mustGenerate = false;
        $this->source = null;
        $this->sourceId = null;
        $this->sourceVersion = null;
        $this->xmlTags = array();
        $this->permalinks = array();
        $this->semanticData = new wcmSemanticData();

        if ($this->workflow)
        {
            $this->workflowState = $this->workflow->initialState;
        }
    }

    /**
     * Unserialize some properties if needed
     * This method is called by nextEnum() and refresh()
     */
    public function unserializeProperties()
    {
        parent::unserializeProperties();

        $this->xmlTags = ($this->xmlTags) ? unserialize($this->xmlTags) : array();
        $this->permalinks = ($this->permalinks) ? unserialize($this->permalinks) : array();

        if (is_string($this->semanticData) && $this->semanticData != false)
        {
            $this->semanticData = unserialize($this->semanticData);
        }
        elseif (!$this->semanticData)
        {
            $this->semanticData = new wcmSemanticData();
        }
    }

    /**
     * Returns the business or system class which defines current sysobject
     *
     * @return object An instance or wcmBizclass or wcmSysclass
     */
    public function getMasterClass()
    {
        if (!isset($this->masterClass))
        {
            $this->workflow = wcmMasterClassRegistry::getWorkflow($this);
        }

        return parent::getMasterClass();
    }

    /**
     * Returns the number of expired objects
     *
     * @return int Number of expired object or null on error
     */
    public function getExpiredCount()
    {
        if (!property_exists($this, 'expirationDate'))
        {
            return 0;
        }

        return $this->getCount('expirationDate < ?', array(date('Y-m-d H:i:s')));

    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     * @param bool $noWorkflow Internal use only (used to bypass lock and workflow)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source = null, $noWorkflow = false)
    {
        // set mustGenerate to true
        $this->mustGenerate = true;

        // Invoke workflow?
        if ($noWorkflow)
        {
            if (!parent::save($source))
            {
                return false;
            }
        }
        else
        {
            // load workflow
            $script = $this->getWorkflowScript();
            $action  = ($this->id) ? 'Update' : 'Create';

            // Execute pre-condition workflow
            if ($script)
            {
                if (!call_user_func(array($script, 'before' . $action)))
                {
                    return false;
                }
            }

            if (!parent::save($source))
            {
                return false;
            }
           
            if ($script)
            { 
                call_user_func(array($script, 'after' . $action));
            }
        }

        return true;
    }

    /**
     * Index object in search engine
     *
     * @return bool true on success, false on failure
     */
    public function index()
    {
        return wcmSearch::getInstance()->indexObject($this);
    }

    /**
     * De-index object from search engine
     *
     * @return bool true on success, false on failure
     */
    public function deindex()
    {
        return wcmSearch::getInstance()->deindexObject($this);
    }

    /**
     * Deletes the object from database.
     *
     * @return bool True on success, false on failure
     *
     */
    public function delete()
    {
        // Remove biz relations
        bizrelation::removeBizobject($this);

        $comments = $this->getComments();
        if ($comments)
        {
            foreach ($comments as $comment)
            {
                $comment->delete();
            }
        }

        // load workflow
        $script = $this->getWorkflowScript();

        // Execute pre-condition workflow
        if ($script)
        {
            if (!call_user_func(array($script, 'beforeDelete')))
            {
                return false;
            }
        }

        // Delete bizobject
        if (!parent::delete())
        {
            return false;
        }

        // Execute post-condition workflow
        if ($script)
        {
            try
            {
                call_user_func(array($script, 'afterDelete'));
            }
            catch (Exception $e)
            {
                wcmProject::getInstance()->logger->logWarning('onDelete failed: ' . $e->getMessage());
            }
        }

        return true;
    }

    /**
     * Explicit serialization of object
     *
     * @return string Serialized object
     */
    public function serialize()
    {
        // Serialize all relations in the serial storage!
        $this->serialStorage['relations'] = bizrelation::archiveBizobject($this);

        return parent::serialize();
    }

    /**
     * Restore dependencies
     * This method is called by restore/archive methods and is used to restore in
     * the database all dependencies (e.g. relations, sub-parts, etc...)
     */
    protected function restoreDependencies()
    {
        // restore relations
        bizrelation::restoreBizobject($this, getArrayParameter($this->serialStorage, 'relations'));

        parent::restoreDependencies();

    }

    /**
     * Updates the 'mustGenerate' property in the business DB
     * Warning: for performance considerations, the business search index will not be affected!
     *
     * @param bool $newValue The optional new value for the 'mustGenerate' property (default: false)
     *
     * @return bool TRUE on success, FALSE on failure
     */
    public function updateMustGenerate($newValue = false)
    {
        $sql = 'UPDATE ' . $this->tableName . ' SET mustGenerate=? WHERE id=?';
        return $this->database->executeStatement($sql, array($newValue, $this->id)) > 0;
    }

    /**
     * Generates specific pages (according to object id).
     * => We search for content_generation with special naming convention
     *    starting either by 'autogen_always' or  'autogen_{classname}'
     *
     * @param bool   $recursive       Try also to generate related bizobject (biz_relation) (default: true)
     * @param wcmLogger $logger          The logger to use (or null) (default: null)
     * @param bool   $forceGeneration Force generation even if 'mustGenerate' property is false (default: false)
     * @param bool   $firstCall       Whether this is the first call (default: true)
     * @param int       $widgetMode      Widget mode (for ICE and design; default: wcmWidget::VIEW_CONTENT)
     *
     * @return bool True on success, false on failure
     */
    public function generate($recursive=true, $logger = null, $forceGeneration = false, $firstCall = true, $widgetMode = wcmWidget::VIEW_CONTENT)
    {
        if (!$this->id)
        {
            return false;
        }

        if (!$forceGeneration && !$this->mustGenerate)
        {
            if ($logger)
            {
                $logger->logMessage(_GENERATION_NOTHING);
            }
            return true;
        }

        $project = wcmProject::getInstance();
        $className = $this->getClass();
        $generator = new wcmTemplateGenerator($logger, false, $widgetMode);

        // Generate the "autogen_always%" and "autogen_{className}%" generation content on first call
        if ($firstCall)
        {
            foreach ($this->getAutoGenerations() as $content)
            {
                // Extend context with current bizobject and set class and id
                $generationContent = new wcmGenerationContent();
                $generationContent->refresh($content->id);
                $generator->executeGenerationContent(
                    $generationContent,
                    array(
                        $className => $this->id,
                        'className' => $className,
                        'id' => $this->id
                    )
                );
            }
        }

        // free generator memory assigned to smarty (without this command the memory is not cleared untill the end of the script process)
        // (memory leak fix issue when looping on generation) (smarty used cross reference)
        $generator->clear_all();

        // Set bizobject 'mustGenerate' to false
        if ($this->mustGenerate)
        {
            $this->updateMustGenerate(false);
        }

        // Generate linked elements
        if ($recursive)
        {
            $sql = "SELECT sourceId, sourceClass FROM biz__relation WHERE destinationClass='" . $className . "' AND destinationId=" . $this->id;
            $rows = $this->database->executeQuery($sql);
            foreach ($rows as $row)
            {
                $linkedClass = $row['sourceClass'];
                $linked = bizobject::create($linkedClass);
                if ($linked->refresh($row['sourceId']))
                {
                    $linked->generate(false, $logger, false);
                }
            }
        }

        return true;
    }

    /**
     * Checks for autoGenerations concerning the current object.
     * This function looks for the autogens available by priority
     * and instanciate the relative generationContents which can be executed.
     *
     * @return array of wcmGenerationContent objects (for autogens)
     */
    public function getAutoGenerations()
    {
        $generationContents = array();

        $enum = new wcmEnumeration('wcmGenerationAutogen', 'objectClass=\'' . get_class($this) . '\'', 'priority ASC');
        foreach ($enum as $autogen)
        {
            $generationContent = new wcmGenerationContent();
            if ($generationContent->refresh($autogen->generationContentId))
            {
                $generationContents[] = $generationContent;
            }
        }
        
        return $generationContents;
    }

    /**
     * Exposes 'language' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return string Language associated to bizobject
     */
    public function getAssoc_language($toXML = false)
    {
        return $this->getLanguage();
    }

    /**
     * Exposes 'site' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array bizobject site's assocArray (or null)
     */
    public function getAssoc_site($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $site = $this->getSite();
        return ($site) ? $site->getAssocArray($toXML) : null;
    }

    /**
     * Exposes 'url' to the getAssocArray
     * 
     * @param  bool  $toXML Optional, defaults to false. 
     * @return string the object's url (or null)
     */
    public function getAssoc_url($toXML = false)
    {
        try
        {
            $config = wcmConfig::getInstance();
            $urlGeneratorFactory = new wcmObjectUrlGeneratorFactory($config);
            $urlGenerator = $urlGeneratorFactory->getDefaultGenerator();
            $defaultPermalink = $urlGenerator->generateUrl($this);
            return $defaultPermalink ? $defaultPermalink : null;
        }
        catch (InvalidArgumentException $e)
        {
            wcmProject::getInstance()->logger->logError($e->getMessage());
            return null;
        }    
    }

    /**
     * Exposes 'channel' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array bizobject channel's assocArray (or null)
     */
    public function getAssoc_channel($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $channel = $this->getChannel();
        return ($channel) ? $channel->getAssocArray($toXML) : null;
    }

    /**
     * Exposes 'photos' to the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of photos getAssocArray
     */
    public function getAssoc_photos($toXML = false)
    {
        $photos = array();
        foreach ($this->getPhotos() as $photo)
        {
            $wcmPhoto = bizobject::create('photo');
            if ($wcmPhoto->refresh($photo['destinationId']))
            {
                $photoAssoc = $wcmPhoto->getAssocArray($toXML);
                if ($toXML)
                {
                    $photoAssoc = $photoAssoc->toArray();
                }
                $photos[] = $photoAssoc;
            }
        }
        return $photos;
    }

    /**
     * Exposes 'publication' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array bizobject publication's assocArray (or null)
     */
    public function getAssoc_publication($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $publication = $this->getPublication();
        return ($publication) ? $publication->getAssocArray($toXML) : null;
    }

    /**
     * Exposes 'issue' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array bizobject issue's assocArray (or null)
     */
    public function getAssoc_issue($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $issue = $this->getIssue();
        return ($issue) ? $issue->getAssocArray($toXML) : null;
    }

    /*
     * Exposes 'comments' to the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of comments getAssocArray
     */
    public function getAssoc_comments($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $return = array();
        $comments = $this->getComments();
        if ($comments)
        {
            foreach ($comments as $comment)
            {
                $return[] = $comment->getAssocArray($toXML);
            }
        }
        return $return;
    }

    /**
     * Exposes 'related' to the getAssocArray (all object with IS_RELATED_TO relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_related($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_RELATED_TO);
    }

    /**
     * Exposes 'partOf' to the getAssocArray (all object with IS_PART_OF relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_partOf($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_PART_OF);
    }

    /**
     * Exposes 'composedOf' to the getAssocArray (all object with IS_COMPOSED_OF relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_composedOf($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_COMPOSED_OF);
    }

    /**
     * Exposes 'relations' to the getAssocArray (all objects in relation with current bizobject)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     * @param int $relationKind Relation kind
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_relations($toXML = false, $relationKind = null)
    {
        if ($toXML)
        {
            return null;
        }

        $objects = array();
        foreach (bizrelation::getBizobjectRelations($this, $relationKind, null, $toXML) as $object)
        {
            $obj = bizobject::create($object['destinationClass']);
            if ($obj->refresh($object['destinationId']))
            {
                $objects[] = $obj->getAssocArray($toXML);
            }
        }
        return $objects;
    }

    /**
     * Exposes 'state' to the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An assoc array of the current wcmWorkflowState (or null)
     */
    public function getAssoc_state($toXML = false)
    {
        // Dont compute for toXML()
        if ($toXML)
        {
            return null;
        }

        $state = $this->getWorkflowState();
        return ($state == null) ? null : $state->getAssocArray($toXML);
    }

    /**
     * Exposes 'transitions' to the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An assoc array of available transitions (key is transition name)
     */
    public function getAssoc_transitions($toXML = false)
    {
        // Dont compute for toXML()
        if ($toXML)
        {
            return null;
        }

        $transitions = array();
        foreach ($this->getAvailableTransitions() as $code => $transition)
        {
            $transitions[$code] = $transition->getAssocArray($toXML);
        }

        return $transitions;
    }

    /**
     * Exposes 'bizRelations' to the getAssocArray (all objects in relation with current bizobject)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     * @param int $relationKind Relation kind
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_bizRelations($toXML = false, $relationKind = null)
    {
        if ($toXML)
        {
            return null;
        }

        $objects = array();
        foreach (bizrelation::getBizobjectRelations($this, $relationKind, null, $toXML) as $object)
        {
            $obj = bizobject::create($object['destinationClass']);
            if ($obj->refresh($object['destinationId']))
            {
                $objects[] = array('bizObject' => $obj->getAssocArray($toXML),
                                   'relTitle' => $object['title'],
                                   'relKind' => $object['kind']);
            }
        }
        return $objects;
    }

    /**
     * Returns all relations of the bizobject
     *
     * @return array An array of wcmBizRelation objects
     */
    public function getRelations()
    {
        $relations = array();
        $where = "sourceClass='" . $this->getClass() . "' AND sourceId=" . $this->id;

        $enum = new wcmEnumeration('bizrelation', $where, 'validityDate, rank');
        foreach ($enum as $relation)
        {
            $relations[] =  $relation;
        }
        return $relations;
    }


    /**
     * Returns the comments of the bizobject
     *
     * @return array An array containing all comments at the root level
     */
    public function getComments()
    {
        $where = 'referentId=' . $this->id . " AND referentClass='" . $this->getClass() . "' AND parentId IS NULL";
        return wcmBizobject::getObjects('contribution', $where);
    }

    /**
     * Returns bizoboject's site
     *
     * @return site Bizobject's site or null
     */
    public function getSite()
    {
        // Is current object a site!? or else does it have a 'siteId' property
        $site = null;

        if ($this instanceof site)
        {
            $site = $this;
        }
        elseif (property_exists($this, 'siteId'))
        {
            $site = bizobject::create('site', null, $this->siteId);
        }
        else
        {
            $session = wcmSession::getInstance();
            $site = $session->getSite();
        }

        return ($site->id) ? $site : null;
    }

    /**
     * Returns bizobject's channel
     *
     * @return channel Bizobject's channel or null
     */
    public function getChannel()
    {
        // Is current object a channel?
        if ($this instanceof channel)
        {
            $channel = bizobject::create('channel', array(null, $this->parentId));
        }
        elseif (property_exists($this, 'channelId'))
        {
            $channel = bizobject::create('channel', array(null, $this->channelId));
        }
        else
        {
            return null;
        }

        return ($channel->id) ? $channel : null;
    }

    /**
     * Gets the photos related to the object.
     *
     * @return array An array of contribution objects
     */
    public function getPhotos()
    {
        return bizrelation::getBizobjectRelations($this, bizrelation::IS_COMPOSED_OF, 'photo', false);
    }

    /**
     * Gets the issue related to the bizobject
     *
     * @return issue The issue object related to the bizobject
     */
    public function getIssue()
    {
        if ($this instanceOf issue)
        {
            return null;
        }

        if (property_exists($this, 'issueId'))
        {
            $issue = new issue(null, $this->issueId);
        }
        else
        {
            $channel = $this->getChannel();
            if (!$channel)
            {
                return null;
            }

            $issue = $channel->getIssue();
        }

        return ($issue && $issue->id) ? $issue : null;
    }

    /**
     * Gets the publication related to the bizobject
     *
     * @return publication The publication related to the bizobject
     */
    public function getPublication()
    {
        if ($this instanceOf publication)
        {
            return null;
        }

        if (property_exists($this, 'publicationId'))
        {
            $publication = bizobject::create('publication', array(null, $this->publicationId));
        }
        else
        {
            $issue = $this->getIssue();
            if (!$issue)
            {
                return null;
            }

            $publication = $issue->getPublication();
        }

        return ($publication && $publication->id) ? $publication : null;
    }

    /**
     * Gets the 'semantic' text that will be passed to the Text-Mining Engine.
     * IMPORTANT: Overwrite this method as it returns NULL by default!
     *
     * @return string The semantic text to mine (or null if none)
     */
    public function getSemanticText()
    {
        return null;
    }

    /**
     * Gets the language of the object.
     *
     * @return string The language of the object
     */
    public function getLanguage()
    {
        $config = wcmConfig::getInstance();

        $site = $this->getSite();
        if (!$site)
        {
            return $config['wcm.default.language'];
        }

        return ($site->language) ?  $site->language : $config['wcm.default.language'];
    }


    /**
     * Gets the contributions related to the object.
     *
     * @param int $parentId  Optional ID of the parent object (default is NULL)
     *
     * @return array An array of contribution objects
     */
    public function getContributions($parentId = null)
    {
        $where  = "referentId='" . $this->id . "'";
        $where .= " AND referentClass='" . $this->getClass() . "' AND parentId";
        $where .= ($parentId) ? ('=' . $parentId) : ' IS NULL';

        return wcmBizobject::getObjects('contribution', $where);
    }

    /**
     * Gets the number of contributions related to the object.
     *
     * @param int $parentId  Optional ID of the parent object (default is NULL)
     *
     * @return int The number of contributions related to the object
     */
    public function getContributionCount($parentId = null)
    {
        $sql  = 'SELECT COUNT(*) FROM #__contribution WHERE';
        $sql .= ' referentClass=? AND referentId=? AND parentId';
        $sql .= ($parentId === null) ? ' IS NULL' : '=?';

        $params = array($this->getClass(), $this->id, $parentId);

        return $this->database->executeScalar($sql, $params);
    }

    /**
     * Refresh current object from source and sourceId
     *
     * @param string $source Source string
     * @param int $sourceId Source ID
     */
    public function refreshFromSource($source, $sourceId)
    {
        $sql = 'SELECT * FROM ' . $this->tableName . ' WHERE source=:source AND sourceId=:sourceId LIMIT 1';
        $resultSet = $this->database->executeQuery($sql, array('source' => $source, 'sourceId' => $sourceId), null, null, wcmDatabase::FETCH_INTO, $this);
        $resultSet->next();
    }

    /**
     * Returns an XML representation of a property
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     * @return string XML string
     */
    protected function propertyToXML($propKey, $propValue)
    {
        $xml = null;

        // Treat special properties
        if ($propKey == 'xmlTags')
        {
            $xml = '<tag-categories>';

            if (!empty($this->xmlTags) && is_array($this->xmlTags))
            {
                foreach ($this->xmlTags as $category => $tags)
                {
                    $xml .= '<tags category="' . $category . '">';
                    if ($tags)
                    {
                        foreach ($tags as $tag)
                        {
                            $xml .= '<tag>' . wcmXML::xmlEncode($tag) . '</tag>';
                        }

                        $xml .= '</tags>';
                    }
                }
            }
            $xml .= '</tag-categories>';
        }
        elseif ($propKey == 'semanticData' && $this->semanticData instanceof wcmSemanticData)
        {
            $xml = $this->semanticData->toXML();
        }
        else
        {
            if ($propKey == 'publicationDate')
            {
                $propValue = dateToISO8601($propValue);
            }
            $xml = parent::propertyToXML($propKey, $propValue);
        }

        return $xml;
    }

    /**
     * This function can be used to customize the initialisation of a specific property
     * from a XML node (invoked by initFromXML() method)
     *
     * @param string  $property  Property name to initialize
     * @param XMLNode $node      XML node used for initialization
     */
    protected function initPropertyFromXMLNode($property, $node)
    {
        if ($property == 'xmlTags')
        {
            // Assume <tags> <tag category="{category}">{tag}</tag> ... </tags>
            $this->xmlTags = array();
            foreach ($node->childNodes as $child)
            {
                $this->xmlTags[$child->nodeValue] = $child->getAttribute('category');
            }
        }
        elseif ($property == 'semanticData')
        {
            $this->semanticData->fromXML($node);
        }
        elseif ($property == 'publicationDate')
        {
            $this->$property = dateFromISO8601($node->nodeValue);
        }
        else
        {
            parent::initPropertyFromXMLNode($property, $node);
        }
    }

    /**
     * This function can be used to customize the initialisation of a
     * specific property from a JSON assoc value (invoked from the
     * initFromJSON method).
     *
     * @param string $property   Property name to initialize
     * @param mixed  $assocValue JSON assoc value used for initialization
     */
    protected function initPropertyFromJSONAssocValue($property, $assocValue)
    {
        if ($property == 'xmlTags')
        {
            // JSON assoc value is an array of category name => tag list
            $this->xmlTags = array();
            if ($assocValue)
            {
                foreach ($assocValue as $category => $tags)
                {
                    $this->xmlTags[$category] = $tags;
                }
            }
        }
        elseif ($property == 'semanticData')
        {
            $this->semanticData->fromJSONAssocValue($assocValue);
        }
        elseif ($property == 'publicationDate')
        {
            $this->$property = dateFromISO8601($assocValue);
        }
        else
        {
            parent::initPropertyFromJSONAssocValue($property, $assocValue);
        }
    }

    /**
     * Returns the JSON representation of a property.
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     *
     * @return string The JSON representation of the property
     */
    protected function propertyToJSON($propKey, $propValue)
    {
        $json = null;

        if ($propKey == 'xmlTags')
        {
            $json = '"tagCategories":{';
            if ($this->xmlTags)
            {
                foreach ($this->xmlTags as $category => $tags)
                {
                    $json .= '"' . addslashes($category) . '":[';
                    if ($tags)
                    {
                        foreach ($tags as $tag)
                        {
                            $json .= json_encode($tag) . ',';
                        }
                    }
                    $json = rtrim($json, ',');
                    $json .= '],';
                }
            }
            $json = rtrim($json, ',');
            $json .= '}';
        }
        elseif ($propKey == 'semanticData' && $this->semanticData instanceof wcmSemanticData)
        {
            $json = $this->semanticData->toJSON();
        }
        else
        {
            if ($propKey == 'publicationDate')
            {
                $propValue = dateToISO8601($propValue);
            }
            $json = parent::propertyToJSON($propKey, $propValue);
        }

        return $json;
    }

    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool True when object is valid
     *
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        if ($this->source  && strlen($this->source) > 255)
        {
            $this->lastErrorMsg = _BIZ_ERROR_SOURCE_TOO_LONG;
            return false;
        }

        if ($this->sourceId  && strlen($this->sourceId) > 255)
        {
            $this->lastErrorMsg = _BIZ_ERROR_SOURCE_ID_TOO_LONG;
            return false;
        }

        if ($this->sourceVersion  && strlen($this->sourceVersion) > 255)
        {
            $this->lastErrorMsg = _BIZ_ERROR_SOURCE_VERSION_TOO_LONG;
            return false;
        }

        if (property_exists($this, 'siteId') && $this->siteId == '')
        {
            $this->lastErrorMsg = _BIZ_ERROR_SITE_IS_MANDATORY;
            return false;
        }

        if (property_exists($this, 'publicationDate') && $this->publicationDate != '')
        {
            if (!dateCheck($this->publicationDate))
            {
                $this->lastErrorMsg =  _BIZ_ERROR_PUBLICATION_DATE_FORMAT;
                return false;
            }
        }

        if (property_exists($this, 'expirationDate') && $this->expirationDate != '')
        {
            if (!dateCheck($this->expirationDate))
            {
                $this->lastErrorMsg =  _BIZ_ERROR_EXPIRATION_DATE_FORMAT;
                return false;
            }
        }

        if (property_exists($this, 'publicationDate') && property_exists($this, 'expirationDate'))
        {
            if (($this->publicationDate != '' && $this->expirationDate != '') &&
                ($this->publicationDate > $this->expirationDate))
            {
                $this->lastErrorMsg = _BIZ_ERROR_EXP_PUB_DATE_FORMAT;
                return false;
            }
        }

        return true;
    }

    /**
     * Update all the chapters by the chapters given in the array
     *
     * @param array $list  Array of new chapters
     * @param array $newRelations Array of new relations
     */
    public function updateBizRelations($list, $newRelations)
    {
        $this->serialStorage['_list'][] = $list;
        $this->serialStorage[$list] = $newRelations;
    }

    /**
     * Update all the chapters by the chapters given in the array
     *
     * @param array $newZones  Array of new chapters
     */
    public function updateZones($newZones)
    {
        $this->serialStorage['_designZones'] = $newZones;
    }

    /**
     * Update serial storage for a specific key
     *
     * @param string $key Serial storage key
     * @param array $array  Array of new chapters
     */
    public function updateSerialStorage($key, $array)
    {
        $this->serialStorage[$key] = serialize($array);
    }

    /**
     * Gets object ready to store by getting modified date, creation date etc
     * Will execute transition.
     *
     * @return bool True on success, false on failure
     */
    protected function store()
    {
        if (!parent::store())
        {
            return false;
        }

        /**
         * Check if there are design zones and save them if necessary (in the same fashion as bizRelations)
         */
        $newZones = getArrayParameter($this->serialStorage, '_designZones');
        $widgetsettings = getArrayParameter($this->serialStorage, '_widgetsettings', '');
        if ($newZones)
        {
            $contents = array();
            parse_str($newZones, $contents);
            wcmProject::getInstance()->layout->setZoneContents($this, $contents, $newZones, unserialize($widgetsettings));
        }
        unset($this->serialStorage['_designZones']);

        return true;
    }


    /**
     * Execute a transition of the workflow
     *
     * @param  wcmWorkflowTransition $transition the transition to execute
     *
     * @return bool TRUE on success, FALSE on failure (check lastErrorMsg)
     */
    public function executeTransition(wcmWorkflowTransition $transition)
    {
        if (!$this->workflow || !$transition)
        {
            return false;
        }

        // Check validity
        if (($transition->fromState != $this->workflowState) &&
            ($transition->fromState != wcmWorkflowTransition::ANY_STATE) &&
            ($transition->fromState != wcmWorkflowTransition::SELF_STATE))
        {
            $this->lastErrorMsg = 'Transition ' . getConst($transition->name) . ' cannot be executed on ' . $this->getClass() . ' with this state';
            return false;
        }

        // Check permission
        $session = wcmSession::getInstance();
        if (!$session->isAllowed($transition, wcmPermission::P_EXECUTE))
        {
            $this->lastErrorMsg = 'Transition ' . getConst($transition->name) . ' is not allowed to be executed on ' . $this->getClass() . ' for current user';
            return false;
        }

        // Load workflow script
        $script = $this->getWorkflowScript();
        
        // Execute pre-condition workflow
        if ($script)
        {
            // execute beforeXXX or else beforeTransition
            $funcName = 'before' . ucfirst($transition->code);
            if (!method_exists($script, $funcName))
            {
                wcmProject::getInstance()->logger->logVerbose('function ' . $funcName . ' not found for workflow; using beforeTransition as fallback');
                $funcName = 'beforeTransition';
            }
            if (!call_user_func(array($script, $funcName), $transition))
            {
                return false;
            }
        }

        // Change state (except for the magic SELF state)
        $previousState = $this->workflowState;
        if ($transition->toState != wcmWorkflowTransition::SELF_STATE)
        {
            $this->workflowState = $transition->toState;
        }

        // Execute post-processing workflow
        if ($script)
        {
            try
            {
                // execute onXXX or else onTransition
                $funcName = 'on' . ucfirst($transition->code);
                if (!method_exists($script, $funcName))
                {
                    wcmProject::getInstance()->logger->logVerbose('function ' . $funcName . ' not found for workflow; using onTransition as fallback');
                    $funcName = 'onTransition';
                }
                call_user_func(array($script, $funcName), $transition);
            }
            catch (Exception $e)
            {
                // restore previous state
                $this->workflowState = $previousState;

                $this->lastErrorMsg = 'executeTransition (' . $transition->name . ') failed: ' . $e->getMessage();
                wcmProject::getInstance()->logger->logWarning($this->lastErrorMsg);
                return false;
            }
        }

        // save without workflow call
        $result = $this->save(null, true);
        
        // Execute post-save workflow
        if ($result && $script)
        {
            // execute onXXX or else onTransition
            $funcName = 'after' . ucfirst($transition->code);
            if (!method_exists($script, $funcName))
            {
                wcmProject::getInstance()->logger->logVerbose('function ' . $funcName . ' not found for workflow; using afterTransition as fallback');
                $funcName = 'afterTransition';
            }
            call_user_func(array($script, $funcName), $transition);
        }
        
        return $result;
    }

    /**
     * Returns an instance of wcmIWorkflowScript according to
     * current sysobject
     *
     * @return wcmIWorkflowScript An instance of wcmIWorkflowScript or null
     * @exception Exception If an error occurs while retrieving the workflow script.
     */
    public function getWorkflowScript()
    {
        if (false === $this->workflowScript)
        {
            $this->workflowScript = wcmMasterClassRegistry::getWorkflowScript($this);
        }
        return $this->workflowScript;
    }

    /**
     * Get this bizobject's associated workflow (through bizclass)
     *
     * @return  wcmWorkflow Workflow object
     */
    public function getWorkflow()
    {
        return $this->workflow;
    }


    /**
     * Get this bizobject's associated workflow state
     *
     * @return  wcmWorkflowState Workflow state object
     */
    public function getWorkflowState()
    {
        if (!$this->workflow)
        {
            return null;
        }

        $project = wcmProject::getInstance();
        $workflowState = $project->workflowManager->getWorkflowStateByCode($this->workflowState);
        if (!$workflowState)
        {
            $workflowState = $project->workflowManager->getWorkflowStateByCode($workflow->initialState);
        }

        return $workflowState;
    }

    /**
     * Get all available and allowed transitions for this object
     *
     * @return  array Assoc array of wcmWorkflowTransition objects (key is transition code)
     */
    public function getAvailableTransitions()
    {
        $transitions = array();
        if ($this->workflow)
        {
            $session = wcmSession::getInstance();
            $project = wcmProject::getInstance();
            foreach ($project->workflowManager->getTransitions() as $code => $transition)
            {
                if ($transition->workflowId == $this->workflow->id &&
                    ($transition->fromState == wcmWorkflowTransition::ANY_STATE ||
                    $transition->fromState == $this->workflowState))
                {
                    // check execute permission
                    if ($session->isAllowed($transition, wcmPermission::P_EXECUTE))
                    {
                        $transitions[$code] = $transition;
                    }
                }
            }
        }
        return $transitions;
    }
    
    /**
     * Update the URL aliases for the bizobject.
     * @param array $aliases The new set of URL aliases (as string).
     * @return boolean True if success, false if a
     * duplicate was detected.
     * @exception InvalidArgumentException If the 
     * bizobject doesn't have a siteId.
     * @exception Exception If an error occurs during save.
     */
    public function updateUrlAliases(array $aliases)
    {
        if (!property_exists($this, 'siteId'))
        {
            throw new InvalidArgumentException('Bizobjects with no siteId field cannot have URL aliases.');
        }
        
        // Load existing aliases and create a map (alias -> object).
        $currentObjects = objectUrlAlias::getAll($this);
        $map = array();
        foreach ($currentObjects as $object)
        {
            $map[$object->alias] = $object;
        }
        $currentAliases = array_keys($map);
        
        // Do a diff between new aliases and current aliases.
        $aliasesToAdd = array_diff($aliases, $currentAliases);

        // Make sure the new aliases do not already exist for another object.
        if (objectUrlAlias::existOne($aliasesToAdd, $this->getSite()))
        {
            $this->setErrorMsg(_BIZ_URL_ALIAS_DUPLICATE);
            return false;
        }

        // Create the new aliases.
        foreach ($aliasesToAdd as $alias)
        {
            $object = new objectUrlAlias();
            $object->setObject($this);
            $object->siteId = $this->siteId;
            $object->alias = $alias;
            if (!$object->save())
            {
                throw new Exception($object->getErrorMsg());
            }
        }
        
        // Do a diff between current aliases and new aliases.
        $aliasesToDelete = array_diff($currentAliases, $aliases);
        
        // Delete removed aliases.
        $objectsToDelete = array();
        foreach ($aliasesToDelete as $alias)
        {
            $objectsToDelete[] = $map[$alias];
        }
        objectUrlAlias::deleteAll($objectsToDelete);
        
        return true;
    }
}
