<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.connector.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmConnector class represents an abstract connection
 * to a database (rdbms, xmldata, ...)
 */
class wcmConnector extends wcmSysobject
{
    /**
     * Connector name
     * @var string
     */
    public $name = null;

    /**
    * Unique reference representing this connector
    * @var string
    */
    public $reference = null;

    /**
    * Connection string (Creole PEAR/DSN style)
    * @var string
    */
    public $connectionString = null;

    /**
    * Table prefix (default is 'biz_')
    * @var string
    */
    public $tablePrefix = 'biz_';

    /**
     * Pointer to related business database
     * @var wcmDatabase
     */
    protected $businessDatabase = null;

    /**
     * Check validity of object
     *
     * @return bool TRUE if object is valid
     */
    public function checkValidity()
    {
        if (trim($this->name . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAME_IS_MANDATORY;
            return false;
        }

        if (strlen($this->name) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAME_TOO_LONG;
            return false;
        }

        if (trim($this->reference . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_REFERENCE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->reference) > 64)
        {
            $this->lastErrorMsg = _ERROR_REFERENCE_TOO_LONG;
            return false;
        }

        if (trim($this->connectionString . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_CONNECTIONSTRING_IS_MANDATORY;
            return false;
        }

        if (trim($this->tablePrefix . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_TABLEPREFIX_IS_MANDATORY;
            return false;
        }

        if (strlen($this->tablePrefix) > 32)
        {
            $this->lastErrorMsg = _ERROR_TABLEPREFIX_TOO_LONG;
            return false;
        }

        $db = $this->getDatabase();
        $id = $db->executeScalar('SELECT id FROM wcm_connector WHERE reference=?', array($this->reference));
        if ($id && $id != $this->id)
        {
            $this->lastErrorMsg = _ERROR_CONNECTOR_CODE_INVALID;
            return false;
        }

        //check the connection string validity
        try {
            $tempdb = new wcmDatabase(trim($this->connectionString));
            $connection = $tempdb->getConnection();
        } 
        catch (Exception $e)
        {
            $connection = false;
        }
        if (!$connection)
        {
            $this->lastErrorMsg = _ERROR_CONNECTION_STRING;
            return false;
        }

        return true;
    }

    /**
     * Refresh connector using a reference
     *
     * @param string $reference An existing connector reference
     *
     * @return wcmConnector freshen connector or null on failure
     */
    public function refreshByReference($reference)
    {
        $id = $this->database->executeScalar('SELECT id FROM ' . $this->tableName . ' WHERE reference=?', array($reference));
        return $this->refresh($id);
    }

    /**
     * Returns an instance of a wcmDatabase object
     *
     * @return wcmDatabase A wcmDtabase object matching connector parameters
     */
    public function getBusinessDatabase()
    {
        return new wcmDatabase($this->connectionString, $this->tablePrefix);
    }

    /**
     * Inserts or Updates object in database
     *
     * @param int     $userId Id of the wcmUser who is creating or updating the object
     *
     * @return bool true on success, false on failure
     */
    protected function store($userId = null)
    {
        if (!parent::store($userId))
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->setElem($this->getType(), $this->id, $this);

        return true;
    }

    /**
     * Deletes current connector from database (and associated tables and fields)
     *
     * @return bool true on success or false on error
     */
    public function delete()
    {
        $oldId = $this->id;

        //make sure the connector is not used by any bizclass
        $result = $this->database->executeQuery('SELECT id FROM wcm_bizclass WHERE connectorId =?', array($this->id));
        if (($result && $result->count() > 0) || $this->reference == 'biz')
        {
            $this->lastErrorMsg = _ERROR_CONNECTOR_USED;
            return false;
        }

        if (!parent::delete())
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->unsetElem($this->getType(), $oldId);

        return true;
    }
}
