<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.generation.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The Generation class represents a generic PnW generation
 * A generation contains one or more {@link wcmGenerationContent} objects
 */
class wcmGeneration extends wcmSysobject
{
    /**
     * Object name
     * @var string
     */
    public $name = null;

    /**
     * Object code
     * @var string
     */
    public $code = '';

    /**
     * Path (relative or absolute) where to generation contents
     * @var string
     */
    public $location = '';

    /**
     * Generation context
     * => A string representing ids of contextual classes (or null)
     * => Example: "story.id=3, section.id=6"
     * @var string
     */
    public $context = '';

    /**
     * Id of generation set this generation belongs to
     *
     * @var int
     */
    public $generationSetId;

    /**
     * Refresh object using his code
     *
     * @param string $code optional argument, if not specifed current code is used
     *
     * @return wcmGeneration freshen object or null on failure
     *
     */
    public function refreshByCode($code = null)
    {
        if (!isset($code))
        {
            $code = $this->code;
        }

        $sql = 'SELECT id FROM ' . $this->getTableName() . ' WHERE code=?';
        $id = $this->database->executeScalar($sql, array($code));
        return $this->refresh($id);
    }


    /**
     * Returns an array of contents belonging to current generation
     *
     * @return array An associative array of {@link wcmGenerationContent} objects (keys are ids)
     */
    public function getContents()
    {
        // Use cache
        $contents = array();
        foreach ($this->getProject()->generator->getGenerationContents() as $content)
        {
            if ($content->generationId == $this->id)
            {
                $contents[$content->id] = $content;
            }
        }

        return $contents;
    }

    /**
     * Inserts or Updates object in database
     *
     * @param int     $userId Id of the wcmUser who is creating or updating the object
     *
     * @return bool true on success, false on failure
     */
    protected function store($userId = null)
    {
        if (!$this->checkUniqueCode($this->code))
        {
            return false;
        }

        if (!parent::store($userId))
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->setElem($this->getType(), $this->id, $this);

        return true;
    }

    /**
     * Delete generation object from database (and associated generation contents)
     *
     * @return bool True on success, false otherwise
     */
    public function delete()
    {
        $isSuccess = true;

        // Delete generation contents
        foreach ($this->getContents() as $content)
        {
            if (!$content->delete())
            {
                $isSuccess = false;
            }
        }

        if (!parent::delete())
        {
            $isSuccess = false;
        }

        // Update cache
        wcmCacheRegistry::get()->unsetElem($this->getType(), $this->id);

        return $isSuccess;
    }

    /**
     * Check validity of object
     *
     * @return bool true if object is valid, false otherwise
     */
    public function checkValidity()
    {
        if (trim($this->name . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAME_IS_MANDATORY;
            return false;
        }

        if (strlen($this->name) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAME_TOO_LONG;
            return false;
        }

        if (trim($this->code . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_CODE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->code) > 255)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        if (strlen($this->location) > 255)
        {
            $this->lastErrorMsg = _ERROR_LOCATION_TOO_LONG;
            return false;
        }

        if (strlen($this->context) > 255)
        {
            $this->lastErrorMsg = _ERROR_CONTEXT_TOO_LONG;
            return false;
        }

        return true;
    }
}
