<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.generationContent.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class represents a content belonging to a {@link wcmGeneration}
 */
class wcmGenerationContent extends wcmSysobject
{
    /**
     * Object name
     * @var string
     */
    public $name = null;

    /**
     * Object code
     * @var string
     */
    public $code = '';

    /**
     * Id of generation on which current content belongs
     * @var int
     */
    public $generationId = null;

    /**
     * Id of template to generate
     * @var int
     */
    public $templateId = null;

    /**
     * Loop parameters (or null)
     * => Example: "name=myItem class=story where='workflowState=valid' from=0 to=6"
     * @var string
     */
    public $loop = '';

    /**
     * Generation context
     * => A string representing ids of contextual classes (or null)
     * => Example: "story.id=3, section.id=6"
     * @var string
     */
    public $context = '';

    /**
     * A pattern (template) representing how to name the created file(s)
     * => Example: "Page_{story.title}.html"
     * @var string
     */
    public $namingRule = '';

    /**
     * Refresh object using his code
     *
     * @param string $code optional argument, if not specifed current id is used
     *
     * @return wcmGenerationContent freshen object or null on failure
     *
     */
    public function refreshByCode($code = null)
    {
        if (!isset($code))
        {
            $code = $this->code;
        }

        $sql = 'SELECT id FROM ' . $this->getTableName() . ' WHERE code=?';
        $id = $this->database->executeScalar($sql, array($code));
        return $this->refresh($id);
    }

    /**
     * Returns the {@link wcmGeneration} associated to this content
     *
     * @return wcmGeneration The generation object associated to current content
     */
    public function getGeneration()
    {
        return $this->getProject()->generator->getGenerationById($this->generationId);
    }

    /**
     * Returns the {@link wcmTemplate} associated to this content
     *
     * @return wcmTemplate The template associated to this content
     */
    public function getTemplate()
    {
        return new wcmTemplate(null, $this->templateId);
    }

    /**
     * Computes the sql where clause matching foreign constraints
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    public function ofClause($of)
    {
        if ($of == null || !is_array($of))
        {
            return null;
        }

        $sql = null;
        foreach ($of as $key => $value)
        {
            switch ($key)
            {
                case 'wcmGeneration':
                    return ('generationId=' . $value);
                    break;
            }
        }
        return $sql;
    }

    /**
     * Inserts or Updates object in database
     *
     * @param int     $userId Id of the wcmUser who is creating or updating the object
     *
     * @return bool true on success, false on failure
     */
    protected function store($userId = null)
    {
        if (!$this->checkUniqueCode($this->code))
        {
            return false;
        }

        if (!parent::store($userId))
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->setElem($this->getType(), $this->id, $this);

        return true;
    }

    /**
     * Delete generationContent object from database
     *
     * @return bool True on success, false otherwise
     */
    public function delete()
    {
        if (!parent::delete())
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->unsetElem($this->getType(), $this->id);

        return true;
    }

    /**
     * Check validity of object
     *
     * @return bool TRUE if object is valid
     */
    public function checkValidity()
    {
        if (trim($this->name . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAME_IS_MANDATORY;
            return false;
        }

        if (strlen($this->name) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAME_TOO_LONG;
            return false;
        }

        if (trim($this->code . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_CODE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->code) > 255)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        if (trim($this->templateId . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_TEMPLATE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->templateId) > 255)
        {
            $this->lastErrorMsg = _ERROR_TEMPLATE_TOO_LONG;
            return false;
        }

        if (trim($this->namingRule . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAMINGRULE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->namingRule) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAMINGRULE_TOO_LONG;
            return false;
        }

        if (strlen($this->loop) > 255)
        {
            $this->lastErrorMsg = _ERROR_LOOP_TOO_LONG;
            return false;
        }

        if (strlen($this->context) > 255)
        {
            $this->lastErrorMsg = _ERROR_CONTEXT_TOO_LONG;
            return false;
        }

        return true;
    }
}
