<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.generator.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The generator class contains methods to help
 * management of templates and sites generation
 *
 */
class wcmGenerator
{
    /**
     * Cache of generations
     *
     * @var array
     */
    public $generations;

    /**
     * Cache of generation sets
     *
     * @var array
     */
    public $generationSets;

    /**
     * Cache of generationContents
     *
     * @var array
     */
    public $generationContents;

    /**
     * Cache of templates
     *
     * @var array
     */
    public $templates;

    /**
     * Cache of templateCategories
     *
     * @var array
     */
    public $templateCategories;


    /**
     * Returns an array of categories matching a specific path
     *
     * @param string  $path      Path on disc where to look for categories
     *
     * @return array An array of categories/directories
     */
    public function getCategories($path = null)
    {
        $config = wcmConfig::getInstance();

        if (!$path)
        {
            $path = $config['wcm.templates.path'];
        }

        $this->templateCategories = array();
        $dir_handle  =  opendir($path);

        if ($dir_handle === false)
        {
            throw new Exception('Unable  to  open  path');
        }

        while (false !== ($resource = readdir($dir_handle)))
        {
            if (is_dir($path . $resource) && $resource != '.' && $resource != '..' && $resource != '.svn')
            {
                $this->templateCategories[] = $resource;
            }
        }

        return $this->templateCategories;
    }

     /**
     * Returns an temples in a given path
     *
     * @param string  $relativePath      Path on disc where to look for templates
     *
     * @return array An array of template files key => value where key is template name (w/o .tpl) abd value
     * is the absolute path.template name
     */
    public function getTemplatesByPath($relativePath = '')
    {
        $config = wcmConfig::getInstance();
        $path = $config['wcm.templates.path'] . $relativePath;

        if ($relativePath && substr($relativePath, -1) != '/')
        {
            $relativePath .= '/';
        }

        $this->templates = array();

        if (is_dir($path))
        {
            $dir = new DirectoryIterator($path);

            foreach ($dir as $resource)
            {
                if ($resource->isFile() && substr($resource->getFilename(), -4) == '.tpl')
                {
                    $code = str_replace('.tpl', '', $resource->getFilename());
                    $this->templates[$code] = $relativePath . $resource->getFilename();
                }
            }
        }

        return $this->templates;
    }

    /**
     * Return a generation object corresponding to a specific id (or null)
     *
     * @param int $id The generation id
     *
     * @return wcmGeneration The generation having given id (or null if id is invalid)
     */
    public function getGenerationById($id)
    {
        // Get list from cache
        return getArrayParameter($this->getGenerations(), $id, null);
    }

    /**
     * Return a generation object corresponding to a specific code (or null)
     *
     * @param string    $code   The generation Code
     *
     * @return wcmGeneration The generation having given code (or null if code is invalid)
     */
    public function getGenerationByCode($code)
    {
        foreach ($this->getGenerations() as $generation)
        {
            if ($generation->code === $code)
            {
                return $generation;
            }
        }

        return null;
    }

    /**
     * Returns an array of generations
     *
     * @param bool $resetCache Optional whether to reset the cache, ie. load from DB (default is false)
     *
     * @return array An assoc array of {@link wcmGeneration} objects (keys are ids)
     */
    public function getGenerations($resetCache = false)
    {
        // Cache objects
        $cached = wcmCacheRegistry::get()->fetch('wcmGeneration');
        if ($resetCache || $cached === false)
        {
            $cached = array();
            $enum = new wcmEnumeration('wcmGeneration');

            foreach ($enum as $generation)
            {
                $cached[$generation->id] = $generation;
            }

            // Cache objects
            wcmCacheRegistry::get()->store('wcmGeneration', $cached);
            wcmCacheRegistry::get()->delete('wcmGenerationContent');
        }

        return $cached;
    }

    /**
     * Return a generationSet object corresponding to a specific id (or null)
     *
     * @param int $id The generation id
     *
     * @return wcmGenerationSet The generation set having given id (or null if id is invalid)
     */
    public function getGenerationSetById($id)
    {
        // Get list from cache
        return getArrayParameter($this->getGenerationSets(), $id, null);
    }


    /**
     * Return a generationSet object corresponding to a specific code (or null)
     *
     * @param string    $code   The generation Code
     *
     * @return wcmGenerationSet The generatiokn set having given code (or null if code is invalid)
     */
    public function getGenerationSetByCode($code)
    {
        foreach ($this->getGenerationSets() as $generationSet)
        {
            if ($generationSet->code === $code)
            {
                return $generationSet;
            }
        }

        return null;
    }


    /**
     * Returns an array of generation sets
     *
     * @param bool $resetCache Optional whether to reset the cache, ie. load from DB (default is false)
     *
     * @return array An assoc array of {@link wcmGeneration} objects (keys are ids) or null on failure
     */
    public function getGenerationSets($resetCache = false)
    {
        // Cache objects
        $cached = wcmCacheRegistry::get()->fetch('wcmGenerationSet');
        if ($resetCache || $cached === false)
        {
            $cached = array();
            $enum = new wcmEnumeration('wcmGenerationSet');

            foreach ($enum as $generationSet)
            {
                $cached[$generationSet->id] = $generationSet;
            }

            // Cache objects
            wcmCacheRegistry::get()->store('wcmGenerationSet', $cached);
            wcmCacheRegistry::get()->delete('wcmGeneration');
            wcmCacheRegistry::get()->delete('wcmGenerationContent');
        }

        return $cached;
    }

    /**
     * Return a generation content object corresponding to a specific id (or null)
     *
     * @param int $id The content generation id
     *
     * @return wcmGenerationContent The generation content having given id (or null if id is invalid)
     */
    public function getGenerationContentById($id)
    {
        // Get list from cache
        return getArrayParameter($this->getGenerationContents(), $id, null);
    }

    /**
     * Return a generationContent object corresponding to a specific code (or null)
     *
     * @param string    $code   The generationContent Code
     *
     * @return wcmGenerationContent The generation content having given code (or null if code is invalid)
     */
    public function getGenerationContentByCode($code)
    {
        foreach ($this->getGenerationContents() as $generationContent)
        {
            if ($generationContent->code === $code)
            {
                return $generationContent;
            }
        }

        return null;
    }


    /**
     * Returns an array of generation contents
     *
     * @param bool $resetCache Optional whether to reset the cache, ie. load from DB (default is false)
     *
     * @return array An assoc array of {@link wcmGenerationContent} objects (keys are ids)
     */
    public function getGenerationContents($resetCache = false)
    {
        // Cache objects
        $cached = wcmCacheRegistry::get()->fetch('wcmGenerationContent');
        if ($resetCache || $cached === false)
        {
            $enum = new wcmEnumeration('wcmGenerationContent');
            $cached = array();

            foreach ($enum as $gc)
            {
                $cached[$gc->id] = $gc;
            }

            // Cache objects
            wcmCacheRegistry::get()->store('wcmGenerationContent', $cached);
        }

        return $cached;
    }
}

