<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.MVC_Action.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class implements the default system action for the
 * MVC controller
 *
 * Basically it instanciate the sysobject matching $_REQUEST['_wcmClass']
 * and execute the specific action according to $_REQUEST['_wcmTodo']
 */
abstract class wcmMVC_Action
{
    /**
     * Action ID
     * @var int
     */
    protected $id;

    /**
     * Action class
     * @var string
     */
    protected $class;

    /**
     * Acton parameters
     * @var array
     */
    protected $params;

    /**
     * Action tree ID
     * @var int
     */
    protected $treeId;

    /**
     * Action to execute
     * @var string
     */
    public $todo;

    /**
     * Instance of object used for current action
     * @var object
     */
    public $context;

    /**
     * Redirect URL
     * @var string
     */
    private $redirectUrl;

    /**
     * The constructor automatically instanciate the context,
     * execute the action and store result in session
     *
     * @param array $parameters An array of initial parameters to protected properties (id, todo, class, ...)
     */
    public function __construct(array $parameters = null)
    {
        if (!is_array($parameters))
        {
            $parameters = array();
        }
        $this->params = $parameters;

        // Set initial properties
        $this->id       = getArrayParameter($parameters, 'id', getArrayParameter($_REQUEST, 'id', null));
        if ($this->id !== null)
        {
            $this->id = intval($this->id);
        }
        $this->todo     = getArrayParameter($parameters, 'todo', getArrayParameter($_REQUEST, '_wcmTodo', null));
        $this->class    = getArrayParameter($parameters, 'class', getArrayParameter($_REQUEST, '_wcmClass', null));
        $this->treeId   = getArrayParameter($parameters, 'tree', wcmSession::getInstance()->getCurrentAction());


        // Get context and execute action
        $session = wcmSession::getInstance();
        $project = wcmProject::getInstance();
        $this->setContext($session, $project);

        // Store new state
        $this->store();
    }

    /**
     * Store current action in session
     */
    protected function store()
    {
        // Memorize current action in $_SESSION
        if (!isset($_SESSION['wcmAction']))
        {
            $_SESSION['wcmAction'] = array();
        }
        $_SESSION['wcmAction']['todo'] = $this->todo;
        $_SESSION['wcmAction']['treeId'] = $this->treeId;
        $_SESSION['wcmAction']['context'] = $this->context;
    }

    /**
     * Redirect page to a new URL
     *
     * @param string $url New URL
     * @param bool $now Define if you want to do the redirect now (By default false)
     */
    protected function redirect($url, $now = false)
    {
        if ($now)
        {
            header('Location: ' . $url);
            die();
        }
        else
        {
            $this->redirectUrl = $url;
        }
    }

    /**
     * Instanciate context
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     *
     * @return object Instanciated context
     */
    protected abstract function setContext($session, $project);

    /**
     * Default action executed when $todo is not set
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     *
     * @return string Action message to display
     */
    protected abstract function on($session, $project);

    /**
     * Execute specific action
     *
     * If the before action method return false, the action will not be executed
     *
     * @param String $action Action to execute
     */
    public function executeAction($action)
    {

        $session = wcmSession::getInstance();
        $project = wcmProject::getInstance();

        $method = 'on' . ucfirst($action);
        try {
            if (method_exists($this, $method))
            {
                $beforeMethod = 'before' . ucfirst($action);
                if (method_exists($this, $beforeMethod))
                {
                    $this->$beforeMethod($session, $project);
                }

                $this->$method($session, $project);

                $afterMethod = 'after' . ucfirst($action);
                if (method_exists($this, $afterMethod))
                {
                    $this->$afterMethod($session, $project);
                }
            }
            else
            {
                if (method_exists($this, 'before'))
                {
                    $this->before($session, $project);
                }

                $this->on($session, $project);

                if (method_exists($this, 'after'))
                {
                    $this->after($session, $project);
                }
            }

            if (null !== $this->redirectUrl)
            {
                $this->redirect($this->redirectUrl, true);
            }
        }
        catch (wcmForwardException $exception)
        {
            if ($exception->getMessage() !== $action)
            {
                $this->executeAction($exception->getMessage());
            }
        }
        catch (wcmAbortException $exception)
        {
            self::setMessage($exception->getMessage(), $exception->getCode());
        }
    }

    /**
     * Clear last action message
     */
    public static function clearMessage()
    {
        self::setMessage(null);
    }

    /**
     * Stop the execution of the current controller and execute the new one instead
     *
     * @param String $newAction New action to execute
     * @throws wcmForwardException
     */
    public static function forward($newAction)
    {
        throw new wcmForwardException($newAction);
    }

    /**
     * Defines the last action message
     *
     * @param string $message Last action message
     * @param int $messageKind One of WCMLOG_MESSAGE, WCMLOG_WARNING or WCMLOG_ERROR
     */
    public static function setMessage($message, $messageKind = WCMLOG_MESSAGE)
    {
        if (!isset($_SESSION['wcmAction']))
        {
            $_SESSION['wcmAction'] = array();
        }
        $_SESSION['wcmAction']['messageKind'] = $messageKind;
        $_SESSION['wcmAction']['message'] = $message;
    }

    /**
     * Defines the last action message (as a warning message)
     *
     * @param string $message Last action message
     * @throws wcmAbortException
     */
    public static function setWarning($message)
    {
        throw new wcmAbortException($message, WCMLOG_WARNING);
    }

    /**
     * Defines the last action message (as a warning message)
     *
     * @param string $message Last action message
     * @throws wcmAbortException
     */
    public static function setError($message)
    {
        throw new wcmAbortException($message, WCMLOG_ERROR);
    }

    /**
     * Returns the last action message
     *
     * @return string Last action message
     */
    public static function getMessage()
    {
        return  (!isset($_SESSION['wcmAction'])) ? null : getArrayParameter($_SESSION['wcmAction'], 'message', null);
    }

    /**
     * Returns the last action message's kind
     *
     * @return int Last action message's kind
     */
    public static function getMessageKind()
    {
        return (!isset($_SESSION['wcmAction'])) ? null : getArrayParameter($_SESSION['wcmAction'], 'messageKind', 0);
    }

    /**
     * Returns the current URL
     *
     * @param string $newAction Optional action to override current action
     * @param string $newTodo Optional todo to override current todo
     * @param int $newId Optional id to override current id
     */
    public static function getURL($newAction=null, $newTodo=null, $newId=0)
    {
        $config = wcmConfig::getInstance();
        $url = $config['wcm.backOffice.url'];

        $url .= '?_wcmAction=' . ($newAction) ? $newAction : self::getAction();
        $url .= '&amp;_wcmTodo=' . ($newTodo) ? $newTodo : self::getTodo();
        if ($newId)
        {
            $url .= '&amp;id=' . $newId;
        }
        else
        {
            $context = self::getContext();
            if ($context && $context->id)
            {
                $url .= '&amp;id=' . $context->id;
            }
        }

        return $url;
    }

    /**
     * Returns the current action
     *
     * @return string Current action
     */
    public static function getAction()
    {
        return wcmSession::getInstance()->getCurrentAction();
    }

    /**
     * Returns the tree id
     *
     * @return string Tree id
     */
    public static function getTreeId()
    {
        return (!isset($_SESSION['wcmAction'])) ? null : getArrayParameter($_SESSION['wcmAction'], 'treeId', null);
    }

    /**
     * Returns the action to-do
     *
     * @return string Action to-do
     */
    public static function getTodo()
    {
        return (!isset($_SESSION['wcmAction'])) ? null : getArrayParameter($_SESSION['wcmAction'], 'todo', null);
    }

    /**
     * Returns the last action sysobject
     *
     * @return wcmObject Last action sysobject
     */
    public static function getContext()
    {
        return (!isset($_SESSION['wcmAction'])) ? null : getArrayParameter($_SESSION['wcmAction'], 'context', null);
    }

    /**
     * Returns a human-friendly title
     *
     * @return string A title for current action
     */
    public static function getTitle()
    {
        $title = wcmProject::getInstance()->title . ' :: ';

        $context = self::getContext();
        if ($context)
        {
            if ($context instanceOf wcmSysobject)
            {
                $title .= getConst($context->getMasterclass()->name);
            }
            elseif ($context instanceOf wcmObject)
            {
                $title .= get_class($context);
            }
            if (isset($context->id) && $context->id)
            {
                $title .= ' #' . $context->id;
            }
        }
        else
        {
            $title .= ucfirst(wcmSession::getInstance()->getCurrentAction());
        }

        return $title;
    }

    /**
     * Returns the tree associated to the last action
     *
     * @return wcmTree Last tree (or null)
     */
    public static function getTree()
    {
        $config = wcmConfig::getInstance();
        $treeParts = explode('/', self::getTreeId());
        $treeId = array_pop($treeParts);
        if (substr(self::getTreeId(), 0, 9) === 'business/')
        {
            $url = $config['wcm.backOffice.url'] . '/business';
            $modulePath = WCM_WWW_BACKOFFICE_DIR . '/system/private/views/modules/tree/';
        }
        else
        {
            $url = null;
            $modulePath = null;
        }
        $tree = new wcmTree($treeId, $url, null, null, null, $modulePath);
        $tree->initFromSession();

        return $tree;

    }

    /**
     * Refresh the tree associated to the last action
     *
     */
    public static function refreshTree()
    {
        $tree = self::getTree();
        $tree->refresh();
        $tree->saveIntoSession();
    }

    /**
     * Update the context (used by modules AJAX calls
     * to patch the concurrency problem with multiple
     * browser tabs or windows).
     * @param wcmObject $object
     */
    public static function updateAjaxContext($id, $class)
    {
        $object = wcmMVC_Action::getContext();
        if (!$object || get_class($object) != $class || $object->id != $id)
        {
            // The context in session is not the right one,
            // so update it.

            $object = new $class();
            $object->refresh($id);

            // Memorize current action in $_SESSION
            if (!isset($_SESSION['wcmAction']))
            {
                $_SESSION['wcmAction'] = array();
            }

            $_SESSION['wcmAction']['todo'] = null;
            $_SESSION['wcmAction']['treeId'] = null;
            $_SESSION['wcmAction']['context'] = $object;
        }
    }

    /**
     * Compute URL to view a specific object
     *
     * @param string $objectClass Name of object class
     * @param int $objectId Id of object
     * @param string $todo Optional todo ('view' by default)
     *
     * @return string URL associated to sysobject
     */
    public static function computeObjectURL($objectClass, $objectId = null, $todo = null, array $params = array())
    {
        // Assume all system class starts with 'wcm' prefix
        if (substr($objectClass, 0, 3) == 'wcm')
        {
            $action = lcfirst(substr($objectClass, 3));
        }
        else
        {
            $action = 'business/' . $objectClass;
        }

        if ($objectId !== null)
        {
            if (!is_array($params))
            {
                $params = array();
            }
            $params['id'] = $objectId;
        }

        return self::computeURL($action, $todo, $params);
    }

    /**
     * Compute the URL corresponding to a specific action controller
     *
     * @param string $action    Action name (or null for current)
     * @param string $todo      Action to execute
     * @param array  $params    Extra parameters (optional assoc array)
     */
    public static function computeURL($action = null, $todo = null, array $params = array())
    {
        $config  = wcmConfig::getInstance();
        $url = $config['wcm.backOffice.url'];

        $params['_wcmAction'] = ($action) ? $action : wcmSession::getInstance()->getCurrentAction();

        if ($todo)
        {
            $params['_wcmTodo'] = $todo;
        }


        return $url . '?' . http_build_query($params);
    }

    /**
     * Execute system action.
     *
     * This function will try to locate {$action}.php in the 'actions' directory and assume
     * that the file contains a class named [wcm]{$action}Action which extends wcmMVC_Action
     * if the file does not exists, the function will instanciate wcmMVC_Sys|BizAction
     *
     * @param string $action   Action to execute
     * @param array  $params   Default parameters (such as 'class' => {aClassName}, ...)
     *
     * @return wcmMVC_Action   An instance of the executed action
     */
    public static function getActionController($action, $params = null)
    {
        // Set current action
        wcmSession::getInstance()->setCurrentAction($action);

        // Check file existence for action and determine class name
        if (substr($action, 0, 9) == 'business/')
        {
            $className = substr($action, 9) . 'Action';
            if (!autoloadManager::classExists($className))
            {
                $className = 'mvcAction';
            }
        }
        else
        {
            $className = 'wcm' . ucfirst($action) . 'Action';
            if (!autoloadManager::classExists($className))
            {
                $className = 'wcmMVC_SysAction';
            }
        }

        // Instanciate action
        return new $className($params);
    }

    /**
     * Execute method will automatically create the action controller and execute
     * the action to do
     *
     * @param string $action   Action to execute
     * @param array  $params   Default parameters (such as 'class' => {aClassName}, ...)
     *
     * @return wcmMVC_Action   An instance of the executed action
     */
    public static function execute($action, $params = null)
    {
        $actionController = self::getActionController($action, $params);

        // Execute action
        $actionController->executeAction(self::getTodo());

        return $actionController;
    }


    public function renderView()
    {
        $session = wcmSession::getInstance();
        $project = wcmProject::getInstance();
        $config = wcmConfig::getInstance();

        // Load page for current action
        $action = $session->getCurrentAction();
        if (substr($action, 0, 9) == 'business/')
        {
            /**
             * TODO : Put constant here
             */
            $page = WCM_WWW_BACKOFFICE_DIR . '/business/private/views/pages/' . substr($action, 9) . '.php';
        }
        else
        {
            $page = WCM_WWW_BACKOFFICE_DIR . '/system/private/views/pages/' . $action . '.php';
        }

        if (!file_exists($page))
        {
            // TODO : This is temporary
            var_dump($page . ' doesn\'t exists !');
            header('Location: ' . wcmMVC_Action::computeURL('404'));
        }
        else
        {
            try
            {
                require_once($page);
            }
            catch (Exception $e)
            {
                $_EXCEPTION = $e;
                $page = WCM_WWW_BACKOFFICE_DIR . '/system/private/views/pages/exception.php';
                require_once($page);
            }
        }
    }
}
