<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.MVC_BizAction.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */


/**
 * This class implements the default business action for the
 * MVC controller
 *
 * Basically it instanciate the bizobject matching $_REQUEST['_wcmClass']
 * and execute the specific action according to $_REQUEST['_wcmTodo']
 */
class wcmMVC_BizAction extends wcmMVC_SysAction
{
    /**
     * Instanciate context (usually a sysobject)
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function setContext($session, $project)
    {
        parent::setContext($session, $project);

        $context = $this->context;

        // Initialize site id
        if ($context && property_exists($context, 'siteId'))
        {
            if ($context->siteId == null)
            {
                $context->siteId = $session->getSiteId();
            }

            // check site permission
            if (!$session->isAllowed('site_' . $context->siteId, wcmPermission::P_READ))
            {
                $this->redirect('?_wcmAction=403', true);
            }
        }

        // Initialize tags and semanticData
        unset($_SESSION['tags']);
        unset($_SESSION['semanticData']);
        if ($context)
        {
            $_SESSION['tags'] = $context->xmlTags;
            $_SESSION['semanticData'] = $context->semanticData;
        }
    }

    /**
     * onChangeSite
     *
     * Executed when an object has to change site IDs and nothing else.
     * Generally called if a site was deleted, but not objects belonging to that site.
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onChangeSite($session, $project)
    {
        $this->context->siteId = $_REQUEST['newSiteId'];
        if (!$this->context->save())
        {
            wcmMVC_Action::setError(_BIZ_ERROR . $this->context->getErrorMsg());
            return;
        }
        $this->redirect(self::computeObjectURL($this->context->getClass(), $this->context->id));
    }

    /**
     * onDelete
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onDelete($session, $project)
    {
        unset($_SESSION['tags']);
        unset($_SESSION['semanticData']);

        parent::onDelete($session, $project);
    }

    /**
     * beforeSaving is called by onCheckin and onSave before storing the sysobject
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function beforeSave($session, $project)
    {
        $this->updateTags();
        $this->updateRelations();
        $this->updateSemanticData();
        $this->updateUrlAliases();
        $this->saveZones();
    }

    /**
     * onPublish
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onPublish($session, $project)
    {
        if ($this->context && method_exists($this->context, 'generate'))
        {
            set_time_limit(3600);
            $logger = new wcmLogger(false, false, null, false);
            $this->context->generate(true, $logger);
            $html = '<div class="header"> ' . _BIZ_GENERATION_RESULT . '</div>';
            $html .= '<ul>' . $logger->display(true) . '</ul>';
            unset($logger);

            // Add statistics
            $session->addStat(wcmSession::STAT_PUBLISH_OBJECT, $this->context, $html);

            // Send message
            wcmMVC_Action::setMessage('Object has been published');
        }
    }

    /**
     * Update tags
     */
    private function updateTags()
    {
        // Retrieve tags from session
        if (isset($_REQUEST['_xmlTags']))
        {
            foreach ($_REQUEST['_xmlTags'] as $type => $tags)
            {
                if ('' != $tags)
                {
                    $this->context->xmlTags[$type] = explode('|', $tags);
                }
                else
                {
                    unset($this->context->xmlTags[$type]);
                }
            }
        }
    }
    
    /**
     * Update the URL aliases.
     * @return void
     */
    private function updateUrlAliases()
    {
        // Retrieve tags from session
        if (isset($_REQUEST['_urlAliases']))
        {
            $aliases = explode('|', $_REQUEST['_urlAliases']);
            foreach ($aliases as & $alias)
            {
                if ((substr($alias, 0, 1) != '/') && (strlen($alias) > 0))
                {
                    $alias = '/' . $alias;
                }
            }
            
            try
            {
                $result = $this->context->updateUrlAliases($aliases);
            }
            catch (Exception $e)
            {
                throw new wcmAbortException($e->getMessage());
            }
            
            if (!$result)
            {
                throw new wcmAbortException($this->context->getErrorMsg());
            }
        }
    }

    /**
     * Update bizrelations
     */
    private function updateRelations()
    {
        // If we have a list save it in the bizobject
        if ($lists = getArrayParameter($_REQUEST, '_list'))
        {
            foreach ($lists as $pk)
            {
                $this->context->updateRelations($pk, getArrayParameter($_REQUEST, '_list' . $pk, array()));
            }
        }
    }

    /**
     * Update semantic data
     */
    private function updateSemanticData()
    {
        if (isset($_REQUEST['_semanticData']))
        {
            if (!$this->context->semanticData)
            {
                $this->context->semanticData = new wcmSemanticData();
            }
            $this->context->semanticData->updateFromAssocArray($_REQUEST['_semanticData']);
        }
    }

    /**
     * Save last edited design zones
     */
    private function saveZones()
    {
        // Retrieve new design zones?
        $zones = getArrayParameter($_REQUEST, '_zones', null);
        $this->context->updateZones($zones);

        //get the widget settings
        $widgetsettings = array();
        foreach ($_REQUEST as $key => $value)
        {
            if (ereg('-settings', $key))
            {
                $widgetsettings[$key] = $value;
            }
        }
        $this->context->updateSerialStorage('_widgetsettings', $widgetsettings);

    }

}
