<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.MVC_SysAction.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */


/**
 * This class implements the default system action for the
 * MVC controller
 *
 * Basically it instanciate the context matching $_REQUEST['_wcmClass']
 * and execute the specific action according to $_REQUEST['_wcmTodo']
 */
class wcmMVC_SysAction extends wcmMVC_Action
{
    /**
     * Instanciate context (usually a wcmObject)
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function setContext($session, $project)
    {
        // Build tree automatically and recover selected object
        $tree = new wcmTree($this->treeId);
        $tree->initFromSession();

        // If no id is defined, retrieve selected node (if exists)
        if ($this->id === null && $tree->selectedNode && $this->class == $tree->selectedNode->class && is_int($tree->selectedNode->id))
        {
            $this->id = $tree->selectedNode->id;
        }
        else
        {
            // If current context does not match selected node
            // clear selection from the tree
            if ($tree->selectedNode)
            {
                if ($this->class != $tree->selectedNode->class || $this->id != $tree->selectedNode->id)
                {
                    $tree->selectedNode = null;
                    $tree->saveIntoSession();
                }
            }
        }

        // Instanciate classname
        $this->context = wcmObjectFactory::create($this->class, $this->id);

        if (!$session->isAllowed($this->context, wcmPermission::P_READ))
        {
            $this->redirect('?_wcmAction=403', true);
        }
    }

    /**
     * Default action (View)
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function on($session, $project)
    {
        // Add statistics
        if ($this->context && $this->context->id)
        {
            $session->addStat(wcmSession::STAT_VIEW_OBJECT, $this->context, getObjectLabel($this->context));
        }
    }

    /**
     * onLock
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onLock($session, $project)
    {
        if (!$this->context->lock($session->userId))
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }
    }

    /**
     * onUnlock
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onUnlock($session, $project)
    {
        if (!$this->context->unlock($session->userId))
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }
    }

    /**
     * onCheckout
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onCheckout($session, $project)
    {
        if (!$this->context->checkout())
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }
    }

    /**
     * beforeSave
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function beforeSave($session, $project)
    {
    }

    /**
     * beforeCheckin
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function beforeCheckin($session, $project)
    {
        $this->beforeSave($session, $project);
    }

    /**
     * After Save
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function afterSave($session, $project)
    {
    }

    /**
     * After Checkin
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function afterCheckin($session, $project)
    {
        $this->afterSave($session, $project);
    }

    /**
     * onCheckin
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onCheckin($session, $project)
    {
        if (!$this->context->checkin($_REQUEST))
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }

        // Add statistics
        $session->addStat(wcmSession::STAT_SAVE_OBJECT, $this->context);

        // Create a new version?
        if (isset($_REQUEST['_comment']))
        {
            $this->executeAction('createVersion');
        }

        // Redirect to 'view' URL
        $this->redirect(self::computeObjectURL($this->context->getClass(), $this->context->id));
    }

    /**
     * onUndocheckout
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onUndocheckout($session, $project)
    {
        if (!$this->context->undoCheckout())
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }
    }

    /**
     * onSave
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onSave($session, $project)
    {
        if (!$this->context->save($_REQUEST))
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }

        // Update history
        $session->updateHistory($this->context, getObjectLabel($this->context));

        // Add statistics
        $session->addStat(wcmSession::STAT_SAVE_OBJECT, $this->context);

        // Create a new version?
        if (isset($_REQUEST['_comment']))
        {
            $this->executeAction('createVersion');
        }
        // Redirect to 'view' URL
        $this->redirect(self::computeObjectURL($this->context->getClass(), $this->context->id));
    }

    /**
     * onDelete
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onDelete($session, $project)
    {
        $context = $this->context;
        $contextId = $this->context->id;

        if (!$context->delete())
        {
            throw new wcmAbortException(_BIZ_ERROR . $this->context->getErrorMsg());
        }

        // Add statistics
        $session->addStat(wcmSession::STAT_DELETE_OBJECT, $this->context);

        // Make sure item isn't linkable in statistics
        $session->disableStatItem(get_class($context), $contextId);

        // Redirect to default action page
        $this->redirect($this->computeURL());
    }

    /**
     * onTransition
     * Launched when a transition is executed from any sysobject
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     * @throws wcmAbortException
     */
    protected function onTransition($session, $project)
    {
        $transition = $project->workflowManager->getWorkflowTransitionById(getArrayParameter($_REQUEST, '_wcmTransitionId', null));
        if (!$this->context->executeTransition($transition))
        {
            $message = $this->context->getErrorMsg();
            throw new wcmAbortException(_TRANSITION_FAILED . ': ' . $message);
        }
    }

    /**
     * onCreateVersion
     * Launched when a version must be created
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onCreateVersion($session, $project)
    {
        // retrieve version comment
        $comment = strip_tags(getArrayParameter($_REQUEST, '_comment', null));
        
        $versionManager = new wcmVersionManager();
        $versions = $versionManager->getObjectHistory($this->context);
        $versionsCount = count($versions);
        $config = wcmConfig::getInstance();
        if (!empty($config['wcm.versioning.maxVersions']) && ($versionsCount < $config['wcm.versioning.maxVersions']))
        {
            // create version
            if ($this->context->archive($comment))
            {
                wcmMVC_Action::setMessage(_BIZ_VERSION_ADDED);
            }
            else
            {
                // display error message
                wcmMVC_Action::setError(_BIZ_VERSION_ADDED_FAILED);
            }
        }
        else
        { 
            if ($versionsCount)
            {    
                // delete the oldest version (first in the versions array), and add the current.
                $version = $versions[0];
                if ($version->delete())
                {
                    unset($versions[0]);
                }
                else
                {
                    wcmMVC_Action::setError(_BIZ_VERSION_DELETED_FAILED);
                }    
            }
           
            if (!empty($config['wcm.versioning.maxVersions']))
            {
                // create version
                if ($this->context->archive($comment))
                {
                    wcmMVC_Action::setMessage(_BIZ_VERSION_ADDED);
                }
                else
                {
                    // display error message
                    wcmMVC_Action::setError(_BIZ_VERSION_ADDED_FAILED);
                }
            } 
            else
            {
                wcmMVC_Action::setError(_BIZ_VERSION_MAX_REACHED);
            } 
  
        }
    }

    /**
     * onRestoreVersion
     * Launched when a version must be restored
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onRestoreVersion($session, $project)
    {
        // retrieve version id to restore
        $versionId = getArrayParameter($_REQUEST, '_versionId', 0);
        if ($versionId)
        {
            // restore version
            if ($this->context->restore($versionId))
            {
                // redirect to default action page
                wcmMVC_Action::setMessage(_BIZ_VERSION_RESTORED);
                $this->redirect(self::computeObjectURL($this->context->getClass(), $this->context->id));
            }
            else
            {
                // display error message
                wcmMVC_Action::setError(_BIZ_VERSION_RESTORE_FAILED . ': ' . $this->context->getErrorMsg());
            }
        }
    }

    /**
     * onRollackVersion
     * Launched when a version must be rolled-back
     *
     * @param wcmSession $session Current session
     * @param wcmProject $project Current project
     */
    protected function onRollbackVersion($session, $project)
    {
        // retrieve version id to rollback
        $versionId = getArrayParameter($_REQUEST, '_versionId', 0);
        if ($versionId)
        {
            // restore version
            if ($this->context->rollback($versionId))
            {
                // redirect to default action page
                wcmMVC_Action::setMessage(_BIZ_VERSION_ROLLEDBACK);
                $this->redirect(self::computeObjectURL($this->context->getClass(), $this->context->id));
            }
            else
            {
                // display error message
                wcmMVC_Action::setError(_BIZ_VERSION_ROLLBACK_FAILED . ': ' . $this->context->getErrorMsg());
            }
        }
    }
}
