<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.importXML.php 9387 2010-04-06 19:48:13Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class is used to import generic bizobjects in XML format.
 * A folder can contains as many XML files (format "{className}_{sourceId}.xml")
 *
 * Each XML can be transformed using an XSL-T and the result is given to initFromXML() methods
 * to build a new bizobject.
 */
class wcmImportXML extends wcmGenericFileImport
{
    /**
     * (string) default root folder used for BizObject
     * Default value is "/data/wcm/WCM/3.1/business/import/in/BizObject/"
     *
     * Important: if the value does NOT start with a dot '.' the path will be
     *             considered as relative to the WCM installation folder
     */
    const DEFAULT_ROOTFOLDER = 'business/import/in/BizObject/';

    /**
     * (string) default relative physical path where XSL are stored (relative to WCM installation folder)
     * Default value is "business/import/xsl/BizObject/"
     *
     * Important: if the value does NOT start with a dot '.' the path will be
     *             considered as relative to the WCM installation folder
     */
    const DEFAULT_XSLFOLDER = 'business/import/xsl/BizObject/';

    /**
     * Default site id used for import (default value is zero)
     * @var int
     */
    public $siteId = 0;

    /**
     * Default channel id used for import (default value is zero)
     * @var int
     */
    public $channelId  = 0;

    /**
     * Path of folder where XSL can be found (the folder should contains 'article.xsl' and 'photo.xsl' files)
     *
     * Important: if the value does NOT start with a dot '.' the path will be
     *             considered as relative to the WCM installation folder
     * @var string
     */
    public $xslFolder = null;

    /**
     * Constructor
     *
     * @param array      $parameters  An associative array used to assign properties (by their name) and extra parameters
     */
    public function __construct(array $parameters)
    {
        parent::__construct($parameters);

        // Setup default values
        $this->fileRegExp = '/([A-Za-z]*)_([A-Za-z0-9]*)\.xml/';
        $this->rootFolder = $parameters['wcmImportXML_sourceFolder'];
        if (!$this->source)
        {
            $this->source = 'WCM';
        }
        if (!$this->rootFolder)
        {
            $this->rootFolder = self::DEFAULT_ROOTFOLDER;
        }
        if (!$this->xslFolder)
        {
            $this->xslFolder = self::DEFAULT_XSLFOLDER;
        }
        $this->getTotal();
    }

    /**
     * Get the total number of news component
     * @return int Total number of news component
     */
    public function getTotal()
    {
        $total = 0;
        if (is_dir($this->rootFolder))
        {
            $dir = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator($this->rootFolder),
                RecursiveIteratorIterator::SELF_FIRST
            );

            foreach ($dir as $d)
            {
                if (preg_match($this->fileRegExp, $d->getFilename()))
                {
                    $total++;

                    // This code will base the total on the number news components in each index.xml file
                    // $xml = simplexml_load_file($d->getPathname());
                    // $newsItems = $xml->xpath('//NewsItem/NewsComponent/NewsComponent');
                    // $total = $total + count($newsItems);
                }
            }
        }
        $this->total = $total;
        return $total;
    }

    /**
     * Launch import process
     */
    public function process()
    {
        // Adjust relative folders if needed
        $path = WCM_DIR . '/';
        if (substr($this->rootFolder, 0, 1) == '.')
        {
            $this->rootFolder = $path . $this->rootFolder;
        }
        if (!$this->siteId)
        {
            $this->siteId = $_SESSION['siteId'];
        }

        // Start process (and process files)
        parent::process(true);
    }

    /**
     * This function is invoked each time a folder has to be processed
     * (the currentFolder property is updated before this function is invoked)
     *
     * @return bool Return FALSE to skip processing of inner files and sub-folders, true otherwise
     */
    public function processFolder()
    {
        $this->logger->logMessage('processFolder : ' . $this->currentFolder);
        return true;
    }

    /**
     * Process one item
     * @param mixed $argItem Item to paterm
     * @return bool TRUE on success
     */
    public function processOne($argItem)
    {
        return false;
    }

    /**
     * This function is invoked each time a file has to be processed
     * (the currentFile and currentFolder properties are updated before this function is invoked)
     * @return bool TRUE on success
     */
    public function processFile()
    {
        $this->logger->logMessage('processFile : ' . $this->currentFile);

        // Deduce classname and id from file name ({className}_{objectSourceId}.xml)

        $fileName = basename($this->currentFile);

        $fileNameParts = explode('_', $fileName);

        $className = $fileNameParts[0];
        $uniqueCode = $fileNameParts[1];

        // Check if class is valid
        $classList = getClassList();
        if (!isset($classList[$className]))
        {
            $this->totalProcessed++;
            $this->writePollData(floor(($this->totalProcessed / $this->total) * 100));
            $this->logger->logError(sprintf(_BIZ_INVALID_CLASSNAME, $className));
            return;
        }

        $bo = wcmObject::create($className);

        $db = wcmMasterClassRegistry::getDatabase($bo);

        $boXML = new DOMDocument();

        if (!$boXML->load($this->currentFile))
        {
            $this->totalProcessed++;
            $this->writePollData(floor(($this->totalProcessed / $this->total) * 100));
            $this->logger->logError('INVALID XML');
            return false;
        }

        $xPath = new DOMXPath($boXML);

        $originalId = wcmXML::getXPathNodeValue($xPath, null, '//id', 0);
        $source = wcmXML::getXPathNodeValue($xPath, null, '//source', 'BizObject');
        $sourceCode = wcmXML::getXPathNodeValue($xPath, null, '//sourceId', $uniqueCode);

        $this->logger->logMessage('Making new class: ' . $className);

        $integrity = isset($this->parameters['wcmImportXML_integrity']);
        $idOverSource = isset($this->parameters['wcmImportXML_idOverSource']);

        $bo->refreshFromSource($source, $sourceCode);


        $boId = $bo->id;
        $bo->initFromXML(file_get_contents($this->currentFile));

        // Was there an ID from the XML and not from the source refresh?
        if (!$boId && $originaId)
        {
            $bo->id = $originalId;
        }
        if ($boId != $originalId)
        {
            if ($integrity)
            {
                // BO ID takes priority
                $bo->id = $originalId;
            }
            else
            {
                $bo->id = $boId;
            }
        }

        if (!$bo->versionNumber)
        {
            $bo->versionNumber = 1;
        }

        if (empty($bo->siteId))
        {
            $bo->siteId = wcmSession::getInstance()->getSiteId();
        }

        $this->logger->logMessage('Saving object. Class: ' . get_class($bo));

        if (!$boId && $bo->id && $integrity)
        {
            // An ID came from XML and we are forced to insert it
            // Gotta check if we need to force an insert or not:
            if (!$db->executeScalar('SELECT id FROM ' . $bo->getTableName . ' WHERE id=?', array($bo->id)))
            {
                if (!$db->insertObject($bo->getTableName(), $bo))
                {
                    $this->logger->logError('Could not insert bizobject: ' . $db->getErrorMsg());
                }
            }
        }

        // Save bizobject
        if ($bo->save())
        {
            $this->logger->logMessage('Saved!');
        }
        else
        {
            $this->logger->logError('Bizobject could not be saved: ' . $bo->getErrorMsg());
        }
        $this->totalProcessed++;
        $this->writePollData(floor(($this->totalProcessed / $this->total) * 100));
        return true;
    }
}