<?php
/**
 * Project:     WCM
 * File:        restForPlugins.php
 *
 * @copyright   (c)2008 Nstein Technologies
 * @package     WCM
 * @version     4.x
 *
 */

/**
 * Class to allow for a dependency-free operation of WCM rest services
 *
 * The idea behind this class to allow for developers of third party applications to easily
 * and quickly access WCM's restful services without relying on PEAR, CURL, or any other
 * dependency. It accesses the server through regular sockets.
 *
 * Callbacks can be given but are triggered only when an error occurs.
 *
 */
class wcmRestForPlugins
{
    /**
     * Current session ID
     *
     * @var string
     */
    private $sessionId;

    /**
     * Connected socket
     *
     * @var resource
     */
    private $fp;

    /**
     * Connection host
     * @var string
     */
    private $host;

    /**
     * Connection port
     * @var int
     */
    private $port;

    /**
     * Connection IP
     * @var string
     */
    private $ip;

    /**
     * Connection user
     * @var string
     */
    private $user;

    /**
     * Connection password
     * @var string
     */
    private $pass;

    /**
     * Connection URL
     * @var string
     */
    private $url;

    /**
     * Connection state, TRUE if connected
     * @var bool
     */
    private $connected = false;

    /**
     * Constructor
     *
     * @param string $argWcmURL URL of WCM
     * @param string $argUser Username
     * @param string $argPass Password
     */
    public function __construct($argWcmURL, $argUser, $argPass)
    {
        $this->setup($argWcmURL);
        $this->user = $argUser;
        $this->pass = $argPass;
    }

    /**
     * Setup connection
     * @param string $argUrl Connection URL (DSN style)
     */
    private function setup($argUrl)
    {
        $url = parse_url($argUrl);
        $this->host = $url['host'];
        $this->ip = gethostbyname($url['host']);
        $this->port = $url['port'];
        $this->path = $url['path'];
        $this->url = $argUrl;

        if (!$this->port)
        {
            $this->port = 80;
        }
    }

    /**
     * Attempt to make a connection
     *
     * @param string $argCallBack Function that gets executed on failure.
     * @return bool TRUE on success
     */
    public function connect($argCallBack = null)
    {
        $fp = fsockopen($this->ip, $this->port, $errno, $errstr, 10);
        if (!$fp)
        {
            if (is_callable($argCallBack))
            {
                call_user_func($argCallBack, 'Could not connect: ' . $errstr);
            }
            return false;
        }
        else
        {
            $this->fp = $fp;
            $this->connected = true;
            return true;
        }
    }

    /**
     * Get a connection, making one if none exists
     *
     * @param string $argCallBack Function that gets executed on failure.
     * @return bool TRUE on success
     */
    public function getConnection($argCallBack = null)
    {
        if ($this->connect($argCallBack))
        {
            return $this->fp;
        }
    }

    /**
     * Closes the socket connection
     *
     */
    public function disconnect()
    {
        fclose($this->fp);
        unset($this->fp);
        $this->connected = false;
    }

    /**
     * Logs out the user from WCM
     *
     */
    public function logout()
    {
        unset($this->sessionId);
    }

    /**
     * Send a message to WCM
     *
     * Take special note of the $argReturn parameter. If enabled,
     * the socket will be read for data untill an EOF is received.
     *
     * Unless you need data back from WCM, $argReturn should always be
     * false. If you request data to be returned, it'll come as an array.
     *
     * The first key in the array will be raw headers, the second key will
     * be the data.
     *
     * @param string $argUrl Location of resource
     * @param string $argCommand HTTP Command (get/post/put/delete)
     * @param array $argData An array of key/value pairs
     * @param string $argReturn Whether or not to return data back
     * @param string $argCallBack Callback function incase of an error
     * @return array If $argReturn is true, the return will be whatever the user spits out.
     * @todo Turn headers into a proper array.
     */
    public function sendMessage($argUrl, $argCommand, array $argData = array(), $argReturn = false, $argCallBack = null)
    {

        if (('http://' == substr($argUrl, 0, 7)) || ('https://' == substr($argUrl, 0, 8)))
        {
            $oldConfig = $this->url;
            $this->setup($argUrl);
            $target = $argUrl;
        }
        else
        {
            $target = $this->path . $argUrl;
        }

        $sessId = $this->authenticate($argCallBack);
        $fp = $this->getConnection($argCallBack);

        if (!$fp)
        {
            if (isset($oldConfig))
            {
                $this->setup($oldConfig);
            }
            return false;
        }

        $data = (isset($argData)) ? http_build_query($argData) : '';

        $cmd = strtoupper($argCommand);
        if ('POST' == $cmd || 'PUT' == $cmd)
        {
            fwrite($fp, $cmd . ' ' . $target . ' HTTP/1.1' . PHP_EOL);
            fwrite($fp, 'x-nstein-session-id: ' . $sessId . PHP_EOL);
            fwrite($fp, 'Host: ' . $this->host . PHP_EOL);
            fwrite($fp, 'Content-Type: application/x-www-form-urlencoded; charset=utf-8' . PHP_EOL);
            fwrite($fp, 'Content-Length: ' . strlen($data) . PHP_EOL . PHP_EOL);
            fwrite($fp, $data);
        }
        else
        {
            if (isset($data))
            {
                $target = $target . '?' . $data;
            }
            fwrite($fp, $cmd . ' ' . $target . ' HTTP/1.1' . PHP_EOL);
            fwrite($fp, 'x-nstein-session-id: ' . $sessId . PHP_EOL);
            fwrite($fp, 'Host: ' . $this->host . PHP_EOL . PHP_EOL);
        }

        if ($argReturn)
        {
            $content = '';
            while ('' != ($buf = fread($fp, 1024)))
            {
                $content .= $buf;
            }
            $this->disconnect();
            $parts = explode("\r\n\r\n", $content);//explode(PHP_EOL . PHP_EOL, $content);
            if (isset($oldConfig))
            {
                $this->setup($oldConfig);
            }
            return $parts;
        }
        $this->disconnect();
        if (isset($oldConfig))
        {
            $this->setup($oldConfig);
        }
        return true;

    }

    /**
     * Attempt to authenticate the user
     *
     * This method is called automatically by sendMessage() and so you don't have
     * to trigger it manually yourself. However it is useful to test the username/password
     * to make sure they are correct.
     *
     * If a callback is supplied, it is called if there is a connection problem or if
     * the user could not authenticate.
     *
     * Returns true on success, false if the user could not authenticate, -1 if there
     * was a connection problem.
     *
     * @param string $argCallBack Callback function
     * @return bool TRUE on success
     */
    public function authenticate($argCallBack = null)
    {
        if (!$this->sessionId)
        {
            if ($fp = $this->getConnection($argCallBack))
            {
                $data = http_build_query(array('username' => $this->user, 'password' => $this->pass));
                fwrite($fp, 'POST ' . $this->path . '/session HTTP/1.1' . PHP_EOL);
                fwrite($fp, 'Host:' . $this->host . PHP_EOL);
                fwrite($fp, 'Content-Type: application/x-www-form-urlencoded' . PHP_EOL);
                fwrite($fp, 'Content-Length: ' . strlen($data) . PHP_EOL . PHP_EOL);

                fwrite($fp, $data);

                $content = '';
                while ('' != ($buf = fread($fp, 1042)))
                {
                    $content .= $buf;
                }

                $header = 'x-nstein-session-id';

                $pattern = '/' . $header  . ': (.*?)\n/';
                $match = preg_match($pattern, $content, $result);
                $sessionId = $result[1];

                if (!$sessionId)
                {
                    if (is_callable($argCallBack))
                    {
                        call_user_func($argCallBack, 'Could not authenticate');
                    }
                    return false;
                }
                $this->sessionId = $sessionId;
                $this->disconnect();
            }
            else
            {
                if (is_callable($argCallBack))
                {
                    call_user_func($argCallBack, 'Not connected');
                }
                $this->disconnect();
                return -1;
            }
        }
        return $this->sessionId;
    }
}
