<?php
/**
 * The base controller for search pages
 * It uses the solr Search engine
 */
class tmeSearchController
{
	/**
	 * The search engine.
	 * @var cmfSearchEngine
	 */
	public $searchEngine;

	/**
	 * The current search results.
	 * @var cmfSearchResult
	 */
	public $results;

	/**
	 * The current page number.
	 * @var int
	 */
	public $page;

	/**
	 * The current search string
	 * @var string
	 */
	public $searchString;

	/**
 	 * The current search result count
	 * @var int
	 */
	public $resultCount = 0;

	/**
	 * The current searchIndex to query
	 * @var string
	 */
	public $searchIndex;

	/**
	 * The number of records to display per page
	 * @var int
	 */
	public $rowPerPage = 10;

	/**
	 * The sort order "readable string"
	 * @var string
	 */
	public $sort;

	/**
	 * Keep query parameters to be able to rebuild the queryString
	 * @var array
	 */
	public $queryParams = array();

    public $hostname;

    public $request;

    public function __construct($hostname, $request)
    {
        $this->hostname = $hostname;
        $this->request = $request;
    }

	/**
	 * (non-PHPdoc)
	 * @see cmf/business/controllers/cmfMainController#always()
	 */
	public function always()
	{
		// Get the type of search
		$this->searchIndex = getArrayParameter($this->request, 'index', 'fulltext');

		// Get the queryString typed by the webuser
		$this->userSearchString = getArrayParameter($this->request, 'q', null);

		// The solrSearchString is the one that will be used for Solr (Removed some unwanted chars)
		// Feel free to change the implementation of parseQueryString, since you may want to allow
		// a user to use some of Solr Operators
		$this->solrSearchString = $this->parseQueryString($this->userSearchString);

		// Get the current page (or 1 if no page number is provided).
		$this->page = getArrayParameter($this->request, 'page', 1);

		// Defines the offset used for pagination (e.g. offset of 2 means : 1 | 2 | 3 | 4 | 5 - when current page is at 3)
		$this->paginationOffset = getArrayParameter($this->request, 'paginationOffset', 2);

		// Get the sorting 
		$this->sort = getArrayParameter($this->request, 'sort', null);

		// Get number of rows to be displayed in a page, default is set to 10
		$this->rows = getArrayParameter($this->request, 'rows', 10);

		$this->filtersFacetQuery = '';

		// Add new facet filters if new one were passed (e.g. title:foo)
		$this->filtersFacet = getArrayParameter($this->request, 'filtersFacet', array());
		if(null !== $this->filtersFacet)
		{
			foreach($this->filtersFacet as $filterFacet)
			{
				$tmp = explode(':', $filterFacet);
				switch($tmp[0])
				{
					case "on":
						$filter = 'list_on:"' . $tmp[1] . '"';
					break;

					case "pn":
						$filter = 'list_pn:"' . $tmp[1] . '"';
						break;

					case "gl":
						$filter = 'list_gl:"' . $tmp[1] . '"';
						break;

                    default:
                        $filter = $tmp[0] . ':"' . $tmp[1] . '"';
                        break;
				}
				// Update filters Facet query	
				$this->filtersFacetQuery .= '&filtersFacet[]=' . $filterFacet;
				
				// Update solr query string
				$this->solrSearchString .= ' AND ' . $filter;
			}
		}

		// Enable highlighting on the searchtext field (take the 200 characters)
		$this->hl = array('searchtext' => 200);

		// Get the starting row (by default we start at 0)
		$this->start = ($this->page - 1) * $this->rows;

		$facetLimit = getArrayParameter($_REQUEST, 'facetLimit', 100);

		// Query for the on, pn, gl and channel facets
		$this->facets['params']['limit'] = $facetLimit;
		$this->facets['fields'][] = array('field' => 'list_on');
		$this->facets['fields'][] = array('field' => 'list_pn');
		$this->facets['fields'][] = array('field' => 'list_gl');
	    $this->facets['fields'][] = array('field' => 'concept');
        $this->facets['fields'][] = array('field' => 'category');

		// Update queries (used for link purposes)
		$this->keywordQuery = 'q=' . $this->userSearchString;
		$this->pageQuery =	'page=' . $this->page;

		// Now that we have all the parameters, lets do the search
		$this->doSearch();
	}

	public function parseQueryString($string)
	{
		// Temporary, keyword tout will display all results (can be used to see nfinder results / facets)
		if($string == 'tout')
			return '*:*';

		$searches = array('.', '*', ':', '?', '~', '^', '&', '|');
		$replaces = array('.', '*', ':', '?', '~', '^', '&', '|');
		$string = str_replace($searches, $replaces, $string);

		return $string;
	}

	/**
	 * Performs a search on the solr search engine
	 */
	public function doSearch()
	{
		// Only search if a query was passed
		if('' != $this->solrSearchString)
		{
			// Instantiage our Solr Helper class, we will use that in order to query solr
			$this->searchEngine = new simpleSolr($this->hostname);

			// Perform the search
			$results = $this->searchEngine->search($this->solrSearchString, $this->start, $this->rows, 
															$this->sort, 'on', $this->hl, $this->facets);

			// Total of result
			$this->resultsCount = $results['response']['numFound'];

			// The document (whole document in a XML format)
			$this->results = $results['response']['docs'];

			// The hightlighted result (using searchtext field)
			$this->resultsHL = $results['highlighting'];

			$this->facetsON = $results['facet_counts']['facet_fields']['list_on'];
			$this->facetsPN = $results['facet_counts']['facet_fields']['list_pn'];
			$this->facetsGL = $results['facet_counts']['facet_fields']['list_gl'];
		    $this->facetsConcepts = $results['facet_counts']['facet_fields']['concept'];
		    $this->facetsCategories = $results['facet_counts']['facet_fields']['category'];
        }		
		else
		{
			$this->resultsCount = 0;
			$this->results = array();
			$this->resultsHL = array();
			$this->facetsON = array();
			$this->facetsPN = array();
			$this->facetsGL = array();
            $this->facetsConcepts = array();
            $this->facetsCategories = array();
		}
	}

	public function getPagination()
	{
		if(0 == $this->rows)
		{
			throw new exception('Rows must be different from zero');
			return array();
		}

		if($this->resultsCount <= $this->rows)
		{
			return array();
		}

		$nbPage = $this->resultsCount / $this->rows;
		if($nbPage > (int)$nbPage)
		{
			$nbPage = (int)$nbPage + 1; 
		}

		// If current page is lower than the pagination offset
		if($this->page <= $this->paginationOffset)
		{
			$pageMin = 1;
		}
		else
		{
			$pageMin = $this->page - $this->paginationOffset;
		}

		if($this->page + $this->paginationOffset >= $nbPage)
		{
			$pageMax = $nbPage;
		}
		else
		{
			$pageMax = $this->page + $this->paginationOffset;
		}

		$result = array();
		$cpt = $pageMin;
		while($cpt <= $pageMax)
		{
			$result[$cpt] = $cpt * $this->rows;
			++$cpt;
		}

		return $result;
	}
}
