<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restAuthorizationController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmRestAuthorizationController class provides a RESTful interface to manage user
 * authorization for resources.
 */
class wcmRestAuthorizationController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('authorization');
    }

    /**
     * HTTP "GET" method.
     *
     * @see getAuthorization
     * @see getUser
     * @see getGroup
     * @see getRole
     * @see getPrivilege
     *
     * @return wcmRestResponse => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid resource
     *                         => responses from get{Authorization|User|Group|Role|Privilege}
     */
    protected function get()
    {
        try
        {
            // Initialize / validate session
            $response = $this->initSession();
            if ($response !== null)
            {
                return $response;
            }

            // Determine method to call based on resource
            $resource = getArrayParameter($_GET, 'resource', 'authorization');
            $method = 'get' . ucfirst($resource);
            if (!method_exists($this, $method))
            {
                return wcmRestResponse::invalidParameter('resource', $resource);
            }

            // Only the root user can query specific authorization info
            if ($resource != 'authorization'
                && $this->session->userId != wcmMembership::ROOT_USER_ID)
            {
                return new wcmRestResponse(wcmRestResponse::S_FORBIDDEN);
            }

            // Get resource
            return $this->$method();
        }
        catch (wcmRestException $error)
        {
            return $error->getResponse();
        }
        catch (exception $error)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, $error->getMessage());
        }
    }

    /**
     * Get authorization
     * @return wcmRestResponse REST Response
     */
    private function getAuthorization()
    {
        // Get current user
        $user = wcmMembership::getUserById($this->session->userId);
        if (!$user)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR);
        }

        $usergroups = $user->getGroups();
        
        // Get user's groups, roles, and privileges - in WCM, a group (class "wcmGroup") actually
        // represents a role, ie. there are no groups per se
        $usergroups = $user->getGroups();
        $privilegesRep = $this->getUserPrivileges($user, $_REQUEST);

        // Response as JSON or XML
        $body = null;
        $headers = array();
        if (isset($_REQUEST['json']))
        {
            $body  = '{';
            $body .= '"groups":' . json_encode($usergroups) . ',';
            $body .= '"privileges":' . $privilegesRep . ',';
            $body  = rtrim($body, ',');
            $body .= '}';
            $headers[wcmRestResponse::H_CONTENT_LENGTH] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
        }
        else
        {
            $body  = wcmRestResponse::XML_DECL . PHP_EOL;
            $body .= '<authorization>' . PHP_EOL;
            $body .= wcmXML::xmlEncodeArray(array('groups' => $usergroups)) . PHP_EOL;
            $body .= $privilegesRep . PHP_EOL;
            $body .= '</authorization>' . PHP_EOL;
            $headers[wcmRestResponse::H_CONTENT_LENGTH] = wcmRestResponse::H_CTCS_TEXT_XML;
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * Get user
     * @return wcmRestResponse REST Response
     * @todo implement
     */
    private function getUser()
    {
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * Get group
     * @param string User
     * @return wcmRestResponse REST Response
     * @todo implement
     */
    private function getGroup($user = null)
    {
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * Get role
     * @return wcmRestResponse REST Response
     * @todo implement
     */
    private function getRole()
    {
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * HTTP "GET" method for "privilege" resource.
     *
     * Gets the privileges of a given user with respect to a given object or a set of objects.
     *
     * QUERY PARAMETERS
     *
     * userId    => [required] Target user ID
     * className => [required] Object class name
     * objectId  => [optional] Object ID (default is null)
     * where     => [optional] SQL "WHERE" clause to select objects (default is null)
     * sort      => [optional] SQL "ORDER BY" clause to sort list of objects (default is null)
     * offset    => [optional] Zero-based offset of first object (default is 0)
     * limit     => [optional] Maximum number of objects (default is 128)
     *
     * @return object Rest content
     */
    private function getPrivilege()
    {
        // Validate user ID
        $userId = getArrayParameter($_REQUEST, 'userId', null);
        if (!$userId)
        {
            return wcmRestResponse::missingParameter('userId');
        }

        // Refresh user object
        $user = wcmMembership::getUserById($userId);
        if (!$user)
        {
            return wcmRestResponse::invalidParameter('userId', $userId);
        }

        // Add internal parameters to request parameters
        $params = $_REQUEST;
        $params['response'] = 1; // we want a wcmRestResponse

        // Return user privileges response
        return $this->getUserPrivileges($user, $params);
    }

    /**
     * Get a user's privilege
     * @param wcmUser $user   [required] Target user
     * @param array   $params [optional] Request parameters
     * @return object Rest content
     * @throws wcmRestException
     */
    private function getUserPrivileges($user, array $params = array())
    {
        // Callback to represent user privileges for a given object
        $callback = array('wcmRestAuthorizationController', 'userPrivileges');

        // Validate object class name
        $className = getArrayParameter($_REQUEST, 'className', null);
        if (!$className)
        {
            throw new wcmRestException(wcmRestResponse::missingParameter('className'));
        }
        if (!autoloadManager::classExists($className))
        {
            throw new wcmRestException(wcmRestResponse::invalidParameter('className', $className));
        }

        // Add internal parameters to request parameters
        $params['baseURL']    = $this->baseURL . 'privilege/';
        $params['controller'] = $this->mode;
        $params['resource']   = 'privilege';
        $params['userId']     = $user->id;

        // Return authorization for a specific object or a list of objects?
        $objectId = getArrayParameter($_REQUEST, 'objectId', null);
        if ($objectId)
        {
            // Refresh object
            $object = new $className;
            $object->refresh($objectId);
            if (!$object->id)
            {
                return wcmRestResponse::invalidParameter('objectId', $objectId);
            }

            // Return authorization information for object
            return wcmRestContent::getObjectWithCallback($callback, $className, $objectId, $params);
        }
        else
        {
            // Return authorization information for a list of objects
            return wcmRestContent::getObjectsWithCallback($callback, $className, $params);
        }
    }

    /**
     * Get a user's privilege
     * @param wcmObject $object User object
     * @param array $fields Useless parameter
     * @param array $params Extra parameters
     * @throws wcmRestException
     */
    public static function userPrivileges(wcmObject $object, $fields, array $params)
    {
        // Validate user ID
        $userId = getArrayParameter($params, 'userId', null);
        if ($userId === null)
        {
            throw new wcmRestException(wcmRestResponse::missingParameter('userId'));
        }

        // Refresh user objects
        $user = new wcmUser;
        $user->refresh($userId);
        if (!$user->id)
        {
            throw new wcmRestException(wcmRestResponse::invalidParameter('userId', $userId));
        }

        // Get mask of user privileges (permissions) for given object
        $mask = $object->getUserPermissions($params['userId']);

        // Ok, P_ALL == P_READ|P_WRITE|P_EXECUTE|P_DELETE|P_NONE
        // Yep, P_ALL includes P_NONE! :-(
        // So unless $mask is P_NONE, we remove it
        $p_all = wcmPermission::P_ALL & ~wcmPermission::P_NONE;
        if ($mask != wcmPermission::P_NONE)
        {
            $mask &= ~wcmPermission::P_NONE;
        }

        // Representation data
        $repData = array();
        $repData['objectClass'] = get_class($object);
        $repData['objectId']    = $object->id;
        $repData['mask']        = $mask;

        // Representation as JSON or XML
        if (isset($_REQUEST['json']))
        {
            $rep = json_encode($repData);
        }
        else
        {
            $rep = wcmXML::arrayToXml($repData, 'privilege', true);
        }

        return $rep;
    }
}