<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restPermissionsController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmRestPermissionsController class provides a RESTful interface
 * to manage object permissions.
 */
class wcmRestPermissionsController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('permissions');
    }

    /**
     * HTTP "GET" method.
     *
     * GET .../permissions/{CLASSNAME}?where={WHERE}&sort={SORT}&offset={OFFSET}&limit={LIMIT}
     * GET .../permissions/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (optional) object ID
     * {WHERE}     => (optional) SQL "WHERE" clause (default is null)
     * {SORT}      => (optional) SQL "ORDER BY" clause (default is "modifiedAt DESC")
     * {OFFSET}    => (optional) zero-based index of first object (defaults to 0)
     * {LIMIT}     => (optional) zero-based index of last object (defaults to 9)
     *
     * Gets the permissions for a given object, or for all matching
     * objects if {ID} is not given.
     *
     * @return wcmRestResponse => S_OK          on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on missing object class
     *                         => S_BAD_REQUEST on invalid object selection criteria
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid object class or ID
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Validate object class name
        $className = getArrayParameter($_GET, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Return permissions for one or many objects?
        $wcmObject = wcmObjectFactory::create($className);
        $permissions = array();

        $id = getArrayParameter($_GET, 'id', null);
        if ($id)
        {
            // Refresh object
            $wcmObject->refresh($id);
            if (!$wcmObject->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Get permissions
            $objPerms = $this->session->getPermissions($wcmObject);
            $permissions[$wcmObject->id] = $this->permissionsToString($objPerms);
        }
        else
        {
            // Get object selection criteria
            $where  = getArrayParameter($_GET, 'where', null);
            $sort   = getArrayParameter($_GET, 'sort', null);
            $offset = intval(getArrayParameter($_GET, 'offset', 0));
            $limit  = intval(getArrayParameter($_GET, 'limit', 128));

            if (!$wcmObject->beginEnum($where, $sort, $offset, $limit))
            {
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CRITERIA);
            }

            // Get permissions
            while ($wcmObject->nextEnum())
            {
                $objPerms = $this->session->getPermissions($wcmObject);
                $permissions[$wcmObject->id] = $this->permissionsToString($objPerms);
            }
            $wcmObject->endEnum();
        }

        // Response is set of permissions as JSON or XML
        $headers = array();
        $body = null;

        if (isset($_REQUEST['json']))
        {
            $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
            $body = json_encode($permissions);
        }
        else
        {
            $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;

            $body .= wcmRestResponse::XML_DECL . PHP_EOL;
            $body .= '<permissions>' . PHP_EOL;
            if ($permissions)
            {
                foreach ($permissions as $id => $perms)
                {
                    $body .= '<object>' . PHP_EOL;
                    $body .= '<id>' . $id . '</id>' . PHP_EOL;
                    $body .= '<perms>' . $perms . '</perms>' . PHP_EOL;
                    $body .= '</object>' . PHP_EOL;
                }
            }
            $body .= '</permissions>' . PHP_EOL;
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * Gets the string representation of given permissions.
     *
     * @param int $permissions The permissions
     *
     * @return string The corresponding string representation
     */
    private function permissionsToString($permissions)
    {
        switch ($permissions)
        {
            case wcmPermission::P_READ:
                $permissions = 'READ';
                break;

            case wcmPermission::P_WRITE:
                $permissions = 'WRITE';
                break;

            case wcmPermission::P_EXECUTE:
                $permissions = 'EXECUTE';
                break;

            case wcmPermission::P_DELETE:
                $permissions = 'DELETE';
                break;

            case wcmPermission::P_NONE:
                $permissions = 'NONE';
                break;

            case wcmPermission::P_ALL:
                $permissions = 'ALL';
                break;
        }

        return $permissions;
    }
}