<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restPhotoController.php 9388 2010-04-06 19:50:22Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmRestPhotoController class provides a RESTful interface to
 * get photo images from WCM.
 */
class wcmRestPhotoController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('photo');
    }

    /**
     * HTTP "GET" method.
     *
     * GET .../photo/{ID}
     * GET .../photo/{ID}?ratio={RATIO}
     * GET .../photo/{ID}?width={WIDTH}&height={HEIGHT}
     *
     * {ID}      => (required) ID of photo object
     * {RATIO}   => (optional) photo image ratio code (default is null)
     * {WIDTH}   => (optional) photo image width (default is original image width)
     * {HEIGHT}  => (optional) photo image height (default is original image height)
     *
     * Gets the image associated with a given photo object.
     *
     * If {RATIO} is given, crops/resizes the image according to the
     * ratio code as defined in the configuration file
     * "business/xml/photosratios/default.xml". Note that this
     * parameter takes precedence over the width/height.
     *
     * If {WIDTH} and/or {HEIGHT} is/are given, resizes the image. If
     * only one of {WIDTH} or {HEIGHT} is given, resizes maintaining
     * aspect ratio.
     * 
     *
     * On success, returns the photo image contents.
     *
     * @return wcmRestResponse => S_OK          on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on missing photo object ID
     *                         => S_BAD_REQUEST on invalid photo image ratio code or image type
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid photo object ID
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }
             
        // Get parameters
        $id = getArrayParameter($_GET, 'id', null);
        $ratio = getArrayParameter($_GET, 'ratio', null);
        $width  = getArrayParameter($_GET, 'width', null);
        $height = getArrayParameter($_GET, 'height', null);
        
        // verify id is present
        if (!$id)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_PHOTO_ID);
        }

        // Refresh photo object
        $photo = wcmObjectFactory::create('photo');
        $photo->refresh($id);
        
        // validate id
        if (!$photo->id)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
        }
        
        // get our mediaManager from the factory
        $photoManagerFactory = new photoManagerFactory();
        $photoManager = $photoManagerFactory->createDefaultPhotoManager();

        // Get the original image url and calculate the size/image_type
        $photoUrl = $photoManager->getImageUrl($photo);
        $imageSize = getimagesize($photoUrl);
        
        if ($ratio) // ratio was given, use the croppingManager
        {
            $photoUrl = $photoManager->getCroppedImageUrl($photo, $ratio);
        }
        elseif ($width || $height)  // width or height given but no ratio
        {
            // If a width or a height was given then resize
            if ($width || $height)
            {
                // If no width is specified then use that
                // of the original, same goes for the height
                // aspect ratio will stay the same as getResizedImageUrl
                // maintains the aspect ratio
                if (!$width)
                {
                    $height = $imageSize[1];
                }
                if (!$height)
                {
                    $height = $imageSize[1];
                }

                // get the resized image url from the media manager
                $photoUrl = $photoManager->getResizedImageUrl($photo, $width, $height);

            }
        }

        // Open photo image file
        $imageFile = fopen($photoUrl, 'r');
        
        if (!$imageFile)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR);
        }

        // Set content type header
        $headers = array(wcmRestResponse::H_CONTENT_TYPE => $imageSize['mime']);
        
        return new wcmRestResponse(wcmRestResponse::S_OK, $imageFile, $headers);
    
    }

    /**
     * HTTP "PUT" method.
     *
     * PUT .../photo/{NAME}
     *
     * {NAME} => (required) name of photo image without path or file name extension
     *
     * Creates/updates a photo image with a given name, determining
     * the path of the image file from the WCM configuration and its
     * file name extension from its MIME content type.
     *
     * @returns wcmRestResponse => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => TODO
     * @see wcmRestPhotoController::post()
     */
    protected function put()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get the content of the PUT request
        list($content, $contentType) = $this->getPutContent();
        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, 'TODO');
        }

        // Let the internal method handle the rest
        return $this->putOrPost($content, $contentType);
    }

    /**
     * HTTP "POST" method.
     *
     * => Currently does the same thing as the HTTP "PUT" method.
     */
    protected function post()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get the content of the POST request
        list($content, $contentType) = $this->getPostContent('content');

        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Let the internal method handle the rest
        return $this->putOrPost($content, $contentType);
    }

    /**
     * Internal HTTP "PUT" or "POST" method.
     *
     * This method exists because PHP uses two different means of
     * providing the input data for the PUT and POST methods:
     *
     * PUT  => content comes from input stream "php://input"
     * POST => content comes from variable $_POST
     *
     * @param string $content     The PUT or POST content
     * @param string $contentType The MIME content type
     *
     * @returns wcmRestResponse => the body of the response contains the image width, height and path
     */
    private function putOrPost($content, $contentType)
    {
        // Validate image content type and determine file name extension
        $imageExt = null;
        switch (strtolower($contentType))
        {
            case 'image/bmp':
                $imageExt = 'bmp';
                break;

            case 'image/gif':
                $imageExt = 'gif';
                break;

            case 'image/jpeg':
                $imageExt = 'jpeg';
                break;

            default:
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_PHOTO_TYPE);
        }

        // Validate image name
        $imageName = getArrayParameter($_REQUEST, 'name', null);
        if (!$imageName)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_PHOTO_NAME);
        }

        // Validate the photo id is present
        $id = getArrayParameter($_REQUEST, 'id', null);
        if (!$id)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_PHOTO_ID);
        }
        
        // Validate the id
        $wcmPhoto = wcmObjectFactory::create('photo');
     
        $wcmPhoto->refresh($id);
        if (!$wcmPhoto->id)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
        }
            
        $wcmPhoto->original = $imageName;
        
        // Save content in upload directory to be able to calculate the image size and then delete(unlink) the file
        $config = wcmConfig::getInstance();
        $photosDir = WCM_WWW_BACKOFFICE_DIR . '/system/public/' . $config['wcm.backOffice.uploadDir'];
        $imagePath = $photosDir . '/' . $imageName . '.' . $imageExt;
        if (file_put_contents($imagePath, $content, LOCK_EX) === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, 'Could not save the file to: ' . $imagePath);
        }

        list($wcmPhoto->width, $wcmPhoto->height) = getimagesize($imagePath);
         
        
        // save the photo object
        try
        {
            // Save object
            if (!$wcmPhoto->save(array('sourcePath' => $imagePath)))
            {
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, $wcmPhoto->getErrorMsg());
            }
            
            // delete the file as it should now have been uploaded to the mediamanager
            unlink($imagePath);
        }
        catch (Exception $e) // Error with the media manager
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, $e->getMessage());
        }
        
        // Return the response
        if ((isset($_REQUEST['json'])) && ((strtolower($_REQUEST['json'])) === 'true'))
        {
                    $data = new stdclass();
                    $data->width = $wcmPhoto->width;
                    $data->height = $wcmPhoto->height;
        
            return new wcmRestResponse(wcmRestResponse::S_CREATED, json_encode($data));
        }
        else
        {
            // Create the xml representation of the data to return
            $dataAsXml = '<width>' . $wcmPhoto->width . '</width>';
            $dataAsXml .= '<height>' . $wcmPhoto->height . '</height>';
            
            return new wcmRestResponse(wcmRestResponse::S_CREATED, $dataAsXml);
        }
    }

    /**
     * Converts an IMAGETYPE_X constant value to the corresponding
     * IMG_X constant value.
     *
     * @param int $imagetype_x The IMAGETYPE_X constant value
     *
     * @return int The corresponding IMG_X constant value, or null if $imagetype_x is unrecognized
     */
    private static function imagetype_x_to_img_x_type($imagetype_x)
    {
        $img_x = null;
        switch ($imagetype_x)
        {
            case IMAGETYPE_GIF:
                $img_x = IMG_GIF;
                break;

            case IMAGETYPE_JPEG:
                $img_x = IMG_JPG;
                break;

            case IMAGETYPE_PNG:
                $img_x = IMG_PNG;
                break;
        }
        return $img_x;
    }
}
