<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restSession.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The wcmRestSession class provides methods for accessing WCM
 * sessions.
 */
final class wcmRestSession
{
    /**
     * The last error message (if any).
     *
     * @var string|null
     */
    protected static $lastErrorMsg = null;

    /**
     * Gets the last error message (if any).
     *
     * @return string|null The last error message,
     *                     or null if none
     */
    public static function getErrorMsg()
    {
        return self::$lastErrorMsg;
    }

    /**
     * Gets the session for a given internal session ID (if any). If
     * the $sessionId parameter is null, returns a new session.
     *
     * @param string|null $sessionId The session ID (default is null for new session)
     * @param string      $language  The session language (default is null to use current language)
     *
     * @return wcmSession|null|false The corresponding session, null
     *                               if the session does not exist,
     *                               false if it has ended
     */
    public static function getSession($sessionId = null, $language = null)
    {
        // Create and possibly refresh the session
        $session = wcmSession::getInstance($sessionId);
        if ($sessionId !== null && !$session->id)
        {
            wcmSession::getInstance()->loadLanguageFiles($language);
            self::$lastErrorMsg = _REST_INVALID_SESSION_ID;
            return null;
        }

        // Check whether the session has ended
        if ($session->endDate)
        {
            wcmSession::getInstance()->loadLanguageFiles($language);
            self::$lastErrorMsg = _REST_EXPIRED_SESSION_ID;
            return false;
        }

        // Set the session language, which causes the language files
        // to be reloaded (this is what we want in all cases)
        if (!$language)
        {
            $language = $session->getLanguage();
        }
        $session->setLanguage($language);

        return $session;
    }

    /**
     * Gets the internal session ID from the external session ID
     * specified by the 'sessionId' request parameter, or from the
     * X-Nstein-Session-ID request header if the $fromHeader parameter
     * is true and the id' request parameter is not specified.
     *
     * If both the request parameter and header are present, the two
     * external session IDs must match.
     *
     * @return string|null|false The internal session ID,
     *                           or null if the external session ID is missing,
     *                           or false if the external session ID is mismatched or corrupt
     */
    public static function getInternalSessionId($fromHeader = false)
    {
        // Get the external session ID
        $externalSessionId_req = getArrayParameter($_REQUEST, 'sessionId', null);

        // TODO Normalize
        $header = 'HTTP_' . str_replace('-', '_', strtoupper(wcmRestResponse::H_X_NSTEIN_SESSION_ID));
        $externalSessionId_hdr = getArrayParameter($_SERVER, $header, null);

        if (!$fromHeader || $externalSessionId_hdr === null)
        {
            $externalSessionId = $externalSessionId_req;
        }
        else
        {
            $externalSessionId = $externalSessionId_hdr;
        }

        // If the external session ID is missing, complain
        if ($externalSessionId === null)
        {
            wcmSession::getInstance()->loadLanguageFiles();
            self::$lastErrorMsg = _REST_MISSING_SESSION_ID;
            return null;
        }

        // If the external session ID is present in both the request
        // parameters and headers, they must match
        if ($externalSessionId_req !== null && $externalSessionId_hdr !== null &&
            $externalSessionId_req != $externalSessionId_hdr)
        {
            wcmSession::getInstance()->loadLanguageFiles();
            self::$lastErrorMsg = _REST_MISMATCHED_SESSION_IDS;
            return false;
        }

        // Decrypt the external session ID (if not empty)
        $decryptedSessionId = null;
        if ($externalSessionId)
        {
            // TODO : Create adequate settings in configuration.xml for encryption keys
            $encryption = new wcmEncryption(ENCRYPTION_KEY, ENCRYPTION_IV);
            $decryptedSessionId = $encryption->decrypt($externalSessionId);
        }
        if (!$decryptedSessionId)
        {
            wcmSession::getInstance()->loadLanguageFiles();
            self::$lastErrorMsg = _REST_CORRUPT_SESSION_ID;
            return false;
        }

        // External session ID == internal session ID + session creation time
        $tmp = explode('_', $decryptedSessionId);
        $sessionId = isset($tmp[0]) ? $tmp[0] : null;
        $createdAt = isset($tmp[1]) ? $tmp[1] : null;
        $garbage = isset($tmp[2]) ? $tmp[2] : null;
        if (!is_numeric($sessionId) || !date_parse($createdAt) || $garbage)
        {
            wcmSession::getInstance()->loadLanguageFiles();
            self::$lastErrorMsg = _REST_CORRUPT_SESSION_ID;
            return false;
        }

        return $sessionId;
    }

    /**
     * Gets the HTTP "GET" method URL for a given session.
     *
     * @param wcmSession $session The session
     * @param string     $baseURL The base URL
     *
     * @return string The HTTP "GET" method URL
     */
    public static function getSessionURL(wcmSession $session, $baseURL)
    {
        $config = wcmConfig::getInstance();
        return $baseURL . $session->getToken();
    }
}
