<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restSessionController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The class wcmRestSessionController provides a RESTful interface to
 * manage WCM sessions.
 */
class wcmRestSessionController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('session');
    }

    /**
     * HTTP "GET" method.
     *
     * GET .../session/{SESSION-ID}
     *
     * {SESSION-ID} => (required) external session ID from previous login
     *
     * => gets the session for an external session ID
     * => validates an X-Nstein-Session-ID header if present
     * => on success, returns the session as XML or JSON (if "json" request parameter present)
     *
     * @return wcmRestResponse => S_OK          on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched sesssion ID
     *                         => S_NOT_FOUND   on invalid sesssion ID
     */
    protected function get()
    {
        // Get the internal session ID
        $sessionId = wcmRestSession::getInternalSessionId();
        if ($sessionId === null || $sessionId === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, wcmRestSession::getErrorMsg());
        }

        // Get the corresponding session
        $session = wcmRestSession::getSession($sessionId);
        if ($session === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, wcmRestSession::getErrorMsg());
        }
        if ($session === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_GONE, wcmRestSession::getErrorMsg());
        }

        // The response is the session encoded as JSON or XML
        if (isset($_REQUEST['json']))
        {
            $body = $session->toJSON();
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body = $session->toXML();
            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "PUT" method.
     *
     * => Currently does the same thing as the HTTP "POST" method
     *
     * @return wcmRestResponse => S_CREATED     on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on conflicting parameters
     *                         => S_BAD_REQUEST on missing object class name or ID
     *                         => S_BAD_REQUEST on missing transition ID or code
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid object class name or ID
     *                         => S_INTERNAL_SERVER_ERROR on transition execution failure
     * @see wcmRestSessionController::post()
     */
    protected function put()
    {
        return $this->post();
    }

    /**
     * HTTP "POST" method.
     *
     * POST .../session?username={USERNAME}&password={PASSWORD}
     *
     * {USERNAME} => (required) user name
     * {PASSWORD} => (required) user password
     *
     * => checks for an X-Nstein-Session-ID header
     * => logs the user into the system (if header not present)
     * => validates the existing session against the username and password (if header present)
     * => on success, returns a "Location" header pointing to the session
     *
     * @return wcmRestResponse => S_CREATED      on success
     *                         => S_BAD_REQUEST  on missing username or password
     *                         => S_BAD_REQUEST  on corrupt session ID (if present)
     *                         => S_NOT_FOUND    on invalid sesssion ID (if present)
     *                         => S_UNAUTHORIZED on invalid username or password
     */
    protected function post()
    {
        // Get the username and password
        $username = getArrayParameter($_REQUEST, 'username', null);
        $password = getArrayParameter($_REQUEST, 'password', null);
        $language = getArrayParameter($_REQUEST, 'language', null);

        if (!$username || !$password)
        {
            wcmSession::getInstance()->loadLanguageFiles($language);
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_CREDENTIALS);
        }

        // Get the internal session ID (if any) from the request
        // headers (note the "true" in the method call)
        $sessionId = wcmRestSession::getInternalSessionId(true);
        if ($sessionId === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, wcmRestSession::getErrorMsg());
        }

        // Get the corresponding session, or a new session if $sessionId is null
        $session = wcmRestSession::getSession($sessionId, $language);
        if ($session === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, wcmRestSession::getErrorMsg());
        }
        if ($session === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_GONE, wcmRestSession::getErrorMsg());
        }

        if ($sessionId === null)
        {
            // New session => log the user into the system
            if (!$session->login($username, $password))
            {
                wcmSession::getInstance()->loadLanguageFiles($language);
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, _REST_INVALID_CREDENTIALS);
            }
        }
        else
        {
            // Existing session => validate user against given username and password
            $user = new wcmUser(null, $session->userId);
            if (!$user->id || $user->login != $username || $user->password != md5($password))
            {
                wcmSession::getInstance()->loadLanguageFiles($language);
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, _REST_INVALID_CREDENTIALS);
            }
            
            $resultSet = $session->getProject()->database->executeQuery('select count(*) from wcm_session where id=' . $session->id);

            // Start session as authenticated user, startSession will return false the second time if
            // called twice in under a second so we also make sure the count of a select for the session returns 0
            // before returning an error
            if (!$session->startSession($session->userId) && $resultSet->count() === 0)
            {
                return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR);
            }
        }

        // The response is a "Location" header pointing to the session
        $status = ($sessionId === null ? wcmRestResponse::S_CREATED : wcmRestResponse::S_FOUND);
        $location = wcmRestSession::getSessionURL($session, $this->baseURL);

        $headers = array(
            wcmRestResponse::H_LOCATION            => $location,
            wcmRestResponse::H_X_NSTEIN_SESSION_ID => $session->getToken(),
        );

        $usergroups = array();
        if ($session->id)
        {
            $user = new wcmUser(null, $session->userId);
            $usergroups = $user->getGroups();

        }
        if (isset($_REQUEST['json']))
        {
            $body = json_encode($usergroups);
        }
        else
        {
            // create an assoc array for xmlEncodeArray so the xml has a root element
            $body = wcmXML::xmlEncodeArray(array('groups' => $usergroups));
        }

        return new wcmRestResponse($status, $body, $headers);
    }

    /**
     * HTTP "DELETE" method.
     *
     * DELETE .../session/{SESSION-ID}
     *
     * {SESSION-ID} => (required) external session ID from previous login
     *
     * => logs the user out of the system
     * => validates an X-Nstein-Session-ID header if present
     *
     * @return wcmRestResponse => S_NO_CONTENT  on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched sesssion ID
     *                         => S_NOT_FOUND   on invalid sesssion ID
     */
    protected function delete()
    {
        // Get the internal session ID
        $sessionId = wcmRestSession::getInternalSessionId();
        if ($sessionId === null || $sessionId === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, wcmRestSession::getErrorMsg());
        }

        // Get the corresponding session
        $session = wcmRestSession::getSession($sessionId);
        if ($session === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, wcmRestSession::getErrorMsg());
        }
        if ($session === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_GONE, wcmRestSession::getErrorMsg());
        }

        // Log the user out of the session
        $session->logout();

        return new wcmRestResponse(wcmRestResponse::S_NO_CONTENT);
    }
}
