<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.imageFileManager.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-mediaManager-api
 * @version     4.x
 */

/**
 * This class handles responsibility of generating
 * file paths for images.
 */
class cmfImageFileManager
{
    /**
     * The extension for image files.
     * @var string
     */
    const IMG_EXTENSION = '.jpg';
    
    /**
     * The root directory where the original image 
     * files are stored.
     * @var string
     */
    private $originalDirectory;

    /**
     * The directory where the thumbnails are stored.
     * @var string
     */
    private $thumbnailDirectory;
    
    /**
     * The builder for image file path generation.
     * @var Nstein_Path_Builder
     */
    private $pathBuilder;
    
    /**
     * Public constructor.
     * @param string $originalDirectory
     * @param string $thumbnailDirectory
     * @param Nstein_Path_Builder $pathBuilder The image file path builder.
     */
    public function __construct($originalDirectory, $thumbnailDirectory, Nstein_Path_Builder $pathBuilder)
    {
        $this->originalDirectory = $originalDirectory;
        $this->thumbnailDirectory = $thumbnailDirectory;
        $this->pathBuilder = $pathBuilder;
    }
    
    /**
     * Get the path of an original image file.
     * @param int $id The ID of the image.
     * @return string The path of the original image file.
     */
    public function getImagePath($id)
    {
        return $this->originalDirectory . $this->getRelativePath($id) . $id . self::IMG_EXTENSION;
    }
    
    /**
     * Get the path of a formatted image file.
     * @param int $id The ID of the image.
     * @param string $format The format.
     * @return string The path of the formatted image file.
     */
    public function getFormattedImagePath($id, $format)
    {
        return $this->getThumbnailDirectory($id) . $this->getThumbnailPrefix($id) . $format . self::IMG_EXTENSION;
    }
    
    /**
     * Get the path of a resized image file.
     * @param int $id The ID of the image.
     * @param int $width The maximum width of the thumbnail.
     * @param int $height The maximum height of the thumbnail.
     * @return string The path of the resized image file.
     */
    public function getResizedImagePath($id, $width, $height)
    {
        return $this->getThumbnailDirectory($id) . $this->getThumbnailPrefix($id) . ($width ? $width : '-') . 'x' . ($height ? $height : '-') . self::IMG_EXTENSION;
    }
    
    /**
     * Get a list of all existing thumbnails for a given
     * image.
     * @param int $id The ID of the image.
     * @return array The path of all existing thumbnails.
     */
    public function getThumbnails($id)
    {
        return glob($this->getThumbnailDirectory($id) . $this->getThumbnailPrefix($id) . '*');
    }
    
    /**
     * Get the file prefix for all thumbnails of a
     * given image ID.
     * @param int $id The image ID.
     * @return string The thumbnail file prefix.
     */
    private function getThumbnailPrefix($id)
    {
        return $id . '_';
    }
    
    /**
     * Get the directory where the thumbnails of a given
     * image are stored.
     * @param int $id The ID of the image.
     * @return string
     */
    private function getThumbnailDirectory($id)
    {
        return $this->thumbnailDirectory . $this->getRelativePath($id);
    }
    
    /**
     * Get the relative path for a given ID.
     * @param int $id The ID.
     * @return string The relative path.
     */
    private function getRelativePath($id)
    {
        return $this->pathBuilder->generatePath($id);
    }
}