<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.imageHelper.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-mediaManager-api
 * @version     4.x
 */

/**
 * An image helper that handles image processing to
 * create automatic thumbnails, cropped thumbnails
 * or resized thumbnails.
 */
class cmfImageHelper
{
    /**
     * Indicate if GD2 is enabled and should be used
     * instead of standard GD functions.
     * @var boolean
     */
    private static $isGd2Enabled = null;
    
    /**
     * Check which GD version should be used.
     * @return void
     */
    private static function checkGdVersion()
    {
        if (null === self::$isGd2Enabled)
        {
            self::$isGd2Enabled = function_exists('imagecreatetruecolor') && function_exists('imagecopyresampled');
        }
    }
    
    /**
     * The quality for JPEG thumbnails.
     * @var int
     */
    private $jpegQuality;
    
    /**
     * The top/bottom ratio when creating portrait 
     * thumbnails (default: 0.5). For example, a value 
     * of 0.25 will crop only 1/4 of the top and the 
     * remaining 3/4 of the bottom of the image.
     * @var float
     */
    private $portraitRatio;
    
    /**
     * Public constructor.
     * @param int $jpegQuality The quality for JPEG 
     * thumbnails (default: 75).
     * @param float $portraitRatio The top/bottom ratio
     * when creating portrait thumbnails (default: 0.5).
     * For example, a value of 0.25 will crop only 1/4
     * of the top and the remaining 3/4 of the bottom
     * of the image.
     */
    public function __construct($jpegQuality = 75, $portraitRatio = 0.5)
    {
        self::checkGdVersion();
        
        $this->jpegQuality = $jpegQuality;
        $this->portraitRatio = $portraitRatio;
    }
    
    /**
     * Get the JPEG quality value, used when creating
     * JPEG thumbnails.
     * @return int
     */
    public function getJpegQuality()
    {
        return $this->jpegQuality;
    }
    
    /**
     * Set the JPEG quality value, used when creating
     * JPEG thumbnails.
     * @param int $value The new value.
     * @return void
     */
    public function setJpegQuality($value)
    {
        $this->jpegQuality = $value;
    }
    
    /**
     * Get the top/bottom ratio when creating portrait 
     * thumbnails (default: 0.5). For example, a value 
     * of 0.25 will crop only 1/4 of the top and the 
     * remaining 3/4 of the bottom of the image.
     * @return float
     */
    public function getPortraitRatio()
    {
        return $this->portraitRatio;
    }
    
    /**
     * Set the top/bottom ratio when creating portrait 
     * thumbnails (default: 0.5). For example, a value 
     * of 0.25 will crop only 1/4 of the top and the 
     * remaining 3/4 of the bottom of the image.
     * @param float $value The new value.
     * @return void
     */
    public function setPortraitRatio($value)
    {
        $this->portraitRatio = $value;
    }
    
    /**
     * Open an image file.
     * @param string $path The path of the image file.
     * @return array An array containing the size info
     * of the image (@see getimagesize), the type of 
     * image (@see imagetypes) and the image resource.
     * @exception Exception If the file doesn't exist or
     * if the image format is not supported.
     */
    private function openImageFile($path)
    {
        if (!file_exists($path))
        {
            throw new Exception('File not found : ' . $path);
        }
        
        $size = getimagesize($path);
        
        switch ($size[2])
        {
            case IMAGETYPE_GIF:
                if (!(imagetypes() & IMG_GIF))
                {
                    throw new Exception('Unsupported image type: GIF');
                }
                $image = imagecreatefromgif($path);
                $type = IMG_GIF;
                break;

            case IMAGETYPE_JPEG:
                if (!(imagetypes() & IMG_JPG))
                {
                    throw new Exception('Unsupported image type: JPEG');
                }
                $image = imagecreatefromjpeg($path);
                $type = IMG_JPG;
                break;

            case IMAGETYPE_PNG:
                if (!(imagetypes() & IMG_PNG))
                {
                    throw new Exception('Unsupported image type: PNG');
                }
                $image = imagecreatefrompng($path);
                $type = IMG_PNG;
                break;
                
            default:
                throw new Exception('Unsupported image type');
        }

        return array($size, $type, $image);
    }
    
    /**
     * Create a thumbnail.
     * @param string $destPath The destination file path.
     * @param int $destType The destination image type.
     * @param int $destQuality The destination image quality.
     * @param int $destWidth The destination image width.
     * @param int $destHeight The destination image height.
     * @param resource $source The source image resource.
     * @param int $srcX The source X offset.
     * @param int $srcY The source Y offset.
     * @param int $srcWidth The source image width.
     * @param int $srcHeight The source image height.
     * @return void
     */
    private function createThumbnail($destPath, $destType, $destQuality, $destWidth, $destHeight, $source, $srcX, $srcY, $srcWidth, $srcHeight)
    {
        if (self::$isGd2Enabled)
        {
            $destImg = imagecreatetruecolor($destWidth, $destHeight);
            $bgc = imagecolorallocate($destImg, 255, 255, 255);
            imagefilledrectangle($destImg, 0, 0, $destWidth, $destHeight, $bgc);
            imagecopyresampled($destImg, $source, 0, 0, $srcX, $srcY, $destWidth, $destHeight, $srcWidth, $srcHeight);
        }
        else
        {
            $destImg = imagecreate($destWidth, $destHeight);
            $bgc = imagecolorallocate($destImg, 255, 255, 255);
            imagefilledrectangle($destImg, 0, 0, $destWidth, $destHeight, $bgc);
            imagecopyresized($destImg, $source, 0, 0, $srcX, $srcY, $destWidth, $destHeight, $srcWidth, $srcHeight);
        }
        
        $this->saveToFile($destImg, $destPath, $destType, $destQuality);
        
        imagedestroy($destImg);
    }
    
    /**
     * Save an image in a given format.
     * @param resource $image The image resource to save.
     * @param string $path The path where the file will be saved.
     * @param int $type The format of the output image file (IMG_GIF, IMG_JPG, IMG_PNG).
     * @param int $quality The quality of the output image.
     * @return void
     * @exception Exception If the image type is not supported.
     */
    private function saveToFile($image, $path, $type, $quality)
    {
        if (!(imagetypes() & $type))
        {
            throw new Exception();
        }

        // Create the directory if needed.
        $directory = dirname($path);
        if (!is_dir($directory))
        {
            mkdir($directory, 0777, true);
        }

        switch ($type)
        {
            case IMG_GIF:
                imagegif($image, $path);
                break;

            case IMG_JPG:
                imagejpeg($image, $path, $quality);
                break;

            case IMG_PNG:
                imagepng($image, $path);
                break;
        }
    }
    
    /**
     * Create a thumbnail of an image.
     * @param string $source The path of the source file.
     * @param string $destination The path where the thumbnail 
     * should be saved.
     * @param int $width The width of the thumbnail.
     * @param int $height The height of the thumbnail.
     * @param int $outputType The thumbnail image type
     * (default: IMG_JPG).
     * @return void
     * @exception Exception If an error occurs.
     */
    public function thumb($source, $destination, $width, $height, $outputType = IMG_JPG)
    {
        list($size, $type, $image) = $this->openImageFile($source);
        
        $w = $size[0];
        $h = $size[1];
        
        $ratioFull = $w / $h;
        $ratioThumb = $width / $height;
        
        if ($ratioThumb > $ratioFull)
        {
            $srcWidth = $w;
            $srcHeight = $srcWidth / $ratioThumb;
            $srcX = 0;
            if ($w >= $h)
            {
                // The image is landscape.
                $srcY = ($h - $srcHeight) * 0.5;
            }
            else
            {
                 // The image is portrait.
                $srcY = ($h - $srcHeight) * $this->portraitRatio;
            }
        }
        else
        {
            $srcHeight = $h;
            $srcWidth = $srcHeight * $ratioThumb;
            $srcY = 0;
            $srcX = ($w - $srcWidth) / 2;
        }
        
        $this->createThumbnail(
            $destination,
            $outputType,
            $this->jpegQuality,
            $width,
            $height,
            $image,
            $srcX,
            $srcY,
            $srcWidth,
            $srcHeight
        );
    }
    
    /**
     * Create a resized version of an image, using a 
     * given maximum size (width, height or both).
     * @param string $source The path of the source file.
     * @param string $destination The path where the thumbnail 
     * should be saved.
     * @param int $width The width of the thumbnail.
     * @param int $height The height of the thumbnail.
     * @param boolean $proportional Indicate if proportions
     * should be conserved (default: true).
     * @param int $outputType The thumbnail image type
     * (default: IMG_JPG).
     * @return void
     * @exception Exception If an error occurs.
     */
    public function resize($source, $destination, $width, $height, $proportional = true, $outputType = IMG_JPG)
    {
        list($size, $type, $image) = $this->openImageFile($source);
        
        $srcWidth = $size[0];
        $srcHeight = $size[1];
        
        if (!$proportional)
        {
            $thumbWidth = $width;
            $thumbHeight = $height;
        }
        else
        {
            $scale = 1;
            if ($width && $srcWidth > $width)
            {
                // We have a maximum width and the
                // photo is wider
                $scale = $width / $srcWidth;
            }
            if ($height && ($srcHeight * $scale) > $height)
            {
                // We have a maximum height and the
                // scaled photo is higher.
                $scale = $height / $srcHeight;
            }

            // Calculate the scaled size.
            $thumbWidth = $srcWidth * $scale;
            $thumbHeight = $srcHeight * $scale;
        }
        
        $this->createThumbnail(
            $destination, 
            $outputType, 
            $this->jpegQuality, 
            $thumbWidth, 
            $thumbHeight, 
            $image, 
            0, 
            0, 
            $srcWidth, 
            $srcHeight
        );
    }
    
    /**
     * Create a cropped thumbnail.
     * @param string $source The path of the source file.
     * @param string $destination The path where the thumbnail 
     * should be saved.
     * @param int $width The width of the thumbnail.
     * @param int $height The height of the thumbnail.
     * @param int $x1 The left offset on the X axis.
     * @param int $y1 The top offset on the Y axis.
     * @param int $x2 The right offset on the X axis.
     * @param int $y2 The bottom offset on the Y axis.
     * @param int $outputType The thumbnail image type
     * (default: IMG_JPG).
     * @return void
     * @exception Exception If an error occurs.
     */
    public function crop($source, $destination, $width, $height, $x1, $y1, $x2, $y2, $outputType = IMG_JPG)
    {
        list($size, $type, $image) = $this->openImageFile($source);
        
        $this->createThumbnail(
            $destination, 
            $outputType, 
            $this->jpegQuality, 
            $width, 
            $height, 
            $image, 
            $x1, 
            $y1, 
            abs($x1 - $x2), 
            abs($y1 - $y2)
        );
    }
}