<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.imageManager.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-mediaManager-api
 * @version     4.x
 */

/**
 * The main media manager class for images.
 */
class cmfImageManager
{
    /**
     * List of the allowed image extensions.
     * @var array
     */
    private static $allowedExtensions = array(
        'jpg',
        'gif',
        'png'
    );

    /**
     * The XML document of photo format definition.
     * @var DOMDocument
     */
    private $photoFormatDefinition = null;
    
    /**
     * Formats (filled from WCM formats).
     * @var array
     */
    private $wcmFormats = null;
    
    /**
     * The file manager.
     * @var cmfImageFileManager
     */
    private $fileManager;
    
    /**
     * The WCM photo format provider.
     * @var cmfWcmPhotoFormatProvider
     */
    private $formatProvider;
    
    /**
     * The image helper used to create thumbnails.
     * @var cmfImageHelper
     */
    private $imageHelper;
    
    /**
     * The image database used to retrieve format 
     * information about images.
     * @var cmfImageDatabase
     */
    private $database;

    /**
     * Public constructor.
     * @param cmfImageFileManager $fileManager The file manager.
     * @param cmfWcmPhotoFormatProvider $formatProvider The
     * WCM photo format provider.
     * @param cmfImageHelper $imageHelper The image helper
     * used to create thumbnails.
     * @param cmfImageDatabase $database The image database.
     */
    public function __construct(cmfImageFileManager $fileManager, cmfWcmPhotoFormatProvider $formatProvider, cmfImageHelper $imageHelper, cmfImageDatabase $database)
    {
        $this->fileManager = $fileManager;
        $this->formatProvider = $formatProvider;
        $this->imageHelper = $imageHelper;
        $this->database = $database;
    }
    
    /**
     * Check if an image exists in the media manager.
     * @param int $id The image ID.
     * @return boolean True if the image exists,
     * false otherwise.
     */
    public function imageExists($id)
    {
        $fileName = $this->fileManager->getImagePath($id);
        
        return file_exists($fileName);
    }
    
    /**
     * Get the path of an original image file.
     * @param int $id The ID of the image.
     * @return string The path of the original image file,
     * or null if no image has been found for the provided ID.
     * @exception InvalidArgumentException If $id is not
     * numeric.
     */
    public function getImage($id)
    {
        if (!is_numeric($id))
        {
            throw new InvalidArgumentException('Argument $id must be numeric.');
        }
        
        $fileName = $this->fileManager->getImagePath($id);
        
        if (!file_exists($fileName))
        {
            return null;
        }
        
        return $fileName;
    }
    
    /**
     * Get the path of a formatted thumbnail, if it exists.
     * @param int $id The ID of the image.
     * @param string $format The format of the thumbnail.
     * @return string The path of the formatted thumbnail,
     * or null if no thumbnail has been found for the
     * provided ID, using the provided format.
     * @exception InvalidArgumentException If $id is not
     * numeric, or if $format is not a string, or if
     * $format is not a valid format.
     */
    public function getFormattedImage($id, $format)
    {
        if (!is_numeric($id))
        {
            throw new InvalidArgumentException('Argument $id must be numeric.');
        }
        
        if (!is_string($format))
        {
            throw new InvalidArgumentException('Argument $format must be a string.');
        }

        $formats = $this->getWcmFormats();
        if (!array_key_exists($format, $formats))
        {
            throw new InvalidArgumentException('Invalid format.');
        }
        
        $fileName = $this->fileManager->getFormattedImagePath($id, $format);
        
        if (!file_exists($fileName))
        {
            return null;
        }
        
        return $fileName;
    }

    /**
     * Generate a thumbnail of a given image, using
     * a given format.
     * @param int $id The ID of the image.
     * @param string $format The format.
     * @return string The path of the generated thumbnail,
     * or null if the original image was not found.
     * @exception InvalidArgumentException If $id is not
     * numeric, or if $format is not a string, or if
     * $format is not a valid format.
     * @exception ErrorException If an error occurs while
     * generating the thumbnail.
     */
    public function generateFormattedImage($id, $format)
    {
        if (!is_numeric($id))
        {
            throw new InvalidArgumentException('Argument $id must be numeric.');
        }
        
        if (!is_string($format))
        {
            throw new InvalidArgumentException('Argument $format must be a string.');
        }

        $formats = $this->getWcmFormats();
        if (!array_key_exists($format, $formats))
        {
            throw new InvalidArgumentException('Invalid format.');
        }
        $formatInfo = $formats[$format];
        $ratio = $formatInfo['ratio'];
        
        $original = $this->fileManager->getImagePath($id);
        if (null === $original)
        {
            // Original image not found.
            return null;
        }

        // Load user formats.
        $userFormats = $this->database->getFormat($id);
        
        // Generate thumbnail path.
        $thumbnail = $this->fileManager->getFormattedImagePath($id, $format);
            
        if (!array_key_exists($ratio, $userFormats) || (0 == $userFormats[$ratio]['x1'] && 0 == $userFormats[$ratio]['x2'] && 0 == $userFormats[$ratio]['y1'] && 0 == $userFormats[$ratio]['y2']))
        {
            // Create automatic thumbnail.
            $this->imageHelper->thumb(
                $original,
                $thumbnail, 
                $formatInfo['width'], 
                $formatInfo['height']
            );
        }
        else
        {
            // Create thumbnail using the image's cropping info.
            $this->imageHelper->crop(
                $original,
                $thumbnail, 
                $formatInfo['width'], 
                $formatInfo['height'], 
                $userFormats[$ratio]['x1'], 
                $userFormats[$ratio]['y1'], 
                $userFormats[$ratio]['x2'], 
                $userFormats[$ratio]['y2']
            );
        }

        return $thumbnail;
    }
    
    /**
     * Get the path of a resized thumbnail. At least
     * one of $width and $height must be specified.
     * @param int $id The ID of the image.
     * @param int $width The maximum width of the
     * thumbnail. If specified, must be a positive value.
     * @param int $height The maximum height of the
     * thumbnail. If specified, must be a positive value.
     * @return string The path of the resized thumbnail.
     * @exception InvalidArgumentException If $id is not
     * numeric, if none of $width and $height has been
     * specified, if specified values for $width and/or
     * $height are not positive integers.
     */
    public function getResizedImage($id, $width, $height)
    {
        if (!is_numeric($id))
        {
            throw new InvalidArgumentException('Argument $id must be numeric.');
        }
        
        if (!$width && !$height)
        {
            throw new InvalidArgumentException('At least one of arguments $width and $height must be not null.');
        }
        
        if ($width && (!is_numeric($width) || (int)$width < 0))
        {
            throw new InvalidArgumentException('Argument $width must be a positive numeric value.');
        }
        
        if ($height && (!is_numeric($height) || (int)$height < 0))
        {
            throw new InvalidArgumentException('Argument $height must be a positive numeric value.');
        }
        
        $fileName = $this->fileManager->getResizedImagePath($id, $width, $height);
        
        if (!file_exists($fileName))
        {
            return null;
        }
        
        return $fileName;
    }

    /**
     * Generate a resized thumbnail of a given image.
     * @param int $id The ID of the image.
     * @param int $width The maximum width of the
     * thumbnail. If specified, must be a positive value.
     * @param int $height The maximum height of the
     * thumbnail. If specified, must be a positive value.
     * @return string The path of the generated resized 
     * thumbnail, or null if the original image was not 
     * found.
     * @exception InvalidArgumentException If $id is not
     * numeric, if none of $width and $height has been
     * specified, if specified values for $width and/or
     * $height are not positive integers.
     * @exception ErrorException If an error occurs while
     * generating the thumbnail.
     */
    public function generateResizedImage($id, $width, $height)
    {
        if (!is_numeric($id))
        {
            throw new InvalidArgumentException('Argument $id must be numeric.');
        }
        
        if (!$width && !$height)
        {
            throw new InvalidArgumentException('At least one of arguments $width and $height must be not null.');
        }
        
        if ($width && (!is_numeric($width) || (int)$width < 0))
        {
            throw new InvalidArgumentException('Argument $width must be a positive numeric value.');
        }
        
        if ($height && (!is_numeric($height) || (int)$height < 0))
        {
            throw new InvalidArgumentException('Argument $height must be a positive numeric value.');
        }

        $original = $this->fileManager->getImagePath($id);
        if (null === $original)
        {
            // Original image not found.
            return null;
        }
        
        // Compute thumbnail path.
        $thumbnail = $this->fileManager->getResizedImagePath($id, $width, $height);

        // Create resized thumbnail.
        $this->imageHelper->resize(
            $original,
            $thumbnail, 
            $width, 
            $height
        );

        return $thumbnail;
    }
    
    /**
     * Posts an image from a given URL to the media manager.
     * @param int $id The image ID.
     * @param string $imageUrl The URL of the image.
     * @param array $formats The format information.
     * @return void
     * @exception ErrorException If an error occurs.
     */
    public function addImage($id, $imageUrl, array $formats)
    {
        // Generate original image path.
        $path = $this->fileManager->getImagePath($id);
        
        // Create directories.
        $dir = dirname($path);
        if (!is_dir($dir))
        {
            mkdir($dir, 0777, true);
        }

        // Copy image from provided URL.
        // ** Works only if PHP stream wrappers are enabled.
        copy($imageUrl, $path);
 
        // Delete existing thumbnails of old image.
        $this->removeThumbnails($id);

        // Save the new format info.
        $this->database->add($id, $formats);
    }
    
    /**
     * Posts an uploaded image to the media manager.
     * @param int $id The image ID.
     * @param string $tempFile The path of the temporary 
     * uploaded file.
     * @param array $formats The format information.
     * @return void
     * @exception ErrorException If an error occurs.
     */
    public function addUploadedImage($id, $tempFile, array $formats)
    {
        // Generate original image path.
        $path = $this->fileManager->getImagePath($id);
        
        // Create directories.
        $dir = dirname($path);
        if (!is_dir($dir))
        {
            mkdir($dir, 0777, true);
        }
        
        // Move uploaded file.
        move_uploaded_file($tempFile, $path);
        
        // Delete existing thumbnails of old image.
        $this->removeThumbnails($id);
        
        // Save the new format info.
        $this->database->add($id, $formats);
    }
    
    /**
     * Update the format information of a given image.
     * @param int $id The image ID.
     * @param array $formats The format information.
     * @return void
     * @exception InvalidArgumentException If the provided
     * ID refers to an unknown image.
     * @exception ErrorException If an error occurs.
     */
    public function updateFormats($id, array $formats)
    {
        if (!$this->imageExists($id))
        {
            // Unknown image.
            throw new InvalidArgumentException('Unknown ID.');
        }
        
        // Save the new format info.
        $this->database->add($id, $formats);
        
        // Delete existing thumbnails of old image.
        $this->removeThumbnails($id);
    }

    /**
     * Remove an image.
     * @param int $id The image ID.
     * @return boolean True if the image was deleted,
     * false if the image did not exist.
     * @exception ErrorException If an error occurs.
     */
    public function removeImage($id)
    {
        if ($this->imageExists($id))
        {
            $this->removeOriginal($id);
            $this->database->delete($id);
            $this->removeThumbnails($id);
            return true;
        }
        
        return false;
    }

    /**
     * Get the WCM photo format XML from the provider.
     * @return DOMDocument The XML document containing the
     * WCM photo formats definition.
     */
    private function retrieveConfigXml()
    {
        return $this->formatProvider->getPhotoFormats();
    }

    /**
     * Get the WCM photo formats.
     * @return array The WCM photo formats.
     */
    private function getWcmFormats()
    {
        if (null === $this->wcmFormats)
        {
            // TODO
            // array should be caches (i.e in a file);
            // if not we call the ws
            if (null === $this->photoFormatDefinition)
            {
                $this->photoFormatDefinition = $this->retrieveConfigXml();
            }

            $ratios = $this->photoFormatDefinition->getElementsByTagName('ratio');

            foreach ($ratios as $ratio)
            {
                $sizes = $ratio->getElementsByTagName('size');

                foreach ($sizes as $size)
                {
                    $this->wcmFormats[$size->getAttribute('code')] = array(
                        'ratio' => $ratio->getAttribute('name'),
                        'width' => (int)$size->getAttribute('width'),
                        'height' => (int)$size->getAttribute('height'),
                    );
                }
            }
        }

        return $this->wcmFormats;
    }
    
    /**
     * Remove all thumbnails of an image.
     * @param int $id The image ID.
     * @return void
     * @exception ErrorException If an error occurs.
     */
    private function removeThumbnails($id)
    {
        $files = $this->fileManager->getThumbnails($id);
        foreach ($files as $filename)
        {
            unlink($filename);
        }
    }

    /**
     * Remove the original image.
     * @param int $id The image ID.
     * @return void
     * @exception ErrorException If an error occurs.
     */
    private function removeOriginal($id)
    {
        unlink($this->fileManager->getImagePath($id));
    }
}