<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.sessionCache.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * Cache implementation that stores the data in the
 * PHP session.
 */
class cmfSessionCache implements cmfCache
{
    /**
     * The key of the $_SESSION entry where
     * the cache array is stored.
     * @var string
     */
    private $sessionKey;

    /**
     * Public constructor.
     * @param string $sessionKey The key of the $_SESSION
     * entry where the cache array is stored.
     */
    public function __construct($sessionKey = null)
    {
        $this->sessionKey = $sessionKey;
        $this->ensureSessionStarted();
    }

    /**
     * Get the key of the $_SESSION entry
     * where the cache array is stored.
     * @return string
     */
    public function getSessionKey()
    {
        return $this->sessionKey;
    }

    /**
     * Set the key of the $_SESSION entry
     * where the cache array is stored.
     * @param string $value The new value.
     * @return void
     */
    public function setSessionKey($value)
    {
        $this->sessionKey = $value;
    }

    /**
     * If the PHP session has not been started, start it.
     * @return void
     */
    private function ensureSessionStarted()
    {
        cmfToolbox::ensureSessionStarted();
    }

    /**
     * (non-PHPdoc)
     * @see cmf/includes/cache/cmfCache#get()
     */
    public function get($key)
    {
        if (!isset($_SESSION[$this->sessionKey][$key]))
        {
            // Cache entry doesn't exist.
            return false;
        }

        list($expire, $value) = $_SESSION[$this->sessionKey][$key];
        if ($expire && ($expire <= $_SERVER['REQUEST_TIME']))
        {
            // Cache entry is expired, so delete it.
            $this->delete($key);
            return false;
        }

        return $value;
    }

    /**
     * (non-PHPdoc)
     * @see cmf/includes/cache/cmfCache#set()
     */
    public function set($key, $value, $ttl = 0)
    {
        if ((!is_string($key) || '' == $key) || !is_numeric($key))
        {
            throw new InvalidArgumentException();
        }
        
        if (!isset($_SESSION[$this->sessionKey]))
        {
            // Initialize the $_SESSION array.
            $_SESSION[$this->sessionKey] = array();
        }

        // Compute the expiration timestamp.
        $expire = $ttl ? time() + $ttl : null;

        // Cache the value in session.
        $_SESSION[$this->sessionKey][$key] = array($expire, $value);
    }

    /**
     * (non-PHPdoc)
     * @see cmf/includes/cache/cmfCache#delete()
     */
    public function delete($key)
    {
        unset($_SESSION[$this->sessionKey][$key]);
    }

    /**
     * (non-PHPdoc)
     * @see cmf/includes/cache/cmfCache#clear()
     */
    public function clear()
    {
        unset($_SESSION[$this->sessionKey]);
    }
}