<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.searchConfig.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The search engine configuration.
 */
class cmfSearchConfig
{
    /**
     * Index definitions. It is required only if the search plugin
     * is not able to load the index definition from the underlying
     * native search engine.
     * @var array of cmfIndexDefinition
     */
    public $indexDefinitions = null;

    /**
     * Facet definitions.
     * @var array of cmfFacetDefinition
     */
    public $facetDefinitions = null;

    /**
     * The search plugin info.
     * @var cmfPluginInfo
     */
    public $searchPluginInfo = null;

    /**
     * Construct a search configuration from a config array.
     * @param array $configuration The configuration data.
     */
    public function __construct(array $configuration = null)
    {
        if ($configuration)
        {
            $this->load($configuration);
        }
    }

    /**
     * Load data from a configuration array.
     * @param array $configuration The configuration data.
     */
    private function load(array $configuration)
    {
        // Load search plugin information
        $pluginData = cmfToolbox::getArrayParameter($configuration, 'searchPlugin', null);

        $this->searchPluginInfo = cmfPluginInfo::fromArray($pluginData);
        if (null === $this->searchPluginInfo)
        {
            throw new Exception('cmfSearchConfig: the search plugin information are required.');
        }

        // Load index definitions (if specified).
        $indexes = cmfToolbox::getArrayParameter($configuration, 'indexes', null);
        if ($indexes)
        {
            $this->indexDefinitions = array();
            foreach ($indexes as $name => $data)
            {
                $type = cmfToolbox::getArrayParameter($data, 'type', null);
                $isMultiValued = cmfToolbox::getArrayParameter($data, 'isMultiValued', false);
                $aliases = cmfToolbox::getArrayParameter($data, 'aliases', array());

                $this->indexDefinitions[$name] = new cmfIndexDefinition($name, $type, $isMultiValued, $aliases);
            }
        }

        // Load facet definitions.
        $this->facetDefinitions = array();
        $facets = cmfToolbox::getArrayParameter($configuration, 'facets', null);
        if ($facets)
        {
            foreach ($facets as $name => $data)
            {
                $index = cmfToolbox::getArrayParameter($data, 'index', null);
                $caption = cmfToolbox::getArrayParameter($data, 'caption', $index);
                $searchIndex = cmfToolbox::getArrayParameter($data, 'searchIndex', $index);
                $sort = cmfToolbox::getArrayParameter($data, 'sort', false);
                $type = cmfToolbox::getArrayParameter($data, 'type', cmfFacetDefinition::FACET_TYPE_SINGLE_VALUE);
                $max = cmfToolbox::getArrayParameter($data, 'max', 0);

                $processorData = cmfToolbox::getArrayParameter($data, 'processor', array());
                $providerData = cmfToolbox::getArrayParameter($data, 'provider', array());

                $this->facetDefinitions[$name] = new cmfFacetDefinition(
                    $name, $caption, $index, $searchIndex, $sort, $type, $max,
                    cmfPluginInfo::fromArray($processorData),
                    cmfPluginInfo::fromArray($providerData));
            }
        }
    }

    /**
     * Return an array containing the name of all facets defined for the search engine.
     * @return array The names of the available facets.
     */
    public function getFacetNames()
    {
        return array_keys($this->facetDefinitions);
    }

    /**
     * Return a facet definition.
     * @param $facet The facet name.
     * @return cmfFacetDefinition
     */
    public function getFacetDefinition($facet)
    {
        return cmfToolbox::getArrayParameter($this->facetDefinitions, $facet, null);
    }

    /**
     * Load data from a PHP search configuration file.
     * @param string $path The absolute path of the configuration file.
     * @return boolean True on success, false if the file does not exist.
     */
    public function loadFromFile($path)
    {
        if (!file_exists($path))
        {
            return false;
        }

        include $path;

        if (!isset($configuration) || !is_array($configuration))
        {
            return false;
        }

        $this->load($configuration);

        return true;
    }

    /**
     * Create a cmfSearchConfig instance from a given PHP search configuration file.
     * @param string $path The absolute path of the configuration file.
     * @return cmfSearchConfig|null The cmfSearchConfig instance or null the file does not exist or is invalid.
     */
    public static function fromFile($path)
    {
        $config = new cmfSearchConfig();
        if ($config->loadFromFile($path))
        {
            return $config;
        }
        return false;
    }
}
