<?php
/**
 * Project:     CMF media manager client module (includes)
 * File:        Client.php
 *
 * @copyright   (c)2009 Nstein Technologies
 * @version     1.0
 */ 

/**
 * The client for the media manager REST API.
 */
class Nstein_Cmf_Mediamanager_Client
{
    /**
     * The URL of the media manager REST service.
     * @var string
     */
    private $url;
    
    /**
     * The login used to authenticate to the media manager
     * when modifying the data.
     * @var string
     */
    private $login;
    
    /**
     * The password used to authenticate to the media manager
     * when modifying the data.
     * @var string
     */
    private $password;
    
    /**
     * Public constructor.
     * @param string $url The media manager REST API URL.
     * @param string $login The login used to authenticate 
     * to the media manager when modifying the data (optional).
     * @param string $password The password used to authenticate 
     * to the media manager when modifying the data (optional).
     */
    public function __construct($url, $login = null, $password = null)
    {
        $this->url = $url;
        $this->login = $login;
        $this->password = $password;
    }
    
    /**
     * Get the media manager REST API URL.
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }
    
    /**
     * Set the media manager REST API URL.
     * @param string $value The new value.
     * @return void
     */
    public function setUrl($value)
    {
        $this->url = $value;
    }
    
    /**
     * Get the login used to authenticate 
     * to the media manager when modifying the data.
     * @return string
     */
    public function getLogin()
    {
        return $this->login;
    }
    
    /**
     * Set the login used to authenticate 
     * to the media manager when modifying the data.
     * @param string $value The new value.
     * @return void
     */
    public function setLogin($value)
    {
        $this->login = $value;
    }
    
    /**
     * Get the password used to authenticate 
     * to the media manager when modifying the data.
     * @return string
     */
    public function getPassword()
    {
        return $this->password;
    }
    
    /**
     * Set the password used to authenticate 
     * to the media manager when modifying the data.
     * @param string $value The new value.
     * @return void
     */
    public function setPassword($value)
    {
        $this->password = $value;
    }
    
    /**
     * Get the URL to get an image from the media manager.
     * @param int $photoId The id of the photo.
     * @return string The URL of the image.
     */
    public function getImageUrl($photoId)
    {
        return sprintf(
            '%s?controllerName=image&action=get&id=%d',
            $this->url,
            $photoId
        );
    }
    
    /**
     * Get the URL to get a cropped image from the media manager.
     * @param int $photoId The id of the photo.
     * @param string $format The format of the cropping.
     * @return string The URL of the cropped image.
     */
    public function getCroppedImageUrl($photoId, $format)
    {
        return sprintf(
            '%s?controllerName=image&action=get&id=%d&format=%s',
            $this->url,
            urlencode($photoId),
            urlencode($format)
        );
    }
    
    /**
     * Get the URL to get a resized image from the media manager.
     * @param int $photoId The id of the photo.
     * @param int $width The maximum width of the image.
     * @param int $height The maximum height of the image.
     * @return string The URL of the resized image.
     */
    public function getResizedImageUrl($photoId, $width, $height)
    {
        return sprintf(
            '%s?controllerName=image&action=get&id=%d&width=%s&height=%s',
            $this->url,
            urlencode($photoId),
            urlencode($width),
            urlencode($height)
        );
    }
    
    /**
     * Check if a given image exists in the media manager, through
     * a HEAD call.
     * @param int $photoId The image ID.
     * @return boolean True if the image exists, false otherwise.
     */
    public function imageExists($photoId)
    {
        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $this->getImageUrl($photoId));
        curl_setopt($curl, CURLOPT_NOBODY, true);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        
        curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        curl_close($curl);
        
        return 200 == $status;
    }
    
    /**
     * Upload an image in the media manager.
     * @param int $photoId The id of the photo.
     * @param array $formats The cropping informations
     * for the photo.
     * @param string $sourcePath The absolute path of the
     * image file to put in the media manager.
     * @return void
     * @exception Exception If an error occured.
     */
    public function uploadImage($photoId, array $formats, $sourcePath)
    {
        $arguments = array(
            'controllerName' => 'image',
            'action' => 'add',
            'id' => $photoId,
            'formats' => json_encode($formats),
            'photo' => '@' . $sourcePath,
            'login' => $this->login,
            'password' => $this->password
        );

        $curl = curl_init();
        
        curl_setopt($curl, CURLOPT_URL, $this->url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $arguments);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        curl_close($curl);
        
        if (200 != $status)
        {
            throw new Exception(
                sprintf(
                    'Unable to upload image file "%s" for photo "%d" in media manager "%s": (%s) %s',
                    $sourcePath,
                    $photoId,
                    $this->url,
                    $status,
                    $response
                )
            );
        }
    }
    
    /**
     * Send an image URL to the media manager. The media
     * manager must have read access to the provided
     * image URL.
     * @param int $photoId The id of the photo.
     * @param array $formats The cropping informations
     * for the photo.
     * @param string $url The URL of the
     * image file to put in the media manager.
     * @return void
     * @exception Exception If an error occured.
     */
    public function addImage($photoId, array $formats, $url)
    {
        $arguments = array(
            'controllerName' => 'image',
            'action' => 'add',
            'id' => $photoId,
            'formats' => json_encode($formats),
            'url' => $url,
            'login' => $this->login,
            'password' => $this->password);

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $this->url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $arguments);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        curl_close($curl);

        if (200 != $status)
        {
            throw new Exception(
                sprintf(
                    'Unable to download image file "%s" for photo "%d" in media manager "%s": (%s) %s',
                    $url,
                    $photoId,
                    $this->url,
                    $status,
                    $response
                )
            );
        }
    }
    
    /**
     * Update the cropping formats for a given photo.
     * @param int $photoId The id of the photo.
     * @param array $formats The cropping informations
     * for the photo.
     * @return void
     * @exception Exception If the update operation fails.
     */
    public function updateFormats($photoId, array $formats)
    {
        $arguments = array(
            'controllerName' => 'image',
            'action' => 'update',
            'id' => $photoId,
            'formats' => json_encode($formats),
            'login' => $this->login,
            'password' => $this->password);

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $this->url);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $arguments);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        curl_close($curl);

        if (200 != $status)
        {
            throw new Exception(
                sprintf(
                    'Unable to update formats for photo "%d" in media manager "%s": (%s) %s',
                    $photoId,
                    $this->url,
                    $status,
                    $response
                )
            );
        }
    }
    
    /**
     * Delete an image from the media manager.
     * @param int $photoId The id of the photo.
     * @return void
     * @exception Exception If an error occured.
     */
    public function deleteImage($photoId)
    {
        $arguments = array(
            'controllerName' => 'image',
            'action' => 'remove',
            'id' => $photoId,
            'login' => $this->login,
            'password' => $this->password);
            
        $curl = curl_init();
        
        curl_setopt($curl, CURLOPT_URL, $this->url);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $arguments);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        
        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        curl_close($curl);
        
        if (200 != $status)
        {
            throw new Exception(
                sprintf(
                    'Unable to delete image %d in media manager: %s',
                    $photoId,
                    $response
                )
            );
        }
    }
}
