<?php
/**
 * Nstein application framework
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

/**
 * Allows a Web client to compute the "Authorization" header necessary to respond to an HTTP Digest
 * authentication challenge, ie. a "401 Unauthorized" response specifying
 * a "WWW-Authenticate" header.
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 */
final class Nstein_Http_Digest
{
    /**
     * The name of the user to authenticate.
     *
     * @var string
     */
    private $username;

    /**
     * The user's password.
     *
     * @var string
     */
    private $password;

    /**
     * The last "WWW-Authenticate" response header parameters.
     *
     * @var string
     */
    private $wwwAuthParams;

    /*
     * The digest header "nonce" value.
     *
     * @var string
     */
    private $nonce;

    /**
     * The digest header "nc" (nonce count) value.
     *
     * @var int
     */
    private $nc;

    /**
     * The digest header A1 string.
     *
     * @var string
     */
    private $a1;

    /**
     * Constructor
     *
     * @param string $username HTTP Digest authentication username (default is null)
     * @param string $password HTTP Digest authentication password (default is null)
     *
     * @return void
     */
    public function __construct($username = null, $password = null)
    {
        $this->setCredentials($username, $password);
    }

    /**
     * Sets the user credentials for subsequent calls to the computeAuthorizationHeader method.
     *
     * @param string $username HTTP Digest authentication username
     * @param string $password HTTP Digest authentication password
     *
     * @return void
     */
    public function setCredentials($username, $password)
    {
        $this->username = $username;
        $this->password = $password;
        $this->wwwAuthParams = null;
        $this->nonce    = null;
        $this->nc       = null;
        $this->a1       = null;
    }

    /**
     * @return string The current username, or null if none has been set
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * @return string The current password, or null if none has been set
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Computes the "Authorization" header value for the current username and password, and given
     * HTTP method, request URI, and "WWW-Authenticate" response header.
     *
     * If the $wwwAuthHdr parameter is null, uses the last "WWW-Authenticate" response header.
     *
     * @param string             $method   The HTTP method
     * @param Zend_Uri_Http      $uri      The request URI
     * @param string        $wwwAuthHdr The "WWW-Authenticate" response header (default is null)
     *
     * @return string The "Authorization" header value
     *
     * @throws Exceptuon(<HTTP-status-meesage>, <HTTP-status-code>) on error
     */
    public function computeAuthorizationHeader($method, Zend_Uri_Http $uri, $wwwAuthHdr = null)
    {
        // Do we have any user credentials?
        if ($this->username === null || $this->password === null)
        {
            $code = Nstein_REST_Response::S_UNAUTHORIZED;
            $message = Nstein_REST_Response::statusCodeToMessage($code);
            throw new Nstein_REST_Exception(sprintf('MISSING_USER_CREDENTIALS : %s', $message), $code);
        }

        // If we have a new "WWW-Authenticate" response header ...
        if ($wwwAuthHdr !== null)
        {
            // Clear the previous "WWW-Authenticate" response header parameters
            $this->wwwAuthParams = null;

            // Check that Digest authentication has been requested
            if (!preg_match('/^Digest\s+/i', $wwwAuthHdr))
            {
                $code = Nstein_REST_Response::S_NOT_IMPLEMENTED;
                $message = Nstein_REST_Response::statusCodeToMessage($code);
                throw new Nstein_REST_Exception(printf('UNSUPPORTED_AUTH_PROTOCOL : %s', $message), $code);
            }

            // Split the "WWW-Authenticate" header value into its constituent parameters
            $this->wwwAuthParams = $this->splitHeaderIntoParams($wwwAuthHdr);
        }

        // Do we have any "WWW-Authenticate" response header parameters?
        if ($this->wwwAuthParams === null)
            return null;

        // Compute the "Authorization" header parameters
        $authParams = $this->computeHeaderParams($method, $uri, $this->wwwAuthParams);

        // Join the "Authorization" parameters into a header
        return $this->joinHeaderParams($authParams);
    }

    /**
     * Computes the "Authorization" header parameters for a given HTTP method, request URI, and set
     * of "WWW-Authenticate" header parameters.
     *
     * @param string        $method        The HTTP method
     * @param Zend_Uri_Http $uri           The request URI
     * @param array         $wwwAuthParams The "WWW-Authenticate" header parameters
     *
     * @return array The "Authorization" header parameters
     */
    private function computeHeaderParams($method, $uri, $wwwAuthParams)
    {
        // Check for required "WWW-Authenticate" header parameters
        if (empty($wwwAuthParams['realm']) || empty($wwwAuthParams['nonce']))
        {
            $code = Nstein_REST_Response::S_BAD_REQUEST;
            $message = Nstein_REST_Response::statusCodeToMessage($code);
            throw new Exception(sprintf('UNEXPECTED_RESPONSE : %s', $message), $code);
        }
        $realm = $wwwAuthParams['realm'];
        $nonce = $wwwAuthParams['nonce'];
        $upath = $uri->getPath();

        // Set required "Authorization" header parameters
        $authParams = array();
        $authParams['username']	= '"'.$this->username.'"';
        $authParams['realm']	= '"'.$realm.'"';
        $authParams['nonce']	= '"'.$nonce.'"';
        $authParams['uri']	= '"'.$upath.'"';

        // Include the optional "opaque" value if present
        if (isset($wwwAuthParams['opaque']))
            $authParams['opaque'] = '"'.$wwwAuthParams['opaque'].'"';

        // Are we retrying the "nonce" value?
        if ($nonce == $this->nonce)
        {
            // Yes, increment nonce count
            $this->nc++;
        }
        else
        {
            // No, new value
            $this->nonce = $nonce;
            $this->nc = 1;
        }

        // Convert decimal "nc" value to hexadecimal
        $nc = str_pad(dechex($this->nc), 8, '0', STR_PAD_LEFT);

        // Generate a random client nonce value
        $cnonce = md5(microtime(true));

        // Check for a "qop" value
        if (isset($wwwAuthParams['qop']))
        {
            $qops = preg_split('/\s+;\s+/', $wwwAuthParams['qop']);
            if (in_array('auth', $qops))
                $qop = 'auth';
            elseif (in_array('auth-int', $qops))
                $qop = 'auth-int';
        }

        // If a "qop" value is specified, include the corresponding parameters
        if (isset($qop))
        {
            $authParams['qop']    = $qop;
            $authParams['nc']     = $nc;
            $authParams['cnonce'] = '"'.$cnonce.'"';
        }

        // Check for an "algorithm" value
        if (isset($wwwAuthParams['algorithm']))
        {
            $algorithm = $wwwAuthParams['algorithm'];
            if ($algorithm == 'MD5' || $algorithm == 'MD5-sess')
                $authParams['algorithm'] = $algorithm;
        }

        // Generate the A1 string based on the "algorithm" value
        if (!isset($algorithm) || $algorithm == 'MD5')
        {
            $a1 = $this->username.':'.$realm.':'.$this->password;
            $this->a1 = null;
        }
        elseif ($algorithm == 'MD5-sess')
        {
            if (isset($this->a1))
            {
                $a1 = $this->a1;
            }
            else
            {
                $a1 = md5($this->username. ':'.$realm.':'.$this->password).':'.$nonce.':'.$cnonce;
                $this->a1 = $a1;
            }
        }
        $ha1 = md5($a1);

        // Generate the A2 string based on the "qop" value
        if (!isset($qop) || $qop == 'auth')
            $a2 = $method.':'.$upath;
        elseif ($qop == 'auth-int')
            $a2 = $method.':'.$upath.':'.md5('TODO-entity-body');
        $ha2 = md5($a2);

        // Generate the "response" value based on the "qop" value
        if (!isset($qop))
            $response = $ha1.':'.$nonce.':'.$ha2;
        elseif ($qop == 'auth' || $qop == 'auth-int')
            $response = $ha1.':'.$nonce.':'.$nc.':'.$cnonce.':'.$qop.':'.$ha2;
        $authParams['response'] = '"'.md5($response).'"';

        return $authParams;
    }

    /**
     * Splits a given "WWW-Authenticate" header value into its constituent parameters.
     *
     * @param string $header The "WWW-Authenticate" header value
     *
     * @return array The corresponding parameters
     */
    private function splitHeaderIntoParams($header)
    {
        // Remove the "Digest" prefix
        $header = preg_replace('/^Digest\s+(.*)$/i', '$1', $header);

        // Match all individual parts of the header
        preg_match_all('/([^=]+)=("[^"]+"|[^,]+)(?:,\s*|$)/', $header, $matches);

        // Add each part to the parameters array
        $params = array();
        if ($matches[1])
        {
            foreach ($matches[1] as $key => $name)
                $params[$name] = trim($matches[2][$key], '"');
        }

        return $params;
    }

    /**
     * Joins given "Authorization" header parameters into the corresponding header.
     *
     * @param array $params The "Authorization" header parameters
     *
     * @return string The corresponding header
     */
    private function joinHeaderParams($params)
    {
        // Join parameters into header value
        $header  = 'Digest ';
        if ($params)
        {
            foreach ($params as $name => $value)
                $header .= $name.'='.$value.', ';

            // Trim trailing comma+space
            $header = rtrim($header, ', ');
        }

        return $header;
    }
}
