<?php
/**
 * Nstein application framework
 *
 * @category   Nstein
 * @package    Nstein_Path
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

/**
 * This class implements a file distribution algorithm
 * that limits to {base} power {blockSize} the number
 * of files that a directory can contain and to {base}
 * power {blockSize} the number of subdirectories a
 * directory can contain. If the files are created
 * using sequential IDs, a new directory (or sub directory)
 * will be created only when the previous directory
 * is full.
 */
class Nstein_Path_DefaultBuilder implements Nstein_Path_Builder
{
	/**
     * Numeric base for the path generation algorithm.
     * This algorithm limits to base power blockSize files
     * and base power blockSize subdirectories in a given
     * directory.
     * @var int
     */
    private $base;

    /**
     * Block size for the path generation algorithm.
     * This algorithm limits to base power blockSize files
     * and base power blockSize subdirectories in a given
     * directory.
     * @var int
     */
    private $blockSize;
    
    /**
     * Public constructor.
     * @param int $base The numeric base for distribution algorithm.
     * @param int $blockSize The block size for distribution algorithm.
     */
    public function __construct($base, $blockSize)
    {
        $this->base = $base;
        $this->blockSize = $blockSize;
    }
    
    /**
     * Generate a path from a numeric ID.
     * @param int $id The ID for which a path must be generated.
     * @return string A relative path for the provided ID.
     * @exception InvalidArgumentException If $id is not numeric.
     */
    public function generatePath($id)
    {
    	if (!is_numeric($id))
    	{
    		throw new InvalidArgumentException('The argument $id must be numeric.');
    	}
    	
        $value = (int)$id;
        
        // Convert the base of the value (if needed).
        if (10 !== $this->base)
        {
            $value = base_convert($value, 10, $this->base);
        }

        // Get the value as a string.
        $value = (string)$value;

        // Pad the string with zeros to the right
        $length = strlen($value);
        $blockCount = ceil($length / $this->blockSize);
        $value = str_pad($value, $blockCount * $this->blockSize, '0', STR_PAD_LEFT);

        // We split the string in blocks
        $fragments = str_split($value, $this->blockSize);

        // We start from the right, omitting the leftmost.
        $path = '';
        for ($i = 0, $n = count($fragments) - 1; $i < $n; ++$i)
        {
            $path = $path . $fragments[$i] . DIRECTORY_SEPARATOR;
        }
        
        return $path;
    }
}