<?php
/**
 * Nstein REST application framework
 *
 * @category   Nstein
 * @package    Nstein_REST
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

/**
 * The Nstein_REST_Client exposes basic GET, PUT, POST, and DELETE methods to interop with a REST
 * controller over HTTP
 *
 * @category   Nstein
 * @package    Nstein_REST
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 */
final class Nstein_REST_Client
{
    /**
     * Underlying HTTP client.
     *
     * @var Nstein_Http_Client
     */
    private $httpClient;

    /**
     * Base URI for the target REST API:
     *
     *     http[s]://[<username>[:<password>]@]<host>[:<port>]<path>[<query>][<fragment>]
     *
     * - If the <username> and <password> components are present, they specify the user credentials
     *   (may be overridden by an explicit $credentials parameter).
     *
     * - The <path> component (with or without a trailing slash - one will be added if not present)
     *   specifies the base URI path up to, but not including, the controller (resource
     *   identification).
     *
     * - If a <query> component is present, the corresponding query parameters are included with
     *   every request.
     *
     * - Any <fragment> component is ignored.
     *
     * As an example, the base URI for the REST API of a WCM installation accessible with:
     *
     *     http://rdwcm1/wcm4/
     *
     * can be specified as:
     *
     *     http://rdwcm1/wcm4/rest/?myparam=foo
     *
     * in which case every request will contain the 'myparam' query parameter, for example:
     *
     *     http://rdwcm1/wcm4/rest/content/article?offset=0&limit=10&myparam=foo
     *
     * @var Zend_Uri_Http
     */
    private $baseUri;

    /**
     * HTTP connection timeout in seconds.
     *
     * @var int
     */
    private $timeout = 60;

    /**
     * Current session ID (may be null - see the "login" method).
     *
     * @var string
     */
    private $sessionId;

    /**
     * Constructor
     *
     * The $baseUri parameter is expected to specify a URI string or a Zend_Uri_Http instance (see
     * $this->baseUri for details).
     *
     * If user credentials are given (explicitly or in the base URI's username/password components),
     * they are set as with the "setCredentials" method.
     *
     * @param mixed  $baseUri  Base URI
     * @param string $username Username (default is null)
     * @param string $password User password (default is null)
     *
     * @return void
     */
    public function __construct($baseUri, $username = null, $password = null)
    {
        // Instantiate the underlying HTTP client
        $this->httpClient = new Nstein_Http_Client;

        /*
         * Use our own client adapter since Zend's client adapter (Zend_Http_Client_Adapter_Socket)
         * is too strict with "chunked" transfer-encoding responses that are empty (it throws an
         * exception).
         */
        $this->httpClient->setConfig(array('adapter' => 'Nstein_Http_Client_Adapter_Socket'));

        // Set the base URI for the target REST API
        $this->setBaseURI($baseUri);

        // If no credentials were explicitly given, use the username and password URI components if
        // present (in any case, clear the username and password URI components)
        if ($username === null && $password === null)
        {
            $username = $this->baseUri->getUsername();
            $password = $this->baseUri->getPassword();
        }
        $this->baseUri->setUsername(null);
        $this->baseUri->setPassword(null);

        // Set the user credentials
        $this->setCredentials($username, $password);
    }

    /**
     * Sets the base URI for the target REST API, first logging the user out with the {@link
     * logout()} method.
     *
     * The $baseUri parameter is expected to specify a URI string or a Zend_Uri_Http instance (see
     * $this->baseUri for details).
     *
     * @param mixed $baseUri Base URI
     *
     * @return void
     */
    public function setBaseURI($baseUri)
    {
        // Log the user out since authentication may depend on the base URI
        $this->logout();

        // Combine the given URI with an empty one, yielding a Zend_Uri_Http instantiate without any
        // fragment component
        $this->baseUri = $this->httpClient->combineURIs(Zend_Uri_Http::factory('http'),
                                                        array($baseUri, false, 'fragment'));

        // Append a slash to the URI path if necessary
        $basePath = $this->baseUri->getPath();
        if ($basePath[strlen($basePath) - 1] != '/')
        {
            $this->baseUri->setPath($basePath . '/');
        }
    }

    /**
     * @return Zend_Uri_Http The current base URI
     */
    public function getBaseURI()
    {
        return $this->baseUri;
    }

    /**
     * @param int The new HTTP connection timeout
     */
    public function setTimeout($timeout)
    {
        $this->timeout = $timeout;
        $this->httpClient->setConfig(array('timeout' => $timeout));
    }

    /**
     * @return int The current HTTP connection timeout
     */
    public function getTimeout()
    {
        return $this->timeout;
    }

    /**
     * Sets the user credentials for subsequent requests, first logging the user out with the {@link
     * logout()} method.
     *
     * @param string $username Username (may be null)
     * @param string $password User password (may be null)
     *
     * @return void
     */
    public function setCredentials($username, $password)
    {
        // Log the user out since authentication depends on the user credentials
        $this->logout();

        // Set the new user credentials
        $this->httpClient->setCredentials($username, $password);
    }

    /**
     * @return string The current username, or null if none has been set
     */
    public function getUsername()
    {
        return $this->httpClient->getUsername();
    }

    /**
     * @return string The current password, or null if none has been set
     */
    public function getPassword()
    {
        return $this->httpClient->getPassword();
    }

    /**
     * Explicitly logs the user into the system by POSTing the current username and password to the
     * target REST API's "session" resource:
     *
     *     POST /session?username=foo&password=bar HTTP/1.1
     *
     * If the POST request is successful, saves the session ID specified by the
     * "X-Nstein-Session-ID" response header in this client's "sessionId" property.
     *
     * @param array  $data    Data to send (default is empty)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response The HTTP response to the POST request
     */
    public function login(array $data = array(), array $headers = array())
    {
    	$data['username'] = $this->getUsername();
    	$data['password'] = $this->getPassword();

        // Post the user credentials to the REST API's "session" resource
        $response = $this->post('/session', $data, $headers);

        // If all is well, save the session ID for subsequent requests
        if ($response && $response->isSuccessful())
        {
            $this->sessionId = $response->headers[Nstein_REST_Response::H_X_NSTEIN_SESSION_ID];
        }

        return $response;
    }

    /**
     * Explicitly logs the user out of the system by resetting any previous session ID or HTTP
     * digest, depending on the authentication method last used.
     *
     * @return void
     */
    public function logout()
    {
        $this->sessionId = null;
        $this->httpClient->resetHttpDigest();
    }

    /**
     * Sets a new session ID, first logging the user out with the {@link logout()} method.
     *
     * @param string The new session ID
     */
    public function setSessionId($sessionId)
    {
        $this->logout();
        $this->sessionId = $sessionId;
    }

    /**
     * @return string The current session ID, or null if none has been set
     */
    public function getSessionId()
    {
        return $this->sessionId;
    }

    /**
     * Sets a new HTTP digest, first logging the user out with the {@link logout()} method.
     *
     * @param Nstein_Http_Digest $httpDigest The new HTTP digest
     */
    public function setHttpDigest(Nstein_Http_Digest $httpDigest)
    {
        $this->logout();
        $this->httpClient->setHttpDigest($httpDigest);
    }

    /**
     * @return Nstein_Http_Digest The current HTTP authentication wrapper, or null if HTTP digest
     *                            authentication has not yet been attempted successfully.
     */
    public function getHttpDigest()
    {
        return $this->httpClient->getHttpDigest();
    }

    /**
     * Prepares a request with a given HTTP method, a URI path, a query string or array of
     * parameters, some data, and some headers. Other than the HTTP method and URI path, all
     * parameters are optional.
     *
     * If the current session ID is non-null (see the {@link login()} method), adds a corresponding
     * "X-Nstein-Session-ID" request header.
     *
     * @param string $method  HTTP request method
     * @param string $uri     URI path to append to the base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param mixed  $data    Data to send (object, array, string, resource, or null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return void
     */
    private function prepareRequest($method, $uri, $query = null,
                                    $data = null, array $headers = array())
    {
        // Combine the given URI (ignoring any username, password, host, port, and fragment
        // components) with the base URI, yielding a Zend_Uri_Http instance
        $uri = array($uri, false, 'username', 'password', 'host', 'port', 'fragment');
        $uri = $this->httpClient->combineURIs($this->baseUri, $uri);

        // Set the URI for the request
        $this->httpClient->setUri($uri);

        // Set the query parameters, if any
        if ($query !== null)
        {
            if (is_string($query))
            {
                // Parse query string into an array of parameters
                parse_str($query, $tmp);
                $query = $tmp;
            }
            $this->httpClient->setParameterGet($query);
        }

        // Set the request headers, if any
        if ($headers)
        {
            $this->httpClient->setHeaders($headers);
        }

        // Set the "X-Nstein-Session-ID" header if possible
        if ($this->sessionId !== null)
        {
            $this->httpClient->setHeaders(Nstein_REST_Response::H_X_NSTEIN_SESSION_ID, $this->sessionId);
        }

        // If we have data ...
        if ($data)
        {
            if (is_string($data) || is_resource($data))
            {
                // String or resource => raw data
                $this->httpClient->setRawData($data);
            }
            else if (is_array($data) || is_object($data))
            {
                // Array or object => form data
                $this->httpClient->setParameterPost((array) $data);
            }

            // Explicitly set content type (encoding) if given in headers
            $contentType = $this->httpClient->getHeader(Nstein_REST_Response::H_CONTENT_TYPE);
            if ($contentType)
            {
                $this->httpClient->setEncType($contentType);
            }
        }
    }

    /**
     * Performs a request using a given HTTP method.
     *
     * @param string $method  HTTP method
     * @param string $path    Path to append to the base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param mixed  $data    Data to send (object, array, string, resource, or null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response
     */
    private function request($method, $path, $query = null, $data = null, array $headers = array())
    {
        // Prepare the request
        $this->prepareRequest($method, $path, $query, $data, $headers);

        // Perform the request
        try
        {
            $response = new Nstein_REST_Response($this->httpClient->request($method));
        }
        catch (Exception $ex)
        {
            $code = ($ex->getCode() > 0) ? $ex->getCode() : Nstein_REST_Response::S_INTERNAL_SERVER_ERROR;
            $response = new Nstein_REST_Response($code, $ex->getMessage());
        }

        // Clear the request
        $this->httpClient->resetParameters();

        return $response;
    }

    /**
     * Performs an HTTP "GET" request.
     *
     * @param string $path    Path to append to base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response
     */
    public function get($path, $query = null, array $headers = array())
    {
        return $this->request('GET', $path, $query, null, $headers);
    }

    /**
     * Performs an HTTP "PUT" request.
     *
     * @param string $path    Path to append to base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param mixed  $data    Data to send (object, array, string, resource, or null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response
     */
    public function put($path, $query = null, $data = null, array $headers = array())
    {
        return $this->request('PUT', $path, $query, $data, $headers);
    }

    /**
     * Adds a file to a "POST" request.
     *
     * @param string      $file        File resource to POST, or name of file to POST
     * @param string      $formName    Name of the corresponding form element
     * @param string|null $data        Data content (null to retrieve file content from disk)
     * @param string|null $contentType Data content type (default is "application/octet-stream")
     *
     * @return void
     */
    public function addFileToPost($file, $formName, $data = null, $contentType = null)
    {
        $this->httpClient->setFileUpload($file, $formName, $data, $contentType);
    }

    /**
     * Performs an HTTP "POST" request.
     *
     * @param string $path    Path to append to base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param mixed  $data    Data to send (object, array, string, resource, or null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response
     */
    public function post($path, $query = null, $data = null, array $headers = array())
    {
        return $this->request('POST', $path, $query, $data, $headers);
    }

    /**
     * Performs an HTTP "DELETE" request.
     *
     * @param string $path    Path to append to base URI, or absolute URI
     * @param mixed  $query   Query string or array of parameter name => value (default is null)
     * @param array  $headers Array of header name => value (default is empty)
     *
     * @return Nstein_REST_Response
     */
    public function delete($path, $query = null, array $headers = array())
    {
        return $this->request('DELETE', $path, $query, null, $headers);
    }
}