<?php
/**
 * Project:     WCM
 * File:        $Id: biz.forum.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * This class represents a forum.
 *
 * A forum is a way for users to add {@link contribution contributions}.
 *
 * There are two kinds of forums : hierarchical (threaded) or flat. This behavior
 * can be extended.
 *
 * Forums can be moderated "a priori" or "a posteriori". This behavior can
 * be extended.
 *
 * Forums can also have restricted topics which, if set to true will
 * allow users to start new subjects.
 *
 * A forum may be enriched with TME
 */
class forum extends bizobject
{
    /**
    * Site ID
    *
    * @var int
    */
    public $siteId;

    /**
    * Channel ID
    *
    * @var int
    */
    public $channelId;

    /**
    * Forum title
    *
    * @var string
    */
    public $title;

    /**
    * Forum description
    *
    * @var string
    */
    public $description;

    /**
    * Forum kind - see getKindList()
    *
    * @var string
    * @see forum::getKindList
    */
    public $kind;

    /**
    * Whether users can initiate new subjects (root contributions)
    *
    * @var bool
    */
    public $restrictedTopics;

    //
    // Web 2.0 properties (moderation / contribution / rating / hits)
    //

    /**
     * Article moderation kind - see getModerationKindList()
     *
     * @var string
     * @see getModerationKindList()
     */
    public $moderationKind;

    /**
     * Contribution mode
     *
     * @var string
     * @see getContributionStateList()
     */
    public $contributionState;

    /**
     * Set all initial values of an object
     *
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->kind              = 'hierarchical';
        $this->contributionState = 'open';
        $this->moderationKind    = 'a_priori';
        $this->restrictedTopics  = false;

        $this->siteId = $this->channelId = null;
    }

    /**
    * Gets the posts for the forum.
    * @param int $from Offset
    * @param int $to Limit
    * @return array Array of posts
    *
    */
    function getPosts($from=0, $to=-1)
    {
        $result = array();
        $enum = new wcmEnumeration('bizrelation', 'sourceClass="forum" AND sourceId=' . $this->id . ' AND destinationClass="item"', null, $from, $to);

        foreach ($enum as $posts)
        {
            $post = wcmObjectFactory::create('item', $posts->destinationId);
            $tempArray = array($post->getAssocArray(false));
            $result = array_merge($result, $tempArray);
        }

        return $result;
    }

    /**
    * Returns the number of posts for the forum.
    *
    * @return int Post count
    */
    public function getPostCount()
    {
        $enum = new wcmEnumeration('bizrelation', 'sourceClass="forum" AND sourceId=' . $this->id . ' AND destinationClass="item"');
        return count($enum);
    }

    /**
     * Return the list of available forum kinds
     *
     * @return array Kind list
     */
    public function getKindList()
    {
        return array(
                  'flat'         => _BIZ_FLAT,
                  'hierarchical' => _BIZ_HIERARCHICAL,
                  );
    }

    /**
     * Exposes 'postCount' in the getAssocArray()
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return int Total number of posts
     */
    public function getAssoc_postCount($toXML = false)
    {
        return $this->getPostCount();
    }

    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param  array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    function ofClause($of)
    {
        if (null === $of || !is_array($of))
        {
            return;
        }

        $sql = null;

        foreach ($of as $key => $value)
        {
            switch ($key)
            {
                case 'site':
                    if (null !== $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'siteId=' . $value;
                    break;

                case 'channel':
                    if (null !== $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'channelId=' . $value;
                    break;
            }
        }
        return $sql;
    }

    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid
     *
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        if ('' === trim($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_IS_MANDATORY;
            return false;
        }

        if (255 < strlen($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_TOO_LONG;
            return false;
        }

        return true;
    }
}
