<?php
/**
 * Project:     WCM
 * File:        $Id: biz.publicationHelper.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * A helper class used when publishing objects.
 */
class publicationHelper
{
    /**
     * The object being published.
     * @var bizobject
     */
    private $object;
    
    /**
     * The site on which the object is published.
     * @var site
     */
    private $site;
    
    /**
     * The data access object for the Apache
     * SQL alias module.
     * @var sqlAliasDataAccess
     */
    private $sqlAliasDAO = null;
    
    /**
     * The URL generator used to generator the
     * published object's permalink.
     * @var wcmObjectUrlGenerator
     */
    private $urlGenerator = null;
    
    /**
     * Initialize the publication helper.
     * @param bizobject $object The object being 
     * published.
     * @param site $site The site on which the 
     * object is published.
     * @return void
     * @exception InvalidArgumentException If no site was provided,
     * and the provided bizobject has no site.
     */
    public function __construct(bizobject $object, site $site = null)
    {
        $this->object = $object;
        $this->site = null !== $site ? $site : $object->getSite();
    }
    
    /**
     * Get the bizobject.
     * @return bizobject
     */
    public function getObject()
    {
        return $this->object;
    }
    
    /**
     * Get the site.
     * @return site
     */
    public function getSite()
    {
        return $this->site;
    }
    
    /**
     * Publish the object's URL aliases to Apache 
     * SQL alias module.
     * @return void
     */
    public function publishSqlAliases()
    {
        $aliases = array();
        foreach (objectUrlAlias::getAll($this->object) as $alias)
        {
            $aliases[] = $alias->alias;
        }
        
        $permalink = $this->getUrlGenerator()->generateUrl(
            $this->object, 
            $this->site
        );
        $this->getSqlAliasDAO()->setAliases(
            $this->object->getClass(),
            $this->object->id,
            $permalink,
            $aliases
        );
    }
    
    /**
     * Remove all object's URL aliases from SQL alias
     * database.
     * @return void
     */
    public function unpublishSqlAliases()
    {
        $this->getSqlAliasDAO()->deleteAliases(
            $this->object->getClass(),
            $this->object->id
        );
    }
    
    /**
     * Initialize the publicationDate property on the object, if it exists,
     * if it is null or equal to '0000-00-00 00:00:00'.
     * @return void
     */
    public function initializePublicationDate()
    {
        if (!property_exists($this->object, 'publicationDate'))
        {
            // No publicationDate property.
            return;
        }
        
        if (null == $this->object->publicationDate || '0000-00-00 00:00:00' == $this->object->publicationDate)
        {
            // Empty publicationDate; set it to now.
            $this->object->publicationDate = date('Y-m-d H:i:s');
        }
    }
    
    /**
     * Mine the object using TME and update the object's semantic data.
     * Note that the object is not saved by this method.
     * @param boolean $nlikeThisIndex Indicate if the object should
     * also be indexed in NLikeThis (default: false).
     * @param boolean $nlikeThisCompare Indicate if NLikeThis Compare
     * should be called (default: false).
     * @return void
     */
    public function mineObject($nlikeThisIndex = false, $nlikeThisCompare = false)
    {
        $config = wcmConfig::getInstance();
        if ($config['tme.enabled'] == '1')
        {
            $tme = new wcmSemanticServer($config['tme.server'], $config['tme.port']);

            // TODO change mineObject() so it throws an exception on failure
            $semanticData = $tme->mineObject(
                $this->object,
                array(
                    'NConceptExtractor' => true,
                    'NFinder' => true,
                    'NCategorizer' => true,
                    'NSummarizer' => true,
                    'NSentiment' => true,
                    'NLikeThis_Index' => $nlikeThisIndex,
                    'NLikeThis_Compare' => $nlikeThisCompare
                )
            );
            if (null !== $semanticData)
            {
                $this->object->semanticData->merge($semanticData);
            }
        }
    }
    
    /**
     * Index the object in TME NLikeThis.
     * @return void
     */
    public function nlikeThisIndexObject()
    {
        $config = wcmConfig::getInstance();
        if ($config['tme.enabled'] == '1')
        {
            $tme = new wcmSemanticServer($config['tme.server'], $config['tme.port']);
            $tme->indexObject($this->object);
        }
    }
    
    /**
     * Perform an NLikeThis Compare on the object, and update
     * the object's semantic data.
     * Note that the object is not saved by this method.
     * @return void
     */
    public function nlikeThisCompareObject()
    {
        $config = wcmConfig::getInstance();
        if ($config['tme.enabled'] == '1')
        {
            $tme = new wcmSemanticServer($config['tme.server'], $config['tme.port']);

            // TODO change mineObject() so it throws an exception on failure
            $semanticData = $tme->mineObject(
                $this->object,
                array(
                    'NLikeThis_Compare' => true
                )
            );
            if (null !== $semanticData)
            {
                $this->object->semanticData->merge($semanticData);
            }
        }
    }
    
    /**
     * Deindex the object from NLikeThis.
     * @return void
     */
    public function nlikeThisDeindexObject()
    {
        $config = wcmConfig::getInstance();
        if ($config['tme.enabled'] == '1')
        {
            $tme = new wcmSemanticServer($config['tme.server'], $config['tme.port']);
            $tme->deindexObject($this->object);
        }
    }
    
    /**
     * Remove the generated blocks for the object.
     * @return void
     */
    public function removeGeneratedBlocks()
    {
        // Create the generator.
        $generator = new wcmTemplateGenerator();
        // $generator->assign('config', wcmConfig::getAssocInstance());

        $extraContext = array('id' => $this->object->id);

        foreach ($this->object->getAutoGenerations() as $generationContent)
        {
            $generator->executeGenerationContent($generationContent, $extraContext, false, false, true);
        }
    }
    
    /**
     * Create a version of the object.
     * @return boolean True if the archive operation
     * was successful, false otherwise.
     */
    public function createVersion()
    {
        return $this->object->archive(_BIZ_VERSION_CREATED_BEFORE_PUBLICATION);
    }

    /**
     * Sends an object to 3S for indexation
     *
     *@return Nstein_REST_Response 
     */
    public function  indexOn3S()
    {
        if ('1' == $this->site->getConfig('SSS.enabled'))
        {    
            $server= new biz3SServer($this->site);
            return $server->indexObject($this->object);
        }
        return true;
    }    

    /**
     * removes an object to 3S for indexation
     *
     *@return Nstein_REST_Response 
     */
    public function  deindexFrom3S()
    {
        if ('1' == $this->site->getConfig('SSS.enabled'))
        {  
            $server= new biz3SServer($this->site);
            return $server->deindexObject($this->object);
        }
        return true;
    }    



    /**
     * Get the data access object for SQL alias module.
     * @return sqlAliasDataAccess
     */
    private function getSqlAliasDAO()
    {
        if (null === $this->sqlAliasDAO)
        {
            $this->sqlAliasDAO = new sqlAliasDataAccess(
                    new wcmDatabase(
                        $this->site->getConfig('sql-alias.connectionString')
                        ),
                    $this->site->getConfig('sql-alias.tableName')
                    );
        }
        return $this->sqlAliasDAO;
    }

    /**
     * Get the instance of wcmObjectUrlGenerator.
     * @return wcmObjectUrlGenerator
     */
    private function getUrlGenerator()
    {
        if (null === $this->urlGenerator)
        {
            $factory = new wcmObjectUrlGeneratorFactory(
                    wcmConfig::getInstance()
                    );
            $this->urlGenerator = $factory->getDefaultGenerator();
        }
        return $this->urlGenerator;
    }
}
