<?php
/**
 * Project:     WCM
 * File:        $Id: biz.objectUrlAlias.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * Defines a URL alias of a bizobject.
 */
class objectUrlAlias extends wcmSecureObject
{
    /**
     * The database table name.
     * @var string
     */
    const TABLENAME = '#__object_url_alias';

    /**
     * The name of the business data connector.
     * @var string
     */
    const CONNECTOR = 'biz';

    /**
     * The ID of the site for which the alias is
     * defined.
     * @var int
     */
    public $siteId;

    /**
     * The class of the object to which this alias
     * belongs.
     * @var string
     */
    public $objectClass;

    /**
     * The ID of the object to which this alias
     * belongs.
     * @var int
     */
    public $objectId;

    /**
     * The URL alias.
     * @var string
     */
    public $alias;

    /**
     * Public constructor.
     * @param int $id The ID of the object to load (optional).
     */
    public function __construct($id = null)
    {
        $this->isLockOptimistic = true;
        parent::__construct($id);
    }

    /**
     * Get the database used to store the objects.
     * @return wcmDatabase
     */
    public function getDatabase()
    {
        if (!$this->database)
        {
            $this->database = self::getClassDatabase();
            $this->tableName = self::TABLENAME;
        }

        return $this->database;
    }

    /**
     * Load the object to which the alias
     * belongs.
     * @return bizobject The bizobject, or null if
     * not found.
     */
    public function loadObject()
    {
        $object = new $this->objectClass();
        if (!$object->refresh($this->objectId))
        {
            return null;
        }
        return $object;
    }

    /**
     * Set the object to which the alias belongs.
     * @param bizobject $object The object.
     * @return void
     */
    public function setObject(bizobject $object)
    {
        $this->objectClass = $object->getClass();
        $this->objectId = $object->id;
    }

    /**
     * Get the wcmDatabase instance for this class.
     * @return wcmDatabase
     */
    private static function getClassDatabase()
    {
        return wcmProject::getInstance()->datalayer->getConnectorByReference(self::CONNECTOR)->getBusinessDatabase();
    }

    /**
     * Check if a given alias exists for a given site.
     * @param string $alias The URL alias.
     * @param site $site The site.
     * @return boolean True if the alias exists for the
     * provided site, false otherwise.
     */
    public static function exists($alias, site $site)
    {
        $where = 'siteId=? and alias=?';
        $args = array($site->id, $alias);

        $enum = new objectUrlAlias();
        return $enum->getCount($where, $args) > 0;
    }

    /**
     * Check if at least one of the provided URL aliases
     * exist for a given site.
     * @param array $aliases The URL aliases.
     * @param site $site The site.
     * @return boolean True if at least one of the aliases
     * exist for the site, false otherwise.
     */
    public static function existOne(array $aliases, site $site)
    {
        if (0 == count($aliases))
        {
            return false;
        }
        
        $where = 'siteId=' . intval($site->id) . ' and alias IN (';
        $values = '';
        foreach ($aliases as $alias)
        {
            if ($values)
            {
                $values .= ',';
            }
            $values .= '\'' . str_replace('\'', '\\\'', $alias) . '\'';
        }
        $where .= $values . ')';

        $enum = new wcmEnumeration('objectUrlAlias', $where);
        return count($enum) > 0;
    }

    /**
     * Add an URL alias for an object. If a site
     * object is not specified, the bizobject's
     * site will be used as default.
     * @param string $alias The URL alias.
     * @param bizobject $object The bizobject.
     * @param site $site The site for which the alias
     * is defined (optional). If not specified, the
     * site of the bizobject is used.
     * @return objectUrlAlias The new objecrUrlAlias
     * instance.
     * @exception Exception If an error occurs during
     * save.
     */
    public static function add($alias, bizobject $object, site $site = null)
    {
        if (null === $site)
        {
            $site = $object->getSite();
        }

        $enum = new objectUrlAlias();
        $enum->alias = $alias;
        $enum->objectClass = $object->getClass();
        $enum->objectId = $object->id;
        $enum->siteId = $site->id;

        if (!$enum->save())
        {
            throw new Exception($enum->getErrorMsg());
        }

        return $enum;
    }

    /**
     * Get a single alias object.
     * @param string $alias The URL alias.
     * @param site $site The site for which the
     * alias is defined.
     * @return objectUrlAlias The alias object,
     * or null if the alias does not exist.
     */
    public static function get($alias, site $site)
    {
        $result = null;
        $where = sprintf(
            'siteId=%d and alias="%s"',
            $site->id,
            $alias
        );

        $enum = new wcmEnumeration('objectUrlAlias', $where, null, 0, 1);
        foreach ($enum as $alias)    
        {
            $result = $alias;
        }
        return $result;
    }

    /**
     * Get all alias objects defined for a given
     * bizobject, for a given site. If no site is 
     * specified, the bizobject's site will be used
     * as default.
     * @param bizobject $object The bizobject.
     * @param site $site The site (optional).
     * @return array
     */
    public static function getAll(bizobject $object, site $site = null)
    {
        if (null === $site)
        {
            $site = $object->getSite();
        }

        $result = array();
        $where = sprintf(
            'siteId=%d and objectClass="%s" and objectId=%d',
            $site->id,
            $object->getClass(),
            $object->id
        );

        $enum = new wcmEnumeration('objectUrlAlias', $where, 'alias');
        foreach ($enum as $alias)
        {
            $result[] = $alias;
        }

        return $result;
    }

    /**
     * Delete a list of objectUrlAlias from database.
     * @param array $aliases The objectUrlAlias list.
     * @return void
     */
    public static function deleteAll(array $aliases)
    {
        if (0 == count($aliases))
        {
            return false;
        }

        $sql = 'DELETE FROM ' . self::TABLENAME . ' WHERE id IN (';
        $values = '';
        $args = array();
        foreach ($aliases as $alias)
        {
            if ($values)
            {
                $values .= ',';
            }
            $values .= '?';
            $args[] = $alias->id;
        }
        $sql .= $values . ')';

        self::getClassDatabase()->executeStatement($sql, $args);
    }
}
