<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.database.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Generic connector to a database
 *
 */
class wcmDatabase
{

    /**
     * Enable/disable debugging mode
     * @var bool
     */
    public $debug = false;

    /**
     * Enable/disable persistent connection
     * @var bool
     */
    public $persistent = true;

    /**
     * Store all connections in an array, re-use them when available
     * @var array
     */
    private static $db = array();

    /**
     * Logger object
     * @var wcmLogger
     */
    private $logger = null;

    /**
     * Table prefix substituted to any occurrence of the magic prefix
     * @var string
     */
    private $tablePrefix;

    /**
     * Database connection string
     * @var string
     */
    private $connectionString;

    /**
     * The hash DSN (in PEAR/DSN style)
     * Available keys are "phptype", "hostspec", "username", "password" and "database"
     * @var array
     */
    private $DSN;

    /**
     * Last error message
     * @var string
     */
    private $lastErrorMsg = null;

    /**
     * Magic prefix used in your statement; this will be substituted by
     * the real table prefix on execute or by invoking substitutePrefix method
     *
     * @see substitutePrefix
     */
    const MAGIC_PREFIX = '#__';

    /**
     * Named Parameter definitions
     *
     * Defines how parameters are stored.
     *
     * @see wcmAbstractStatement::bindValue()
     * @see wcmAbstractStatement::bindParam()
     */
    const PARAM_NAMED = 1;

    /**
     * Anonymous Parameter definitions
     *
     * Defines how parameters are stored.
     *
     * @see wcmAbstractStatement::bindValue()
     * @see wcmAbstractStatement::bindParam()
     */
    const PARAM_ANON = 2;

    /**
     * FETCH MODES
     *
     * This is a list of common fetch modes. These are initially set
     * with the same values as PDO. I know, not very abstract,
     * but since WCM's primary data connector will be PDO and now it'll
     * be a requirement in the installation, I fail to see the harm
     * in doing this.
     *
     * Only using the most commont type of fetching plus fetching
     * into blank objects (FETCH_OBJ), creating instances of a class (FETCH_CLASS)
     * and fetching into instances of existing objects (FETCH_INTO).
     */

    /**
     * Fetch row into an instance of an object
     */
    const FETCH_INTO   = PDO::FETCH_INTO;

    /**
     * Fetch row into a blank object
     */
    const FETCH_OBJ    = PDO::FETCH_OBJ;

    /**
     * Fetch row into an assoc array
     */
    const FETCH_ASSOC  = PDO::FETCH_ASSOC;

    /**
     * Fetch row into an instance of a specified class
     */
    const FETCH_CLASS  = PDO::FETCH_CLASS;

    /**
     * Used with getScalarValue
     */
    const FETCH_COLUMN = PDO::FETCH_COLUMN;

    /**
     * Fetch a numerical array (eg: mysql_fetch_row())
     */
    const FETCH_NUM    = PDO::FETCH_NUM;


    /**
     * Database object constructor
     *
     * @param string $connectionString Database connection string (DSN syntax)
     * @param string $tablePrefix Common prefix for all tables in the database
     * @param wcmLogger $logger A logger object to debug database operations (or null)
     */
    public function __construct($connectionString, $tablePrefix = '', &$logger = null)
    {
        $config = wcmConfig::getInstance();

        $this->tablePrefix = $tablePrefix;
        $this->connectionString = $connectionString;
        $this->logger = ($logger !== null) ? $logger : wcmProject::getInstance()->logger;
        $this->debug = $config['wcm.logging.debug'];

        if (!$this->getConnection())
        {
            throw new Exception('Fatal error, unable to connect to database.');
        }
    }

    /**
     * This exists to establish a connection on demand
     *
     * If you do $this->connection it will automatically call
     * $this->getConnection().
     *
     * What's good about this is that __get() is only called once
     * since a new public property containing the PDO connection
     * is created.
     *
     * What's bad about this is that the PDO connection is exposed now.
     *
     * This came about due to a broader issue with storing data into session.
     *
     * @param $arg Name of argument
     * @return PDO PDO instance
     */
    public function __get($arg)
    {
        if ($arg == 'connection')
        {
            $this->connection = $this->getConnection();
        }
        return $this->connection;
    }

    /**
     * This function is automatically called on serialization of current object
     *
     * @return array An array of properties (their names) to store (serialize)
     */
    function __sleep()
    {
        return array('connectionString', 'tablePrefix', 'logger', 'debug');
    }

    /**
     * Reset (re-open) a connection
     *
     * @param string $connectionString Database connection string (DSN syntax)
     *               eg. "mysql://dbuser:dbpass@localhost/database"
     *
     * @return PDO A connection
     */
    public function resetConnection($connectionString)
    {
        $this->connection = null;
        if (isset(self::$db[$connectionString]))
        {
            unset(self::$db[$connectionString]);
        }
        $this->connectionString = $connectionString;
        return $this->getConnection();
    }

    /**
     * Open a connection
     *
     * @return PDO A connection or null if a exception is throw
     * @throws Exception When connection can't be established, or scheme is not supported
     */
    public function getConnection()
    {
        if ($this->connectionString != '')
        {
            // $this->DSN must always be defined,
            // regardless if the connection exists or not
            $dsnParts = parse_url($this->connectionString);
            $this->DSN = $dsnParts;

            // Since $this->connectionString is the full path to the database,
            // it acts as a unique key.  We can use this to store and reuse
            // the connection
            if (!isset(self::$db[$this->connectionString]))
            {
                try
                {
                    /**
                     * Here are various examples of PDO dsn strings
                     *
                     * mssql:host=localhost;dbname=testDb
                     * sybase:host=localhost;dbname=testDb
                     * dblib:host=localhost;dbname=testDb
                     * firebird:User=john;Password=mypass;Database=DATABASE.GDE;DataSource=localhost;Port=3050
                     * mysql:host=localhost;port=3307;dbname=testdb
                     * mysqlsock:unix_socket=/tmp/mysql.sock;dbname=testdb
                     * oci:dbname=mydb  // Defined in an .ORA file
                     * ociclient:dbname=//localhost:1521/mydb // Oracle Instant Client
                     * pgsql:host=localhost;port=5432;dbname=testdb;user=bruce;password=mypass
                     * sqlite:/opt/databases/mydb.sq3
                     * sqlite::memory:
                     * sqlite2:/opt/databases/mydb.sq2
                     * sqlite2::memory:
                     *
                     * PDO Supports more.
                     */

                    $dsnParts = parse_url($this->connectionString);

                    if (!$dsnParts || !isset($dsnParts['path']) || !isset($dsnParts['scheme']))
                    {
                        throw new Exception(_ERROR_CONNECTION_STRING);
                    }

                    $dsnParts['database'] = substr($dsnParts['path'], 1, strlen($dsnParts['path']));

                    switch ($dsnParts['scheme'])
                    {
                        case 'mssql':
                        case 'dblib';
                        case 'sysbase':
                        case 'mysql':
                            $dsn = $dsnParts['scheme'] . ':host=' . $dsnParts['host'] . ';';

                            if (isset($dsnParts['port']))
                            {
                                $dsn .= 'port=' . $dsnParts['port'] . ';';
                            }

                            $dsn .= 'dbname=' . $dsnParts['database'];
                            $username = getArrayParameter($dsnParts, 'user');
                            $password = getArrayParameter($dsnParts, 'pass');
                            break;

                        case 'pgsql':
                            $dsn = $dsnParts['scheme'] . ':host=' . $dsnParts['host'] . ';';

                            if (isset($dsnParts['port']))
                            {
                                $dsn .= 'port=' . $dsnParts['port'] . ';';
                            }

                            $dsn .= 'dbname=' . $dsnParts['database'];
                            $dsn .= 'user=' . getArrayParameter($dsnParts, 'user') . ';password=' . getArrayParameter($dsnParts, 'pass');
                            $username = null;
                            $password = null;
                            break;

                        default:
                            throw new Exception($dsnParts['scheme'] . ' presently not a supported database type in this version');
                            break;
                    }

                    $this->connection = new PDO($dsn, $username, $password, array(
                        PDO::ATTR_PERSISTENT => $this->persistent
                        ));
                    // General
                    $this->connection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
                    $this->connection->setAttribute(PDO::ATTR_ORACLE_NULLS, PDO::NULL_NATURAL);

                    // Set the connection for reuse
                    self::$db[$this->connectionString] =  $this->connection;

                }
                catch (Exception $e)
                {
                    $this->lastErrorMsg = 'getConnection (' . $this->connectionString . ') :: ' . $e->getMessage();
                    $this->logger->logError($this->lastErrorMsg);
                    return null;
                }
            }
            return self::$db[$this->connectionString];
        }
        return false;
    }

    /**
     * Gets the DSN (hash PEAR/DSN style) associated to connection
     * => Available keys are "phptype", "hostspec", "username", "password" and "database"
     *
     * @return array The hash array representing the DSN (PEAR/DSN style)
     */
    public function getDSN()
    {
        return $this->DSN;
    }

    /**
     * Get database table names
     * @return array Array of db table names
     */
    public function getDatabaseTables()
    {
        $query = 'SHOW TABLES';
        $rs = $this->executeNative($query);

        $tables = array();
        foreach ($rs as $table)
        {
            $tables[$table[0]] = $table[0];
        }
        return $tables;
    }

    /**
     * Gets the PHP connection kind
     *
     * @return string The connection kind ("mysql", "oracle", "mssql", ...)
     */
    public function getConnectionKind()
    {
        return $this->DSN['scheme'];
    }

    /**
     * Returns the last error message
     *
     * @return string Last error message
     */
    public function getErrorMsg()
    {
        return $this->lastErrorMsg;
    }

    /**
     * Returns the default table prefix used for this connection database
     *
     * @return string Default table prefix
     */
    public function getPrefix()
    {
        return $this->tablePrefix;
    }

    /**
     * Substitute the magic prefix with the table prefix
     *
     * @param string $sql   An sql statement which may contains the magic prefix
     *
     * @return string An sql statement with magic prefix substituted with the table prefix
     */
    private function substitutePrefix($sql)
    {
        return str_replace(self::MAGIC_PREFIX, $this->tablePrefix, $sql);
    }

    /**
     * Quotes an identifier depending on the connection kind
     *
     * => For mysql:  foo becomes `foo`
     * => For oracle: foo becomes "foo"
     * => For mssql:  foo becomes [foo] ...
     *
     * If $identifier is an array, then all values in the array
     * will be quoted. If $keys is set to true, then all keys in the array
     * will be quoted.
     *
     * @param mixed $identifier Identifier(s) to quote
     * @param bool $keys If set true and $identifier is an array, quote keys (defaults false)
     * @return string Quoted identifier
     */
    public function quoteIdentifier($identifier, $keys = false)
    {
        switch ($this->getConnectionKind())
        {
            case 'mysql':
            case 'mysqli':
                $quote[] = '`';
                $quote[] = '`';
                break;

            case 'mssql':
            case 'pgsql':
            case 'sqlite':
                $quote[] = '[';
                $quote[] = ']';
                break;

            case 'oracle':
            case 'odbc':
                $quote[] = '"';
                $quote[] = '"';
                break;

            default:
                $quote[] = '';
                $quote[] = '';
                break;
        }

        if (is_array($identifier))
        {
            $newArray = array();
            if ($keys)
            {
                foreach ($identifier as $key => $value)
                {
                    $key = $quote[0] . $key . $quote[1];
                    $newArray[$key] = $value;
                }
            }
            else
            {
                foreach ($identifier as $key => $value)
                {
                    $value = $quote[0] . $value . $quote[1];
                    $newArray[$key] = $value;
                }
            }
            return $newArray;
        }
        else
        {
            return $quote[0] . $identifier . $quote[1];
        }
    }

    /**
     * Create a table in the database
     *
     * @param string $tableName Name of the table to create
     * @param array $fields Fields of the table ($name => array($type = autoinc|integer|float|bool|date|time|datetime|varchar|text),
     *                                                          $notnull, $default)
     * @param string $primaryKey Name of the primary key (or null)
     * @param array $indexList Array of field names used as index (or null)
     */
    public function createTable($tableName, $fields, $primaryKey = null, $indexList = null)
    {
        switch ($this->getConnectionKind())
        {
            case 'mysql':
            case 'mysqli':
                $sql = 'CREATE TABLE ' . $this->quoteIdentifier($tableName) . '(' . PHP_EOL;
                $index = 0;
                foreach ($fields as $name => $field)
                {
                    $sql .= $this->quoteIdentifier($name);
                    $type = getArrayParameter($field, 'type', 'varchar(255)');
                    switch ($type)
                    {
                        case 'autoinc':
                            $sql .= ' integer not null auto_increment';
                            $primaryKey = $name;
                            break;

                        case 'integer':
                        case 'float':
                        case 'bool':
                        case 'date':
                        case 'time':
                        case 'datetime':
                        case 'text':
                            $sql .= ' ' . $type;
                            break;

                        default:
                            if ($type == 'varchar')
                            {
                                $sql .= ' varchar(255)';
                            }
                            elseif (substr($type, 0, 8) == 'varchar(')
                            {
                                $sql .= ' ' . $type;
                            }
                            else
                            {
                                die('Unexpected type "' . $type . ' for table creation!');
                            }
                            break;
                    }

                    if (getArrayParameter($field, 'notnull', false))
                    {
                        $sql .= ' not null';
                    }

                    if (getArrayParameter($field, 'default', null))
                    {
                        $sql .= ' default ' . $field['default'];
                    }

                    $sql .= ',' . PHP_EOL;
                }

                if ($primaryKey)
                {
                    $sql .= ' PRIMARY KEY (' . $this->quoteIdentifier($primaryKey) . ')';
                }

                if (is_array($indexList))
                {
                    foreach ($indexList as $index)
                    {
                        $sql .= ',' . PHP_EOL . 'KEY ' . $this->quoteIdentifier($index) .
                                ' (' . $this->quoteIdentifier($index) . ')';
                    }
                }

                $sql .= ') ENGINE=MyISAM DEFAULT CHARSET=utf8;';
                break;

            case 'mssql':
            case 'pgsql':
            case 'oracle':
            case 'odbc':
                die('Create table is not implemented for this connector : ' . $this->getConnectionKind());
        }

        $this->executeStatement($sql);
    }

    /**
     * Executes a native MySQL query
     *
     * This method will not PREPARE the query and should only be used
     * in extreme cases. It will return a PDOStatement
     *
     * @param string $sqlQuery Query to execute
     * @return PDOStatement PDO Statement object
     */
    private function executeNative($sqlQuery)
    {
        cmsDebugQuery::add($sqlQuery);
        return $this->connection->query($sqlQuery);
    }

    /**
     * Executes an sql query
     *
     * @param string $sqlQuery Sql query to execute
     * @param array $parameters Parameters used in conjunction with '?' occurrences in the sql statement
     * @param int $offset Offset of first returned row (0 by default)
     * @param int $limit Maximum number of returned row (0, by default, to retrieve all rows)
     * @param int $fetchmode Refer to the wcmDatabase::FETCH_* constants
     * @param mixed $fetchParam If the fetch mode requires an additional parameter
     *
     * @return wcmAbstractResultSet Result set object or False if a exception is throw
     */
    public function executeQuery($sqlQuery, $parameters = null, $offset = 0, $limit = 0, $fetchmode = self::FETCH_ASSOC, $fetchParam = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        // Prepare statement
        // Add the limit clause?
        // MySQL and pgSQL support this syntax
        // SQL Server does not
        if ($offset >= 0 && $limit > 0)
        {
            $sqlQuery .= ' LIMIT ' . $limit;
            $sqlQuery .= ' OFFSET ' . $offset;
        }
        $sqlQuery = $this->substitutePrefix($sqlQuery);

        $this->logger->logQuery($sqlQuery);

        try
        {
            // Load wcmDbStatement using a PDOStatement instance
            $stmt = new wcmStatement($this->connection->prepare($sqlQuery));

            cmsDebugQuery::add($sqlQuery, $parameters);
            $stmt->setFetchMode($fetchmode, $fetchParam);

            if ($resultSet = $stmt->executeQuery($parameters))
            {
                return $resultSet;
            }
            else
            {
                $this->lastErrorMsg = $stmt->getErrorMsg();
                $this->logger->logError('executeQuery (' . $sqlQuery . ') :: ' . $this->lastErrorMsg);
            }
        }
        catch (Exception $e)
        {
            $this->lastErrorMsg = 'executeQuery (' . $sqlQuery . ') :: ' . $e->getMessage();
            $this->logger->logError($this->lastErrorMsg);
        }
        return false;
    }

    /**
     * Executes an sql statement (if you want to execute a SELECT use {@link executeQuery} instead)
     *
     * @param string $sql Sql statement to execute (an INSERT, UPDATE or DELETE statement, ...)
     * @param array $parameters Parameters used in conjunction with '?' occurrences in the sql statement
     *
     * @return int Number of affected rows or -1 on failure
     */
    public function executeStatement($sql, array $parameters = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        // Substitute magic prefix in statement
        $sql = $this->substitutePrefix($sql);

        $this->logger->logQuery($sql);

        try
        {
            // Prepare statement
            $stmt = new wcmStatement($this->connection->prepare($sql));
            cmsDebugQuery::add($sql, $parameters);
            if ($stmt->executeStatement($parameters))
            {
                return $stmt->rowCount();
            }
            else
            {
                $this->lastErrorMsg = $stmt->getErrorMsg();
                $this->logger->logError($this->lastErrorMsg);
                return -1;
            }
        }
        catch (Exception $e)
        {
            $this->lastErrorMsg = 'executeStatement (' . $sql . ') :: ' . $e->getMessage();
            $this->logger->logError($this->lastErrorMsg);
        }
        return -1;
    }

    /**
     *
     * Executes a "SELECT" statement and returns the first value of the first row
     *
     * @param string $sql SQL query with possible occurrences of '?' for parameters
     * @param array  $parameters parameters matching the '?' occurences or null
     *
     * @return mixed The value of the first column of the first row (or null on empty/error)
     */
    public function executeScalar($sql, $parameters = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        try
        {
            // Retrieve resultset (first row only, no assoc mode) and do not trace again!
            $stmt = $this->executeQuery($sql, $parameters, 0, 1, self::FETCH_NUM, false);

            if (!$stmt)
            {
                return null;
            }

            $row = $stmt->next();
            return $row[0];
        }
        catch (Exception $e)
        {
            $this->lastErrorMsg = 'executeScalar (' . $sql . ') ::' . $e->getMessage();
            $this->logger->logError($this->lastErrorMsg);
        }
        return null;
    }

    /**
     *
     * Executes a "SELECT" statement and returns the first row values
     *
     * @param string $query SQL query with possible occurrences of '?' for parameters
     * @param array  $parameters parameters matching the '?' occurences (or null by default)
     * @param int $fetchmode Refer to the wcmDatabase::FETCH_* constants - defaults to wcmDatabase::FETCH_ASSOC
     * @param mixed $fetchParam Additional parameter for the fetch mode if necessary
     *
     * @return array An assoc array containings the first row values (keys are column names) or null
     */
    public function getFirstRow($query, $parameters = null, $fetchmode = self::FETCH_ASSOC, $fetchParam = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        try
        {
            // Retrieve resultset (first row only)
            $rs = $this->executeQuery($query, $parameters, 0, 1, $fetchmode, $fetchParam);
            if (false !== $rs && false !== $rs->next())
            {
                return $rs->current();
            }
        }
        catch (Exception $e)
        {
            $this->lastErrorMsg = 'getFirstRow (' . $query . ') :: ' . $e->getMessage();
            $this->logger->logError($this->lastErrorMsg);
        }
        return null;
    }

    /**
     * Inserts any object into the database
     *
     * @param string $tableName
     * @param wcmObject $object Object (by ref) to insert (public properties must match column names)
     * @param array  $ignore An optional array of fields/properties to ignore on insert (default is null)
     *
     * @return bool True on success, false on failure
     */
    public function insertObject($tableName, &$object, Array $ignore = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        $objectVars = getPublicProperties($object);

        // Process ignored properties
        if (is_array($ignore))
        {
            foreach ($ignore as $prop)
            {
                unset($objectVars[$prop]);
            }
        }

        $props = array_keys($objectVars);

        $sql = sprintf('INSERT INTO ' . $tableName . ' (%s) VALUES (:%s)', join(',', $this->quoteIdentifier($props)), join(',:', $props));

        return ($this->executeStatement($sql, $objectVars) >= 1);
    }

    /**
     * Updates any object into the database
     *
     * @param string $tableName
     * @param wcmObject $object Object (by ref) to insert (public properties must match column names)
     * @param string $primaryKey Name of column representing the primary key (default is "id")
     * @param array  $ignore An optional array of fields/properties to ignore on insert (default is null)
     *
     * @return bool True on success, false on failure
     */
    public function updateObject($tableName, wcmObject &$object, $primaryKey = 'id', Array $ignore = null)
    {
        // Clear last error message
        $this->lastErrorMsg = null;

        $objectVars = getPublicProperties($object);
        if (is_array($ignore))
        {
            foreach ($ignore as $prop)
            {
                unset($objectVars[$prop]);
            }
        }

        $sql = 'UPDATE ' . $object->getTableName() . ' SET ';
        foreach ($objectVars as $prop => $value)
        {
            $sql .= $this->quoteIdentifier($prop) . '=:' . $prop . ', ';
        }
        $sql = trim($sql, ', ');
        $sql .= ' WHERE id=' . $object->id;
        $sql = $this->substitutePrefix($sql);

        $res = $this->executeStatement($sql, $objectVars);
        return (is_int($res) && $res >= 0);
    }

    /**
     *
     * Load objects properties from the database using a specific id
     *
     * @param wcmObject $object Object to bind
     * @param int $id Id used to select row (or null to use object->id)
     *
     * @return bool True on success, false on failure
     */
    public function bindSysObject($wcmObject, $id = null)
    {
        // We assume that the primary key of a sysobject is "id"
        if ($id == null)
        {
            $id = $wcmObject->id;
        }

        $query = 'SELECT * FROM ' . $wcmObject->getTableName() . ' WHERE id=' . $id;

        return (null !== $this->getFirstRow($query, null, self::FETCH_INTO, $wcmObject));
    }

    /**
     *
     * Update or insert a wcmSysobject into the database
     *
     * @param wcmSysObject $object SysObject (by ref) to fill in
     * @param array  $ignore An optional array of fields/properties to ignore on insert (default is null)
     * @param bool   $forcedInsert Force an insert even if an object ID is present
     *
     * @return bool True on success, false on failure
     */
    public function storeSysObject($object, $ignore = null, $forcedInsert = false)
    {

        if (isset($object->id) && $object->id > 0 && !$forcedInsert)
        {
            // Update existing row (primary key is 'id', so don't update this value)
            return $this->updateObject($object->getTableName(), $object, 'id', array('id'));
        }
        else
        {

            try
            {
                // Inserting object will generate a new id (so don't set any value for 'id')

                $ignoreFields = (!$forcedInsert) ? array('id') : null;

                if (!$this->insertObject($object->getTableName(), $object, $ignoreFields))
                {
                    return false;
                }

                // Retrieve auto-increment id
                if (!$forcedInsert || ($forcedInsert && !$object->id))
                {
                    $object->id = $this->executeScalar('SELECT LAST_INSERT_ID()');
                }
                return ($object->id && $object->id > 0);
            }
            catch (Exception $e)
            {
                $this->lastErrorMsg = 'storeSysObject :: ' . $e->getMessage();
                $this->logger->logError($this->lastErrorMsg);
            }
        }
        return false;
    }

    /**
     * Delete a wcmSysobject
     *
     * @param wcmSysObject $object SysObject (by ref) to delete
     *
     * @return bool True on success, false on failure
     */
    public function deleteSysObject(&$object)
    {
        $sql = 'DELETE FROM ' . $this->quoteIdentifier($object->getTableName()) . ' WHERE id=?';

        return ($this->executeStatement($sql, array($object->id)) >= 1);
    }
}
