<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.generationSet.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The Generationset class represents a PnW generation set
 * A generation set will contain N generations that can be secured and launched together
 */
class wcmGenerationSet extends wcmSysobject
{
    /**
     * Object name
     * @var string
     */
    public $name = null;

    /**
     * GenerationSet code
     * @var string
     */
    public $code = '';

    /**
     * Path (relative or absolute) where to generation contents
     * @var string
     */
    public $location = '';

    /**
     * Generation set context
     * => A string representing ids of contextual classes (or null)
     * => Example: "story.id=3, section.id=6"
     * @var string
     */
    public $context = '';

    /**
     * Refresh object using his code
     *
     * @param string $code optional argument, if not specifed current id is used
     *
     * @return wcmGenerationSet freshen object or null on failure
     *
     */
    public function refreshByCode($code = null)
    {
        if (!isset($code))
        {
            $code = $this->code;
        }

        $sql = 'SELECT id FROM ' . $this->getTableName() . ' WHERE code=?';
        $id = $this->database->executeScalar($sql, array($code));
        return $this->refresh($id);
    }

    /**
     * Returns an array of generations belonging to current generation set
     *
     * @param bool $resetCache Reset cache afterwards?
     * @return array An associative array of {@link wcmGeneration} objects (keys are ids)
     */
    public function getGenerations($resetCache = false)
    {
        $generations = array();
        foreach ($this->getProject()->generator->getGenerations($resetCache) as $generation)
        {
            if ($generation->generationSetId == $this->id)
            {
                $generations[$generation->id] = $generation;
            }
        }

        return $generations;
    }

    /**
     * Inserts or Updates object in database
     *
     * @return bool true on success, false on failure
     */
    protected function store()
    {
        if (!$this->checkUniqueCode($this->code))
        {
            return false;
        }

        if (!parent::store())
        {
            return false;
        }

        // @todo : check if this is the correct place for the permissions update
        // delete permissions for this secure object
        $permissionType = $this->getType();
        $sql = "DELETE FROM #__permission WHERE target like '%" . $permissionType . "%'";
        $params = array($permissionType);

        if (wcmProject::getInstance()->database->executeStatement($sql, $params) == -1)
        {
            $this->lastErrorMsg = $permissionType . '::DELETE failed : ' . wcmProject::getInstance()->database->getErrorMsg();
            return false;
        }

        // insert the new permissions
        $newPermissions = getArrayParameter($this->serialStorage, 'permissions');

        if ($newPermissions)
        {
            $permsArray = array();
            foreach ($newPermissions as $permission)
            {
                // get values
                $perms = explode('*', $permission, 2);
                $permsArray[$perms[0]][] = $perms[1];
            }

            foreach ($permsArray as $wcmObject => $permissionMask)
            {
                $securedObject = explode('_', $wcmObject);
                $groupId   = $securedObject[0];

                $finalMask = 0;
                foreach ($permissionMask as $mask)
                {
                    // get group and values
                    $values = explode('#', $mask);
                    $finalMask += $values[0];

                    // set permissions
                    $this->setGroupPermission($groupId, $finalMask);
                }
            }
        }
        // @todo : remove to here if this is wrong

        // Update cache
        wcmCacheRegistry::get()->setElem($this->getType(), $this->id, $this);
        return true;
    }

    /**
     * Delete generationSet object from database (and associated generations and generation contents)
     *
     * @return bool True on success, false otherwise
     */
    public function delete()
    {
        // delete generations
        foreach ($this->getGenerations() as $generation)
        {
            $generation->delete();
        }

        if (!parent::delete())
        {
            return false;
        }

        // Update cache
        wcmCacheRegistry::get()->unsetElem($this->getType(), $this->id);

        return true;
    }

    /**
     * Check validity of object
     *
     * @return bool true if object is valid, else false
     */
    public function checkValidity()
    {
        if (trim($this->name . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAME_IS_MANDATORY;
            return false;
        }

        if (strlen($this->name) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAME_TOO_LONG;
            return false;
        }

        if (trim($this->code . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_CODE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->code) > 255)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        if (strlen($this->location) > 255)
        {
            $this->lastErrorMsg = _ERROR_LOCATION_TOO_LONG;
            return false;
        }

        if (strlen($this->context) > 255)
        {
            $this->lastErrorMsg = _ERROR_CONTEXT_TOO_LONG;
            return false;
        }

        return true;
    }
}
