<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.smartyModifiers.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class holds a set of Smarty modifiers
 * They call be called through a generic |wcm
 * modifier. e.g. {$bizobject|@wcm:name:param1:param2}
 * where 'name' is the function name
 *
 * Note: this class is extended by smartyModifier in
 * the business directory to allow overwrite and extension
 * for profesionnal services, partners and customers
 */
abstract class wcmSmartyModifiers
{

    /**
     * Returns the permissions on a given object
     *
     * Usage: {$elem|wcm:permission:permission_value}
     *
     * @param array $bizobject Assoc array of the bizobject
     * @param int $permission Permission for the object
     *
     * @return int permissions on given bizobject
     */
    public function permission($bizobject, $permission)
    {
        $object = wcmObjectFactory::create($bizobject['className'], $bizobject['id']);
        return wcmSession::getInstance()->isAllowed($object, $permission);
    }

    /**
     * Returns the content of a channel of collection
     *
     * Usage: {$channel|@wcm:content[:$limit[:$date]]}
     *
     * @param array $bizobject Assoc array of the bizobject
     * @param int $limit Maximum number of items to return (default is 10)
     * @param string $date Date used to compute channel or collection content (default is today's date)
     * @param string $orderby Order by string
     *
     * @return array An assoc array representing channel's content (mixed of bizobject and bizrelation assoc arrays)
     *               or null if classname is not found
     */
    public function content($bizobject, $limit = 10, $date = null, $orderby = null)
    {
        // set default value for date parameter?
        if ($date === null)
        {
            $date = date('Y-m-d');
        }

        $className = $bizobject['className'];
        switch ($className)
        {
            case 'channel':
            case 'collection':
                $result = array();
                $channel = $bizobject['this'];
                foreach ($channel->getContent($limit, $date, false, false, $orderby) as $item)
                {
                    if ($item !== null)
                    {
                        $result[] = ($item instanceOf wcmObjectAssocArray) ? $item : $item->getAssocArray(false);
                    }
                }
                return $result;

            default:
                return null;
        }
    }

    /**
     * Compute the permalink (on the web site) of a bizobject
     *
     * @param array $bizobject Assoc array of the bizobject
     *
     * @return string URL of the bizobject or null
     */
    public function permalink($bizobject)
    {
        $className = $bizobject['className'];
        switch ($className)
        {
            case 'site':
                return 'site' . $bizobject['id'] . '/';

            case 'chapter':
                $article = wcmObjectFactory::create('article', $bizobject['articleId']);
                return 'site' . $article->siteId . '/article/' . $article->id . 'c' . $bizobject['rank'] . '/'
                . safeFileName(strtolower($article->title)) . '.html';

            case 'contribution':
                $comment = $bizobject['this'];
                if (!($comment instanceof wcmBizclass) && $comment->getReferent())
                {
                    return self::permalink($comment->getReferent()->getAssocArray());
                }
                break;

            case 'channel':
                return 'site' . $bizobject['siteId'] . '/' . safeFileName(strtolower($bizobject['title'])) . '.html';

            case 'newsitem':
            case 'article':
            case 'slideshow':
            case 'forum':
            default:
                return 'site' . $bizobject['siteId'] . '/' . $className . '/' . $bizobject['id'] . '/'
                . safeFileName(strtolower($bizobject['title'])) . '.html';
        }

        return null;
    }

    /**
     * Compute the URL (on the web site) of a bizobject
     *
     * @param array $bizobject Assoc array of the bizobject
     *
     * @return string URL of the bizobject or null
     */
    public function url($bizobject)
    {
        $url = self::permalink($bizobject);
        if (!$url)
        {
            return null;
        }

        $config = wcmConfig::getInstance();
        return $config['wcm.webSite.url'] . $url;
    }

    /**
     * Returns a photo assocArray corresponding to the first bizobject's photo
     * or null if the bizobject has no photo.
     *
     * @param array $bizobject Assoc array of the bizobject
     *
     * @return array An assoc array representing first photo or null
     */
    public function firstPhoto($bizobject)
    {
        if (!$bizobject && !is_array($bizobject))
        {
            return null;
        }
        
        $sql = sprintf(
            'sourceClass=\'%s\' AND sourceID=%d AND destinationClass=\'%s\' AND kind=%d',
            $bizobject['className'], 
            $bizobject['id'], 
            'photo', 
            bizrelation::IS_COMPOSED_OF
        );
        
        $photo = null;
        $relation = new bizrelation();
        if ($relation->beginEnum($sql, 'rank', 0, 1))
        {
            if ($relation->nextEnum())
            {
                $photo = wcmObjectFactory::create('photo', $relation->destinationId);
            }
            $relation->endEnum();
        }

        if (null === $photo)
        {
            return null;
        }
        
        $result = $photo->getAssocArray(false);
        unset($photo);

        return $result;
    }

    /**
     * Returns the first element of a channel's content
     *
     * Usage:  {assign var=first value=$bizobject|@wcm:firstElement}
     *         {$first->title}
     *
     * @param array $bizobject Assoc array of the bizobject (channel)
     *
     * @return bizobject bizobject representing the first element to be displayed
     */
    public function firstElement($bizobject)
    {
        if (!$bizobject && !is_array($bizobject))
        {
            return null;
        }

        $channel = $bizobject['this'];

        $element = $channel->getfirstelem();

        return $element;
    }

    /**
     * Returns an array of bizrelation associated to a bizobject
     *
     * @param array $bizobject Assoc array of the bizobject
     * @param int   $kind Kind of relation (or bizrelation::IS_RELATED_TO by default)
     *
     * @return array An array of bizrelation's getAssocArray
     */
    public function relations($bizobject, $kind = null)
    {
        if (!$kind)
        {
            $kind = bizrelation::IS_RELATED_TO;
        }

        $relations = array();
        $where = 'sourceClass=\'' . $bizobject['className'] . '\' AND sourceId=' . $bizobject['id'] . ' AND kind=' . $kind;

        $enum = new wcmEnumeration('bizrelation', $where, 'rank');
        foreach ($enum as $bizrelation)
        {
            $relations[] = $bizrelation->getAssocArray(false);
        }

        return $relations;
    }

    /**
     * Return the URL of a bizPhoto
     *
     * @param array     $bizobject  Assoc array of the bizobject
     * @param btring    $format     Format of the image to generate (Default to 'original')
     * @param bool      $forceGen   set to yes if you want to force the generation (Default to false)
     *
     * @return string URL of the photo
     */
    public function photoUrl($bizobject, $format = 'original', $forceGen = false)
    {
        $config = wcmConfig::getInstance();
        $cmanager = new croppingManager($bizobject['this'], WCM_BACKOFFICE_CONF_DIR . '/photosratios/default.xml');

        if (!$path = $cmanager->getPhotoUrl($format, $forceGen))
        {
            if ($format == 'thumbnail')
            {
                $path = $bizobject['thumbnail'];
            }
            else
            {
                $path = $bizobject['original'];
            }
        }
        return $config['wcm.backOffice.url'] . $path;
    }

    /**
     * Return a specific entry from the result of URL parsing (parse_url() method)
     *
     * @param string $url  URL to parse
     * @param string $key  Key to retrieve (e.g. 'path', 'protocol', ...)
     *
     * @return mixed Either result of php function parse_url($url) or specific key when $key is set
     */
    public function parseUrl($url, $key)
    {
        $parsed = parse_url($url);
        return ($key) ? $parsed[$key] : $parsed;
    }
}
