<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.templateGenerator.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */


/**
 * The wcmTemplateGenerator class represents the generator used
 * in smarty to extend native tags (loop, exitloop, load, ..)
 *
 */
class wcmTemplateGenerator extends Smarty
{
     /**
     * Defines normal generation mode
     */
    const MODE_NORMAL = 0;

    /**
     * Defines preview generation mode 
     */
    const MODE_PREVIEW = 1;
    
     /**
     * The current generation mode
     * @var int
     */
    public $mode;
    
    /**
     * The current generation set
     * @var wcmGenerationSet
     */
    public $generationSet;

    /**
     * The current generation
     * @var wcmGeneration
     */
    public $generation;

    /**
     * The current generation content
     * @var wcmGenerationContent
     */
    public $generationContent;

    /**
     * The current template
     * @var wcmTemplate
     */
    public $template;

    /**
     * The logger
     * @var wcmLogger
     */
    public $logger;

    /**
     * The computer output file name for current template
     * @var string
     */
    public $outputFilename;

    /**
     * The folder corresponding to the computed output file name for current template
     * @var string
     */
    public $outputFolder;

    /**
     * Default widget mode
     * @var int (bit field)
     */
    public $widgetMode;

    /**
     * Global variables (associative array)
     * @var array
     */
    public $globals = array();

    /**
     * Loop information
     * @var array
     */
    private $loopInfo;

    /**
     * Current context
     * @var array
     */
    private $currentContext;

    /**
     * Context stack
     * @var array
     */
    private $contextStack;

    /**
     * Template content
     * @var string
     */
    private $templateContent;

    /**
     * Output filename pattern (smarty-like)
     * @var string
     */
    private $outputFilenamePattern;

    /**
     * Output base path
     * @var string
     */
    private $outputRootPath;

    /**
     * Loop parameters
     * @var array
     */
    private $loopParameters;

    /**
     * Constant-like: matches double quoted strings: "foobar", "foo\"bar"
     * @var string
     */
    private $wcm_db_qstr_regexp = '"[^"\\\\]*(?:\\\\.[^"\\\\]*)*"';

    /**
     * Constant-like: matches single quoted strings: 'foobar', 'foo\'bar'
     * @var string
     */
    private $wcm_si_qstr_regexp = '\'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\'';

    /**
     * Constant-like: matches single or double quoted strings
     * @var string
     */
    private $wcm_qstr_regexp = '(?:"[^"\\\\]*(?:\\\\.[^"\\\\]*)*"|\'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\')';

    /**
     * Constant-like: matches numerical constants 30, -12, 13.22
     * @var string
     */
    private $wcm_num_const_regexp = '(?:\-?\d+(?:\.\d+)?)';

    /**
     * Parse attribute string
     *
     * @param string $tag_args Tag arguments
     * @return array Attributes array
     */
    private function wcm_parse_attrs($tag_args)
    {
        /* Tokenize tag attributes. */
        preg_match_all('~(?:' . $this->wcm_qstr_regexp . ' | (?>[^"\'=\s]+))+ | [=]~x', $tag_args, $match);
        $tokens = $match[0];

        $attrs = array();
        /* Parse state:
         *   0 - expecting attribute name
         *   1 - expecting '='
         *   2 - expecting attribute value (not '=')
         */
        $state = 0;

        foreach ($tokens as $token)
        {
            switch ($state)
            {
                case 0:
                    /* If the token is a valid identifier, we set attribute name
                     and go to state 1. */
                    if (preg_match('~^\w+$~', $token))
                    {
                        $attr_name = $token;
                        $state = 1;
                    }
                    else
                    {
                        $this->logger->logError('invalid attribute name: \'' . $token . '\' ' . __FILE__ . 'line ' . __LINE__);
                        return null;
                    }
                    break;

                case 1:
                    /* If the token is '=', then we go to state 2. */
                    if ($token == '=')
                    {
                        $state = 2;
                    }
                    else
                    {
                        $this->logger->logError('expecting = after attribute name \'' . $last_token . '\' ' .  __FILE__ . 'line ' . __LINE__);
                        return null;
                    }
                    break;

                case 2:
                    /* If token is not '=', we set the attribute value and go to
                     state 0. */
                    if ($token != '=')
                    {
                        /* We booleanize the token if it's a non-quoted possible
                         boolean value. */
                        if (preg_match('~^(on|yes|true)$~', $token))
                        {
                            $token = 'true';
                        }
                        elseif (preg_match('~^(off|no|false)$~', $token))
                        {
                            $token = 'false';
                        }
                        elseif ($token == 'null')
                        {
                            $token = 'null';
                        }
                        /*
                        elseif (preg_match('~^' . $this->wcm_num_const_regexp . '|0[xX][0-9a-fA-F]+$~', $token))
                        {
                             // treat integer literally
                        }
                        */

                        // Remove extra double quotes
                        if ($token[0] == '"' && $token[strlen($token)-1] == '"')
                        {
                            $token = substr($token, 1, strlen($token) - 2);
                        }

                        $attrs[$attr_name] = $token;

                        $state = 0;
                    }
                    else
                    {
                        $this->logger->logError('\'=\' cannot be an attribute value ' . __FILE__ . 'line: ' . __LINE__);
                        return null;
                    }
                    break;
            }
            $last_token = $token;
        }

        if ($state != 0)
        {
            if ($state == 1)
            {
                $this->logger->logError('expecting \'=\' after attribute name ' . $last_token . '\' ' . __FILE__ . 'line ' . __LINE__);
                return null;
            }
            else
            {
                $this->logger->logError('missing attribute value \'' . $last_token . '\'' . __FILE__ . 'line ' . __LINE__);
                return null;
            }
        }

        $this->wcm_parse_vars_props($attrs);

        return $attrs;
    }

    /**
     * compile multiple variables and section properties tokens into
     * PHP code
     *
     * @param array $tokens Tokens
     */
    private function wcm_parse_vars_props(&$tokens)
    {
        foreach ($tokens as $key => $val)
        {
            $tokens[$key] = $this->wcm_parse_var_props($val);
        }
    }

    /**
     * compile single variable and section properties token into
     * PHP code
     *
     * @param string $val Var value
     * @return string Compiled value
     */
    private function wcm_parse_var_props($val)
    {
        $val = trim($val);

        if (preg_match('~^' . $this->wcm_db_qstr_regexp . '$~', $val))
        {
            // double quoted text
            preg_match('~^(' . $this->wcm_db_qstr_regexp . ')$~', $val, $match);
            // replace double quoted literal string with no quotes
            return preg_replace('~^"([\s\w]+)"$~', '\\1', $match[1]);
        }
        elseif (preg_match('~^' . $this->wcm_num_const_regexp . '$~', $val))
        {
            // numerical constant
            preg_match('~^(' . $this->wcm_num_const_regexp . ')$~', $val, $match);
        }
        elseif (preg_match('~^' . $this->wcm_si_qstr_regexp . '$~', $val))
        {
            // single quoted text
            preg_match('~^(' . $this->wcm_si_qstr_regexp . ')$~', $val, $match);
        }
        elseif (!is_numeric($val))
        {
            // literal
            return $val;
        }
        return $val;
    }

    /**
     * Constructor
     *
     * @param wcmLogger  $logger      Specific logger (or null to create a new one)
     * @param bool       $stripFilter Enable WrappedStripFilter (optimize code par removing extra white spaces and carriage returns)
     * @param bool       $widgetMode  Default widget mode (default is wcmWidget::VIEW_CONTENT)
     */
    public function __construct($logger = null, $stripFilter = true, $widgetMode = wcmWidget::VIEW_CONTENT, $mode = wcmTemplateGenerator::MODE_NORMAL)
    {
        $config  = wcmConfig::getInstance();

        $this->widgetMode = $widgetMode;
        $this->mode = $mode;

        // Initialize logger
        if ($logger)
        {
            $this->logger = $logger;
        }
        else
        {
            $this->logger = new wcmLogger($config['wcm.logging.verbose'], $config['wcm.logging.debug'], null, false);
        }

        // Base constructor
        $this->Smarty();

        // Initialize template directories and caching
        $this->temp_dir      = sys_get_temp_dir() . '/' . $config['wcm.project.guid'] . '/wcm/Smarty/';
        $this->compile_dir   = $this->temp_dir . 'templates_c/';
        $this->cache_dir     = $this->temp_dir . 'cache/';
        $this->config_dir    = $config['smarty.path'] . 'configs/';
        $this->template_dir  = $config['wcm.templates.path'];
        $this->plugins_dir[] = WCM_BIZ_API_DIR . '/common/generator';

        // Create template directories if necessary
        makeDirectory($this->compile_dir);
        makeDirectory($this->cache_dir);

        // Add special variables
        $this->assign_by_ref('this', $this);
        $assocConfig = wcmConfig::getInstance()->getAssocArray();
        $this->assign('config', $assocConfig);
        $this->assign('session', wcmSession::getInstance()->getAssocArray(false));

        //
        // Register blocks and special functions
        //

        // globals
        $this->register_function('global', array($this, 'func_global'));
        $this->globals = array();
        $this->assign_by_ref('globals', $this->globals);

        // context
        $this->register_block('context', array($this, 'func_context'));
        $this->contextStack = array();
        $this->currentContext = array();
        $this->assign_by_ref('contextStack', $this->contextStack);
        $this->assign_by_ref('context', $this->currentContext);

        // dump
        $this->register_function('dump', array($this, 'func_dump'));

        // load...
        $this->register_function('load', array($this, 'func_load'));
        $this->register_function('unload', array($this, 'func_unload'));
        $this->register_function('loadArray', array($this, 'func_loadArray'));
        $this->register_function('count', array($this, 'func_count'));

        // loop
        $this->register_compiler_function('loop', array($this, 'func_loop'));
        $this->register_compiler_function('/loop', array($this, 'func_endloop'));
        $this->register_compiler_function('exitloop', array($this, 'func_exitloop'));
        $this->loopInfo = array();
        $this->assign_by_ref('loop', $this->loopInfo);

        // search
        $this->register_compiler_function('search', array($this, 'func_search'));
        $this->register_compiler_function('/search', array($this, 'func_endsearch'));
        $this->register_compiler_function('exitsearch', array($this, 'func_exitsearch'));

        // zone
        $this->register_compiler_function('zone', array($this, 'func_zone'));

        // Widget
        $this->register_function('widget', array($this, 'func_widget'));

        // Wcm function and modifier
        $this->register_function('wcm', array($this, 'func_wcm'));
        $this->register_modifier('wcm', array($this, 'mod_wcm'));

        // 
        $this->register_modifier('constant', array($this, 'mod_constant'));
        $this->register_modifier('safe_filename', array($this, 'mod_safe_filename'));
    }

    /**
     * Destructor
     */
    function __destruct()
    {
        // Free resources
        unset($this->logger);
    }

    /**
     * Used to clean memory usage of smarty function registered
     */
    public function unregister_all_function()
    {
        unset($this->_plugins['function']);
    }

    /**
     * Used to clean memory usage of smarty block registered
     */
    public function unregister_all_block()
    {
        unset($this->_plugins['block']);
    }

    /**
     * Used to clean memory usage of smarty compile function registered
     */
    public function unregister_all_compiler()
    {
        unset($this->_plugins['compiler']);
    }

    /**
     * Used to clean memory usage of smarty object registered
     */
    public function unregister_all_object()
    {
        unset($this->_reg_objects);
    }

    /**
     * Free memory usage of assign variables, registered functions, blocks, objects and compiled functions
     */
    public function clear_all()
    {
        $this->clear_all_assign();
        $this->unregister_all_function();
        $this->unregister_all_block();
        $this->unregister_all_compiler();
        $this->unregister_all_object();
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    // INTERNAL OBJECT MANAGEMENT (generation set, generation, generationContent, template //
    /////////////////////////////////////////////////////////////////////////////////////////
    /**
    * Open a generation set
    *
    * @param wcmGenerationSet $generationSetId Generation set object
    *
    */
    private function openGenerationSet($generationSet)
    {
        // Reset context stack
        $this->contextStack = array();
        $this->currentContext = array();

        // set generation object
        $this->generationSet = $generationSet;

        // Adjust output root path to ensure it ends with a '/'
        $this->outputRootPath = $this->generationSet->location;
        $lastchar = substr($this->outputRootPath, -1);
        if ($lastchar != '/' && $lastchar != '\\')
        {
            $this->outputRootPath .= '/';
        }

        // Add generation context
        $context = $this->createContext($this->generationSet->context);
        $this->pushContext($context);

        $this->logger->logVerbose('Executing generation set ' . $this->generationSet->name . ' at ' . $this->outputRootPath . ' with context ' . $this->generationSet->context);
    }

    /**
     * Open a generation
     *
     * @param wcmGeneration $generation Generation object
     *
     */
    private function openGeneration(wcmGeneration $generation)
    {
        // set generation object
        $this->generation = $generation;

        // open generationSet?
        if (!$this->generationSet || $this->generationSet->id != $this->generation->generationSetId)
        {
            $generationSet = new wcmGenerationSet();
            $generationSet->refresh($this->generation->generationSetId);
            $this->openGenerationSet($generationSet);
        }

        // if generation location is defined, override generationSet location
        if ($this->generation->location)
        {
            $this->outputRootPath = $this->generation->location;
            $lastchar = substr($this->outputRootPath, -1);
            if ($lastchar != '/' && $lastchar != '\\')
            {
                $this->outputRootPath .= '/';
            }
        }

        // Add generation context
        $context = $this->createContext($this->generation->context);
        $this->pushContext($context);

        $this->logger->logVerbose('Executing generation ' . $this->generation->name . ' at ' . $this->outputRootPath . ' with context ' . $this->generation->context);
    }

    /**
     * Close generation set
     *
     */
    private function closeGenerationSet()
    {
        $this->logger->logVerbose('Generation set ' . $this->generationSet->name . ' ended');

        $this->generationSet = null;

        // Remove generation context
        $this->popContext();
    }

    /**
     * Close generation
     *
     */
    private function closeGeneration()
    {
        if ($this->generation)
        {
            $this->logger->logVerbose('Generation ' . $this->generation->name . ' ended');
        }

        $this->generationContent = null;
        $this->generation = null;

        // Remove generation context
        $this->popContext();
    }

    /**
     * Loads parameters from current generation content
     */
    private function openGenerationContent()
    {
        // Load template
        $this->openTemplate($this->generationContent->templateId);

        $this->outputFilenamePattern = $this->generationContent->namingRule;
        $this->loopParameters = trim($this->generationContent->loop);

        // Add generationContent context
        $context = $this->createContext($this->generationContent->context);
        $this->pushContext($context);

        $this->logger->logVerbose('Executing generationContent ' . $this->generationContent->name . ' with context ' . $this->generationContent->context);
    }

    /**
     * Close generation content
     */
    private function closeGenerationContent()
    {
        // Unload template
        $this->closeTemplate();

        $this->generationContent = null;

        // Remove generation context
        $this->popContext();
    }

    /**
     * Open a template
     *
     * @param int $generationContentId Id of template to load (or null to use current)
     */
    private function openTemplate($templateId)
    {
        // Load template
        $this->template = new wcmTemplate();
        $this->template->refresh($templateId);
        $this->templateContent = $this->template->content;
    }

    /**
     * Close current template
     */
    private function closeTemplate()
    {
        $this->template = null;
        $this->templateContent = null;
    }

    //////////////////////////////////////////////////////////////////////////////////
    // CONTEXT MANAGEMENT
    //////////////////////////////////////////////////////////////////////////////////

    /**
     * Push a context array in the context stack
     *
     * @param array $context Associative array representing context to push
     */
    private function pushContext($context)
    {
        // Ensure we have an array
        if ($context == null || !is_array($context))
        {
            $context = array();
        }

        // Note: when a context is pushed, it overload the existing one
        // we have to retrieve the current one and overload with new one
        $newContext = $this->getContext();
        foreach ($context as $key => $val)
        {
            $newContext[$key] = $val;
        }

        // Push context and update current context variable
        $this->contextStack[] = $newContext;
        $this->currentContext = $this->getContext();

        $this->logger->logVerbose('Pushed context: ' . print_r($this->currentContext, true));
    }

    /**
     * Pop last context out of context stack
     */
    private function popContext()
    {
        if (count($this->contextStack) > 0)
        {
            // Pop context and update current context variable
            array_pop($this->contextStack);
            $this->currentContext = $this->getContext();

            $this->logger->logVerbose('Popped context: ' . print_r($this->currentContext, true));
        }
        else
        {
            $this->logger->logError('Unexpected POP context as contextStack is empty!');
        }
    }

    /**
     * Returns the current context (on top of context stack)
     *
     * @return array Associative array representing current context
     */
    private function getContext()
    {
        $n = count($this->contextStack);
        if ($n > 0)
        {
            // Return last context (top of stack)
            return $this->contextStack[$n-1];
        }

        // Return an empty array
        return array();
    }

    /**
     * Create an assoc array containing the variable extracted from a context string
     *
     * @param string $values A context string (e.g. "class1[.id]=value1, class2[.id]=value2, ...")
     * @return array Assoc array
     */
    private function createContext($values)
    {
        $context = array();
        $elements = explode(',', $values);
        foreach ($elements as $element)
        {
            // Expect '='
            if (strpos($element, '='))
            {
                $parts = explode('=', $element, 2);

                $key = $parts[0];
                $val = $parts[1];
                // only supress one of the surrounding value...
                if ($val[0] == '"' || $val[0] == "'" )
                {
                    $val[0] = ' ';
                }

                $lastIdx = mb_strlen($val) - 1;
                if ($val[$lastIdx] == '"' || $val[$lastIdx] == "'")
                {
                    $val[$lastIdx] = ' ';
                }

                $key = trim($parts[0]);
                $val = trim($parts[1]);

                // Remove extra '.id' in key
                if (substr($key, -3) == '.id')
                {
                    $key = substr($key, 0, strlen($key)-3);
                }

                // Add key/value pair if the key is not empty after trimming
                if ($key != '' && $val != '')
                {
                    $context[$key] = $val;
                }
            }
        }

        $this->logger->logVerbose('Create context from ' . $values . ': ' . print_r($context, true));
        return $context;
    }

    /**
     * Extra the associative array needed to computre an ofClause from current context
     *
     * @param string $of The of clause (from a {loop} tag or multipleGeneration)
     *
     * @return array Associative array used to compute ofClause()
     */
    private function extractContext($of)
    {
        // Prepare result
        $ofContext = array();

        // Retrieve current context
        $context = $this->getContext();


        // Extract the list of classes to retrieve
        $classes = explode(',', $of);
        foreach ($classes as $key)
        {
            $key = trim($key);
            if (isset($context[$key]))
            {
                $ofContext[$key] = $context[$key];
            }
        }

        return $ofContext;
    }

    //////////////////////////////////////////////////////////////////////////////////
    // EXECUTION (generate template, generationContent, generation)
    //////////////////////////////////////////////////////////////////////////////////

    /**
     * Compute output filename from pattern
     *
     * @param string $sPattern The filename pattern
     * @return string Output filename
     */
    private function computeOutputFileName($sPattern)
    {
        $outputFilename = $this->computeVariable($sPattern);

        // Append root path ?
        if ($this->outputRootPath != '')
        {
            // Do not append root path if output file name is absolute (linux/windows/mac)
            if ($outputFilename[0] != '/' && $outputFilename[1] != ':' && $outputFilename[0] != ':')
            {
                // Parse root path through smarty engine
                $outputPath = $this->computeVariable($this->outputRootPath);
                $outputFilename = $outputPath . $outputFilename;
            }
        }

        return $outputFilename;
    }

    /**
     * Compute output from template content
     *
     * @param string $sPattern The template source code
     * @return string Compiled template content
     */
    private function computeVariable($sPattern)
    {
        $_var_compiled = null;
        $this->_compile_source(__CLASS__, $sPattern, $_var_compiled);
        ob_start();
        $this->_eval('?>' . $_var_compiled);
        $_contents = ob_get_contents();
        ob_end_clean();
        return $_contents;
    }

    /**
     * Execute a single template
     *
     * @param wcmTemplate $template template to execute
     * @param array $parameters Extra parameters (assoc array) used to create smarty variables
     *
     * @return string Template execution result
     */
    public function executeTemplate($template, array $parameters = null)
    {
        // Reset context
        $this->generation = null;
        $this->generationContent = null;

        // Keep current generator variable set.
        $currentContext = $this->_tpl_vars;

        // Add extra context variables.
        if ($parameters)
        {
            foreach ($parameters as $name => $value)
            {
                $this->assign($name, $value);
            }
        }

        $this->template = $template;
        $this->templateContent = $this->template->content;
        $result = $this->computeVariable($this->templateContent);
        $this->closeTemplate();

        // Reset the previous generator variable set.
        $this->_tpl_vars = $currentContext;

        return $result;
    }

    /**
     * Execute a specific generation set
     *
     * @param wcmGenerationSet $generationSet generation to execute
     * @param array $extraContext An optional assoc array to extend generation context
     * @param bool   $generateFile         Indicates if the output must be a file or echoed
     *
     * @return array Empty array when $generateFile is set to TRUE or an array of associative arrays
     *         (structured as "name" => output file name, "content => generation result)
     *         or null on error
     *
     */
    public function executeGenerationSet($generationSet, $extraContext = null, $generateFile = true)
    {
        $result = array();
        $this->openGenerationSet($generationSet);
        foreach ($this->generationSet->getGenerations() as $generation)
        {
            $result = array_merge($result, $this->executeGeneration($generation, $extraContext, $generateFile));
        }
        $this->closeGenerationSet();

        return $result;
    }

    /**
     * Execute a specific generation
     *
     * @param wcmGeneration $generation	Generation to execute
     * @param array $extraContext An optional assoc array to extend generation context
     * @param bool   $generateFile   Indicates if the output must be a file or echoed
     *
     * @return array Empty array when $generateFile is set to TRUE or an array of associative arrays
     *         (structured as "name" => output file name, "content => generation result)
     *         or null on error
     */
    public function executeGeneration($generation, $extraContext = null, $generateFile = true)
    {
        $results = array();

        // Open generation
        $this->openGeneration($generation);

        // Execute each generation content
        $contents = $this->generation->getContents();
        foreach ($contents as $gcId => $generationContent)
        {
            // Execute generation content and avoid re-opening of current generation
            $generationContent = new wcmGenerationContent();
            $generationContent->refresh($gcId);
            $result = $this->executeGenerationContent($generationContent, $extraContext, $generateFile, true);
            $results = array_merge($results, $result);
        }

        // Close generation
        $this->closeGeneration();

        return $results;
    }

    /**
     * Execute a specific generation content
     *
     * @param wcmGenerationContent  $generationContent    Generation Content
     * @param array     $extraContext         An optional Assoc array to extend generation context
     * @param bool   $generateFile         Indicates if the output must be a file or echoed
     * @param bool   $doNotOpenGeneration  Do not open generation (WARNING: for internal use only. Set to FALSE)
     * @param boolean $removeFile Indicates if we want to remove the file file removal has precedence over file generation.
     *
     * @return array Empty array when $generateFile is set to true or an array of associative arrays
     *         (structured as "name" => output file name, "content => generation result)
     *         or null on error
     */
    public function executeGenerationContent(wcmGenerationContent $generationContent, $extraContext = null, $generateFile = true, $doNotOpenGeneration = false, $removeFile = false)
    {
        $result = null;

        // Open generation is needed
        if (!$doNotOpenGeneration)
        {
            $generation = $generationContent->getGeneration();
            $this->openGeneration($generation);
        }

        // Open generation content
        $this->generationContent = $generationContent;
        $this->openGenerationContent();

        // Single or multiple generation (loop?)
        if ($this->loopParameters == '')
        {
            $result = $this->singleGeneration($extraContext, $generateFile, $removeFile);
        }
        else
        {
            $result = $this->multipleGeneration($extraContext, $generateFile, $removeFile);
        }

        // Close generation content
        $this->closeGenerationContent();

        // Close generation is needed
        if (!$doNotOpenGeneration)
        {
            $this->closeGeneration();
        }

        return $result;
    }

    /**
     * Performs a single generation
     *
     * @param array $extraContext Assoc array containing extra information for context
     * @param bool $generateFile Indicates if the output must be a file or echoed
     * @param boolean $removeFile Indicates if we want to remove the file file removal has precedence over file generation.
     *
     * @return array Empty array when $generateFile is set to TRUE or associative array
     *         (structured as "name" => output file name, "content => generation result)
     *         or null on error
     */
    private function singleGeneration($extraContext = null, $generateFile = true, $removeFile = false)
    {
        // Push extra context
        $this->pushContext($extraContext);

        // Compute output filename and execute template
        $this->outputFilename = $this->computeOutputFileName($this->outputFilenamePattern);
        $this->outputFolder = dirname($this->outputFilename) . '/';

        if ($removeFile)
        {
            if (file_exists($this->outputFilename))
            {
                @unlink($this->outputFilename);
                $this->logger->logMessage('file : ' . $this->outputFilename . ' removed successfully');
            }
            $result = array();
        }
        else
        {
            // Generate into array
            $this->logger->logMessage('Generation of: ' . $this->outputFilename);
            $result = array('name' => $this->outputFilename, 'content' => $this->computeVariable($this->templateContent));

            // Generate file?
            if ($generateFile)
            {
                saveToFile($this->outputFilename, $result['content']);
                $result = array();
            }
        }

        // Pop extra context
        $this->popContext();

        return $result;
    }

    /**
     * Performs a multiple generation (act as a loop in generationContent)
     *
     * @param array $extraContext Assoc array containing extra information for context
     * @param bool $generateFile Indicates if the output must be a file or echoed
     * @param boolean $removeFile Indicates if we want to remove the file file removal has precedence over file generation.
     *
     * @return array Empty array when $generateFile is set to true or an array of associative arrays
     *         (structured as "name" => output file name, "content => generation result)
     *         or null on error
     */
    private function multipleGeneration($extraContext = null, $generateFile = true, $removeFile = false)
    {
        // Parse loop parameters
        $tParams = $this->wcm_parse_attrs($this->loopParameters);
        $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
        $assocname = getArrayParameter($tParams, 'name', $tablename);
        $objname   = getArrayParameter($tParams, 'object', 'o' . $tablename);
        $of        = getArrayParameter($tParams, 'of', null);
        $where     = getArrayParameter($tParams, 'where', null);
        $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
        $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
        $limit     = getArrayParameter($tParams, 'limit', -1);
        $to        = getArrayParameter($tParams, 'to', -1);
        $batchsize = getArrayParameter($tParams, 'batchsize', 1);
        $batchinfo = getArrayParameter($tParams, 'batchinfo', 'batchinfo');

        if ($to != -1)
        {
            $limit = ($to - $from + 1);
        }
        $this->pushContext($extraContext);
        $ofContext = $this->extractContext($of);

        //
        // Parse where clause through smarty engine!
        //
        if ($where && strpos($where, '{') !== null)
        {
            // Parse root path through smarty engine
            $where = $this->computeVariable($where);
        }

        if ($tablename == null)
        {
            $this->logger->logError('Multiple generation fatal error: [class] parameter in loop is mandatory');
            $this->popContext();
            return null;
        }

        // Prepare result
        $globalResult = array();
        // Iterate bizobjects (loop)
        $enum = new wcmEnumeration($tablename, $where, $order, $from, $limit, $ofContext);
        // Initalize object counter to zero
        $objectsProcessed = 0;

        // Fetch total objects to loop through
        $objectsExpected = ceil(count($enum) / $batchsize);
        $this->logger->logMessage('Generation expected to create ' . $objectsExpected . ' files');

        if ($batchsize > 1)
        {
            $batch_info_array = array('total' => $objectsExpected, 'current' => 0);
            $this->assign_by_ref($batchinfo, $batch_info_array);
        }

        $enum->rewind();
        while (true)
        {
            if ($batchsize == 1)
            {
                if (!$enum->valid())
                {
                    break;
                }
                else
                {
                    $bizobject = $enum->current();
                    // Update assoc array
                    $assocarray = $bizobject->getAssocArray(false);
                    $this->assign_by_ref($objname, $bizobject);
                    $this->assign_by_ref($assocname, $assocarray);
                    $enum->next();
                }
            }
            else
            {
                $batch_assocarrays = array();
                $batch_bizobjects  = array();
                for ($count = 0 ; $count < $batchsize ; $count++)
                {
                    if (!$enum->valid())
                    {
                        break;
                    }
                    $batch_bizobjects[$count] = $enum->current();
                    $batch_assocarrays[$count] = $batch_bizobjects[$count]->getAssocArray(false);
                    $enum->next();
                }
                if ($count == 0)
                {
                    break;
                }
                else
                {
                    $batch_info_array['current']++;
                    $this->assign_by_ref($objname, $batch_bizobjects);
                    $this->assign_by_ref($assocname, $batch_assocarrays);
                }
            }

            // Compute output filename and execute template
            $this->outputFilename = $this->computeOutputFileName($this->outputFilenamePattern);
            $this->outputFolder = dirname($this->outputFilename) . '/';

            if ($removeFile)
            {
                if (file_exists($this->outputFilename))
                {
                    @unlink($this->outputFilename);
                    $this->logger->logMessage('Removed file: ' . $this->outputFilename);
                }
            }
            else
            {
                // Generate into array
                $objectsProcessed++;
                $this->logger->logMessage('Generation of file ' . $objectsProcessed . ': ' . $this->outputFilename);
                $result = array('name' => $this->outputFilename, 'content' => $this->computeVariable($this->templateContent));

                // Generate file?
                if ($generateFile)
                {
                    saveToFile($this->outputFilename, $result['content']);
                    $result = array();
                }
                else
                {
                    // Push result into global result
                    $globalResult[] = $result;
                }
            }

        }

        if ($removeFile)
        {
            $this->logger->logMessage('Removed ' . $objectsProcessed . ' out of ' . $objectsExpected . ' files');
            if ($objectsProcessed != $objectsExpected)
            {
                $this->logger->logError('Error: failed to remove the expected amount of files');
                }
        }
        else
        {
            $this->logger->logMessage('Generated ' . $objectsProcessed . ' out of ' . $objectsExpected . ' files');
            if ($objectsProcessed != $objectsExpected)
            {
                $this->logger->logError('Error: failed to produce the expected amount of files');
            }
        }

        $this->clear_assign($objname);
        $this->clear_assign($assocname);
        $this->clear_assign($batchinfo);

        // Pop extra context
        $this->popContext();
        return $globalResult;
    }

    //////////////////////////////////////////////////////////////////////////////////
    // EXTRA TAGS AND BLOCKS
    //////////////////////////////////////////////////////////////////////////////////

    /**
     * Handles {global} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     */
    public function func_global($tParams, &$oSmarty)
    {
        $varname = getArrayParameter($tParams, 'var', null);
        $value = getArrayParameter($tParams, 'value', null);

        if ($varname)
        {
            $this->globals[$varname] = $value;
            $this->logger->logVerbose('gobal assign ' . $varname . ' to ' . $value);
        }
    }

    /**
     * Handles {dump} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     */
    public function func_dump($tParams, &$oSmarty)
    {
        // dump output
        $filename = getArrayParameter($tParams, 'file', null);
        $content  = getArrayParameter($tParams, 'content', null);
        $utf8 = (null != getArrayParameter($tParams, 'utf8', null));

        if ($filename)
        {
            saveToFile($filename, $content, false, $utf8);
            $this->logger->logMessage('Generation of: ' . $filename);
        }
    }

    /**
     * Handles {loadArray} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     *
     * @return null or also null on failure
     */
    public function func_loadArray($tParams, &$oSmarty)
    {
        // Parse parameters
        $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
        $assocname = getArrayParameter($tParams, 'name', $tablename);
        $objname   = getArrayParameter($tParams, 'object', 'o' . $tablename);
        $of        = getArrayParameter($tParams, 'of', null);
        $where     = getArrayParameter($tParams, 'where', null);
        $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
        $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
        $limit     = getArrayParameter($tParams, 'limit', -1);
        $to        = getArrayParameter($tParams, 'to', -1);
        if ($to != -1)
        {
            $limit = ($to - $from + 1);
        }

        $ofContext = $this->extractContext($of);

        if ($tablename == null)
        {
            $this->logger->logError('loadArray fatal error: [class] parameter is mandatory');
            return null;
        }

        // Prepare array and enumerate bizobject
        $bigarray = array();
        $enum = new wcmEnumeration($tablename, $where, $order, $from, $limit, $ofContext);

        foreach ($enum as $bizobject)
        {
            $bigarray[] = $bizobject->getAssocArray(false);
        }

        // Expose array
        $this->assign($assocname, $bigarray);
    }

    /**
     * Handles {load} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     *
     * @return null or also null on failure
     */
    public function func_load($tParams, &$oSmarty)
    {
        // Parse parameters
        $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
        $assocname = getArrayParameter($tParams, 'name', $tablename);
        $objname   = getArrayParameter($tParams, 'object', 'o' . $tablename);
        $of        = getArrayParameter($tParams, 'of', null);
        $where     = getArrayParameter($tParams, 'where', null);
        $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
        $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
        $ofContext = $this->extractContext($of);

        if ($tablename == null)
        {
            $this->logger->logError('load fatal error: [class] parameter is mandatory');
            return null;
        }

        // Enumerate bizobject
        $bizobject = wcmObjectFactory::create($tablename);
        $assocarray = array();
        if ($bizobject->beginEnum($where, $order, $from, 1, $ofContext))
        {
            if ($bizobject->nextEnum())
            {
                // Expose object and assocarray
                $this->logger->logVerbose('load class=' . $tablename . ' where=' . $where . ' of=' . $of . ' offset=' . $from);
                $this->assign_by_ref($objname, $bizobject);
                $this->assign($assocname, $bizobject->getAssocArray(false));
            }
            else
            {
                $this->logger->logError('load failed: enumerating ' . $tablename . ' returns no result');
            }
            $bizobject->endEnum();
        }
        else
        {
            $this->logger->logError('load fatal error: enumerating ' . $tablename . ' failed: ' . $bizobject->getErrorMsg());
        }
    }

    /**
     * Handles {unload} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     */
    public function func_unload($tParams, &$oSmarty)
    {
        // Parse parameters
        $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
        $assocname = getArrayParameter($tParams, 'name', $tablename);
        $objname   = getArrayParameter($tParams, 'object', 'o' . $tablename);

        // Unload assoc array and object
        $this->clear_assign($assocname);
        $this->clear_assign($objname);
    }

    /**
     * Handles {context} tag
     * @param array $tParams tag parameters
     * @param string $content Content
     * @param Smarty $oSmarty Smarty object
     * @param bool $repeat Repeat context?
     *
     * @return string $content if is set else null
     */
    public function func_context($tParams, $content, &$oSmarty, &$repeat)
    {
        if (!isset($content))
        {
            // handle open tag
            $values = getArrayParameter($tParams, 'values', null);
            $this->logger->logVerbose('Overload context with values ' . $values);
            $context = $this->createContext($values);
            $this->pushContext($context);
        }
        else
        {
            // handle closing tag
            $this->logger->logVerbose('Restore previous context');
            $this->popContext();
            return $content;
        }
    }

    /**
     * Handles {count} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     *
     * @return null or also null on failure
     */
    public function func_count($tParams, &$oSmarty)
    {
        // Parse parameters
        $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
        $varname   = getArrayParameter($tParams, 'var', $tablename . 'Count');
        $of        = getArrayParameter($tParams, 'of', null);
        $where     = getArrayParameter($tParams, 'where', null);
        $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
        $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
        $limit     = getArrayParameter($tParams, 'limit', -1);
        $to        = getArrayParameter($tParams, 'to', -1);
        if ($to != -1)
        {
            $limit = ($to - $from + 1);
        }

        $ofContext = $this->extractContext($of);

        if ($tablename == null)
        {
            $this->logger->logError('count fatal error: [class] parameter is mandatory');
            return null;
        }

        // Enumerate bizobject
        $bizobject = wcmObjectFactory::create($tablename);
        if ($bizobject->beginEnum($where, $order, $from, $offset, $ofContext))
        {
            $this->assign($varname, $bizobject->enumCount());
            $bizobject->endEnum();
        }
        else
        {
            $this->logger->logError('count fatal error: enumerating ' . $tablename . ' failed: ' . $bizobject->getErrorMsg());
        }
        unset($bizobject);
    }

    /**
     * Handles {zone} tag
     * @param array $tParams tag parameters
     * @param Smarty $oSmarty Smarty object
     *
     * @return string output or null on failure
     */

    public function func_zone($tParams, &$oSmarty)
    {
        $params = $this->wcm_parse_attrs($tParams);

        $bizobject = getArrayParameter($params, 'bizobject', null);
        if ($bizobject == null)
        {
            $this->logger->logError('error missing [bizobject] parameter in zone tag; file ' . __FILE__ . ' line ' . __LINE__);
            return;
        }

        $name = getArrayParameter($params, 'name', null);
        if ($name == null)
        {
            $this->logger->logError('error missing [name] parameter in zone tag; file ' . __FILE__ . ' line ' . __LINE__);
            return;
        }

        $output = '';

        $output .= '$_bizobject = $this->_tpl_vars["' . $bizobject . '"];';
        $output .= '$_content = new wcmDesignZone(get_class($_bizobject), $_bizobject->id, "' . $name . '");';
        $output .= '$_content->refresh();';
        $output .= '$_contentArray = $_content->getZoneContent();';
        $output .= 'if (is_array($_contentArray) && count($_contentArray) > 0) {';
        $output .= 'foreach($_contentArray as $_blockGuid => $_block) {';
        $output .= '$_blockId = $_block["name"] . "-" . $_blockGuid;';
        $output .= '$_blockTitle = $_blockClass = $_blockIndex = $_block["name"];';
        $output .= '$_smarty_tpl_vars = $this->_tpl_vars;';
        $output .= '$_widget = new $_block["name"]($this->widgetMode, $_block["settings"], $_bizobject, $_blockGuid, $this);';
        $output .= 'echo $_widget->display();';
        $output .= '$this->_tpl_vars = $_smarty_tpl_vars; unset($_smarty_tpl_vars); }';
        $output .= '}';

        return $output;
    }

    /**
     * Handles {loop} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     * => This code is inspired from smarty code (foreach)
     *
     * @return string output
     */
    function func_loop($tag_args, &$smarty)
    {
        $arg_list = $smarty->_compile_arg_list('block', 'loop', $smarty->_parse_attrs($tag_args), $cache);

        $output  = '';
        $output .= '$_loop_repeat = true;' . PHP_EOL;
        $output .= '$this->_tag_stack[] = array("loop", array(' . implode(',', $arg_list) . '));' . PHP_EOL;
        $output .= '$this->handle_loop($this->_tag_stack[count($this->_tag_stack)-1][1], null, $this, $_loop_repeat, false);'  . PHP_EOL;
        $output .= 'while ($_loop_repeat) {' . PHP_EOL;
        $output .= '    ob_start();' . PHP_EOL;

        return $output;
    }

    /**
     * Handles {exitloop} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     * => This code is inspired from smarty code (foreach)
     *
     * @return string output
     */
    function func_exitloop($tag_args, &$smarty)
    {
        $output  = '';
        $output .= '    $_loop_repeat = false; $_block_content = ob_get_contents(); ob_end_clean();' . PHP_EOL;
        $output .= '    echo $this->handle_loop($this->_tag_stack[count($this->_tag_stack)-1][1], $_block_content, $this, $_loop_repeat, true);' . PHP_EOL;
        $output .= '    break;' . PHP_EOL;

        return $output;
    }

    /**
     * Handles {/loop} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     * => keep comment for memory as this code is inspired from smarty code
     *    and smarty code is subject to evolve. will be helpful to locate
     *    into smarty code.
     *
     * @return string output
     */
    function func_endloop($tag_args, &$smarty)
    {
        $output  = '';
        $output .= '    $_loop_repeat = false; $_block_content = ob_get_contents(); ob_end_clean();' . PHP_EOL;
        $output .= '    echo $this->handle_loop($this->_tag_stack[count($this->_tag_stack)-1][1], $_block_content, $this, $_loop_repeat, false);' . PHP_EOL;
        $output .= '}' . PHP_EOL;
        $output .= 'array_pop($this->_tag_stack);' . PHP_EOL;

        return $output;
    }

    /**
     * This function is called during template execution ("func_loop", "func_exitloop" and
     * "func_endloop" methods write PHP code to call this function when appropriate)
     *
     * @param array   $tParams  Loop parameters
     * @param string  $content  Smarty content (unset when {loop} start)
     * @param Smarty  &$oSmarty Smarty object
     * @param bool &$repeat  true to repeat block
     * @param bool $bExit    true to exit loop
     *
     * @return string $content if is set else null or null on failure
     * @access private
     */
    function handle_loop($tParams, $content, &$oSmarty, &$repeat, $bExit)
    {
        if (!isset($content))
        {
            // Handle opening tag {loop}

            //Parse parameters
            $tablename = getArrayParameter($tParams, 'class', getArrayParameter($tParams, 'table', null));
            $assocname = getArrayParameter($tParams, 'name', $tablename);
            $objname   = getArrayParameter($tParams, 'object', 'o' . $tablename);
            $of        = getArrayParameter($tParams, 'of', null);
            $where     = getArrayParameter($tParams, 'where', null);
            $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
            $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
            $limit     = getArrayParameter($tParams, 'limit', -1);
            $to        = getArrayParameter($tParams, 'to', -1);
            if ($to != -1)
            {
                $limit = ($to - $from + 1);
            }

            $ofContext = $this->extractContext($of);

            if ($tablename == null)
            {
                $this->logger->logError('loop fatal error: [class] parameter is mandatory');

                // Exit loop!
                $repeat = false;
                return null;
            }

            // Check if there is another loop with same assocname
            if (isset($this->loopInfo[$assocname]) && $this->loopInfo[$assocname]['iteration'] != -1)
            {
                $this->logger->logError('loop fatal error: loop conflit: a loop with same [name] parameter has already started');

                // Exit loop!
                $repeat = false;
                return null;
            }

            // Enumerate bizobject and store it in current tag
            $this->logger->logVerbose('loop class=' . $tablename . ' offset=' . $from . ' limit=' . $limit);

            if (is_subclass_of($tablename, 'wcmObject'))
            {
                $bizobject = wcmObjectFactory::create($tablename);
            }
            else
            {
                $bizobject = new $tablename();
            }
            $assocarray = array();

            // Points to current tag values in the tag stack
            $i = count($this->_tag_stack) - 1;
            $this->_tag_stack[$i][1]['o'] = $bizobject;
            $this->_tag_stack[$i][1]['objname'] = $objname;
            $this->_tag_stack[$i][1]['assocname'] = $assocname;

            // Enumeration failed?
            if (!$bizobject->beginEnum($where, $order, $from, $limit, $ofContext))
            {
                $this->logger->logError('loop fatal error: enumerating ' . $tablename . ' failed: ' . $bizobject->getErrorMsg());
                $repeat = false;
                return null;
            }

            // Create loop info (even for empty loop)
            $this->createLoopInfo($assocname, $from, $bizobject->enumCount());

            // Empty loop?
            if (!$bizobject->nextEnum())
            {
                $this->clearLoopInfo($assocname);
                $repeat = false;
                return null;
            }

            // Expose bizobject and assocarray
            $this->assign_by_ref($objname, $bizobject);
            $this->assign($assocname, $bizobject->getAssocArray(false));
        }
        else
        {
            // Handle closing tag {/loop} or exiting tag {exitloop}

            // Retrieve enumerated bizobject
            $i = count($this->_tag_stack) - 1;
            $objname   = $this->_tag_stack[$i][1]['objname'];
            $assocname = $this->_tag_stack[$i][1]['assocname'];
            $bizobject = $this->_tag_stack[$i][1]['o'];

            if ($bExit || !$bizobject->nextEnum())
            {
                // Exitloop (or end of loop)
                $this->logger->logVerbose(($bExit) ? 'exitloop' : 'endloop');
                $bizobject->endEnum();
                $this->clear_assign($objname);
                $this->clear_assign($assocname);
                $this->clearLoopInfo($assocname);
                $repeat = false;
            }
            else
            {
                // Refresh assocarray
                $this->assign($assocname, $bizobject->getAssocArray(false));
                $this->updateLoopInfo($assocname);
                $repeat = true;
            }

            return $content;
        }
    }

    /**
     * Handles {search} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     *
     * @return string output
     * => This code is inspired from smarty code (foreach)
     */
    function func_search($tag_args, &$smarty)
    {
        $arg_list = $smarty->_compile_arg_list('block', 'search', $smarty->_parse_attrs($tag_args), $cache);

        $output  = '';
        $output .= '$_search_repeat = true;' . PHP_EOL;
        $output .= '$this->_tag_stack[] = array("search", array(' . implode(',', $arg_list) . '));' . PHP_EOL;
        $output .= '$this->handle_search($this->_tag_stack[count($this->_tag_stack)-1][1], null, $this, $_search_repeat, false);' . PHP_EOL;
        $output .= 'while ($_search_repeat) {' . PHP_EOL;
        $output .= '    ob_start();' . PHP_EOL;

        return $output;
    }

    /**
     * Handles {exitsearch} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     *
     * @return string output
     * => This code is inspired from smarty code (foreach)
     */
    function func_exitsearch($tag_args, &$smarty)
    {
        $output  = '';
        $output .= '    $_search_repeat = false; $_block_content = ob_get_contents(); ob_end_clean();' . PHP_EOL;
        $output .= '    echo $this->handle_search($this->_tag_stack[count($this->_tag_stack)-1][1], $_block_content, $this, $_search_repeat, true);' . PHP_EOL;
        $output .= '    break;' . PHP_EOL;

        return $output;
    }

    /**
     * Handles {/search} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     *
     * @return string output
     * => keep comment for memory as this code is inspired from smarty code
     *    and smarty code is subject to evolve. will be helpful to locate
     *    into smarty code.
     *
     */
    function func_endsearch($tag_args, &$smarty)
    {
        $output  = '';
        $output .= '    $_search_repeat = false; $_block_content = ob_get_contents(); ob_end_clean();' . PHP_EOL;
        $output .= '    echo $this->handle_search($this->_tag_stack[count($this->_tag_stack)-1][1], $_block_content, $this, $_search_repeat, false);' . PHP_EOL;
        $output .= '}' . PHP_EOL;
        $output .= 'array_pop($this->_tag_stack);' . PHP_EOL;

        return $output;
    }


    /**
     * This function is called during template execution ("func_search", "func_exitsearch" and
     * "func_endsearch" methods write PHP code to call this function when appropriate)
     *
     * @param array   $tParams  search parameters
     * @param string  $content  Smarty content (unset when {search} start)
     * @param Smarty  &$oSmarty Smarty object
     * @param bool &$repeat  true to repeat block
     * @param bool $bExit    true to exit search
     *
     * @return string $content if is set else null or null on failure
     *
     * @access private
     */
    function handle_search($tParams, $content, &$oSmarty, &$repeat, $bExit)
    {
        if (!isset($content))
        {
            // Handle opening tag {search}
            $config = wcmConfig::getInstance();

            //Parse parameters
            $query     = getArrayParameter($tParams, 'where', getArrayParameter($tParams, 'where', null));
            $assocname = getArrayParameter($tParams, 'name', getArrayParameter($tParams, 'name', 'bizobject'));
            $order     = getArrayParameter($tParams, 'orderby', getArrayParameter($tParams, 'order', null));
            $from      = getArrayParameter($tParams, 'offset', getArrayParameter($tParams, 'from', 0));
            $limit     = getArrayParameter($tParams, 'limit', -1);
            $to        = getArrayParameter($tParams, 'to', -1);
            $uid       = getArrayParameter($tParams, 'uid', 'search_' . uniqid());
            $engine    = getArrayParameter($tParams, 'engine', $config['wcm.plugins.search.default']);
            if ($to != -1)
            {
                $limit = ($to - $from + 1);
            }

            if ($query == null)
            {
                $this->logger->logError('search fatal error: [query] parameter is mandatory');

                // Exit search!
                $repeat = false;
                return null;
            }

            // Check if there is another search with same assocname
            if (isset($this->loopInfo[$assocname]) && $this->loopInfo[$assocname]['iteration'] != -1)
            {
                $this->logger->logError('search fatal error: search conflit: a search or loop with same [name] parameter has already started');

                // Exit search!
                $repeat = false;
                return null;
            }

            // Search bizobjects and store results in current tag
            $this->logger->logVerbose('search query=' . $query . ' order=' . $order . ' offset=' . $from . ' limit=' . $limit);
            $search = wcmSearch::getInstance($engine);

            // Total is equals to the min value between total and limit
            $total = min($search->search($uid, $query, $order), $limit);
            $this->logger->logVerbose('search : total iterations will be ' . $total);

            // Points to current tag values in the tag stack
            $i = count($this->_tag_stack) - 1;
            $this->_tag_stack[$i][1]['o'] = $search;
            $this->_tag_stack[$i][1]['uid'] = $uid;
            $this->_tag_stack[$i][1]['assocname'] = $assocname;

            // Get firstresult
            $rs = $search->getSearchResults($from, $limit, false);
            if (!$rs)
            {
                $this->logger->logError('search fatal error: retrieve search item #' . $from . ' failed');
                $repeat = false;
                return null;
            }

            $this->_tag_stack[$i][1]['resultSearch'] =& $rs;

            // Create search info (even for empty search)
            $this->createloopInfo($assocname, $from, $total);

            // Empty search?
            if ($total == 0)
            {
                $repeat = false;
                return null;
            }

            // Expose bizobject assocArray
            $newObj = array_shift($rs);
            $this->assign($assocname, $newObj->getAssocArray(false));
        }
        else
        {
            // Handle closing tag {/search} or exiting tag {exitsearch}

            // Retrieve enumerated bizobject
            $i = count($this->_tag_stack) - 1;
            $assocname   = $this->_tag_stack[$i][1]['assocname'];
            $uid = $this->_tag_stack[$i][1]['uid'];
            $search = $this->_tag_stack[$i][1]['o'];
            $rs =& $this->_tag_stack[$i][1]['resultSearch'];

            // Forced end of search {/search}?
            if (!$bExit)
            {
                // End of search loop?
                $bExit = $this->loopInfo[$assocname]['last'];
                if (!$bExit)
                {
                    // Retrieve next item
                    $this->updateloopInfo($assocname);
                    $offset = $this->loopInfo[$assocname]['current'];
                    $total = $this->loopInfo[$assocname]['total'];
                    $this->logger->logVerbose('search : retrieving item ' . $offset . ' of ' . $total);
                    $search->setSearchId($uid);
                    // $rs = $search->getSearchResults($offset, $offset, false);
                    // Error in search retrieval?
                    // $bExit = (!$rs || count($rs) == 0);
                    
                    $newObj = array_shift($rs);
                    $bExit = (!is_object($newObj));

                    if (!$bExit)
                    {
                        $this->assign($assocname, $newObj->getAssocArray(false));
                        // Refresh assoc array and repeat loop
                        // $this->assign($assocname, $rs[0]->getAssocArray(false));
                        $repeat = true;
                    }
                }
            }

            // Check if we have reach last item
            if ($bExit)
            {
                // Exitsearch (or end of search)
                $this->logger->logVerbose(($bExit) ? 'exitsearch' : 'endsearch');
                $this->clear_assign($assocname);
                $this->clearloopInfo($assocname);
                $repeat = false;

                // Clear memory
                unset($_SESSION['wcmSearch_' . $uid]);
                unset($search);
            }

            return $content;
        }
    }

    /**
     * Create a loop info structure
     *
     * @param string $loopname  Name of loop
     * @param int    $from      First index in loop
     * @param int    $total     Total number of rows to enumerate
     */
    private function createLoopInfo($loopname, $from, $total)
    {
        $this->loopInfo[$loopname] = array(
            'index' => 0,
            'iteration' => 1,
            'total' => $total,
            'first' => 1,
            'last' => ($total == 1) ? 1 : 0,
            'current' => $from );
    }

    /**
     * Update a loop info structure
     *
     * @param string $loopname  Name of loop
     */
    private function updateLoopInfo($loopname)
    {
        // Increate index, iteration and current
        $index = 1 + $this->loopInfo[$loopname]['index'];
        $iteration = 1 + $index;
        $current = 1 + $this->loopInfo[$loopname]['current'];

        $this->loopInfo[$loopname]['index'] = $index;
        $this->loopInfo[$loopname]['iteration'] = $iteration;
        $this->loopInfo[$loopname]['current'] = $current;
        $this->loopInfo[$loopname]['first'] = 0;
        $this->loopInfo[$loopname]['last'] = ($this->loopInfo[$loopname]['total'] == $iteration) ? 1 : 0;
    }

    /**
     * Clear a loop info structure
     *
     * @param string $loopname  Name of loop
     */
    private function clearLoopInfo($loopname)
    {
        // Mark loop as done (keep info for global $loop variable)
        $this->loopInfo[$loopname]['iteration'] = -1;
    }


    /**
     *  Handles {widget} tag
     * @param array $tag_args tag parameters
     * @param Smarty $smarty Smarty object
     * @return string html or null on failure
     */
    public function func_widget($params, &$smarty)
    {

        $required_params = array('type', 'guid', 'context');
        foreach ($required_params as $key)
        {
            if (empty($params[$key]))
            {
                $smarty->trigger_error("widget: missing '" . $key . "' parameter");
                return null;
            }
            else
            {
                $key = $params[$key];
            }
        }

        $blockid = $type . '-' . $guid;
        $designZone = new wcmDesignZone(get_class($context), $context->id, 'null');
        $designZone->refresh();
        $content = $designZone->getZoneContent();

        if (isset($content[$guid]))
        {
            $settings = $content[$guid]['settings'];
        }
        else
        {
            $settings = array();
        }
        $widget = new $type($smarty->widgetMode | wcmWidget::IS_FIXED, $settings, $context, $guid, $smarty);

        $html = $widget->display();

        if ($smarty->widgetMode & wcmWidget::VIEW_SETTINGS)
        {
            $html .= '<script type="text/javascript">Event.observe(window, \'load\', function () { portal.widgets["' . $blockid . '"] = new Widget("' . $blockid . '", portal, "null"); });</script>';
        }

        return $html;
    }

    /**
     * WCM generic function
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Smarty instance
     *
     * @return mixed Result of function
     */
    function func_wcm($params, &$smarty)
    {
        if (!isset($params['name']))
        {
            $smarty->trigger_error("wcm: missing 'name' parameter");
            return;
        }

        // Retrieve function name
        $funcName = $params['name'];
        unset($params['name']);

        // Check function existence
        $functions = new smartyFunctions;
        if (!method_exists($functions, $funcName))
        {
            $smarty->trigger_error("wcm: invalid function name '" . $funcName . "'");
            return;
        }

        // Execute function
        return call_user_func_array(array($functions, $funcName), array($params, $smarty));
    }

    /**
     * WCM friendly modifier plugin
     *
     * Type:     modifier<br>
     * Name:     wcm<br>
     *
     * @param string $source Source to be modified
     * @param string $modifier Name of modifier to execute
     * @param mixed  $extra Extra parameters
     *
     * @return mixed Modifier result
     */
    function mod_wcm()
    {
        $args = func_get_args();

        // Retrieve modifier name (shift twice, and unshift source)
        $source = array_shift($args);
        $modifier = array_shift($args);
        array_unshift($args, $source);

        // Execute modifier
        $modifiers = new smartyModifiers;
        if (!method_exists($modifiers, $modifier))
        {
            trigger_error('Invalid wcm modifier: ' . $modifier);
            return null;
        }
        return call_user_func_array(array($modifiers, $modifier), $args);
    }

    /**
     * Translate a constant
     * 
     * @param string $name Name of the constant
     * @return string value of the constant
     */
    function mod_constant($name)
    {
        return getConst($name);
    }
    
    /**
     * Smarty creates friendly link modifier plugin
     *
     * @param string $string String to convert
     * @return string the safe file name
     */
    function mod_safe_filename($string)
    {
        return safeFileName($string);
    }
}
