<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.designZone.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */


/**
 * The wcmDesignZone class
 * Used by the generator and defined by the drag and drop editor to save
 * the zone defined by the drag and drop layout.
 */

class wcmDesignZone implements wcmEnumerable
{
    /**
     * Unique identifier
     * @var int
     */
    public $id = 0;

    /**
     * Class of the source
     * @var string
     */
    public $sourceClass = null;

    /**
     * Id of the source
     * @var int
     */
    public $sourceId = 0;

    /**
     * Name of the zone
     * @var string
     */
    public $zoneName = null;

    /**
     * Content of the zone
     * @var string
     */
    public $zoneContent = null;

    /**
     * Zone content
     * @var string
     */
    private $zoneContentArray = array();

    /**
     * Enumeration query
     * @var string
     */
    protected $enumQuery = '';

    /**
     * Enumeration result
     * @var wcmAbstractResultSet
     */
    protected $enumResultSet  = null;

    /**
     * Enumeration total count
     * @var int
     */
    protected $enumTotalCount = null;

    /**
     * Database object
     * @var wcmDatabase
     */
    protected $database = null;

    /**
     * Table name
     * @var string
     */
    protected $tableName = '#__design_zone';


    /**
     * Constructor
     *
     * @param string $sourceClass Source name of the zone
     * @param int $sourceId Source id of the zone
     * @param int $zoneName Name of the zone
     * @param array $zoneContentArray Content of the zone
     */
    public function __construct($sourceClass = null, $sourceId = 0, $zoneName = null, $zoneContentArray = array())
    {
        $this->sourceClass = $sourceClass;
        $this->sourceId = $sourceId;
        $this->zoneName = $zoneName;
        $this->zoneContentArray = $zoneContentArray;

        $this->database = wcmProject::getInstance()->database;
    }

    /**
     * This methods return the table name where to store designZones.
     * This is used by the wcmDatabase.storeSysObject() method
     *
     * @return string The table name where design zones are stored
     */
    public function getTableName()
    {
        return $this->tableName;
    }

    /**
     * Load or refresh a zone content
     *
     *	@return	array array of the current zone content
     */
    public function refresh()
    {
        $project    = wcmProject::getInstance();

        $params = array($this->sourceClass, $this->sourceId, $this->zoneName);
        $query  = 'SELECT zoneContent FROM ' . $this->tableName . ' WHERE sourceClass=? AND sourceId=? AND zoneName=?';

        $this->zoneContent = $project->database->executeScalar($query, $params);
        $this->zoneContentArray = unserialize($this->zoneContent);

        return $this->zoneContentArray;
    }

    /**
     *	Save the current zone content in mem and in the database if require
     */
    public function save()
    {
        $this->zoneContent = serialize($this->zoneContentArray);

        $project = wcmProject::getInstance();
        $this->delete();
        $project->database->storeSysObject($this);
    }


    /**
     *	Replace the current zone content by the array given in parameters
     *
     *	@param array $widgets new widget array
     */
    public function setWidgets($widgets)
    {
        $this->zoneContentArray = $widgets;
    }

    /**
     *	Set a widget of the current zone content
     *
     *	@param string $widgetName Name of the widget
     *  @param int $widgetGuid ID of the widget
     *	@param array $settings	Settings apply to the widget
     */
    public function setWidget($widgetName, $widgetGuid, $settings = array())
    {
        $this->zoneContentArray[$widgetGuid] = array(
             'name' => $widgetName,
             'settings' => $settings
        );
    }

    /**
     *	Add a widget at the bottom(end) of the current zone content
     *
     *	@param string $widgetName Name of the widget
     *  @param int $widgetGuid ID of the widget
     *	@param array $settings	Settings apply to the widget
     */
    public function addWidget($widgetName, $widgetGuid, $settings = array())
    {
        // assure that widget is going to be add at bottom(end) of the current zone content
        while (array_key_exists($widgetGuid, $this->zoneContentArray))
        {
            $widgetGuid++;
        }
        
        $this->setWidget($widgetName, $widgetGuid, $settings);
    }

    /**
     * return the getContent Array
     * @return array content array
     */
    public function getZoneContent()
    {
        return $this->zoneContentArray;
    }

    /**
     * Set zone content
     * @param array $array Zone content array
     *
     */
    public function setZoneContent($array)
    {
        $this->zoneContentArray = $array;
    }


    /**
     *	Remove a widget from the content zone at a given position
     *
     *	@param int $position Postion of the current widget (start at 0)
     */
    public function removeWidget($position)
    {
        while (array_key_exists($position + 1, $this->zoneContentArray))
        {
            $this->zoneContentArray[$position] = $this->zoneContentArray[$position + 1];
            $position++;
        }

        unset($this->zoneContentArray[$position]);
    }

    /**
     * Delete the current zone in memory and database
     */
    public function	delete()
    {
        $project = wcmProject::getInstance();

        $params = array($this->sourceClass, $this->sourceId, $this->zoneName);
        $query = 'DELETE FROM ' . $this->tableName . ' WHERE sourceClass=? AND sourceId=? AND zoneName=?';

        $project->database->executeStatement($query, $params);
    }

    /**
     * Return the bizObject generated by the sourceClass and the sourceId
     *
     * @return bizobject Bizobject from sourceClass/sourceId
     */
    public function getBizObject()
    {
        $project = wcmProject::getInstance();
        $className = $this->sourceClass;

        return $className($project, $this->sourceId);
    }

    /**
     * Initialize and starts a new enumeration on current wcmSysobject
     *
     * @param string $where     Optional where clause
     * @param string $orderby   Optional order clause
     * @param int    $offset    Optional offset of first row (default is zero)
     * @param int    $limit     Optional maximum number of returned rows (default is zero, means return all rows)
     * @param array  $of        Optional assoc Array with foreign constraint (key=className, value=id)
     *
     * @return bool  True on success, false on failure
     */
    public function beginEnum($where = null, $orderby = null, $offset = 0, $limit = 0, $of = null)
    {
        // Ensure to free previous enumeration
        $this->endEnum();

        // First of all, compute total number of items
        if ($where == null || (strcmp($where, $this->enumQuery) != 0))
        {
            // Remember actual query
            $this->enumQuery = $where;

            // Compute number of wcmSysObject matching the query
            $sql = 'SELECT COUNT(*) FROM ' . $this->tableName;
            if ($where != null && $where != '')
            {
                $sql .= ' WHERE ' . $where;
            }

            $this->enumTotalCount = $this->database->executeScalar($sql);
        }

        // Prepare sql query
        $sql = 'SELECT * FROM ' . $this->tableName;
        if ($where !== null && $where != '')
        {
            $sql .= ' WHERE ' . $where;
        }
        if ($orderby != null)
        {
            $sql .= ' ORDER BY ' . $orderby;
        }

        // Load resultset
        $this->enumResultSet = $this->database->executeQuery($sql, null, $offset, $limit);
        if ($this->enumResultSet == null)
        {
            $this->lastErrorMsg = $this->database->getErrorMsg();
            return false;
        }

        return true;
    }

    /**
     * Moves to the next item of current enumeration
     *
     * @return bool True if enumeration has succeed, false otherwise
     */
    public function nextEnum()
    {
        if ($this->enumResultSet == null)
        {
            return false;
        }

        // Erase existing property
        $properties = getPublicProperties($this);
        foreach ($properties as $property => $value)
        {
            $this->$property = null;
        }

        if (!$this->enumResultSet->next())
        {
            // We have reach the end of the resultset
            $this->endEnum();
            return false;
        }

        // Bind current object to current resultset row
        bindArrayToObject($this->enumResultSet->current(), $this);

        $this->zoneContentArray = unserialize($this->zoneContent);

        return $this;
    }

    /**
     * Stops current enumeration
     */
    public function endEnum()
    {
        // Free resultset
        $this->enumResultSet = null;
        $this->enumQuery = null;
        $this->enumTotalCount = 0;
    }

    /**
     * Returns the total number of objects which can be enumerated (regardless of the
     * offset and limit parameters passed in the {@link beginEnum()} method)
     *
     * @return int Total number of enumerable objects
     */
    public function enumCount()
    {
        return $this->enumTotalCount;
    }
}

