<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.widget.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 *  Widget
 *
 */
abstract class wcmWidget
{
    /**
     * Defines a widget to be viewed in a frame
     */
    const VIEW_FRAME = 1;

    /**
     * Defines a widget view settings
     */
    const VIEW_SETTINGS = 2;

    /**
     * Defines a widget to be viewed in content
     */
    const VIEW_CONTENT = 4;

    /**
     * Defines a fixed widget
     */
    const IS_FIXED = 8;

    /**
     * Defines a view all on a widget
     */
    const VIEW_ALL = 7;

    /**
     * Widget config
     * @var array
     */
    private static $config = null;

    /**
     * Widget's guid
     * @var int
     */
    public $guid = 0;

    /**
     * widget's display mode
     * @var int
     */
    public $mode = null;

    /**
     * widget's context
     * @var mixed
     */
    public $context = null;

    /**
     * widget's settings
     * @var array
     */
    public $settings = array();

    /**
     * Smarty instance
     * @var Smarty
     */
    public $smarty = null;

    /**
     * Constructor
     *
     *  @param int         $mode       Display mode
     *  @param array|null  $settings   Settings array
     *  @param mixed|null  $context    Default context
     *  @param  int         $guid       Widget's guid
     *  @param  Smarty      $smarty     Smarty instance
     */
    public function __construct($mode, $settings = null, $context = null, $guid = false, $smarty = null)
    {
        $this->guid = ($guid === false) ? rand(0, 999) : $guid;
        $this->mode = $mode;
        $this->init($settings, $context);
        $this->smarty = $smarty;
    }

    /**
     * Return the label of the box in designMode
     *
     * @return string   Label to display
     */
    public function getLabel()
    {
        return get_class($this);
    }

    /**
     *  Initialise the widget's settings and context
     *
     *  @param array|null   $settings   Settings array
     *  @param mixed|null   $context    Default context
     */
    public function init($settings = null, $context = null)
    {
        $this->initSettings($settings);
        $this->initContext($context);
    }

    /**
     * Initialise the widget's context
     *
     * @param mixed|null    $context    Default context
     */
    public function initContext($context = null)
    {
        $this->context = $context;
    }

    /**
     * Initialise the widget's settings
     *
     *  @params array|null  $settings   Settings array
     */
    public function initSettings($settings = null)
    {
        $this->settings = $settings;
    }

    /**
     * Save the data of the widget
     *
     * @param  array   $dataSet    Array of data to save
     */
    public function saveData($dataSet)
    {
        $modifiedObjects = array();

        foreach ($dataSet as $key => $value)
        {
            $identifiers = explode('-', $key);

            if (count($identifiers) == 3 && class_exists($identifiers[0], false))
            {
                $objectKey = $identifiers[0] . '-' . $identifiers[1];
                $propertie = $identifiers[2];

                if (!array_key_exists($objectKey, $modifiedObjects))
                {
                    $bizobject = $modifiedObjects[$objectKey] = wcmObjectFactory::create($identifiers[0], $identifiers[1]);
                }
                else
                {
                    $bizobject = $modifiedObjects[$objectKey];
                }

                $bizobject->$propertie = $value;
            }
            else
            {
                $method = array_shift($identifiers);
                if (method_exists($this, $method))
                {
                    $this->$method($identifiers, $value);
                }
            }
        }
        foreach ($modifiedObjects as $bizobject)
        {
            $bizobject->save();
        }
    }

    /**
     *  Display the widget in preview mode
     *
     *  @param array|null  $settings   Settings array
     *  @return string      HTML of the widget
     */
    public function preview($settings = null)
    {
        $this->mergeSettings($settings);
        $this->mode &= ~wcmWidget::VIEW_SETTINGS;
        return $this->render();
    }

    /**
     *  Display the widget in edit mode
     *
     *  @params array|null  $settings   Settings array
     *  @return string      HTML of the widget
     */
    public function edit($settings = null)
    {
        $this->mergeSettings($settings);
        $this->mode |= wcmWidget::VIEW_SETTINGS;
        return $this->render();
    }


    /**
     *  Merge the settings and the given array
     *
     *  @params array   $settings   settings array
     */
    public function mergeSettings($settings)
    {
        if (is_array($settings))
        {
            array_merge($this->settings, $settings);
        }
    }

    /**
     *  Function which render the widget
     *
     *  @params string|null     $templateFile   Path to the template file, if null the default template will be take
     *  @return     HTML string of the widget
     */
    public function render($templateFile = null)
    {
        if ($templateFile == null)
        {
            $templateFile = self::getPath(get_class($this)) . 'widget.tpl';
        }
        $generator = ($this->smarty!=null) ? $this->smarty : new wcmTemplateGenerator(null, false, $this->mode);

        $template = new wcmTemplate();
        $template->refresh($templateFile);
        return $generator->executeTemplate($template, array('owidget' => $this, 'widget' => $this->getAssocArray(false)));
    }

    /**
     *  Function which return the innerHTML of the settings form
     *
     *  @return string     HTML string or false if no settings
     */
    public function displaySettings()
    {
        return false;
    }

    /**
     * Returns an associative array containing public properties and their values
     * The following keys and values are added to the array:
     *      'acontext' => an associative array with actual context->getAssocArray(false)
     *
     * @param bool $toXML TRUE when called in the 'toXML' context
     * @return wcmObjectAssocArray Assoc array
     */
    public function getAssocArray($toXML)
    {
        return new wcmObjectAssocArray($this, $toXML);
    }

    /**
     * get the assoc array of the context
     *
     * @param bool $toXML TRUE when called in the 'toXML' context
     * @return wcmObjectAssocArray Assoc array
     */
    public function getAssoc_acontext($toXML = false)
    {
        return $this->context->getAssocArray($toXML);
    }

    /**
     *  Function which display the HTML of the widget depending on the widget's mode
     *
     *  @return string HTML of the widget
     */
    final public function display()
    {
        $className = get_class($this);
        $widgetId = $className . '-' . $this->guid;

        $html = '';

        if ($this->mode & wcmWidget::VIEW_FRAME)
        {
            $html .= '<div class="block" id="' . $widgetId . '">';
        }

        if ($this->mode & wcmWidget::VIEW_SETTINGS)
        {
            $settings = $this->displaySettings();

            $html .= '<div class="handle">';

            if (!($this->mode & wcmWidget::IS_FIXED))
            {
                $html .= '    <div class="block-remove" title="' . _BIZ_REMOVE . '" onclick="$(\'' . $widgetId . '\').remove();"><span>x</span></div>';
            }

            if ($this->mode & wcmWidget::VIEW_CONTENT)
            {
                $html .= '<div class="block-savedata" title="' . _BIZ_SAVE_WIDGETS . '" onclick="javascript:portal.getWidget(\'' . $widgetId . '\').saveData();"><span>Save data</span></div>';
                $html .= '<div class="block-refresh" title="' . _BIZ_REFRESH_WIDGET_BUGS . '" onclick="javascript:portal.getWidget(\'' . $widgetId . '\').refreshData();"><span>Refresh Data</span></div>';
            }

            if ($settings)
            {
                $html .= '    <div class="block-editsettings" title="' . _BIZ_EDIT_SETTINGS . '" onclick="javascript:portal.getWidget(\'' . $widgetId . '\').displaySettings();"><span>Edit</span></div>';
            }

            $html .= $this->getLabel();
            $html .= '</div>';

            if ($settings)
            {
                $html .= '<div class="settings">';
                $html .= '<div id="' . $widgetId . '-settings">' . $settings . '</div>';
                $html .= '<div class="settingsToolbar">';
                $html .= '<a href="#" onclick="javascript:return portal.getWidget(\'' . $widgetId . '\').saveSettings();">' . _BIZ_SAVE . '</a>';

                if ($this->mode & wcmWidget::VIEW_CONTENT)
                {
                    $html .= '<a href="#" onclick="javascript:return portal.getWidget(\'' . $widgetId . '\').applySettings();">' . _BIZ_APPLY . '</a>';
                }

                $html .= '<a href="#" onclick="javascript:return portal.getWidget(\'' . $widgetId . '\').cancelSettings();">' . _BIZ_CANCEL . '</a>';
                $html .= '</div>';
                $html .= '</div>';
            }
        }
        elseif ($this->mode & wcmWidget::VIEW_FRAME)
        {
            $html .= '<div>' . $this->getLabel() . '</div>';
        }

        if ($this->mode & wcmWidget::VIEW_SETTINGS)
        {
            $html .= '<div class="content">' . PHP_EOL;
        }

        if ($this->mode & wcmWidget::VIEW_FRAME && $this->mode & wcmWidget::VIEW_CONTENT)
        {
            $html .= '<form id="' . $className . '-' . $this->guid . '-context">';
        }

        if ($this->mode & wcmWidget::VIEW_CONTENT)
        {
            $html .= $this->render();
        }

        if ($this->mode & wcmWidget::VIEW_FRAME && $this->mode & wcmWidget::VIEW_CONTENT)
        {
            $html .= '</form>';
        }

        if ($this->mode & wcmWidget::VIEW_SETTINGS)
        {
            $html .= '</div>';
        }

        if ($this->mode & wcmWidget::VIEW_FRAME)
        {
            $html .= '</div>';
        }

        return $html;
    }



    /**
     *  Get the path of a given object
     *  @param string $widgetName Widget name
     *  @return string Path to widget, or false on error
     */
    public static function getPath($widgetName)
    {
        static $pathArray = null;
        if ($pathArray == null)
        {
            $project = wcmProject::getInstance();
            $pathArray = array();
            foreach ($project->layout->loadWidgetConfig() as $widget)
            {
                $pathArray[(string)($widget->class)] = $widget->path . DIRECTORY_SEPARATOR;
            }
        }

        if (array_key_exists($widgetName, $pathArray))
        {
            return $pathArray[$widgetName];
        }
        else
        {
            return false;
        }
    }

}
