<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.ldap.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Default WCM Authentication plugin
 *
 * This authentication plugin takes a username and password
 * as credentials and checks them against the wcm_user database
 * table.
 *
 * The credentials must be in this format:
 * array(
 *     'username' => 'Your Username',
 *     'password' => 'Your Password
 * );
 *
 */
class wcmAuthenticateLDAP implements wcmIAuthPlugin
{

    /**
     * User ID
     * @var	int
     */
    private $user_id;

    /**
     * Error message
     * @var string
     */
    private $errorMsg;

    /**
     * This is expected to be an array containing the following:
     *
     * key: 'host'
     * This contains a full URL of the ldap host
     * eg: ldap://example.com or ldaps://example.com
     *
     * key: 'port'
     * Port number for LDAP
     *
     * key: 'startTLS'
     * Flag to start using TLS on a regular LDAP connection or not.
     *
     * key: 'baseDN'
     * This contains the base DN to search for entries in
     *
     * key: 'usernameParam'
     * The name of the parameter used to store user names
     * Usually cn on openLDAP or samaccountname on ActiveDirectory
     *
     * key: 'createUser'
     * If set to true, then if the ldap user doesn't exist in
     * WCM, it will be automatically created by this plugin. If
     * false, the user won't be able to login
     *
     * @var array
     */
    private $config = array(
            'host' => 'ldap://localhost',
            'port' => null,
            'startTLS' => false,
            'baseDN' => null,
            'usernameParam' => 'cn',
            'createUser' => true,
            'searchFirst' => false,
            'usernamePrefix' => null
            );

    /**
     * Group mapping
     * @var array
     */
    private $groupMapping = array();

    /**
     * Profile mapping
     * @var array
     */
    private $profileMapping = array();

    /**
     * Logger
     *
     * @var wcmLogger
     *
     */
    private $logger;

    /**
     * LDAP Connection
     *
     * @var resource
     */
    private $conn;

    /**
     * Constructor
     */
    public function __construct()
    {
        $config = simplexml_load_file(WCM_BACKOFFICE_CONF_DIR . '/plugins/ldap.xml');
        $this->config = object2Array($config);

        if (isset($this->config['maps']['groups']['param']))
        {
            $this->groupMapping = $this->config['maps']['groups']['param'];
        }
        if (isset($this->config['maps']['profile']['param']))
        {
            $this->profileMapping = $this->config['maps']['profile']['param'];
        }
        unset($this->config['maps']);
        $this->logger = wcmProject::getInstance()->logger;
    }

    /**
     * Get config
     *
     * @return array Configuration array
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Get maps
     *
     * @return array Maps array
     */
    public function getMaps()
    {
        return $this->maps;
    }

    /**
     * Connect to LDAP server
     * @return bool True on success, false on failure
     */
    private function ldapConnect()
    {
        $conn = ldap_connect($this->config['host'], $this->config['port']);
        if (!$conn)
        {
            return false;
        }
        else
        {
            $this->conn = $conn;
            return true;
        }
    }

    /**
     * Get user domain name
     *
     * @param string $argUser Username
     * @param string $argPassword Password
     * @return string Domain name, false on error
     */
    private function getUserDN($argUser, $argPassword)
    {
        $usernameParam = $this->config['usernameParam'];
        $usernamePrefix = $this->config['stripDomainPrefix'];

        $baseDN = $this->config['baseDN'];

        if (!ldap_set_option($this->conn, LDAP_OPT_PROTOCOL_VERSION, 3))
        {
            $this->logger->logError('WCM only supports LDAP Protocol v3.');
            return false;
        }
        ldap_set_option($this->conn, LDAP_OPT_REFERRALS, 0);


        // Do we search first or auth first
        if (!$this->config['searchFirst'])
        {
            // Leave the @ symbol, it's necessary here
            if (!@ldap_bind($this->conn, $argUser, $argPassword))
            {
                $this->logger->logWarning('Could not authenticate ' . $argUser . ' against directory service');
                $this->logger->logWarning('LDAP ERROR: ' . ldap_error($this->conn));
                return false;
            }

            // Here we have to strip the prefix
            if ($this->config['stripDomainPrefix'])
            {
                $argUser = array_pop(explode('\\', $argUser));
            }

            $res = ldap_search($this->conn, $this->config['baseDN'], $this->config['usernameParam'] . '=' . $argUser);
            if (!$res)
            {
                $this->logger->logError('User authenticated, but searching for the user DN failed.');
                return false;
            }

            $user = ldap_get_entries($this->conn, $res);
            if (0 >= $user['count'])
            {
                $this->logger->logError('User authenticated, but searching for the user DN returned no results.');
                return false;
            }

            $dn = $user[0]['dn'];
            return $dn;
        }
        else
        {
            // Search for user
            if (!ldap_bind($this->conn))
            {
                $this->logger->logError('Could not do anonymous bind to LDAP server');
                return false;
            }

            $searchString = $usernameParam . '=' . $argUser;

            $results = ldap_search($this->conn, $baseDN, $searchString);
            $entries = ldap_get_entries($this->conn, $results);

            if (0 < $entries['count'])
            {
                // Make sure user can authenticate now
                $dn = $entries[0]['dn'];
                if (ldap_bind($this->conn, $dn, $argPassword))
                {
                    return $dn;
                }
                else
                {
                    return false;
                }
            }
        }
    }

    /**
     * Explode a domain name
     *
     * @param string $argDN Domain name to explode
     * @return array Array of domain name parts
     */
    public function explodeDN($argDN)
    {
        $parts = ldap_explode_dn($argDN, false);
        unset($parts['count']);
        foreach ($parts as $part)
        {
            $separatedPart = explode('=', $part);
            $partsList[$separatedPart[0]] = $separatedPart[1];
        }
        return $partsList;
    }

    public function array_isearch($argStr, Array $argArray)
    {
        foreach ($argArray as $k => $v)
        {
            if (strcasecmp($argStr, $v) == 0)
            {
                return $k;
            }
        }
        return false;
    }

    /**
     * Authenticate credentials
     *
     * @param array $argCredentials Array of credentials: username, password
     * @return bool True on sucess, false on failure
     */
    public function authenticate(Array $argCredentials)
    {
        if (!$this->ldapConnect())
        {
            $this->logger->logError('Could not connect to LDAP host!');
            return false;
        }

        $dn = $this->getUserDN($argCredentials['username'], $argCredentials['password']);

        if (!$dn)
        {
            return false;
        }

        /**
         * try to find a matching user_id based on username
         * if the user doesnt exist, create it
         */
        $query = 'SELECT id, password FROM wcm_user WHERE login=:username';
        $db = wcmProject::getInstance()->database;
        $user = $db->executeQuery($query, array('username' => $argCredentials['username']));
        $user->next();

        $row = $user->current();
        if ($row)
        {
            if ($row['password'] != md5($argCredentials['password']))
            {
                // Password changed on the LDAP server, change it in the WCM too
                $user = new wcmUser();
                $user->refresh($row['id']);
                $user->password = md5($argCredentials['password']);
                if ($user->save())
                {
                    $this->user_id = $row['id'];
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                $this->user_id = $row['id'];
                return true;
            }
        }
        else
        {
            if (!$this->config['createUser'])
            {
                return false;
            }

            /**
             * Determine groups the user can belong to, and  map different ldap settings
             * to different properties of wcmUser
             * We do this by getting a list of IDs from the database based on group names.
             */
            $dnParts = ldap_explode_dn($dn, false);
            unset($dnParts['count']);

            $userGroups = array();

            if (is_array($this->groupMapping) && count($this->groupMapping) > 0)
            {
                reset($this->groupMapping);
                if (key($this->groupMapping) === 0)
                {
                    foreach ($this->groupMapping as $map)
                    {
                        if (false !== $this->array_isearch($map['paramName'], $dnParts))
                        {
                            $allowedGroups = explode(',', $map['groups']);
                            $userGroups = array_merge($allowedGroups, $userGroups);
                        }
                    }
                }
                else
                {
                    if (false !== $this->array_isearch($this->groupMapping['paramName'], $dnParts))
                    {
                        $allowedGroups = explode(',', $this->groupMapping['groups']);
                        $userGroups = array_merge($allowedGroups, $userGroups);
                    }
                }

                // Create the group if it doesn't exist
                foreach ($userGroups as $groupName)
                {
                    $query = 'SELECT id FROM wcm_group WHERE name=:groupName';
                    $id = $db->executeScalar($query, array('groupName' => $groupName));
                    if (!$id)
                    {
                        $g = new wcmGroup();
                        $g->name = $groupName;
                        $g->save();
                        $id = $g->id;
                    }

                    $groupIDlist[] = $id;
                }
            }

            // Map profile parameters
            reset($this->profileMapping);
            if (key($this->profileMapping) === 0)
            {
                foreach ($this->profileMapping as $map)
                {
                    $profileMaps[$map['paramName']] = $map['userProperty'];
                }
            }
            elseif (!is_array($this->profileMapping))
            {
                $profileMaps[$this->profileMapping['paramName']] = $this->profileMapping['userProperty'];
            }

            if ($this->config['stripDomainPrefix'])
            {
                $username = ($this->config['stripDomainPrefix']) ? array_pop(explode('\\', $argCredentials['username'])) : $argCredentials['username'];
            }

            // Mandatory fields.
            $userArray['login'] = $argCredentials['username'];
            $userArray['password'] = $argCredentials['password'];
            $userArray['email'] = 'change@me.com';

            // If there is any mapping to do:
            if (isset($profileMaps))
            {

                $filter = '(' . $this->config['usernameParam'] . '=' . $username . ')';

                ldap_bind($this->conn, $dn, $argCredentials['password']);
                $res = ldap_search($this->conn, $this->config['baseDN'], $filter, array_keys($profileMaps));
                $entries = array_pop(ldap_get_entries($this->conn, $res));

                // TODO wcmUser forces bind by an array, dunno why, won't change it for now
                foreach ($profileMaps as $ldapParam => $userParam)
                {
                    $userArray[$userParam] = $entries[$ldapParam][0];
                }
            }

            $user = new wcmUser;
            if (isset($groupIDlist))
            {
                $user->setGroups($groupIDlist);
            }
            $user->save($userArray, true);

            if ($user->getErrorMsg())
            {
                $this->logger->logError('LDAP User Creation Error: ' . $user->getErrorMsg());
                return false;
            }

            $this->user_id = $user->id;
            return true;
        }
        return false;

    }

    /**
     * Get last reported error message
     *
     * @return string Error message
     */
    public function getErrorMsg()
    {
        return $this->errorMsg;
    }

    /**
     * Get the WCM User ID that matches the supplied credentials
     *
     * @return int User ID
     */
    public function getWcmUser()
    {
        return $this->user_id;
    }

    /**
     * Ouptut login form
     */
    static public function loginForm()
    {
        wcmGUI::openFieldset(_SIGN_IN, array('id' => 'usernamePassword'));
        wcmGUI::renderTextField('username', getArrayParameter($_REQUEST, 'username', null), _USERNAME);
        wcmGUI::renderPasswordField('password', null, _PASSWORD);
        wcmGUI::closeFieldset();
        wcmGUI::closeFieldset();
    }
}
