<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.openid.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Authentication plugin to use OpenID
 *
 * The plugin pulls a few tricks in order to work. First,
 * OpenID works in a two step process for verifying credentials. The
 * first step takes the user out of WCM and brings them to the Open ID
 * provider's login form. Afterwards, the user is returned to WCM
 * with the action still being login.
 *
 * In order to do this, the plugin will spit out a form and automatically
 * submit it to url provided by the user, and the authenticate() method will return
 * false.
 *
 * When the user returns from the provider, the information returned from the provider
 * is verified by the authenticate method and in this case, it might return
 * true if the information is correct.
 *
 * The plugin maps the open ID url to a user_id in the system table wcm_openid
 *
 * CREATE TABLE wcm_openid (
 *     id int(11) NOT NULL auto_increment,
 *     user_id int(11) NOT NULL,
 *     openid varchar(255) NOT NULL,
 *     PRIMARY KEY  (id)
 * );
 *
 * This needs to be put into the WCM system database.
 *
 * The plugin relies on php-openid version 2.1.2 or higher available
 * at http://openidenabled.com/php-openid/
 *
 * It expects this API to be in wcmRoot/includes/php-openid
 *
 */
class wcmAuthenticateOpenID implements wcmIAuthPlugin
{
    /**
     * Open ID resource key name
     */
    const ID_KEYNAME = 'openIDresource';

    /**
     * Open ID verify key name
     */
    const REDIRECT_KEYNAME = 'openIDverify';

    /**
     * Open ID action key name
     */
    const ACTION_KEYNAME = 'openIDaction';

    /**
     * Login action key name
     */
    const ACTION_LOGIN = 'login';

    /**
     * Verify action key name
     */
    const ACTION_VERIFY = 'verify';

    /**
     * Last returned error message
     *
     * @var string
     */
    private $lastErrorMsg;

    /**
     * The verified open ID
     *
     * @var string
     */
    private $verifiedOpenID;

    /**
     * Constructor
     *
     * The constructor loads necessary APIs this plugin needs.
     *
     */
    public function __construct()
    {
        // TODO This should be configurable
        $newdir = WCM_INCLUDES_DIR . '/php-openid/';
        ini_set('include_path', ini_get('include_path') . PATH_SEPARATOR . $newdir);

        include_once('Auth/OpenID.php');
        include_once('Auth/OpenID/Consumer.php');
        include_once('Auth/OpenID/FileStore.php');
        include_once('Auth/OpenID/SReg.php');
    }

    /**
     * Output the login form
     *
     */
    static public function loginForm()
    {
        wcmGUI::openFieldset(_SIGN_IN, array('id' => 'wcmopenid'));
        wcmGUI::renderTextField(self::ID_KEYNAME, getArrayParameter($_REQUEST, self::ID_KEYNAME, null), _OPENID_RESOURCE_URI);
        wcmGUI::renderHiddenField(self::ACTION_KEYNAME, self::ACTION_LOGIN);
        wcmGUI::closeFieldset();
    }

    /**
     * Authenticate credentials
     *
     * OpenID is a two step process. The first step is to login using the OpenID
     * provider's website, then to verify the information returned by the OpenID provider.
     *
     * On the first step, this method will always return false. Instead it
     * generates a form and submits it. This form transmits necessary information
     * to the OpenID provider.
     *
     * @param array $argCreds Array of credentials
     * @return bool True on success, false on failure
     */
    public function authenticate(Array $argCreds)
    {
        // OpenID requires two stage process
        // One is to login
        // Two is to verify the login

        switch ($argCreds[self::ACTION_KEYNAME])
        {
            case (self::ACTION_LOGIN) :
                if ($this->login($argCreds))
                {
                    // We die here because the login method will create a form
                    // that gets automagically submitted to the open id provider
                    // TODO: we may replace die by an exception. need some investigation.
                    die;
                }
                else
                {
                    return false;
                }
                break;
            case self::ACTION_VERIFY:
                return $this->verify($argCreds);
                break;
        }
    }

    /**
     * Get the last generated error message
     *
     * @return string Error message
     */
    public function getErrorMsg()
    {
        return $this->lastErrorMsg;
    }

    /**
     * Return the WCM User ID thats associated with the verified
     * OpenID url.
     *
     * @return int User ID
     */
    public function getWcmUser()
    {
        $query = 'SELECT user_id FROM wcm_openid WHERE openid=:openid';
        $db = wcmProject::getInstance()->database;
        $user_id = $db->executeScalar($query, array('openid' => $this->verifiedOpenID));
        return $user_id;
    }

    /**
     * Returns the current scheme of the URL (http or https)
     *
     * @return string Scheme
     */
    public function getScheme()
    {
        $scheme = 'http';
        if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on')
        {
            $scheme .= 's';
        }
        return $scheme;
    }

    /**
     * The trusted root for OpenID is the URL accessed by the user at login time
     *
     * @return string Trusted root URL
     */
    public function getTrustRoot()
    {
        return sprintf('%s://%s:%s%s/', $this->getScheme(), $_SERVER['SERVER_NAME'], $_SERVER['SERVER_PORT'], dirname($_SERVER['PHP_SELF']));
    }

    /**
     * Perform the login step
     *
     * @param array $argCreds Credentials
     * @return bool True on success, false on failure
     */
    private function login(Array $argCreds)
    {
        $oid = new Auth_OpenID_Consumer(new Auth_OpenID_FileStore(sys_get_temp_dir()));
        $authRequest = $oid->begin($argCreds[self::ID_KEYNAME]);
        if (!$authRequest)
        {
            return false;
        }

        $root = $this->getTrustRoot();
        $verifyUrl = $root . '?' . self::ACTION_KEYNAME . '=' . self::ACTION_VERIFY . '&_wcmAction=processLogin&_authPlugin=openid';
        $result = $authRequest->htmlMarkup($this->getTrustRoot(), $verifyUrl, false, array('id' => 'form-id'));
        print $result;

        return true;
    }

    /**
     * Preform the verification step
     *
     * Returns true if the data is verified, false otherwise.
     *
     * @param array $argCreds Credentials
     * @return bool
     */
    private function verify(Array $argCreds)
    {
        foreach ($argCreds as $key => $value)
        {
            if ('openid' == substr($key, 0, 6))
            {
                $pos = strpos($key, '_');
                $oldkey = $key;
                $key{$pos} = '.';
                unset($argCreds[$oldkey]);
                $argCreds[$key] = $value;
            }
        }
        // Remove useless element
        unset($argCreds['wcm4']);

        $consumer = new Auth_OpenID_Consumer(new Auth_OpenID_FileStore(sys_get_temp_dir()));

        $result = $consumer->complete($this->getTrustRoot());
        if ($result->status == 'success')
        {
            $this->verifiedOpenID = $result->endpoint->claimed_id;
            return true;
        }
        else
        {
            // TODO Log the error message here
            return false;
        }
    }
}
