<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.sysobject.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The System class is the parent abstract class
 * of all WCM system and business objects
 */
abstract class wcmSysobject extends wcmSecureObject
{
    /**
     * Date and time of creation
     * @var string
     */
    public $createdAt;

    /**
     * Identifier of creator
     * @var int
     */
    public $createdBy;

    /**
     * revision number (auto-increment at each save() call)
     * @var int
     */
    public $revisionNumber;

    /**
     * Date and time of last modification
     * @var string
     */
    public $modifiedAt;

    /**
     * Identifier of modifier
     * @var int
     */
    public $modifiedBy;

    /**
     * version number (auto-increment at each archive() call)
     * @var int
     */
    public $versionNumber;

    /**
     * An assoc array of extended properties (name => value)
     * @var array
     */
    public $properties = array();

    /**
     * sysobject's master class
     * @var wcmSysclass
     */
    protected $masterClass;

    /**
     * Constructor
     *
     * Can be overloaded/supplemented by the child class
     *
     * @param wcmProject  $project    wcmProject object
     * @param int         $id         Optional id (used to refresh object)
     *
     */
    public function __construct($project = null, $id = 0)
    {
        // Initialize masterclass, connector, database and workflow
        $this->getMasterClass();

        // Set default values
        $this->setDefaultValues();

        // Refresh object?
        if ($id)
        {
            $this->refresh($id);
        }
    }

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->createdBy = $this->modifiedBy = null;
        $this->createdAt = $this->modifiedAt = date('Y-m-d H:i:s');
        $this->revisionNumber = 0;
        $this->versionNumber = 0;
        $this->properties = array();

    }

    /**
     * Unserialize some properties if needed
     * This method is called by nextEnum() and refresh()
     */
    public function unserializeProperties()
    {
        $this->properties = ($this->properties) ? unserialize($this->properties) : array();
    }

    /**
     * Returns the business or system class which defines current sysobject
     *
     * @return object An instance or wcmBizclass or wcmSysclass
     */
    public function getMasterClass()
    {
        if (!isset($this->masterClass))
        {
            $this->database = wcmMasterClassRegistry::getDatabase($this);
            $this->masterClass = wcmMasterClassRegistry::getMC($this);

            // Retrieve connector table, lock mode and workflow
            $this->tableName = $this->masterClass->connectorTable;
            $this->isLockOptimistic = $this->masterClass->allowOptimisticLock;
        }

        return $this->masterClass;
    }

    /**
     * Index object in search engine
     *
     * @return bool true on success, false on failure
     */
    public function index()
    {
        return true;
    }

    /**
     * De-index object from search engine
     *
     * @return bool true on success, false on failure
     */
    public function deindex()
    {
        return true;
    }

    /**
     * Store current object into database
     *
     * @return bool true on success.
     */
    protected function store()
    {
        // Retrieve userId from session
        $userId = wcmSession::getInstance()->userId;
        // Always update the modifiedBy and modifiedAt fields
        if ($userId)
        {
            $this->modifiedBy = $userId;
        }
        $this->modifiedAt = date('Y-m-d H:i:s');

        // An null (or zero) id implies a "create" operation
        if (!$this->id)
        {
            // Update the createdBy and createdAt fields
            if ($userId)
            {
                $this->createdBy = $userId;
            }
            $this->createdAt = date('Y-m-d H:i:s');
        }

        // Update or insert object in database
        $this->revisionNumber++;
        if (!$this->database->storeSysObject($this))
        {
            $this->revisionNumber--;
            $this->lastErrorMsg = $this->getType() . '::store failed : ' . $this->database->getErrorMsg();
            return false;
        }

        // Call onStore callback
        $this->onStore();

        return true;
    }

    /**
     * Deletes current object from database
     *
     * @return bool true on success.
     */
    public function delete()
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        if (!parent::delete())
        {
            return false;
        }

        return true;
    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source = null)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Check lock
        if (!$this->checkLock('save'))
        {
            return false;
        }

        // Bind, check and store
        if (!$this->bind($source))
        {
            return false;
        }

        // Check validity and finally store
        if (!$this->checkValidity())
        {
            return false;
        }
        if (!$this->store())
        {
            return false;
        }

        // As we cannot ensure transactional mode between store and index,
        // even if the indexation may fail the method will return true...
        $this->index();

        return true;
    }

    /**
     * Refresh and then attempt to lock current object
     *
     * @return bool TRUE on success, FALSE otherwise (the object can be locked or just deleted)
     */
    public function checkout()
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Ignore new object
        if (!$this->id)
        {
            return true;
        }

        // Remember oldId in case this object has been deleted
        $oldId = $this->id;

        // Check existence
        if (!$this->refresh())
        {
            $this->lastErrorMsg = $this->getType() . '::checkout failed : object ' . $oldId . ' has been deleted';
            return false;
        }

        // Lock
        return $this->lock();
    }

    /**
     * Cancel previous checkout operation (remove lock)
     *
     * @return bool TRUE on success, FALSE otherwise (the object has not been locked by current user)
     */
    public function undoCheckout()
    {
        return $this->unlock();
    }

    /**
     * Check-in object in database (save and then unlock)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function checkin($source = null)
    {
        if (!$this->save($source))
        {
            return false;
        }

        //unlock if we are in pessimistic mode
        if (!$this->isLockOptimistic())
        {
            $this->unlock();
        }

        return true;
    }

    /**
     * Explicit unserialization of object
     *
     * @param string $serialized The serialized content
     */
    public function unserialize($serialized)
    {
        // Rebuild masterClass, database, workflow, ...
        $this->getMasterClass();

        // Unserialize content
        parent::unserialize($serialized);
    }

    /**
     * Check if current object has been updated (or deleted) since last memory load
     * Note: if object has been deleted, $this->id will be set to zero
     *
     * @return bool TRUE if object is obsolete (updated/deleted), FALSE otherwise
     */
    public function isObsolete()
    {
        // Retrieve last revision number from database
        $revisionNumber = $this->database->executeScalar('SELECT revisionNumber FROM ' . $this->tableName . ' WHERE id=?', array($this->id));

        // If result is null object has been deleted!
        if ($revisionNumber === null && get_class($this)!= 'wcmTemplate' && get_class($this)!= 'wcmTemplateCategory')
        {
            $this->id = 0;
        }

        return ($this->revisionNumber != $revisionNumber);
    }

    /**
     * Archive a version of current sysobject in
     * the version manager
     *
     * @param string $comment Comment associated to version
     *
     * @return bool true on success, false otherwise
     */
    public function archive($comment = null)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        $this->versionNumber++;
        $vm = new wcmVersionManager();
        if (!$vm->archive($this, $comment))
        {
            $this->versionNumber--;
            $this->lastErrorMsg = $vm->getErrorMsg();
            return false;
        }
        else
        {
            // update version number in database
            $sql  = 'UPDATE ' . $this->tableName . ' SET versionNumber=' . $this->versionNumber;
            $sql .= ' WHERE id=' . $this->id;
            $this->database->executeStatement($sql);
        }

        // Free some memory
        $this->serialStorage = array();

        return true;
    }

    /**
     * Rollback to a previous stored version of current sysobject
     *
     * @param int $versionId Internal unique ID representing version to restore
     *
     * @return bool true on success, false otherwise
     */
    public function rollback($versionId = 0)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        return $this->restore($versionId, true);
    }

    /**
     * Restore a previous stored version of current sysobject
     *
     * @param int $versionId Internal unique ID representing version to restore
     * @param bool $rollBack True to rollback, i.e. remove in-between versions (false by default)
     *
     * @return bool true on success, false otherwise
     */
    public function restore($versionId = 0, $rollBack = false)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Check lock
        if (!$this->checkLock('restore'))
        {
            return false;
        }

        // When you RESTORE an object, the revisionNumber must increase
        $oldRevisionNumber = $this->revisionNumber;

        // Restore object
        $vm = new wcmVersionManager();
        $version = $vm->restore($this->getType(), $this->id, $versionId, $rollBack);
        if (!$version)
        {
            return false;
        }

        // When you RESTORE an object, the revisionNumber must increase!
        if (!$rollBack)
        {
            $version->revisionNumber = $oldRevisionNumber;
        }

        // The version has been retrieve, store it in the database and update current object
        // IMPORTANT: do not execute workflow on restore/rollback and adjust revision number
        $version->revisionNumber--;
        if (!$version->save(null, true))
        {
            return false;
        }

        // Free some memory
        $this->serialStorage = array();

        // Update current object
        $this->refresh($version->id);

        return true;
    }

    /**
     * Expose 'fulltext' to getAssocArray by computing a default full-text
     * string used for indexing
     *
     * Remark: this method is exposed by getAssocArray and may be
     * overloaded by inherited classes
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return string A full-text representation of the object
     */
    public function getAssoc_fulltext($toXML = false)
    {
        // => Retrieve all string keys
        // => Remove all tags (html, xml)
        // => Parse result text and discard stop words
        $fulltext = null;
        foreach (getPublicProperties($this) as $value)
        {
            if (is_string($value))
            {
                $fulltext .= ' ' . $value;
            }
        }

        $fulltext = getRawText($fulltext);

        // Parse text (no stop word, retrieve a single occurence for each word)
        return trim(implode(' ', parseText($fulltext, null, true)));
    }

    /**
     * Returns an XML representation of a property
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     * @return string XML representation of the property
     */
    protected function propertyToXML($propKey, $propValue)
    {
        $xml = null;

        // Treat special properties
        if ($propKey == 'properties')
        {
            $xml = '<properties>';
            if (!empty($this->properties) && is_array($this->properties))
            {
                foreach ($this->properties as $name => $value)
                {
                    $xml .= '<' . $name . '>';
                    $xml .= is_object($value) ? serialize($value) : wcmXML::xmlEncode($value);
                    $xml .= '</' . $name . '>';
                }
            }
            $xml .= '</properties>';
        }
        else
        {
            if ($propKey == 'createdAt' || $propKey == 'modifiedAt')
            {
                $propValue = dateToISO8601($propValue);
            }
            $xml = parent::propertyToXML($propKey, $propValue);
        }

        return $xml;
    }

    /**
     * This function can be used to customize the initialisation of a specific property
     * from a XML node (invoked by initFromXML() method)
     *
     * @param string  $property  Property name to initialize
     * @param XMLNode $node      XML node used for initialization
     */
    protected function initPropertyFromXMLNode($property, $node)
    {
        if ($property == 'properties')
        {
            // Assume <properties> <{propName}>{value}</propName> ... </properties>
            $this->properties = array();
            foreach ($node->childNodes as $child)
            {
                if ($child->nodeName != '#text')
                {
                    $this->properties[$child->nodeName] = $child->nodeValue;
                }
            }
        }
        elseif ($property == 'createdAt' || $property == 'modifiedAt')
        {
            $this->$property = dateFromISO8601($node->nodeValue);
        }
        else
        {
            parent::initPropertyFromXMLNode($property, $node);
        }
    }

    /**
     * This function can be used to customize the initialisation of a
     * specific property from a JSON assoc value (invoked by the
     * initFromJSON method).
     *
     * @param string $property   Property name to initialize
     * @param mixed  $assocValue JSON assoc value used for initialization
     */
    protected function initPropertyFromJSONAssocValue($property, $assocValue)
    {
        if ($property == 'properties')
        {
            $this->properties = array();
            if (is_array($assocValue))
            {
                $this->properties = $assocValue;
            }
        }
        elseif ($property == 'createdAt' || $property == 'modifiedAt')
        {
            $this->$property = dateFromISO8601($assocValue);
        }
        else
        {
            parent::initPropertyFromJSONAssocValue($property, $assocValue);
        }
    }

    /**
     * Returns the JSON representation of a property.
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     *
     * @returns string The JSON representation of the property
     */
    protected function propertyToJSON($propKey, $propValue)
    {
        $json = null;

        if ($propKey == 'properties')
        {
            $json = '"properties":{';
            foreach ($this->properties as $name => $value)
            {
                $json .= parent::propertyToJSON($name, $value) . ',';
            }
            $json = rtrim($json, ',');
            $json .= '}';
        }
        else
        {
            if ($propKey == 'createdAt' || $propKey == 'modifiedAt')
            {
                $propValue = dateToISO8601($propValue);
            }
            $json = parent::propertyToJSON($propKey, $propValue);
        }

        return $json;
    }

    /**
     * Check if code is unique, but only when it's a new code
     *
     * @return bool True when code exist, false otherwise
     */
    protected function checkUniqueCode()
    {
        $sql = 'SELECT COUNT(*) FROM ' . $this->getTableName() . ' WHERE code=\'' . $this->code . '\' AND id !=\'' . $this->id . '\'';
        if ($this->database->executeScalar($sql) > 0)
        {
            $this->lastErrorMsg = _ERROR_UNIQUE_CODE;
        }
        else
        {
            return true;
        }
    }

    /**
     * Function used to serialize the current object
     *
     * @return array array of properties to serialize
     */
    public function __sleep()
    {
        $properties = array_keys(get_object_vars($this));
        $prop = array_diff($properties, array('database', 'masterClass', 'tableName', 'isLockOptimistic', 'workflow'));
        return $prop;
    }

    /**
     * Function used to restore non serialized properties
     */
    public function __wakeup()
    {
        $this->getMasterClass();
    }

}
