<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.semanticData.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Provides a bridge to the Text-Mining Engine Server
 */
class wcmSemanticData implements ArrayAccess
{
    /**
     * Text language (for auto-detection)
     * @var string
     */
    public $language;

    /**
     * Array of concepts
     * 'concept' => array('relevancy' => , 'frequency' =>)
     * @var array
     */
    public $concepts;

    /**
     * Array of organization names
     *
     * 'entity' => array('relevancy' => , 'frequency' =>, 'weight' =>, 'confidence' =>)
     * @var array
     */
    public $ON;

    /**
     * Array of geographic locations
     *
     * 'entity' => array('relevancy' => , 'frequency' =>, 'weight' =>, 'confidence' =>)
     * @var array
     */
    public $GL;

    /**
     * Array of people names
     *
     * 'entity' => array('relevancy' => , 'frequency' =>, 'weight' =>, 'confidence' =>)
     * @var array
     */
    public $PN;

    /**
     * Text summary
     * @var string
     */
    public $summary;

    /**
     * Text tone (between -100 and 100)
     * @var int
     */
    public $tone;

    /**
     * Text positiveTone (between 0 and 100)
     * @var int
     */
    public $positiveTone;

    /**
     * Text negativeTone (between 0 and 100)
     * @var int
     */
    public $negativeTone;

    /**
     * Text subjectivity
     * @var int
     */
    public $subjectivity;

    /**
     * Array of categories
     *
     * 'category' => ('weight' =>, 'confidence' =>)
     * @var array
     */
    public $categories;

    /**
     * Array of  similar items
     *
     * 'className_id' => ('className' =>, 'id' => )
     * @var array
     */
    public $similars;

    /**
     * Constructor
     *
     * @param string $xml An optional XML represeting result from TME
     */
    public function __construct($xml = null)
    {
        $this->cleanup();
        if ($xml)
        {
            $dom = new DOMDocument();
            $dom->loadXML($xml);
            $xpath = new DOMXpath($dom);

            foreach ($xpath->query('//Category') as $el)
            {
                $categoryArray = explode(' - ', $el->nodeValue);
                $tmeId = $categoryArray[0];
                $key = ucfirst(array_pop(explode(' - ', $el->nodeValue)));
                $this->categories[$key] = array(
                        'weight' => $el->getAttribute('Weight'),
                        'tmeId' => $tmeId
                        );
            }

            foreach ($xpath->query('//Concept') as $el)
            {
                $key = ucfirst($el->nodeValue);
                $this->concepts[$key] = array(
                        'frequency' => $el->getAttribute('Frequency'),
                        'relevancy' => $el->getAttribute('RelevancyScore')
                        );
            }

            foreach ($xpath->query('//Entity[@Kind="ON"]') as $el)
            {
                $key = ucfirst($el->nodeValue);
                $this->ON[$key] = array(
                        'frequency' => $el->getAttribute('Frequency'),
                        'relevancy' => $el->getAttribute('RelevancyScore'),
                        'weight' => $el->getAttribute('Weight'),
                        'confidence' => $el->getAttribute('ConfidenceScore')
                        );
            }

            foreach ($xpath->query('//Entity[@Kind="PN"]') as $el)
            {
                $key = ucfirst($el->nodeValue);
                $this->PN[$key] = array(
                        'frequency' => $el->getAttribute('Frequency'),
                        'relevancy' => $el->getAttribute('RelevancyScore'),
                        'weight' => $el->getAttribute('Weight'),
                        'confidence' => $el->getAttribute('ConfidenceScore'));
            }

            foreach ($xpath->query('//Entity[@Kind="GL"]') as $el)
            {
                $key = ucfirst($el->nodeValue);
                $this->GL[$key] = array(
                        'frequency' => $el->getAttribute('Frequency'),
                        'relevancy' => $el->getAttribute('RelevancyScore'),
                        'weight' => $el->getAttribute('Weight'),
                        'confidence' => $el->getAttribute('ConfidenceScore')
                        );
            }

            foreach ($xpath->query('//Summary') as $el)
            {
                $this->summary = $el->nodeValue;
            }

            foreach ($xpath->query('//Sentiment') as $el)
            {
                $this->positiveTone = intval(floatval($el->getAttribute('PositiveTone')) * 100);
                $this->negativeTone = intval(floatval($el->getAttribute('NegativeTone')) * 100);
                $this->subjectivity = intval(floatval($el->getAttribute('Subjectivity')) * 33);
                if ($this->positiveTone > $this->negativeTone)
                {
                    $this->tone = $this->positiveTone;
                }
                else
                {
                    $this->tone = - $this->negativeTone;
                }
            }

            foreach ($xpath->query('//SimilarText') as $el)
            {
                list($className, $id) = explode('_', $el->getAttribute('Id'));

                // Ensure similar object still exists
                $bizobject = wcmObjectFactory::create($className, $id);
                if ($bizobject->id)
                {
                    $this->similars[$el->getAttribute('Id')] = array(
                            'className' => $className,
                            'id' => $id,
                            'score' => intval(floatval($el->getAttribute('Weight')) * 100),
                            'title' => getObjectLabel($bizobject),
                            );
                }
            }
        }
    }

    /**
     * Initialize semantic data from an XML node
     *
     * *@param DOMElement An XML node representing semantic data
     */
    public function fromXML($node)
    {
        $this->cleanup();
        if (!$node)
        {
            return;
        }

        if ($node->childNodes)
        {
            foreach ($node->childNodes as $child)
            {
                $name = $child->nodeName;
                switch ($name)
                {
                    case 'language':
                    case 'tone':
                    case 'positiveTone':
                    case 'negativeTone':
                    case 'subjectivity':
                    case 'summary':
                        $this->$name = $child->nodeValue;
                        break;
                    case 'concepts':
                        foreach ($child->childNodes as $el)
                        {
                            if ($el->nodeName == '#text')
                            {
                                continue;
                            }

                            $keys[] = $el->nodeValue;
                            $vals[] = array(
                                    'frequency' => $el->getAttribute('frequency'),
                                    'relevancy' => $el->getAttribute('relevancy')
                                    );

                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'categories':
                        foreach ($child->childNodes as $el)
                        {
                            if ($el->nodeName == '#text')
                            {
                                continue;
                            }

                            $keys[] = $el->nodeValue;
                            $categoryArray = explode(' - ', $el->nodeValue);
                            $tmeId = $categoryArray[0];
                            $vals[] = array(
                                    'weight' => $el->getAttribute('weight'),
                                    'confidence' => $el->getAttribute('confidence'),
                                    'tmeId' => $tmeId
                                    );
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'entitiesON':
                    case 'entitiesPN':
                    case 'entitiesGL':
                        $name = substr($name, -2);
                        foreach ($child->childNodes as $el)
                        {
                            if ($el->nodeName == '#text')
                            {
                                continue;
                            }

                            $keys[] = $el->nodeValue;
                            $vals[] = array(
                                    'frequency' => $el->getAttribute('frequency'),
                                    'relevancy' => $el->getAttribute('relevancy'),
                                    'weight' => $el->getAttribute('weight'),
                                    'confidence' => $el->getAttribute('confidence')
                                    );
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'similars':
                        foreach ($child->childNodes as $el)
                        {
                            if ($el->nodeName == '#text')
                            {
                                continue;
                            }

                            $keys[] = $el->nodeValue;
                            $vals[] = array(
                                    'className' => $el->getAttribute('className'),
                                    'id' => $el->getAttribute('id'),
                                    'title' => $el->getAttribute('title')
                                    );
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                }
            }
        }
    }

    /**
     * Returns an XML representing the semantic data
     *
     * @return string XML representing the data
     */
    public function toXML()
    {
        $xml = '<semanticData>';
        $xml .= '<language>' . wcmXML::xmlEncode($this->language) . '</language>';
        $xml .= '<tone>' . wcmXML::xmlEncode($this->tone) . '</tone>';
        $xml .= '<positiveTone>' . wcmXML::xmlEncode($this->positiveTone) . '</positiveTone>';
        $xml .= '<negativeTone>' . wcmXML::xmlEncode($this->negativeTone) . '</negativeTone>';
        $xml .= '<subjectivity>' . wcmXML::xmlEncode($this->subjectivity) . '</subjectivity>';
        $xml .= '<summary>' . wcmXML::xmlEncode($this->summary) . '</summary>';
        $xml .= $this->assocToXML($this->concepts, 'concepts', 'concept');
        $xml .= $this->assocToXML($this->categories, 'categories', 'category');
        $xml .= $this->assocToXML($this->ON, 'entitiesON', 'ON');
        $xml .= $this->assocToXML($this->PN, 'entitiesPN', 'PN');
        $xml .= $this->assocToXML($this->GL, 'entitiesGL', 'GL');
        $xml .= $this->assocToXML($this->similars, 'similars', 'similar');
        $xml .= '</semanticData>';

        return $xml;
    }

    /**
     * Returns an XML representing the semantic assoc array
     *
     * @param array $assoc Semantic assoc array
     * @param string $mainTag Main tag
     * @param string $tag Subsequent tag
     * @return string XML representing the array
     */
    private function assocToXML($assoc, $mainTag, $tag)
    {
        $xml = '<' . $mainTag . '>';
        if ($assoc)
        {
            foreach ($assoc as $item => $attributes)
            {
                $xml .= '<' . $tag;
                foreach ($attributes as $attribute => $value)
                {
                    $xml .= ' ' . $attribute . '="' . wcmXML::xmlEncode($value) . '"';
                }
                $xml .= '>' . wcmXML::xmlEncode($item) . '</' . $tag . '>';
            }
        }
        $xml .= '</' . $mainTag . '>';

        return $xml;
    }

    /**
     * Initialize semantic data from a JSON assoc. value
     *
     * @param mixed $assocValue A JSON assoc value representing the semantic data
     */
    public function fromJSONAssocValue($assocValue)
    {
        $this->cleanup();

        if ($assocValue)
        {
            foreach ($assocValue as $name => $values)
            {
                switch ($name)
                {
                    case 'language':
                    case 'tone':
                    case 'positiveTone':
                    case 'negativeTone':
                    case 'subjectivity':
                    case 'summary':
                        $this->$name = $values;
                        break;
                    case 'concepts':
                        foreach ($values as $value => $props)
                        {
                            $keys[] = $value;
                            $vals[] = $props;
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'categories':
                        foreach ($values as $value => $props)
                        {
                            $keys[] = $value;
                            $vals[] = $props;
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'entitiesON':
                    case 'entitiesPN':
                    case 'entitiesGL':
                        $name = substr($name, -2);
                        foreach ($values as $value => $props)
                        {
                            $keys[] = $value;
                            $vals[] = $props;
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                    case 'similars':
                        foreach ($values as $value => $props)
                        {
                            $keys[] = $value;
                            $vals[] = $props;
                            $this->$name = array_combine($keys, $vals);
                        }
                        break;
                }
            }
        }
    }

    /**
     * Returns the JSON representing of the semantic data.
     *
     * @return string The JSON representing of the semantic data
     */
    public function toJSON()
    {
        $json  = '"semanticData":{';
        $json .= '"language":' . json_encode($this->language) . ',';
        $json .= '"tone":' . json_encode($this->tone) . ',';
        $json .= '"positiveTone":' . json_encode($this->positiveTone) . ',';
        $json .= '"negativeTone":' . json_encode($this->negativeTone) . ',';
        $json .= '"subjectivity":' . json_encode($this->subjectivity) . ',';
        $json .= '"summary":' . json_encode($this->summary) . ',';
        $json .= '"concepts":' . json_encode($this->concepts) . ',';
        $json .= '"categories":' . json_encode($this->categories) . ',';
        $json .= '"entitiesON":' . json_encode($this->ON) . ',';
        $json .= '"entitiesPN":' . json_encode($this->PN) . ',';
        $json .= '"entitiesGL":' . json_encode($this->GL) . ',';
        $json .= '"similars":' . json_encode($this->similars);
        $json .= '}';

        return $json;
    }

    /**
     * Cleanup the structure
     */
    public function cleanup()
    {
        $this->language = 'ENGLISH';
        $this->concepts = array();
        $this->ON = array();
        $this->PN = array();
        $this->GL = array();
        $this->categories = array();
        $this->similars = array();
        $this->summary = null;
        $this->tone = 0;
        $this->positiveTone = 0;
        $this->negativeTone = 0;
        $this->subjectivity = 0;
    }

    /**
     * Implements interface ArrayAcccess
     *
     * @param string $key Configuration key
     *
     * @return bool True is key exists
     */
    public function offsetExists($key)
    {
        return isset($this->$key);
    }

    /**
     * Implements interface ArrayAcccess
     *
     * @param string $key Configuration key
     *
     * @return mixed Configuration key value
     */
    public function offsetGet($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    /**
     * Implements interface ArrayAcccess
     *
     * @param string $key Configuration key
     *
     * @return Exception Configuration is read only!
     * @todo Says it returns an exception, it doesn't, and it doesn't seem readonly either
     */
    public function offsetSet($key, $value)
    {
        $this->$key = $value;
    }

    /**
     * Implements interface ArrayAcccess
     *
     * @param string $key Configuration key
     *
     * @return Exception Configuration is read only!
     * @todo Says it returns an exception, it doesn't, and it doesn't seem readonly either
     */
    public function offsetUnset($key)
    {
        $this->$key = null;
    }

    /**
     * Updates semantic data from an associative array
     * (typically used with $_REQUEST['_semanticData'])
     *
     * @param array $data Associative array containing data to update
     *                    (the key must match a public property of the
     *                     semanticData and is case-sensitive)
     */
    public function updateFromAssocArray(array $data)
    {
        if (!$data)
        {
            return;
        }

        foreach ($data as $key => $value)
        {
            if (property_exists($this, $key))
            {
                // are we dealing with an array? (e.g. ON, PN, GL, categories, concepts)
                if (is_array($this->$key))
                {
                    // assume value is a list of string separated by pipe
                    $array = array();
                    $values = explode('|', $value);
                    foreach ($values as $val)
                    {                    	
                        //keep old settings values
                        $exists = $this->$key;                    	
                        if ($val != '' && !isset($exists[$val]))
                        {
                            $array[$val] = array(
                                    'frequency' => 0, 'relevancy' => 100,
                                    'weight' => 100, 'confidence' => 100
                                    );
                        }
                        elseif (isset($exists[$val]))
                        {
                            $array[$val] = $exists[$val];
                        }
                    }
                    $this->$key = $array;
                }
                else
                {
                    // store literal value (e.g. summary, tone, subjectivity, ...)
                    $this->$key = $value;
                }
            }
            else
            {
                // unexpected property... log warning
                wcmProject::getInstance()->logger->logWarning('Invalid semanticData property: "' . $key . '"');
            }
        }
    }

    /**
     * Using custom method to merge arrays. array_merge would treat
     * integer indexes as integers, not as strings.
     *
     * @param array $arr1 Left array
     * @param array $arr2 Right array
     *
     * @return merged array
     */
    public function array_kmerge($arr1, $arr2)
    {
        foreach ($arr2 as $key => $val)
        {
            $arr1[$key] = $val;	
        }
        return $arr1;
    }

    /**
     * Merge current semantic data with extra data. If the extra data contains
     * new values (i.e. non-existing in the current semanticData) those values will
     * be added. The original values (for list like ON,PN,GL...) will be kept.
     *
     * @param wcmSemanticData $extraData Extra semantic data
     */
    public function merge(wcmSemanticData $extraData)
    {
        foreach ($this as $key => $source)
        {
            // manage array (ON, PN, GL, concepts, categories...)
            $extra = $extraData->$key;
            if (is_array($source) && is_array($extra))
            {
                $source = $this->array_kmerge($extra, $source);
                $this->$key = $source;
            }
            elseif ($extra && !$source)
            {
                $this->$key = $extra;
            }
        }
    }
}
