<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.XML.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class provides static static function to facilitate
 * XML, XSL and XPATH manipulation
 */
class wcmXML
{
    /**
     * Encode a string to be xml compliant ()replace <, >, ', " and &)
     *
     * @param mixed $v      The data to encode
     * @param bool  $strict Whether to be strict about null and false
     *
     * @return string   xml encoded string
     */
    static function xmlEncode($v, $strict = false)
    {
        // Encode array
        if (is_array($v))
        {
            return self::xmlEncodeArray($v, $strict);
        }
        // Handle booleans correctly
        if ($strict && is_bool($v))
        {
            $v = intval($v);
        }

        // Ignore empty string and object
        if (($strict && $v === null) || (!$strict && $v == null) || is_object($v))
        {
            return null;
        }

        // Decode all entities
        $v = html_entity_decode($v, ENT_COMPAT, 'UTF-8');

        return htmlspecialchars($v, ENT_COMPAT, 'UTF-8');
    }

    /**
     * Decode a xml to be good string ()replace <, >, ', " and &)
     *
     * @param mixed $v  The data to decode
     *
     * @return string   string decoded xml
     */
    static function xmlDecode($v)
    {
        // Ignore empty string and object
        if ($v == null || is_object($v))
        {
            return '';
        }

        // Decode all entities
        $v = htmlentities($v, ENT_COMPAT, 'UTF-8');

        return htmlspecialchars_decode($v, ENT_COMPAT);
    }

    /**
     * Encode an array to be xml compliant
     *
     * @param array $array  An array to encode
     * @param bool  $strict Whether to be strict about null and false
     *
     * @return $xml string  array transformed in xml
     */
    static function xmlEncodeArray($array, $strict = false)
    {
        if (!is_array($array))
        {
            return null;
        }

        $xml = '';
        foreach ($array as $key => $val)
        {
            if (preg_match('`^[a-zA-Z][a-zA-Z0-9]+$`', $key))
            {
                // Alphanumeric key, let's create a tag!
                $xml .= '<' . $key . '>' . wcmXML::xmlEncode($val, $strict) . '</' . $key . '>';
            }
            else
            {
                // Numeric key, let's create a standard tag
                $xml .= '<item key="' . $key . '">' . wcmXML::xmlEncode($val, $strict) . '</item>';
            }
        }

        return $xml;
    }

    /**
     * Returns an xml string representing a given associative array
     * Each key is mapped to a tag
     *
     * @param array  $array   the array to "xmlize"
     * @param string $mainTag (optional) the main surrounding tag
     * @param bool   $strict  (optional) whether to be strict about null and false
     *
     * @return string An XML representing the array
     */
    static function arrayToXml($array, $maintag='', $strict = false)
    {
        $xml = '';
        if ($maintag != '')
        {
            $xml .= '<' . $maintag . '>';
        }
        foreach ($array as $key => $value)
        {
            $xml .= '<' . $key . '>' . wcmXML::xmlEncode($value, $strict) . '</' . $key . '>';
        }

        if ($maintag != '')
        {
            $xml .= '</' . $maintag . '>';
        }

        return $xml;
    }

    /**
     * Returns an associative array representing a given xml string
     * Each tag is mapped to a key
     *
     * @param string $xml  xml string to pars
     *
     * @return array An assoc array representing the xml string
     * @throws Exception
     */
    static function xmlToArray($xml)
    {
        $domXml = new DOMDocument();
        if (!$domXml->loadxml($xml))
        {
            throw new Exception(_BIZ_INVALID_XML);
        }
        $rootNode = $domXml->documentElement;
        $array = array();
        foreach ($rootNode->childNodes as $child)
        {
            $array[$child->nodeName] = self::xmlDecode($child->nodeValue);
        }
        return $array;
    }

    /**
     * Returns the innerXML from a DOMDocument node
     *
     * @param DOMNode $node A valid DOMNode
     *
     * @return string Textual value of the node (without children)
     */
    static function getNodeValue($node)
    {
        if ($node == null)
        {
            return null;
        }

        // Stop at the first text node
        foreach ($node->childNodes as $child)
        {
            if ($child->nodeType == XML_TEXT_NODE)
            {
                return trim($child->nodeValue);
            }
        }
        return trim($node->nodeValue);
    }

    /**
     * Returns the outer XML content of a node (i.e. current tag include)
     * @param DOMNode $node A valid DOMNode
     * @return string Outer XML content of a node
     */
    static function getOuterXml($node)
    {
        return ($node == null) ? null : $node->ownerDocument->saveXML($node);
    }

    /**
     * Returns the inner XML content of a node
     *
     * @param DOMNode $node A valid DOMNode
     * @return string Inner XML content of a node
     */
    static function getInnerXml($node)
    {
        if ($node == null)
        {
            return null;
        }

        $innerXml = '';
        $xmlDoc = $node->ownerDocument;
        foreach ($node->childNodes as $child)
        {
            $innerXml .= $xmlDoc->saveXML($child);
        }
        return $innerXml;
    }

    /**
     * Retrieve the first node corresponding to a specific XPath query
     *
     * @param DomXPath $xPath A valid DomXPath object
     * @param DomNode $contextNode  Context node (or null)
     * @param string $query   XPath query
     *
     * @return string   The first node (or null)
     */
    static function getXPathFirstNode($xPath, $contextNode = null, $query = null)
    {
        if (null === $query)
        {
            return null;
        }

        if ($contextNode)
        {
            $nodeList = $xPath->query($query, $contextNode);
        }
        else
        {
            $nodeList = $xPath->query($query);
        }

        // Retrieve first value or else return default value
        if ($nodeList && $nodeList->length > 0)
        {
            return $nodeList->item(0);
        }
        else
        {
            return null;
        }
    }

    /**
     * Retrieve the first node value corresponding to a specific XPath query
     *
     * @param DomXPath $xPath  A valid DomXPath object
     * @param DomNode $contextNode  Context node (or null)
     * @param string $query  XPath query
     * @param mixed $defaultValue  Default value to return if query does not match any node (default is null)
     *
     * @return string   The node value (or the default value)
     */
    static function getXPathNodeValue($xPath, $contextNode = null, $query = null, $defaultValue = null)
    {
        // XPath query
        if ($contextNode)
        {
            $nodeList = $xPath->query($query, $contextNode);
        }
        else
        {
            $nodeList = $xPath->query($query);
        }

        // Retrieve first value or else return default value
        if ($nodeList && $nodeList->length > 0)
        {
            return trim($nodeList->item(0)->nodeValue);
        }
        else
        {
            return $defaultValue;
        }
    }

    /**
     * Process the XSL with the XML and returns the result
     *
     * @param string $xml    string  Contains the XML
     * @param string $xsl    string  Contains filename of the XSL
     * @param array  $params array   Optional parameters for XSL-T (assoc array of null)
     *
     * @return string The result of the XSL transformation
     * @throws Exception
     */
    static function processXSLT($xml, $xsl, $params = null)
    {
        // Create the domXSL
        $domXsl = new DOMDocument;
        if (!$domXsl->load($xsl))
        {
            wcmProject::getInstance()->logger->logError('Invalid XSL: ' . $xsl);
            throw new Exception(_BIZ_INVALID_XML);
        }

        // Create the domXML
        $domXml = new DOMDocument();

        if (empty($xml))
        {
            wcmProject::getInstance()->logger->logError('Invalid XML: ' . $xml);
            throw new Exception(_BIZ_INVALID_XML);
        }
        else
        {
            if (!$domXml->loadXML($xml))
            {
                wcmProject::getInstance()->logger->logError('Invalid XML: ' . $xml);
                throw new Exception(_BIZ_INVALID_XML);
            }
        }

        // Process the XSL with optional parameters
        $proc = new XSLTProcessor;
        $proc->registerPHPFunctions();
        $proc->importStyleSheet($domXsl);
        if (is_array($params))
        {
            foreach ($params as $param => $value)
            {
                $proc->setParameter('', $param, $value);
            }
        }

        return ($proc->transformToXML($domXml));
    }
}
