<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.version.php 9403 2010-04-08 18:32:52Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The version class represents an instance of
 * a wcmObject's version
 */
class wcmVersion extends wcmObject
{
    /**
     * Class of versioned object
     * @var string
     */
    private $objectClass;

    /**
     * Id of versioned object
     * @var int
     */
    public $objectId;

    /**
     * Serialized content
     * @var string
     */
    public $objectContent;

    /**
     * Date of creation
     * @var string
     */
    public $createdAt;

    /**
     * ID of user who has created this version
     * @var int
     */
    public $createdBy;

    /**
     * Version number
     * @var int
     */
    public $versionNumber;

    /**
     * Revision number
     * @var int
     */
    public $revisionNumber;

    /**
     * Version comment
     * @var string
     */
    public $comment;

    /**
     * Default constructor
     *
     * @param string $objectClass Class of versioned object
     * @param int $objectId ID of versioned object
     */
    public function __construct($objectClass, $objectId = null)
    {
        // Pre-set objectClass and objectId
        $this->objectClass = $objectClass;
        $this->objectId = $objectId;
        $this->objectType = 'wcmVersion';

        parent::__construct();
    }

    /**
     * Returns the database used to store/fetch object
     *
     * @return wcmDatabase Database used to store/fetch object
     */
    public function getDatabase()
    {
        if (!$this->database)
        {
            $vm = new wcmVersionManager();
            $this->database = $vm->getDatabase();
            $this->tableName = $this->getTableName();
        }
        return $this->database;
    }

    public function getTableName()
    {
        return $this->tableName = '#__' . $this->objectClass . '_versions';
    }

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->objectContent = null;
        $this->createdAt = date('Y-m-d H:i:s');
        $this->createdBy = wcmSession::getInstance()->userId;
        $this->comment = null;
        $this->revisionNumber = 1;
        $this->versionNumber = null;
    }

    /**
     * Returns the last version stored in the database
     *
     * @return int Last version number (or null if table does not exist)
     */
    public function getLastVersion()
    {
        $sql = 'SELECT MAX(versionNumber) FROM ' . $this->tableName . ' WHERE objectId=' . $this->objectId;
        return $this->database->executeScalar($sql);
    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source = null)
    {
        if (!$this->versionNumber)
        {
            $this->versionNumber = 1 + intval($this->getLastVersion());
        }
        return parent::save($source);
    }

    /**
     * Exposes 'creator' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array The session's user getAssocArray() (or null)
     */
    public function getAssoc_creator($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $user = $this->getCreator();
        return ($user) ? $user->getAssocArray($toXML) : null;
    }

    /**
     * Returns the creator of the version
     *
     * @return wcmUser Creator of the current version
     */
    public function getCreator()
    {
        $user = new wcmUser(null, $this->createdBy);
        return ($user->id == 0) ? null : $user;
    }

    /**
     * Finalize rollback (delete all in-between versions)
     */
    public function rollback()
    {
        $sql = 'DELETE FROM ' . $this->tableName . ' WHERE id >= ' . $this->id . ' AND objectId = ' . $this->objectId; 
        $this->database->executeStatement($sql);
    }

    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    public function ofClause($of)
    {
        if (is_array($of))
        {
            foreach ($of as $className => $id)
            {
                // Assume we want to browse versions 'of' a specific object
                if ($className == $this->objectClass)
                {
                    return 'objectId=' . $id;
                }
            }
        }

        return null;
    }
}
