<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.workflowTransition.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmWorkflowTransition is used to describe and create workflows
 */
class wcmWorkflowTransition extends wcmSysobject
{
    /**
     * name of this workflow
     * @var string
     */
    public $name;
    /**
     * Code of this workflow
     * @var string
     */
    public $code;

    /**
     * (unique) code of starting wcm_workflowState
     * @var string
     */
    public $fromState;

    /**
     * (unique) code of ending wcm_workflowState
     * @var string
     */
    public $toState;

    /**
     * id of workflow this transition belongs too
     * @var int
     */
    public $workflowId;


    /**
     * The magic 'ANY' state (can be used for a fromState)
     */
    const ANY_STATE = '_ANY_STATE';

    /**
     * The magic 'SELF' state (can be used for a fromState or a toState)
     */
    const SELF_STATE = '_SELF_STATE';

    /**
     * Check validity of object
     *
     * @return bool TRUE if object is valid
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        if (!$this->checkUniqueCode())
        {
            return false;
        }

        if (trim($this->name . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_NAME_IS_MANDATORY;
            return false;
        }

        if (strlen($this->name) > 255)
        {
            $this->lastErrorMsg = _ERROR_NAME_TOO_LONG;
            return false;
        }

        if (trim($this->code . ' ') == '')
        {
            $this->lastErrorMsg = _ERROR_CODE_IS_MANDATORY;
            return false;
        }

        if (strlen($this->code) > 64)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        if ($this->fromState  && strlen($this->fromState) > 64)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        if ($this->toState  && strlen($this->toState) > 64)
        {
            $this->lastErrorMsg = _ERROR_CODE_TOO_LONG;
            return false;
        }

        return true;
    }

    /**
     * Inserts or Updates object in database
     *
     * @param int   $userId Id of the wcmUser who is creating or updating the object
     *
     * @return bool true on success, false on failure
     */
    protected function store($userId = null)
    {
        if (!parent::store($userId))
        {
            return false;
        }

        if (isset($this->serialStorage['permissionTypes']))
        {
            // Erase permissions for the sysclass and all its related objects
            $project = wcmProject::getInstance();
            $sql = 'DELETE FROM #__permission WHERE target=?';
            $params = array($this->getPermissionTarget());
            $project->database->executeStatement($sql, $params);
            if ($this->getClass())
            {
                $sql = "DELETE FROM #__permission WHERE target LIKE '" . $this->getClass() . "%'";
                $project->database->executeStatement($sql);
            }

            // insert the new permissions
            $newPermissions = getArrayParameter($this->serialStorage, 'permissions');
            if ($newPermissions)
            {
                $permsArray = array();
                foreach ($newPermissions as $permission)
                {
                    // get values
                    $perms = explode('*', $permission, 2);
                    $permsArray[$perms[0]][] = $perms[1];
                }

                foreach ($permsArray as $wcmObject => $permissionMask)
                {
                    $securedObject = explode('_', $wcmObject);
                    $groupId   = $securedObject[0];

                    $finalMask = 0;
                    foreach ($permissionMask as $mask)
                    {
                        // get group and values
                        $values = explode('#', $mask);
                        $finalMask += $values[0];

                        // when a permission is set (WRITE, EXECUTE, DELETE)
                        // permission P_READ is always available
                        if ($finalMask)
                        {
                            $finalMask |= wcmPermission::P_READ;
                        }

                        // when a permission is none keep only that one!
                        if ($finalMask & wcmPermission::P_NONE)
                        {
                            $finalMask = wcmPermission::P_NONE;
                        }

                        // set permissions
                        $this->setGroupPermission($groupId, $finalMask);
                    }
                }
            }
        }

        // Update cache
        wcmCacheRegistry::get()->setElem($this->getType(), $this->id, $this);

        return true;
    }

    /**
     * Deletes current object from database
     *
     * @return bool true on success or an error message (string)
     */
    public function delete()
    {
        // Update cache
        wcmCache::unsetElem($this->getType(), $this->id);

        if (!parent::delete())
        {
            return false;
        }

        return true;
    }

    /**
     * Exposes 'fromState' to getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array The 'from' state getAssocArray (or null)
     */
    public function getAssoc_fromState($toXML = false)
    {
        $project = wcmProject::getInstance();
        $state = $project->workflowManager->getWorkflowStateByCode($this->fromState);
        return ($state == null) ? null : $state->getAssocArray($toXML);
    }

    /**
     * Exposes 'toState' to getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array The 'to' state getAssocArray (or null)
     */
    public function getAssoc_toState($toXML = false)
    {
        $project = wcmProject::getInstance();
        $state = $project->workflowManager->getWorkflowStateByCode($this->toState);
        return ($state == null) ? null : $state->getAssocArray($toXML);
    }

    /**
     * Returns an XML representation of a property (taken from getAssocArray() method)
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     * @return string XML representation of property
     */
    protected function propertyToXML($propKey, $propValue)
    {
        // create a tag for each property
        $xml = '<' . $propKey . '>';

        if (is_bool($propValue))
        {
            $xml .= ($propValue) ? '1' : '0';
        }
        elseif ($propValue instanceof wcmObjectAssocArray)
        {
            $xml .= wcmXML::xmlEncode($propValue->toArray());
        }
        elseif (is_object($propValue))
        {
            $xml .= $propValue->toXML();
        }
        else
        {
            $xml .= wcmXML::xmlEncode($propValue);
        }

        $xml .= '</' . $propKey . '>' . PHP_EOL;

        return $xml;
    }

    /**
     * Returns the JSON representation of a property (as obtained with
     * the getAssocArray method).
     * This method over-rides the wcmObject propertyToJSON() function
     * to adapt to the toState and fromState properties the are wcmAssocArray
     * objects
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     *
     * @return string The JSON representation of the property
     */
    protected function propertyToJSON($propKey, $propValue)
    {
        $json = '"' . addslashes($propKey) . '":';
        if ($propKey === 'toState' || $propKey === 'fromState')
        {
            $json .= '{';

            // transform the state into an assoc array
            $state = $this->getAssoc_toState();
            foreach ($state as $k => $v)
            {
                $json .= '"' . addslashes($k) . '":';
                $json .= json_encode($v) . ',';
            }

            $json = rtrim($json, ',');
            $json .= '}';
        }
        else
        {
            $json .= json_encode($propValue);
        }

        return $json;
    }
}
