<?php
/**
 * Multi thread indexation script for WCM4 & SolR
 * This script can be launched via the command line
 * or it can be called via the WCM Backoffice (Indexation page)
 */

DEFINE('CURRENT_DIR', dirname(__FILE__));

if (!function_exists('xdebug_time_index'))
{
    define('START_TIME', microtime(true));
    function xdebug_time_index()
    {
        return microtime(true) - START_TIME;
    }
}

// wcmCli class is to render nice CLI msg
require(CURRENT_DIR . '/wcmCli.php');

// Main class in charge of creating and managing processes
require(CURRENT_DIR . '/wcmThreadManager.php');

// This file requires WCM initApi
if(file_exists(CURRENT_DIR . '/../../initWcmApi.php'))
	require_once(CURRENT_DIR . '/../../initWcmApi.php');
else    
	die(wcmCli::COLOR_RED . 'This script need to be in the WCM root folder to work' . wcmCli::COLOR_NORMAL . PHP_EOL);

// Script that will perform the indexation (for each subprocesses)
$scriptName = CURRENT_DIR . '/indexation_process.php';

// Script in charge of doing a commit every $commitEvery seconds
$commitScriptName =  CURRENT_DIR . '/indexation_commit_process.php';

// default value for number of processes
$maxProcess = 4;

// Default value for number of document to add before doing a commit
$commitEvery = 60;

// Nb of document to cuncatene together when sending to Solr
$cuncateneDocument = 20;

// Default value for environment variable
$varEnv = 'qa';

// Defines if output should be html
$htmlOutput = false;

// Retrieve default solr Service 
$cfg = wcmConfig::getInstance();
$solrService = $cfg['wcm.solr.service'];
$search = wcmSearch::getInstance('solr');

// Default SQL where clause
$sqlWhere = null;

// Not indexable classes
$NotIndexable = array("clientLocation", "clientSpecification", "clientTypeSpecification", "travelSpecial", "travelSpecialDetail");

// HELP Message
$HELP = <<<EOH

NStein WCM - Solr Indexation Script ver1.0
This script allows to perform a mass indexation from WCM4 to a Solr instance. It will serialize object into XML (using WCM api
and the toXML() method) and then index it to Solr. This script is usually used to resynch WCM and Solr.

\tUsage: php {$_SERVER['argv'][0]} [className1 [className2 [...]]] [-a] [-l] [-v] [-s] [-h] [--sqlWhere] [--maxProcess] [--commitEvery] [--solrService] [--cuncateneDocument] 

\t\t-a, --all \t\tindex all objects 
\t\t-l, --list \t\tlist all objects 
\t\t-v, --verbose \t\tenable verbose mode 
\t\t-s, --stats \t\tdisplay complete stats 
\t\t-h, --help \t\tprint a summary of the options 
\t\t--sqlWhere	\tAllow to define a generic SQL where clause to limit what will be indexed for every business object
\t\t--maxProcess \t\tDefine maximum number of process to launch at same time (multi-threads, default is {$maxProcess} processes) 
\t\t--commitEvery \t\tDefine in seconds how often the commit will be performed (default is {$commitEvery} sec)
\t\t--solrService \t\tDefine what solrService will be used (default is {$cfg['wcm.solr.service']}) 
\t\t--cuncateneDocument \tDefine how many documents to cuncatene together when submitting to solr (default is {$cuncateneDocument})
\t\t--env \tDefine the environment variable for WCM (WCM_ENV, e.g. qa, dev, prod)
\t\t--htmlOutput \tDefine if the output shoulb be html (define is false)

\tExamples:

\tIndex all articles and photos belonging to site 1, use 8 processes and commit every 30 seconds : 
\t\tphp {$_SERVER['argv'][0]} article photo --sqlWhere "siteId=1" --maxProcess 8 --commitEvery 30

\tIndex all articles, use 12 processes and use a specific solr instance :
\t\tphp {$_SERVER['argv'][0]} article --maxProcess 12 --solrService http://10.10.1.11:8983/solr/myInstance/



EOH;

session_start();

$verbose = false;
$showStats = false;
$classes = array();

if($_SERVER['argc'] <= 1)
	die($HELP);

// Parsing parameter passed to the script
for($i = 1 ; $i < $_SERVER['argc'] ; $i++)
{
	switch($_SERVER['argv'][$i])
	{
	case '-h':
	case '--help':
		die($HELP);
        break;

	case '-a':
	case '--all':
		$classes = array_keys(getClassList());
		break;

	case '-l':
	case '--list':
		echo PHP_EOL . 'List of indexable objects :' . PHP_EOL . PHP_EOL;
        echo "\t" . str_pad('    Class name', 30, ' ', STR_PAD_RIGHT);
        echo "\t MySql\tSolR" . PHP_EOL;
        echo "\t" . str_pad('', 50, '=', STR_PAD_RIGHT) . PHP_EOL . PHP_EOL;
        foreach(getClassList() as $className => $val)
        {        
            if (class_exists($className) && (!in_array($className, $NotIndexable))) 
            {
                $bizobject = new $className();
                $dbCount = $bizobject->getCount();
                $indexCount = $search->getObjectCount($className);

                echo "\t";
                if($dbCount != $indexCount)
                {
                    echo "(*) ";
                }
                else
                {
                    echo "    ";
                }
                echo str_pad($className, 30, ' ', STR_PAD_RIGHT);
                echo $dbCount . "\t" . $indexCount . PHP_EOL;
            }
        }
        
		die(PHP_EOL);					
		break;

	case '-v':
	case '--verbose':
		$verbose = true; 
		break;

	case '-s':
	case '--stats':
		$showStats = true;
		break;

	case '--maxProcess':
		$maxProcess = $_SERVER['argv'][$i+1];
		++$i;
		break;

	case '--commitEvery':
		$commitEvery = $_SERVER['argv'][$i+1];
		++$i;
		break;

	case '--sqlWhere':
		$sqlWhere = $_SERVER['argv'][$i+1];
		++$i;
		break;

	case '--solrService':
		$solrService = $_SERVER['argv'][$i+1];
		++$i;
		break;
	
	case '--cuncateneDocument':
		$cuncateneDocument = $_SERVER['argv'][$i+1];
		++$i;
		break;

	case '--env':
		$varEnv = $_SERVER['argv'][$i+1];
		++$i;
		break;
	
	case '--htmlOutput':
		$htmlOutput = $_SERVER['argv'][$i+1];		
		++$i;
		break;

	default:
		$classes[] = $_SERVER['argv'][$i];
		break;
	}
}

if(empty($classes))
{
    die($HELP);
}

if('true' == $htmlOutput)
{
	wcmCli::setOutputHtml(true);
}

$stats = array();
$cptDocument = 0;
$initialMaxProcess = $maxProcess;

// Set environment variable
putenv('WCM_ENV=' . $varEnv);


echo PHP_EOL . "Starting multi-threaded indexation" . PHP_EOL;
foreach($classes as $className)
{
	if(!class_exists($className)) continue;
	if(in_array($className, $NotIndexable)) continue;
	
	$stats[$className]['errors'] = array();
	$stats[$className]['count'] = 0;
	$startTime = microtime(true);

	echo PHP_EOL .'Indexing ' . $className . '...' . PHP_EOL;
	
    // Reindex all implies removing previously indexed content
	$search->deindexobjects($className, $sqlWhere);

	$bizobject = new $className;
	$bizobject->beginEnum($sqlWhere, 'id');
	$count = $bizobject->enumCount();

	echo 'Total of documents : ' . $count . PHP_EOL;

	// If nothing to index for this object then jump to next one
	if($count == 0)
	{
		echo 'Nothing to index for object (' . $className . ')' . PHP_EOL;
		continue;
	}
	
	if($maxProcess > $count)
	{
		$maxProcess = $count;
		echo 'There cant be more processes launched than the number of documents.' . PHP_EOL;
	}

	// Number of process
	echo 'Launching ' . $maxProcess . ' process(es).' . PHP_EOL . PHP_EOL;
	
	// Create a separate thread in charge of doing the commit
	$threadCommit = wcmThreadManager::create($commitScriptName, array('--commitEvery' => $commitEvery, '--solrService' => $solrService));
	echo "\t" . $threadCommit->message . PHP_EOL . PHP_EOL;

	$cpt = 1;
	$totalProcessed = 0;
	$limit = floor($count / $maxProcess);
	$from = 0;
	$threads = array();
	$begin = microtime(true);
	while($cpt <= $maxProcess)
	{	
		// Take all remaining records if it is the last process
		if($cpt == $maxProcess)
		{
			$limit += $count % $maxProcess;
		}

		// Array containing all the different threads in charge of doing the indexation
		$thread = wcmThreadManager::create($scriptName, array(0 => $className, '--from' => $from, '--to' => $limit, '--nbDocuments' => $cuncateneDocument));
		echo "\t" . $thread->message . PHP_EOL;
		$threads[] = $thread;
		++$cpt;

		$from += $limit;
	}

	echo PHP_EOL;

	// Display progess status while threads are active
	while(wcmThreadManager::areActive($threads))
	{
		foreach($threads as $thread)
		{
			$res = $thread->listen();
			if(null != $res && $result = @unserialize($res))
			{
				$cptDocument += $result['nbDocuments'];
				$totalProcessed += $result['nbDocuments'];
				$msg = ' (' . $totalProcessed . '/' . $count . ' documents processed...)';
				echo wcmCli::displayProgress(round(($totalProcessed / $count) * 100), $msg);
				$msg = null;
			}
		}

		// Commit every $commitEvery seconds
		if(microtime(true) - $begin >= ($commitEvery))
		{
			$search->commit();
			$begin = microtime(true);
		}
	}

	foreach($threads as $thread)
	{
		$thread->close();
	}

	// Close commit thread
	$threadCommit->close();

	// Performing a last commit 
	$search->commit();
	
	echo PHP_EOL;

    // Set back maxprocess to its original value for the next object to index
    $maxProcess = $initialMaxProcess;
}

echo PHP_EOL . PHP_EOL;
echo 'Done.' . PHP_EOL;
echo 'Total time spent: ' . number_format(xdebug_time_index(), 2) . PHP_EOL . PHP_EOL;
