<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restContentController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmContentRestController class provides a RESTful interface to
 * manage objects (CRUD).
 */
class wcmRestContentController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('content');
    }

    /**
     * Gets the public object URL for the REST interface.
     *
     * @return string The public object URL
     */
    private function getObjectURL($className, $id)
    {
        $config = wcmConfig::getInstance();
        return sprintf('%s%s/%s', $this->baseURL, $className, $id);
    }

    /**
     * HTTP "GET" method.
     *
     * GET .../content/{CLASSNAME}?where={WHERE}&sort={SORT}&offset={OFFSET}&limit={LIMIT}
     *
     * {CLASSNAME} => (required) object class name
     * {WHERE}     => (optional) SQL "WHERE" clause (default is null)
     * {SORT}      => (optional) SQL "ORDER BY" clause (default is "modifiedAt DESC")
     * {OFFSET}    => (optional) zero-based index of first object (defaults to 0)
     * {LIMIT}     => (optional) zero-based index of last object (defaults to 9)
     *
     * => Gets objects of a given class matching given criteria.
     * => On success, returns the object as XML or JSON, or the object locations
     *    as XML or JSON (if "json" request parameter present).
     *
     * GET .../content/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (required) object ID
     *
     * => Gets a specific object.
     * => On success, returns the object as XML or JSON (if "json"
     *    request parameter present).
     *
     * @return wcmRestResponse => S_OK          on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on invalid object selection criteria
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid object class or ID
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Validate object class name
        $className = getArrayParameter($_GET, 'className', null);
        
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }

        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Return one or many objects?
        $wcmObject = wcmObjectFactory::create($className);
        $id = getArrayParameter($_GET, 'id', null);
        if ($id)
        {
            // Refresh object
            $wcmObject->refresh($id);
            if (!$wcmObject->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Response is object as JSON or XML
            if (isset($_REQUEST['json']))
            {
                $body = $wcmObject->toJSON();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_APPLICATION_JSON);
            }
            else
            {
                $body = $wcmObject->toXML();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_TEXT_XML);
            }

            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        // Return object locations
        $where  = getArrayParameter($_GET, 'where', null);
        $sort   = getArrayParameter($_GET, 'sort', null);
        $offset = intval(getArrayParameter($_GET, 'offset', 0));
        $limit  = intval(getArrayParameter($_GET, 'limit', 128));

        if (!$wcmObject->beginEnum($where, $sort, $offset, $limit))
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CRITERIA);
        }

        // Response is collection of object locations as JSON or XML
        if (isset($_REQUEST['json']))
        {
            $output = array();
            while ($wcmObject->nextEnum())
            {
                $output[] = $this->getObjectURL($className, $wcmObject->id) . '?json';
            }

            $body = json_encode($output);
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $output = wcmRestResponse::XML_DECL . PHP_EOL;
            $output .= '<response>' . PHP_EOL;
            while ($wcmObject->nextEnum())
            {
                $output .= '<location>'
                    . $this->getObjectURL($className, $wcmObject->id)
                    . '</location>' . PHP_EOL;
            }
            $output .= '</response>' . PHP_EOL;

            $body = $output;
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_TEXT_XML);
        }
        $wcmObject->endEnum();

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "PUT" method.
     *
     * PUT .../content/{CLASSNAME}
     *
     * {CLASSNAME} => (required) object class name
     *
     * => Creates a new object.
     * => Assumes request body specifies object content (XML or JSON).
     * => On success, returns a "Location" header pointing to the object.
     *
     * PUT .../content/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (required) object ID
     *
     * => Updates an existing object.
     * => Assumes request body specifies object content (XML or JSON).
     * => On success, returns a "Location" header pointing to the object.
     *
     * @returns wcmRestResponse => S_CREATED      on success
     *                          => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_BAD_REQUEST  on invalid object content
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => S_NOT_FOUND    on invalid object class or ID
     *                          => S_BAD_REQUEST  on save failure
     */
    protected function put()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get the content of the PUT request
        list($content, $contentType) = $this->getPutContent();
        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Let the internal method handle the rest
        return $this->putOrPost($content, $contentType);
    }

    /**
     * HTTP "POST" method.
     *
     * => Currently does the same thing as the HTTP "PUT" method.
     */
    protected function post()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get the content of the POST request
        list($content, $contentType) = $this->getPostContent('content');
        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Let the internal method handle the rest
        return $this->putOrPost($content, $contentType);
    }

    /**
     * Internal HTTP "PUT" or "POST" method.
     *
     * This method exists because PHP uses two different means of
     * providing the input data for the PUT and POST methods:
     *
     * PUT  => content comes from input stream "php://input"
     * POST => content comes from variable $_POST
     *
     * @param string $content     The PUT or POST content
     * @param string $contentType The MIME content type
     *
     * @returns wcmRestResponse => S_CREATED      on success
     *                          => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_BAD_REQUEST  on invalid object content
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => S_NOT_FOUND    on invalid object class or ID
     *                          => S_BAD_REQUEST  on save failure
     */
    private function putOrPost($content, $contentType)
    {
        // Validate content type
        $json = false;
        if (strncasecmp(
            $contentType, wcmRestResponse::H_CT_APPLICATION_JSON,
            strlen(wcmRestResponse::H_CT_APPLICATION_JSON)
        ) === 0)
        {
            $json = true;
        }
        elseif (strncasecmp(
            $contentType, wcmRestResponse::H_CT_TEXT_XML,
            strlen(wcmRestResponse::H_CT_TEXT_XML)
        ) !== 0)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                       _REST_INVALID_OBJECT_CONTENT_TYPE);
        }

        // Validate class name
        $className = getArrayParameter($_REQUEST, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Create or update object?
        $wcmObject = wcmObjectFactory::create($className);
        $id = getArrayParameter($_REQUEST, 'id', null);
        if ($id)
        {
            // Refresh existing object
            $wcmObject->refresh($id);
            if (!$wcmObject->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }
        }

        // If the object content has source information, try to first
        // refresh the object from that source information
        $source = null;
        if ($json)
        {
            if (!empty($content->source[0]) && !empty($content->sourceId[0]))
            {
                $wcmObject->refreshFromSource($content->source[0], $content->sourceId[0]);
            }
        }
        else
        {
            // supress simplexml warnings, the warnings are accessible from libxml_get_errors()
            libxml_use_internal_errors(true);
            $xml = simplexml_load_string($content);
            if (!$xml)
            {
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                           _REST_INVALID_OBJECT_CONTENT);
            }

            if ($xml->source[0] && $xml->sourceId[0])
            {
                $wcmObject->refreshFromSource((string)$xml->source[0], (string)$xml->sourceId[0]);
            }
        }

        // Initialize object from content
        try
        {
            if ($json)
            {
                if (!$wcmObject->initFromJSON($content))
                {
                    throw new exception('Invalid JSON');
                }
            }
            else
            {
                $wcmObject->initFromXML($content);
            }
        }
        catch (Exception $e)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Check that the object class and ID in the object content
        // match the ones given in the request
        if ($className != $wcmObject->getType() || (isset($id) && $id != $wcmObject->id))
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Save object
        if (!$wcmObject->save())
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, $wcmObject->getErrorMsg());
        }

        // The response is a "Location" header pointing to the object
        $url = $this->getObjectURL($className, $wcmObject->id);
        $headers = array(wcmRestResponse::H_LOCATION => $url);

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $wcmObject->id, $headers);
    }

    /**
     * HTTP "DELETE" method.
     *
     * DELETE .../content/{CLASSNAME}?where={WHERE}
     *
     * {CLASSNAME} => (required) object class name
     * {WHERE}     => (optional) SQL "WHERE" clause (default is null)
     *
     * DELETE .../content/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (required) object ID
     *
     * => deletes a specific object
     *
     * @return wcmRestResponse => S_NO_CONTENT   on success
     *                         => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND    on invalid session ID
     *                         => S_NOT_FOUND    on invalid object class or ID
     *                         => S_UNAUTHORIZED on deletion failure
     */
    protected function delete()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Validate object class name
        $className = getArrayParameter($_REQUEST, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Create object
        $wcmObject = wcmObjectFactory::create($className);
        $id = getArrayParameter($_REQUEST, 'id', null);
        if ($id)
        {
            // Refresh object
            $wcmObject->refresh($id);
            if (!$wcmObject->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Delete object
            if (!$wcmObject->delete())
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $wcmObject->getErrorMsg());
            }
        }
        else
        {
            // Delete objects matching criteria
            $where = getArrayParameter($_GET, 'where', null);
            if (!$wcmObject->beginEnum($where))
            {
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CRITERIA);
            }

            while ($wcmObject->nextEnum())
            {
                $wcmObject->delete();
            }

            $wcmObject->endEnum();
        }

        return new wcmRestResponse(wcmRestResponse::S_NO_CONTENT);
    }
}
