<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restController.php 9389 2010-04-06 19:57:40Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The wcmRestController class helps to implement a REST controller on
 * the back-end.
 *
 * The controller handles default HTTP methods (get / put / post /
 * delete) and echoes a valid HTTP response (see wcmRestResponse).
 *
 * By default, requests are processed through the following base URL:
 * {wcm.backOffice.url}/rest/{mode}/
 *
 * Note that on the back-end (see the rest/ directory) the assumption
 * is made that the class matching a mode will follow the pattern
 * 'wcmRest{mode}Controller', e.g. 'content' =>
 * wcmRestContentController.
 */
abstract class wcmRestController
{
    /**
     * Last response sent by the controller.
     * @var wcmRestResponse
     */
    private $lastResponse;

    /**
     * Base URL of the REST controller.
     * @var string
     */
    protected $baseURL;

    /**
     * Controller mode.
     * @var string
     */
    protected $mode;

    /**
     * HTTP protocol (e.g. 'HTTP/1.1').
     * @var string
     */
    protected $protocol;

    /**
     * HTTP method (e.g. 'get', 'put', 'post', 'delete').
     * @var string
     */
    protected $method;

    /**
     * HTTP request headers.
     * @var array
     */
    protected $headers;

    /**
     * Path of the HTTP request.
     * @var string
     */
    protected $path;

    /**
     * Query string of the HTTP request.
     * @var string
     */
    protected $query;

    /**
     * The current session
     */
    protected $session;

    /**
     * Constructor
     *
     * @param string $mode Optional controller mode used to infer the base URL
     *
     */
    public function __construct($mode = null)
    {
        // Infer the base URL
        $config = wcmConfig::getInstance();
        $this->baseURL = $config['wcm.rest.url'];
        if ($mode)
        {
            $this->baseURL .= $mode . '/';
            $this->mode = $mode;
        }
    }

    /**
     * Processes an HTTP request (analyses the protocol and method, then
     * invokes the corresponing plug-in method).
     *
     * This method sends back a valid HTTP response (see
     * wcmRestResponse) with the corresponding header according to the
     * method.
     *
     */
    public function processRequest()
    {
        // Retrieve the HTTP protocol & method
        $this->protocol = getArrayParameter($_SERVER, 'SERVER_PROTOCOL', 'HTTP/1.1');
        $this->method = getArrayParameter($_SERVER, 'REQUEST_METHOD', 'get');

        // Normalize the request header names to lowercase
        $this->headers = array_change_key_case(getallheaders());

        // Parse the request
        $url = parse_url($_SERVER['REQUEST_URI']);
        $this->path = getArrayParameter($url, 'path');
        $this->query = getArrayParameter($url, 'query');

        $method = $this->method;

        try
        {
            $this->lastResponse = $this->{$method}();
            $this->lastResponse->send();
        }
        catch (Exception $e)
        {
            // Log and return an internal-error message (HTTP response code 500)
            $logger = wcmProject::getInstance()->logger;
            $logger->logError('REST controller error: ' . $e->getTraceAsString());
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, $e->getMessage());
        }
    }

    /**
     * Returns the last response sent by the controller.
     *
     * @return wcmRestResponse The last response sent by the processRequest method
     */
    public function getLastResponse()
    {
        return $this->lastResponse;
    }

    /**
     * Initializes the session from the X-Nstein-Session-ID request
     * header.
     *
     * @return null|wcmRestResponse => null on success
     *                              => response to return on error
     */
    protected function initSession()
    {
        // Get session ID from X-Nstein-Session-ID header
        $sessionId = wcmRestSession::getInternalSessionId(true);
        if ($sessionId === null || $sessionId === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, wcmRestSession::getErrorMsg());
        }

        // Get corresponding session
        $session = wcmRestSession::getSession($sessionId);
        if ($session === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, wcmRestSession::getErrorMsg());
        }
        if ($session === false)
        {
            return new wcmRestResponse(wcmRestResponse::S_GONE, wcmRestSession::getErrorMsg());
        }
        $resultSet = $session->getProject()->database->executeQuery('select count(*) from wcm_session where id=' . $session->id);

        // Start session as authenticated user, startSession will return false the second time if
        // called twice in under a second so we also make sure the count of a select for the session returns 0
        // before returning an error
        if (!$session->startSession($session->userId) && $resultSet->count() === 0)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR);
        }

        $this->session = $session;
        return null;
    }

    /**
     * Gets the content (body) of a PUT request.
     *
     * @return array|null The content and content type (in that order), or null on error
     */
    protected function getPutContent()
    {
        // Get content type
        if (isset($this->headers[wcmRestResponse::H_CONTENT_TYPE]))
        {
            $contentType = $this->headers[wcmRestResponse::H_CONTENT_TYPE];
        }
        else
        {
            $contentType = wcmRestResponse::H_CT_X_WWW_FORM_URLENCODED;
        }

        // Content comes from input stream "php://input"
        $input = fopen('php://input', 'r');
        if ($input === false)
        {
            return null;
        }

        // Read content
        $content = '';
        while ($chunk = fread($input, 1024))
        {
            $content .= $chunk;
        }

        // Close input stream
        fclose($input);

        // Check for error
        if ($chunk === false)
        {
            return null;
        }

        return array($content, $contentType);
    }

    /**
     * Gets the content of a POST request.
     *
     * If the request content type is 'x-www-form-urlencoded', gets
     * the content from the $_POST request parameter named
     * $contentName and URL-decodes it.
     *
     * If the request content type is 'multipart/form-data', gets the
     * content from the file named $contentName in $_FILES array.
     *
     * Otherwise, assumes that the content is the raw request body.
     *
     * @param string $contentName The content name
     *
     * @return array|null The content and content type (in that order), or null on error
     */
    protected function getPostContent($contentName)
    {
        // Get content type
        if (isset($this->headers[wcmRestResponse::H_CONTENT_TYPE]))
        {
            $contentType = $this->headers[wcmRestResponse::H_CONTENT_TYPE];
        }
        else
        {
            $contentType = wcmRestResponse::H_CT_X_WWW_FORM_URLENCODED;
        }

        $content = null;
        if (strncasecmp(
            $contentType, wcmRestResponse::H_CT_MULTIPART_FORM_DATA,
            strlen(wcmRestResponse::H_CT_MULTIPART_FORM_DATA)
        ) === 0)
        {
            // Get the content from the named file or form data
            if (isset($_FILES[$contentName]))
            {
                $file = $_FILES[$contentName];
                $contentType = $file['type'];
                $content = file_get_contents($file['tmp_name']);
                if ($content === false)
                {
                    $content = null;
                }
            }
            elseif (isset($_POST[$contentName]))
            {
                $content = getArrayParameter($_POST, $contentName, null);
                if (isset($_POST['json']))
                {
                    $contentType = wcmRestResponse::H_CTCS_APPLICATION_JSON;
                }
                else
                {
                    $contentType = wcmRestResponse::H_CTCS_TEXT_XML;
                }
            }
        }
        elseif (strncasecmp(
            $contentType, wcmRestResponse::H_CT_X_WWW_FORM_URLENCODED,
            strlen(wcmRestResponse::H_CT_X_WWW_FORM_URLENCODED)
        ) === 0)
        {
            // Get the content from the named request parameter
            $content = getArrayParameter($_POST, $contentName, null);
            if ($content)
            {
                $content = $content;
                if (isset($_POST['json']))
                {
                    $contentType = wcmRestResponse::H_CTCS_APPLICATION_JSON;
                }
                else
                {
                    $contentType = wcmRestResponse::H_CTCS_TEXT_XML;
                }
            }
        }
        else
        {
            // TODO how to get raw POST data?
        }

        return array($content, $contentType);
    }

    /**
     * Handles the 'get' method and sends the corresponding HTTP
     * response.
     *
     * This method should be overriden by the concreate subclass,
     * otherwise an HTTP 501 response ('Not Implented') will be
     * returned.
     *
     * @return wcmRestResponse HTTP response to return to the caller
     */
    protected function get()
    {
        // Returns a 'Not Implemented' HTTP response (header code 501)
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * Handles the 'put' method and sends the corresponding HTTP
     * response.
     *
     * This method should be overriden by the concreate subclass,
     * otherwise an HTTP 501 response ('Not Implented') will be
     * returned.
     *
     * @return wcmRestResponse HTTP response to return to the caller
     */
    protected function put()
    {
        // Returns a 'Not Implemented' HTTP response (header code 501)
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * Handles the 'post' method and sends the corresponding HTTP
     * response.
     *
     * This method should be overriden by the concreate subclass,
     * otherwise an HTTP 501 response ('Not Implented') will be
     * returned.
     *
     * @return wcmRestResponse HTTP response to return to the caller
     */
    protected function post()
    {
        // Returns a 'Not Implemented' HTTP response (header code 501)
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }

    /**
     * Handles the 'delete' method and sends the corresponding HTTP
     * response.
     *
     * This method should be overriden by the concreate subclass,
     * otherwise an HTTP 501 response ('Not Implented') will be
     * returned.
     *
     * @return wcmRestResponse HTTP response to return to the caller
     */
    protected function delete()
    {
        // Returns a 'Not Implemented' HTTP response (header code 501)
        return new wcmRestResponse(wcmRestResponse::S_NOT_IMPLEMENTED);
    }
}
