<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restGenerationController.php 9389 2010-04-06 19:57:40Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The wcmRestGenerationController class provides a RESTful interface
 * to launch content generations.
 */
class wcmRestGenerationController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('generation');
    }

    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }
        
        // Determine method to call based on resource
        $resource = getArrayParameter($_REQUEST, 'resource', 'generation');

        $id = getArrayParameter($_REQUEST, 'id', null);
           
        if (empty($id))
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_PARAMETER);
        }
        
        //classname is set or template or content
        if ($resource == 'generation')
        {
            $className = 'wcmGeneration';
        }
        else
        {
            $className = 'wcmGeneration' . ucfirst($resource);
        }
        
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Create the object
        $wcmObject = wcmObjectFactory::create($className);

        // Refresh existing object
        $wcmObject->refresh($id);
        
        if (!$wcmObject->id)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
        }
        
        // Response is object as JSON or XML
        if (strcasecmp($_REQUEST['json'], 'true') === 0)
        {
            $body = $wcmObject->toJSON();
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body = $wcmObject->toXML();
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_TEXT_XML);
        }
                                 
        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }
    /**
     * HTTP "POST" method.
     *
     * if $_REQUEST['id'] is set, execute the corresponding generation/generationContent/generationSet/generationTemplate
     * if $_REQUEST['id'] is not set, create the corresponding object
     *
     * @see postGeneration
     * @see postGenerationContent
     * @see postGenerationSet
     * @see postGenerationTemplate
     *
     * @see createGenerationObject
     *
     * @return wcmRestResponse => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid resource
     *                         => responses from post<Resource> method
     */
    protected function post()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Determine method to call based on resource
        $resource = getArrayParameter($_REQUEST, 'resource', 'generation');

        $id = getArrayParameter($_REQUEST, 'id', null);
        $code = getArrayParameter($_REQUEST, 'code', null);

        if (!$id && !$code)
        {
            $method = 'createGenerationObject';
        }
        else
        {
            $method = 'postGeneration' . (($resource != 'generation') ? ucfirst($resource) : '');
        }

        if (!method_exists($this, $method))
        {
            return wcmRestResponse::invalidParameter('resource', $resource);
        }

        // Get resource
        return $this->$method();
    }

    /**
     * HTTP "PUT" method.
     *
     * PUT .../generation/{ID}
     * PUT .../generation/content/{ID}
     * PUT .../generation/set/{ID}
     * PUT .../generation/template/{ID}
     *
     * {ID}        => (required) object ID - If ID is null, create an object
     *
     * => Updates an existing object.
     * => Assumes request body specifies object content (XML or JSON).
     * => On success, returns a "Location" header pointing to the object.
     *
     * @returns wcmRestResponse => S_CREATED      on success
     *                          => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_BAD_REQUEST  on invalid object content
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => S_NOT_FOUND    on invalid object class or ID
     *                          => S_UNAUTHORIZED on save failure
     */
    protected function put()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        $id = getArrayParameter($_REQUEST, 'id', null);
        //if (!$id)
        //    return wcmRestResponse::missingParameter('id');

        // Get the content of the PUT request
        list($content, $contentType) = $this->getPutContent();
        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        return $this->createOrUpdate($id, $content, $contentType);
    }

    /**
     * HTTP "POST" method for "generation" resource.
     *
     * Executes a generation.
     *
     * RESOURCE:
     *
     * generation
     *
     * QUERY PARAMETERS:
     *
     * id   => [required] Generation ID
     * extraContext => [optional] extraContext in the form name1:value1,name2:value2,etc...
     * json => [optional] Return JSON representation if present, XML otherwise
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */
    private function postGeneration()
    {
        $id = getArrayParameter($_REQUEST, 'id', null);

        $extraContextAssoc = $this->getExtraContext();

        $generator = new wcmTemplateGenerator;
        $generation = new wcmGeneration();
        if (!$generation->refresh($id))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_RESOURCE);
        }

        $results = $generator->executeGeneration($generation, $extraContextAssoc, true);

        if ($results === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, _REST_EXECUTE_GENERATION_FAILED);
        }

        list($body, $headers) = $this->getExecutionResponse($results);

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $body, $headers);
    }

    /**
     * HTTP "POST" method for "generation" resource.
     *
     * Executes a generation.
     *
     * RESOURCE:
     *
     * generation/content
     *
     * QUERY PARAMETERS:
     *
     * id   => [required] Generation Content ID
     * or
     * code   =>[required] Generation Content CODE
     * extraContext => [optional] extraContext in the form name1:value1,name2:value2,etc...
     * json => [optional] Return JSON representation if present, XML otherwise
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */

    private function postGenerationContent()
    {
        $id = getArrayParameter($_REQUEST, 'id', null);
        $code = getArrayParameter($_REQUEST, 'code', null);
        $previewMode = getArrayParameter($_REQUEST, 'previewMode', false);
        $returnOutput = getArrayParameter($_REQUEST, 'returnOutput', false);

        $extraContext = $this->getExtraContext();
        
        $mode = wcmTemplateGenerator::MODE_NORMAL;
        if ($previewMode)
        {
            $mode = wcmTemplateGenerator::MODE_PREVIEW;
        }

        $generator = new wcmTemplateGenerator(null, true, wcmWidget::VIEW_CONTENT, $mode);
        $generationContent = new wcmGenerationContent();

        if (null !== $id)
        {
            if (!$generationContent->refresh($id))
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }
        }
        else
        {
            if (!$generationContent->refreshByCode($code))
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CODE);
            }
        }

        $results = $generator->executeGenerationContent($generationContent, $extraContext, !$returnOutput, false);

        if ($results === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, _REST_EXECUTE_GENERATION_FAILED);
        }

        list($body, $headers) = $this->getExecutionResponse($results);

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $body, $headers);
    }

    /**
     * HTTP "POST" method for "generation" resource.
     *
     * Executes a generation set.
     *
     * RESOURCE:
     *
     * generation/set
     *
     * QUERY PARAMETERS:
     *
     * id   => [required] Generation Set ID
     * extraContext => [optional] extraContext in the form name1:value1,name2:value2,etc...
     * json => [optional] Return JSON representation if present, XML otherwise
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */
    private function postGenerationSet()
    {
        $id = getArrayParameter($_REQUEST, 'id', null);

        $extraContextAssoc = $this->getExtraContext();

        $generator = new wcmTemplateGenerator;
        $generationSet = new wcmGenerationSet();
        
        if (!$generationSet->refresh($id))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
        }
        
        $results = $generator->executeGenerationSet($generationSet, $extraContext, true);

        if ($results === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, _REST_EXECUTE_GENERATION_FAILED);
        }

        //@TODO executeGenerationSet returns nothing, so an internal_server_error might be returned. To correct.
        list($body, $headers) = $this->getExecutionResponse($results);

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $body, $headers);
    }

    /**
     * HTTP "POST" method for "generation" resource.
     *
     * Executes a generation template.
     *
     * RESOURCE:
     *
     * generation/template
     *
     * QUERY PARAMETERS:
     *
     * id   => [required] Generation Template ID
     * extraContext => [optional] extraContext (equivalent to parameters in WS) in the form name1:value1,name2:value2,etc...
     * json => [optional] Return JSON representation if present, XML otherwise
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */
    private function postGenerationTemplate()
    {
        $id = getArrayParameter($_REQUEST, 'id', null);

        $config = wcmConfig::getInstance();

        // Check that the template file specified by $id exists
        if ($id == null || !file_exists($config['wcm.templates.path'] . $id))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_EXECUTE_GENERATION_TEMPLATE_NOT_FOUND);
        }

        $extraContextAssoc = $this->getExtraContext();

        $generator = new wcmTemplateGenerator;
        $template = new wcmTemplate();
        $template->refresh($id);
        $results = $generator->executeTemplate($template, $extraContextAssoc);

        if ($results === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, _REST_EXECUTE_GENERATION_FAILED);
        }

        //executeTemplate returns a string
        $body = null;
        $headers = array();
        if ($results)
        {
            if (isset($_REQUEST['json']))
            {
                $body = json_encode($results);
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
            }
            else
            {
                $body  = wcmRestResponse::XML_DECL . PHP_EOL;
                $body .= '<templateContent>' . PHP_EOL;
                $body .= $results;
                $body .= '</templateContent>' . PHP_EOL;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;
            }
        }

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $body, $headers);
    }


    /**
     * HTTP "POST" method for "generation" resource.
     *
     * Creates a generation.
     *
     * RESOURCE:
     *
     * generation
     * generation/content
     * generation/set
     * generation/template
     *
     * QUERY PARAMETERS:
     *
     * => Assumes request body specifies object content (XML or JSON).
     * => On success, returns a "Location" header pointing to the object.
     *
     * @returns wcmRestResponse => S_CREATED      on success
     *                          => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_BAD_REQUEST  on invalid object content
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => S_NOT_FOUND    on invalid object class or ID
     *                          => S_UNAUTHORIZED on save failure
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */
    private function createGenerationObject()
    {
        // Get the content of the POST request
        list($content, $contentType) = $this->getPostContent('content');
        if ($content === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        return $this->createOrUpdate(null, $content, $contentType);
    }

    /**
     * createOrUpdate
     * Helper method for creating or updating a generation
     * Called by crateGenerationObject (from post) and from put
     *
     * @param string $content     The PUT or POST content
     * @param string $contentType The MIME content type
     *
     * @returns wcmRestResponse => S_CREATED      on success
     *                          => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                          => S_BAD_REQUEST  on invalid object content
     *                          => S_NOT_FOUND    on invalid session ID
     *                          => S_NOT_FOUND    on invalid object class or ID
     *                          => S_UNAUTHORIZED on save failure
     *
     */
    private function createOrUpdate($id, $content, $contentType)
    {
        // Validate content type
        $json = false;
        if (strncasecmp(
            $contentType, wcmRestResponse::H_CT_APPLICATION_JSON,
            strlen(wcmRestResponse::H_CT_APPLICATION_JSON)
        ) === 0)
        {
            $json = true;
        }
        elseif (strncasecmp(
            $contentType, wcmRestResponse::H_CT_TEXT_XML,
            strlen(wcmRestResponse::H_CT_TEXT_XML)
        ) !== 0)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                       _REST_INVALID_OBJECT_CONTENT_TYPE);
        }

        // Validate class name
        $className = $resource = getArrayParameter($_REQUEST, 'resource', 'generation');

        //classname is set or template or content
        if ($className == 'generation')
        {
            $className = 'wcmGeneration';
        }
        else
        {
            $className = 'wcmGeneration' . ucfirst($className);
        }

        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Create or update object?
        $wcmObject = wcmObjectFactory::create($className);

        if ($id)
        {
            // Refresh existing object
            $wcmObject->refresh($id);
            if (!$wcmObject->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }
        }

        // If the object content has source information, try to first
        // refresh the object from that source information
        $source = null;
        if ($json)
        {
            if ($content->source[0] && $content->sourceId[0])
            {
                $wcmObject->refreshFromSource($content->source[0], $content->sourceId[0]);
            }
        }
        else
        {
            // this will supress the warnings simplexml issues
            libxml_use_internal_errors(true);
            $xml = simplexml_load_string($content);
            
            if (!$xml)
            {
                $message = null;
                
                if (is_array(libxml_get_errors()))
                {
                    $message = 'XML error';
                
                    foreach (libxml_get_errors() as $error) 
                    {
                        $message .= "\n\t" . $error->message;
                    }
                }

                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                           _REST_INVALID_OBJECT_CONTENT . PHP_EOL . $message);
            }

            if ($xml->source[0] && $xml->sourceId[0])
            {
                $wcmObject->refreshFromSource((string)$xml->source[0], (string)$xml->sourceId[0]);
            }
        }

        // Initialize object from content
        try
        {
            if ($json)
            {
                $wcmObject->initFromJSON($content);
            }
            else
            {
                $wcmObject->initFromXML($content);
            }
        }
        catch (Exception $e)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Check that the object class and ID in the object content
        // match the ones given in the request
        if ($className != $wcmObject->getType() || (isset($id) && $id != $wcmObject->id))
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CONTENT);
        }

        // Save object
        if (!$wcmObject->save())
        {
            return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $wcmObject->getErrorMsg());
        }

        // The response is a "Location" header pointing to the object
        $url = $this->getObjectURL($className, $wcmObject->id);
        $headers = array(wcmRestResponse::H_LOCATION => $url);

        return new wcmRestResponse(wcmRestResponse::S_CREATED, $wcmObject->id, $headers);

    }

    /**
     *
     * Executes general post preprocessing tasks
     * Fetches the extraContext
     *
     * @return array assoc array
     */

    private function getExtraContext()
    {
        $extraContextAssoc = array();
        $extraContext = getArrayParameter($_REQUEST, 'extraContext');
        if ($extraContext)
        {
            // string exemple : name:value,name:value
            if (is_string($extraContext))
            {
                $extraContext = explode(',', $extraContext);
                foreach ($extraContext as $pair)
                {
                    list($name, $value) = explode(':', $pair);
                    $extraContextAssoc[$name] = $value;
                }
            }
            // array
            else
            {
                foreach ($extraContext as $name => $value)
                {
                    $extraContextAssoc[$name] = $value;
                }
            }
            
        }

        return $extraContextAssoc;
    }

    /**
     * @TODO put this method in wcmRestController
     * Gets the public object URL for the REST interface.
     *
     * @return string The public object URL
     */
    private function getObjectURL($className, $id)
    {
        $config = wcmConfig::getInstance();
        
        $resource = null;
        
        // Map the class name to a resource name
        if ($className === 'wcmGenerationSet')
        {
            $resource = 'set';
        }
        elseif($className === 'wcmGenerationContent')
        {
            $resource = 'content';
        }

        if (empty($resource))
        {
            $response = sprintf('%s%s', $this->baseURL, $id);
        }
        else
        {
            $response = sprintf('%s%s/%s', $this->baseURL, $resource, $id);
        }
        
        return $response;
    }

    /**
     *
     * Executes general post postprocessing tasks
     * constructs the body of the Rest response
     *
     * @param array $result associative arrays
     *              (structured as "name" => output file name, "content => generation result)
     * 				It is the actual result returned by executeGeneration, executeGenerationContent or executeTemplate from wcmTemplateGenerator
     *
     * @return array [$body, $header]
     */
    private function getExecutionResponse($results)
    {
        $body = null;
        $headers = array();
        if ($results)
        {
            if (isset($_REQUEST['json']))
            {
                $body = json_encode($results);
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
            }
            else
            {
                $body  = wcmRestResponse::XML_DECL . PHP_EOL;
                $body .= '<generation-list>' . PHP_EOL;
                foreach ($results as $result)
                {
                    $body .= '<generation>' . PHP_EOL;
                    $body .= '<name>' . wcmXML::xmlEncode($result['name']) . '</name>' . PHP_EOL;
                    $body .= '<content>' . wcmXML::xmlEncode($result['content']) . '</content>' . PHP_EOL;
                    $body .= '</generation>' . PHP_EOL;
                }
                $body .= '</generation-list>' . PHP_EOL;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;
            }
        }

        return array($body, $headers);
    }
}
