<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restSearchController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The class wcmRestSearchController provides a RESTful interface to
 * perform searches and manage search indexes.
 */
class wcmRestSearchController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('search');
    }

    /**
     * HTTP "GET" method.
     *
     * @see getResults
     * @see getFacets
     *
     * @return wcmRestResponse => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on missing search resource
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid search resource
     *                         => responses from get{Results|Facets}
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get required resource ('results' or 'facets')
        $resource = getArrayParameter($_GET, 'resource', null);
        if (!$resource)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_RESOURCE);
        }

        $method = 'get' . ucfirst($resource);
        if (!method_exists($this, $method))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_RESOURCE);
        }

        // Get search instance
        $search = wcmSearch::getInstance();
        if ($search === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                       wcmSearch::getGlobalErrorMsg());
        }

        return $this->$method($search);
    }

    /**
     * HTTP "GET" method for "results" resource.
     *
     * GET .../search/results?query={QUERY}&sort={SORT}&offset={OFFSET}&limit={LIMIT}
     * GET .../search/results/{ID}?query={QUERY}&sort={SORT}&offset={OFFSET}&limit={LIMIT}
     *
     * {ID}     => (optional) Search ID (default is null to auto-generate an ID)
     * {QUERY}  => (optional) Lucene query (default is null to use the result set of a previous search)
     * {SORT}   => (optional) Sorting criteria (default is null to leave unsorted)
     * {OFFSET} => (optional) Zero-based offset of first result to return (default is 0)
     * {LIMIT}  => (optional) Maximum number of result to return (default is 0)
     *
     * Gets the results of a search.
     *
     * If {QUERY} is specified, performs a new search and associates
     * the results with an auto-generated ID, or the given one if {ID}
     * is specified. Otherwise, returns the results of the previous
     * search identified by the given ID. In this case, a missing or
     * empty search ID causes an error to occur.
     *
     * If {SORT} is specified along with {QUERY}, sorts the results
     * according to the SQL-style sorting criteria. Note that
     * specifying sorting criteria with a query is an error.
     *
     * @param wcmSearch $search The search instance
     *
     * @see wcmRestSearchController::get()
     */
    private function getResults(wcmSearch $search)
    {
        // Complain if neither a search ID nor a query were given
        $searchId = getArrayParameter($_GET, 'searchId', null);
        $query = getArrayParameter($_GET, 'query', null);
        if ($searchId === null && $query === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_SEARCH_QUERY);
        }

        // Complain if sorting criteria were given without a query
        $sort = getArrayParameter($_GET, 'sort', null);
        if ($sort !== null && $query === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_SEARCH_QUERY);
        }

        // Perform new search if a query was given
        $elapsedTime = null;
        $numItemsFound = null;
        if ($query !== null)
        {
            // Automatically generate a search ID if none was given or it is empty
            if (!$searchId)
            {
                $searchId = uniqid('wcmRestSearchController_getResults_');
            }

            // Perform the search
            $startTime = microtime(true);
            $numItemsFound = $search->search($searchId, $query, $sort);
            $elapsedTime = microtime(true) - $startTime;

            if ($numItemsFound === null)
            {
                return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                           $search->getErrorMsg());
            }
        }
        else
        {
            // Set ID of previous search
            $search->setSearchId($searchId);
        }

        // Get the search results
        $json   = isset($_GET['json']);
        $offset = getArrayParameter($_GET, 'offset', 0);
        $limit  = getArrayParameter($_GET, 'limit', 128);
        
        if (!is_numeric($offset))
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, 'Offset must be an integer');
        }

        if (!is_numeric($limit))
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, 'Limit must be an integer');
        }

        if ( $limit <= 0 )
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, 'Limit must be greater than or equal to 0');
        }

        if ( $offset < 0 )
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, 'Offset must be greater than or equal to 0');
        }
        
        // retreive the search results
        $items = $search->getSearchResults($offset, $offset + $limit - 1, !$json);
        
        if ($items === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                       $search->getErrorMsg());
        }

        // The response is the requested range of result set items as JSON or XML
        if ($json)
        {
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
            $body = '{';

            if ($elapsedTime)
            {
                $body .= '"elapsedTime":' . $elapsedTime . ',';
            }
            if ($numItemsFound)
            {
                $body .= '"numResults":' . $numItemsFound . ',';
            }

            $body .= '"resultSet":[';
            if ($items)
            {
                foreach ($items as $item)
                {
                    $body .= $item->toJSON() . ',';
                }
            }
            $body = rtrim($body, ',');
            $body .= ']';

            $body .= '}';
        }
        else
        {
            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);

            $body  = wcmRestResponse::XML_DECL . PHP_EOL;
            $body .= '<search>' . PHP_EOL;

            if ($elapsedTime)
            {
                $body .= '<elapsedTime>' . $elapsedTime . '</elapsedTime>' . PHP_EOL;
            }
            if ($numItemsFound)
            {
                $body .= '<numResults>' . $numItemsFound . '</numResults>' . PHP_EOL;
            }
            if ($items)
            {
                $body .= $items . PHP_EOL;
            }

            $body .= '</search>' . PHP_EOL;
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "GET" method for "facets" resource.
     *
     * GET .../search/facets?query={QUERY}&facets={FACETS}&sort={SORT}&limit={LIMIT}
     * GET .../search/facets/{ID}?query={QUERY}&facets={FACETS}&sort={SORT}&limit={LIMIT}
     *
     * {ID}     => (optional) Search ID (default is null to auto-generate an ID)
     * {QUERY}  => (optional) Lucene query (default is null to use the result set of a previous query)
     * {FACETS} => (optional) CSV list of facets (default is null to cover all facets)
     * {SORT}   => (optional) Whether to combine and sort the facet values (default is true)
     * {LIMIT}  => (optional) Maximum number of facet values to return (default is 128)
     *
     * Gets the facet values for a given list of facets.
     *
     * If {QUERY} is specified, performs a new search and returns the
     * facet values occurring in the result set of that
     * search. Otherwise, returns the facet values occuring in the
     * result set of the previous search identified by the given
     * ID. In this case, a missing or empty search ID causes an error
     * to occur.
     *
     * If {FACETS} is specified, returns the facet values
     * corresponding to the given facets, where {FACETS} is expected
     * to be a CSV list of {NAME}[:{MAX-VALUES}] pairs and
     * {MAX-VALUES} defaults to 10. Otherwise, returns the facet
     * values for all facets.
     *
     * If {SORT} is specified, combines and sorts the facet values,
     * disregarding the facet name. Possible values are 'false' and '0'
     * for false, anything else for true.
     *
     * @param wcmSearch $search The search instance
     *
     * @returns wcmRestResponse => S_OK           on success
     *                          => S_BAD_REQUEST  on invalid facet
     *                          => S_UNAUTHORIZED on get facet values failure
     *
     * @see wcmRestSearchController::get()
     */
    private function getFacets(wcmSearch $search)
    {
        // Complain if neither a search ID nor a query were given
        $searchId = getArrayParameter($_GET, 'searchId', null);
        $query = getArrayParameter($_GET, 'query', null);
        if ($searchId === null && $query === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_MISSING_SEARCH_QUERY);
        }

        // Perform a new search if a query was given
        if ($query !== null)
        {
            // Auto-generate a search ID if none was given or it is empty
            if (!$searchId)
            {
                $searchId = uniqid('wcmRestSearchController_getFacets');
            }

            // Perform the search
            if ($search->search($searchId, $query) === null)
            {
                return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                           $search->getErrorMsg());
            }
        }

        // Convert request facets parameter to assoc array of facet
        // name => maximum values
        $facetsAssoc = array();
        $facetInfos = wcmSearchFacetValue::getFacetInfos();
        $facets = getArrayParameter($_GET, 'facets', null);
        if ($facets)
        {
            foreach (explode(',', $facets) as $facet)
            {
                list($name, $maxValues) = explode(':', $facet, 2);

                // Validate facet name
                if (!$name || !isset($facetInfos[$name]))
                {
                    return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                               _REST_INVALID_SEARCH_FACET);
                }

                // Validate maximum values
                if (!$maxValues)
                {
                    $maxValues = 10;
                }
                elseif (!is_numeric($maxValues) || intval($maxValues) <= 0)
                {
                    return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                               _REST_INVALID_SEARCH_FACET);
                }

                $facetsAssoc[$name] = intval($maxValues);
            }
        }
        else
        {
            $names = array_keys($facetInfos);
            if ($names)
            {
                foreach ($names as $name)
                {
                    $facetsAssoc[$name] = 10;
                }
            }
        }

        // Get other request parameters
        $limit = getArrayParameter($_GET, 'limits', 128);
        $sort = getArrayParameter($_GET, 'sort', 'true');
        $sort  = ($sort != 'false' && $sort != '0');

        // Get facet values
        $startTime = microtime(true);
        $facetValues = $search->getFacetValues($facetsAssoc, $searchId, $limit, $sort);
        $elapsedTime = microtime(true) - $startTime;

        //if ($facetValues === null)
        //    return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());

        // Remove facets with a zero count
        if ($facetValues)
        {
            $facetValuesTemp = array();
            foreach ($facetValues as $value => $facet)
            {
                if ($facet->count > 0)
                {
                    $facetValuesTemp[$value] = $facet;
                }
            }
            $facetValues = $facetValuesTemp;
        }

        // Format response as JSON or XML
        $numFacets = count($facetValues);
        if (isset($_GET['json']))
        {
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
            $body  = '{';
            $body .= '"elapsedTime":' . $elapsedTime . ',';
            $body .= '"numFacets":' . $numFacets . ',';
            $body .= '"facets":' . json_encode($facetValues);
            $body .= '}';
        }
        else
        {
            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);

            $body  = wcmRestResponse::XML_DECL . PHP_EOL;
            $body .= '<search>' . PHP_EOL;
            $body .= '<elapsedTime>' . $elapsedTime . '</elapsedTime>' . PHP_EOL;
            $body .= '<numFacets>' . $numFacets . '</numFacets>' . PHP_EOL;

            $body .= '<facets>' . PHP_EOL;
            if ($facetValues)
            {
                foreach ($facetValues as $facetValue => $facet)
                {
                    $body .= '<facet>' . PHP_EOL;
                    $body .= '<name>' . $facet->name . '</name>' . PHP_EOL;
                    $body .= '<value>' . $facet->value . '</value>' . PHP_EOL;
                    $body .= '<text>' . $facet->text . '</text>' . PHP_EOL;
                    $body .= '<index>' . $facet->index . '</index>' . PHP_EOL;
                    $body .= '<searchIndex>' . $facet->searchIndex . '</searchIndex>' . PHP_EOL;
                    $body .= '<count>' . $facet->count . '</count>' . PHP_EOL;
                    $body .= '</facet>' . PHP_EOL;
                }
            }
            $body .= '</facets>' . PHP_EOL;

            $body .= '</search>' . PHP_EOL;
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }
}
