<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.frontDispatcher.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The cmfFrontDispatcher is in charge of loading the
 * appropriate controller and executing the appropriate
 * action.
 */
class cmfFrontDispatcher
{
    /**
     * Normal operation mode.
     * @var int
     */
    const MODE_NORMAL = 0;

    /**
     * Preview mode. Means that all WCM blocks will
     * be regenerated on the fly for every request.
     * @var int
     */
    const MODE_PREVIEW = 1;

    /**
     * ICE mode.
     * @var int
     */
    //const MODE_ICE = 2;
    
    /**
     * The services and configuration container that
     * is injected to controllers so they can
     * retrieve services and configuration values
     * they need.
     * @var cmfContainer
     */
    private $container;
    
    /**
     * The path of the default view (displayed if no
     * controller or an invalid controller is requested).
     * @var string
     */
    private $defaultView;
    
    /**
     * The name of the controller
     * @var string
     */
    private $controllerName;

    /**
     * The action to perform.
     * @var string
     */
    private $action;

     /**
     * The context.
     * @var array
     */
    private $context;

    /**
     * The mode.
     * @see self::MODE_NORMAL
     * @see self::MODE_PREVIEW
     * @var string
     */
    private $mode;

    /**
     * The request data.
     * @var array
     */
    private $request = array();
    
    /**
     * Public constructor.
     * @param string $defaultView The path of the default 
     * view (displayed if no controller or an invalid 
     * controller is requested).
     * @param cmfContainer $container The services and 
     * configuration container that is injected to 
     * controllers so they can retrieve services and 
     * configuration values they need.
     */
    public function __construct($defaultView, cmfContainer $container = null)
    {
        if (null === $container)
        {
            $container = new cmfContainer();
        }
        $this->defaultView = $defaultView;
        $this->container = $container;
    }

    /**
     * Load the appropriate controller and execute
     * the action.
     * @return void
     */
    public function dispatch()
    {
        // Get an instance of the controller.
        $controller = $this->getController();
        if (null === $controller)
        {
            // No controller, create default controller.
            $controller = new cmfDefaultController();
        }

        // Initialize the controller.
        $controller->initialize(
            $this->request,
            $this->controllerName,
            $this->context,
            $this->mode
        );
        
        // Inject dependencies to the controller.
        $controller->injectDependencies($this->container);

        // Dispatch the request to the controller.
        $controller->always();
        $controller->execute($this->action);
        if (!$controller->renderView())
        {
            // Cannot render current view, display default view.
            include($this->defaultView);
        }
    }
    
    /**
     * Get an instance of the controller.
     * @return cmfController The controller, or null
     * if no controller name was specified in the request
     * or if the controller name is invalid.
     */
    protected function getController()
    {
        return cmfControllerFactory::getObject($this->controllerName);
    }
    
    /**
     * Get the services and configuration container that
     * is injected to controllers so they can
     * retrieve services and configuration values
     * they need.
     * @return cmfContainer
     */
    public function getContainer()
    {
        return $this->container;
    }
    
    /**
     * Set the services and configuration container that
     * is injected to controllers so they can
     * retrieve services and configuration values
     * they need.
     * @param cmfContainer $value The new container.
     * @return void
     */
    public function setContainer(cmfContainer $value)
    {
        $this->container = $value;
    }

    /**
     * Get the request data.
     * @return array
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     *  Set the request data.
     *  @param array $parameters The request data.
     *  @return void
     */
    public function setRequest(array $request = array())
    {
        $this->request = $request;
        $this->parseRequest();
    }

    /**
     * Parse the request data to extract needed 
     * information for the dispatch operation.
     * @return void
     */
    private function parseRequest()
    {
        $this->controllerName = cmfToolbox::getArrayParameter($this->request, 'controllerName', null);
        $this->action = cmfToolbox::getArrayParameter($this->request, 'action', null);
        $this->context = cmfToolbox::getArrayParameter($this->request, 'context', array());
    }

    /**
     * Get the controller name.
     * @return string
     */
    public function getControllerName()
    {
        return $this->controllerName;
    }

    /**
     * Get the action.
     * @return string
     */
    public function getAction()
    {
        return $this->action;
    }

    /**
     * Get the context.
     * @return array
     */
    public function getContext()
    {
        return $this->context;
    }


    /**
     * Set the context.
     * @param array $context The new context.
     * @return void
     */
    public function setContext(array $context)
    {
        $this->context = $context;
    }

    /**
     * Get the mode (could be either MODE_PREVIEW or
     * MODE_NORMAL).
     * @return int
     */
    public function getMode()
    {
        return $this->mode;
    }

    /**
     * Set the mode (could be either MODE_PREVIEW or
     * MODE_NORMAL).
     * @param int $mode The new value.
     * @return void
     * @exception InvalidArgumentException If the
     * provided value is not a valid mode.
     */
    public function setMode($mode)
    {
        switch ($mode)
        {
            case self::MODE_PREVIEW:
            case self::MODE_NORMAL;
                $this->mode = $mode;
                break;

            default:
                throw new InvalidArgumentException();
                break;
        }
    }
}
