<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.searchResult.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The result of a search.
 */
class cmfSearchResult
{
    /**
     * The search plugin.
     * @var cmfSearchPlugin
     */
    public $searchPlugin;

    /**
     * The search configuration.
     * @var cmfSearchConfig
     */
    public $configuration;

    /**
     * The query used to obtain this result.
     * @var cmfSearchQuery
     */
    public $query;

    /**
     * The number of results.
     * @var int
     */
    public $resultCount;

    /**
     * The facet values cache (holds the facet values that have already been
     * loaded and processed).
     * @var array
     */
    public $facetValuesCache = array();

    /**
     * Construct a new search result.
     * @param cmfSearchQuery $query The query used to obtain this result.
     * @param int $resultCount The number of results.
     * @param cmfSearchConfig $config The search configuration.
     * @param cmfSearchPlugin $searchPlugin The search plugin.
     */
    public function __construct(cmfSearchQuery $query, $resultCount, cmfSearchConfig $config, cmfSearchPlugin $searchPlugin)
    {
        $this->query = $query;
        $this->resultCount = $resultCount;
        $this->configuration = $config;
        $this->searchPlugin = $searchPlugin;
    }

    /**
     * Get a range of resut items.
     * @param int $from The index of the first item.
     * @param int $limit The maximum number of items to obtain.
     * @return array
     */
    public function getResultItems($from, $limit)
    {
        return $this->searchPlugin->getDocumentRange(
            $this->query->searchId,
            $from,
            min($this->resultCount - 1, $from + $limit - 1)
        );
    }

    /**
     * Get the values of a given facet.
     * @param string $facet The name of a facet.
     * @return array An array of cmfFacetValue.
     */
    public function getFacetValues($facet)
    {
        if (!array_key_exists($facet, $this->facetValuesCache))
        {
            $this->facetValuesCache[$facet] = $this->computeFacetValues($facet);
        }
        return $this->facetValuesCache[$facet];
    }

    /**
     * Compute the values of a given facet.
     * @param string $facets The name of the facet to compute.
     * @return array An array of cmfFacetValue.
     */
    private function computeFacetValues($facet)
    {
        $facetValues = array();

        $facetInfo = $this->configuration->getFacetDefinition($facet);
        if (null !== $facetInfo)
        {
            // Try to get the values provider for the facet
            $valuesProvider = $facetInfo->getProvider();

            if (null !== $valuesProvider)
            {
                // Get values from the provider
                $facetValues = $valuesProvider->getFacetValues($facetInfo, $this);
            }
            else
            {
                // Get the values from the plugin
                $facetValues = $this->searchPlugin->getRawFacetValues(
                    $this->query->searchId,
                    $facetInfo
                );
            }

            // Remove values with no hits.
            if ($facetValues)
            {
                $facetValues = array_filter($facetValues, array('cmfFacetValue', 'hasHits'));
            }

            // Possibly sort the facets in decreasing order by count
            if ($facetValues && $facetInfo->sort)
            {
                $callback = array('cmfFacetValue', 'compareFacetValuesByCount');
                uasort($facetValues, $callback);
            }

            // Keep only the top $maxValues values
            if ($facetInfo->maxValues)
            {
                $facetValues = array_slice($facetValues, 0, $facetInfo->maxValues);
            }

            // Process the facet values
            if ($facetValues)
            {
                $facetValues = $this->processFacetValues($facetInfo, $facetValues);
            }
        }

        return $facetValues;
    }

    /**
     * If a facet processor class exists for the provided facet, use it
     * to process the facet values; otherwise return the values as is.
     * @param cmfFacetDefinition $facetInfo The definition of the facet.
     * @param array $facetValues An array of cmfFacetValue.
     * @return array An array of cmfFacetValue.
     */
    private function processFacetValues(cmfFacetDefinition $facetInfo, array $facetValues)
    {
        $processor = $facetInfo->getProcessor();

        if (null !== $processor)
        {
            $facetValues = $processor->processFacetValues($this, $facetInfo, $facetValues);
        }

        return $facetValues;
    }
}
