<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.searchPlugin.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-system-api
 * @version     4.x
 */

/**
 * The abstract search plugin class.
 */
abstract class cmfSearchPlugin
{
    /**
     * The stack of sub-queries while converting a query.
     *
     * @var array
     */
    protected $queryStack;

    /**
     * The index definitions.
     * @var array of cmfIndexDefinition
     */
    protected $indexDefinitions = null;

    /**
     * The facet definitions.
     * @var array of cmfFacetDefinition
     */
    protected $facetDefinitions;

    /**
     * Construct the plugin. The expected arguments are :
     *     'facetDefinitions' => an array of cmfFacetDefinition. If this
     *     						 argument is not provided, not facets will
     *     						 be available.
     * @param array $arguments The parameters for the plugin.
     */
    public function __construct(array $arguments)
    {
        $this->facetDefinitions = cmfToolbox::getArrayParameter($arguments, 'facetDefinitions', array());
    }

    /**
     * Load the index definitions, if possible, from the native search engine.
     * @return array|null An array of cmfIndexDefinition, with the index
     * names as keys, or null if the native search engine does not support
     * this operation (in which case the index definition must be provided
     * through configuration).
     */
    protected abstract function loadIndexDefinitions();

    /**
     * Initialize a search.
     * @param string $searchId The ID of the search.
     * @param string $query A Lucene query.
     * @param string|null $filterQuery A Lucene filter query, or null if no
     * filters are applied.
     * @param string|null $sortingCriteria The sorting criteria (uses
     * SQL order by syntax), or null to use the natural native search
     * engine sort order.
     * @return int The number of results that match the query.
     */
    public abstract function initSearch($searchId, $query, $filterQuery = null, $sortingCriteria = null);

    /**
     * Get the raw values of a given facet, for a given search.
     * @param string $searchId The ID of the search.
     * @param cmfFacetDefinition $facetInfo The definition of the facet.
     * @return array An array (cmfFacetValue->value => cmfFacetValue).
     */
    public abstract function getRawFacetValues($searchId, cmfFacetDefinition $facetInfo);

    /**
     * Get the number of hits for a given facet value, for a given search.
     * @param string $searchId The ID of the search.
     * @param cmfFacetDefinition $facetInfo The definition of the facet.
     * @param mixed $rawFacetValue The raw value of the facet.
     * @return int The number of hits for the given facet value.
     */
    public abstract function getFacetValueCount($searchId, cmfFacetDefinition $facetInfo, $rawFacetValue);

    /**
     * Get a range of document from a given search.
     * @param string $searchId The ID of the search.
     * @param int $from The index of the first document (zero based).
     * @param int $to The index of the last document (zero based).
     * @return array An array of documents (each formatted as an array).
     */
    public abstract function getDocumentRange($searchId, $from, $to);

    /**
     * Get the facet definitions.
     * @return array of cmfFacetDefinition
     */
    public function getFacetDefinitions()
    {
        return $this->facetDefinitions;
    }

    /**
     * Get the definition of a given facet.
     * @param string $facet The facet name.
     * @return cmfFacetDefinition|null The facet definition (or null if not found).
     */
    public function getFacetDefinition($facet)
    {
        $definitions = $this->getFacetDefinitions();
        if (array_key_exists($facet, $definitions))
        {
            return $definitions[$facet];
        }
        return null;
    }

    /**
     * Get all the index definitions.
     * @return array An array of cmfIndexDefinition (with index names as keys).
     */
    public function getIndexDefinitions()
    {
        if (null === $this->indexDefinitions)
        {
            $this->indexDefinitions = $this->loadIndexDefinitions();
        }
        return $this->indexDefinitions;
    }

    /**
     * Get the definition of a given index.
     * @param string $index The name of an index.
     * @return cmfIndexDefinition|null The index definition
     * (or null if index not found).
     */
    public function getIndexDefinition($index)
    {
        $indexDefinitions = $this->getIndexDefinitions();
        if (array_key_exists($index, $indexDefinitions))
        {
            return $indexDefinitions[$index];
        }
        return null;
    }

    /**
     * Parses a Lucene-syntax query string to generate a native query.
     *
     * @param string $queryString Lucene-syntax query string
     *
     * @return array|null The native query and updated query string (in that order), or null on error
     */
    protected function parseQuery($queryString)
    {
        // Remove leading and trailing whitespace, as well as any
        // trailing backslash as it causes the Lucene parser to
        // trigger a "notice"
        $queryString = trim($queryString, " \t\\");

        // Attempt to parse the query string in two passes:
        //
        // Pass 1: Attempt to parse the query string as is.
        //
        // Pass 2: If an error occurred during the first pass, attempt
        // to parse the query string by double-quoting each word.
        //
        $query = null;
        $error = null;
        while ($query === null)
        {
            try
            {
                // We want exceptions
                Zend_Search_Lucene_Search_QueryParser::dontSuppressQueryParsingExceptions();

                // Our query strings can contain text or numbers in UTF-8
                Zend_Search_Lucene_Analysis_Analyzer::setDefault(
                    new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8Num_CaseInsensitive()
                );

                // HACK[
                // Work around a limitation in the Zend query parser
                $hackRE = '/(publicationdate|(?:created|modified)at):\\\\?"?-([0-9]+)"?/i';
                $hackedQueryString = preg_replace($hackRE, '$1:m$2', $queryString);

                $hackRE = '/(publicationdate|(?:created|modified)at):"?([0-9]+)-([0-9]+)-([0-9]+)"?/i';
                $hackedQueryString = preg_replace($hackRE, '$1:$2x$3x$4', $hackedQueryString);
                // HACK]

                // Parse the query string
                //$query = Zend_Search_Lucene_Search_QueryParser::parse($queryString, 'utf-8');
                $query = Zend_Search_Lucene_Search_QueryParser::parse($hackedQueryString, 'utf-8');
                $error = null;
            }
            catch (Zend_Search_Lucene_Search_QueryParserException $e)
            {
                // If this is pass 2, that's it
                if ($error !== null)
                {
                    break;
                }

                // Pass 1 failed, so on to pass 2
                $error = $e;

                // The query string does not appear to be a valid
                // Lucene query, so we quote each word and try again
                $words = preg_split('/\\s+/', $queryString);
                if ($words)
                {
                    foreach ($words as &$word)
                    {
                        $word = '"' . trim(trim(trim($word), '"')) . '"';
                    }

                    $queryString = implode(' ', $words);
                }
            }
        }

        // If an error occurred during the second pass, just log it
        if ($error)
        {
            throw $error;
        }

        // Convert the query object
        $nativeQuery = $this->convertQuery($query);
        if (null === $nativeQuery)
        {
            return null;
        }

        return array($nativeQuery, $queryString);
    }

    /**
     * Converts a given Zend_Search_Lucene_Search_Query object into a
     * native query.
     *
     * This method dispatches to a method named
     * 'convertQuery_<queryType>' based on the query object class.
     *
     * The concrete subclass is expected to implement the query object
     * class-specific methods to build the native query based on the
     * nature of the query object.
     *
     * However, some of the conversion methods have a default
     * implementation in this abstract class and need not be
     * overridden unless something specific needs to be done, namely
     * (which see):
     *
     *     convertQuery_Fuzzy
     *     convertQuery_Insignificant
     *     convertQuery_Term
     *
     * The $context parameter can be used to pass in contextual
     * information.
     *
     * @param Zend_Search_Lucene_Search_Query $query   The query object
     * @param array|null                      $context The current context
     *
     * @return mixed The native query
     */
    protected function convertQuery(Zend_Search_Lucene_Search_Query $query, $context = null)
    {
        // Clear the query stack
        $this->queryStack = array();

        // Delegare to the query-specific conversion method
        $parts = explode('_', get_class($query));
        $functionName = 'convertQuery_' . array_pop($parts);
        return $this->$functionName($query, $context);
    }

    /**
     * Partitions and converts (by calling the convertQuery method)
     * the optional, required, and prohibited sub-queries of a given
     * boolean query object.
     *
     * @param Zend_Search_Lucene_Search_Query_Boolean $query   The boolean query object
     * @param array|null                              $context The current context
     *
     * @return array An assoc of 'optional', 'required', and 'prohibited' sub-queries
     */
    protected function convertQuery_BooleanHelper(Zend_Search_Lucene_Search_Query_Boolean $query, $context = null)
    {
        $optionalSubQueries = array();
        $requiredSubQueries = array();
        $prohibitedSubQueries = array();

        // A boolean query consists of one or more sub-queries, each
        // with a corresponding sign indicating whether it is optional
        // (null), required (true), or prohibited (false)
        $subQueries = $query->getSubqueries();
        if ($subQueries)
        {
            // Push the current query onto the query stack
            array_push($this->queryStack, $query);

            $signs = $query->getSigns();
            foreach ($subQueries as $subQuery)
            {
                // Convert the sub-query to its native equvalent
                $subQuery = $this->convertQuery($subQuery, $context);

                // Partition the subQueries according to their sign:
                //  foo => optional (null)
                // +foo => required (true)
                // -foo => prohibited (false)
                $sign = array_shift($signs);
                if (null === $sign)
                {
                    $optionalSubQueries[] = $subQuery;
                }
                elseif ($sign)
                {
                    $requiredSubQueries[] = $subQuery;
                }
                else
                {
                    $prohibitedSubQueries[] = $subQuery;
                }
            }

            // Pop the query stack
            array_pop($this->queryStack);
        }

        return array(
            'optional' => $optionalSubQueries,
            'required' => $requiredSubQueries,
            'prohibited' => $prohibitedSubQueries,
        );
    }

    /**
     * Partitions and converts (by calling the convertTerm method) the
     * optional, required, and prohibited terms of a given multi-term
     * query object.
     *
     * @param Zend_Search_Lucene_Search_Query_MultiTerm $query   The multi-term query object
     * @param array|null                                $context The current context
     *
     * @return array An assoc of 'optional', 'required', and 'prohibited' terms
     */
    protected function convertQuery_MultiTermHelper(Zend_Search_Lucene_Search_Query_MultiTerm $query, $context = null)
    {
        $optionalTerms = array();
        $requiredTerms = array();
        $prohibitedTerms = array();

        // A multi-term query consists of one or more terms, each with
        // a corresponding sign - very much like a boolean query which
        // consists of one or more sub-queries, each with a
        // corresponding sign - can you say *abstraction*? Or lack
        // thereof as far as Zend is concerned? :(
        $terms = $query->getTerms();
        if ($terms)
        {
            // Push the current query onto the query stack
            array_push($this->queryStack, $query);

            $signs = $query->getSigns();
            foreach ($terms as $term)
            {
                // Convert the term to its native equvalent
                $term = $this->convertTerm($term, $context);

                // Partition the terms according to their sign:
                //  foo => optional (null)
                // +foo => required (true)
                // -foo => prohibited (false)
                $sign = ($signs ? array_shift($signs) : null);
                if (null === $sign)
                {
                    $optionalTerms[] = $term;
                }
                elseif ($sign)
                {
                    $requiredTerms[] = $term;
                }
                else
                {
                    $prohibitedTerms[] = $term;
                }
            }

            // Pop the query stack
            array_pop($this->queryStack);
        }

        return array(
            'optional' => $optionalTerms,
            'required' => $requiredTerms,
            'prohibited' => $prohibitedTerms,
        );
    }

    /**
     * Converts a Lucene fuzzy query object to its native equvalent.
     *
     * The default implementation simply converts the associated term
     * without the fuzziness factor by calling the convertTerm method.
     *
     * @param Zend_Search_Lucene_Search_Query_Fuzzy $query   The fuzzy query object
     * @param array|null                            $context The current context
     *
     * @return string The native equvalent of the fuzzy query object
     */
    protected function convertQuery_Fuzzy(Zend_Search_Lucene_Search_Query_Fuzzy $query, $context = null)
    {
        // The fuzzy query object doesn't want to give up its term, so
        // we'll just create a term from its string representation
        list($field, $value) = explode(':', (string)$query);
        if (!$value)
        {
            $value = $field;
            $field = null;
        }
        list($text, $similarity) = explode('~', $value);
        if (!$similarity)
        {
            $similarity = '0.0';
        }
        $term = new Zend_Search_Lucene_Index_Term($text, $field);

        // We assume there's no built-in support for fuzzy searches by
        // default. Doing it manually would be too heavy, so we just
        // search for the term as-is
        array_push($this->queryStack, $query);
        $nativeQuery = $this->convertTerm($term, $context);
        array_pop($this->queryStack);

        return $nativeQuery;
    }

    /**
     * Converts a Lucene insignificant query object to its native
     * equvalent.
     *
     * The default implementation simply returns an empty string.
     *
     * @param Zend_Search_Lucene_Search_Query_Insignificant $query   The insignificant query object
     * @param array|null                                    $context The current context
     *
     * @return string The native equvalent of the insignificant query object
     */
    protected function convertQuery_Insignificant(Zend_Search_Lucene_Search_Query_Insignificant $query, $context = null)
    {
        // An insignificant query is one that is meaningless to Lucene
        // and would not yield any results, so we'll just behave as if
        // nothing had been given
        return '';
    }

    /**
     * Extracts the field, term texts, and proximity factor (slop) of
     * a Lucene phrase query.
     *
     * @param Zend_Search_Lucene_Search_Query_Phrase $query   The phrase query object
     * @param array|null                             $context The current context
     *
     * @return array An assoc. array of 'field' => string, 'texts' => array, and 'slop' => int
     */
    protected function convertQuery_PhraseHelper($query, $context = null)
    {
        $field = '';
        $texts = array();

        // A phrase consists of one or more terms, possibly with an
        // associated proximity factor
        $terms = $query->getTerms();
        if ($terms)
        {
            // All terms in a phrase have the same field
            $field = $terms[0]->field;

            // Convert and collect each term's text
            foreach ($terms as $term)
            {
                $texts[] = $term->text;
            }
        }

        // Extract the proximity factor
        $slop = intval($query->getSlop());

        return array(
            'field' => $field,
            'texts' => $texts,
            'slop' => $slop,
        );
    }

    /**
     * Converts a range query object to its native equvalent.
     *
     * This method need not be overridden unless something specific
     * needs to be done since it essentially creates a term of the
     * form <lower-term>..<upper-term> and then converts it.
     *
     * @param Zend_Search_Lucene_Search_Query $query   The range query object
     * @param array|null                      $context The current context
     *
     * @return string The native equvalent
     */
    protected function convertQuery_Range($query, $context = null)
    {
        // Push the current query onto the query stack
        array_push($this->queryStack, $query);

        // The convertTerm method handles ranges for us
        $text = $query->getLowerTerm()->text . '..' . $query->getUpperTerm()->text;
        $term = new Zend_Search_Lucene_Index_Term($text, $query->getField());
        $nativeQuery = $this->convertTerm($term, $context);

        // Pop the query stack
        array_pop($this->queryStack);

        return $nativeQuery;
    }

    /**
     * Converts a Lucene term query object to its native equvalent.
     *
     * This method need not be overridden unless something specific
     * needs to be done. It is equivalent to:
     *
     *     return $this->convertTerm($query->getTerm(), $context);
     *
     * @param Zend_Search_Lucene_Search_Query_Term $query   The term query object
     * @param array|null                           $context The current context
     *
     * @return string The native equvalent
     */
    protected function convertQuery_Term(Zend_Search_Lucene_Search_Query_Term $query, $context = null)
    {
        // A term query just holds... er... a query term
        array_push($this->queryStack, $query);
        $nativeQuery = $this->convertTerm($query->getTerm(), $context);
        array_pop($this->queryStack);

        return $nativeQuery;
    }

    /**
     * Converts a Lucene index term object to its native equvalent.
     *
     * This method need not be overridden unless something specific
     * needs to be done. It is equivalent to:
     *
     *     $nativeText = $this->convertTermText($term->field, $term->text, $context);
     *     return $this->convertTermField($term->field, $nativeText, $context);
     *
     * @param Zend_Search_Lucene_Index_Term $term    The index term object
     * @param array|null                    $context The current context
     *
     * @return string The native equvalent of the index term object
     */
    protected function convertTerm(Zend_Search_Lucene_Index_Term $term, $context = null)
    {
        $field = $term->field;

        // Convert the term's text to its native equvalent
        $nativeText = $this->convertTermText($field, $term->text, $context);

        // Convert the term's field to its native equvalent, including
        // the native text in the result
        return $this->convertTermField($field, $nativeText, $context);
    }

    /**
     * Escapes Lucene special characters in a given query string.
     *
     * @param string $queryString The query string to escape
     *
     * @return string The escaped query string
     */
    public static function escapeQuery($queryString)
    {
        return preg_replace('/([\"\\[\\]\\\\&|!(){}^~*?:])/', '\\\\$1', $queryString);
    }
}
