<?php
/**
 * Project:     WCM
 * File:        $Id: cmf.imageController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     cmf-www-mediaManager-restws
 * @version     4.x
 */

/**
 * This class manages the media for the front end.
 * Its basic role is to provide a webservice to serve
 * images.
 */
class cmfImageController extends cmfController
{
    /**
     * The image manager.
     * @var cmfImageManager
     */
    private $imageManager;
    
    /**
     * The login for add/delete operations.
     * @var string
     */
    private $login;
    
    /**
     * The password for add/delete operations.
     * @var string
     */
    private $password;
    
    /**
     * The image HTTP time to live in seconds, used to build the HTTP Expires header.
     * @var int
     */
    private $imageTimeToLive;
    
    /**
     * Get the image HTTP time to live in seconds, used to build the HTTP Expires header.
     * @return int
     */
    public function getImageTimeToLive()
    {
        return $this->imageTimeToLive;
    }
    
    /**
     * (non-PHPdoc)
     * @see mvc/controllers/cmfController#injectDependencies($container)
     */
    public function injectDependencies(cmfContainer $container)
    {
        parent::injectDependencies($container);
        
        $this->imageManager = cmfToolbox::getArrayParameter($container, 'mediaImageManager');
        $this->login = cmfToolbox::getArrayParameter($container, 'mediaImageLogin');
        $this->password = cmfToolbox::getArrayParameter($container, 'mediaImagePassword');
        $this->imageCacheTimeToLive = cmfToolbox::getArrayParameter($container, 'mediaImageTimeToLive', 0);
    }

    /**
     * Handle a GET request for an image.
     * @return void
     */
    public function onGet()
    {
        // Get arguments from the request.
        $request = $this->getRequest();
        $id = cmfToolbox::getArrayParameter($request, 'id');
        $format = cmfToolbox::getArrayParameter($request, 'format');
        $width = cmfToolbox::getArrayParameter($request, 'width');
        $height = cmfToolbox::getArrayParameter($request, 'height');
        
        try
        {
            if ($format)
            {
                // Get formatted image.
                $path = $this->getFormattedImage($id, $format);
            }
            elseif ($width || $height)
            {
                // Get resized image.
                $path = $this->getResizedImage($id, $width, $height);
            }
            else
            {
                // Get original image.
                $path = $this->getOriginalImage($id);
            }
        }
        catch (InvalidArgumentException $e)
        {
            // Invalid argument.
            $this['status'] = '400 Bad Request';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
            return;
        }
        catch (Exception $e)
        {
            // Another error occured.
            $this['status'] = '500 Internal Server Error';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
            return;
        }
        
        if (null === $path)
        {
            // Image file not found.
            $this['status'] = '404 Not Found';
            $this['message'] = 'Not Found';
            $this->setView('message.php');
            return;
        }
        
        // Display image.
        $this['path'] = $path;
        $this->setView('image.php');
    }

    /**
     * Handle an add request.
     */
    public function onAdd()
    {
        // Get arguments from the request.
        $request = $this->getRequest();
        
        $id = cmfToolbox::getArrayParameter($request, 'id');
        $formats = cmfToolbox::getArrayParameter($request, 'formats');
        $url = cmfToolbox::getArrayParameter($request, 'url');

        $login = cmfToolbox::getArrayParameter($request, 'login');
        $password = cmfToolbox::getArrayParameter($request, 'password');

        // Decode & validate formats.
        $formats = json_decode($formats, true);
        if (null === $formats)
        {
            $this['status'] = '400 Bad Request';
            $this['message'] = 'Invalid format';
            $this->setView('message.php');
            return;
        }

        // Validate credentials.
        if (!$this->login($login, $password))
        {
            $this['status'] = '403 Forbidden';
            $this['message'] = 'Invalid login/password';
            $this->setView('message.php');
            return;
        }

        if (isset($_FILES['photo']['tmp_name']))
        {
            // A photo was uploaded.
            if (is_uploaded_file($_FILES['photo']['tmp_name']))
            {
                // Upload succeeded.
                $this->addUploadedImage(
                    $id, 
                    $_FILES['photo']['tmp_name'], 
                    $formats
                );
            }
            else
            {
                // The file upload failed.
                $this['status'] = '400 Bad Request';
                $this['message'] = 'File upload failed';
                $this->setView('message.php');
            }
        }
        elseif ($url)
        {
            // A URL was provided.
            $this->addImage(
                $id, 
                $url, 
                $formats
            );
        }
        else
        {
            // Either no file was uploaded or no URL provided.
            $this['status'] = '400 Bad Request';
            $this['message'] = 'A URL must be provided or a file must be uploaded';
            $this->setView('message.php');
        }
    }
    
    public function onUpdate()
    {
        // Get arguments from the request.
        $request = $this->getRequest();
        
        $id = cmfToolbox::getArrayParameter($request, 'id');
        $formats = cmfToolbox::getArrayParameter($request, 'formats');

        $login = cmfToolbox::getArrayParameter($request, 'login');
        $password = cmfToolbox::getArrayParameter($request, 'password');
        
        // Decode & validate formats.
        $formats = json_decode($formats, true);
        if (null === $formats)
        {
            $this['status'] = '400 Bad Request';
            $this['message'] = 'Invalid format';
            $this->setView('message.php');
            return;
        }

        // Validate credentials.
        if (!$this->login($login, $password))
        {
            $this['status'] = '403 Forbidden';
            $this['message'] = 'Invalid login/password';
            $this->setView('message.php');
            return;
        }
        
        try
        {
            $this->imageManager->updateFormats($id, $formats);
            $this['status'] = '200 Ok';
            $this['message'] = 'Success';
            $this->setView('message.php');
        }
        catch (InvalidArgumentException $e)
        {
            // Invalid argument (unknown image ID).
            $this['status'] = '404 Not Found';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
        catch (cmfMediaManagerDatabaseException $e)
        {
            // Invalid argument (unknown image ID).
            $this['status'] = '403 Forbidden';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
        catch (Exception $e)
        {
            // Another error occured.
            $this['status'] = '500 Internal Server Error';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
    }

    /**
     * Handle a delete request.
     */
    public function onRemove()
    {
        // Get arguments from the request.
        $id = cmfToolbox::getArrayParameter($_REQUEST, 'id', null);
        $login = cmfToolbox::getArrayParameter($_REQUEST, 'login', null);
        $password = cmfToolbox::getArrayParameter($_REQUEST, 'password', null);

        // Validate credentials.
        if (!$this->login($login, $password))
        {
            $this['status'] = '403 Forbidden';
            $this['message'] = 'Invalid login/password';
            $this->setView('message.php');
            return;
        }

        try
        {
            // Delete the image.
            if ($this->imageManager->removeImage($id))
            {
                $this['status'] = '200 Ok';
                $this['message'] = 'Success';
            }
            else
            {
                $this['status'] = '404 Not Found';
                $this['message'] = 'Unknown image ID : ' . $id;
            }
            $this->setView('message.php');
        }
        catch (InvalidArgumentException $e)
        {
            // Invalid argument.
            $this['status'] = '400 Bad Request';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
        catch (cmfMediaManagerDatabaseException $e)
        {
            // Invalid argument.
            $this['status'] = '403 Forbidden';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
            return;
        }
        catch (Exception $e)
        {
            // Another error occured.
            $this['status'] = '500 Internal Server Error';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
    }

    /**
     * Get the path of an original image file.
     * @param int $id The ID of the image.
     * @return string The path of the image file,
     * or null if not found.
     */
    private function getOriginalImage($id)
    {
        // Get the image file path from the image manager.
        return $this->imageManager->getImage($id);
    }

    /**
     * Get the path of a formatted image file.
     * @param int $id The ID of the image.
     * @param string $format The format.
     * @return string The path of the image file,
     * or null if not found.
     * @exception ErrorException If an error occurs.
     */
    private function getFormattedImage($id, $format)
    {
        // Get the image file path from the image manager.
        $path = $this->imageManager->getFormattedImage($id, $format);

        if (null === $path)
        {
            // Thumbnail doesn't exist, try to generate it.
            $path = $this->imageManager->generateFormattedImage($id, $format);
        }

        return $path;
    }

    /**
     * Get the path of a resized image file.
     * @param int $id The ID of the image.
     * @param string $width The maximum width of the
     * thumbnail.
     * @param string $height The maximum width of the
     * thumbnail.
     * @return string The path of the image file,
     * or null if not found.
     * @exception ErrorException If an error occurs.
     */
    private function getResizedImage($id, $width, $height)
    {
        // Get the image file path from the image manager.
        $path = $this->imageManager->getResizedImage($id, $width, $height);

        if (null === $path)
        {
            // Thumbnail doesn't exist, try to generate it.
            $path = $this->imageManager->generateResizedImage($id, $width, $height);
        }

        return $path;
    }

    /**
     * Add an image using a URL to the media manager.
     * @param int $id The ID of the image.
     * @param string $url The URL of the image.
     * @param array $formats The WCM format coordinates.
     * @return void
     */
    private function addImage($id, $url, array $formats)
    {
        try
        {
            $this->imageManager->addImage($id, $url, $formats);

            $this['status'] = '200 Ok';
            $this['message'] = 'Success';
            $this->setView('message.php');
        }
        catch (InvalidArgumentException $e)
        {
            // Invalid argument.
            $this['status'] = '400 Bad Request';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
        catch (cmfMediaManagerDatabaseException $e)
        {
            // Invalid argument.
            $this['status'] = '403 Forbidden';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
            return;
        }
        catch (Exception $e)
        {
            // Another error occured.
            $this['status'] = '500 Internal Server Error';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
    }

    /**
     * Add an uploaded image to the media manager.
     * @param int $id The ID of the image.
     * @param string $file The path of the uploaded file.
     * @param array $formats The WCM format coordinates.
     * @return void
     */
    private function addUploadedImage($id, $file, array $formats)
    {
        try
        {
            $this->imageManager->addUploadedImage($id, $file, $formats);

            $this['status'] = '200 Ok';
            $this['message'] = 'Success';
            $this->setView('message.php');
        }
        catch (InvalidArgumentException $e)
        {
            // Invalid argument.
            $this['status'] = '400 Bad Request';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
        catch (cmfMediaManagerDatabaseException $e)
        {
            // Invalid argument.
            $this['status'] = '403 Forbidden';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
            return;
        }    

        catch (Exception $e)
        {
            // Another error occured.
            $this['status'] = '500 Internal Server Error';
            $this['message'] = $e->getMessage();
            $this->setView('message.php');
        }
    }

    /**
     * Check login information supplied by the user.
     * @param string $username The username.
     * @param string $password The password.
     * @return boolean True if the login information
     * are correct, false otherwise.
     */
    private function login($username, $password)
    {
        return $username == $this->login && $password == $this->password;
    }
}
