<?php
/**
 * Nstein application framework
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 * @version    1.0
 */

/**
 * The Nstein_Http_Response class emcapsulates a response from an HTTP server.
 *
 * @category   Nstein
 * @package    Nstein_Http
 * @copyright  Copyright (c) 2008 Nstein Technologies (http://www.nstein.com)
 */
class Nstein_Http_Response
{
    // Common headers (normalize to lowercase to follow Zend's convention)
    const H_AUTHORIZATION       = 'authorization';
    const H_CONTENT_LENGTH      = 'content-length';
    const H_CONTENT_TYPE        = 'content-type';
    const H_LOCATION            = 'location';
    const H_WWW_AUTHENTICATE    = 'www-authenticate';

    // Common content types without charset
    const H_CT_APPLICATION_JSON      = 'application/json';
    const H_CT_MULTIPART_FORM_DATA   = 'multipart/form-data';
    const H_CT_TEXT_HTML             = 'text/html';
    const H_CT_TEXT_PLAIN            = 'text/plain';
    const H_CT_TEXT_XML              = 'text/xml';
    const H_CT_X_WWW_FORM_URLENCODED = 'application/x-www-form-urlencoded';

    // Common content types with UTF-8 charset parameter
    const H_CTCS_APPLICATION_JSON = 'application/json; charset=UTF-8';
    const H_CTCS_TEXT_HTML        = 'text/html; charset=UTF-8';
    const H_CTCS_TEXT_PLAIN       = 'text/plain; charset=UTF-8';
    const H_CTCS_TEXT_XML         = 'text/xml; charset=UTF-8';

    // Default values
    const DEFAULT_CONTENT_TYPE  = 'text/plain; charset=utf-8';
    const DEFAULT_HTTP_PROTOCOL = 'HTTP/1.1';

    // HTTP response status codes: Informational 1xx
    const S_CONTINUE            = 100;
    const S_SWITCHING_PROTOCOLS = 101;

    // HTTP response status codes: Success 2xx
    const S_OK                            = 200;
    const S_CREATED                       = 201;
    const S_ACCEPTED                      = 202;
    const S_NON_AUTHORITATIVE_INFORMATION = 203;
    const S_NO_CONTENT                    = 204;
    const S_RESET_CONTENT                 = 205;
    const S_PARTIAL_CONTENT               = 206;

    // HTTP response status codes: Redirection 3xx
    const S_MULTIPLE_CHOICES   = 300;
    const S_MOVED_PERMANENTLY  = 301;
    const S_FOUND              = 302; // 1.1
    const S_SEE_OTHER          = 303;
    const S_NOT_MODIFIED       = 304;
    const S_USE_PROXY          = 305;
    // 306 is deprecated but reserved
    const S_TEMPORARY_REDIRECT = 307;

    // HTTP response status codes: Client Error 4xx
    const S_BAD_REQUEST                     = 400;
    const S_UNAUTHORIZED                    = 401;
    const S_PAYMENT_REQUIRED                = 402;
    const S_FORBIDDEN                       = 403;
    const S_NOT_FOUND                       = 404;
    const S_METHOD_NOT_ALLOWED              = 405;
    const S_NOT_ACCEPTABLE                  = 406;
    const S_PROXY_AUTHENTICATION_REQUIRED   = 407;
    const S_REQUEST_TIMEOUT                 = 408;
    const S_CONFLICT                        = 409;
    const S_GONE                            = 410;
    const S_LENGTH_REQUIRED                 = 411;
    const S_PRECONDITION_FAILED             = 412;
    const S_REQUEST_ENTITY_TOO_LARGE        = 413;
    const S_REQUEST_URI_TOO_LONG            = 414;
    const S_UNSUPPORTED_MEDIA_TYPE          = 415;
    const S_REQUESTED_RANGE_NOT_SATISFIABLE = 416;
    const S_EXPECTATION_FAILED              = 417;

    // HTTP response status codes: Server Error 5xx
    const S_INTERNAL_SERVER_ERROR      = 500;
    const S_NOT_IMPLEMENTED            = 501;
    const S_BAD_GATEWAY                = 502;
    const S_SERVICE_UNAVAILABLE        = 503;
    const S_GATEWAY_TIMEOUT            = 504;
    const S_HTTP_VERSION_NOT_SUPPORTED = 505;
    const S_BANDWIDTH_LIMIT_EXCEEDED   = 509;

    /**
     * HTTP response status messages (map of codes => messages).
     *
     * @var array
     */
    protected static $statusMessages = array(
        // Informational 1xx
        self::S_CONTINUE            => 'Continue',
        self::S_SWITCHING_PROTOCOLS => 'Switching Protocols',

        // Success 2xx
        self::S_OK                            => 'OK',
        self::S_CREATED                       => 'Created',
        self::S_ACCEPTED                      => 'Accepted',
        self::S_NON_AUTHORITATIVE_INFORMATION => 'Non-Authoritative Information',
        self::S_NO_CONTENT                    => 'No Content',
        self::S_RESET_CONTENT                 => 'Reset Content',
        self::S_PARTIAL_CONTENT               => 'Partial Content',

        // Redirection 3xx
        self::S_MULTIPLE_CHOICES   => 'Multiple Choices',
        self::S_MOVED_PERMANENTLY  => 'Moved Permanently',
        self::S_FOUND              => 'Found',  // 1.1
        self::S_SEE_OTHER          => 'See Other',
        self::S_NOT_MODIFIED       => 'Not Modified',
        self::S_USE_PROXY          => 'Use Proxy',
        // 306 is deprecated but reserved
        self::S_TEMPORARY_REDIRECT => 'Temporary Redirect',

        // Client Error 4xx
        self::S_BAD_REQUEST                     => 'Bad Request',
        self::S_UNAUTHORIZED                    => 'Unauthorized',
        self::S_PAYMENT_REQUIRED                => 'Payment Required',
        self::S_FORBIDDEN                       => 'Forbidden',
        self::S_NOT_FOUND                       => 'Not Found',
        self::S_METHOD_NOT_ALLOWED              => 'Method Not Allowed',
        self::S_NOT_ACCEPTABLE                  => 'Not Acceptable',
        self::S_PROXY_AUTHENTICATION_REQUIRED   => 'Proxy Authentication Required',
        self::S_REQUEST_TIMEOUT                 => 'Request Timeout',
        self::S_CONFLICT                        => 'Conflict',
        self::S_GONE                            => 'Gone',
        self::S_LENGTH_REQUIRED                 => 'Length Required',
        self::S_PRECONDITION_FAILED             => 'Precondition Failed',
        self::S_REQUEST_ENTITY_TOO_LARGE        => 'Request Entity Too Large',
        self::S_REQUEST_URI_TOO_LONG            => 'Request-URI Too Long',
        self::S_UNSUPPORTED_MEDIA_TYPE          => 'Unsupported Media Type',
        self::S_REQUESTED_RANGE_NOT_SATISFIABLE => 'Requested Range Not Satisfiable',
        self::S_EXPECTATION_FAILED              => 'Expectation Failed',

        // Server Error 5xx
        self::S_INTERNAL_SERVER_ERROR      => 'Internal Server Error',
        self::S_NOT_IMPLEMENTED            => 'Not Implemented',
        self::S_BAD_GATEWAY                => 'Bad Gateway',
        self::S_SERVICE_UNAVAILABLE        => 'Service Unavailable',
        self::S_GATEWAY_TIMEOUT            => 'Gateway Timeout',
        self::S_HTTP_VERSION_NOT_SUPPORTED => 'HTTP Version Not Supported',
        self::S_BANDWIDTH_LIMIT_EXCEEDED   => 'Bandwidth Limit Exceeded'
        );

    /**
     * Gets the message corresponding to a given HTTP status code.
     *
     * @param status code
     *
     * @return string The corresponding status message
     */
    public static function statusCodeToMessage($code)
    {
        return isset(self::$statusMessages[$code]) ? self::$statusMessages[$code] : strval($code);
    }

    /**
     * The HTTP response protocol.
     *
     * @var string
     */
    public $protocol;

    /**
     * The HTTP response status code.
     *
     * @var int
     */
    public $status;

    /**
     * The HTTP response status message from the corresponding code, e.g. 'Not Found' for 404 or
     * 'Internal Server Error' for 500.
     *
     * @var string
     */
    public $message;

    /**
     * The HTTP response headers.
     *
     * @var array
     */
    public $headers;

    /**
     * The HTTP response body.
     *
     * @var string
     */
    public $body;

    /**
     * Constructor
     *
     * If the $response parameter refers to a Zend HTTP response object, the $body, $headers, and
     * $protocol parameters are ignored.
     *
     * Otherwise:
     *
     * - The $response parameter is assumed to refer to a numerical HTTP response status code.
     *
     * - The protocol, if not specified, defaults to self::DEFAULT_HTTP_PROTOCOL.
     *
     * - The $body parameter may refer to:
     *
     *   + A resource, in which case it is assumed to be an open file handle to stream content back
     *     to the client (useful for large binary content such as photos).
     *
     *     OR
     *
     *   + The content itself as a string, in which case the 'Content-Length' header is added
     *     automatically and the 'Content-Type' header, if not given, defaults to
     *     self::DEFAULT_CONTENT_TYPE.
     *
     *     OR
     *
     *   + An array of parameters to be passed to sprintf(). This case is useful with error
     *     responses having formatted messages.
     *
     * @param mixed  $response HTTP status code or Zend HTTP response
     * @param mixed  $body     Optional response body (default is null for none)
     * @param array                  $headers  Optional response headers (default is empty array)
     * @param string $protocol Optional response protocol (default is class default)
     *
     * @return void
     */
    public function __construct($response, $body = null, array $headers = array(), $protocol = null)
    {
        if ($response instanceof Zend_HTTP_Response)
        {
            // Use Zend HTTP response object directly and ignore the $body, $headers, and $protocol
            // parameters
            $this->protocol = 'HTTP/' . $response->getVersion();
            $this->status = $response->getStatus();
            $this->headers = array_change_key_case($response->getHeaders());
            $this->body = $response->getBody();
        }
        else
        {
            // Set the HTTP response protocol
            if ($protocol)
                $this->protocol = $protocol;
            else
                $this->protocol = self::DEFAULT_HTTP_PROTOCOL;

            // Set the HTTP response status code and message
            $this->status = $response;

            // Normalize header names to lowercase
            $this->headers = array_change_key_case($headers);

            // If the body is an array, format it using sprintf()
            if (is_array($body))
                $body = call_user_func_array('sprintf', $body);

            // If a body was given...
            $this->body = $body;
            if ($this->body)
            {
                // Set the content length only if the it specifies the contnent itself
                if (!is_resource($this->body))
                    $this->headers[self::H_CONTENT_LENGTH] = strlen($this->body);

                // If no content type header was specified, assume plain text
                if (!isset($this->headers[self::H_CONTENT_TYPE]))
                    $this->headers[self::H_CONTENT_TYPE] = self::DEFAULT_CONTENT_TYPE;
            }
        }

        // Set status message
        $this->message = self::statusCodeToMessage($this->status);
    }

    /**
     * Sends the HTTP response to the Web client, i.e. echoes the protocol, status line, headers,
     * and body.
     *
     * @return void
     */
    public function send()
    {
        // Send the HTTP response status line
        header($this->protocol . ' ' . $this->status . ' ' . $this->message);

        // Send the response headers
        if (is_array($this->headers))
        {
            foreach ($this->headers as $name => $value)
                header("{$name}: {$value}");
        }

        // Send the response body
        if ($this->body)
        {
            // If the body is a resource, we assume that it is an open file handle to stream content
            // back to the client
            if (is_resource($this->body))
            {
                fpassthru($this->body);
                fclose($this->body);
            }
            else
            {
                echo $this->body;
            }
        }
    }

    /**
     * Returns a human-readable HTML representation of the HTTP response.
     *
     * @return string The HTML representation of the HTTP response
     */
    public function toHTML()
    {
        $html = '<ul>';
        $html .= sprintf('<li><label>protocol:</label>%s</li>', $this->protocol);
        $html .= sprintf('<li><label>status:</label><b>%s</b> %s</li>', $this->status, $this->message);

        $html .= sprintf('<li><label>headers:</label><ul>');
        if (is_array($this->headers))
        {
            foreach ($this->headers as $name => $value)
                $html .= sprintf('<li><label>%s:</label> %s</li>', $name, $value);
        }
        $html .= '</ul></li>';

        $html .= sprintf('<li><label>body:</label>%s</li>', htmlentities($this->body));
        $html .= '</ul>';

        return $html;
    }

    /**
     * Checks whether the response is an error (status == 4xx or 5xx).
     *
     * @return boolean
     */
    public function isError()
    {
        $kind = floor($this->status / 100);
        return ($kind == 4 || $kind == 5);
    }

    /**
     * Checks whether the response in successful (status == 1xx or 2xx or 3xx).
     *
     * @return boolean
     */
    public function isSuccessful()
    {
        $kind = floor($this->status / 100);
        return ($kind == 3 || $kind == 2 || $kind == 1);
    }

    /**
     * Checks whether the response is a redirection (status == 3xx).
     *
     * @return boolean
     */
    public function isRedirect()
    {
        $kind = floor($this->status / 100);
        return ($kind == 3);
    }
}