<?php
/**
 * File:        simpleSolr.php
 * 
 * @author      Al-Fallouji Bashar & Charron Pierrick
 * @version     1.0
 */

/**
 * simpleSolr class
 *
 * A lightweight class that allows to index, deindex and search via SolR
 */
class simpleSolr
{
    /**
     * host 
     * 
     * @var string Hostname of Solr (e.g. http://localhost:8080/solr)
     */
    public $host;

    /**
     * Constructor
     * 
     * @param string $host Hostname to Solr
     */
    public function __construct($host)
    {
 		$this->host = $host;
    }

    /**
     * Index a object in Solr
     * 
     * @param object $object 
     *
     * @return string Solr response
     */
    public function indexObject($object)
    {
		// An object must have an id and implement the getIndexableProperties method
		// Change this behaviour if it doesn't fit your requirements
		if(null === $object->id)
        {
            return false;
        }

        $xml  = '<add><doc>';

        // A uniqId is built based on the classname and id of the object
        // E.g. article_12, photo_3, etc...
        $xml .= '<field name="uniqId"><![CDATA['. get_class($object) . '_' . $object->id .']]></field>';
        $xml .= '<field name="id">' . $object->id . '</field>';

		// Object must implement a method named getIndexableProperties
        foreach($object->getIndexableProperties() as $key => $value)
        {
            $xml .= '<field name="' . $key . '"><![CDATA[' .  $value . ']]></field>';                
        }

        $xml .= '</doc></add>';

        if($this->index($xml))
        {
            return $this->commit();
        }

        return false;
    }

    /**
     * Remove a object from Solr
     * 
     * @param string $className Classname of object to deindex 
     * @param int $id Id of the object to deindex
     * @return boolean True on success, False on failure
     */
    public function deindexObject($className, $id)
    {
        // A uniqId is built based on the className and the id
        // E.g. article_12, photo_3, etc...
        if($this->deindex($className . '_' . $id))
        {
            return $this->commit();
        }

        return false;    
    }

    /**
     * Index multiple objects at once 
     * 
     * @param array $objects Array of objects 
     *
     * @return boolean True on success, else false
     */
    public function indexObjects(array $objects)
    {
        $xml  = '<add>';

        foreach($objects as $object)
        {
            $xml .= '<doc>';
            $xml .= '<field name="id"><![CDATA['. $object->id .']]></field>';

            foreach($object->getIndexableProperties() as $key => $value)
            {
                $xml .= '<field name="' . $key . '"><![CDATA[' .  $value . ']]></field>';                
            }

            $xml .= '</doc>';

            $response[] = $object->id;
        }

        $xml .= '</add>';

        if($this->index($xml))
        {
            if($this->commit())
            {
                return $response;
            }

            return false;
        }

        return false;
    }

    /**
     * Index a document in Solr 
     * 
     * @param string $xml String of the xml document to index
     *
     * @return string Solr response
     */
    public function index($xml)
    {
        $url = $this->host . DIRECTORY_SEPARATOR . 'update';
        $params = array('stream.body' => $xml, 'allowDups' => "false");

        return $this->sendPostData($url, $params);
    }

    /**
     * Remove a document from Solr
     * 
     * @param int $id Id of the document to remove
     * @param string $qry Lucene based syntax query
     *
     * @return string Solr response
     */
    public function deindex($id = null, $qry = null)
    {
        $url = $this->host . DIRECTORY_SEPARATOR . 'update';
        $streamBody = '<delete>';
        if(null !== $id)
        {
            $streamBody .= '<id>'. $id . '</id>';
        }

        if(null !== $qry)
        {
            $streamBody .= '<query>' . $qry . '</query>';
        }

        $streamBody .= '</delete>';

        $params = array('stream.body' => $streamBody);
        return $this->sendPostData($url, $params);
    }

    /**
     * Performs a commit on Solr
     * 
     * @return string Solr response
     */
    public function commit()
    {
        $url = $this->host . DIRECTORY_SEPARATOR . 'update';
        $params = array('stream.body' => '<commit optimize="true"/>');

        return $this->sendPostData($url, $params);
    }

    /**
     * Performs a search on Solr 
     * 
     * @param string $qry Lucene based syntax query to be sent to Solr
     * @param int $start Start at that specific row
     * @param int $rows Number of rows to fetch
     * @param string $indent 
     * @param string $version 
     *
     * @return array Returns an array containing the Solr response
     */
    public function search($qry, $start = 0, $rows = 10, $sort = '', $indent = 'on', $hl = null, $facets = null, $version = '2.2')
    {
        $url = $this->host . DIRECTORY_SEPARATOR . 'select';
        $params['q'] = $qry;
        $params['start'] = $start;
        $params['rows'] = $rows;
        $params['indent'] = $indent;
        $params['version'] = $version;
        $params['sort'] = $sort;

        if(is_array($hl))
        {
            foreach($hl as $key => $fragSize)
            {
                $params['f.'.$key.'.hl.fragsize'] = $fragSize;
            }
            $params['hl'] = 'true';
            $params['hl.fl'] = implode(',', array_keys($hl));
        }

		// Computing parameters for facetted search
		$extraParams = null;
		if(is_array($facets))
		{
			$params['facet'] = 'true';

			// Retrieve number of facet items wanted
			$params['facet.limit'] = getArrayParameter($facets['params'], 'limit', 10);

			// Retrieve sorting 
			$params['facet.sort'] = getArrayParameter($facets['params'], 'sort', 'true');

			// Retrieve minimum counts for facet fields that should be included in the response
			$params['facet.mincount'] = getArrayParameter($facets['params'], 'mincount', 1);

			foreach($facets['fields'] as $facet)
			{
				$extraParams .= '&facet.field=' . $facet['field'];
			}
		}

        // With this parameter, Solr will return a PHP serialized array
        $params['wt'] = 'phps';

        $res = $this->sendGetData($url, $params, $extraParams);

        // If $res is false, then Solr didn't response 
        if(false === $res)
        {
            $this->searchResult['numFound'] = 0;
            $this->searchResult['docs'] = array();            
            return false;
        }

        return unserialize($res);
    }

    public function classCount($className)
    {
        $response = $this->search('type:' . $className, 0, 0);
        return $response['response']['numFound'];
    }

    /**
     * Send data to a ressource via a GET request 
     * 
     * @param string $url Url to send data to
     * @param mixed $params Array of parameters to be sent by POST
     *
     * @return string Result of the GET request 
     */
    private function sendGetData($url, $params, $extraParams = null)
    {
		$qry = $url . DIRECTORY_SEPARATOR . '?' . http_build_query($params, null, '&') . $extraParams;
        
        // On production, put error_level to ~ E_WARNING, so it can intercept warning message in case of Solr is down 
		// file_get_contents will return false if Solr is down
		return file_get_contents($qry);
    }

    /**
     * Invoke an URL and returns the result 
     * 
     * @param string $url Url to query
     * @param array $params Array of parameters to be sent by POST
     *
     * @return string Result of the query
     */
    private function sendPostData($url, array $params = array())
    {
        $contextOpts = array('http' => array('method' => 'POST'));

        $stream = stream_context_create($contextOpts);            
        stream_context_set_option($stream, 'http', 'header', 'Content-Type: text/xml; charset=UTF-8');
        if(null !== getArrayParameter($params, 'stream.body', null))
        {
            stream_context_set_option($stream, 'http', 'content', $params['stream.body']);
        }

        // On production, put error_level to ~ E_WARNING, so it can intercept warning message in case of Solr is down 
        // file_get_contents will return false if Solr is down
        return file_get_contents($url, false, $stream);
    }
}

