<?php

require_once('PHPUnit/Framework.php');
require_once(dirname(__FILE__) . '/../config/config.php');

abstract class WcmRestToolbox extends PHPUnit_Framework_TestCase
{
    private $ch; // the curl handler

    private $method;

    private $response; // the http response
    private $headers; // the response headers
    private $body; // the response body

    private $requestBody; // the request body

    private $resource; // the parent resource
    private $query; // the query string for the request


    /*
     * setup the enviroment for the tests, called before each test
     */
    protected function setUp()
    {

        parent::setUp();
        // initialize the curl object
        $this->curlInit();
    }

    /*
     * Clean up, close all handlers, called after each test
     */
    protected function tearDown()
    {
        parent::tearDown();

        if (isset($this->ch))
        {
            curl_close($this->ch);
        }
    }

    /*
     * Initialize curl object and generic curl options
     */
    private function curlInit()
    {
        // initialize the curl object
        $this->ch = curl_init();

        // set generic curl options
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->ch, CURLOPT_HEADER, true);
        curl_setopt($this->ch, CURLOPT_PORT, PORT);

        $this->query = null;
        $this->url = null;
    }

    /* Set the curl options for GET
     *
     * @param $params The parameters for the GET query string(default is null)
     */
    protected function setupGET($params = null, $headers = null)
    {
        $this->method = 'GET';

        $this->curlInit();

        // add parameters if they exist
        if ($params != null)
        {
            $this->addParameters($params);
        }

        // create the url
        $this->url = BASE_URI . '/' . $this->resource . $this->query;

        //initialize and set up Curl options
        curl_setopt($this->ch, CURLOPT_URL, $this->url);
        curl_setopt($this->ch, CURLOPT_POST, false);
        curl_setopt($this->ch, CURLOPT_PUT, false);
        curl_setopt($this->ch, CURLOPT_HTTPGET, true);

        // set specific headers if any were passed in the $headers array
        if ($headers != null)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
        }

        if (DEBUG)
        {
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
        }
    }


    /*
     * Set the curl options for PUT
     *
     * @param $params The parameters for the PUT query string
     */
    protected function setupPUT($params = null, $headers = null, $body = null)
    {
        $this->method = 'PUT';

        // set the request body for possible debugging
        $this->requestBody = $body;
        
        $this->curlInit();
        // add parameters if they exist
        if ($params != null)
        {
            $this->addParameters($params);
        }

        // create the url
        $this->url = BASE_URI . '/' . $this->resource . $this->query;

        if ($body != null)
        {
            // Prepare the data for HTTP PUT.
            $putData = tmpfile();
            fwrite($putData, $body);
            fseek($putData, 0);
            curl_setopt($this->ch, CURLOPT_INFILE, $putData);
            curl_setopt($this->ch, CURLOPT_INFILESIZE, strlen($body));
            curl_setopt ($this->ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
        }
        
        // set specific headers if any were passed in the $headers array
        if ($headers != null)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
        }
        
        //initialize and set up Curl options
        curl_setopt($this->ch, CURLOPT_URL, $this->url);
        curl_setopt($this->ch, CURLOPT_POST, false);
        curl_setopt($this->ch, CURLOPT_HTTPGET, false);
        curl_setopt($this->ch, CURLOPT_PUT, true);
        



        if (DEBUG)
        {
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
            echo 'Request body:' . $this->requestBody . PHP_EOL;
        }
    }

    /*
     * Set the curl options for POST
     */
    protected function setupPOST($params = null, $body = null, $headers = null, $photo = false)
    {
        $this->method = 'POST';

        // set the request body for possible debugging
        $this->requestBody = $body;

        $this->curlInit();
        
        // add parameters if they exist to the request body(the request body won't be sent but we keep it in
        // this variable for debugging purposes
        if ($params != null)
        {
            $this->addParameters($params, true);
        }

        // create the url
        $this->url = BASE_URI . '/' . $this->resource . $this->query;

        curl_setopt($this->ch, CURLOPT_HTTPGET, false);
        curl_setopt($this->ch, CURLOPT_PUT, false);
        curl_setopt($this->ch, CURLOPT_POST, true);
        curl_setopt($this->ch, CURLOPT_URL, $this->url);

        if ($this->requestBody == null)
        {
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, '');
        }
        else
        {
            if ($photo)
            {
                // if this is a file upload then the parameters must be passed
                // as an assoc. array for curl to set the content type correctly
                curl_setopt($this->ch, CURLOPT_POSTFIELDS, $params);
            }
            else
            {
            // If this isn't a file upload then send the parameters as a string(from the request body)
            // so the content type is correctly set
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $this->requestBody);
            }
        }

        // set specific headers if any were passed in the $headers array
        if ($headers != null)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
        }

        if (DEBUG)
        {
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
            echo 'Request body:' . $this->requestBody . PHP_EOL;
        }
    }

    /*
     * Set the curl options for DELETE
     */
    protected function setupDELETE($params = null, $headers = null)
    {
        $this->method = 'DELETE';

        $this->curlInit();
        // add parameters if they exist
        if ($params != null)
        {
            $this->addParameters($params);
        }

        // create the url
        $this->url = BASE_URI . '/' . $this->resource . $this->query;

        curl_setopt($this->ch, CURLOPT_HTTPGET, false);
        curl_setopt($this->ch, CURLOPT_PUT, false);
        curl_setopt($this->ch, CURLOPT_POST, false);
        curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
        curl_setopt($this->ch, CURLOPT_URL, $this->url);

        // set specific headers if any were passed in the $headers array
        if ($headers != null)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
        }

        if (DEBUG)
        {
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
        }
    }

    /*
     * Setter for $resource
     * @param $resource String The value to set $this->resource to
     */
    protected function setResource($resource)
    {
        $this->resource = $resource;
    }

    /*
     * Getter for $resource
     * @returns the resource to request
     */
    protected function getResource()
    {
        return $this->resource;
    }

    /* Getter for $body
     * @returns the body of the http response
     */
    protected function getBody()
    {
        return $this->body;
    }

    /* Getter for $headers
     * @returns the headers of the http response
     */
    protected function getHeaders()
    {
        return $this->headers;
    }

    /*
     * Execute the curl request
     */
    protected function executeRequest()
    {

        // execute the Curl http request
        $this->response = curl_exec($this->ch);

        // retreive and test if a curl error occurred
        $error = curl_error($this->ch);
        $this->assertTrue(
            $error == '' || $error == null, 'Curl error: ' . $error . ' '
        );

        // extract the headers and body
        $header_size = curl_getinfo($this->ch, CURLINFO_HEADER_SIZE);
        $this->headers = substr($this->response, 0, $header_size);
        $this->body = substr($this->response, $header_size);

        if (DEBUG)
        {
            echo 'The body of the response:' . PHP_EOL;
            echo $this->getBody() . PHP_EOL;
            echo 'The curl info array of the last request:' . PHP_EOL;
            print_r(curl_getinfo($this->ch));
        }
    }

    /*
     * Validates the actual status code of the response with $status_code
     *
     * @param $status_code the status code to check for
     * @param $ch a curl handle to use to get the return status, if none
     *            is given then the object curl handle will be used
     *            (default is null)
     * @param $location Boolean True if the status code to validate comes
     *                          from a location header
     */
    protected function validateReturnStatus($status_code, $ch = null, $location = false)
    {
        // if no curl handler was specified then use $this objects curl handler
        if ($ch == null && $this->ch != null)
        {
            $ch = $this->ch;
        }
        if ($ch != null)
        {
            // retreive and test the status code of the response
            $returned_status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            $this->assertTrue(
                $returned_status_code == $status_code,
                ($location ? 'Location header error' . 
                PHP_EOL : '') .
                'Returned status code should be \'' . $status_code .
                '\' but is: ' . $returned_status_code . PHP_EOL .
                "The requested url was:'" .
                curl_getinfo($ch, CURLINFO_EFFECTIVE_URL) . PHP_EOL .
                ($this->requestBody ? 'The url decoded request body is: ' .
                urldecode(print_r($this->requestBody, true)) . PHP_EOL : '') .
                PHP_EOL . "The request response was: " . $this->body . PHP_EOL
            );
        }

    }

    /*
     * Returns the header specified as an agrument
     *
     * @param $key The header that value is to be returned
     *
     * @returns the value of the header specified as $key
     */
    protected function validateHeader($key)
    {
        return $this->getHeader($key);
    }



    /*
     * Validates that the body of the http response is a collection of valid
     * locations
     *
     * @param $query the query string to append to the location
     * @param $expect String A object represented in XML, all locations
     *                              should return objects of the same class,
     *                              only <id>, <name>, <className>
     *                              and <objectType> are validated
     * @param $id Array(int) An array of ids corresponding to the locations
     */
    protected function validateLocations($query, $expect=null, $ids=null)
    {

        $sxml = $this->validateBodyIsXML();

        // validate each location
        foreach ($sxml->xpath('//location') as $location) {

            $url = (string)$location[0] . $query;

            if (is_array($expect) && is_array($ids))
            {
                $this->validateLocation($url, null, $expect, array_shift($ids));
            }
            else
            {
                $this->validateLocation($url);
            }
        }

    }

    /* Validates that a url is valid by checking the return status of a request and
     * verifies the body contains a valid xml document. This method should have no
     * side effects on the state of $this
     * TODOTODOTODO: Validate in both xml and json here...
     *
     * @param string $url The url to validate
     * @param string $sessionId an optionnal sessionId parameter to be sent with
     *                          the request(default is null)
     * @param string $expect The expected object to be found at the url as a
     *                       string in xml excluding its object id; default is null
     * @param array[int] $id The id of the expected object to be found at the url;
     *                       default is null
     *
     * @return The body of the GET request for the url
     * TODO: verify that the sessionId parameter is used
     * TODO: $sessionId should be an array of parameters, take the WCM logic out of here
     * TODO: verify $expect and $id are being used, alot of parameters for the method...
     */
    protected function
        validateLocation($url, $sessionId=null, $expect=null, $id=null, $json=false)
    {
        // initialize a new curl object(default method is GET)
        $ch = curl_init($url);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        if (DEBUG)
        {
            curl_setopt($ch, CURLOPT_VERBOSE, 1);
        }

        $response = curl_exec($ch);

        if (DEBUG)
        {
            echo '*** VALIDATING THE LOCATION RETURNED IN THE \'location\' HEADER ***' . PHP_EOL;
            echo 'The body of the response:' . PHP_EOL . $response . PHP_EOL;
            echo 'The curl info array of the last request:' . PHP_EOL;
            print_r(curl_getinfo($ch));
        }

        $this->validateReturnStatus(200, $ch, true);

        // validate the body
        if ($json) // if json encoding was specified
        {
            // try and decode the json into an object, assert a failure on error
            if (json_decode($response) == null)
            {
                $this->assertTrue(
                    false, 'The json of the body response from "' . $url .
                    '" was not valid: ' . "\n" . $response
                );
            }
        }
        else // if xml encoding was specified
        {
            try // Attempt to create a simpleXML object with the response
            {
                new SimpleXMLElement($response);
            }
            catch (Exception $e)
            {
                $this->assertTrue(
                    false, 'The xml of the body response from "' . $url .
                    '" was not valid: ' . "\n" . $response
                );
            }
        }

        // if a specific object is expected
        if ($expect != null && $id != null)
        {
            // TODO: Put the $id in $expect as <id>$id</id>
            // validate the returned object is the expected one
            try
            {
                $this->validateReturnedObject($expect, $id, $response);
            }
            catch (Exception $e)
            {
                $this->assertTrue(
                    false, 'The returned object was not the expected one'
                );
            }
        }
        
        return $response;
    }

    /*
     * Validates that the body of the http response is a valid object id
     *
     * @param $resource String the resource to call
     * @param $query String the query string to be passed to the resource
     * TODO: Take the wcm logic out of this class and move it to WcmTest
     */
    protected function validateObjectId($resource, $query)
    {
        // setup the url
        $url = self::$baseURI . '/' . $resource . $query;

        // initalize the curl request
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        if (DEBUG)
        {
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
        }

        // execute the Curl http request
        $response = curl_exec($ch);

        // validate the return status code
        $this->validateReturnStatus(200, $ch);

    }

    /*
     * Verifies of the body of the response represents an object in XML and
     * validates its id tag againt the id recieved as a parameter
     *
     * @param $id The expect id of the xml object contained in the body
     * TODO: Take the wcm logic out of this class and move it to WcmTest
     */
    protected function xmlBodyHasId($id)
    {
        try
        {
            $sxml = new SimpleXMLElement($this->body);
        }
        catch (Exception $e)
        {
            $this->assertTrue(
                false, $e . 'The xml of the body was not valid: ' . "\n" .
                $this->body
            );
        }

        try
        {
            $idArray = $sxml->xpath('//id');
            $bodyId = (String)$idArray[0];
        }
        catch (Exception $e)
        {
            $this->assertTrue(false, 'Error with the xml structure.');
        }

        // Validate that the id contained in the xml is the same as $id
        $this->assertTrue(
            ($bodyId == $id), 'Unexpected object id.' . "\n" . 'Expected id: ' .
            $id . "\n" . 'Returned object:' . $this->body
        );

    }

    /*
     * Validates that the body of the http response in valid xml
     */
    protected function validateBodyIsXML()
    {
        if ($this->body != null && $this->body != '')
        {
            try
            {
                $sxml = new SimpleXMLElement($this->body);
            }
            catch (Exception $e)
            {
                $this->assertTrue(
                    false, 'The xml of the body was not valid: ' . "\n" .
                    $this->body
                );
            }

            return $sxml;
        }
    }

    protected function validateBodyIsJson()
    {
        if (json_decode($this->body) == null)
        {
            $this->assertTrue(
                false, 'The json of the body was not valid: ' . "\n" .
                $this->body
            );
        }
    }

    /*
     * Returns the header specified as an agrument
     *
     * @param $key The header that value is to be returned
     *
     * @returns the value of the header specified as $key
     */
    protected function getHeader($key)
    {
        // retreive the value of the key from the headers
        $pattern = '/' . $key  . ': (.*?)\n/';
        $match = preg_match($pattern, $this->headers, $result);
        $value = $result[1];
        $this->assertTrue(
            $match == 1, 'The response header \'' . $key . '\' was not found'
        );

        // return the value associated with $key
        return $value;
    }

    /*
     * Adds a parameter to $this->url
     *
     * @param string  $key the key of a key => value pair
     * @param string  $value the value of the key
     */
    private function addParameters($array)
    {
        foreach ($array as $key => $value)
        {
            $this->addParameter(urlencode($key), urlencode($value));
        }
    }

    /*
     * Adds a parameter to $this->url
     * 
     * This function, when used for a POST will contain an '&' at the end
     * of the returned string, it should be 
     *
     * @param string $key the key of a key => value pair
     * @param string $value the value of the key
     */
    private function addParameter($key, $value)
    {
        // If this is a POST request then
        if ($this->method === 'POST')
        {
            $symbol = '';
            
            if ($this->requestBody != null || $this->requestBody != '')
            {
                $symbol = '&';
            }
            
            $this->requestBody .=  $symbol . $key . '=' . $value;
        }
        else // otherwise it's a GET request
        {
            $symbol = '?'; // add an and symbol or a question mark for the query string

            if ($this->query != null || $this->query != '')
            {
                $symbol = '&';
            }
            
            $this->query .= $symbol . $key . '=' . $value;
        }
    }

    /*
     * Double encodes the sessionId part of the url
     * THIS IS A HACK THAT IS NECESSARY BECAUSE OF AN APACHE BUG(#34602)
     *
     * @param $url the url containing the sessionId to double encode
     */
    protected function doubleURLEncodeSessionId($url)
    {
        $url = trim($url);
        // retreive the sessionId part of the url
        $pattern = '/session\/(.*)/';
        $match = preg_match($pattern, $url, $result);

        $newSessionId = urlencode($result[1]);

        $newSessionId = urlencode($newSessionId);

        $url = str_replace($result[1], $newSessionId, $url);

        return $url;
    }

}