<?php
/**
 * Project:     WCM
 * File:        $Id: biz.item.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * This class represents a basic item of the website
 * An item can be public/private/protected and downloadable
 */
class item extends bizobject
{
    /**
     * (1) Represents a public access
     */
    const ACCESS_PUBLIC = 1;

    /**
     * (2) Represents a protected access
     */
    const ACCESS_PROTECTED = 2;

    /**
     * (3) Represents a private access
     */
    const ACCESS_PRIVATE = 3;

    /**
     * site id
     * @var int
     */
    public $siteId;

    /**
     * channel id
     * @var int
     */
    public $channelId;

    /**
     * title
     * @var string
     */
    public $title;

    /**
     * description
     * @var string
     */
    public $description;

    /**
     * free HTML text
     * @var string
     */
    public $text;

    /**
     * item type
     * @var string
     */
    public $type;

    /**
     * physical location of item (or null)
     * @var string
     */
    public $location;

    /**
     * access kind: either ACCESS_PUBLIC, ACCESS_PROTECTED or ACCESS_PRIVATE
     * @var int
     */
    public $access;

    /**
     * is bizitem an external URL?
     * @var bool
     */
    public $isURL;

    /**
     * publication date
     * @var string
     */
    public $publicationDate = null;

    /**
     * expiration date
     * @var string
     */
    public $expirationDate = null;



    //
    // Web 2.0 properties (moderation / contribution / rating / hits)
    //

    /**
     * Article moderation kind - see getModerationKindList()
     * @var string
     */
    public $moderationKind;

    /**
     * Contribution mode - see getContributionStateList()
     * @var string
     */
    public $contributionState;

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->publicationDate = null;
        $this->access = self::ACCESS_PUBLIC;
        $this->isURL = 0;

        $this->siteId = $this->channelId = null;
    }


    /**
    * Returns an array of access (access int => human readable description)
    *
    * @return array Possible access mode
    */
    public static function getAccessList()
    {
        return array(
            self::ACCESS_PUBLIC => _BIZ_ACCESS_PUBLIC,
            self::ACCESS_PROTECTED => _BIZ_ACCESS_PROTECTED,
            self::ACCESS_PRIVATE => _BIZ_ACCESS_PRIVATE
            );
    }

    /**
    * Returns an associative array of available types for a bizitem
    *
    * @return array (value => caption)
    */
    public static function getTypeList()
    {
        return array(
          'html'    => _BIZ_FREE_HTML,
          'audio'   => _BIZ_AUDIO,
          'video'   => _BIZ_VIDEO,
          'flash'   => _BIZ_FLASH,
          'pdf'     => _BIZ_PDF
          );
    }


    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    function ofClause($of)
    {
        if (null === $of || !is_array($of))
        {
            return;
        }

        $sql = null;

        foreach ($of as $key => $value)
        {
            switch ($key)
            {
                case 'site':
                    if (null !== $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'siteId=' . $value;
                    break;

                case 'channel':
                    if (null !== $sql)
                    {
                        $sql .= ' AND ';
                    }
                    $sql .= 'channelId=' . $value;
                    break;
            }
        }
        return $sql;
    }

    /**
    * Makes a record that this item was downloaded
    *
    * @param int $webuserId Id of the webuser who has download this item
    * @return int Number of affected rows
    */
    public function markAsDownloaded($webuserId = 0)
    {
        $query = 'INSERT INTO biz_item_downloads(itemId, webuserId, downloadDate, ipAddress) VALUES (?,?,?,?)';
        $params = array($this->id, $webuserId, date('Y-m-d H:i:s'), $_SERVER['REMOTE_ADDR']);

        return $this->database->executeStatement($query, $params);
  }

    /**
     * Get semantic text for item
     * @return string Semantic text
     */
    public function getSemanticText()
    {
        $content = '';
        $content .= trim($this->title, "\t\n\r\0\x0B.") . "\n";
        $content .= trim($this->text, " \t\n\r\0\x0B.") . "\n";

        return $content;
    }


    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid
     *
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        if ('' == trim($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_IS_MANDATORY;
            return false;
        }

        if (255 < strlen($this->title))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TITLE_TOO_LONG;
            return false;
        }

        return true;
    }

    /**
     * Gets the access types of this item
     *
     * @param bool $toXML TRUE if called by toXML() method
     *
     * @return array An array of the newsletter's subscriptions (as subscription objects)
     */
    public function getAssoc_accessTypes($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        return $this->getAccessList();
    }
}
