<?php
/**
 * Project:     WCM
 * File:        $Id: biz.photoManager.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-business-api
 * @version     4.x
 */

/**
 * The class that manages image files for photo bizobjects.
 */
class photoManager
{
    /**
     * Send file URL to the media manager, so the media
     * manager can grab the file from its current location. 
     * @var string
     */
    const FILE_SEND_METHOD_URL = 'url';
    
    /**
     * Send file to media manager via HTTP file upload.
     * @var string
     */
    const FILE_SEND_METHOD_UPLOAD = 'upload';
    
    /**
     * The media manager client.
     * @var Nstein_Cmf_Mediamanager_Client
     */
    private $mediaManagerClient;
    
    /**
     * The method to send image files to the media manager
     * (url or upload).
     * @var string
     */
    private $fileSendMethod;
    
    /**
     * Public constructor.
     * @param Nstein_Cmf_Mediamanager_Client $mediaManagerClient
     * The media manager client, used to connect to the back-end
     * media manager.
     * @param string $fileSendMethod The method to send files to
     * the media manager (url or upload).
     */
    public function __construct(Nstein_Cmf_Mediamanager_Client $mediaManagerClient, $fileSendMethod)
    {
        $this->mediaManagerClient = $mediaManagerClient;
        
        try
        {
            $this->setFileSendMethod($fileSendMethod);
        }
        catch (InvalidArgumentException $e)
        {
            $this->setFileSendMethod(self::FILE_SEND_METHOD_URL);
        }
    }
    
    /**
     * Set the file send method.
     * @param string $value
     */
    private function setFileSendMethod($value)
    {
        switch ($value)
        {
            case self::FILE_SEND_METHOD_URL:
            case self::FILE_SEND_METHOD_UPLOAD:
                $this->fileSendMethod = $value;
                break;
                
            default:
                throw new InvalidArgumentException('Invalid value.');
        }
    }
    
    /**
     * Check if the image of a given photo object exists.
     * @param photo $photo The photo.
     * @return boolean True if the image exists, false otherwise.
     */
    public function imageExists(photo $photo)
    {
        return $this->mediaManagerClient->imageExists($photo->id);
    }
    
    /**
     * Set the image for a given photo.
     * @param photo $photo The photo.
     * @param string $filePath The path of the source image file.
     * @return void
     * @exception Exception If an error occurs.
     */
    public function setImage(photo $photo, $filePath)
    {
        switch ($this->fileSendMethod)
        {
            case self::FILE_SEND_METHOD_URL:
                // Send image file URL.
                $this->mediaManagerClient->addImage(
                    $photo->id,
                    $photo->ratios,
                    $filePath
                );
                break;
                
            case self::FILE_SEND_METHOD_UPLOAD:
                // Send image file via HTTP file upload.
                $this->mediaManagerClient->uploadImage(
                    $photo->id,
                    $photo->ratios,
                    $filePath
                );
                break;
                
            default:
                throw new Exception('Invalid file send method.');
        }
    }
    
    /**
     * Update the cropping coordinates for a given photo object.
     * @param photo $photo The photo.
     * @return void
     * @exception Exception If an error occurs.
     */
    public function updateFormats(photo $photo)
    {
        $this->mediaManagerClient->updateFormats(
            $photo->id,
            $photo->ratios
        );
    }
    
    /**
     * Delete the image for a given photo object.
     * @param photo $photo The photo.
     * @return void
     * @exception Exception If an error occurs.
     */
    public function deleteImage(photo $photo)
    {
        $this->mediaManagerClient->deleteImage(
            $photo->id
        );
    }
    
    /**
     * Get the URL to the image of a given photo object.
     * @param photo $photo The photo.
     * @return string The URL of the image.
     */
    public function getImageUrl(photo $photo)
    {
        return $this->mediaManagerClient->getImageUrl(
            $photo->id
        );
    }
    
    /**
     * Get the URL of a thumbnail for a given photo object,
     * cropped with a given format.
     * @param photo $photo The photo.
     * @param string $format The name of the format.
     * @return string The URL of the thumbnail.
     */
    public function getCroppedImageUrl(photo $photo, $format)
    {
        return $this->mediaManagerClient->getCroppedImageUrl(
            $photo->id,
            $format
        );
    }
    
    /**
     * Get the URL of a thumbnail for a given photo object,
     * resized to fit in a given width/height set of boudaries.
     * At least one of width and height must be specified. The 
     * image will never be scaled up, so if the image is smaller 
     * than the boudaries, it will be unchanged.
     * @param photo $photo The photo.
     * @param int $width The maximum width, or null if unspecified.
     * @param int $height The maximum height, or null if unspecified.
     * @return string The URL of the thumbnail.
     */
    public function getResizedImageUrl(photo $photo, $width, $height)
    {
        return $this->mediaManagerClient->getResizedImageUrl(
            $photo->id,
            $width,
            $height
        );
    }
}
