<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.objectrelation.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class represents a relation between two business objects
 * A wcmObjectrelation is ordered (rank) and typed (kind)
 * Also, a wcmObjectrelation as a validity periode (validityDate and expirationDate)
 *
 */
class wcmObjectrelation extends wcmObject
{
    //
    // Kind of relations : an integer value from 1 to 5
    //

    /**
     * (1) IS_PART_OF     destination is part of source (e.g. chapitre of an article)
     */
    const IS_PART_OF = 1;

    /**
     * (2) IS_RELATED_TO  destination is related to source (e.g. link of an article)
     */
    const IS_RELATED_TO = 2;

    /**
     * (3) IS_COMPOSED_OF destination is composed of source (e.g. photo of a slideshow)
     */
    const IS_COMPOSED_OF = 3;

    /**
     * (4) IS_EMPTY specifically used to explain that there is no wcmObjectrelation
     */
    const IS_EMPTY = 4;

     /**
      * (5) IS_FORCED destination rank and value is forced by user (e.g. content of a channel)
      */
    const IS_FORCED = 5;


    //
    // Public members
    //

    /**
     * Unique identifier of wcmObjectrelation
     * @var int
     */
    public $id;

    /**
     * Class name of source bizobject
     * @var string
     */
    public $sourceClass;

    /**
     * Id of source bizobject
     * @var int
     */
    public $sourceId;

    /**
     * Class name of destination bizobject
     * @var string
     */
    public $destinationClass;

    /**
     * Id of destination bizobject
     * @var int
     */
    public $destinationId;

    /**
     * Kind of relation existing between source and destination
     * @var int
     */
    public $kind;

    /**
     * Rank of relation in siblings (for same source/destination/kind)
     * @var int
     */
    public $rank;

    /**
     * Optional header associated to relation
     * @var string
     */
    public $header;

    /**
     * Optional title associated to relation
     * @var string
     */
    public $title;

    /**
     * Optional validity date associated to relation
     * @var string
     */
    public $validityDate;

    /**
     * Optional expiration date associated to relation
     * (set to NULL (by default) to preserve from expiration)
     * @var string
     */
    public $expirationDate;

    /**
     * An assoc array of extended properties (name => value)
     * @var array
     */
    public $properties;


    /**
     * Constructor
     *
     * @param int $id Optional id to restore an existing relation
     */
    public function __construct($id = null)
    {
        parent::__construct($id);
        $this->objectType = 'wcmObjectrelation';
    }


    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();

        $this->kind = 1;
        $this->rank = 1;
        $this->header = $this->title = null;
        $this->validityDate = $this->expirationDate = null;
        $this->properties = array();
    }


    /**
     * Unserialize some properties if needed
     * This method is called by nextEnum() and refresh()
     */
    public function unserializeProperties()
    {
        $this->properties = ($this->properties) ? unserialize($this->properties) : array();
    }

    /**
     * Get the database used to store/fetch object
     *
     * @return wcmDatabase Database object
     * @throws Exception    When invalid connector reference is supplied
     */
    public function getDatabase()
    {
        // Retrieve connector and then database
        if (!$this->database)
        {
            $config = wcmConfig::getInstance();
            $connectorCode = $config['wcm.default.relation-connector'];
            $connector = wcmProject::getInstance()->datalayer->getConnectorByReference($connectorCode);
            if (!$connector)
            {
                throw new Exception(__CLASS__ . " :: invalid connector reference '" . $connectorCode . "'");
            }

            $this->database = $connector->getBusinessDatabase();
            $this->tableName = '#___relation';
        }
        return $this->database;
    }

    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    public function ofClause($of)
    {
        if ($of == null || !is_array($of))
        {
            return null;
        }

        $sql = null;
        foreach ($of as $className => $id)
        {
            // Assume we want to browse relation 'of' a specific source
            if ($sql != null)
            {
                $sql .= ' AND ';
            }
            $sql .= 'sourceClass="' . $className . '" AND sourceId=' . $id;
        }
        return $sql;
    }

    /**
     * Removes any relation which has expired (using the 'expirationDate' property).
     *
     * @return int Number of affected rows
     */
    public static function purge()
    {
        // Build a relation to retrieve database and tableName
        $relation = new wcmObjectrelation();

        // Remove any expired relation
        $sql = 'DELETE FROM ' . $relation->getTableName() . ' WHERE expirationDate IS NOT NULL and expirationDate < ?';
        return $relation->getDatabase()->executeStatement($sql, array(date('Y-m-d H:i:s')));
    }

    /**
     * Retrieve relations related to a bizobject and return their corresponding associative arrays
     *
     * @param bizobject $bizobject Bizobject used as source object (and potentially destination object)
     * @param int $kind Used to restrict to a specific kind of relation (or null for all)
     * @param string $destinationClass Used to restrict to a specific destinationClass (or null for all)
     * @param bool $toXML TRUE when this method is called in the context of 'toXML()' method
     * @param string $validityDate Used as a validityDate filter on biz_relations
     *
     * @return array An array of assoc arrays (wcmObjectrelation->getAssocArray(false) calls)
     */
    public static function getBizobjectRelations($bizobject, $kind=null, $destinationClass=null, $toXML = false, $validityDate = null)
    {
        $where = "sourceClass='" . $bizobject->getClass() . "' AND sourceId=" . $bizobject->id;
        if ($destinationClass)
        {
            $where .= " AND destinationClass='" . $destinationClass . "'";
        }
        if ($kind)
        {
            $where .= ' AND kind=' . $kind;
        }
        if ($validityDate)
        {
            $where .= " AND validityDate='" . $validityDate . "' ";
        }

        $relations = array();

        $enum = new wcmEnumeration('wcmObjectrelation', $where, 'validityDate, rank');
        foreach ($enum as $rel)
        {
            $relations[] =  $rel->getAssocArray($toXML);
        }
        return $relations;
    }

    /**
     * Retrieve inbound relations of a bizobject and return their corresponding associative arrays
     *
     * @param bizobject $bizobject Bizobject used as source object (and potentially destination object)
     * @param int $kind Used to restrict to a specific kind of relation (or null for all)
     * @param string $sourceClass Used to restrict to a specific sourceClass (or null for all)
     * @param bool $toXML TRUE when this method is called in the context of 'toXML()' method
     * @param string $validityDate Used as a validityDate filter on biz_relations
     *
     * @return array An array of assoc arrays (wcmObjectrelation->getAssocArray(false) calls)
     */
    public static function getBizobjectInboundRelations($bizobject, $kind=array(wcmObjectrelation::IS_COMPOSED_OF, wcmObjectrelation::IS_RELATED_TO), $sourceClass=null, $toXML = false, $validityDate = null)
    {
        $where = "destinationClass='" . $bizobject->getClass() . "' AND destinationId=" . $bizobject->id;
        if ($sourceClass)
        {
            $where .= " AND sourceClass='" . $sourceClass . "'";
        }
        if ($kind)
        {
            $where .= ' AND (kind=' . implode(' OR kind= ', $kind) . ')';
        }
        if ($validityDate)
        {
            $where .= " AND validityDate='" . $validityDate . "' ";
        }

        $relations = array();

        $enum = new wcmEnumeration('wcmObjectrelation', $where, 'validityDate, rank');
        foreach ($enum as $rel)
        {
            $relations[] = $rel->getAssocArray($toXML);
        }
        return $relations;
    }

    /**
     * Removes any reference to a specific bizobject.
     *
     * @param bizobject $bizobject The bizobject to remove
     *
     * @return int The number of affected rows
     */
    public static function removeBizobject($bizobject)
    {
        // Ignore empty bizobject
        if (!$bizobject || !$bizobject->id)
        {
            return 0;
        }

        // Remove any reference from/to bizobject
        $className = $bizobject->getClass();
        $id = $bizobject->id;

        // Build a relation to retrieve database and tableName
        $relation = new wcmObjectrelation();

        $sql = 'DELETE FROM ' . $relation->getTableName() . ' WHERE (sourceClass=? AND sourceId=?) OR (destinationClass=? AND destinationId=?)';
        return $relation->getDatabase()->executeStatement($sql, array($className, $id, $className, $id));
    }

    /**
     * Serialize all relations of a bizobject.
     * This method is used to serialize the bizobject and its result will be
     * used by the restoreBizobject method.
     *
     * @param bizobject $bizobject The bizobject to serialize
     *
     * @return array An array of wcmObjectrelation's public properties
     */
    public static function archiveBizobject($bizobject)
    {
        $relations = array();
        $where = "sourceClass='" . $bizobject->getClass() . "' AND sourceId=" . $bizobject->id;
        
        $enum = new wcmEnumeration('wcmObjectrelation', $where, 'validityDate, rank');
        foreach ($enum as $relation)
        {
            $relations[] = getPublicProperties($relation);
        }
        return $relations;
    }

    /**
     * Restore all relations of a bizobject.
     * This methods will delete all current relations and rebuild them.
     *
     * @param bizobject $bizobject The bizobject to restore
     * @param array $relations An array of array representing public properties of wcmObjectrelation
     */
    public static function restoreBizobject($bizobject, $relations)
    {
        // Ignore empty bizobject
        if (!$bizobject || !$bizobject->id)
        {
            return;
        }

        // delete relations currently stored in DB
        $relation = new wcmObjectrelation();
        $sql = 'DELETE FROM ' . $relation->getTableName() . ' WHERE (sourceClass=? AND sourceId=?) OR (destinationClass=? AND destinationId=?)';
        $relation->getDatabase()->executeStatement(
            $sql,
            array(
                $bizobject->getClass(),
                $bizobject->id,
                $bizobject->getClass(),
                $bizobject->id
            )
        );

        // restore relations (from $relations parameter)
        if (is_array($relations))
        {
            foreach ($relations as $relationArray)
            {
                // restore relation
                bindArrayToObject($relationArray, $relation);
                $relation->id = 0;
                $relation->sourceClass = $bizobject->getClass();
                $relation->sourceId = $bizobject->id;
                $relation->save();
            }
        }
    }


    /**
     * Gets the closest date with a wcmObjectrelation that has the same
     * sourceId, sourceClass and kind as this relation.
     *
     * @param string $referenceDate Date of reference (Y-m-d) to search for (or null for today)
     *
     * @return string The nearest date in the format Y-m-d
     */
    public function getLastValidityDate($referenceDate=null)
    {
        if (!$referenceDate)
        {
            $referenceDate = date('Y-m-d');
        }

        $sql  = 'SELECT MAX(validityDate) FROM ' . $this->database->quoteIdentifier($this->tableName);
        $sql .= ' WHERE sourceClass=? AND sourceId=? AND kind =? AND validityDate < ?';

        $params = array($this->sourceClass, $this->sourceId, $this->kind, $referenceDate);

        return $this->database->executeScalar($sql, $params);
    }

    /**
     * Gets all validity dates with same sourceId, sourceClass and kind
     *
     * @param int $limit Maximum number of dates returned (0, by default, for all dates)
     *
     * @return array All validity dates with same sourceId, sourceClass and kind by decreasing order
     */
    public function getValidityDates($limit = 0)
    {
        $dates = array();

        $sql = 'SELECT DISTINCT validityDate FROM ' . $this->tableName . ' WHERE sourceClass=? AND sourceId=? AND kind=?';
        $params = array($this->sourceClass, $this->sourceId, $this->kind);

        $rs = $this->database->executeQuery($sql, $params, 0, $limit, wcmDatabase::FETCH_NUM);
        if ($rs != null)
        {
            while ($rs->next())
            {
                $dates[] = $rs->get(1);
            }
        }

        return $dates;
    }

    /**
     * Validation check.
     *
     * This will see if an identical relationship already exists
     *
     * @see api/core/wcmObject#checkValidity()
     * @return bool
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        // We're checking for duplicate relationship, so if this has an ID, a duplicate won't be made
        if ($this->id)
        {
            return true;
        }

        $sql = 'SELECT id FROM biz__relation WHERE sourceClass=:sourceClass AND sourceId=:sourceId AND destinationClass=:dest AND destinationId=:destId AND kind=:kind';
        $id = $this->database->executeScalar(
            $sql,
            array(
                'sourceClass' => $this->sourceClass,
                'sourceId' => $this->sourceId,
                'dest' => $this->destinationClass,
                'destId' => $this->destinationId,
                'kind' => $this->kind
            )
        );
        return !($id > 0);
    }

    /**
     * Gets the last used position (rank) for this source.
     *
     * @return int The highest rank of this kind of wcmObjectrelation linked to the sourceObject
     */
    function getLastRank()
    {
        $sql  = 'SELECT MAX(rank) AS nextPos FROM ' . $this->database->quoteIdentifier($this->tableName);
        $sql .= " WHERE sourceClass='" . $this->sourceClass . "' AND sourceId=" . $this->sourceId . ' AND kind =' . $this->kind;
        if (null != $this->validityDate)
        {
            $sql .= " AND validityDate ='" . $this->validityDate . "'";
        }

        return $this->database->executeScalar($sql);
    }

    /**
     * Removes the wcmObjectrelation at a specified rank  (for same sourceClass and sourceId)
     *
     * @param int $rank The rank of the relation to remove (or null to use current)
     * @param bool $shift To shift previous positions
     *
     * @return bool True on success, false on failure
     */
    function removeByRank($rank = null, $shift=true)
    {
        // Get rank from current item ?
        if (null == $rank)
        {
            $rank = $this->rank;
        }

        // Delete reference
        $sql  = 'DELETE FROM ' . $this->database->quoteIdentifier($this->tableName) . ' WHERE sourceClass=? AND sourceId=? AND rank=?';

        $params = array($this->sourceClass, $this->sourceId, $rank);

        if ($this->validityDate != null)
        {
            $sql .= ' AND validityDate=?';
            $params[] = $this->validityDate;
        }


        if ($this->database->executeStatement($sql, $params) <= 0)
        {
            return false;
        }

        if ($shift)
        {
            // Shift previous ranks
            $sql  = 'UPDATE ' . $this->database->quoteIdentifier($this->tableName) . ' SET rank=rank-1 WHERE rank>? AND sourceClass=? and sourceId=?';
            $params = array($rank, $this->sourceClass, $this->sourceId);
            if ($this->validityDate != null)
            {
                $sql .= ' AND validityDate=?';
                $params[] = $this->validityDate;
            }

            return $this->database->executeStatement($sql, $params) > 0;
        }

        return true;
    }

    /**
     * Remove all relations with a specific kind (for same sourceClass and sourceId)
     *
     * @param int  $kind Kind of relations to remove (or null to use current)
     * @param bool $sameValidityDate TRUE to remove only relations with same validity date (TRUE by default)
     *
      *@return int Number of affected rows
     */
    function removeByKind($kind = null, $sameValidityDate = true)
    {
        $sql = 'DELETE FROM ' . $this->tableName . ' WHERE sourceClass=? AND sourceId=? ';

        $params = array($this->sourceClass, $this->sourceId);

        if ($sameValidityDate)
        {
            $sql .= 'AND validityDate=?';
            $params[] = $this->validityDate;
        }
        if (!$kind)
        {
            $kind = $this->kind;
        }

        $sql .= 'AND kind=?';
        $params[] = $kind;

        return $this->database->executeStatement($sql, $params);
    }

    /**
     * Remove all relations with a specific validityDate (for same sourceClass and sourceId)
     *
     * @param date $validityDate Validity date of relations to remove (or null to use current)
     * @param int  $kind Optional kind of relations to remove (or null for any kind)
     *
      *@return int Number of affected rows
     */
    function removeByValidityDate($validityDate = null, $kind = null)
    {
        if ($validityDate === null)
        {
            $validityDate = date('Y-m-d');
        }
        $sql = 'DELETE FROM ' . $this->tableName . ' WHERE sourceClass=? AND sourceId=? AND validityDate=?';
        if ($kind)
        {
            $sql .= 'AND kind=?';
        }

        $params = ($kind == null)
            ? array($this->sourceClass, $this->sourceId, $validityDate)
            : array($this->sourceClass, $this->sourceId, $validityDate, $kind);

        return $this->database->executeStatement($sql, $params);
    }

    /**
     * Adds the relation to the DB.
     */
    public function addBizrelation()
    {
        $this->database->storeSysObject($this);
    }

    /**
     * Gets the last used position (rank) for this source.
     *
     * @return int The highest rank of this kind of bizrelation linked to the sourceObject
     *
     */
    function getLastPosition()
    {
        $sql  = 'SELECT MAX(rank) AS nextPos FROM ' . $this->database->quoteIdentifier($this->tableName);
        $sql .= " WHERE sourceClass='" . $this->sourceClass . "' AND sourceId=" . $this->sourceId . ' AND kind =' . $this->kind;
        if ($this->validityDate != null)
        {
            $sql .= " AND validityDate ='" . $this->validityDate . "'";
        }

        return $this->database->executeScalar($sql);
    }

    /**
     * Removes one kind of bizrelations where the destinationClass.
     *
     * @param int $kind The kind of bizrelations to remove
     *
     * @return boolean True on success, false on failure
     *
     */
    function removeOne($kind = null)
    {
        if (!$kind)
        {
            $kind = $this->kind;
        }
        // Delete reference
        $sql = 'DELETE FROM ' . $this->database->quoteIdentifier($this->tableName) . ' WHERE sourceClass=? AND sourceId=? AND destinationClass=? AND kind=?';
        return $this->database->executeStatement($sql, array($this->sourceClass, $this->sourceId, $this->destinationClass, $kind)) > 0;
    }

    /**
     * Moves the wcmObjectrelation from one rank to another
     *
     * @param int  $from The rank from which to move
     * @param int  $to   The rank to which to move
     *
     * @return bool True on success, false on failure
     *
     */
    function move($from, $to)
    {
        // Switch ranks
        $sql = 'UPDATE ' . $this->database->quoteIdentifier($this->tableName) . ' SET rank=? WHERE rank=? AND sourceClass=? AND sourceId=? AND kind=?';
        $array = array('99999', $from, $this->sourceClass, $this->sourceId, $this->kind);
        if ($this->validityDate != null)
        {
            $sql .= ' AND validityDate=?';
            $array = array('99999', $from, $this->sourceClass, $this->sourceId, $this->kind, $this->validityDate);
        }
        $this->database->executeStatement($sql, $array);

        $sql = 'UPDATE ' . $this->database->quoteIdentifier($this->tableName) . ' SET rank=? WHERE rank=? AND sourceClass=? AND sourceId=? AND kind=?';
        $array = array($from, $to, $this->sourceClass, $this->sourceId, $this->kind);
        if ($this->validityDate != null)
        {
            $sql .= ' AND validityDate=?';
            $array = array($from, $to, $this->sourceClass, $this->sourceId, $this->kind, $this->validityDate);
        }
        $this->database->executeStatement($sql, $array);

        $sql = 'UPDATE ' . $this->database->quoteIdentifier($this->tableName) . ' SET rank=? WHERE rank=? AND sourceClass=? AND sourceId=? AND kind=?';
        $array = array($to, '99999', $this->sourceClass, $this->sourceId, $this->kind);
        if ($this->validityDate != null)
        {
            $sql .= ' AND validityDate=?';
            $array = array($to, '99999', $this->sourceClass, $this->sourceId, $this->kind, $this->validityDate);
        }
        $this->database->executeStatement($sql, $array);

        return true;
    }

    /**
     * Copy all wcmObjectrelation from one validityDate to another
     *
     * @param string $fromDate The source validityDate
     * @param string $toDate   The destination validityDate
     *
     * @return bool True if copy was successfull, otherwise false
     */
    public function copy($fromDate, $toDate)
    {
        $where = "sourceId = '" . $this->sourceId . "' AND sourceClass = '" . $this->sourceClass . "' AND  kind = '" . $this->kind . "' AND validityDate = '" . $fromDate . "'";
        $enum = new wcmEnumeration('wcmObjectrelation', $where, 'rank');

        foreach ($enum as $relation)
        {
            $relation->id = 0;
            $relation->validityDate = $toDate;
            if (!$relation->save())
            {
                $relation->endEnum();
                return false;
            }
        }

        return true;
    }

    /**
     * Returns the destination object of the wcmObjectrelation
     *
     * @return bizobject The destination bizobject or null if not found
     */
    public function getDestination()
    {
        $bo = wcmObjectFactory::create($this->destinationClass, $this->destinationId);
        return ($bo->id) ? $bo : null;
    }

    /**
     * Exposes 'destination' to the get assoc array
     *
     * @param bool $toXML True when called in the context of toXML
     * @return array the destination bizobject's getAssocArray (or null if not found)
     */
    public function getAssoc_destination($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $bo = $this->getDestination();
        if ($bo === null)
        {
            return null;
        }

        return $bo->getAssocArray($toXML);
    }

    /**
     * Returns the source object of the wcmObjectrelation
     *
     * @return bizobject The source bizobject or null if not found
     */
    public function getSource()
    {
        $bo = wcmObjectFactory::create($this->sourceClass, $this->sourceId);
        return ($bo->id) ? $bo : null;
    }

    /**
     * Exposes 'source' to the get assoc array
     *
     * @param bool $toXML True when called in the context of toXML
     * @return array the source bizobject's getAssocArray (or null if not found)
     */
    public function getAssoc_source($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $bo = $this->getSource();
        if ($bo === null)
        {
            return null;
        }

        return $bo->getAssocArray($toXML);
    }

    /**
     * Starts enumeration of wcmObjectrelation using existing characteristics
     * (sourceClass, sourceId, destinationClass, destinationId, kind)
     * and a specific validityDate
     *
     * @param string $referenceDate Date of reference (Y-m-d)
     * @param int $offset Optional offset to start enumeration
     * @param int $limit Optional limit to end enumeration
     *
     * @return bool TRUE on success
     */
    public function beginEnumOnDate($referenceDate, $offset, $limit)
    {
        $where = "validityDate='" . $referenceDate . "'";

        if ($this->sourceClass)
        {
            $where .= " AND sourceClass='" . $this->sourceClass . "'";
        }
        if ($this->sourceId)
        {
            $where .= " AND sourceId='" . $this->sourceId . "'";
        }
        if ($this->destinationClass)
        {
            $where .= " AND destinationClass='" . $this->destinationClass . "'";
        }
        if ($this->destinationId)
        {
            $where .= " AND destinationId='" . $this->destinationId . "'";
        }
        if ($this->kind)
        {
            $where .= " AND kind='" . $this->kind . "'";
        }

        return $this->beginEnum($where, 'rank', $offset, $limit);
    }
}
