<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.cache.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class supports different caching mechanism based on configuration file.
 * The best practice is to use the 'APC' extension, However, if APC is not set,
 * wcmCache will use Cache_Lite (as 'Lite' mode)
 */
class wcmCache
{

    /**
     * Static cache for the request
     * @var array
     */
    private $registryCache = array();

    /**
     * Cache plugin
     * @var wcmICachePlugin
     */
    private $plugin = null;

    /**
     * Prefix key
     * @var String
     */
    private $prefixKey = null;

    /**
     * __construct
     *
     * @param wcmICachePlugin $plugin
     * @param mixed $prefixKey
     */
    public function __construct(wcmICachePlugin $plugin, $prefixKey)
    {
        $this->plugin = $plugin;
        $this->prefixKey = $prefixKey;
    }

    /**
     * Store a variable in cache
     *
     * @param string $key Unique key used to retrieve (fetch) variable
     * @param mixed $var Variable to cache
     * @param int $ttl Optional time to live (default is configuration ttl)
     *
     * @return bool FALSE on failure, TRUE on success
     */
    public function store($key, $var, $ttl = 0)
    {
        $key = $this->prefixKey . $key;

        // use default ttl?
        if ($ttl == 0)
        {
            $config = wcmConfig::getInstance();
            $ttl = intval($config['wcm.cache.lifeTime']);
        }

        $this->registryCache[$key] = $var;

        return $this->plugin->store($key, $var, $ttl);
    }

    /**
     * Fetch a cached variable
     *
     * @param string $key Unique key used to retrieve (fetch) variable
     *
     * @return mixed Variable value or else FALSE on failure
     */
    public function fetch($key)
    {
        $key = $this->prefixKey . $key;
        if (isset($this->registryCache[$key]))
        {
            return $this->registryCache[$key];
        }

        $cached = $this->plugin->fetch($key);
        if (false === $cached)
        {
            return false;
        }
        $this->registryCache[$key] = $cached;
        return $cached;
    }

    /**
     * Delete a cached variable
     *
     * @param string $key Unique key representing the variable to delete
     *
     * @return bool FALSE on failure, TRUE on success
     */
    public function delete($key)
    {
        $key = $this->prefixKey . $key;

        unset($this->registryCache[$key]);
        return $this->plugin->delete($key);
    }

    /**
     * Clear the entire cache
     * @return bool TRUE on success
     */
    public function clear()
    {
        $this->registryCache = array();
        return $this->plugin->clear();
    }

    /**
     * Retrieve the value of an element of a cached assoc array
     *
     * @param string $key   Key of cached array
     * @param string $elem  Key of elem in the cached array
     *
     * @return mixed Cached element or NULL if not exists
     */
    public function getElem($key, $elem)
    {
        return getArrayParameter(wcmCacheRegistry::get()->fetch($key), $elem);
    }

    /**
     * Set an element of a cached assoc array
     *
     * @param string $key   Key of cached array
     * @param string $elem  Key of element in the cached array
     * @param mixed  $value The element value to cache
     */
    public function setElem($key, $elem, $value)
    {
        $assocArray = wcmCacheRegistry::get()->fetch($key);
        if ($assocArray === false || !is_array($assocArray))
        {
            $assocArray = array();
        }
        $assocArray[$elem] = $value;
        wcmCacheRegistry::get()->store($key, $assocArray);
    }

    /**
     * Unset an element of a cached assoc array
     *
     * @param string $key   Key of cached array
     * @param string $elem  Key of element in the cached array
     */
    public function unsetElem($key, $elem)
    {
        $assocArray = wcmCacheRegistry::get()->fetch($key);
        
        
        if ($assocArray === false || !is_array($assocArray))
        {
            $assocArray = array();
        }
        else
        {
            if (isset($assocArray[$elem]))
            {
                unset($assocArray[$elem]);
            }
        }   
        wcmCacheRegistry::get()->store($key, $assocArray);
    }
}
