<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.logger.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * WCM log error constant
 */
define('WCMLOG_ERROR', 1);

/**
 * WCM log warning constant
 */
define('WCMLOG_WARNING', 2);

/**
 * WCM log message constant
 */
define('WCMLOG_MESSAGE', 3);

/**
 * WCM log verbose constant
 */
define('WCMLOG_VERBOSE', 4);

/**
 * WCM log debug constant
 */
define('WCMLOG_DEBUG', 5);

/**
 * The logger class helps implementation of a simple log tool
 *
 */
class wcmLogger
{
    /**
     * Stack of messages : an array of array ($kind, $time, $message)
     * @var array
     */
    public $messages = array();

    /**
     * Is verbose?
     * @var bool
     */
    protected $verbose;

    /**
     * Debug?
     * @var bool
     */
    protected $debug;

    /**
     * Flush log to file?
     * @var bool
     */
    protected $flushToFile;

    /**
     * If set, then wcmLogger will attempt to send the log message
     * to this broker queue via wcmMessage
     *
     * @var string
     */
    protected $msgQueue;

    /**
     * Total number of db queries
     *
     * @var int
     */
    protected $totalQueries = 0;

    /**
     * Previous error handler function name
     * @var string
     */
    private $previousHandler = null;

    /**
     * Constructor
     *
     * @param bool   $verbose       Store verbose message (default true)
     * @param bool   $debug         Display error message (default false when error is handled)
     * @param string $flushToFile   Name of file to flush (default null)
     * @param bool   $handleError   Handle all PHP error (default false)
     */
    public function __construct($verbose = false, $debug = false, $flushToFile = null, $handleError = false)
    {

        $conf = wcmConfig::getInstance();
        if (isset($conf['wcm.messaging.destinations.logging']))
        {
            $this->msgQueue = $conf['wcm.messaging.destinations.logging'];
        }

        $this->debug = $debug;
        $this->verbose = $verbose;
        $this->flushToFile = $flushToFile;
        if ($handleError)
        {
            $this->startErrorHandling();
        }
    }

    /**
     * Destructor
     */
    function __destruct()
    {
        if (null !== $this->previousHandler)
        {
            $this->stopErrorHandling();
        }
        $this->eraseLog();
        $this->messages = null;
    }

    /**
     * Stops error handling
     */
    private function stopErrorHandling()
    {
        if (null != $this->previousHandler)
        {
            restore_error_handler($this->previousHandler);
        }
    }

    /**
     * Starts error handling
     */
    private function startErrorHandling()
    {
        $this->previousHandler = set_error_handler(array(&$this, 'errorHandler'));
    }

    /**
     * Custom error handler
     * @param int Error code
     * @param string Error string
     * @param string File in which error happened
     * @param int Line on which error happened
     */
    public function errorHandler($errno, $errstr, $errfile, $errline)
    {
        // Does caller has preceded function invokation with '@' ?
        if (error_reporting() == 0)
        {
            return;
        }

        // Format error message
        $message = $errstr . ' : ' . $errfile . ', line #' . $errline;

        switch ($errno)
        {
            case E_ERROR:
            case E_PARSE:
            case E_CORE_ERROR:
            case E_CORE_WARNING:
            case E_COMPILE_ERROR:
            case E_COMPILE_WARNING:
            case E_USER_ERROR:
                $this->logError($message, $this->debug);
                break;

            case E_WARNING:
            case E_NOTICE:
            case E_USER_WARNING:
                $this->logWarning($message, $this->debug);
                break;

            case E_USER_NOTICE:
                $this->logMessage($message, $this->debug);
                break;

            case E_STRICT:
                $this->logVerbose($message, $this->debug);
                break;

            default:
                // We should never reach this part of the code !
                $this->logError($message, $this->debug);
                break;
        }
    }

    /**
     * Erase all messages
     */
    public function eraseLog()
    {
        unset($this->messages);
        $this->messages = array();
    }

    /**
     * Adds a new log message into the stack
     *
     * @param int $kind         Log kind (WCMLOG_XXX)
     * @param string $message   Mesage to log
     */
    public function addLog($kind, $message)
    {
        // Display verbose message ?
        if ((WCMLOG_VERBOSE == $kind) && (false == $this->verbose))
        {
            return;
        }

        // Add message to log stack
        $t = date('Y-m-d H:i:s');

        // Flush to file ?
        if ($this->flushToFile)
        {
            $text = null;
            switch ($kind)
            {
                case WCMLOG_ERROR:
                    $text = 'ERR: [' . $t . '] ' . $message . PHP_EOL;
                    break;
                case WCMLOG_WARNING:
                    $text = 'WRN: [' . $t . '] ' . $message . PHP_EOL;
                    break;
                case WCMLOG_MESSAGE:
                    $text = 'MSG: [' . $t . '] ' . $message . PHP_EOL;
                    break;
                case WCMLOG_VERBOSE:
                    $text = 'VRB: [' . $t . '] ' . $message . PHP_EOL;
                    break;
                case WCMLOG_DEBUG:
                    $text = 'DBG: [' . $t . '] ' . $message . PHP_EOL;
                    break;
            }

            // Is the output a resource (STDOUT, STDERR)?
            if (is_resource($this->flushToFile))
            {
                fwrite($this->flushToFile, $text);
            }
            else
            {
                // Save text in "append" mode
                saveToFile($this->flushToFile, $text, true);
            }
        }
        else
        {
            // store message in memory
            $this->messages[] = array($t, $kind, $message);
        }
    }

    /**
     * Log an error message
     *
     * @param string $message Message to log
     */
    public function logError($message)
    {
        $this->addLog(WCMLOG_ERROR, $message);
    }

    /**
     * Log a warning message
     *
     * @param string $message Message to log
     */
    public function logWarning($message)
    {
        $this->addLog(WCMLOG_WARNING, $message);
    }

    /**
     * Log an standard message
     *
     * @param string $message Message to log
     */
    public function logMessage($message)
    {
        $this->addLog(WCMLOG_MESSAGE, $message);
    }

    /**
     * Logs a database query using logMessage()
     *
     * The only difference between this and logMessage() is that logQuery
     * will increment the totalQueries property of wcmLogger and show that total
     * in the message.
     *
     * @param string $argQuery Query to log
     */
    public function logQuery($argQuery)
    {
        $this->totalQueries++;
        $this->logMessage('{' . $this->totalQueries . '} ' . $argQuery);
    }

    /**
     * Log a verbose message
     *
     * @param string $message Message to log
     */
    public function logVerbose($message)
    {
        if ($this->verbose)
        {
            $this->addLog(WCMLOG_VERBOSE, $message);
        }
    }

    /**
     * Get the string representation of an error level
     *
     * @param int $level error level
     * @return string error level string representation
     */
    private function getErrorLevel($level)
    {
        switch ($level)
        {
            case WCMLOG_ERROR:
                return 'ERR';
            case WCMLOG_WARNING:
                return 'WRN';
            case WCMLOG_MESSAGE:
                return 'MSG';
            case WCMLOG_VERBOSE:
                return 'VRB';
            case WCMLOG_DEBUG:
                return 'DBG';
        }
    }

    /**
     * Dump log content into a filename
     *
     * @param string  $filename Destination file name
     * @param bool $append   Append log content (default false)
     */
    public function dump($filename, $append = false)
    {
        $text = null;
        foreach ($this->messages as $key => $message)
        {
            $t = $message[0];
            $n = $message[1];
            $s = $message[2];

            $text .= '[' . $t . '] ' . $this->getErrorLevel($n) . ': ' . $s . PHP_EOL;
            unset($this->messages[$key]);
        }

        // Save content to file
        saveToFile($filename, $text, $append);
    }

    /**
     * Display log content (echo)
     *
     * @param bool $html Display html rather than raw text (format <li class="???">...</li>)
     * @param bool $return   Return result instead of echo (default false)
     *
     * @return string When $return is true, returns the logger result. Otherwise, returns nothing.
     */
    function display($html = true, $return = false)
    {
        $buffer = null;

        foreach ($this->messages as $message)
        {
            $t = $message[0];
            $n = $message[1];
            $s = $message[2];

            if ($html)
            {
                $buffer .= '<li class="' . $this->getErrorLevel($n) . '">[' . $t . '] ' . $s . '</font></li>';
            }
            else
            {
                $buffer .= '[' . $t . '] ' . $this->getErrorLevel($n) . ': ' . $s . PHP_EOL;
            }
        }

        // Return buffer instead of echoing it ?
        if ($return)
        {
            return $buffer;
        }

        // Echo buffer
        echo $buffer;
    }

    /**
     * Returns the file where log is written
     *
     * @return string File where log is written (or null)
     */
    public function getFile()
    {
        return $this->flushToFile;
    }

    /**
     * Format textual log (e.g. ERR: [datetime] {msg} PHP_EOL ....) in a
     * HTML friendly output (<li class="ERR"> [datetime] {msg} </li> ...)
     *
     * @param string $text Text from logger
     *
     * @return string HTML text
     */
    static function formatTextLog($text)
    {
        $output = null;

        $lines = explode("\n", $text);
        foreach ($lines as $line)
        {
            $line = trim($line);
            if ($line)
            {
                $parts = explode(':', $line, 2);

                // handle special output of error when php.ini is set to output them!
                if ($parts[0] == 'Fatal error')
                {
                    $parts[0] = 'ERR';
                }

                if (count($parts) == 2)
                {
                    $output .= '<li class="' . $parts[0] . '">' . $parts[1] . '</li>' . PHP_EOL;
                }
            }
        }
        unset($lines);

        return $output;
    }
}
