<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.session.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The session class represents a user's session
 */
class wcmSession extends wcmObject
{
    /**
     * Singleton object
     * @var object
     */
    private static $singleton;

    /**
     * Constant used for statistics
     * when user start a new session
     */
    const STAT_START_SESSION = 1;

    /**
     * Constant used for statistics
     * when user navigate into the back-office
     */
    const STAT_NAVIGATE = 2;

    /**
     * Constant used for statistics
     * when user view an object
     */
    const STAT_VIEW_OBJECT = 3;

    /**
     * Constant used for statistics
     * when user save an object
     */
    const STAT_SAVE_OBJECT = 4;

    /**
     * Constant used for statistics
     * when user delete an object
     */
    const STAT_DELETE_OBJECT = 5;

    /**
     * Constant used for statistics
     * when user publish an object
     */
    const STAT_PUBLISH_OBJECT = 6;

    /**
     * Constant used for statistics
     * when user search for something
     */
    const STAT_SEARCH = 7;

    /**
     * Constant used for statistics
     * when user generate some content
     */
    const STAT_GENERATE = 8;

    /**
     * Constant used for statistics
     * when user import some content
     */
    const STAT_IMPORT = 9;


    /**
     * Identifier of session's owner
     * @var int
     */
    public $userId      = null;

    /**
     * Name of session's owner
     * @var string
     */
    public $userName    = null;

    /**
     * When session has started
     * @var string
     */
    public $startDate   = null;

    /**
     * When session has ended (or null)
     * @var string
     */
    public $endDate     = null;

    /**
     * Private members
     * @var wcmUser
     */
    private $user = null;

    /**
     * Is the user admin?
     * @var bool
     */
    private $isAdmin = false;

    /**
     * Site object
     * @var site
     */
    private $site = null;

    /**
     * Language
     * @var string
     */
    private $language;

    /**
     * Current action
     * @var string
     */
    private $currentAction = 'login';

    /**
     * Last ping
     * @var int
     */
    private $lastPing = 0;

    /**
    * Cache of permissions
    * @var array
    */
    private $permissions = array();

    /**
     * Contains the previous REQUEST_URI and QUERY_STRING (if applicable)
     *
     * This gets set by setCurrentAction on several conditions:
     * 1) when the requested action is not 'login' or 'processLogin'
     * 2) when there is no user associated with the session
     *
     * The main purpose of this to allow for bookmarkable links
     *
     * @var string
     */
    private $previousRequest;

    /**
     * Returns the current session
     *
     * @param int $id Optional ID to refresh from DB (default is null)
     *
     * @return wcmSession Current session object
     */
    public static function getInstance($id = null)
    {
        if (!isset(self::$singleton))
        {
            // Enable web session ?
            if (null == session_id())
            {
                $config = wcmConfig::getInstance();
                session_name($config['wcm.project.guid'] . $config['wcm.default.sessionName']);
                session_start();
            }

            // Try to load previous session form $_SESSION
            if ($id === null && isset($_SESSION['wcmSession']))
            {
                self::$singleton = $_SESSION['wcmSession'];
            }
            else
            {
                // Create a new session and persist in $_SESSION
                self::$singleton = new self($id);
                $_SESSION['wcmSession'] = self::$singleton;
            }
        }

        return self::$singleton;
    }

    /**
     * Loads the language files (resources) for a given language.
     *
     * @param string $language The language (default is null to use WCM default language)
     *
     * @return bool True on success, false otherwise
     */
    public function loadLanguageFiles($language = null)
    {
        $config = wcmConfig::getInstance();
        if (null === $language)
        {
            $language = $config['wcm.default.language'];
        }

        /**
         * TODO : CHANGE THIS FUNCTION TO CHECK IF WE WANT TO CACHE IT OR NOT
         */
        if (extension_loaded('apc'))
        {
            $path = sys_get_temp_dir();
            if (apc_load_constants($config['wcm.project.guid'] . '_wcmConstants_' . $language))
            {
                return true;
            }

            /**
             * Right now labels are using DEFINE
             * We load a temporary file containing an array of the defined constants
             */
            $fileName = $path . DIRECTORY_SEPARATOR . $config['wcm.project.guid'] . '_wcmConstants_' . $language . '.php';
            if (file_exists($fileName))
            {
                require_once($fileName);
                unlink($fileName);
                apc_define_constants($config['wcm.project.guid'] . '_wcmConstants_' . $language, $wcmConstants);
                return true;
            }

            $constants = get_defined_constants(true);
            $before = $constants['user'];
        }

        // Note that re-defining existing constant raise a warning
        if (!DEFINED('_WCM_LANGUAGE') || 0 <> strcmp(_WCM_LANGUAGE, $language))
        {
            // Load language resources (i.e. re-define constants)
            // => Retrieve default language and ensure it is supported
            $lgfile = WCM_SYS_API_DIR . '/languages/' . $language . '/main.php';
            if (!file_exists($lgfile))
            {
                return false;
            }

            require($lgfile);
        }

        // Re-defining existing constant raise a warning
        if (!DEFINED('_BIZ_WCM_LANGUAGE') || 0 <> strcmp(_BIZ_WCM_LANGUAGE, $language))
        {
            // Load language resources (i.e. re-define constants)
            // => Retrieve default language and ensure it is supported
            $lgfile = WCM_BIZ_API_DIR . '/common/languages/' . $language . '/main.php';
            if (!file_exists($lgfile))
            {
                return false;
            }

            require($lgfile);
       }

        if (extension_loaded('apc'))
        {
            // Creating a temporary file containing an array of the different
            // constant that were defined in WCM
            $constants = get_defined_constants(true);
            $code  = '<?php ' . PHP_EOL;
            $code .= '$wcmConstants = ' . var_export(array_diff_assoc($constants['user'], $before), true) . ';';

            file_put_contents($path . DIRECTORY_SEPARATOR . $config['wcm.project.guid'] . '_wcmConstants_' . $language . '.php', $code);
        }
        return true;
    }

    /**
     * Constructor
     *
     * @param int $id Optional id (used to refresh object)
     *
     */
    public function __construct($id = null)
    {
        $this->database = wcmProject::getInstance()->database;
        $this->tableName = '#__session';
        $this->objectType = 'wcmSession';
        parent::__construct($id);
    }

    /**
     * Clone method defined as private (for singleton design pattern)
     */
    private function __clone()
    {
    }

    /**
     * Login (open session)
     *
     * @param string $userLogin  User name
     * @param string $password  User password
     *
     * @return bool True if login succeed, false otherwise
     */
    public function login($userLogin, $password)
    {
        $project = wcmProject::getInstance();

        // Search for user
        $query = 'SELECT id, login, password FROM #__user WHERE login=? AND password=?';
        $parameters = array($userLogin, md5($password));

        // Check password validity and retrieve user information
        $id = $project->database->executeScalar($query, $parameters);
        if (null == $id)
        {
            $project->logger->logWarning('wcmSession::login failed: invalid login or password for ' . $userLogin);
            return false;
        }

        return $this->startSession($id);
    }

    /**
     * Login with admistrator password and impersonate another user
     *
     * @param string $userLogin Login of user to impersonate
     * @param string $rootPassword Password of the 'root' user (id=1)
     *
     * @return bool True if login succeed, false otherwise
     */
    public function loginAs($userLogin, $rootPassword)
    {
        $project = wcmProject::getInstance();

        // Check administrator password
        $sql = 'SELECT id FROM #__user WHERE id=? AND password=?';
        $parameters = array(wcmMembership::ROOT_USER_ID, md5($rootPassword));

        // Check root password validity
        $id = $project->database->executeScalar($sql, $parameters);
        if (null == $id)
        {
            $project->logger->logWarning('wcmSession::loginAs failed: invalid root password');
            return false;
        }

        // Search id of userLogin
        $sql = 'SELECT id FROM #__user WHERE login=?';
        $id = $project->database->executeScalar($sql, array($userLogin));
        if (null == $id)
        {
            $project->logger->logWarning('wcmSession::loginAs failed: cannot find impersonated login: ' . $userLogin);
            return false;
        }

        return $this->startSession($id);
    }

    /**
     * Update history
     *
     * When an object changes, the historical record needs to reflect
     * the change.
     *
     * @param wcmObject
     */
    public function updateHistory(wcmObject $object, $info)
    {
        $query = 'UPDATE #__statistics SET objectClass=:objectClass, info=:info WHERE objectClass=:objectClass AND objectId=:objectId AND kind=:kind';
        $this->database->executeStatement(
            $query,
            array(
                'objectClass' => $object->getType(),
                'info' => $info,
                'objectId' => $object->id,
                'kind' => self::STAT_VIEW_OBJECT
            )
        );
    }

    /**
     * Start a new session with a specific user and a specific action
     * Note: this method does not check authentication
     *
     * @param int $userId ID of user's session
     * @param string $startingAction Optional starting action (or 'home' by default)
     * @return bool True on success, false on failure
     */
    public function startSession($userId, $startingAction = 'home')
    {
        $project = wcmProject::getInstance();
        $config = wcmConfig::getInstance();

        // Retrieve user's information
        $this->user = new wcmUser($project, $userId);
        if (!$this->user->id)
        {
            $project->logger->logError('wcmSession::startSession failed: Invalid user ID ' . $userId);
            return false;
        }

        $this->userId = $this->user->id;
        $this->userName = getConst($this->user->name);
        $this->isAdmin = $this->user->isAdministrator;

        // Set start date
        $this->startDate = date('Y-m-d H:i:s');

        // Set default values (language, workingSite, action)
        $this->setCurrentAction($startingAction);
        $this->setSiteId($config['wcm.default.siteId']);

        // Save session in db
        $_SESSION['wcmSession'] = $this;
        if (!$this->store())
        {
            $project->logger->logError('wcmSession::startSession failed: ' . $this->lastErrorMsg);
            return false;
        }

        // Add statistic
        $this->addStat(self::STAT_START_SESSION);
        $project->logger->logMessage('wcmSession::startSession #' . $this->id . ' for user ' . $this->userName);

        return true;
    }

    /**
     * Logout (close session)
     */
    public function logout()
    {
        $project = wcmProject::getInstance();

        // Trace logout information
        $project->logger->logMessage('wcmSession::endSession #' . $this->id . ' for user ' . $this->userName);

        // Set end date and update db
        $this->endDate = date('Y-m-d H:i:s');
        if ($this->userId != null)
        {
            $this->store();
        }

        // Clear session
        session_destroy();

        // Clear resources and set next action to 'login'
        $this->currentAction = 'login';
        $this->userId = 0;
        $this->userName = null;
        $this->user = null;
    }

    /**
     * Ping the session to handle timeout
     */
    public function ping()
    {
        $this->lastPing = time();
    }

    /**
     * Check if session has expired
     *
     * @return bool TRUE if session has expired
     */
    public function hasTimedout()
    {
        $config = wcmConfig::getInstance();

        // If it's set to 0, then there is no timeout
        // If a user is not logged in, then there is no timeout
        if (!$config['wcm.default.sessionTimeout'] || !$this->user)
        {
            return false;
        }

        $ct = time(); // current time

        if (isset($this->user))
        {

            return ($ct >= ($this->lastPing
                            ? $this->lastPing + $config['wcm.default.sessionTimeout']
                            : strtotime($this->startDate) + $config['wcm.default.sessionTimeout']));
        }

        return false;
    }

    /**
     * Set current action
     *
     * @param string $action Current action name
     */
    public function setCurrentAction($action)
    {
        // Don't change action if user is not logged-in
        if ($this->userId)
        {
            // Remove relative path elements
            $action = str_replace(array('../', '/..', './', '/.'), '', $action);

            $this->currentAction = $action;
            $_SESSION['wcmSession'] = $this;
        }
        else
        {
            // Store the request URI if the current action is not login
            if ($action != 'login' && $action != 'processLogin' && 'logout' != $action)
            {
                $config = wcmConfig::getInstance();
                $baseUrl = $config['wcm.backOffice.url'];
                // Parse the query string, see if we can't make a nice looking url!
                if (!empty($_SERVER['QUERY_STRING']))
                {
                    // parse_str() is broken, have to do this manually, le sigh
                    $qsParts = explode('&', $_SERVER['QUERY_STRING']);
                    foreach ($qsParts as $var)
                    {
                        list($name, $value) = explode('=', $var);
                        $qs[$name] = $value;
                    }

                    if (isset($qs['_wcmAction']))
                    {
                        if ($qs['_wcmAction'] == 'business/search')
                        {
                            if (!isset($qs['search_baseQuery']))
                            {
                                $url = 'browseAll/';
                            }
                            else
                            {
                                $url = 'browse/';
                                // Parse base query
                                $bq = explode(':', $qs['search_baseQuery']);
                                $url .= $bq[1] . '/';
                            }

                            if (isset($qs['search_query']))
                            {
                                $url .= $qs['search_query'];
                            }

                            if (isset($qs['_wcmTodo']) && $qs['_wcmTodo'] == 'initSearch')
                            {
                                unset($qs['_wcmTodo']);
                            }

                            if (isset($qs['view']) && $qs['view'] == 'list')
                            {
                                unset($qs['view']);
                            }
                        }
                        else
                        {
                            $url = $qs['_wcmAction'];
                            if (isset($qs['id']))
                            {
                                $url .= '/' . $qs['id'];
                            }
                        }

                        // Unset parameters from the query
                        unset($qs['_wcmAction']);
                        unset($qs['search_query']);
                        unset($qs['id']);
                        unset($qs['search_baseQuery']);

                        $url .= '?' . http_build_query($qs);

                        $this->previousRequest = $baseUrl . $url;
                    }
                    else
                    {
                        // Who knows what happened
                        $this->previousRequest = $baseUrl . '?' . $_SERVER['QUERY_STRING'];
                    }
                }
                else
                {
                    // Nothing happened
                    $this->previousRequest = $baseUrl;
                }
                $_SESSION['wcmSession'] = $this;
            }
        }
    }

    /**
     * Returns the last request
     *
     * @return string Previous request
     */
    public function getPreviousRequest()
    {
        return $this->previousRequest;
    }

    /**
     * Get current action
     *
     * @return string The current action
     */
    public function getCurrentAction()
    {
        return $this->currentAction;
    }

    /**
     * Set current language (and load corresponding ressources)
     *
     * @param string $language Current language name (null means use configured default)
     *
     * @return bool False if the given language is invalid, true otherwise
     */
    public function setLanguage($language = null)
    {
        if ($language === null)
        {
            $config = wcmConfig::getInstance();
            $language = $config['wcm.default.language'];
        }

        $this->language = $language;
        $_SESSION['wcmSession'] = $this;

        return $this->loadLanguageFiles($this->language);
    }

    /**
     * Get current language
     *
     * @return string The current language
     */
    public function getLanguage()
    {
        return $this->language;
    }

    /**
     * Returns the connected user
     *
     * @return wcmUser The connected user
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * Set the working site
     *
     * @param site $site The working site
     */
    public function setSite($site)
    {
        $this->site = $site;
        $_SESSION['wcmSession'] = $this;
        $_SESSION['siteId'] = $site->id;

        // Special operatiom: clear GUI cache
        unset($_SESSION['wcmGUI_renderWorkingSite']);
    }

    /**
     * Set the working site
     *
     * @param int $siteId Id of the working site
     */
    public function setSiteId($siteId)
    {
        $site = wcmObjectFactory::create('site');

        if ($siteId != 0)
        {
            if (!$site->refresh($siteId))
            {
                wcmProject::getInstance()->logger->logError('wcmSession::setSite failed: Invalid site id ' . $siteId);
            }
        }
        $this->setSite($site);
    }

    /**
     * Returns the working site
     *
     * @return site The current working site (or null)
     */
    public function getSite()
    {
        return $this->site;
    }

    /**
     * Returns the id of the current working site
     *
     * @return int ID of the current workfing site (or zero)
     */
    public function getSiteId()
    {
        return ($this->site) ? $this->site->id : 0;
    }

    /**
     * Returns the project instance
     *
     * @return wcmProject The project
     */
    public function getProject()
    {
        return wcmProject::getInstance();
    }

    /**
     * Checks whether current user is administrator
     *
     * @return bool True is current is is an administrator
     */
    public function isAdministrator()
    {
        return $this->isAdmin;
    }

    /**
     * Checks if a given action is allowed for current user (use session cache)
     *
     * @param mixed  $sysobject  sysobject (or permission target) on which to check permssion
     * @param int    $permission permission to check (constant from wcmPermission class)
     *
     * @return bool True if role is granted, false if role is denied
     */
    public function isAllowed($sysobject, $permission)
    {
        if (!$this->userId)
        {
            return false;
        }

        return $permission == ($this->getPermissions($sysobject) & $permission);
    }

    /**
     * Get permission value (and cache result during session)
     *
     * @param object $sysobject  sysobject (or permission target) on which to check permssion
     *
     * @return int permission value
     */
    public function getPermissions($sysobject)
    {
        if ($this->isAdmin)
        {
            return wcmPermission::P_ALL;
        }

        // Retrieve permission target
        $key = (is_string($sysobject)) ? $sysobject : $sysobject->getPermissionTarget();

        // Compute permission and store value in sesion cache?
        if (!isset($this->permissions[$key]))
        {
            // Handle P_NONE as special permission set (deny all)
            $permissions = $sysobject->getUserPermissions($this->userId);
            if (wcmPermission::P_NONE == $permissions)
            {
                $permissions = 0;
            }
            $this->permissions[$key] = $permissions;
            $_SESSION['wcmSession'] = $this;
        }

        return $this->permissions[$key];
    }

    /**
     * Gets an encrypted token representing this session.
     *
     * @return string The encrypted token
     */
    public function getToken()
    {
        // Session token == session ID + session creation time
        $sessionToken = $this->id . '_' . $this->startDate;

        // Encrypt and return the session token
        // TODO : Create adequate settings in configuration.xml for encryption keys
        $wcmEnc = new wcmEncryption(ENCRYPTION_KEY, ENCRYPTION_IV);
        return $wcmEnc->encrypt($sessionToken);
    }

    /**
     * Exposes 'user' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array The session's user getAssocArray() (or null)
     */
    public function getAssoc_user($toXML = false)
    {
        return ($this->user) ? $this->user->getAssocArray($toXML) : null;
    }

    /**
     * Exposes 'history' in the getAssocArray
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array The session's user history as an assoc array
     *               of assoc array (url => (objectClass =>, objectId =>, info =>))
     */
    public function getAssoc_history($toXML = false)
    {
        if ($toXML)
        {
            return null;
        }

        $history = array();

        $rs = $this->getViewHistory(null, 0, 10);
        if ($rs)
        {
            while ($rs->next())
            {
                $row = $rs->current();
                if ($row['linkable'])
                {
                    $url = wcmMVC_Action::computeObjectURL($row['objectClass'], $row['objectId']);
                    $history[$url] = array(
                            'objectClass' => $row['objectClass'],
                            'objectId' => $row['objectId'],
                            'info' => getConst($row['info'])
                            );
                }
            }
            unset($rs);
        }

        return $history;
    }

    /**
     * Adds an entry in the statistics table
     *
     * @param int $kind Kind of event
     * @param object $relatedObject Related object (or null)
     * @param mixed $info Extra info (any serializable object, or null)
     *
     * @return int Number of affected rows (should be 1, or -1 on error)
     */
    public function addStat($kind, $relatedObject = null, $info = null)
    {
        $sql  = 'INSERT INTO #__statistics(sessionId, userId, userName, kind, dateAndTime, objectClass, objectId, info) ';
        $sql .= 'VALUES(?,?,?,?,?,?,?,?)';

        if (null !== $relatedObject)
        {
            $objectClass = get_class($relatedObject);
            $objectId = (property_exists($relatedObject, 'id')) ? $relatedObject->id : null;
        }
        else
        {
            $objectClass = $objectId = '';
        }

        // Prepare info for DB storage
        if ($info !== null && (is_array($info)|| is_object($info)))
        {
            $info = serialize($info);
        }

        $params = array($this->id, $this->userId, $this->userName, $kind, date('Y-m-d H:i:s'),
                $objectClass, $objectId, $info);

        return $this->database->executeStatement($sql, $params);
    }

    /**
     * Returns a database resultset on the statistics table
     *
     * @param string $where Condition to apply on statistics table
     * @param array $params Optional parameters to substitute '?' chars in condition
     * @param int $offset Optional offset to retrieve statistics
     * @param int $limit Optional limit to retrieve statistics
     *
     * @return wcmAbstractResultSet Resultset corresponding to query or null on failure
     */
    public function getStats($where, $params = null, $offset=0, $limit=0)
    {
        $sql = 'SELECT * FROM #__statistics';
        if ($where)
        {
            $sql .= ' WHERE ' . $where;
        }

        return $this->database->executeQuery($sql, $params, $offset, $limit);
    }

    /**
     * Disable a statistic item
     *
     * @param string $argClass Class name
     * @param int $argId Object ID
     */
    public function disableStatItem($argClass, $argId)
    {
        $sql = 'UPDATE #__statistics SET linkable=0 WHERE objectClass=? AND objectId=?';
        $this->database->executeQuery($sql, array($argClass,$argId));
    }

    /**
     * Returns a database resultset representing a view history
     * (list of previously viewed objects for current user)
     *
     * @param string $objectClass Optional object class
     * @param int $offset Optional offset to retrieve statistics
     * @param int $limit Optional limit to retrieve statistics
     *
     * @return wcmAbstractResultSet Resultset corresponding to query or null on failure
     *                   With 'objectClass', 'objectId' and 'info' columns
     */
    public function getViewHistory($objectClass = null, $offset=0, $limit=20)
    {
        $sql = 'SELECT MAX(id) id, objectClass, objectId, info, linkable, dateAndTime FROM #__statistics';
        $sql .= ' WHERE userId=' . $this->userId;
        $sql .= ' AND kind=' . self::STAT_VIEW_OBJECT;
        if ($objectClass)
        {
            $sql .= ' AND objectClass=\'' . $objectClass . '\'';
        }
        $sql .= ' GROUP BY objectClass, objectId';
        $sql .= ' ORDER BY id DESC';

        return $this->database->executeQuery($sql, null, $offset, $limit);
    }

    /**
     * Remove a historic item
     *
     * @param string $objectClass Object Class
     * @param id $objectId Object ID
     */
    public function removeHistoricItem($objectClass, $objectId)
    {
        $sql = 'DELETE FROM #__statistics WHERE objectId=? AND objectClass=?';
        $this->database->executeStatement($sql, array($objectClass, $objectId));
    }

    /**
     * Remove a user History
     *
     * @param string $userId User Id
     */
    public function removeUserHistory($userId)
    {
        $sql = 'DELETE FROM #__statistics WHERE userId=?';
        return $this->database->executeStatement($sql, array($userId));
    }
}
