<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.abstractResultSet.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Abstract ResultSet
 *
 * This class is the foundation for database result sets. Pay
 * close to attention, it implements Iterator but does not define
 * any methods for it. Any extending class will have to implement those
 * methds.
 *
 * The result set wraps around a database statement (created after a query is prepared).
 * Some database abstraction layers (PDO) do not distinguish between a resultSet and a
 * prepared statement, while others do.
 *
 */
abstract class wcmAbstractResultSet implements Iterator
{
    /**
     * PDOStatement instance
     *
     * @var wcmStatement
     */
    protected $statement;

    /**
     * The value of the current row
     *
     * @var mixed
     */
    protected $currentRow;

    /**
     * The number of the current row.
     *
     * The current row number is relative to the total rows in the result set.
     * This means that the first row number will always be 1 and the last row number
     * will be the total number of rows.
     *
     * @var int
     */
    protected $currentRowNumber = 0;

    /**
     * Fetch mode - set using the wcmDatabase::FETCH_* constants
     *
     * @var int
     */
    protected $fetchMode;

    /**
     * Fetch parameters
     *
     * If the fetch mode requires additional parameters, they are stored here.
     *
     * @var mixed
     */
    protected $fetchParam;

    /**
     * Total number of rows
     *
     * @var int
     */
    protected $rows = 0;

    /**
     * Constructor
     *
     * @param wcmStatement $argStatement Statement
     * @param int $argFetchMode Fetch mode taken from wcmDatabase::FETCH_* constants
     * @param mixed $argFetchParam If the fetchMode requires a second parameters
     * @throws Exception When statement hasn't yet been executed
     */
    public function __construct(wcmAbstractStatement $argStatement, $argFetchMode = null, $argFetchParam = null)
    {
        if (!$argStatement->hasBeenExecuted())
        {
            throw new Exception('Resultset cant be built with a statement that has not been executed first');
        }
        $this->statement = $argStatement;
        $this->fetchMode = $argFetchMode;
        $this->fetchParam = $argFetchParam;
    }

    public abstract function count();

    /**
     * Sets the fetch mode for the resultSet
     *
     * @param int   $argFetchMode One of the wcmDatabase::FETCH_* constants
     * @param mixed $argFetchParam For additional parameters required by the fetch mode
     */
    public function setFetchMode($argFetchMode, $argFetchParam = null)
    {
        $this->fetchMode = $argFetchMode;
        $this->fetchParam = $argFetchParam;
    }


}