<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.abstractStatement.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Abstract Prepared Statement
 *
 * This class represents the core of a prepared statement. A prepared statement
 * is a query that has been 'prepared' by the database server, allowing for quicker
 * reuse and placeholders for values.
 *
 */
abstract class wcmAbstractStatement
{
    /**
     * Contains the current query
     *
     * @var string
     */
    protected $query;

    /**
     * Contains the fetch mode the result set will use
     *
     * @var int
     */
    protected $fetchMode;

    /**
     * Contains additional parameters for the fetch mode, if any
     *
     * @var mixed
     */
    protected $fetchParam;

    /**
     * Last generated error message
     *
     * @var string
     */
    protected $errorMsg;

    /**
     * Vendor specific error code
     *
     * @var mixed
     */
    protected $errorCode;

    /**
     * List of parameters for the prepared statement
     *
     * @var array
     */
    protected $parameters = array();

    /**
     * Whether parameters are bound anonymously or with named placeholders
     *
     * @var int
     */
    protected $parameterType = null;

    /**
     * Whether or not this statement has been executed.
     *
     * @var bool
     */
    protected $executed = false;

    /**
     * Logger
     *
     * @var wcmLogger
     */
    protected $logger;

    /**
     * Constructor
     *
     * @param string    $argQuery Query
     * @param wcmLogger $argLogger Logger object
     */
    public function __construct($argQuery, wcmLogger $argLogger = null)
    {
        $this->query = $argQuery;
        $this->logger = (isset($argLogger)) ? $argLogger : wcmProject::getInstance()->logger;
    }

    /**
     * Sets the fetch mode for the resultSet
     *
     * @param int $argMode one of the wcmDatabase::FETCH_* constants
     * @param string $argParam Fetch parameters
     */
    public function setFetchMode($argMode, $argParam = null)
    {
        if ($this->fetchMode == wcmDatabase::FETCH_ASSOC ||
            $this->fetchMode == wcmDatabase::FETCH_CLASS ||
            $this->fetchMode == wcmDatabase::FETCH_COLUMN ||
            $this->fetchMode == wcmDatabase::FETCH_INTO ||
            $this->fetchMode == wcmDatabase::FETCH_NUM ||
            $this->fetchMode == wcmDatabase::FETCH_OBJ)
        {
            $this->fetchMode = $argMode;
        }
        $this->fetchParam = $argParam;
    }

    /**
     * Execute a query
     *
     * Queries return result sets
     *
     * @param array $argParams Parameters for the query
     */
    abstract public function executeQuery(Array $argParams = null);

    /**
     * Exectute a statement
     *
     * Statements do not return result sets
     *
     * @param array $argParams Parameters for the query
     */
    abstract public function executeStatement(Array $argParams = null);

    /**
     * Get the total affected rows.
     *
     * @return int Number of affected rows
     */
    abstract public function rowCount();

    /**
     * Bind value to anonymous placeholder
     *
     * Values in queries can be replaced by either named placeholders
     * or anonymous placeholders.
     *
     * Anonymous placeholders are simply question
     * marks in place of actual values.
     *
     * SELECT id FROM table WHERE username=? AND password=?
     *
     * This means that the first time you call bindValue, you will be binding
     * that value to the first question mark, the second time to the second
     * question mark. For this reason it's generally better to use named placeholders.
     *
     * If you pass an array or an object, it will automatically be serialized
     * first.
     *
     * @see wcmAbstractStatement::bindParam
     * @param mixed $argValue Value to bind
     * @throws Exception When combining anonymous and named parameters
     * @throws Exception When trying to serialized resources
     */
    public function bindValue($argValue)
    {
        if (isset($this->parameterType) && $this->parameterType != self::PARAM_ANON)
        {
            throw new Exception('Cant combine anonymous and named parameters');
        }

        $this->parameterType = self::PARAM_ANON;

        if (!is_scalar($argValue))
        {
            if (is_resource($argValue))
            {
                // WRONG - Resources are not REAL
                throw new Exception('Cant serialized resources');
            }

            $argValue = serialize($argValue);
        }

        $this->parameters[] = $argValue;
    }

    /**
     * Bind value to named placeholder
     *
     * Values in queries can be replaced by either named placeholders
     * or anonymous placeholders.
     *
     * Named placeholders are referenced by string names, making them
     * easier to manage versuses anonymous placeholders.
     *
     * SELECT id FROM table WHERE username=:username AND password=:password
     *
     * If you pass an array or an object, it will be automatically
     * serialized first.
     *
     * @param string $argParamName Parameter name
     * @param mixed $argValue Value
     * @throws Exception When combining anonymous and named parameters
     * @throws Exception When trying to serialized resources
     */
    public function bindParam($argParamName, $argValue)
    {
        if (isset($this->parameterType) && $this->parameterType != self::PARAM_NAMED)
        {
            throw new Exception('Cant combine anonymous and named parameters');
        }

        $this->parameterType = self::PARAM_NAMED;

        if (!is_scalar($argValue))
        {
            if (is_resource($argValue))
            {
                // WRONG - Resources are not REAL
                throw new Exception('Cant serialized resources');
            }

            $argValue = serialize($argValue);
        }

        $this->parameters[$argParamName] = $argValue;
    }

    /**
     * Get last generated error message
     *
     * @return string Error message
     */
    public function getErrorMsg()
    {
        return $this->errorMsg;
    }

    /**
     * Get last vendor-specific error code
     *
     * @return mixed Error code
     */
    public function getErrorCode()
    {
        return $this->errorCode;
    }

    /**
     * Get the current query
     *
     * @return string Current query
     */
    public function getQuery()
    {
        return $this->query;
    }

    /**
     * Cleans up variables that cant be stored properly as is
     *
     * This method will automatically serialize arrays and objects. In order
     * to affect this behaviour, objects can use the __sleep() method to perform
     * any tasks before serialization. __sleep() is expected to return an array
     * containing a list of properties that will be serialized.
     *
     * Upon unserialization, if the object has implemeneted a __wakeup() method, it
     * will be called.
     *
     * @param array $argParams Parameters used in conjunction with '?' occurrences in the sql statement
     * @return array Clean parameters
     *
     */
    protected function cleanParameters(Array &$argParams = null)
    {
        if (is_array($argParams))
        {
            foreach ($argParams as $key => $value)
            {
                if (is_object($value) or is_array($value))
                {
                    $argParams[$key] = serialize($value);
                }

                // Clean up possible problems with boolean and null values
                if ($value === true)
                {
                        $argParams[$key] = 1;
                }
                elseif ($value === false)
                {
                    $argParams[$key] = 0;
                }

                if (empty($argParams[$key]) && ($argParams[$key] !== 0 && $argParams[$key] !== '0'))
                {
                    $argParams[$key] = null;
                }
            }

            return $argParams;
        }

    }

    /**
     * Rreturns true if statement has already been executed
     *
     * @return bool TRUE if statement has already been executed
     */
    public function hasBeenExecuted()
    {
        return $this->executed;
    }

    /**
     * Destructor
     *
     */
    public function __destruct()
    {
        unset($this->query);
    }

    /**
     * This shouldn't be in the session or a cookie or stored anywhere anyways.
     *
     * @return array Empty array
     */
    public function __sleep()
    {
        return array();
    }

}