<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.statement.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Database prepared statement class
 */
class wcmStatement extends wcmAbstractStatement
{
    /**
     * An instance of PDOStatement
     *
     * This is defined in wcmStatement for documentation
     * purposes only. See {@link wcmAbstractStatement::query} for more details.
     *
     * @var PDOStatement
     */
    public $query;

    /**
     * Prepared statement fetch mode
     * @var int
     */
    protected $fetchMode = wcmDatabase::FETCH_ASSOC;

    /**
     * Constructor
     *
     * @param PDOStatement $argQuery Prepared statement
     * @param int $argFetchMode Fetch mode
     * @see wcmDatabase::FETCH_ASSOC
     */
    public function __construct(PDOStatement $argQuery, $argFetchMode = null, $argFetchParam = null, wcmLogger $argLogger = null)
    {
        parent::__construct($argQuery, $argLogger);
        // Set default fetch mode
        $fm = (empty($argFetchMode)) ? wcmDatabase::FETCH_ASSOC : $argFetchMode;
        $this->setFetchMode($fm, $argFetchParam);
    }

    /**
     * Get number of rows from last query
     *
     * @return int Number of rows in result
     */
    public function rowCount()
    {
        return $this->query->rowCount();
    }

    /**
     * Set query fetch mode
     *
     * @param int $argMode Fetch mode to use
     * @param string $argParam Fetch parameters
     */
    public function setFetchMode($argMode, $argParam = null)
    {
        parent::setFetchMode($argMode, $argParam);

        if (empty($argParam))
        {
            $this->query->setFetchMode($argMode);
        }
        else
        {
            $this->query->setFetchMode($argMode, $argParam);
        }
    }

    /**
     * Bind values to a prepared statement
     *
     * @param array $argParams Values to bind, if int bind anomynously, otherwise uses the key name
     */
    protected function bindValuesToQuery(Array $argParams = null)
    {
        // Are we binding with names or anonymously?
        // Grab the first key in the array, if it's an
        // int then we are binding anonymously!

        if (isset($argParams))
        {
            reset($argParams);
            list($key,) = each($argParams);

            if (is_int($key) || $key == '0')
            {
                foreach ($argParams as $key => $value)
                {
                    // Question mark placeholders are 1-index
                    $index = $key + 1;
                    if ($value === 0)
                    {
                        $this->query->bindValue($index, $value, PDO::PARAM_INT);
                    }
                    elseif (is_null($value))
                    {
                        $this->query->bindValue($index, $value, PDO::PARAM_NULL);
                    }
                    else
                    {
                        $this->query->bindValue($index, $value);
                    }
                }
            }
            else
            {
                foreach ($argParams as $name => $value)
                {
                    if ($value === 0)
                    {
                        $this->query->bindValue(':' . $name, $value, PDO::PARAM_INT);
                    }
                    elseif (is_null($value))
                    {
                        $this->query->bindValue(':' . $name, $value, PDO::PARAM_NULL);
                    }
                    else
                    {
                        $this->query->bindValue(':' . $name, $value);
                    }
                }
            }
        }
    }

    /**
     * Execute a query
     *
     * @param array $argParams Extra query parameters
     * @return bool true on success, true on error
     */
    protected function execute(Array $argParams = null)
    {
        $this->cleanParameters($argParams);

        if ($this->query->execute($argParams))
        {
            $this->executed = true;
            return true;
        }
        else
        {
            $err = $this->query->errorInfo();
            if (count($err) > 2)
            {
                $this->errorMsg = $err[2];
            }
            if (count($err) > 1)
            {
                $this->errorCode = $err[1];
            }
        }
        return false;
    }

    /**
     * executeQuery returns a result set
     *
     * @param array $argParams Query parameters
     * @return wcmResultSet Result set object, false on error
     */
    public function executeQuery(Array $argParams = null)
    {
        if ($this->execute($argParams))
        {
            return new wcmResultSet($this, $this->fetchMode, $this->fetchParam);
        }
        echo '<b>Erreur while trying to execute the query : </b>' . $this->query->queryString . '<br />';
        echo '<pre>';
        debug_print_backtrace();
        echo '</pre>';
        die();
    }

    /**
     * Execute a prepared statement
     *
     * @param array $argParams Statement parameters
     * @return bool true on success, false on error
     */
    public function executeStatement(Array $argParams = null)
    {
        return $this->execute($argParams);
    }
}
