<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.smartyFunctions.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class holds a set of Smarty functions
 * They can be called through a generic wcm
 * function. e.g. {wcm name='funcName' param1='' param2=''...}
 * where 'funcName' is a valid method name of this class
 *
 * Note: this class is extended by smartyFunctions in
 * the business/api/smarty directory to allow overwrite and extension
 * for profesionnal services, partners and customers
 */
abstract class wcmSmartyFunctions
{
    /**
     * The path builder used by the generatePath function.
     * @var Nstein_Path_Builder
     */
    private $pathGenerator = null;
    
    /**
     * Get the path builder used by generatePath function. The default value
     * is a Nstein_Path_DefaultBuilder instance using 10 as block size and
     * 10 as base value.
     * @return Nstein_Path_Builder
     */
    public function getPathGenerator()
    {
        if (null === $this->pathGenerator)
        {
            $this->pathGenerator = new Nstein_Path_DefaultBuilder(10, 10);
        }
        return $this->pathGenerator;
    }
    
    /**
     * Set the path builder used by generatePath function.
     * @param Nstein_Path_Builder $builder The path builder.
     * @return void
     */
    public function setPathGenerator(Nstein_Path_Builder $builder)
    {
        $this->pathGenerator = $builder;
    }
    
    /**
     * Returns a photo assocArray corresponding to the first bizobject's photo
     * or null if the bizobject has no photo.
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return string An assoc array representing first photo or null
     */
    public function firstPhoto($params, $smarty)
    {
        $bizobject = getArrayParameter($params, 'bizobject', null);
        $assign = getArrayParameter($params, 'assign', null);

        if (!$bizobject && !is_array($bizobject))
        {
            $smarty->trigger_error('wcm: firstPhoto => invalid or missing "bizobject" parameter');
            return;
        }

        if (!$assign)
        {
            $smarty->trigger_error('wcm: firstPhoto => missing "assign" parameter');
            return;
        }

        $sql = sprintf(
            'sourceClass=\'%s\' AND sourceID=%d AND destinationClass=\'%s\' AND kind=%d',
            $bizobject['className'], 
            $bizobject['id'], 
            'photo', 
            bizrelation::IS_COMPOSED_OF
        );
        
        $photo = null;
        $relation = new bizrelation();
        if ($relation->beginEnum($sql, 'rank', 0, 1))
        {
            if ($relation->nextEnum())
            {
                $photo = wcmObjectFactory::create('photo', $relation->destinationId);
            }
            $relation->endEnum();
        }

        if (null === $photo)
        {
            return;
        }
        
        $result = $photo->getAssocArray(false);
        unset($photo);

        $smarty->assign($assign, $result);
    }

    /**
     * Extract facet values from a search (or from entire corpus)
     * The returned is an array of 'facets' having the following properties
     * - name: facet index name
     * - value: facet indexed value
     * - text: facet literal representation
     * - count: number of matching objects
     *
     * Usage: {wcm  name="facets"
     *              facet="index-name" // name of index
     *              assign="var-name"  // name of variable that will be assigned
     *              sid="search-id"    // optional search id (default:null for the entire corpus)
     *              max="10"           // optional number of values to return (default:10)
     *              sort="0|1"}        // 1 to sort by occurence, 0 to sort by value (default:1)
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     */
    public function facets($params, $smarty)
    {
        // retrieve parameters
        $searchId = getArrayParameter($params, 'sid', null);
        $facet = getArrayParameter($params, 'facet', null);
        $assign = getArrayParameter($params, 'assign', null);
        $max = getArrayParameter($params, 'max', 10);
        $sort = getArrayParameter($params, 'sort', 1);

        if (!$facet)
        {
            $smarty->trigger_error('wcm: facets => missing "facet" parameter');
            return;
        }

        if (!$assign)
        {
            $smarty->trigger_error('wcm: facets => missing "assign" parameter');
            return;
        }

        // extract facet values
        $result = array();
        $search = wcmSearch::getInstance();
        $values = $search->getFacetValues(array($facet => $max), $searchId, 0, $sort);
        if ($values)
        {
            foreach ($values as $value => $facet)
            {
                // display on facets with positive count
                if ($facet->count > 0)
                {
                    $result[] = array(
                                    'name' => $facet->name,
                                    'value' => $facet->value,
                                    'text' => $facet->text,
                                    'count' => $facet->count);
                }
            }
        }

        // Assign result
        $smarty->assign($assign, $result);
    }

    /**
     * Returns an array of bizrelation associated to a bizobject
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     */
    public function relations($params, $smarty)
    {
        $bizobject = getArrayParameter($params, 'bizobject', null);
        $assign = getArrayParameter($params, 'assign', null);
        $kind = intval(getArrayParameter($params, 'kind', bizrelation::IS_RELATED_TO));

        if (!$bizobject && !is_array($bizobject))
        {
            $smarty->trigger_error('wcm: relations => invalid or missing "bizobject" parameter');
            return;
        }

        if (!$assign)
        {
            $smarty->trigger_error('wcm: relations => missing "assign" parameter');
            return;
        }

        if (!$kind)
        {
            $kind = bizrelation::IS_RELATED_TO;
            return;
        }

        $relations = array();


        $where = 'sourceClass=\'' . $bizobject['className'] . '\' AND sourceId=' . $bizobject['id'] . ' AND kind=' . $kind;
        $enum = new wcmEnumeration('bizrelation', $where, 'rank');
                
        foreach ($enum as $temp)
        {
            $relations[] = $temp->getAssocArray(false);
        }

        // Assign result
        $smarty->assign($assign, $relations);
    }
    
    /**
     * Return the backoffice / frontend URL BizPhoto by mode
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return the URL of a bizPhoto or null on failure
     */
    public function photoUrl($params, $smarty)
    {
        $bizobject = getArrayParameter($params, 'bizobject', null);
        $assign = getArrayParameter($params, 'assign', null);
        
        if (!$bizobject && !is_array($bizobject))
        {
            $smarty->trigger_error('wcm: photoUrl => invalid or missing "bizobject" parameter');
            return null;
        }
        
        if (is_object($bizobject) && $bizobject instanceof photo)
        {
            $photo = $bizobject;
        }
        else
        {
            $photo = $bizobject['this'];
            if (!$photo || !(is_object($photo) && $photo instanceof photo))
            {
                $smarty->trigger_error('wcm: photoUrl => invalid or missing "bizobject" parameter');
                return null;
            }
        }
       
        if (wcmTemplateGenerator::MODE_PREVIEW === $smarty->mode)
        {
            // We're in preview mode, so use the backOffice image.
            $url = $this->buildBackOfficePhotoUrl($photo, $params, $smarty);
        }
        else
        {
            $url = $this->buildFrontEndPhotoUrl($photo, $params, $smarty);
        }

        if ($assign)
        {
            $smarty->assign($assign, $url);
        }
        
        return $url;
    }
    
    /**
     * Return the backoffice or frontend URL of a bizObject
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return the URL of a bizObject or null on failure
     */
    public function url($params, $smarty)
    {
        $bizobject = getArrayParameter($params, 'bizobject', null);
        $assign = getArrayParameter($params, 'assign', null);

        if (!$bizobject && !is_array($bizobject))
        {
            $smarty->trigger_error('wcm: url => invalid or missing "bizobject" parameter');
            return null;
        }
        
        $previewMode = (wcmTemplateGenerator::MODE_PREVIEW === $smarty->mode);
        $urlGenerator = new objectUrlGenerator();
        $url = $urlGenerator->generateUrl($bizobject['this'], null, $previewMode);

        if (null === $url)
        {
            $smarty->trigger_error('wcm: url => cannot generate URL: ' . $msg);
            return null;
        }

        if ($assign)
        {
            $smarty->assign($assign, $url);
        }
        
        return $url;
    }
    
    /**
     * Return the backoffice URL of a bizPhoto
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return the backoffice URL of a bizObject or null on failure
     */
    public function backOfficePhotoUrl($params, $smarty)
    {
        $bizobject = getArrayParameter($params, 'bizobject', null);
        $assign = getArrayParameter($params, 'assign', null);

        if (!$bizobject && !is_array($bizobject))
        {
            $smarty->trigger_error('wcm: backOfficePhotoUrl => invalid or missing "bizobject" parameter');
            return null;
        }
        
        if (is_object($bizobject) && $bizobject instanceof photo)
        {
            $photo = $bizobject;
        }
        else
        {
            $photo = $bizobject['this'];
            if (!$photo || !(is_object($photo) && $photo instanceof photo))
            {
                $smarty->trigger_error('wcm: photoUrl => invalid or missing "bizobject" parameter');
                return null;
            }
        }
        
        $url = $this->buildBackOfficePhotoUrl($photo, $params, $smarty, $bizobject);

        if ($assign)
        {
            $smarty->assign($assign, $url);
        }
        
        return $url;
    }
    
    /**
     * Build the backoffice URL of a bizPhoto
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return the backoffice URL of a bizObject or null on failure
     */
    private function buildBackOfficePhotoUrl(photo $photo, $params, $smarty)
    {
        $format = getArrayParameter($params, 'format');
        $width = getArrayParameter($params, 'width');
        $height = getArrayParameter($params, 'height');
                
        $factory = new photoManagerFactory();
        $photoManager = $factory->createDefaultPhotoManager();
        
        if ($format)
        {
            return $photoManager->getCroppedImageUrl($photo, $format);
        }
        elseif ($width || $height)
        {
            return $photoManager->getResizedImageUrl($photo, $width, $height);
        }
        else
        {
            return $photoManager->getImageUrl($photo);
        }
    }
    
    /**
     * Build the frontend URL of a bizPhoto
     *
     * @param array $params Assoc array of parameters
     * @param Smarty $smarty Instance of smarty
     *
     * @return the backoffice URL of a bizObject or null on failure
     */
    private function buildFrontEndPhotoUrl(photo $photo, $params, $smarty)
    {
        $format = getArrayParameter($params, 'format');
        $width = getArrayParameter($params, 'width');
        $height = getArrayParameter($params, 'height');
        
        $site = $photo->getSite();
        if (null === $site)
        {
            $smarty->trigger_error('wcm: photoUrl => invalid or missing site id');
            return null;
        }
        
        $client = new Nstein_Cmf_Mediamanager_Client(
            $site->getConfig('media-manager.rest-url')
        );
        
        if ($format)
        {
            return $client->getCroppedImageUrl($photo->id, $format);
        }
        elseif ($width || $height)
        {
            return $client->getResizedImageUrl($photo->id, $width, $height);
        }
        else
        {
            return $client->getImageUrl($photo->id);
        }
    }

    /**
     * Renders a given object with wcmGUI::renderObject().
     *
     * Specific parameters:
     *
     *     $params['object']          => (wcmSysobject) The object to render
     *     $params['template']        => (string) The Smarty template to use
     *     $params['default_template' => (string) The default template (if $template does not exist)
     *
     * Any other parameters are passed as-is to wcmGUI::renderObject().
     *
     * @param array  $params Assoc array of parameters
     * @param Smarty $smarty The Smarty instance
     *
     * @return string The rendered object or null on failure
     */
    function render_object($params, $smarty)
    {
        $config = wcmConfig::getInstance();
        // Validate specific parameters
        $msg = 'wcm: render_object => missing or invalid "%s" parameter';

        if (!isset($params['object']) || !is_object($object = $params['object']))
        {
            $smarty->trigger_error(sprintf($msg, 'object'));
            return null;
        }
        unset($params['object']);

        if (!isset($params['template']) || !($template = $params['template']))
        {
            $smarty->trigger_error(sprintf($msg, 'template'));
            return null;
        }
        unset($params['template']);

        if (!isset($params['default_template']) || !($default_template = $params['default_template']))
        {
            $smarty->trigger_error(sprintf($msg, 'default_template'));
            return null;
        }
        unset($params['default_template']);

        // Use default template if $template does not exist
        if (!file_exists($config['wcm.templates.path'] . $template))
        {
            $template = $default_template;
        }

        // Render the object
        return wcmGUI::renderObject($object, $template, $params);
    }

    /**
     * Includes a template with a fallback option
     *
     *
     * Specific parameters:
     *     $params['file']     => (string) Template to include
     *     $params['fallback'] => (string) Fallback template is file does not exists
     *
     * @param array  $params Assoc array of parameters
     * @param Smarty $smarty The Smarty instance
     *
     * @return string The rendered object
     */
    function include_template($params, $smarty)
    {
        // Is this a relative or absolute path?
        if ($params['file']{0} == '/')
        {
            $tpl = $params['file'];
        }
        else
        {
            $tpl = $smarty->template_dir . '/' . $params['file'];
        }

        if (file_exists($tpl))
        {
            return $smarty->fetch($tpl);
        }
        else
        {
            if ($params['fallback']{0} == '/')
            {
                $tpl = $params['fallback'];
            }
            else
            {
                $tpl = $smarty->template_dir . '/' . $params['fallback'];
            }
            return $smarty->fetch($tpl);
        }
    }
    
    /**
     * Generate a relative path from a given id.
     * Parameters are:
     *      $params['id'] => The id for which a path must be generated (required).
     *      $params['assign'] => The name of the Smarty variable where the result must be assigned
     *                           (optional). If this value is not specified, the generated path will
     *                           be returned.
     * @param $params
     * @param $smarty
     * @return unknown_type
     */
    function generatePath($params, $smarty)
    {
        $id = getArrayParameter($params, 'id');
        if (!$id)
        {
            $smarty->trigger_error('Argument id is required.');
            return null;
        }
        
        $assign = getArrayParameter($params, 'assign');
        
        $path = $this->getPathGenerator()->generatePath($id);
        if ($assign)
        {
            $smarty->assign($assign, $path);
        }
        else
        {
            return $path;
        }
    }
}
