<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.template.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The template class represents a generic PnW template
 * => A template may belongs to a {@link wcmTemplateCategory}
 */
class wcmTemplate extends wcmSysobject
{
    /**
     * Template old ID
     * @var int
     */
    private $oldId;

    /**
     * Template name (filename without extension)
     * @var string
     */
    public $name = null;

    /**
     * Template content
     * @var string
     */
    public $content = '';

    /**
     * Template category ID
     * @var int
     */
    public $categoryId = null;

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();
        $this->oldId = $this->id = null;
    }

    /**
     * Check if the template is obsolete
     * @return bool TRUE if obsolete, false if not
     */
    public function isObsolete()
    {
        return false;
    }

    /**
     * Computes the target for a permission
     *
     * @return string the target for a specific permission
     */
    public function getPermissionTarget()
    {
        return $this->getMasterClass()->getPermissionTarget();
    }

    /**
     * Constructor
     * @param wcmProject $project Project object
     * @param int $id Template ID
     */
    public function __construct($project = null, $id = null)
    {
        // Set default values
        $this->setDefaultValues();

        // Refresh object?
        if ($id)
        {
            $this->refresh($id);
        }
    }

    /**
     * Load or refresh object content
     *
     * @param int $id optional argument, if not specifed current id is used
     *
     * @return true on success, false otherwise
     */
    public function refresh($id = null)
    {
        if (!isset($id) && isset($this->id))
        {
            $id = $this->id;
        }

        if (!empty($id))
        {
            $this->oldId = $id;
            $this->id = $id;

            $parts = explode('/', $this->id);

            $this->name = array_pop($parts);
            $this->name = substr($this->name, 0, strlen($this->name) - 4);

            $this->categoryId = implode('/', $parts);
            if ($this->categoryId)
            {
                $this->categoryId .= '/';
            }

            $this->content = $this->getFileContent();
            $this->modifiedAt = date('Y-m-d, H:i:s', $this->getFileModifiedAt());
        }

        return true;
    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Bind and fix (new) id
        if (!$this->bind($source))
        {
            return false;
        }
        
        if ($this->checkValidity())
        {
            
            $category = (substr($this->categoryId, -1) != '/') ? $this->categoryId . '/' : $this->categoryId;
            $category = (substr($category, 0, 1) == '/') ? (substr($category, 1)) : $category;
            $name = (substr($this->name, -4, 4) != '.tpl') ? $this->name . '.tpl' : $this->name;
            $this->id = $category . $name;

            //Remove previous file?
            if ((!empty($this->oldId)) && ($this->oldId != $this->id))
            {
                // Delete old file
                if (false === $this->delete())
                {
                    return false;
                }
            }

            $this->setFileContent();
            $this->index();

            $this->oldId = $this->id;
            return true;
        }

        return false;
    }
    /**
     * Returns the template category
     *
     * @return int category ID
     */
    public function getCategory()
    {
        return $this->categoryId;
    }

    /**
     * Returns the location on disk of the template
     *
     * @return string Location on disk of current template
     */
    public function getOldFilename()
    {
        $config = wcmConfig::getInstance();
        return ($config['wcm.templates.path'] . $this->oldId);
    }

    /**
     * Returns the location on disk of the template
     *
     * @return string Location on disk of current template
     */
    public function getFilename()
    {
        if (isAbsolutePath($this->id))
        {
            return $this->id;
        }
        else
        {
            $config = wcmConfig::getInstance();
            return ($config['wcm.templates.path'] . $this->id);
        }
    }

    /**
     * Returns the content on disk of the template
     *
     * @return string Template content saved on disk or null
     */
    private function getFileContent()
    {
        $filename = $this->getFilename();

        $content = null;
        readFromFile($filename, $content);
        return $content;
    }

    /**
     * Save template on disk
     */
    private function setFileContent()
    {
        saveToFile($this->getFileName(), $this->content);
    }

    /**
     * Returns the last date of modification of the template file
     *
     * @return string The last date of modification of the template file or null if not found
     */
    private function getFileModifiedAt()
    {
        $filename = $this->getFilename();
        if (!is_file($filename))
        {
            return null;
        }

        return filemtime($this->getFileName());
    }

    /**
     * Deletes the object from database.
     *
     * @return bool True on success, false on failure
     *
     */
    public function delete()
    {
        if (!$this->hasLock() && !$this->checkLock('delete'))
        {
            $this->lastErrorMsg = $this->getType() . '::delete failed : object ' . $this->id . ' is locked';
            return false;
        }

        if (!$this->oldId)
        {
            return false;
        }

        try
        {
            removeFile($this->getOldFilename());
        }
        catch (Exception $ex)
        {
            return false;
        }

        return true;
    }

    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid, false otherwise
     *
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        // Test for improper characters in category name (directory) and name (file)

        if (preg_match('/[^\w\/]+/', $this->categoryId))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TPLCATEGORY_INVALID . ' ' . _BIZ_ERROR_INVALID_FILE_CHARS;
            return false;
        }

        if (preg_match('/[^\w\/]+/', $this->name))
        {
            $this->lastErrorMsg = _BIZ_ERROR_TPLNAME_INVALID . ' ' . _BIZ_ERROR_INVALID_FILE_CHARS;
            return false;
        }

        if (strlen($this->categoryId) > 90)
        {
            $this->lastErrorMsg = _BIZ_ERROR_TPLCATEGORY_TOO_LONG;
            return false;
        }

        if (strlen($this->name) > 31)
        {
            $this->lastErrorMsg = _BIZ_ERROR_NAME_TOO_LONG;
            return false;
        }
        
        return true;
    }
}
