<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.templateCategory.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The template class represents a generic PnW template
 * => A template may belongs to a {@link wcmTemplateCategory}
 */
class wcmTemplateCategory extends wcmSysobject
{
    /**
     * Old template ID
     * @var int
     */
    private $oldId;

    /**
     * Category name (Dirname without extension)
     * @var string
     */
    public $name = null;


    /**
     * Parent category (relative file path)
     * @var int
     */
    public $categoryId = null;

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        parent::setDefaultValues();
        $this->oldId = $this->id = null;
    }

    /**
     * Computes the target for a permission
     *
     * @return string the target for a specific permission
     */
    public function getPermissionTarget()
    {
        return $this->getMasterClass()->getPermissionTarget();
    }

    /**
     * Load or refresh object content
     *
     * @param int $id optional argument, if not specifed current id is used
     *
     * @return true on success, false otherwise
     *
     */
    public function refresh($id = null)
    {
        if (!isset($id) && isset($this->id))
        {

            $id = $this->id;
        }

        if (!empty($id))
        {
            $this->oldId = $id;
            $this->id = $id;

            $parts = explode('/', $this->id);

            $this->name = array_pop($parts);

            $this->categoryId = implode('/', $parts);
            if ($this->categoryId)
            {
                $this->categoryId .= '/';
            }
        }

        return true;
    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Bind and fix (new) id
        if (!$this->bind($source))
        {
            return false;
        }

        if ($this->name{strlen($this->name)-1} != '/')
        {
            $this->name . '/';
        }

        if (!empty($this->categoryid))
        {
            if ($this->categoryId{strlen($this->categoryId)-1} != '/')
            {
                $this->categoryId . '/';
            }
        }
        else
        {
            $this->categoryId = '/';
        }

        $this->id = $this->categoryId . $this->name;
        $config = wcmConfig::getInstance();

        if (!$this->oldId)
        {
            mkdir($config['wcm.templates.path'] . $this->id);
            chmod($config['wcm.templates.path'] . $this->id, 0777);
        }
        elseif ($this->oldId != $this->id)
        {
            if (file_exists($this->categoryId))
            {
                if (false === rename($this->getOldDirname(), $this->getDirname()))
                {
                    $this->lastErrorMsg = _DIRECTORY_NOT_EXIST . ': ' . $this->categoryId;
                    return false;
                }
            }
            else
            {
                $this->lastErrorMsg = _DIRECTORY_NOT_EXIST . ': ' . $this->categoryId;
                return false;
            }
        }

        $this->oldId = $this->id;
        return true;
    }
    /**
     * Returns the template category
     *
     * @return int category ID
     */
    public function getCategory()
    {
        return $this->categoryId;
    }

    /**
     * Returns the location on disk of the template
     *
     * @return string Location on disk of current template
     */
    public function getOldDirname()
    {
        $config = wcmConfig::getInstance();
        return ($config['wcm.templates.path'] . $this->oldId);
    }

    /**
     * Returns the location on disk of the template
     *
     * @return string Location on disk of current template
     */
    public function getDirname()
    {
        $config = wcmConfig::getInstance();
        return ($config['wcm.templates.path'] . $this->id);
    }

    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid
     *
     */
    public function checkValidity()
    {
        if (!parent::checkValidity())
        {
            return false;
        }

        if (strlen($this->name) > 31)
        {
            $this->lastErrorMsg = _BIZ_ERROR_NAME_TOO_LONG;
            return false;
        }

        return true;
    }

    /**
     * Deletes the object from database.
     *
     * @return bool True on success, false on failure
     *
     */
    public function delete()
    {
        if (!$this->oldId)
        {
            return false;
        }

        try
        {
            if (!rmdir($this->getOldDirname()))
            {
                $this->lastErrorMsg = _CANNOT_DELETE_TEMPLATE_CATEGORY;
                return false;
            }
        }
        catch (Exception $ex)
        {
            return false;
        }

        return true;
    }

    /**
     * Returns all sub-categories of this category
     *
     * @return array Array of sub-categories
     *
     */

    public function getSubCategories()
    {
        $subdirs = array();
        $dir = new DirectoryIterator($this->getDirname());
        foreach ($dir as $resource)
        {
            if ($resource->isDir() && $resource->getFilename() != '.' && $resource->getFilename() != '..' && $resource->getFilename() != '.svn')
            {
                $subdirs[] = $resource->getFilename();
            }
        }

        return $subdirs;

    }
}
