<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.GUI.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * This class contains helper to render
 * the graphical user interface
 */
class wcmGUI extends wcmFormGUI
{
    /**
     * Renders the typical form for a wcmObject, including some
     * hidden fields (class, id, todo) and mainActionController
     *
     * @param wcmObject $wcmObject Related wcmObject
     * @param array $attributes Attributes array
     */
    static function openObjectForm($wcmObject, $attributes = null)
    {
        if (!is_array($attributes))
        {
            $attributes = array();
        }

        $attributes['class'] = 'mainForm';
        echo '<script type="text/javascript">var wcmActionController  = new WCM.ActionController();</script>';

        self::openForm(
            'mainForm',
            wcmMVC_Action::computeURL(null, null, array('id' => $wcmObject->id)),
            array('_wcmClass' => $wcmObject->getType(), 'id' => $wcmObject->id, '_wcmTodo' => ($wcmObject->isLockOptimistic()) ? 'save' : 'checkin'),
            $attributes
        );
    }

    /**
     * Renders an object given its class and ID by calling
     * wcmGUI::renderObject().
     *
     * @param string     $objectClass The class of the object to render
     * @param int        $objectId    The ID Of the object to render
     * @param string     $xsl    The template, XSL, etc. to use
     * @param array|null $xslParams      Assoc. array of contextual parameters
     *
     * @return string|null The result of rendering the object, or null on error
     */
    public static function renderObjectById($objectClass, $objectId, $xsl = null, $xslParams = null)
    {
        if (!$objectClass || !is_subclass_of($objectClass, 'wcmSysObject'))
        {
            return null;
        }

        $object = wcmObjectFactory::create($objectClass, $objectId);
        return self::renderObject($object, $template, $params);
    }

    /**
     * Renders an object using a template, an XSL, etc.
     *
     * Determines the nature of $template from the file name extension:
     *
     *     .php => PHP script to include
     *     .tpl => Smarty template to execute
     *     .xsl => XSL stylesheet to process
     *
     * If $template is relative, the base directory corresponding to
     * the nature of $template is prepended:
     *
     *     .php => WCM_DIR
     *     .tpl => wcmConfig['wcm.templates.path']
     *     .xsl => {business/}xsl (depending on object type)
     *
     * @param wcmSysObject $object   The object to render
     * @param string       $template The template, XSL, etc. to use
     * @param array        $params   Assoc. array of contextual parameters
     *
     * @return string The result of rendering the object or null on error
     */
    public static function renderObject(wcmSysObject $object, $template, array $params = array())
    {
        $config = wcmConfig::getInstance();
        $project = wcmProject::getInstance();

        $params['object'] = $object;

        if (!isset($params['callback']))
        {
            $params['callback'] = 'onSelectItem';
        }

        if ($object->createdBy && !isset($params['createdByUserName']))
        {
            $user = new wcmUser($object);
            if ($user->refresh($object->createdBy))
            {
                $params['createdByUserName'] = $user->name;
            }
        }

        if ($object->modifiedBy && !isset($params['modifiedByUserName']))
        {
            $user = new wcmUser($object);
            if ($user->refresh($object->modifiedBy))
            {
                $params['modifiedByUserName'] = $user->name;
            }
        }

        if ($object->workflowState && !isset($params['workflowState']))
        {
            $workflowState = $project->workflowManager->getWorkflowStateByCode($object->workflowState);
            if ($workflowState)
            {
                $params['workflowState'] = $workflowState->name;
            }
        }

        // get available transitions
        if (!isset($params['transitions']))
        {
            $transitions = array();
            foreach ($object->getAvailableTransitions() as $transition)
            {
                $transitions[] = array('id' => $transition->id, 'name' => getConst($transition->name));
            }

            $params['transitions'] = $transitions;
        }

        if (!isset($params['locked']))
        {
            if (method_exists($object, 'isLocked') && $object->isLocked())
            {
                $locked = 'TRUE';
            }
            elseif (!method_exists($object, 'getLockInfo'))
            {
                $locked = 'FALSE';
            }
            elseif ($object->getLockInfo()->userId == wcmSession::getInstance()->userId)
            {
                $locked = 'ME';
            }
            else
            {
                $locked = 'FALSE';
            }
            $params['locked'] = $locked;
        }

        if (!isset($params['lockedBy']))
        {
            $lockedBy = $object->getLockInfo()->userId;
            $params['lockedBy'] = ($lockedBy) ? $object->getProject()->membership->getUserById($lockedBy)->name : '';
        }

        $extension = pathinfo($template, PATHINFO_EXTENSION);
        switch ($extension)
        {
            case '':
            case 'php':
                if ($template[0] != '/')
                {
                    $template = WCM_DIR . $template;
                }
                extract($params);
                ob_start();
                require($template);
                $html = ob_get_clean();
                break;

            case 'tpl':
                if (!ereg('^(\\\\|[a-zA-Z]:/|/)', $template))
                {
                    $template = $config['wcm.templates.path'] . $template;
                }

                $templateGenerator = new wcmTemplateGenerator($project);
                $templateObj = new wcmTemplate();
                $templateObj->refresh($template);
                $html = $templateGenerator->executeTemplate($templateObj, $params);
                break;

            case 'xsl':
                if ($template[0] != '/')
                {
                    $isBizobject = is_subclass_of($object, 'wcmBizobject');
                    $template = WCM_DIR . ($isBizobject ? '/business' : '') . '/xsl/' . $template;
                }

                $xmlDoc = new DOMDocument();
                if (!$xmlDoc->loadXML($object->toXML()))
                {
                    $msg = 'renderObject: invalid XML representation for object %s(%d)';
                    $project->logger->logError(sprintf($msg, $object->getType(), $object->id));
                    return null;
                }

                $xslDoc = new DOMDocument();
                if (!$xslDoc->load($template))
                {
                    $msg = 'renderObject: invalid XSL stylesheet "%s"';
                    $project->logger->logError(sprintf($msg, $template));
                    return null;
                }

                $xsltProc = new XSLTProcessor;
                $xsltProc->importStyleSheet($xslDoc);
                $xsltProc->registerPHPFunctions();

                if ($params)
                {
                    foreach ($params as $name => $value)
                    {
                        $xsltProc->setParameter('', $name, $value);
                    }
                }
                $html = $xsltProc->transformToXML($xmlDoc);
                break;

            default:
                $msg = 'renderObject: unsupported template type "%s"';
                $project->logger->logError(sprintf($msg, strtoupper($extension)));
                $html = null;
                break;
        }

        return $html;
    }

    /**
     * Renders an object as a search result given its class and ID.
     *
     * @param string      $objectClass   The class of the object to render
     * @param int         $objectId      The ID of the object to render
     * @param string|null $xmlParameters XML-encoded parameters if any
     *
     * @return string The result of rendering the object or null on error
     */
    public static function renderSearchResultObject($objectClass, $objectId, $xmlParameters = null)
    {
        $xsl = null;

        $parameters = wcmXML::xmlToArray($xmlParameters);

        if (isset($parameters['viewName']))
        {
            $isBizobject = is_subclass_of($objectClass, 'wcmBizobject');
            $xslPrefix = WCM_WWW_BACKOFFICE_DIR . ($isBizobject ? '/business' : '/system') . '/private/xsl/';
            $xslSuffix = '.xsl';

            $config = wcmConfig::getInstance();
            $configFile = $config['wcm.plugins.search.config'];
            $searchConfig = new wcmSearchConfig($configFile);
            $view = $searchConfig->getView(
                $parameters['configId'],
                $parameters['pageType'],
                $parameters['viewName']
            );

            if ($view)
            {
                $xsls = $view->xpath('bizObjects/bizObject[@name="' . $objectClass . '"]/xsl');
                if ($xsls)
                {
                    $xsl = array_shift($xsls);
                    if ($xsl)
                    {
                        if (!file_exists($xslPrefix . $xsl . $xslSuffix))
                        {
                            $xsl = null;
                        }
                    }
                }

                if (!$xsl)
                {
                    $xsls = $view->xpath('bizObjects/bizObject[@name="default"]/xsl');
                    if ($xsls)
                    {
                        $xsl = array_shift($xsls);
                        if ($xsl)
                        {
                            if (!file_exists($xslPrefix . $xsl . $xslSuffix))
                            {
                                $xsl = null;
                            }
                        }
                    }
                }
            }
        }

        if (!$xsl)
        {
            $xsl = $objectClass;
            $parameters['viewName'] = $xsl;
        }

        $xslParams = array();
        $xslParams['xmlParameters'] = wcmXML::arrayToxml($parameters);

        return self::renderObjectById($objectClass, $objectId, $xsl, $xslParams);
    }

    /**
     * Returns DHTML string representing a quick search box
     *
     * @param string $filter Optional filter to apply to the query
     * @param string $xsl Optional xsl code (default 'quickSearch')
     * @param int $pageSize Optional page size (default 10)
     * @param string $uid Optiona unique id to save info in session (default 'quickSearch')
     * @param int $start_search if set, the search is started automatically on page load
     * @param string $defaultOrderBy the default orderBy for the quick search
     * @param array $sortFields array of sorted fields name and associate label (default createdAt and sort_title)
     *
     * Note: this real XSL will be in /business/xsl/$xsl.xsl
     *
     * @return string DHTML
     */
    static function renderQuickSearchBox($filter = null, $xsl = 'quickSearch', $pageSize = 10, $uid = 'quickSearch', $start_search = null, $defaultOrderBy = 'createdAt DESC', $sortFields = array('createdAt' => _BIZ_DATE, 'title_sort' => _TITLE), $engine = null)
    {
        ob_start();
        ob_implicit_flush(false);

        $currentPage = 1;
        $query = '';
        $html = '';

        //if search engine was not specified, select default one
        if (!isset($engine))
        {
            $config = wcmConfig::getInstance();
            $engine = $config['wcm.search.engine'];
        }

        //if the last quick search is set for this filter, load it by default
        if (isset($filter) && isset($_SESSION['lastQuickSearch'][$filter]))
        {
            $pageSize = $_SESSION['lastQuickSearch'][$filter]['pageSize'];
            $defaultOrderBy = $_SESSION['lastQuickSearch'][$filter]['orderBy'];
            $query = $_SESSION['lastQuickSearch'][$filter]['query'];
            $html = $_SESSION['lastQuickSearch'][$filter]['html'];
            $start_search = false;
        }
        ?>
        <script type="text/javascript">

        orderByField = '';
        orderByDirection = '';

        <?php
        // load the default OrderBy parameter in the javascript
        $arrayDefaultOrderBy = array();
        $arrayDefaultOrderBy  = explode(' ', $defaultOrderBy);
        echo 'orderByField=\'' . $arrayDefaultOrderBy[0] . '\';';

        if (isset($arrayDefaultOrderBy[1]))
        {
            echo 'orderByDirection=\'' . $arrayDefaultOrderBy[1] . '\';';
        }
        else
        {
            echo 'orderByDirection=\'ASC\';';
            $arrayDefaultOrderBy[1] = 'ASC';
        }
        ?>

        // set the options before the search
        relationSearch.params.set('quickSearch',{
            useLinkManager: false,
            filter: '<?php echo $filter; ?>',
            pageSize: '<?php echo $pageSize; ?>',
            style: 'quickSearch',
            currentPage: <?php echo $currentPage; ?>,
            resultSet: '_wcm_qs_resultSet',
            ajaxHandler: 'search/quickSearch',
            uid: 'quickSearch',
            orderBy: '<?php echo $defaultOrderBy; ?>',
            query: '<?php echo $query; ?>',
            engine: '<?php echo $engine; ?>'
        });

        leftHand_quickSearch = function()
        {
            relationSearch.search('quickSearch',$('_wcm_qs_query').value, {});

            // show the order by filter
            $('sortQuickSearch').show();
            $('_wcm_qs_resultSet').innerHTML = '<span class="wait">' + $I18N.LOADING + '</span>';
        }

        // Apply the sorted filters
        quickSearchOrderBy = function(argOrderByField)
        {
            // reset the classname
            $('quickSearchOrderBy' + orderByField).removeClassName('sort' + orderByDirection);

            if (orderByField == argOrderByField)
            {
                if (orderByDirection == 'ASC')
                {
                    orderByDirection = 'DESC';
                }
                else
                {
                    orderByDirection = 'ASC';
                }
            }
            else
            {
                // add the default classname on the old filter
                $('quickSearchOrderBy' + orderByField).addClassName('sort');
                orderByField = argOrderByField;
                orderByDirection = 'ASC';
            }

            // add the new classname for the current filter
            $('quickSearchOrderBy' + orderByField).addClassName('sort' + orderByDirection);
            relationSearch.orderBy('<?php echo $uid; ?>', argOrderByField + " " + orderByDirection);
        }

        <?php
        if ($start_search)
        {
            echo 'window.onload = leftHand_quickSearch;';
        }
        ?>
        </script>

        <div id="quickSearch">
            <form onsubmit="leftHand_quickSearch(); return false">
                            <input type="text" name="query" id="_wcm_qs_query" value="<?php echo $query; ?>" />
                            <input type="submit" value="<?php echo _SEARCH; ?>">
            </form>
            <div id="sortQuickSearch" <?php echo (!isset($_SESSION['lastQuickSearch'][$filter])) ? 'style="display: none"' : ''?>>
                <strong><?php echo _BIZ_ORDER_BY; ?>:</strong>
        <?php
        foreach ($sortFields as $sortField => $sortFieldLabel) 
        { ?>
                    <a id="quickSearchOrderBy<?php echo $sortField ?>" href="javascript:quickSearchOrderBy('<?php echo $sortField ?>')"<?php echo $arrayDefaultOrderBy[0] == $sortField ? ' class="sort' . $arrayDefaultOrderBy[1] . '"' : ' class="sort"';?>><?php echo ucfirst($sortFieldLabel); ?></a>
        <?php } /* endforeach */ ?>
            </div>
            <div id="_wcm_qs_resultSet"><?php echo $html; ?>
            </div>
        </div>
        <?php

        $html = ob_get_contents();
        ob_end_clean();

        return $html;
    }

    /**
     * Returns DHTML string representing a browse panel
     * (a hierarchy of sections for current site)
     *
     * @return string DHTML
     */
    static function renderBrowsePanel()
    {
        $html  = '<div class="resultBar">';
        $html .= _BIZ_SECTIONS_HIERARCHY;
        $html .= '</div>';
        $html .= '<ul class="browse">';

        $channels = channel::getChannelHierarchy();

        foreach ($channels as $id => $path)
        {
            // make a nice display for current channel
            $parts = explode(' :: ', $path);
            $html .= '<li class="channel level-' . strtolower(chr(64 + min(5, count($parts)))) . '">';
            $html .= '<a href="';
            $html .= wcmMVC_Action::computeObjectURL('channel', $id);
            $html .= '">' . array_pop($parts) . '</a></li>';
        }

        unset($channels);
        $html .= '</ul>';

        return $html;
    }

    /**
     * Returns DHTML string representing the user history
     *
     * @param string $objectClass Optional class object to restrict history
     * @param int $offset Optional offset to retrieve statistics
     * @param int $limit Optional limit to retrieve statistics
     *
     * @return string DHTML
     */
    static function renderObjectHistory($objectClass = null, $offset=0, $limit = 12)
    {
        $rs = wcmSession::getInstance()->getViewHistory($objectClass, $offset, $limit);
        if ($rs)
        {
            $html  = '<div class="resultBar">';
            $html .= _LAST_VIEWED_ITEMS;
            $html .= '</div>';
            $html .= '<ul class="history">';
            while ($rs->next())
            {
                $item = $rs->current();
                $html .= '<li class="' . $item['objectClass'] . '">';

                if ($item['linkable'])
                {
                    $html .= '<a href="' . wcmMVC_Action::computeObjectURL($item['objectClass'], $item['objectId']) . '">' . getConst($item['info']) . '</a>';
                }
                else
                {
                    $html .= getConst($item['info']);
                }
                // Added the history date and time
                $html .= '<div class="historyDateAndTime">' . $item['dateAndTime'] . '</div>';
            }
            unset($rs);
            $html .= '</ul>';

            return $html;
        }
    }

    /**
     * Renders the main DHTML menu of the application
     * @return string DHMTL
     */
    static function renderMainMenu()
    {
        $config = wcmConfig::getInstance();
        $key = $config['wcm.project.guid'] . 'htmlmenu';

        // Store in session because it is proper for each user
        // and we want to refresh it on Logout
        if (!isset($_SESSION[$key]))
        {
            $session = wcmSession::getInstance();

            $html = null;
            foreach (wcmProject::getInstance()->layout->getRootMenus() as $rootMenu)
            {
                // Manage Menu Permission
                if (!$session->isAllowed($rootMenu, wcmPermission::P_READ))
                {
                        continue;
                }

                $html .= '<ul class="' . $rootMenu->name . '">';
                $html .= self::renderSubMenus($rootMenu->id, true);
                $html .= '</ul>';
            }
            $_SESSION[$key] = $html;
        }
        echo $_SESSION[$key];
    }

    /**
     * Returns a DHTML string corresponding to a specific menu entry
     * Note: this method is invoked from renderMainMenu.
     *
     * @param int $menuId      The menu id to display
     * @param bool $parentIsRoot  True to exclude bizMenus
     * @return string DHTML
     */
    static function renderSubMenus($menuid, $parentIsRoot = false)
    {
        $session = wcmSession::getInstance();
        $project = wcmProject::getInstance();
        $config  = wcmConfig::getInstance();

        // Display menus
        $dhtml = '';

        $menus = $project->layout->getMenus();
        if ($menuid == null)
        {
            $menuid = 0;
        }

        foreach ($menus as $menu)
        {
            if ($menu->parentId != $menuid)
            {
                continue;
            }

            // Manage Menu Permission
            if (!$session->isAllowed($menu, wcmPermission::P_READ))
            {
                continue;
            }

            // Use constants from DB for I18N
            $name = (defined($menu->name)) ? addslashes(constant($menu->name)) : addslashes($menu->name);

            $dhtml .= '<li>';

            if (!$menu->isDynamic && $menu->subMenusCount() <= 0 && $parentIsRoot)
            {
                $url = isset($menu->action) ? $config['wcm.backOffice.url'] . '?_wcmAction=' . $menu->action : '';
                $dhtml .= '<a href="' . $url . '" class="single"><span class="' . strtolower($menu->action) . '">';
                $dhtml .= stripslashes($name);
                $dhtml .= '</span></a>';
            }
            elseif (!$menu->url && !$parentIsRoot)
            {
                $dhtml .= '<span class="' . strtolower(stripslashes($name)) . '">';
                $dhtml .= stripslashes($name);
                $dhtml .= '</span>';
            }
            elseif (!$menu->url && $parentIsRoot)
            {
                $dhtml .= '<a href=""><span class="' . strtolower(stripslashes($name)) . '">';
                $dhtml .= stripslashes($name);
                $dhtml .= '</span></a>';
            }
            else
            {
                $url = '';
                // determine menu url
                if (strpos($menu->url, 'javascript:') !== 0)
                {
                    // prefix with back-office absolute url
                    $url = $config['wcm.backOffice.url'];
                    if ($menu->url != 'index.php')
                    {
                        $url .= $menu->url;
                    }
                    // suffix with action code
                    // $url .= (strpos($url, '?')) ? '&amp;' : '?';
                    $url .= $menu->action;
                }
                else
                {
                    $url .= $menu->url;
                }
                $dhtml .= '<a href="' . $url . '"><span class="' . strtolower(stripslashes($name)) . '">';
                $dhtml .= stripslashes($name);
                $dhtml .= '</span></a>';
            }

            // Add sub-menus ?
            if ($menu->isDynamic)
            {
                $dhtml .= '<ul>';
                $dhtml .= self::renderSubMenusDynamic($menu);
                $dhtml .= '</ul>';
            }
            elseif ($menu->subMenusCount() > 0)
            {
                $dhtml .= '<ul>';
                $dhtml .= self::renderSubMenus($menu->id);
                $dhtml .= '</ul>';
            }

            $dhtml .= '</li>' . PHP_EOL;
        }

        return $dhtml;
    }

    /**
     * Returns a DHTML string corresponding to a specific dynamic menu entry
     * Note: this method is invoked from renderSubMenus.
     *
     * @param wcmMenu $menu      The menu to display
     * @return string DHTML
     */
    static function renderSubMenusDynamic($menu)
    {
        $config = wcmConfig::getInstance();

        $dhtml = '';

        switch ($menu->name)
        {
            case '_MENU_USER_ADVANCED_SEARCH':
                $config = wcmConfig::getInstance();
                $configFile = $config['wcm.plugins.search.config'];
                $searchConfig = new wcmSearchConfig($configFile);
                $searches = $searchConfig->getSearches();

                foreach ($searches as $search)
                {
                    if ($search['menuAction'])
                    {
                        $configId = $search['id'];
                        $page = $searchConfig->getPage($configId, $search['defaultPage']);

                        $url = $config['wcm.backOffice.url']
                        . 'index.php?_wcmAction=' . $search['menuAction']
                        . '&_wcmTodo=' . $page['todo']
                        . '&configId=' . $configId;

                        $dhtml .= '<li><a href="' . $url . '">';
                        $dhtml .= htmlspecialchars(getConst($search['name']));
                        $dhtml .= '</a></li>';
                    }
                }
                break;

            case '_MENU_USER_SAVED_SEARCHES':
                $where = 'userId = ' . wcmSession::getInstance()->userId;
                $enum = new wcmEnumeration('wcmSavedSearch', $where, 'name');

                foreach ($enum as $savedSearch)
                {
                    $url = $config['wcm.backOffice.url'] . 'index.php?' . $savedSearch->url;

                    $dhtml .= '<li><a href="' . $url . '">';
                    $dhtml .= htmlspecialchars($savedSearch->name);
                    $dhtml .= '</a></li>';
                }
                break;

            case '_MENU_DASHBOARDS':
                $name = getArrayParameter($_REQUEST, 'dashboard', 'personal');
                $dashboards = wcmDashboard::getDashboards();

                foreach ($dashboards as $dashboard)
                {
                    $dhtml .= '<li> <a';
                    //if ($dashboard['name'] == $name) echo ' class="selected"';
                    $dhtml .= ' href="' . $config['wcm.backOffice.url'] . 'home/' . $dashboard['name'] . '">' . getConst($dashboard['title']) . '</a>';
                    $dhtml .= '</li>';
                }
                break;
        }

        return $dhtml;
    }

    /**
     * Returns a DHTML string corresponding to the current
     * object menu, retrieved from wcmMVC_Action::getContext()
     *
     * @param bool $renderMainDiv TRUE to render main 'objectMenuBar' div (by default)
     * @param array $disabled   array may contain manually disabled buttons
     * @return string DHTML
     */
    static function renderObjectMenu($renderMainDiv = true, $disabled = array())
    {
        if ($renderMainDiv)
        {
            echo '<div id="objectMenu">';
        }

        $sysobject = wcmMVC_Action::getContext();

        if ($sysobject && $sysobject instanceOf wcmObject)
        {
            $session = wcmSession::getInstance();
            $project = wcmProject::getInstance();
            $config = wcmConfig::getInstance();

            // Retrieve information from sysobject
            $id = $sysobject->id;
            $className = $sysobject->getType();
            if ($sysobject instanceOf wcmSysObject)
            {
                $classLabel = getConst($sysobject->getMasterClass()->name);
            }
            else
            {
                $classLabel = getConst('_' . strtoupper($className));
            }

            // Retrieve lock information
            $isEditable = $sysobject->isEditable();
            $lockInfo = $sysobject->getLockInfo(false);
            $isLocked = ($lockInfo->userId != 0);
            $lockUser = new wcmUser($project, $lockInfo->userId);

            // Lock is obsolete is user has been deleted
            if ($isLocked && !$lockUser->id)
            {
                $lockInfo->delete();
                $lockInfo->userId = 0;
                $isLocked = false;
            }

            // Is bizobject part of the current working site?
            if (property_exists($sysobject, 'siteId') && isset($sysobject->siteId) && $sysobject->siteId != $session->getSiteId())
            {
                $disabled['save'] = true;
                $disabled['lock'] = true;
                $disabled['unlock'] = true;
                $disabled['checkin'] = true;
                $disabled['checkout'] = true;
                $disabled['undocheckout'] = true;
                $disabled['delete'] = true;

                $objSite = wcmObjectFactory::create('site');

                if ($objSite->refresh($sysobject->siteId))
                {
                    ?>
                    <script type="text/javascript">
                    var params = {
                            \'ajax/controller.php?module=business/modalbox/tme_suggest\'
                        url: '<?php echo $config['wcm.backOffice.url']; ?>ajax/controller.php?module=business/modalbox/wrong_site',
                        parameters: {
                            currentSiteId: <?php echo $session->getSiteId(); ?>,
                            objectSiteId: <?php echo $sysobject->siteId; ?>
                        }
                    };

                    wcmModal.showOkCancel('<?php echo addslashes(_BIZ_TITLE_OBJ_IN_DIFFERENT_SITE); ?>', params, function(argResponse) {
                        if (argResponse == 'OK')
                        {
                            newLoc = '?&_wcmSiteId=<?php echo $sysobject->siteId; ?>';
                            window.location = newLoc;
                        }
                    });
                    </script>
                    <?php
                }
                else
                {
                    ?>
                    <script type="text/javascript">
                    var params = {
                        url: '<?php echo $config['wcm.backOffice.url']; ?>ajax/controller.php?module=business/modalbox/wrong_site',
                        parameters: {
                            currentSiteId: <?php echo $session->getSiteId(); ?>,
                            objectSiteId: <?php echo $sysobject->siteId; ?>
                        }
                    };

                    wcmModal.showOkCancel('<?php echo addslashes(_BIZ_TITLE_OBJ_IN_BAD_SITE); ?>', params, function(argResponse)
                    {
                        if (argResponse == 'OK')
                        {
                            newLoc = '?_wcmAction=business/<?php echo get_class($sysobject); ?>&_wcmClass=<?php echo get_class($sysobject); ?>&id=<?php echo $sysobject->id; ?>&_wcmTodo=changeSite&newSiteId=<?php echo $session->getSiteId(); ?>';
                            window.location = newLoc;
                        }
                    });
                    </script>

                <?php
                }


            }

            // Compute permissions
            $permissions = $session->getPermissions($sysobject);

            $workflowInfo = null;
            if (property_exists($sysobject, 'workflowState'))
            {
                $workflowState = $project->workflowManager->getWorkflowStateByCode($sysobject->workflowState);
                if ($workflowState)
                {
                    if ($sysobject->id && $isEditable)
                    {
                        // get available transitions
                        $transitions = array();
                        $availableTransitions = $sysobject->getAvailableTransitions();
                        if ($availableTransitions)
                        {
                            foreach ($availableTransitions as $transition)
                            {
                                $transitions[] = array(
                                    'code' => $transition->code,
                                    'name' => getConst($transition->name),
                                    'url'  => self::getTodoURL('transition', array('transition' => $transition->id))
                                );
                            }
                        }
                        $workflowInfo = array(
                            'code'   => 'workflowState',
                            'wkCode' => $workflowState->code,
                            'name'   => getConst($workflowState->name),
                            'url'    => null,
                            'child'  => $transitions
                        );
                    }
                    else
                    {
                        $workflowInfo = array(
                            'code'  => 'workflowState',
                            'name'  => $workflowState->name
                        );
                    }
                }
            }

            $a_mine = array(
                'code' => 'mine',
                'name' => _ENRICH,
                'url' => self::getTodoURL('mine'),
                'inactive' => in_array('mine', $disabled)
            );

            $a_save = array(
                'code' => 'save',
                'name' => _SAVE,
                'url' => self::getTodoURL('save'),
                'inactive' => in_array('save', $disabled)
            );

            $a_refresh = array(
                'code' => 'reload',
                'name' => _RELOAD,
                'url' => self::getTodoURL('reload'),
                'inactive' => in_array('reload', $disabled) || (!$sysobject->isLockOptimistic() && $isEditable)
            );

            $a_locked = array(
                'code' => 'locked',
                'name' => _LOCKED_BY . '...',
                'title' => _LOCKED_BY . ' ' . getConst($lockUser->name) . ' ' . _SINCE . ' ' . $lockInfo->lockDate,
                'url' => 'mailto:' . $lockUser->email,
                'inactive' => in_array('locked', $disabled)
            );

            // do not show lock button if it's a new object or if the object is locked by another user
            $a_lock = array(
                'code' => 'lock',
                'name' => _LOCK,
                'url' => self::getTodoURL('lock'),
                'inactive' => (!$id || ($lockInfo->userId == $session->userId)) || in_array('lock', $disabled)
            );

            // do not show unlock button if the user who locked the object is not the current user
            $a_unlock = array(
                'code' => 'unlock',
                'name' => _UNLOCK,
                'url' => self::getTodoURL('unlock'),
                'inactive' => (!$id || $lockInfo->userId != $session->userId) || in_array('unlock', $disabled)
            );

            // do not show checkout button if it's a new object or if it's already checked out by another user
            $a_checkout = array(
                'code' => 'checkout',
                'name' => _CHECKOUT,
                'url' => self::getTodoURL('checkout'),
                'inactive' => (!$id || ($lockInfo->userId == $session->userId)) || in_array('checkout', $disabled)
            );

            // do not show checkin button if it's a new object or if it's already checked out by another user
            $a_checkin = array(
                'code' => 'checkin',
                'name' => _CHECKIN,
                'url' => self::getTodoURL('checkin'),
                'inactive' => ($id && ($lockInfo->userId != $session->userId)) || in_array('checkin', $disabled)
            );

            // do not show undo checkout button if it's a new object or if it's already checked out by another user
            $a_undocko = array(
                'code' => 'undocheckout',
                'name' => _CANCEL,
                'url' => self::getTodoURL('undocheckout'),
                'inactive' => (!$id || ($lockInfo->userId != $session->userId)) || in_array('undocheckout', $disabled)
            );

            // do not show delete button if it's a new object or if the user doesn't have delete privileges
            $a_delete = array(
                'code' => 'delete',
                'name' => _DELETE,
                'url' => self::getTodoURL('delete'),
                'inactive' => ((($sysobject instanceof wcmSysclass) && !($sysobject instanceof wcmBizclass)) || !$id || 0 == ($permissions & wcmPermission::P_DELETE)) || in_array('delete', $disabled)
            );


            // Build menu actions
            $actions = array();

            if (!$isEditable)
            {
                if ($sysobject->isLockOptimistic())
                {
                    $actions[] = $a_refresh;
                    $actions[] = $a_locked;
                }
                elseif ($isLocked)
                {
                    $actions[] = $a_refresh;
                    $actions[] = $a_locked;
                }
                else
                {
                    $actions[] = $a_checkin;
                    $actions[] = $a_refresh;
                    $actions[] = $a_checkout;
                    $actions[] = $a_undocko;
                }
            }
            else
            {
                $disable_enrich = array('channel', 'site', 'webuser', 'publication', 'issue');
                if ($sysobject instanceOf bizobject  && !in_array($sysobject->getType(), $disable_enrich))
                {
                    $actions[] = $a_mine;
                }

                if (0 != ($permissions & wcmPermission::P_WRITE))
                {
                    if ($sysobject->isLockOptimistic())
                    {
                        $actions[] = $a_save;
                        $actions[] = $a_refresh;
                        $actions[] = $a_lock;
                        $actions[] = $a_unlock;
                    }
                    else
                    {
                        $actions[] = $a_checkin;
                        $actions[] = $a_refresh;
                        $actions[] = $a_checkout;
                        $actions[] = $a_undocko;
                    }
                }
                $actions[] = $a_delete;
            }

            // Determine sysobject title
            if ($sysobject->id)
            {
                $title = getObjectLabel($sysobject);

                // Cut title to 35 chars max :: needs to fit in 1024 resolution
                if (strlen($title) > 35)
                {
                    $title = mb_substr($title, 0, 35) . '...';
                }
            }
            else
            {
                $title = '&lt;' . _BIZ_NEW . '&gt;';
            }

            // Compute system information
            if ($id && ($sysobject instanceOf wcmSysObject))
            {
                $sysinfo = 'Id <em>' . $sysobject->id . '</em> - ';
                if ($sysobject->revisionNumber > 0)
                {
                    $sysinfo .= _REVISION_NUMBER . ' <em>' . $sysobject->revisionNumber . '</em> - ';
                }
                $sysinfo .= _MODIFIED_ON . ' <em>' . $sysobject->modifiedAt . '</em>';
            }
            else
            {
                $sysinfo = '&nbsp;';
            }
            if ($sysobject->id)
            {
                if ($sysobject instanceOf wcmBizObject)
                {
                    $urlGeneratorFactory = new wcmObjectUrlGeneratorFactory($config);
                    $urlGenerator = $urlGeneratorFactory->getDefaultGenerator();

                    $url = $urlGenerator->generateUrl($sysobject, null, true);
                    
                    $title = '<a href="' . $url . '" class="preview" title="' . _PREVIEW .
                             '" target="_blank"><span>' . _PREVIEW . '</span></a>' . mb_convert_encoding($title, 'HTML-ENTITIES', 'UTF-8');
                }
            }
        }
        else
        {
            $classLabel = 'Unexpected context';
            $title = 'The context is not an instance of wcmObject';
            $actions = array();
            $sysinfo = null;
            $workflowInfo = null;
        }

        self::renderAssetBar($classLabel, $title, $actions, $workflowInfo, $sysinfo);

        if ($renderMainDiv)
        {
            echo '</div>';
        }
    }

    /**
     * Compute the URL to perform a todo on current page
     * Note: this will invoke the wcmActionController.triggerEvent() method
     *
     * @param string $todo Action to perform
     * @param array $params Optional Assoc array representing parameters
     *
     * @return string URL to perfom the specific action
     */
    static function getTodoURL($todo, $params = null)
    {
        return 'javascript:wcmActionController.triggerEvent(\'' . $todo . '\','
               . self::getJavascriptOptions($params) . ');';
    }

    /**
     * Echo a DHTML string corresponding the generic asset bar
     *
     * @param string $header  Header to display (before the title)
     * @param string $title   Title to display
     * @param array  $actions Array of menu actions (can be null)
     * @param array  $workflowInfo Optional array representing the workflow information
     * @param string $sysinfo Optional information which appears below the asset bar
     */
    static function renderAssetBar($header, $title, $actions = array(), $workflowInfo = null, $sysinfo = null)
    {
        // Render asset bar
        $dhtml  = '';
        $dhtml .= '<div id="assetbar">';
        $dhtml .= '<h3><em>' . $header . '</em> ' . getConst($title) . '</h3>';
        $dhtml .= '<ul>';
        foreach ($actions as $action)
        {
            $class = getArrayParameter($action, 'code', '');
            if (isset($action['child']))
            {
                $class .= ' parent';
            }

            $url = null;
            if (getArrayParameter($action, 'inactive', 0))
            {
                $class .= ' inactive';
            }
            elseif (isset($action['url']))
            {
                $url   = ' href="' . $action['url'] . '"';
            }

            if (isset($action['title']))
            {
                $url .= ' title="' . mb_convert_encoding($action['title'], 'HTML-ENTITIES', 'UTF-8') . '"';
            }

            $dhtml .= '<li><a class="' . $class . '"' . $url . '><span><span>' . getArrayParameter($action, 'name', '?') . '</span></span></a>';
            if (isset($action['child']))
            {
                $dhtml .= '<ul>';
                foreach ($action['child'] as $action)
                {
                    $dhtml .= '<li><a class="' . getArrayParameter($action, 'code', '') . '" href="' . getArrayParameter($action, 'url', '#') . '"><span><span>' . getArrayParameter($action, 'name', '?') . '</span></span></a></li>';
                }
                $dhtml .= '</ul>';
            }

            $dhtml .= '</li>';
        }
        $dhtml .= '</ul>';
        $dhtml .= '</div>';
        $dhtml .= '<div id="sysinfo">';
        if ($workflowInfo)
        {
            $dhtml .= '<div id="workflow">';
            $dhtml .= '<h4 class="' . strtolower($workflowInfo['code']) . '"><strong>' . getConst($workflowInfo['name']) . '</strong></h4>';
            if (isset($workflowInfo['child']))
            {
                $dhtml .= '<ul>';
                foreach ($workflowInfo['child'] as $transition)
                {
                    $dhtml .= '<li><a class="' . strtolower($transition['code']) . '" href="' . $transition['url'] . '">' . getConst($transition['name']) . '</a></li>';
                }
                $dhtml .= '</ul>';
            }
            $dhtml .= '</div>';
        }
        $dhtml .= '<ul class="info">';
        $dhtml .= '<li class="stats">' . $sysinfo . '</li>';
        $dhtml .= '</ul>';
        $dhtml .= '</div>';

        echo $dhtml;
    }


    /**
     * Echo a DHTML string corresponding to current working site
     * and list of available web sites
     */
    static function renderWorkingSite()
    {
        if (!isset($_SESSION['wcmGUI_renderWorkingSite']))
        {
            $session = wcmSession::getInstance();
            $config = wcmConfig::getInstance();
            $site = $session->getSite();

            $html = '<strong>' . _WORKING_IN . '</strong>';
            $html .= '<span class="site-' . $site->id . '">' . ((strlen($site->title) <= 30) ? textH8($site->title) : (substr(textH8($site->title), 0, 30) . '...')) . '</span>';
            $html .= '<a href="' . $config['wcm.backOffice.url'] . 'index.php?_wcmAction=business/site&amp;id=';
            $html .= $site->id . '" class="edit" title="' . _EDIT . '">' . _EDIT . '</a>';

            if ($site->getConfig('previewURL'))
            {
                $html .= '<a href="' . $site->getConfig('previewURL') . '" class="preview" title="' . _PREVIEW . '" target="preview">' . _PREVIEW . '</a>';
            }

            $sql = 'SELECT COUNT(id) FROM biz_site WHERE id != ' . $session->getSiteId();

            $db = wcmMasterClassRegistry::getDatabase($site);
            $count = $db->executeScalar($sql);

            if ($count)
            {
                $html .= '<ul id="siteSelector">';
                $html .= '<li><a href="" title="' . _CHANGE_SITE . '" onclick="openmodal(\'' . _CHANGE_SITE . '\'); modalPopup(\'changesite\',\'list\', \'\'); return false;" class="title"><span><span>' . _CHANGE . '</span></span></a></li>';
                $html .= '</ul>';
            }
            $_SESSION['wcmGUI_renderWorkingSite'] = $html;
        }
        echo $_SESSION['wcmGUI_renderWorkingSite'];
    }

     /**
     * Open the collapsable panel
     *
     * @param string $label Label of the panel
     * @param bool   $expanded Is expanded?
     * @param string $info  (optionnal) extra html
     * @param bool $useCookie (optionnal) Use cookie to remember the change of the panel (default : true)
     */
    static function openCollapsablePane($label, $expanded = true, $info = null, $useCookie = true)
    {
        // name of the cookie
        $NAME_COOKIE_FIRST_PART = 'collapsablePane';

        echo '<div class="collapsable">';
        echo '<div class="action-header">';

        $bizobject = wcmMVC_Action::getContext();

        // use cache with cookie or not ?
        if (!$useCookie || $bizobject == null || !($bizobject instanceof bizobject))
        {
            echo '<h3 onclick="$(this).up().siblings().shift().toggle(); if ($(this).className == \'collapsed\') $(this).removeClassName(\'collapsed\'); else $(this).addClassName(\'collapsed\');">';
        }
        else
        {
            $session = wcmSession::getInstance();

            // encode the label in md5
            $cookieId = hash('md5', $label);

            // create unique name of cookie
            $cookieName = $NAME_COOKIE_FIRST_PART . '_' . $bizobject->getType() . '_' . $bizobject->id . '_' . $session->getUser()->id;

            // create the WebCollapsablePane object in javascript
            echo '<script language="JavaScript" type="text/javascript">';
            echo '	' . $NAME_COOKIE_FIRST_PART . '_' . $cookieId . ' = new WebCollapsablePane(\'' . $cookieId . '\',\'' . $cookieName . '\')';
            echo '</script>';

            echo '<h3 id="' . $cookieId . '" onclick="' . $NAME_COOKIE_FIRST_PART . '_' . $cookieId . '.setCollapsable()"';

            // if cookie exist, check if $cookieId exist in the cookie
            // if yes, dont collapse
            if (isset($_COOKIE[$cookieName]) && strpos($_COOKIE[$cookieName], $cookieId))
            {
                echo 'class="collapsed"';
                $expanded = false;
            }
            elseif ($expanded === false)
            {
                echo 'class="collapsed"';
            }
            echo ' >';

        }

        echo $label . '</h3>';
        echo $info;
        echo '</div>';
        echo '<div';
        if (!$expanded)
        {
            echo ' style="display:none;"';
        }
        echo '>';
    }

    /**
     * Close a collapsable pane
    */
    static function closeCollapsablePane()
    {
        echo '</div>';
        echo '</div>';
    }


    /**
     * Render the html code used to open a dialog (an anchor tag)
     *
     * @param string $title         Text of the dialog button
     * @param string $dialog        Dialog name
     * @param mixed  $parameters    Query string or associative array
     * @param int    $width         Dialog width (default 740)
     * @param int    $height        Dialog height (default 540)
     * @param string $className     Optional class for the dialog button
     */
    static function renderHtmlDialogButton($title, $dialog, $parameters = null, $width = 740, $height = 540, $className = null)
    {
        if ($className)
        {
            $className = 'class="' . $className . '"';
        }
        echo '<a' . $className . ' href="' . wcmDialogUrl($dialog, $parameters, $width, $height) . '">' . $title . '</a>';
    }


    /**
     * Render a modal button html
     *
     * @param string $label         Text of the button
     * @param string $command		Command executed by the button
     * @param int $kind				Kind of button
     * @param mixed $id				ID of the button
     */
    static function renderModalButton($label, $command, $kind, $id)
    {
        echo '<a href="" onclick="openmodal(\'' . $label . '\'); modalPopup(\'' . $command . '\',\'' . $kind . '\', \'' . $id . '\'); return false;" class="title"><span>' . $label . '</span></a>';
    }

    /**
     * Render the html code used to display available templates for this bizobject
     * The available templates for each bizobject is described in an xml configuration file
     *
     * @param wcmBizobject $bizobject   the bizobject that has associated templates
     * @param string       $xmlFile     the path to the xml configuration file
     */
    static function renderTemplateChooser(wcmBizobject $bizobject, $xmlFile = '/design/default.xml')
    {
        $config = wcmConfig::getInstance();
        $xml = simplexml_load_file(WCM_BACKOFFICE_CONF_DIR . $xmlFile);

        if ($bizobject->templateId)
        {
            echo '<p>' . _BIZ_SELECTED_TEMPLATE . ' <em>' . $bizobject->templateId . '</em></p>';
        }
        else
        {
            echo '<p>' . _BIZ_NO_SELECTED_TEMPLATE . '</p>';
        }

        foreach ($xml->children() as $child)
        {
            if ($child->getName() == $bizobject->getType())
            {
                foreach ($child as $template)
                {
                    // only display template information if the template exists
                    if (file_exists($config['wcm.templates.path'] . $template->id))
                    {
                        $class = ($bizobject->templateId == $template->id) ? 'templateThumb selected' : 'templateThumb';
                        echo '<div class="' . $class . '" onclick="_wcmSelectTemplate(this, \'' . $template->id . '\')">';
                        echo '<p>' . $template->name . '</p>';
                        echo self::getTemplateThumbnail($template->id);
                        echo '<p>' . $template->description . '</p>';
                        echo '</div>';
                    }
                }
            }
        }
    }


    /**
     * Get the thumbnail of a specific template
     *
     * @param   string $templateId  id of template
     * @return  string HTML image tag to display the thumbnail if it exists, else will display the default thumbnail
     */
    static function getTemplateThumbnail($templateId)
    {
        $config = wcmConfig::getInstance();

        $thumbPath = substr($templateId, 0, -4);
        $allowedExtensions = array('.gif', '.jpg', '.png');

        foreach ($allowedExtensions as $extension)
        {
            if (file_exists(WCM_WWW_BACKOFFICE_DIR . '/system/public/img/' . $thumbPath . $extension))
            {
                return '<img src="' . $config['wcm.backOffice.url'] . 'img/' . $thumbPath . $extension . '" />';
            }
        }
        return '<img src="' . $config['wcm.backOffice.url'] . 'img/design/default.png" />';
    }


     /**
     * Preview bizobject
     * => Execute template 'preview/{className|'default'}.tpl with following parameters
     *          widgetMode  The widget mode (see params)
     *          bizobject   The bizobject
     *          context     The bizobject assoc array
     *
     * @param wcmBizobject $bizobject
     * @param int $widgetMode Widget display mode (bit field, default is wcmWidget::VIEW_CONTENT)
     * @param string $templateId Optional template to use (default is bizobject->templateId)
     * @param wcmLogger $logger Optional logger (if null, the generator will create its own)
     *
     * @return string Result of template execution
     */
    public static function previewBizObject($bizobject, $widgetMode = wcmWidget::VIEW_CONTENT, $templateId = null, $logger = null)
    {
        // Inject templateId if needed
        if (!$templateId)
        {
            $templateId = 'preview/' . $bizobject->getType() . '.tpl';
        }

        // Find template
        $config = wcmConfig::getInstance();
        if (!file_exists($config['wcm.templates.path'] . $templateId))
        {
            $templateId = 'preview/default.tpl';
        }

        // Execute template
        $generator = new wcmTemplateGenerator($logger, false, $widgetMode);

        return $generator->executeTemplate(
            $templateId,
            array('widgetMode' => $widgetMode, 'obizobject' => $bizobject, 'bizobject' => $bizobject->getAssocArray(false))
        );

    }


}
