<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.dashboard.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmDashboard manages dashboards for users
 *
 * A dashboard is composed of 1 to n zones that, in turn,
 * are composed of 1 to n modules.
 */
class wcmDashboard
{
    /**
     * Name of XML file representing the dashboard
     */
    private $fileName;

    /**
     * Is personal?
     * @var bool
     */
    private $isPersonal;

    /**
     * Simple XML object containing dashboard representation
     * @var SimpleXMLElement
     */
    private $dashboard;

    /**
     * Dashboard's constructor.
     *
     * @param string $xmlFile XML file to use (or null to use default)
     * @throws Exception When XML is not valid
     */
    public function __construct($xmlFile = null)
    {
        // Determine which XML file to use
        if (!$xmlFile)
        {
            $xmlFile = WCM_BACKOFFICE_CONF_DIR . '/dashboard/personal.xml';
        }

        if (file_exists($xmlFile))
        {
            // Load xml
            try
            {
                $this->dashboard = simplexml_load_file($xmlFile);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
            }
        }
        else
        {
            throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
        }

        $this->fileName = $xmlFile;
        $this->isPersonal = (substr($xmlFile, -12) == 'personal.xml');
    }

    /**
     * Basic rendring method
     *
     * @return  string  containing final html of dashboard
     */
    public function render()
    {
        $html  = '';
        $html .= '<div id="dashboard">';
        $zid = 0;
        foreach ($this->dashboard->children() as $child)
        {
            switch ($child->getName())
            {
                case 'zone':
                    $html .= $this->renderZone($child, $zid);
                    $zid++;
                    break;
                case 'literal':
                    $html .= self::parseText(strval($child));
                    break;
            }
        }
        $html .= '</div>';
        return $html;
    }

    /**
     * Render zones
     *
     * @param SimpleXMLElement simple xml object $zone
     * @param int $zid zone index
     *
     * @return string containing final html of zone
     */
    private function renderZone($zone, $zid)
    {
        $html  = '';
        $html .= '<style type="text/css">.collapsed {border: 3px red solid;}</style>';
        $html .= '<div class="zone">';
        $html .= '<h3 onclick="$(\'dbzone_' . $zid . '\').toggle();';
        $html .= 'if ($(this).className == \'collapsed\') $(this).removeClassName(\'collapsed\');';
        $html .= 'else $(this).addClassName(\'collapsed\');">' . getConst($zone->title) . '</h3>';
        $html .= '<div id="dbzone_' . $zid . '">';

        $mid = 0;
        foreach ($zone->children() as $child)
        {
            switch ($child->getName())
            {
                case 'module':
                    $html .= $this->renderModule($child, $zid, $mid);
                    $mid++;
                    break;

                case 'widgets':
                    $html .= $this->renderWidgets($zid);
                    break;
            }
        }

        $html .= '</div>';
        $html .= '</div>';

        return $html;
    }

    /**
     * Render the personal widgets for current user
     *
     * @param int $zid zone index
     *
     * @return string Html of the widgets
     */
    private function renderWidgets($zid)
    {
        // Determine which XML file to use
        $session = wcmSession::getInstance();
        $xmlFile = WCM_BACKOFFICE_CONF_DIR . '/dashboard/users/user-' . $session->userId . '.xml';

        $html = null;
        if (file_exists($xmlFile))
        {
            try
            {
                $widgets = simplexml_load_file($xmlFile);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
            }

            if ($widgets)
            {
                foreach ($widgets->children() as $module)
                {
                    if ($module->getName() == 'module')
                    {
                        $html .= $this->renderModule($module, $zid, $module['uid'], true);
                    }
                }
            }
            unset($widgets);
        }

        return $html;
    }

    /**
     * Render module (use cache if @ttl is provided)
     * Set values that will be interpreted in the smarty template
     *
     * @param SimpleXMLElement simple xml object $module
     * @param int $zid zone index
     * @param int $mid module index
     * @param bool $widget TRUE when it's a personal widget
     *
     * @return string  Html of the module
     */
    private function renderModule($module, $zid, $mid, $widget = false)
    {
        // Render title
        $html = '';

        if ($module['class'] != 'tasks')
        {
            $html = '<div id="droppable_' .  $zid . '_' . $mid . '" class="droppable-module">';
        }

        $html .= '<div id="draggable_' . $zid . '_' . $mid . '" class="' . trim('module ' . $module['class']) . '">';
        $html .= $this->renderModuleTitle($module, $zid, $mid, $widget);
        $html .= '<div id="dbmodule_' . $zid . '_' . $mid . '" class="scroll">';

        // Cache module?
        $ttl = intval($module['ttl']);
        if ($ttl)
        {
            $key  = $module->id . wcmSession::getInstance()->userId;
            $content = wcmCacheRegistry::get()->fetch($key);
            if (false === $content)
            {
                $content = $this->renderModuleContent($module, $zid, $mid);
                wcmCacheRegistry::get()->store($key, $content, $ttl);
            }
            $html .= $content;
        }
        else
        {
            $html .= $this->renderModuleContent($module, $zid, $mid);
        }

        $html .= '</div>';
        $html .= '</div>';

        if ($module['class'] != 'tasks')
        {
            $html .= '</div>';
        }

        return $html;
    }

    /**
     * Render module title with options to refresh and add/remove to personal dashboard
     *
     * @param SimpleXMLElement  simple xml object $module
     * @param int $zid zone index
     * @param int $mid module index
     * @param bool $widget true when it's a widget
     *
     * @return string $html containing final html of module's title
     */
    private function renderModuleTitle($module, $zid, $mid, $widget = false)
    {
        $html = null;

        $html .= '<h4  class="modulehandle">';
        $html .= '<span>' . getConst($module->title) . '</span>';

        if ($this->isPersonal)
        {
            if ($widget)
            {
                $link = '_wcmRemoveModule(this,' . $zid . ',\'' . $mid . '\')';
                $html .= '<span class="remove" onclick="' . $link . '">&nbsp;</span>';
            }
        }
        else
        {
            $link = '_wcmAddModule(this, \'' . str_replace('\\', '/', $this->fileName) . '\', ' . $zid . ', ' . $mid . ')';
            $html .= '<span class="add" onclick="' . $link . '">&nbsp;</span>';
        }
        $html .= '</h4>';

        return $html;
    }


    /**
     * Render module content (no cache)
     *
     * @param SimpleXMLElement  simple xml object $module
     * @param   int $zid zone index
     * @param   int $mid module inddex
     *
     * @return  string  $html   containing final html of module
     */
    private function renderModuleContent($module, $zid, $mid)
    {
        $parameters = array();
        $parameters['dashboard'] = $this;
        $parameters['zone'] = array('id' => $zid);
        $parameters['module'] = array('id' => $mid, 'title' => $module->title);
        $params = array();
        if ($module->parameters)
        {
            foreach ($module->parameters->param as $param)
            {
                if (isset($param->param))
                {
                    $subparams = array();
                    foreach ($param->param as $subparam)
                    {
                        $subparams[strval($subparam['name'])] = self::parseText(strval($subparam));
                    }
                    $params[strval($param['name'])] = $subparams;
                }
                else
                {
                    $params[strval($param['name'])] = self::parseText(strval($param));
                }
            }
        }
        $parameters['params'] = $params;

        $generator = new wcmTemplateGenerator();
        $template = new wcmTemplate();
        $template->refresh($module->template);
        
        return $generator->executeTemplate($template, $parameters);
    }

    /*
     * Replace predefined variable references with their values.
     * Available variables are:
     * @SessionID   The current session ID
     * @UserID      The current connected user ID
     * @SiteID      The current context (working site ID)
     * @Date        The current date in sql format 'Y-m-d'
     * @Time        The current time in sql format 'H-i-s'
     * @DateTime    The current date and time in sql format 'Y-m-d H:i:s'
     *
     * @param string $text Text containing references to variables
     *
     * @return string Text with variable references replaced by their values
     */
    private static function parseText($text)
    {
        $session = wcmSession::getInstance();

        $text = str_replace('@SessionID', $session->id, $text);
        $text = str_replace('@SiteId', $session->getSiteId(), $text);
        $text = str_replace('@UserID', $session->userId, $text);
        $text = str_replace('@Date', '\'' . date('Y-m-d') . '\'', $text);
        $text = str_replace('@Time', '.' . date('H:i:s') . '\'', $text);
        $text = str_replace('@DateTime', '.' . date('Y-m-d H:i:s') . '\'', $text);

        return $text;
    }

    /**
     * Compute the list of available dashboards
     *
     * @return array Assoc array of existing dashboards (title => path)
     */
    public static function getDashboards()
    {
        $list = array();
        $list[] = array('title' => _DASHBOARD_MY_DASHBOARD, 'name' => 'personal');

        $path =  WCM_BACKOFFICE_CONF_DIR . '/dashboard/';
        $d = dir($path);
        while (false !== ($entry = $d->read()))
        {
            if (is_file($path . $entry) && substr($entry, -4) == '.xml')
            {
                if ($entry == 'personal.xml')
                {
                    continue;
                }

                try
                {
                    $xml = simplexml_load_file($path . $entry);
                }
                catch (Exception $ex)
                {
                    throw new Exception(_BIZ_INVALID_XML . ': ' . $path . $entry);
                }

                if ($xml)
                {
                    // check authorizations
                    $granted = true;
                    $user = wcmSession::getInstance()->getUser();
                    if (!$user->isAdministrator && isset($xml->authorizations))
                    {
                        $granted = false;
                        foreach ($xml->authorizations->children() as $node)
                        {
                            $acl = strval($node);
                            if ($node->getName() == 'user')
                            {
                                $granted = $user->id === intval($acl) || $user->login === $acl;
                                if ($granted)
                                {
                                    break;
                                }
                            }
                            elseif ($node->getName() == 'group')
                            {
                                $granted = $user->isMemberOf($acl);
                            }
                        }
                    }
                    if ($granted)
                    {
                        $title = isset($xml->title) ? $xml->title : '_DASHBOARD';
                        $list[] = array('title' => $title, 'name' => substr($entry, 0, strlen($entry) - 4));
                    }
                }
            }
        }
        $d->close();

        return $list;
    }

    /**
     * Add a module to the personal file
     *
     * @param string $xmlFile Name of the XML file containing the module
     * @param int $zid Zone index containing the module to add
     * @param int $mid Index of the module to add
     */
    public static function addModule($xmlFile, $zid, $mid)
    {
        $widgets = null;
        $session = wcmSession::getInstance();
        $personal = WCM_BACKOFFICE_CONF_DIR . '/dashboard/users/user-' . $session->userId . '.xml';
        unset($session);

        if (file_exists($personal))
        {
            try
            {
                $widgets = simplexml_load_file($personal);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $personal);
            }
        }
        else
        {
            $widgets = simplexml_load_string('<?xml version="1.0" encoding="utf-8" ?><widgets/>');
        }

        // add module
        $module = null;
        if (file_exists($xmlFile))
        {
            try
            {
                $dashboard = simplexml_load_file($xmlFile);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
            }

            // retrieve zone
            $index = 0;
            foreach ($dashboard->children() as $zone)
            {
                if ($zone->getName() == 'zone')
                {
                    if ($index == $zid)
                    {
                        // retrieve module
                        $index = 0;
                        foreach ($zone->children() as $module)
                        {
                            if ($module->getName() == 'module')
                            {
                                if ($index == $mid)
                                {
                                    // retrieve module as DOM node
                                    $module = dom_import_simplexml($module);
                                    $module->setAttribute('class', 'personal');
                                    $module->setAttribute('uid', uniqid());
                                    $widgets = dom_import_simplexml($widgets);
                                    $module = $widgets->ownerDocument->importNode($module, true);
                                    $widgets->appendChild($module);
                                    $widgets->ownerDocument->save($personal);
                                    echo 'Saving in ' . $personal . PHP_EOL . $widgets->saveXML();
                                    return;
                                }
                                $index++;
                            }
                        }
                        throw new Exception('No module ' . $mid . ' in zone ' . $zid);
                    }
                    $index++;
                }
            }
            throw new Exception('No zone ' . $zid);
        }
        else
        {
            throw new Exception('No file ' . $xmlFile);
        }
    }

    /**
     * Remove a module from the personal file
     *
     * @param int $mid Index of the module to remove
     */
    public static function removeModule($mid)
    {
        $session = wcmSession::getInstance();
        $xmlFile = WCM_BACKOFFICE_CONF_DIR . '/dashboard/users/user-' . $session->userId . '.xml';
        unset($session);
        if (file_exists($xmlFile))
        {
            try
            {
                $widgets = simplexml_load_file($xmlFile);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
            }

            if ($widgets)
            {
                $widgets = dom_import_simplexml($widgets);

                $xpath = new DOMXPath($widgets->ownerDocument);
                $module = $xpath->query('//module[@uid=\'' . $mid . '\']')->item(0);
                $widgets->removeChild($module);
                $widgets->ownerDocument->save($xmlFile);
            }
            unset($widgets);
        }
    }

    /**
     * Reoder modules after a drag and drop takes place
     *
     * @param int $mid_from Index of the module at the dragged location
     * @param int $mid_to Index of the module at the dropped location
     *
     */
    public static function reoderModule($mid_from, $mid_to)
    {
        $session = wcmSession::getInstance();
        $xmlFile = WCM_BACKOFFICE_CONF_DIR . '/dashboard/users/user-' . $session->userId . '.xml';
        unset($session);
        if (file_exists($xmlFile))
        {
            try
            {
                $widgets = simplexml_load_file($xmlFile);
            }
            catch (Exception $ex)
            {
                throw new Exception(_BIZ_INVALID_XML . ': ' . $xmlFile);
            }

            if ($widgets)
            {
                //get the modules
                $modules = $widgets->xpath('/widgets/module');
                //reorder them
                $temp = $modules[$mid_from];
                $modules[$mid_from] = $modules[$mid_to];
                $modules[$mid_to] = $temp;
                //create new xml
                $widgets = simplexml_load_string('<?xml version="1.0" encoding="utf-8" ?><widgets/>');
                $widgets = dom_import_simplexml($widgets);

                try
                {
                    //add the modules
                    foreach ($modules as $module)
                    {
                        $module = dom_import_simplexml($module);
                        $module = $widgets->ownerDocument->importNode($module, true);
                        $widgets->appendChild($module);
                    }
                    //write to file
                    $widgets->ownerDocument->save($xmlFile);
                }
                catch (DOMException $e)
                {
                    print_r($e);
                }
                unset($widgets);
            }
        }
    }
}
