<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.layout.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmGenerator class is an helper class used to manage menus and design zones
 *
 * Remark: menus are stored in cache through static variables
 */
class wcmLayout
{
    /**
     * Returns hierarchical array of menus to populate dropdown list
     *
     * @param array $menus Array to update (should be initialized)
     * @param int $excludeId Optional ID of menu to exclud from the list
     * @param string $prefix Prefix
     * @param int $parentId ParentID of the menu
     */
    function getMenuHierarchy(array &$menus, $excludeId = 0, $prefix = null, $parentId = 0)
    {
        foreach ($this->getMenus() as $menu)
        {
            if ($menu->parentId == $parentId)
            {
                // Add menu (unless it is excluded)
                $path = ($prefix) ? $prefix . ' :: ' . getConst($menu->name) : getConst($menu->name);
                if ($menu->id != $excludeId)
                {
                    $menus[$menu->id] = $path;
                }

                // Recursively add sub-menus
                $this->getMenuHierarchy($menus, $excludeId, $path, $menu->id);
            }
        }
    }

    /**
     * Returns a menu corresponding to a specific id
     *
     * @param int $id The menu id
     *
     * @return wcmMenu The menu matching given id (or null if id is invalid)
     */
    public function getMenuById($id)
    {
        // Use cache
        return getArrayParameter($this->getMenus(), $id, null);
    }

    /**
     * Returns a menu object corresponding to a specific action code
     *
     * @param string $action The menu action to search for
     *
     * @return wcmMenu The menu matching given id (or null if id is invalid)
     */
    public function getMenuByAction($action)
    {
        // Use cache
        foreach ($this->getMenus() as $menu)
        {
            if ($menu->action == $action)
            {
                return $menu;
            }
        }

        return null;
    }

    /**
     * Returns an array of root menus
     *
     * @param bool $resetCache Whether to reset the cache, ie. load from DB (default is false)
     *
     * @return array An assoc array of {@link wcmMenu} objects (keys are ids)
     */
    public function getRootMenus($resetCache = false)
    {
        // Use cache
        $menus = array();
        foreach ($this->getMenus($resetCache) as $menu)
        {
            // Return only root menus (i.e. no parent id)
            if (!$menu->parentId)
            {
                $menus[$menu->id] = $menu;
            }
        }

        return $menus;
    }

    /**
     * Returns an array of all menus
     *
     * @param bool $resetCache Whether to reset the cache, ie. load from DB (default is false)
     *
     * @return array An assoc array of {@link wcmMenu} objects (keys are ids) or null on failure
     */
    public function getMenus($resetCache = false)
    {
        $cached = wcmCacheRegistry::get()->fetch('wcmMenu');
        if ($resetCache || false === $cached)
        {
            $enum = new wcmEnumeration('wcmMenu', null, 'parentId, rank');
            
            $cached = array();
            foreach ($enum as $menu)
            {
                $cached[$menu->id] = $menu;
            }

            // Cache objects
            wcmCacheRegistry::get()->store('wcmMenu', $cached);
        }

        return $cached;
    }

    /**
     * Returns an associative array of zones belonging to a bizobject.
     *
     * @param bizobject $bizobject Bizobject owner of the zones to retrieve
     *
     * @return array Zones belonging to bizobject
     */
    public function getZones($bizobject)
    {
        $zones = array();
        $enum = new wcmEnumeration('wcmDesignZone', 'sourceClass=\'' . get_class($bizobject) . '\' AND sourceId=' . $bizobject->id);
        foreach ($enum as $zone)
        {
            $zones[] = $zone; 
        }
        return $zones;
    }

    /**
     * Resets the zone contents belonging to a bizobject in
     * the current session, ie. unsets the session values.
     *
     * @param bizobject $bizobject Bizobject owner of the zone
     */
    public function resetZoneContents($bizobject)
    {
        // Get current zones
        $zones = $this->getZones($bizobject);

        // Reset zones
        if ($zones)
        {
            foreach ($zones as $zone)
            {
                $zones->delete();
            }
        }
    }

    /**
     * Set the zone contents
     * @param bizobject $bizobject Bizobject owner of the zone
     * @param array $array Zone array in format zoneName=>widgets
     * @param array $datas Useless parameter
     * @param array $widgetsettings Widget settings
     */
    public function setZoneContents($bizobject, $array, $datas, $widgetsettings = null)
    {
        foreach ($array as $zoneName => $widgets)
        {
            $zone = new wcmDesignZone(get_class($bizobject), $bizobject->id, $zoneName);
            if (is_array($widgets))
            {
                foreach ($widgets as $widget)
                {
                    if (empty($widget))
                    {
                        continue;
                    }
                    list($widgetName, $widgetGuid) = explode('-', $widget);

                    $settings = array();
                    if ($serialisation = getArrayParameter($widgetsettings, $widget . '-settings', false))
                    {
                        parse_str($serialisation, $settings);
                    }
                    $zone->addWidget($widgetName, $widgetGuid, $settings);
                }
            }
            $zone->save();
        }
    }

    /**
     *  Return a widget list
     *
     *  @param wcmObject|null   $object If the object is define the array will just return the widgets for this object
     *
     *  @return array widgetClassname => Human readable name
     */
    public function getWidgets($object = null)
    {
        $widgetList = array();
        foreach ($this->loadWidgetConfig() as $widget)
        {
            if ($object === null)
            {
                $widgetList[(string)($widget->class)] = (string)($widget->name);
            }
            else
            {
                foreach (explode(',', $widget->context) as $context)
                {
                    if ($object instanceof $context)
                    {
                        $widgetList[(string)($widget->class)] = (string)($widget->name);
                        break;
                    }
                }
            }
        }
        return $widgetList;
    }

    /**
     *  Create the widgets.xml configuration file
     *  @param string $fileName Filename to config file
     */
    public function createWidgetConfig($fileName = null)
    {
        if ($fileName === null)
        {
            $filename = WCM_BACKOFFICE_CONF_DIR . '/widgets.xml';
        }

        $config = wcmConfig::getInstance();

        $xml = new DOMDocument();
        $rootNode = $xml->createElement('widgets');
        $xml->appendChild($rootNode);

        $wd = new RecursiveDirectoryIterator($config['wcm.widgets.path']);
        foreach (new RecursiveIteratorIterator($wd, RecursiveIteratorIterator::SELF_FIRST) as $file)
        {
            if ($file->getFilename() !== 'infos.xml')
            {
                continue;
            }

            $widgetdir = dirname($file->getPathname());
            $widgetDOM = new DOMDocument;
            $widgetDOM->load($file->getPathname());
            $widgetXML = $xml->importNode($widgetDOM->documentElement, true);

            $widgetXML->appendChild($xml->createElement('path', $widgetdir));

            if ($widgetDOM->getElementsByTagName('class')->length == 0)
            {
                $className = 'wcmWidget';
                foreach (explode(DIRECTORY_SEPARATOR, substr($widgetdir, strlen($config['wcm.widgets.path']))) as $ns)
                {
                    $className .= ucfirst($ns);
                }
                $widgetXML->appendChild($xml->createElement('class', $className));
            }

            $xml->documentElement->appendChild($widgetXML);

        }
        $xml->save($fileName);
    }

    /**
     * Load the widgets configuration file
     * @return SimpleXMLElement Configuration XML object
     */
    public function loadWidgetConfig()
    {
        static $xmlconfig = null;
        if ($xmlconfig === null)
        {
            $config = wcmConfig::getInstance();
            $filename = WCM_BACKOFFICE_CONF_DIR . '/widgets.xml';
            if (!file_exists($filename))
            {
                $this->createWidgetConfig($filename);
            }
            $xmlconfig = simplexml_load_file($filename);
        }
        return $xmlconfig;
    }

}
