<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.acl.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * ACL class initializable through a specific 'acl.ini' configuration file
 * Expected format is:
 * ----
 * [roles]
 * {role} = [{inherited-role}]
 * {role} = [{inherited-role}]
 * ...
 *
 * [resources]
 * {resource} = [{inherited-resource}]
 * {resource} = [{inherited-resource}]
 * ...
 *
 * [permissions]
 * [allow|deny].{resource} = {role}[,{role}]* [ : {privilege}[,{privilege}]* ]
 * [allow|deny].{resource} = {role}[,{role}]* [ : {privilege}[,{privilege}]* ]
 * ...
 * ----
 */
class wcmAcl extends Zend_Acl
{
    /**
     * Constructor
     *
     * @param string $config Configuration file path (default is APPLICATION_PATH.'/config/acl.ini')
     */
    public function __construct($config = null)
    {

        // when no ACL file exists, allow all to everyone!
        if (!file_exists($config))
        {
            $this->allow(null, null);
            $this->addRole(new Zend_Acl_Role('current-user'));
            return;
        }

        $config = parse_ini_file($config, true);

        $this->addRoles($config);

        // get resources from config file
        // {resource} = [{inherited-resource}]
        if (isset($config['resources']))
        {
            foreach ($config['resources'] as $resource => $inherited)
            {
                if ($inherited)
                {
                    $this->add(new Zend_Acl_Resource($resource), $inherited);
                }
                else
                {
                    $this->add(new Zend_Acl_Resource($resource));
                }
            }
        }

        $this->addPermissions($config);

    }

    /**
     * addRoles: adds roles to the ACL from the configuration file
     *
     * @param array $config Parsed configuration file
     */
    public function addRoles($config)
    {
        // get roles from config file
        // {role} = [{inherited-role}]
        if (isset($config['roles']))
        {
            foreach ($config['roles'] as $role => $inherited)
            {
                if ($inherited)
                {
                    $this->addRole(new Zend_Acl_Role($role), $inherited);
                }
                else
                {
                    $this->addRole(new Zend_Acl_Role($role));
                }
            }
        }

    }

    /**
     * addPermission: adds permissions to the ACL from the configuration file
     *
     * @param array $config Parsed configuration file
     */
    public function addPermissions($config)
    {
        // get permissions from config file
        // [allow|deny].{resource} = {role}[,{role}]*
        if (isset($config['roles']))
        {
            foreach ($config['permissions'] as $permission => $roles)
            {
                // retrieve mode (allow | deny) and resource
                list($mode, $resource) = explode('.', $permission, 2);
                $roles = explode(',', trim($roles));

                // when allowing specific roles, we must deny to everyone else
                if ($mode == 'allow' && count($roles) > 0)
                {
                    $this->deny(null, $resource);
                }

                // allow or deny each role the respective resource and privileges
                foreach ($roles as $role)
                {
                    $role = trim($role);
                    if ($role == '')
                    {
                        $role = null;
                    }
                    $this->$mode($role, $resource);
                }
            }
        }

    }
    /**
     * Returns true if and only if the Role has access to the Resource
     *
     * The $role and $resource parameters may be references to, or the string identifiers for,
     * an existing Resource and Role combination.
     *
     * If either $role or $resource is null, then the query applies to all Roles or all Resources,
     * respectively. Both may be null to query whether the ACL has a "blacklist" rule
     * (allow everything to all). By default, Zend_Acl creates a "whitelist" rule (deny
     * everything to all), and this method would return false unless this default has
     * been overridden (i.e., by executing $acl->allow()).
     *
     * If a $privilege is not provided, then this method returns false if and only if the
     * Role is denied access to at least one privilege upon the Resource. In other words, this
     * method returns true if and only if the Role is allowed all privileges on the Resource.
     *
     * This method checks Role inheritance using a depth-first traversal of the Role registry.
     * The highest priority parent (i.e., the parent most recently added) is checked first,
     * and its respective parents are checked similarly before the lower-priority parents of
     * the Role are checked.
     *
     * @param  Zend_Acl_Role_Interface|string     $role
     * @param  Zend_Acl_Resource_Interface|string $resource
     * @param  string                             $privilege
     * @uses   Zend_Acl::get()
     * @uses   Zend_Acl_Role_Registry::get()
     *
     * @return boolean
     */
    public function isAllowed($role = null, $resource = null, $privilege = null)
    {
        // avoid to throw exception when role is not found => instead return false
        if (!$this->hasRole($role))
        {
            return false;
        }

        // avoid to throw exception when resource is not found => instead return true
        if ($resource !== null && !$this->has($resource))
        {
            return true;
        }

        return parent::isAllowed($role, $resource, $privilege);
    }

}
