<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.pluginManager.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * WCM Plugin Manager
 */
class wcmPluginManager
{
    /**
     * Hash table of wcmPluginConfigurations by type
     * E.g. array('search' => array(...), 'import' => array(...))
     */
    private $_pluginConfigs = array();

    /**
     * Folders where plugins are located
     * Hash table of folders
     * E.g. array('search' => array(...), 'import' => array(...))
     */
    private $_pluginFolders = array();

    public function updateCache($pluginType)
    {
        $content = '<?php return ' . var_export($this->_pluginConfigs[$pluginType], true) . ';';
        $filename = WCM_BACKOFFICE_CONF_DIR . '/plugins/' . $pluginType . '.php';
        file_put_contents($filename, $content);
    }

    /**
     * initConfig
     *
     * @param mixed $pluginType
     * @return void
     */
    public function initConfig($pluginType)
    {
        $filename = WCM_BACKOFFICE_CONF_DIR . '/plugins/' . $pluginType . '.php';

        if (!file_exists($filename))
        {
            $config = wcmConfig::getInstance();

            $folders = array();
            for ($i = 0; $folder = $config['wcm.plugins.' . $pluginType . '.folders.folder.' . $i]; $i++)
            {
                $folders[] = $folder;
            }
            file_put_contents($filename, $this->buildCache($folders));
        }

        $this->_pluginConfigs[$pluginType] = require($filename);
    }

    /**
     * Returns a wcmPluginConfig associated a specific plugin
     */
    public function getPluginConfig($pluginType, $pluginId)
    {
        $array = getArrayParameter($this->_pluginConfigs, $pluginType, null);

        if (null !== $array)
        {
            return getArrayParameter($array, $pluginId, null);
        }

        return null;
    }

    /**
     * Add a new plugin config
     */
    public function addPluginConfig($pluginType, $pluginConfig)
    {
        $id = $pluginConfig->id;
        $this->_pluginConfigs[$pluginType][$id] = $pluginConfig;
    }

    /**
     * Remove a plugin config
     */
    public function removePluginConfig($pluginType, $pluginId)
    {
        unset($this->_pluginConfigs[$pluginType][$pluginId]);
    }

    /**
     * Activate a plugin
     */
    public function switchActivate($pluginType, $pluginId)
    {
        if (array_key_exists($pluginType, $this->_pluginConfigs))
        {
            $this->_pluginConfigs[$pluginType][$pluginId]['general']['activated'] = !$this->_pluginConfigs[$pluginType][$pluginId]['general']['activated'];
            return true;
        }

        return false;
    }

    /**
     * Deactivate plugin
     */
    public function deactivatePlugin($pluginType, $pluginId)
    {
        $pluginConfig = $this->getPluginConfig($pluginType, $pluginId);

        if (null !== $pluginConfig)
        {
            $pluginConfig['general']['activated'] = false;
            return true;
        }

        return false;
    }

    /**
     * Returns an array of plugins of a specific type
     */
    public function getPluginsList($pluginType, $ignoreDisabled = true)
    {
        $result = array();
        if (array_key_exists($pluginType, $this->_pluginConfigs))
        {
            if (true === $ignoreDisabled)
            {
                foreach ($this->_pluginConfigs[$pluginType] as $pluginName => $plugin) 
                {
                    if ($plugin['general']['activated'])
                    {
                        $result[$pluginName] = $plugin;
                    }
                }
            } 
            else
            {
                $result = $this->_pluginConfigs[$pluginType];
            }
        }

        return $result;
    }

    /**
     * Register folders to be scanned
     */
    public function registerFolders($pluginType, array $folders)
    {
        $this->_pluginFolders[$pluginType] = $folders;
    }

    /**
     * buildCache
     *
     * @param array $folders
     * @return void
     */
    public function buildCache(array $folders = array())
    {
        return '<?php return ' . var_export($this->parseFolders($folders), true) . ';';
    }

    /**
     * parseFolders
     *
     * @param array $folders
     * @return void
     */
    public static function parseFolders(array $folders = array())
    {
        $result = array();

        foreach ($folders as $folder)
        {

            if (!is_dir($folder)) 
            {
                continue;
            }

            $dir = new RecursiveDirectoryIterator($folder);

            foreach (new RecursiveIteratorIterator($dir, RecursiveIteratorIterator::SELF_FIRST) as $file)
            {
                if ($file->getFilename() !== 'info.ini')
                {
                    continue;
                }

                $array = parse_ini_file($file->getPathname(), true);
                $result[$array['general']['id']] = $array;
            }
        }

        return $result;
    }
}
