<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.search.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

final class wcmSearch
{
    /**
     * The last class-wide error message (if any).
     * @var string
     */
    private static $lastErrorMsg = null;

    /**
     * The search plugin configuration.
     * @var SimpleXMLElement
     */
    private static $plugins = null;

    /**
     * The current plugin-specific search instances.
     *
     * @var array
     */
    private static $instances = array();

    /**
     * Returns the last global error message
     *
     * @return string The last global error message
     */
    public static function getGlobalErrorMsg()
    {
        return self::$lastErrorMsg;
    }

    /**
     * Gets a search instance using a given plugin. If the
     * $pluginName parameter is null, uses the plugin specified in the
     * WCM configuration.
     *
     * Creates a given plugin-specific search instance only
     * once per session.
     *
     * @param string|null $pluginName The name of the plugin to use (default is null)
     *
     * @return wcmIsearch|null The search instance, or null on error
     */
    public static function getInstance($pluginName = null)
    {
        // Select the plugin to use from the configuration if not given
        if (!$pluginName)
        {
            $config = wcmConfig::getInstance();
            $pluginName = $config['wcm.plugins.search.default'];
        }
        if (!$pluginName)
        {
            self::$lastErrorMsg = sprintf(_INVALID_SEARCH_PLUGIN, $pluginName);
            wcmProject::getInstance()->logger->logError(self::$lastErrorMsg);
            return null;
        }

        // Create the plugin-specific search instance if it
        // does not already exist
        if (!isset(self::$instances[$pluginName]))
        {
            // Load the search plugin configuration if not already loaded
            if (self::$plugins === null)
            {

                $pm = new wcmPluginManager();
                $pm->initConfig('search');
                $plugins = $pm->getPluginsList('search');

                if (empty($plugins) || !array_key_exists($pluginName, $plugins))
                {
                    self::$lastErrorMsg = sprintf(_INVALID_SEARCH_PLUGIN_CONFIG, $pluginName);
                    wcmProject::getInstance()->logger->logError(self::$lastErrorMsg);
                    return null;
                }
                self::$plugins = $plugins[$pluginName];
            }

            $className = $plugins[$pluginName]['general']['className'];

            // Create the search instance and its plugin
            $instance = new wcmSearch();
            $instance->plugin = new $className($instance);
            self::$instances[$pluginName] = $instance;
        }

        return self::$instances[$pluginName];
    }

    /**
     * Escapes Lucene special characters in a given query string.
     *
     * @param string $queryString The query string to escape
     *
     * @return string The escaped query string
     */
    public static function escapeQuery($queryString)
    {
        return preg_replace('/(["\\[\\]\\\\&|!(){}^~*?:])/', '\\\\$1', $queryString);
    }

    /**
     * Unescapes Lucene special characters in a given query string.
     *
     * @param string $queryString The query string to unescape
     *
     * @return string The unescaped query string
     */
    public static function unescapeQuery($queryString)
    {
        return preg_replace('/\\\\(["\\[\\]\\\\&|!(){}^~*?:+-])/', '$1', $queryString);
    }

    /**
     * Quotes a query string value if it contains whitespace
     * characters.
     *
     * @param string $value The value to quoted
     *
     * @return string The quoted value
     */
    public static function quoteQueryValue($value)
    {
        $value = trim(trim(trim($value), '"'));

        $quotedValue = $value;
        if (strpos($value, ' ') !== false)
        {
            $quotedValue = '"' . $value . '"';
        }
        return $quotedValue;
    }

    /**
     * The search plugin used.
     *
     * @var wcmISearchPlugin
     */
    private $plugin = null;

    /**
     * The current search ID.
     *
     * @var string
     */
    private $searchId = null;

    /**
     * Constructor
     *
     * The constructor is private to enforce the use of the static
     * getInstance method.
     */
    private function __construct()
    {
    }

    /**
     * Deindex all objects of a given class.
     *
     * @param string $className The name of the class for which to deindex
     *
     * @return bool False on failure, True on success
     */
    public function deindexObjects($className)
    {
        return $this->plugin->deindexObjects($className);
    }

    /**
     * Deindex a object
     *
     * @param wcmObject $object  Object to remove
     *
     * @return bool False on failure, True on success
     */
    public function deindexObject(wcmObject $object)
    {
        return $this->plugin->deindexObject($object);
    }

    /**
     * Index a object in the search table
     *
     * @param wcmObject $object  Object to index
     *
     * @return bool False on failure, True on success
     */
    public function indexObject(wcmObject $object)
    {
        return $this->plugin->indexObject($object);
    }

    /**
     * Returns the last error message
     *
     * @return string The last error message
     */
    public function getErrorMsg()
    {
        return $this->plugin->getErrorMsg();
    }

    /**
     * Count the number of elements indexed for the sourceClass
     *
     * @param string $className class to search for (or null for all)
     *
     * @return int Number of indexed elements of a class
     */
    public function getObjectCount($className = null)
    {
        return $this->plugin->getObjectCount($className);
    }

    /*
     * SEARCHING METHODS
     */

    /**
     * {@inheritdoc}
     *
     * @param string The new search ID
     */
    public function setSearchId($searchId)
    {
        $this->searchId = $searchId;
    }

    /**
     * {@inheritdoc}
     *
     * @return string|null The current search ID, or null if none was
     *                       set or the search method was not called
     *                       or failed
     */
    public function getSearchId()
    {
        return $this->searchId;
    }

    /**
     * Initiates a new search given a unique ID, a query, and sorting
     * criteria.
     *
     * The $query parameter may specify:
     *
     * - a string, in which case its value is expected to conform to
     *   the Lucene query syntax; or
     *
     * - An assoc. array of name-value pairs in which each name refers
     *   to a valid search index.
     *
     * The assoc. array form of the query is equivalent to a Lucene
     * query in which each name-value pair forms a term and the terms
     * are joined by the "AND" operator, thus:
     *
     *   name1:value1 AND name2:value2 AND ...
     *
     * If non-null, the $sortingCriteria parameter is expected to
     * follow the SQL "order by" syntax with the default sort order
     * being "ASC", like so:
     *
     * name1 [ASC|DESC], name2 [ASC|DESC], ...
     *
     * @param string      $searchId        The unique ID to identify the search
     * @param mixed       $query           The Lucene query or assoc. array of name-value pairs
     * @param string|null $sortingCriteria The sorting criteria (default is null)
     *
     * @return int|null The number of search results, or null on failure
     */
    public function search($searchId, $query, $sortingCriteria = null)
    {
        if (!$searchId)
        {
            self::$lastErrorMsg = sprintf(_MISSING_SEARCH_ID);
            wcmProject::getInstance()->logger->logError(self::$lastErrorMsg);
            return null;
        }
        $this->setSearchId($searchId);

        return $this->plugin->search($query, $sortingCriteria);
    }

    /**
     * Gets the current query.
     *
     * @return string The current query, or null if none
     */
    public function getQuery()
    {
        return $this->plugin->getQuery();
    }

    /**
     * Gets the current sorting criteria.
     *
     * @return string The current sorting criteria, or null if none
     */
    public function getSortingCriteria()
    {
        return $this->plugin->getSortingCriteria();
    }

    /**
     * Gets the native query initialized by the last call to the
     * initSearch method.
     *
     * @return string The native query
     */
    public function getNativeQuery()
    {
        return $this->plugin->getNativeQuery();
    }

    /**
     * Get a range of documents after a successful call to the
     * initSearch method.
     *
     * @param int  $from The index of the first document to get
     * @param int  $to   The index of the last document to get
     * @param bool $xml  Whether to return an XML or an array of objects
     *
     * @return mixed|null Raw XML or an array containing requested objects
     *                    or null if the initSearch method failed or was not called
     */
    public function getSearchResults($from, $to, $xml = true)
    {
        return $this->plugin->getSearchResults($from, $to, $xml);
    }

    /**
     * Gets the facet values for a given list of facets.
     *
     * If the $searchId parameter is non-null, only those facet values
     * present in the corresponding result set are returned.
     *
     * The $facets parameter is expected to be an assoc. array of:
     * facet name => maximum number of values for that facet
     *
     * If the $facets parameter specifies more than one facet, then:
     *
     * - first, merges the facet values for the given facet names,
     *   keeping only the top-N values for each facet, where N is the
     *   assoc. value for that facet (default is 512; null for no
     *   limit); and
     *
     *   and
     *
     * - second, keeps only the top-M values in the merged result,
     *   where M is the value of the $maxValues parameter (default is
     *   512; null for no limit).
     *
     * If the $sort parameter is true, sorts the list of facet values
     * in decreasing order by occurence count before returning it.
     *
     * @param array       $facets    The assoc. array of facet name => maximum number of values
     * @param string|null $searchId  The reference search ID (default is null)
     * @param int|null    $maxValues The total maximum number of values (default is 512)
     * @param bool        $sort      Whether to sort the list of facet values (default is true)
     *
     * @return array|null Assoc. array of facet value => wcmSearchFacetValue, or null on failure
     */
    public function getFacetValues(array $facets, $searchId = null, $maxValues = 512, $sort = true)
    {
        if (!$searchId)
        {
            $searchId = uniqid('wcmSearch_getFacetValues_');
            if ($this->search($searchId, '') === null)
            {
                return null;
            }
        }

        $facetValues = array();

        if ($facets)
        {
            $facetInfos = wcmSearchFacetValue::getFacetInfos();
            foreach ($facets as $facet => $partialMaxValues)
            {
                // Facet names are case-insensitive
                $facet = strtolower($facet);

                // Attempt to get "small set size" facet values first
                $partialFacetValues = wcmSearchFacetValue::getFacetValues($facet);
                if ($partialFacetValues)
                {
                    foreach ($partialFacetValues as $value => $facetValue)
                    {
                        $facetValue->count = $this->plugin->getFacetValueCount($facetValue->index, $facetValue->value, $searchId);
                    }
                }
                elseif (isset($facetInfos[$facet]))
                {
                    $facetInfo = $facetInfos[$facet];
                    $index = $facetInfo->index;

                    $rawFacetValues = $this->plugin->getRawFacetValues($index, $searchId);
                    if ($rawFacetValues === null)
                    {
                        return null;
                    }
                    if ($rawFacetValues)
                    {
                        foreach ($rawFacetValues as $value => $count)
                        {
                            $partialFacetValues[$facet . ':' . $value] = new wcmSearchFacetValue($facet, $value, null, $index, $facetInfo->searchIndex, $count);
                        }
                    }
                }

                if ($partialFacetValues)
                {
                    // Possibly sort the facets in decreasing order by count
                    if ($sort)
                    {
                        $callback = array('wcmSearchFacetValue', 'compareFacetValuesByCount');
                        uasort($partialFacetValues, $callback);
                    }

                    // Keep only the top-$partialMaxValues values
                    if ($partialMaxValues)
                    {
                        $partialFacetValues = array_slice($partialFacetValues, 0, $partialMaxValues, true);
                    }
                    // Merge facet values
                    $facetValues = array_merge($facetValues, $partialFacetValues);
                }
            }

            // Keep only the top-$maxValues values
            if ($maxValues)
            {
                $facetValues = array_slice($facetValues, 0, $maxValues, true);
            }
        }

        return $facetValues;
    }

    /**
     * Reindex the content from the main database
     *
     * @param string      $className The class of objects to reindex
     * @param string|null $where     Optional where clause (default is null)
     * @param string|null $orderby   Optional order clause (default is null)
     * @param int         $offset    Optional offset of first row (default is 0)
     * @param int         $limit     Optional maximum number of returned rows
     *                                   (default is 0 to return all rows)
     *
     * @return bool True on success, false otherwise
     */
    public function indexObjects($className, $where = null, $orderBy = null, $offset = 0, $limit = null)
    {
        $enum = new wcmEnumeration($className, $where, $orderBy, $offset, $limit);
        foreach ($enum as $object)
        {
            $this->indexObject($object);
        }
        return true;
    }

    /**
     * Do a commit on the plugin search
     *
     * @return bool True on success, false otherwise
     */
    public function commit()
    {
        return $this->plugin->commit();
    }

    /**
     * Set auto commit value (if needed)
     * @param Boolean $value Boolean flag for the auto commit value
     * 
     * @return Boolean True on success, false otherwise
     */
    public function setAutoCommit($value)
    {
        return $this->plugin->setAutoCommit($value);
    }
}
