<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.searchFacetValue.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * Represents a search facet value.
 *
 */
class wcmSearchFacetValue
{
    /**
     * Information about available facets.
     *
     * @see self::getFacetInfos()
     * @var array
     */
    protected static $facetInfos = null;

    /**
     * Gets information about available facets.
     *
     * Information about individual facets are returned in
     * wcmSearchFacetValue objects with empty value and text
     * properties.
     *
     * @return array Assoc. array of: facet name => wcmSearchFacetValue
     */
    public static function getFacetInfos()
    {
        if (self::$facetInfos === null)
        {
            self::$facetInfos = array();
            $config = simplexml_load_file(WCM_BACKOFFICE_CONF_DIR . '/search/facets.xml');
            for ($offset = 0; ; $offset++)
            {
                if (!isset($config->facet[$offset]->name))
                {
                    break;
                }

                // Lowercase facet name et al. to allow case-insensitivity
                $name = strtolower($config->facet[$offset]->name);
                $index = strtolower($config->facet[$offset]->index);
                $searchIndex = strtolower($config->facet[$offset]->searchIndex);

                self::$facetInfos[$name] = new wcmSearchFacetValue($name, null, null, $index, $searchIndex);
            }
        }

        return self::$facetInfos;
    }

    /**
     * Gets the raw facet values (ie., without any occurence count)
     * for a given facet.
     *
     * This function only supports the following facets:
     *
     * - classname
     * - channelid
     * - modifiedat
     *
     * @param string $facet The facet name
     *
     * @return array Assoc of raw facet value => wcmSearchFacetValue
     */
    public static function getFacetValues($facet)
    {
        // Facet names are case-insensitive
        $facet = strtolower($facet);

        $facetValues = array();

        $facetInfos = self::getFacetInfos();
        if (isset($facetInfos[$facet]))
        {
            $facetInfo = $facetInfos[$facet];
            $index = $facetInfo->index;
            $searchIndex = $facetInfo->searchIndex;

            switch ($facet)
            {
                case 'classname':
                    $bizlogic = new wcmBizlogic();
                    $classes = $bizlogic->getBizclasses();
                    if ($classes)
                    {
                        foreach ($classes as $className => $class)
                        {
                            $facetValues[$facet . ':' . $className] = new wcmSearchFacetValue(
                            $facet,
                            $className,
                            getConst($class->name),
                            $index,
                            $searchIndex
                            );
                        }
                    }
                    break;

                case 'channelid':
                    $objects = wcmObject::getObjects('channel');
                    if ($objects)
                    {
                        foreach ($objects as $object)
                        {
                            $facetValues[$facet . ':' . $object->id] = new wcmSearchFacetValue($facet, $object->id, $object->title, $index, $searchIndex);
                        }
                    }
                    break;

                case 'modifiedat':
                    $today = mktime(0, 0, 0, date('m'), date('d'), date('Y'));
                    $facetValues = array(
                            $facet . ':' . $today   => new wcmSearchFacetValue($facet, $today, _TODAY, $index, $searchIndex),
                            $facet . ':' . -1       => new wcmSearchFacetValue($facet, -1, _YESTERDAY, $index, $searchIndex),
                            $facet . ':' . -7       => new wcmSearchFacetValue($facet, -7, _SINCE_7DAYS, $index, $searchIndex),
                            $facet . ':' . -30      => new wcmSearchFacetValue($facet, -30, _SINCE_1MONTH, $index, $searchIndex),
                            $facet . ':' . -90      => new wcmSearchFacetValue($facet, -90, _SINCE_3MONTHES, $index, $searchIndex),
                            $facet . ':' . -365     => new wcmSearchFacetValue($facet, -365, _SINCE_1YEAR, $index, $searchIndex),
                            );
                    break;
            }
        }

        return $facetValues;
    }

    /**
     * Compares two facet values in decreasing order of count.
     *
     * @param wcmSearchFacetValue $facetValue1 The first facet value
     * @param wcmSearchFacetValue $facetValue2 The second facet value
     *
     * @return int The difference $facetValue2->count - $facetValue1->count
     */
    public static function compareFacetValuesByCount($facetValue1, $facetValue2)
    {
        return $facetValue2->count - $facetValue1->count;
    }

    /**
     * The facet name.
     *
     * @var string
     */
    public $name;

    /**
     * The facet value per se.
     *
     * @var mixed
     */
    public $value;

    /**
     * The human-readable representation of the facet value.
     *
     * @var string
     */
    public $text;

    /**
     * The facet value index, ie., the index from which to get the facet values.
     *
     * @var string
     */
    public $index;

    /**
     * The facet search index, ie., the index with which to perform a
     * search on a particular facet value.
     *
     * @var string
     */
    public $searchIndex;

    /**
     * The occurence count of the facet value.
     *
     * @var int
     */
    public $count;

    /**
     * Constructor
     *
     * @param string      $name        The facet name
     * @param mixed       $value       The facet value per se
     * @param string|null $text       The human-readable text (default is $value)
     * @param string|null $index       The facet value index (default is $name)
     * @param string|null $searchIndex The facet search index (default is $index or its default)
     * @param int         $count       The facet value occurence count (default is 0)
     */
    public function __construct($name, $value, $text = null, $index = null, $searchIndex = null, $count = 0)
    {
        $this->name = $name;
        $this->value = $value;
        $this->text = $text ? $text : $this->value;
        $this->index = $index ? $index : $this->name;
        $this->searchIndex = $searchIndex ? $searchIndex : $this->index;
        $this->count = $count;

        // Small hack: the text of a "category" facet should not
        // include the IPTC code
        if ($this->name == 'category')
        {
            $parts = explode(' - ', $this->text, 2);
            if (count($parts) == 2)
            {
                $this->text = $parts[1];
            }
        }
    }
}
