<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.object.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The object class is the parent abstract class
 * of all WCM database objects
 */
abstract class wcmObject implements wcmEnumerable
{
    /**
     * Gets an array of objects of a given class and possibly matching
     * specific WHERE and ORDER BY clauses.
     *
     * @param string $className The class of objects to return
     * @param string $where     An optional WHERE clause to apply (default is null)
     * @param string $orderBy   An optional ORDER BY clause (default is null)
     *
     * @return array An assoc. array of object ID => object instance, or null on error
     */
    public static function getObjects($className, $where = null, $orderBy = null)
    {
        $objects = array();
        $enum = new wcmEnumeration($className, $where, $orderBy);
        
        foreach ($enum as $object)
        {
            $objects[$object->id] = $object;
        }
        return $objects;
    }

    /**
     * Editable state
     * @var int
     */
    private $editable = -1;

    /**
     * Unique identifier
     * @var int
     */
    public $id;

    /**
     * Memory storage used for serialization purpose
     * @var array
     */
    protected $serialStorage = array();

    /**
     * The database corresponding to the sysclass or bizclass
     * @var wcmDatabase
     */
    protected $database;

    /**
     * The table name where to store the object (or null for disconnected object)
     * @var string
     */
    protected $tableName;

    /**
     * The last error message that occurs during execution of a method
     * @var string
     */
    protected $lastErrorMsg;


    //
    // Protected members used for enumeration (wcmEnumerable interface)
    //

    /**
     * The query that correspond to the current enumeration (see beginEnum() method)
     * @var string
     */
    protected $enumQuery;

    /**
     * The query that correspond to the computation of current enumeration count  (see enumCount() method)
     * @var string
     */
    protected $enumCountQuery;

    /**
     * The resulset that correspond to the current enumeration (see beginEnum() method)
     * $var wcmAbstractResultSet
     */
    protected $enumResultSet;

    /**
     * The total number of results corresponding to the current enumeration (see beginEnum() method)
     * @var int
     */
    protected $enumTotalCount;

    /**
     * Tell if this object is lock in "optimistic" mode (vs. "pessimistic")
     * @var bool
     */
    protected $isLockOptimistic;

    /**
     * last lock info computed by isLocked() or isEditable() or getLockInfo()
     * @var wcmLock
     */
    protected $lockInfo;

    /**
     * Type of object
     *
     * Some objects may have extenstion classes, but should still be considered to be
     * of the parent type.
     *
     * @var string
     */
    protected $objectType;

    /**
     * Constructor
     *
     * Can be overloaded/supplemented by the child class
     *
     * @param int $id Optional id (used to refresh object)
     *
     */
    public function __construct($id = null)
    {
        $this->setDefaultValues();
        $this->getDatabase();
        if ($id)
        {
            $this->refresh($id);
        }
    }

    /**
     * Get the type of object.
     *
     * @param string $argClass Class name to check, if null will use the instance's class name
     * @return string
     */
    public function getType($argClass = null)
    {
        $id = uniqid();
        if (!$this->objectType)
        {
            $type = isset($argClass) ? $argClass : get_class($this);
            // The upper most object names are either:
            // 'wcmSysobject' OR 'bizobject' OR 'wcmObject'
            // check in this order:
            // 'bizobject' then 'wcmSysobject' and last 'wcmObject'
            // so drill down still we find one class above one of those two
            $parent = get_parent_class($type);

            if ($parent != 'bizobject' && $parent != 'wcmSysobject' && $parent != 'wcmObject' && $parent != 'wcmSecureObject')
            {
                $type = $this->getType($parent);
            }

            $this->objectType = $type;
        }

        return $this->objectType;
    }

    /**
     * Set all initial values of an object
     * This method is invoked by the constructor
     */
    protected function setDefaultValues()
    {
        $this->lastErrorMsg = null;
        $this->enumQuery = null;
        $this->enumResultSet = null;
        $this->enumTotalCount = -1;
        $this->serialStorage = array();
        $this->id = 0;
    }

    /**
     * Explicit serialization of object
     *
     * @return string Serialized object
     */
    public function serialize()
    {
        // Serialize all relations in the serial storage!
        $this->serialStorage['relations'] = bizrelation::archiveBizobject($this);

        // We assume that serializable stuff has been added to our
        // magic array 'serialStorage'
        $serial = array();
        foreach (getPublicProperties($this) as $key => $val)
        {
            $serial[$key] = serialize($val);
        }
        $serial['tableName'] = serialize($this->tableName);
        $serial['serialStorage'] = serialize($this->serialStorage);

        return serialize($serial);
    }

    /**
     * Explicit unserialization of object
     *
     * @param string $serialized Serialized object
     */
    public function unserialize($serialized)
    {
        // Rebuild database and tableName
        $this->getDatabase();
        $this->setDefaultValues();


        // Unserialize content
        $serial = unserialize($serialized);
        foreach ($serial as $key => $val)
        {
            $this->$key = unserialize($val);
        }
        unset($serial);

        // Restore dependencies
        $this->restoreDependencies();
    }

    /**
     * Unserialize some properties if needed
     * This method is called by nextEnum() and refresh()
     * and may be invoked after a bindArrayToObject() call
     */
    public function unserializeProperties()
    {
    }

    /**
     * Restore dependencies
     * This method is called by restore/archive methods and is used to restore in
     * the database all dependencies (e.g. relations, sub-parts, etc...)
     *
     */
    protected function restoreDependencies()
    {
        bizrelation::restoreBizobject($this, getArrayParameter($this->serialStorage, 'relations'));
    }

    /**
     * Get the database used to store/fetch object
     *
     * IMPORTANT: This method is called by the constructor
     * Therefore, you may need to override this method to
     * define your database and tableName the first time.
     *
     * @return wcmDatabase Database used to store/fetch object
     */
    public function getDatabase()
    {
        if (!$this->database)
        {
            throw new LogicException($this->getType() . ' fatal error: you must define your database and tableName when extending class wcmObject');
        }

        return $this->database;
    }


    /**
     * Returns unique instance of project
     *
     * @return wcmProject Instance of wcmProject
     */
    public function getProject()
    {
        return wcmProject::getInstance();
    }

    /**
     * Returns name of table in database where object is stored (or null if object is disconnected)
     *
     * @return string Name of table where object is stored (or null if object is disconnected)
     */
    public function getTableName()
    {
        return $this->tableName;
    }

    /**
     * Set the last error message of an object
     *
     * @param string $message Error message to remember
     */
    public function setErrorMsg($message)
    {
        $this->lastErrorMsg = $message;
    }

    /**
     * Returns the last error message
     *
     * @return string Error message
     */
    public function getErrorMsg()
    {
        return $this->lastErrorMsg;
    }

    /**
     * Update all relations of current object
     *
     * @param string $kind Kind of relation to update (e.g. '_br_{kind}')
     * @param array  $relations Array containing all new relations (as an assoc array)
     */
    public function updateRelations($kind, $relations)
    {
        $this->serialStorage['_list'][] = $kind;
        $this->serialStorage[$kind] = $relations;
    }

    /**
     * Returns the class name of the current object
     *
     * @return string Classname of current object
     */
    public function getClass()
    {
        return get_class($this);
    }

    /**
     * Expose 'className' as a key in tnhe assoc array
     *
     * @param bool $toXML TRUE if getAssocArray is called by toXML()
     *
     * @return string The object class name
     */
    public function getAssoc_className($toXML = false)
    {
        return $this->getClass();
    }

    /**
     * Expose 'objectType' as a key in the assoc array
     *
     * @param $toXML
     * @return string
     */
    public function getAssoc_objectType($toXML = false)
    {
        return $this->getType();
    }

    /**
     * Exposes 'related' to the getAssocArray (all object with IS_RELATED_TO relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_related($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_RELATED_TO);
    }

    /**
     * Exposes 'partOf' to the getAssocArray (all object with IS_PART_OF relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_partOf($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_PART_OF);
    }

    /**
     * Exposes 'composedOf' to the getAssocArray (all object with IS_COMPOSED_OF relationship)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_composedOf($toXML = false)
    {
        return $this->getAssoc_relations($toXML, bizrelation::IS_COMPOSED_OF);
    }

    /**
     * Exposes 'relations' to the getAssocArray (all objects in relation with current bizobject)
     *
     * @param bool $toXML TRUE if method is called in the context of toXML()
     *
     * @return array An array of the related bizobject's getAssocArray
     */
    public function getAssoc_relations($toXML = false, $relationKind = null)
    {
        if ($toXML)
        {
            return null;
        }

        $objects = array();
        foreach (bizrelation::getBizobjectRelations($this, $relationKind, null, $toXML) as $object)
        {
            $obj = wcmObjectFactory::create($object['destinationClass']);
            $obj->refresh($object['destinationId']);
            $objects[] = $obj->getAssocArray($toXML);
        }
        return $objects;
    }

    /**
     * Returns all relations of the bizobject
     *
     * @return array An array of wcmBizRelation objects
     */
    public function getRelations()
    {
        $relations = array();
        $where = 'sourceClass=\'' . $this->getType() . '\' AND sourceId=' . $this->id;

        $enum = new wcmEnumeration('bizrelation', $where, 'validityDate, rank');
        foreach ($enum as $relation)
        {
            $relations[] =  $relation;
        }

        return $relations;
    }

    /**
     * Returns an associative array containing public properties and their values
     * and additional 'dynamic' properties computed from public methods named 'getAssoc_XXX()'
     *
     * @param bool $toXML TRUE if getAssocArray should returns methods used in the context of toXML()
     *                    By default, this parameter is set to false.
     *
     * @return wcmObjectAssocArray An instance of dynamic ArrayAcces/Iterator class
     */
    public function getAssocArray($toXML = false)
    {
        return new wcmObjectAssocArray($this, $toXML);
    }

    /**
     * Binds an assoc array to this object
     *
     * @param array  $assocArray (or null to ignore bindings)
     *
     * @return bool true on success, false otherwise
     */
    public function bind($assocArray = null)
    {
        // Clear last error message
        $this->lastErrorMsg = '';
        if (!$assocArray)
        {
            return true;
        }

        return bindArrayToObject($assocArray, $this);
    }

    /**
     * Load or refresh object content
     *
     * @param int $id optional argument, if not specifed current id is used
     *
     * @return true on success, false otherwise
     */
    public function refresh($id = null)
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        // Refresh object itself?
        if (null === $id)
        {
            $id = $this->id;
        }

        // Clear all "public" properties before refresh
        foreach (getPublicProperties($this) as $key => $value)
        {
            $this->$key = is_bool($value) ? false : null;
        }

        // Restore default
        $this->setDefaultValues();
        $this->id = 0;

        // Ignore new object
        if (!$id)
        {
            $this->lastErrorMsg = $this->getType() . ':: refresh failed : No id given';
            return false;
        }

        // Load values from database and bind them to current object
        if (!$this->database->bindSysObject($this, $id))
        {
            $this->lastErrorMsg = $this->getType() . ':: refresh failed : ' . $this->database->getErrorMsg();
            return false;
        }

        $this->unserializeProperties();
        return true;
    }

    /**
     * Check validity of object
     *
     * A generic method which can (should ?) be overloaded by the child class
     *
     * @return bool true when object is valid
     *
     */
    public function checkValidity()
    {
        // Clear last error message
        $this->lastErrorMsg = '';

        return true;
    }

    /**
     * Index object in search engine
     *
     * @return bool true on success, false on failure
     */
    public function index()
    {
        return true;
    }

    /**
     * De-index object from search engine
     *
     * @return bool true on success, false on failure
     */
    public function deindex()
    {
        return true;
    }

    /**
     * This callback is invoked just after a successful call to 'store' method
     */
    protected function onStore()
    {
        /**
         * Save new relations?
         *
         * Check is serialStorage contains specific entries
         * '_list' => an array of list to save (e.g _br_{kind} to save relation of kind {kind})
         * '_br_{kind}' => an array of array containing bizrelation properties
         */
        $newLists = getArrayParameter($this->serialStorage, '_list');

        if (is_array($newLists))
        {
            foreach ($newLists as $listId)
            {

                $listInfos = explode('_', $listId);
                if ('br' === getArrayParameter($listInfos, 1) && $kind = getArrayParameter($listInfos, 2))
                {

                    // remove previous relations
                    $relation = new bizrelation();
                    $relation->sourceClass = $this->getType();
                    $relation->sourceId = $this->id;

                    $validity_date_set = false;

                    // add new relations (picked-up from serialStorage)
                    $newRelations = getArrayParameter($this->serialStorage, $listId, array());

                    //remove only the dates that were seen (do not remove all) - applies for management by date
                    foreach (getArrayParameter($newRelations, 'destinationId', array()) as $relationKey => $destId)
                    {
                        foreach ($newRelations as $prop => $value)
                        {
                            if ('validityDate' === $prop)
                            {
                                $relation->removeByValidityDate($newRelations[$prop][$relationKey], false);
                                $validity_date_set = true;
                            }
                        }
                    }

                    //if we're not in date management (regulare relation) remove all relations by kind
                    if (false === $validity_date_set)
                    {
                        $relation->removeByKind($kind, false);
                    }

                    $relation->rank = 0;
                    $relation->kind = $kind;

                    foreach (getArrayParameter($newRelations, 'destinationId', array()) as $relationKey => $destId)
                    {
                        $relation->id = 0;
                        $relation->rank++;

                        if (0 != $newRelations['destinationId'][$relationKey])
                        {
                            foreach ($newRelations as $prop => $value)
                            {
                                //if rank is set, it will overide the previous assigned rank
                                $relation->$prop = $newRelations[$prop][$relationKey];
                            }
                            $relation->save();
                        }

                    }
                }
                unset($this->serialStorage[$listId]);
            }
        }
        unset($this->serialStorage['_list']);

    }

    /**
     * Store current object into database
     *
     * @return bool true on success or an error message (string)
     */
    protected function store()
    {
        // Update or insert object in database
        $result = $this->database->storeSysObject($this);
        if ($result === false || $result < 0)
        {
            $this->lastErrorMsg = $this->getType($this) . ' ::store failed : ' . $this->database->getErrorMsg();
            return false;
        }
        $this->onStore();

        return true;
    }

    /**
     * Deletes current object from database
     *
     * @return bool true on success or false on failure
     */
    public function delete()
    {
        // Check lock
        if (!$this->hasLock() && !$this->checkLock('delete'))
        {
            $this->lastErrorMsg = $this->lastErrorMsg = $this->getType() . '::delete failed : object ' . $this->id . ' is locked';
            return false;
        }

        // Remove sysobject from database
        if (!$this->database->deleteSysObject($this))
        {
            $this->lastErrorMsg = $this->getType() . '::delete failed : ' . $this->database->getErrorMsg();
            return false;
        }

        // Remove biz relations
        bizrelation::removeBizobject($this);

        // De-index
        $this->deindex();

        // Unlock
        $this->unlock();

        // Clear all "public" properties
        foreach (getPublicProperties($this) as $key => $value)
        {
            if ('_' != $key[0])
            {
                $this->$key = null;
            }
        }

        return true;
    }

    /**
     * Save and index object (bind, checkValidity, store and index)
     *
     * @param array $source An assoc array for binding to class vars (or null)
     *
     * @return bool true on success, false otherwise
     */
    public function save($source = null)
    {
        // Check lock
        if (false === $this->checkLock('save'))
        {
            return false;
        }
        
        // Bind, check and store
        if (!$this->bind($source))
        {
            return false;
        }
        if (!$this->checkValidity())
        {
            return false;
        }
        if (!$this->store())
        {
            return false;
        }
        
        // As we cannot ensure transactional mode between store and index,
        // even if the indexation may fail the method will return true...
        $this->index();

        return true;
    }

    /**
     * Fill-in public properties of the sysobject from an XML fragment
     * as returned by toXML() method.
     *
     * @param string      $xml       XML fragment representing the object
     * @param string $xslFile   XSL xsl file (if defined, apply the XSL transformation)
     * @param string $className Object class name (if root tag is different than actual class name)
     *
     * @return bool True on success, false otherwise
     * @throws Exception        When XML is invalid
     */
    public function initFromXML($xml, $xslFile = null, $className = null)
    {
        // Create XML document
        $domXml = new DOMDocument();
        if (!$domXml->loadXML($xml))
        {
            throw new Exception(_BIZ_INVALID_XML);
        }

        return $this->initFromXMLDocument($domXml, $xslFile, $className);
    }

    /**
     * Fill-in public properties of the sysobject from a valid XML DOMDocument
     * If a XSL is defined, apply the XSL transformation
     *
     * @param DOMDocument $domXml    XML DOMDocument representing the object
     * @param string $xslFile   XSL xsl file (if defined, apply the XSL transformation)
     * @param string $className Object class name (if root tag is different than actual class name)
     *
     * @return bool True on success, false otherwise
     * @throws Exception        When XML is invalid
     */
    public function initFromXMLDocument($domXml, $xslFile = null, $className = null)
    {
        // If XSL, Transform XML document
        if ($xslFile)
        {
            // Create XSL document
            $xslDoc = new DOMDocument();
            $xslDoc->load($xslFile);

            // Create XSLT processor
            $xsltProc = new XSLTProcessor;
            $xsltProc->importStyleSheet($xslDoc);

            // Transform XML document
            $xml = $xsltProc->transformToXML($domXml);

            // Create XML document
            $domXml = new DOMDocument();
            if (!$domXml->loadXML($xml))
            {
                throw new Exception(_BIZ_INVALID_XML);
            }
        }

        if (null === $className)
        {
            $className = $this->getType();
        }

        $domXPath = new DOMXPath($domXml);

        // Expected format is <$className> <$propertyName> $propertyValue </$propertyName> ... </$className>
        foreach (getPublicProperties($this) as $property => $value)
        {
            $xpath = '/' . $className . '/' . $property;
            $node = wcmXML::getXPathFirstNode($domXPath, null, $xpath);
            if ($node)
            {
                // Use custom initialization for this property
                $this->initPropertyFromXMLNode($property, $node);
            }
        }

        return true;
    }

    /**
     * This function can be used to customize the initialisation of a specific property
     * from a XML node (invoked by initFromXML() method)
     *
     * @param string  $property  Property name to initialize
     * @param XMLNode $node      XML node used for initialization
     */
    protected function initPropertyFromXMLNode($property, $node)
    {
        $value = wcmXML::getNodeValue($node);
        if (is_int($this->$property))
        {
            $value = intval($value);
        }
        $this->$property = $value;
    }

    /**
     * Returns an XML representation of current object
     *
     * => the root tag name is equal to the class name
     * => the xml contains a tag for each properties found in getAssocArray(true) method
     * => when a property value is an array, a subtag <key id=""> is generated for each array entry
     *
     * @param array $properties List of properties to return (default is null for all properties)
     *
     * @return string An XML fragment representing current object
     */
    public function toXML($properties = null)
    {
        $className = $this->getType();

        $xml = '<' . $className . '>' . PHP_EOL;
        foreach ($this->getAssocArray(true) as $k => $v)
        {
            if ($properties === null || in_array($k, $properties))
            {
                $xml .= $this->propertyToXML($k, $v);
            }
        }
        $xml .= '</' . $className . '>' . PHP_EOL;
        return $xml;
    }

    /**
     * Returns an XML representation of a property (taken from getAssocArray() method)
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     * @return string XML representation of property
     */
    protected function propertyToXML($propKey, $propValue)
    {
        // create a tag for each property
        $xml = '<' . $propKey . '>';

        if (is_bool($propValue))
        {
            $xml .= ($propValue) ? '1' : '0';
        }
        elseif (is_object($propValue))
        {
            $xml .= serialize($propValue);
        }
        else
        {
            $xml .= wcmXML::xmlEncode($propValue);
        }

        $xml .= '</' . $propKey . '>' . PHP_EOL;

        return $xml;
    }

    /**
     * Fill-in public properties of the sysobject from a JSON fragment
     * as returned by the toJSON method.
     *
     * @param string $json JSON fragment representing the object
     *
     * @return bool True on success, false otherwise
     */
    public function initFromJSON($json)
    {
        // Decode the JSON fragment to an assoc. array
        return $this->initFromJSONAssocArray(json_decode($json, true));
    }

    /**
     * Fill-in public properties of the sysobject from a JSON
     * assoc. array as returned by the json_decode method.
     *
     * @param array $assocArray JSON assoc array
     *
     * @return bool True on success, false otherwise
     */
    protected function initFromJSONAssocArray($assocArray)
    {
        if (!is_array($assocArray))
        {
            return false;
        }
        foreach (getPublicProperties($this) as $property => $assocValue)
        {
            if (isset($assocArray[$property]))
            {
                $this->initPropertyFromJSONAssocValue($property, $assocArray[$property]);
            }
        }

        return true;
    }

    /**
     * This function can be used to customize the initialisation of a
     * specific property from a JSON assoc. value (invoked by the
     * initFromJSON method).
     *
     * @param string $property   Property name to initialize
     * @param mixed  $assocValue JSON assoc. value used for initialization
     */
    protected function initPropertyFromJSONAssocValue($property, $assocValue)
    {
        if (is_int($this->$property))
        {
            $assocValue = intval($assocValue);
        }
        $this->$property = $assocValue;
    }

    /**
     * Returns the JSON representation of the current object.
     *
     * @param array $properties List of properties to return (default is null for all properties)
     *
     * @return string The JSON representation of the current object
     */
    public function toJSON($properties = null)
    {
        $json = '{';
        foreach ($this->getAssocArray(true) as $k => $v)
        {
            if ($properties === null || in_array($k, $properties))
            {
                $json .= $this->propertyToJSON($k, $v) . ',';
            }
        }
        $json = rtrim($json, ',');
        $json .= '}';

        return $json;
    }

    /**
     * Returns the JSON representation of a property (as obtained with
     * the getAssocArray method).
     *
     * @param string $propKey    Property key
     * @param mixed  $propValue  Property value
     *
     * @return string The JSON representation of the property
     */
    protected function propertyToJSON($propKey, $propValue)
    {
        $json = '"' . addslashes($propKey) . '":';
        if (is_object($propValue))
        {
            $json .= '"' . addslashes(serialize($propValue)) . '"';
        }
        else
        {
            $json .= json_encode($propValue);
        }

        return $json;
    }

    /**
     * Returns the number of object of current class matching a query
     *
     * @param string $where Optional where clause to match (default is  null)
     * @param array $params Optional parameters for the where clause (default is null)
     *
     * @return int Total number of object matching the clause or null on error
     */
    public function getCount($where = null, $params = null)
    {
        $sql = 'SELECT COUNT(*) FROM ' . $this->tableName;
        if ($where)
        {
            $sql .= ' WHERE ' . $where;
        }
        return $this->database->executeScalar($sql, $params);
    }

    /**
     * Computes the sql where clause matching foreign constraints
     * => This method must be overloaded by child class
     *
     * @param array $of Assoc Array with foreign constrains (key=className, value=id)
     *
     * @return string Sql where clause matching "of" constraints or null
     */
    public function ofClause($of)
    {
        return null;
    }

    /**
     * Initialize and starts a new enumeration on current wcmSysobject
     *
     * @param string $where     Optional where clause
     * @param string $orderby   Optional order clause
     * @param int    $offset    Optional offset of first row (default is zero)
     * @param int    $limit     Optional maximum number of returned rows (default is zero, means return all rows)
     * @param array $of        Optional assoc Array with foreign constraint (key=className, value=id)
     *
     * @return bool  True on success, false on failure
     */
    public function beginEnum($where = null, $orderby = null, $offset = 0, $limit = 0, $of = null)
    {
        // Ensure to free previous enumeration
        $this->endEnum();

        // Extends where clause with "of" clause
        $ofClause = $this->ofClause($of);
        if ($ofClause != null)
        {
            if ($where)
            {
                $where = '(' . $where . ') and ' . $ofClause;
            }
            else
            {
                $where = $ofClause;
            }
        }

        // First of all, compute total number of items
        if ($where == null || (strcmp($where, $this->enumQuery) != 0))
        {
            // Remember actual query
            $this->enumQuery = $where;

            // Compute number of wcmSysObject matching the query
            $sql = 'SELECT COUNT(*) FROM ' . $this->tableName;

            if ($where != null && $where != '')
            {
                $sql .= ' WHERE ' . $where;
            }

            $this->enumCountQuery = $sql;
        }

        // Prepare sql query
        $sql = 'SELECT * FROM ' . $this->tableName;
        if ($where !== null && $where != '')
        {
            $sql .= ' WHERE ' . $where;
        }
        if ($orderby != null)
        {
            $sql .= ' ORDER BY ' . $orderby;
        }

        // Load resultset
        $this->enumResultSet = $this->database->executeQuery($sql, null, $offset, $limit, wcmDatabase::FETCH_INTO, $this);
        if ($this->enumResultSet == null)
        {
            $this->lastErrorMsg = $this->database->getErrorMsg();
            return false;
        }
        return true;
    }

    /**
     * Moves to the next item of current enumeration
     *
     * @return bool True if enumeration has succeed, false otherwise
     */
    public function nextEnum()
    {
        if (null == $this->enumResultSet)
        {
            return false;
        }

        // Erase existing property
        $properties = getPublicProperties($this);
        foreach ($properties as $property => $value)
        {
            $this->$property = null;
        }

        if (!$this->enumResultSet->next())
        {
            // We have reach the end of the resultset
            $this->endEnum();
            return false;
        }

        // Bind current object to current resultset row
        $this->unserializeProperties();

        return true;
    }

    /**
     * Stops current enumeration
     */
    public function endEnum()
    {
        // Free resultset
        $this->enumResultSet = null;
        $this->enumQuery = null;
        $this->enumTotalCount = -1;
    }

    /**
     * Returns the total number of objects which can be enumerated (regardless of the
     * offset and limit parameters passed in the {@link beginEnum()} method)
     *
     * @return int Total number of enumerable objects
     */
    public function enumCount()
    {
        // compute enum count on demand and only once!
        if ($this->enumTotalCount == -1)
        {
            $this->enumTotalCount = $this->database->executeScalar($this->enumCountQuery);
        }

        return $this->enumTotalCount;
    }

    /**
     * Check is lock mode is optimistic (and not pessimistic)
     *
     * @return bool TRUE is lock mode is optimistic
     */
    public function isLockOptimistic()
    {
        return $this->isLockOptimistic;
    }

    /**
     * Lock current object
     *
     * @return bool TRUE if lock has been set, FALSE is object was locked by another user
     */
    public function lock()
    {
        // Check lock
        if (!$this->checkLock('lock'))
        {
            return false;
        }

        $this->editable = (wcmLock::lock($this) !== null);
        return $this->editable;
    }

    /**
     * Get current information on lock
     *
     * @param bool $refresh TRUE to refresh lock info (by default) or retrieve last info
     *
     * @return wcmLock Lock information for current sysobject
     */
    public function getLockInfo($refresh = false)
    {
        if ($refresh)
        {
            $this->lockInfo = wcmLock::get($this);
        }

        return $this->lockInfo;
    }

    /**
     * Unlock current object
     *
     * @return bool TRUE if lock has been removed, FALSE if object was locked by another user
     */
    public function unlock()
    {
        // Check lock
        if ($this->checkLock('unlock'))
        {
            $this->editable = -1;
            return wcmLock::unlock($this);
        }

        return false;
    }

    /**
     * Check if current object is editable from the point of view of the currently connected user.
     * An object is editable if one of the 3 conditions below are matched:
     * - Object allow optimistic lock
     * - Object is a new object
     * - Object is unlocked or locked by current user
     *
     * @return bool TRUE when object is editable by current user
     */
    public function isEditable()
    {
        if (!$this->id)
        {
            $this->lockInfo = new wcmLock($this);
            return true;
        }

        $this->getLockInfo(true);

        // First computation?
        if ($this->editable == -1)
        {
            if ($this->isLockOptimistic)
            {
                $this->editable = ($this->lockInfo->userId == 0 || $this->lockInfo->userId == wcmSession::getInstance()->userId);
            }
            else
            {
                $this->editable = ($this->lockInfo->userId == wcmSession::getInstance()->userId);
            }
        }

        return $this->editable;
    }

    /**
     * Check if object is locked from the point of view of the currently connected user.
     * For instance: if current user (from session) has locked this object, isLocked() will return FALSE
     * if another user has locked this object, isLocked() will return TRUE
     * if nobody has locked this object, isLocked() will return FALSE
     *
     * @return bool TRUE if current connected user cannot modify/delete the object
     */
    public function isLocked()
    {
        if (!$this->lockInfo)
        {
            $this->lockInfo = wcmLock::get($this);
        }
        return ($this->lockInfo->userId != 0 && $this->lockInfo->userId != wcmSession::getInstance()->userId);
    }

    /**
     * Check if object has a lock on it. This is different from isLocked() as hasLock() will return
     * true regardless of which session user is calling this method.
     *
     * @return bool True if object has a lock, false if object doesn't.
     */
    public function hasLock()
    {
        if (!$this->lockInfo)
        {
            $this->lockInfo = wcmLock::get($this);
        }
        return !($this->lockInfo->userId != 0);
    }

    /**
     * This function test is the connected user (from session) can modify/delete the current object
     * and update the lastErrorMessage property if object is locked.
     *
     * @param string $operation Name of operation to attempt (e.g. 'delete', 'save', ...)
     *
     * @return bool TRUE if connected user can alter the current object
     */
    protected function checkLock($operation)
    {
        if ($this->isLocked())
        {
            $this->lastErrorMsg = $this->getType() . '::' . $operation . ' failed : object ' . $this->id . ' is locked';
            return false;
        }

        return true;
    }

    /**
     * Factory class for creating object related to bizobject
     * Extension classes should be stored in business/extensions
     * and be named ext.<simpleClassname>.php
     *
     * @param string $type Type
     * @param array $args Arguments
     * @return object Created object
     */
    public static function create($class /*...*/)
    {
        $config = wcmConfig::getInstance();

        $args = func_get_args();

        if (!class_exists($class))
        {
            throw new Exception($class . ' does not exist');
        }

        $reflectionObj = new ReflectionClass($class);
        array_shift($args);

        if (method_exists($class, '__construct') && !empty($args))
        {
            $obj = call_user_func_array(array($reflectionObj, 'newInstance'), $args);
        }
        else
        {
            $obj = $reflectionObj->newInstance();
        }
        return $obj;
    }

    /**
     * Get the serialStorage array.
     *
     * @return array  The serialStorage array.
     */
    public function getSerialStorage()
    {
        return $this->serialStorage;
    }
}
