<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.workflowManager.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-system-api
 * @version     4.x
 */

/**
 * The wcmWorkflowManager class is an helper class used to manage workflows
 */
class wcmWorkflowManager
{
    /**
     * Returns the workflow corresponding to a specific id
     *
     * @param int $id Id of workflow to retrieve
     *
     * @return wcmWorkflow Corresponding workflow or NULL
     */
    public function getWorkflowById($id)
    {
        // Use cache
        return getArrayParameter($this->getWorkflows(), $id, null);
    }

    /**
     * Returns an array of workflows matching a specific where clause
     *
     * @param bool $resetCache Whether to reset the cache, ie. reload from DB (default is false)
     *
     * @return array An assoc array of {@link wcmWorkflows} objects (keys are ids)
     */
    public function getWorkflows($resetCache = false)
    {
        $cached = wcmCacheRegistry::get()->fetch('wcmWorkflow');
        if ($resetCache || $cached === false)
        {
            $cached = array();
            $enum = new wcmEnumeration('wcmWorkflow');

            foreach ($enum as $workflow)
            {
                $cached[$workflow->id] = $workflow;
            }

            // Update cache
            wcmCacheRegistry::get()->store('wcmWorkflow', $cached);
            wcmCacheRegistry::get()->delete('wcmWorkflowTransition');
        }
        return $cached;
    }


    /**
     * Returns an array of wcmWorkflowStates matching a specific where clause
     *
     * @param bool $resetCache Whether to reset the cache, ie. reload from DB (default is false)
     *
     * @return array An assoc array of {@link wcmWorkflowStates} objects (keys are ids)
     */
    public function getWorkflowStates($resetCache = false)
    {
        $cached = wcmCacheRegistry::get()->fetch('wcmWorkflowState');
        if ($resetCache || $cached === false)
        {
            $project = wcmProject::getInstance();
            $cached = array();

            $enum = new wcmEnumeration('wcmWorkflowState');
            foreach ($enum as $workflowState)
            {
                $cached[$workflowState->code] = $workflowState;
            }

            // Update cache
            wcmCacheRegistry::get()->store('wcmWorkflowState', $cached);
        }

        return $cached;
    }

    /**
     * Returns the workflow  state corresponding to a specific code
     *
     * @param string $code Code of workflow state to retrieve
     *
     * @return wcmWorkflowState Corresponding workflwo state or NULL
     */
    public function getWorkflowStateByCode($code)
    {
        // Use cache
        return getArrayParameter($this->getWorkflowStates(true), $code, null);
    }

    /**
     * Returns an array of wcmWorkflowTransitions matching a specific where clause
     *
     * @param bool $resetCache Whether to reset the cache, ie. reload from DB (default is false)
     *
     * @return array An assoc array of {@link wcmWorkflowTransitions} objects (keys are ids)
     */
    public function getTransitions($resetCache = false)
    {
        $cached = wcmCacheRegistry::get()->fetch('wcmWorkflowTransition');
        if ($resetCache || $cached === false)
        {
            $project = wcmProject::getInstance();
            $cached = array();

            $enum = new wcmEnumeration('wcmWorkflowTransition');
            foreach ($enum as $workflowTransition)
            {
                $cached[$workflowTransition->code] = $workflowTransition;
            }

            // Update cache
            wcmCacheRegistry::get()->store('wcmWorkflowTransition', $cached);
        }

        return $cached;
    }

    /**
     * Returns the workflow transition corresponding to a specific id
     *
     * @param int $id Id of workflow transition to retrieve
     *
     * @return wcmWorkflowTransition Corresponding workflow transition or NULL
     */
    public function getWorkflowTransitionById($id)
    {
        // Use cache (but cache is stored by code)
        foreach ($this->getTransitions() as $code => $transition)
        {
            if ($transition->id === $id)
            {
                return $transition;
            }
        }

        return null;
    }

    /**
     * Returns the workflow transition corresponding to a specific code
     *
     * @param int $code Code of workflow transition to retrieve
     *
     * @return wcmWorkflowTransition Corresponding workflow transition or NULL
     */
    public function getWorkflowTransitionByCode($code)
    {
        // Use cache
        return getArrayParameter($this->getTransitions(), $code);
    }


    /**
     * Recursive method to get all possible states for a specific bizclass
     *
     * @param   string  $className Class name to get workflows for
     * @param   string  $state From state
     *
     * @return  array   all workflow states (wcmWorkflowStates) for this bizClass
     */
    public function getWorkflowStatesByClassName($className, $state = null)
    {
        $project = wcmProject::getInstance();
        $wkfManager = $project->workflowManager;

        // Use static array to allow access during recursive calls
        static $statesArray;

        if (!$state)
        {
            $bizClass = $project->bizlogic->getBizclassByClassName($className);
            $workflow = $wkfManager->getWorkflowById($bizClass->workflowId);
            $state = $workflow->initialState;
            $statesArray = array();
        }

        $statesArray[$state] = $wkfManager->getWorkflowStateByCode($state);

        $enum = new wcmEnumeration('wcmWorkflowTransition', 'fromState="' . $state . '" OR fromState="_ANY_STATE"');
        foreach ($enum as $workflowTransition)
        {
            if (!isset($statesArray[$workflowTransition->toState]))
            {
                $toStateArray = $wkfManager->getWorkflowStatesByClassName($className, $workflowTransition->toState);
                $statesArray = array_merge($statesArray, $toStateArray);
            }
        }

        return $statesArray;
    }

    /**
     * Returns an array of wcmWorkflowScripts matching a specific where clause
     *
     * @param bool $resetCache Whether to reset the cache, ie. reload from DB (default is false)
     *
     * @return array An array of workflow scripts (file)
     */
    public function getWorkflowScripts($resetCache = false)
    {
        $workflowScripts = array();
        if ($resetCache || !isset($this->workflowScripts))
        {
            // Script location
            $path = WCM_BIZ_API_DIR . '/common/workflow/';
            $scripts = array();
            if (is_dir($path))
            {
                if ($dh = opendir($path))
                {
                    $col  = 0;
                    while (($file = readdir($dh)) !== false)
                    {
                        if (substr($file, -4) == '.php')
                        {
                            $workflowScripts[] = $file;
                        }
                    }
                    closedir($dh);
                }
            }
        }

        return $workflowScripts;
    }
    
    /**
     * Get the workflow script object for a given script name.
     * @param string $scriptName The script name.
     * @param wcmObject $object The object on which the script
     * will operate.
     * @return wcmIWorkflowScript
     * @exception Exception If the script file cannot be found, or if the
     * script class doesn't implement the wcmIWorkflowScript interface.
     */
    public function createWorkflowScript($scriptName, wcmObject $object)
    {
        // Compute the absolute filename of the script.
        $filename = WCM_BIZ_API_DIR . '/common/workflow/' . $scriptName;
        if (file_exists($filename))
        {
            require_once($filename);
        }
        else
        {
            // The script file doesn't exist.
            $error = 'WCM is not well configured: cannot find workflow script: ' . $filename;
            wcmProject::getInstance()->logger->logError($error);
            throw new Exception($error);
        }

        //  Load script (assuming workflow script class name equals to filename, without extension)
        $className = substr($scriptName, 0, strlen($scriptName) - 4);
        $script = new $className($object);
        if (!($script instanceOf wcmIWorkflowScript))
        {
            // The script class doesn't implement the wcmIWorkflowScript interface.
            $error = 'Invalid workflow script: "' . $filename . '" does not implement wcmIWorkflowScript';
            wcmProject::getInstance()->logger->logError($error);
            throw new Exception($error);
        }

        return $script;
    }
}
