<?php
/**
 * Project:     WCM
 * File:        $Id: session.class.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-backoffice
 * @version     4.x
 */

/**
 * Install Session
 *
 */
class wcmInstall_session
{
    /**
     * Singleton instance
     *
     * @var wcmInstall_session
     */
    static private $instance;

    /**
     * Previous step
     * @var int
     */
    private $previousStep;

    /**
     * Current step
     * @var int
     */
    private $currentStep = 0;

    /**
     * Configuration file location
     * @var string
     */
    private $installationConfigFile;

    /**
     * Default values
     * @var array
     */
    private $advDefaultValues = array();

    /**
     * Logfile
     * @var string
     */
    private $logFile;

    /**
     * Initiliazed?
     * @var bool
     */
    private $init = false;

    /**
     * Configuration
     * @var array
     */
    private $config = array();

    /**
     * List of error messsages shown at the top of the template
     * @var array
     */
    private $errors = array();

    /**
     * Instructions
     * @var array
     */
    private $instructions = array();

    /**
     * WCM configuration
     * @var array
     */
    private $wcmConfig = array();

    /**
     * Config file
     * @var string
     */
    private $confFile;

    /**
     * Constructor
     */
    private function __construct()
    {
    }

    /**
     * Get singleton instance of the session
     *
     * @return wcmInstall_session
     */
    public static function getInstance()
    {
        if (!isset($_SESSION['wcmInstall']['install']))
        {
            $_SESSION['wcmInstall']['install'] = new wcmInstall_session;
        }
        return $_SESSION['wcmInstall']['install'];
    }

    /**
     * Initiliaze configuration
     */
    public function initConfiguration()
    {
        $dom = new DOMDocument();
        $dom->preserveWhitespace = false;
        $dom->load($this->config['xmlConfigFile']);

        $xq = new DOMXpath($dom);

        $query = '//*[@desc]';
        $list = $xq->query($query);

        foreach ($list as $element)
        {
            $confItem['path'][] = $element->nodeName;
            $confItem['desc'] = $element->getAttribute('desc');
            $confItem['value'] = $element->nodeValue;
            $confItem['name'] = $element->nodeName;

            while ($element->parentNode)
            {
                if ($element->parentNode->nodeType != XML_DOCUMENT_NODE)
                {
                    $confItem['path'][] = $element->parentNode->nodeName;
                    $element = $element->parentNode;
                }
                else
                {
                    break;
                }
            }

            $confItem['path'] = join('_', array_reverse($confItem['path']));

            $this->wcmConfig[$confItem['path']] = $confItem;
            unset($confItem);
        }

        // Special changes for special conf items
        $this->addWcmConfigVar('config_wcm_cache_path', sys_get_temp_dir() . '/wcm/cache/');
    }

    /**
     * Initializiation
     * @param string $argConfFile Configuration file to use
     */
    public function init($argConfFile)
    {
        require_once($argConfFile);

        $this->config = $wcm;

        $this->errors = array();
        $this->instructions = array();

        // Setup the first two instructions that must happen before anything else
        $this->registerInstruction(new wcmInstallInstructions_cleanupInstall());
        $this->registerInstruction(new wcmInstallInstructions_configFile());
        $this->registerInstruction(new wcmInstallInstructions_dbSetup());
    }

    /**
     * Add a log message
     * @param string $argMsg The message
     */
    public function log($argMsg)
    {
        $msg = '[' . date('Y-m-d\TH:i:s') . ']';
        $msg .= ' ' . $argMsg . "\n";
        if ($fp = @fopen('install.log', 'a'))
        {
            fwrite($fp, $msg);
            fclose($fp);
        }
    }

    /**
     * Execute a installation step
     * @param string $argStep The step to execute
     * @throws Exception
     */
    public function step($argStep)
    {
        $stepPath = 'steps/' . $argStep . '.class.php';
        if (file_exists($stepPath))
        {
            require_once($stepPath);

            $className = 'wcmInstallSteps_' . $argStep;
            $step = new $className;

            if ($step->redirect())
            {
                header('Location: ' . $step->redirect());
                exit();
            }

            if ($step->output())
            {
                echo $step->output();
            }
        }
        else
        {
            throw new Exception('You have reached an invalid step');
        }
    }

    /**
     * Get configuration
     * @return array Configuration array
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Get WCM configuration
     * @return array WCM configuration array
     */
    public function getWcmConfig()
    {
        return $this->wcmConfig;
    }

    /**
     * Get WCM configuration value
     * @param string $argName Key name
     * @return mixed Key value, or null if $argName not found.
     */
    public function getWcmConfigValue($argName)
    {
        if (isset($this->wcmConfig[$argName]))
        {
            return $this->wcmConfig[$argName]['value'];
        }
        return null;
    }

    /**
     * Get all WCM configuration values
     * @return array WCM configuration array in format
     * key=>value.
     */
    public function getWcmConfigValues()
    {
        foreach ($this->wcmConfig as $name => $conf)
        {
            $return[$name] = $conf['value'];
        }

        return $return;
    }

    /**
     * Get WCM configuration var
     * @param string $argName Key name
     * @return array Key data, or null if $argName not found.
     */
    public function getWcmConfigVar($argName)
    {
        if (isset($this->wcmConfig[$argName]))
        {
            return $this->wcmConfig[$argName];
        }
        return null;
    }

    /**
     * Add WCM config var
     * @param string $argName Key name
     * @param mixed $argValue Key value
     * @param string $argDescription Key description
     */
    public function addWcmConfigVar($argName, $argValue, $argDescription = null)
    {
        // If the thing already exists, then don't override anything but the value
        if (isset($this->wcmConfig[$argName]))
        {
            $this->wcmConfig[$argName]['value'] = $argValue;
            $this->log('CHANGED configuration item: ' . $argName . ' with value: ' . $argValue);
            return;
        }

        $configItem = array();
        $configItem['name'] = $argName;
        $configItem['value'] = $argValue;
        $configItem['desc'] = $argDescription;

        $this->wcmConfig[$argName] = $configItem;
        $this->log('ADDED configuration item: ' . $argName . ' with value: ' . $argValue);
    }

    /**
     * Check if the instruction is registered
     * @param string $argClassName Instruction name
     * @return bool TRUE if registered
     */
    public function isRegisteredInstruction($argClassName)
    {
        return isset($this->instructions[$argClassName]);
    }

    /**
     * Add an error
     * @param Exception $e Error to add
     */
    public function error(Exception $e)
    {
        $this->errors[] = $e->getMessage();
        $this->step('error');
        $this->errors = array();
    }

    /**
     * Get an instruction data
     * @param string $argClassName Class name
     * @return wcmInstall_instruction Instruction object
     */
    public function getInstruction($argClassName)
    {
        return $this->instructions[$argClassName];
    }

    /**
     * Get all instructions
     * @return array Array of instructions
     */
    public function getInstructions()
    {
        return $this->instructions;
    }

    /**
     * Unregister an instruction
     * @param string $instruction Instruction class name
     */
    public function unregisterInstruction($instruction)
    {
        unset($this->instructions[$instruction]);
    }

    /**
     * Register an instruction
     * @param wcmInstall_instruction $instruction Instruction to register
     */
    public function registerInstruction(wcmInstall_instruction $instruction)
    {
        $this->instructions[get_class($instruction)] = $instruction;
    }

    /**
     * Get a list of errors
     * @return array Array of errors
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * Destructor
     */
    public function __destruct()
    {
        @fclose($this->logFile);
    }
}
