<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restContent.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmRestContent class provides methods to manipulate objects (CRUD operations) to various
 * controllers, namely wcmRestContentController.
 */
class wcmRestContent
{
    /**
     * Gets a specific object representation using the getObjectWithCallback method (which see) and
     * a predefined callback corresponding to the desired representation (as specified by
     * $params['json']).
     *
     * @param string $className [required] Object class name
     * @param int    $id        [required] Object ID
     * @param array  $params    [optional] Other request parameters (default is empty)
     *
     * @return string Object representation
     *
     * @throws wcmRestException on error
     */
    public static function getObject($className, $id, array $params = array())
    {
        // Determine which predefined callback to invoke based on the desired representation
        $json = !!isset($params['json']);
        $method = ($json ? 'objectAsJSON' : 'objectAsXML');
        $callback = array('wcmRestContent', $method);

        return self::getObjectWithCallback($callback, $className, $id, $params);
    }

    /**
     * Gets a list of objects matching given criteria using the getObjectsWithCallback method (which
     * see) and a predefined callback corresponding to the desired representation (as specified by
     * $params['json']).
     *
     * @param string $className [required] Object class name, or CSV of class names
     * @param array  $params    [optional] Other request parameters (default is empty)
     *
     * @return string Representation of object list
     *
     * @throws wcmRestException on error
     */
    public static function getObjects($className, array $params = array())
    {
        // Determine which predefined callback to invoke based on the desired representation
        $json = !!isset($params['json']);
        $method = ($json ? 'objectAsJSON' : 'objectAsXML');
        $callback = array('wcmRestContent', $method);

        return self::getObjectsWithCallback($callback, $className, $params);
    }

    /**
     * Gets a specific object using a callback to represent it.
     *
     * Invokes the callback with the following parameters:
     *
     * - wcmObject $object [required] Object to represent
     * - mixed     $fields [required] Fields to include in representation ("*", "@", or array of field names)
     * - array     $params [required] Other request parameters
     *
     * The callback is expected to return the string representation of the object as specified by
     * $params['json'], or throw a wcmRestException on error.
     *
     * The $params array may/must contain the following entries:
     *
     * - 'baseURL' => [required if $params['fields'] == '@'] base URL of object with trailing slash
     *                          up to, but not including, object ID
     *
     * - 'fields' => [optional] fields to return in object representation: "*" (all fields), "@"
     *                          (object location), or CSV of field names (default is "*")
     *
     * - 'json' => [optional] return JSON representation if present, otherwise XML
     *
     * - 'response' => [optional] return representation in a response object if present
     *
     * @param callback $callback  [required] Callback to invoke to represent object
     * @param string   $className [required] Object class name
     * @param int      $id        [required] Object ID
     * @param array    $params    [optional] Other request parameters (default is empty)
     *
     * @return mixed Object representation as string or in a wcmRestResponse instance
     *
     * @throws wcmRestException on error
     */
    public static function getObjectWithCallback($callback, $className, $id, array $params = array())
    {
        // Validate class name
        if (!$className || !autoloadManager::classExists($className))
        {
            throw new wcmRestException(wcmRestResponse::invalidParameter('className', $className));
        }

        // Refresh object
        $object = wcmObjectFactory::create($className);
        $object->refresh($id);
        if (!$object->id)
        {
            throw new wcmRestException(wcmRestResponse::invalidParameter('id', $id));
        }

        // Explode field list into array if possible
        $fields = getArrayParameter($params, 'fields', '*');
        if ($fields != '*' && $fields != '@')
        {
            $fields = explode(',', $fields);
        }

        // Invoke callback to get object representation
        $rep = call_user_func($callback, $object, $fields, $params);

        // Return representation in a response if desired
        if (isset($params['response']))
        {
            $body = null;
            $headers = array();
            if (isset($params['json']))
            {
                $body = $rep;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
            }
            else
            {
                $body = wcmRestResponse::XML_DECL . PHP_EOL . $rep;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;
            }
            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        return $rep;
    }

    /**
     * Gets a list of objects matching given criteria using a callback to represent each object.
     *
     * Invokes the callback for each object in the list with the following parameters:
     *
     * - wcmObject $object [required] Object to represent
     * - mixed     $fields [required] Fields to include in representation ("*", "@", or array of field names)
     * - array     $params [required] Other request parameters
     *
     * The callback is expected to return the string representation of the object as specified by
     * $params['json'], or throw a wcmRestException on error.
     *
     * The $className parameter may specify a single class name or a CSV of class names.
     *
     * The $params array may contain the following entries:
     *
     * - 'baseURL' => [required if $params['fields'] == '@'] base URL of object with trailing slash
     *                          up to, but not including, object ID
     *
     * - 'controller' => [required if $params['json'] is not present] REST controller name
     *
     * - 'fields' => [optional] fields to return in object representation: "*" (all fields), "@"
     *                          (object location), or CSV of field names (default is "*")
     *
     * - 'json' => [optional] return JSON representation if present, otherwise XML
     *
     * - 'limit' => [optional] maximum number of objects in list (default is 128)
     *
     * - 'offset' => [optional] zero-based offset of first object in list (default is 0)
     *
     * - 'resource' => [required if $params['json'] is not present] resource name, not necessarily
     *                           the same as the object class name
     *
     * - 'response' => [optional] return representation in a response object if present
     *
     * - 'sort' => [optional] SQL "ORDER BY" clause to sort list (default is null)
     *
     * - 'where' => [optional] SQL "WHERE" clause to select particular objects (default is null)
     *
     * @param callback $callback  [required] Callback to invoke to represent a given object
     * @param string   $className [required] Object class name, or CSV of class names
     * @param array    $params    [optional] Other request parameters (default is empty)
     *
     * @return mixed Representation of object list as string or wcmRestResponse instance
     *
     * @throws wcmRestException on error
     */
    public static function getObjectsWithCallback($callback, $className, array $params = array())
    {
        // Explode field list into array if possible
        $fields = getArrayParameter($params, 'fields', '*');
        if ($fields != '*' && $fields != '@')
        {
            $fields = explode(',', $fields);
        }

        // Get object selection criteria
        $where  = getArrayParameter($params, 'where', null);
        $sort   = getArrayParameter($params, 'sort', null);
        $offset = getArrayParameter($params, 'offset', 0);
        $limit  = getArrayParameter($params, 'limit', 128);

        // List header
        $json = !!isset($params['json']);
        if ($json)
        {
            $sep = ',';
            $rep = '[';
        }
        else
        {
            if (!isset($params[$paramName = 'controller']) ||
                !isset($params[$paramName = 'resource']))
            {
                throw new wcmRestException(wcmRestResponse::missingParameter($paramName));
            }
            if (!($controller = $paramValue = $params[$paramName = 'controller']) ||
                !($resource   = $paramValue = $params[$paramName = 'resource']))
            {
                throw new wcmRestException(wcmRestResponse::invalidParameter($paramName, $paramValue));
            }
            $sep  = PHP_EOL;
            $rep  = '<' . $controller . '>' . $sep;
            $rep .= '<' . $resource . '-list>' . $sep;
        }

        // Parameter $className may be a CSV of class names
        $classNameOrig = $className;
        $classNames = explode(',', $className);
        foreach ($classNames as $className)
        {
            // Validate class name
            if (!$className || !autoloadManager::classExists($className))
            {
                throw new wcmRestException(wcmRestResponse::invalidParameter('className', $classNameOrig));
            }

            // Begin object enumeration
            $object = wcmObjectFactory::create($className);
            if (!$object->beginEnum($where, $sort, $offset, $limit))
            {
                throw new wcmRestException(
                    new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CRITERIA));
            }

            // List body
            while ($object->nextEnum())
            {
                $rep .= call_user_func($callback, $object, $fields, $params) . $sep;
            }

            // End object enumeration
            $object->endEnum();
        }

        // List footer
        if ($json)
        {
            $rep = rtrim($rep, $sep);
            $rep .= ']';
        }
        else
        {
            $rep .= '</' . $resource . '-list>' . $sep;
            $rep .= '</' . $controller . '>' . $sep;
        }

        // Return representation in a response if desired
        if (isset($params['response']))
        {
            $body = null;
            $headers = array();
            if ($json)
            {
                $body = $rep;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_APPLICATION_JSON;
            }
            else
            {
                $body = wcmRestResponse::XML_DECL . $sep . $rep;
                $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;
            }

            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        return $rep;
    }

    /**
     * Callback that represents a given object as JSON, for use with the getObjectWithCallback and
     * getObjectsWithCallback methods (which see for information on the expected contents of the
     * $params parameter).
     *
     * @param wcmObject $object [required] Object to represent
     * @param mixed     $fields [required] Fields to include in representation ("*", "@", or list of field names)
     * @param array     $params [required] Other request parameters
     *
     * @return string Object JSON representation
     *
     * @throws wcmRestException on error
     */
    public static function objectAsJSON(wcmObject $object, $fields, array $params)
    {
        $rep = '';
        if ($fields == '@')
        {
            if (!isset($params['baseURL']))
            {
                throw new wcmRestException(wcmRestResponse::missingParameter('baseURL'));
            }
            if (!($baseURL = $params['baseURL']) || $baseURL[strlen($baseURL) - 1] != '/')
            {
                throw new wcmRestException(wcmRestResponse::invalidParameter('baseURL', $baseURL));
            }
            $location = sprintf('%s%s', $baseURL, $object->id);
            $rep .= '"' . addslashes($location) . '"';
        }
        elseif ($fields == '*')
        {
            $rep .= $object->toJSON();
        }
        else
        {
            $rep .= $object->toJSON($fields);
        }

        return $rep;
    }

    /**
     * Callback that represents a given object as XML, for use with the getObjectWithCallback and
     * getObjectsWithCallback methods (which see for information on the expected contents of the
     * $params parameter).
     *
     * @param wcmObject $object [required] Object to represent
     * @param mixed     $fields [required] Fields to include in representation ("*", "@", or array of field names)
     * @param array     $params [required] Other request parameters
     *
     * @return string Object XML representation
     *
     * @throws wcmRestException on error
     */
    public static function objectAsXML(wcmObject $object, $fields, array $params)
    {
        $rep = '';
        if ($fields == '@')
        {
            if (!isset($params['baseURL']))
            {
                throw new wcmRestException(wcmRestResponse::missingParameter('baseURL'));
            }
            if (!($baseURL = $params['baseURL']) || $baseURL[strlen($baseURL) - 1] != '/')
            {
                throw new wcmRestException(wcmRestResponse::invalidParameter('baseURL', $baseURL));
            }
            $location = sprintf('%s%s', $baseURL, $object->id);
            $rep .= '<location>' . wcmXML::xmlEncode($location) . '</location>';
        }
        elseif ($fields == '*')
        {
            $rep .= $object->toXML();
        }
        else
        {
            $rep .= $object->toXML($fields);
        }

        return $rep;
    }
}