<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restIndexController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * The class wcmRestIndexController provides a RESTful interface to
 * manage search indexes.
 */
class wcmRestIndexController extends wcmRestController
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('index');
    }

    /**
     * HTTP "GET" method.
     *
     * GET .../index/{CLASSNAME}
     *
     * {CLASSNAME} => (required) object class name
     *
     * Gets the number of indexed objects for a given class.
     *
     * @return wcmRestResponse => S_OK           on success
     *                         => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST  on missing object class
     *                         => S_NOT_FOUND    on invalid session ID
     *                         => S_NOT_FOUND    on invalid object class
     *                         => S_UNAUTHORIZED on get object count failure
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get required claas name
        $className = getArrayParameter($_GET, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Get search instance
        $search = wcmSearch::getInstance();
        if ($search === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                       wcmSearch::getGlobalErrorMsg());
        }

        // Index the corresponding objects
        $count = $search->getObjectCount($className);
        if ($count === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());
        }

        if (isset($_GET['json']))
        {
            $body = json_encode(array('count' => $count));
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body  = wcmRestResponse::XML_DECL . PHP_EOL;
            $body .= '<index>' . PHP_EOL;
            $body .= '<className>' . $className . '</className>' . PHP_EOL;
            $body .= '<objectCount>' . $count . '</objectCount>' . PHP_EOL;
            $body .= '</index>' . PHP_EOL;
            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "POST" method.
     *
     * POST .../index/{CLASSNAME}?where={WHERE}&orderby={ORDERBY}&offset={OFFSET}&limit={LIMIT}
     * POST .../index/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (optional) object ID
     * {WHERE}     => (optional) SQL WHERE clause (default is null)
     * {ORDERBY}   => (optional) SQL ORDER BY clause (default is null)
     * {OFFSET}    => (optional) Start offset (default is 0)
     * {LIMIT}     => {optional} Limit on number of objects (default is 128)
     *
     * Indexes all objects of a given class matching given criteria,
     * or indexes a specific object if an object ID is given.
     *
     * @return wcmRestResponse => S_CREATED      on success
     *                         => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND    on invalid session ID
     *                         => S_NOT_FOUND    on invalid object class or ID
     *                         => S_UNAUTHORIZED on indexing failure
     */
    protected function post()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get required claas name
        $className = getArrayParameter($_GET, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Get search instance
        $search = wcmSearch::getInstance();
        if ($search === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                       wcmSearch::getGlobalErrorMsg());
        }

        // If an ID was given ...
        $id = getArrayParameter($_GET, 'id', null);
        if ($id)
        {
            // Instantiate the object
            $object = wcmObjectFactory::create($className);
            $object->refresh($id);
            if (!$object->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Index the object
            if (!$search->indexObject($object))
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());
            }
        }
        else
        {
            // Get the optional criteria
            $where   = getArrayParameter($_GET, 'where', null);
            $orderby = getArrayParameter($_GET, 'orderby', null);
            $offset  = getArrayParameter($_GET, 'offset', 0);
            $limit   = getArrayParameter($_GET, 'limit', 128);

            // Index the corresponding objects
            if (!$search->indexObjects($className, $where, $orderby, $offset, $limit))
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());
            }
        }

        return new wcmRestResponse(wcmRestResponse::S_CREATED);
    }

    /**
     * HTTP "PUT" method.
     *
     * Currently does the same thing as the HTTP "POST" method.
     *
     * @return wcmRestResponse => S_CREATED      on success
     *                         => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND    on invalid session ID
     *                         => S_NOT_FOUND    on invalid object class or ID
     *                         => S_UNAUTHORIZED on indexing failure
     */
    protected function put()
    {
        return $this->post();
    }

    /**
     * HTTP "DELETE" method.
     *
     * DELETE .../index/{CLASSNAME}
     * DELETE .../index/{CLASSNAME}/{ID}
     *
     * {CLASSNAME} => (required) object class name
     * {ID}        => (optional) object ID
     *
     * Deindexes all objects of a given class, or deindexes a specific
     * object if an object ID is given.
     *
     * @return wcmRestResponse => S_NO_CONTENT   on success
     *                         => S_BAD_REQUEST  on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST  on missing object class
     *                         => S_NOT_FOUND    on invalid session ID
     *                         => S_NOT_FOUND    on invalid object class or ID
     *                         => S_UNAUTHORIZED on deindexing failure
     */
    protected function delete()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Get required claas name
        $className = getArrayParameter($_REQUEST, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Get search instance
        $search = wcmSearch::getInstance();
        if ($search === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR,
                                       wcmSearch::getGlobalErrorMsg());
        }

        // If an ID was given ...
        $id = getArrayParameter($_REQUEST, 'id', null);
        if ($id)
        {
            // Instantiate the object
            $object = wcmObjectFactory::create($className);
            $object->refresh($id);
            if (!$object->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Deindex the object
            if (!$search->deindexObject($object))
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());
            }
        }
        else
        {
            // Deindex the objects of the given class
            if (!$search->deindexObjects($className))
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED, $search->getErrorMsg());
            }
        }

        return new wcmRestResponse(wcmRestResponse::S_NO_CONTENT);
    }
}