<?php
/**
 * Project:     WCM
 * File:        $Id: wcm.restWorkflowController.php 9341 2010-03-26 19:47:48Z charronp $
 *
 * @copyright   (c)2010 Nstein Technologies
 * @package     wcm-www-restws
 * @version     4.x
 */

/**
 * Thw wcmRestWorkflowController class provides a RESTful interface to
 * manage workflows.
 */
class wcmRestWorkflowController extends wcmRestController
{
    /**
     * Workflow manager.
     *
     * @var wcmWorkflowManager
     */
    private $workflowManager;

    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct('workflow');

        $this->workflowManager = wcmProject::getInstance()->workflowManager;
    }

    /**
     * Gets the public URL for a workflow resource.
     *
     * @param mixed       $id       The resource ID, code, etc.
     * @param string|null $resource The resource type (default is null for "workflow")
     *
     * @return string The public URL
     */
    private function getWorkflowURL($id, $resource = null)
    {
        if ($resource)
        {
            return sprintf('%s%s/%s', $this->baseURL, $resource, $id);
        }

        return sprintf('%s%s', $this->baseURL, $id);
    }

    /**
     * HTTP "GET" method.
     *
     * @see getWorkflow
     * @see getState
     * @see getTransition
     * @see getScript
     *
     * @return wcmRestResponse => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid resource
     *                         => responses from get{Workflow|State|Transaction|Script}
     */
    protected function get()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Determine method to call based on resource
        $resource = getArrayParameter($_GET, 'resource', 'workflow');
        $method = 'get' . ucfirst($resource);
        if (!method_exists($this, $method))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_RESOURCE);
        }

        // Handle resource
        return $this->$method();
    }

    /**
     * HTTP "GET" method for "workflow" resource.
     *
     * GET .../workflow
     * GET .../workflow/{ID}
     *
     * {ID} => (optional) Workflow ID
     *
     * Gets a specific workflow, or a list of workflows if {ID} is not
     * given.
     *
     * @return wcmRestResponse => S_OK        on success
     *                         => S_NOT_FOUND on invalid workflow ID
     */
    private function getWorkflow()
    {
        // Return one or many objects?
        $id = getArrayParameter($_GET, 'id', null);
        if ($id)
        {
            $workflow = $this->workflowManager->getWorkflowById($id);
            if (!$workflow)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_WORKFLOW_ID);
            }

            // Response is workflow as JSON or XML
            if (isset($_REQUEST['json']))
            {
                $body = $workflow->toJSON();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_APPLICATION_JSON);
            }
            else
            {
                $body = $workflow->toXML();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_TEXT_XML);
            }

            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        // Response is collection of workflow locations as JSON or XML
        $workflows = $this->workflowManager->getWorkflows();
        if (isset($_REQUEST['json']))
        {
            $output = array();
            foreach ($workflows as $id => $workflow)
            {
                $output[$id] = $this->getWorkflowURL($id) . '?json';
            }

            $body = json_encode($output);
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $output = '<?xml version="1.0" encoding="utf-8"?>' . PHP_EOL;
            $output .= '<workflows>' . PHP_EOL;
            foreach ($workflows as $id => $workflow)
            {
                $output .= '<location>'
                    . $this->getWorkflowURL($id)
                    . '</location>' . PHP_EOL;
            }
            $output .= '</workflows>' . PHP_EOL;

            $body = $output;
            $headers[wcmRestResponse::H_CONTENT_TYPE] = wcmRestResponse::H_CTCS_TEXT_XML;
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "GET" method for "state" resource.
     *
     * GET .../workflow/state
     * GET .../workflow/state/{CODE}
     * GET .../workflow/state/{CLASSNAME}
     *
     * {CODE}      => (optional) Workflow state code
     * {CLASSNAME} => (optional) Object class name
     *
     * Gets a specific workflow state if {CODE} is given, a list of
     * workflow states otherwise. If {CLASS} is given, the list of
     * workflows is specific to the given object class.
     *
     * @return wcmRestResponse => S_BAD_REQUEST on conflicting parameters
     *                         => S_NOT_FOUND    on invalid workflow state code or object class name
     *                         => S_UNAUTHORIZED on get workflow states failure
     */
    private function getState()
    {
        // Complain if both a workflow stats code and an object class name were given
        $code = getArrayParameter($_GET, 'code', null);
        $className = getArrayParameter($_GET, 'className', null);
        if ($code && $className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_CONFLICTING_PARAMETERS);
        }

        // Return one or many workflow states?
        if ($code)
        {
            // Get workflow state by code
            $workflowState = $this->workflowManager->getWorkflowStateByCode($code);
            if (!$workflowState)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND,
                                           _REST_INVALID_WORKFLOW_STATE_CODE);
            }

            // Response is workflow state as JSON or XML
            if (isset($_REQUEST['json']))
            {
                $body = $workflowState->toJSON();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_APPLICATION_JSON);
            }
            else
            {
                $body = $workflowState->toXML();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
            }

            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        // Return all or some workflow states?
        $workflowStates = null;
        if ($className)
        {
            // Validate object class name
            if (!autoloadManager::classExists($className))
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
            }

            // Get workflow states by object class name
            $workflowStates = $this->workflowManager->getWorkflowStatesByClassName($className);
            if ($workflowStates === null)
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED);
            }
        }
        else
        {
            // Get all workflow states
            $workflowStates = $this->workflowManager->getWorkflowStates();
            if ($workflowStates === null)
            {
                return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED);
            }
        }

        // Response is set of workflow state locations as JSON or XML
        $locations = array();
        foreach ($workflowStates as $id => $workflowState)
        {
            $locations[$id] = $this->getWorkflowURL($id, 'state');
        }

        if (isset($_REQUEST['json']))
        {
            $body = json_encode($locations);
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body  = '<?xml version="1.0" encoding="utf-8"?>' . PHP_EOL;
            $body .= '<workflow>' . PHP_EOL;
            $body .= '<states>' . PHP_EOL;
            foreach ($locations as $location)
            {
                $body .= '<location>'
                    . $location
                    . '</location>' . PHP_EOL;
            }
            $body .= '</states>' . PHP_EOL;
            $body .= '</workflow>' . PHP_EOL;

            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "GET" method for "transition" resource.
     *
     * GET .../workflow/transition?className={CLASSNAME}&objectId={OBJECT-ID}
     * GET .../workflow/transition/{ID}
     * GET .../workflow/transition/{CODE}
     *
     * {ID}        => (optional) Workflow transition ID
     * {CODE}      => (optional) Workflow transition code
     * {CLASSNAME} => (optional) Object class name
     * {OBJECT-ID} => (optional) Object ID
     *
     * Gets a specific workflow transition, or a list of workflow
     * transitions if neither {ID} nor {CODE} are given.
     *
     * If both {CLASSNAME} and {OBJECT-ID} are given, returns the list
     * of available transitions for that object.
     *
     * @return wcmRestResponse => S_BAD_REQUEST  on conflicting parameters
     *                         => S_NOT_FOUND    on invalid workflow transition ID or code
     *                         => S_UNAUTHORIZED on get workflow transitions failure
     */
    private function getTransition()
    {
        // Complain if both a workflow transition ID and code were given
        $id = getArrayParameter($_GET, 'id', null);
        $code = getArrayParameter($_GET, 'code', null);
        if ($id && $code)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_CONFLICTING_PARAMETERS);
        }

        // Return one or many workflow transitions?
        if ($id || $code)
        {
            if ($id)
            {
                // Get workflow transition by ID
                $transition = $this->workflowManager->getWorkflowTransitionById($id);
                if (!$transition)
                {
                    return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND,
                                               _REST_INVALID_WORKFLOW_TRANSITION_ID);
                }
            }
            else
            {
                // Get workflow transition by code
                $transition = $this->workflowManager->getWorkflowTransitionByCode($code);
                if (!$transition)
                {
                    return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND,
                                               _REST_INVALID_WORKFLOW_TRANSITION_CODE);
                }
            }

            // Response is workflow transition as JSON or XML
            if (isset($_REQUEST['json']))
            {
                $body = $transition->toJSON();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                                 wcmRestResponse::H_CTCS_APPLICATION_JSON);
            }
            else
            {
                $body = $transition->toXML();
                $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
            }

            return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
        }

        // Return all or some workflow transitions?
        $className = getArrayParameter($_GET, 'className', null);
        $objectId  = getArrayParameter($_GET, 'objectId', null);
        if ($className && $objectId)
        {
            // Validate object class name
            if (!$className)
            {
                return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
            }
            if (!autoloadManager::classExists($className))
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
            }

            // Refresh object
            $object = wcmObjectFactory::create($className);
            $object->refresh($objectId);
            if (!$object->id)
            {
                return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
            }

            // Return transitions for specified object
            $transitions = $object->getAvailableTransitions();
        }
        else
        {
            // Return all transitions
            $transitions = $this->workflowManager->getTransitions();
        }
        if ($transitions === null)
        {
            return new wcmRestResponse(wcmRestResponse::S_UNAUTHORIZED);
        }

        // Response is set of workflow transition locations as JSON or XML
        $locations = array();
        foreach ($transitions as $id => $transition)
        {
            $locations[$id] = $this->getWorkflowURL($id, 'transition');
        }

        if (isset($_REQUEST['json']))
        {
            $body = json_encode($locations);
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body  = '<?xml version="1.0" encoding="utf-8"?>' . PHP_EOL;
            $body .= '<workflow>' . PHP_EOL;
            $body .= '<transitions>' . PHP_EOL;
            foreach ($locations as $location)
            {
                $body .= '<location>'
                    . $location
                    . '</location>' . PHP_EOL;
            }
            $body .= '</transitions>' . PHP_EOL;
            $body .= '</workflow>' . PHP_EOL;

            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "GET" method for "script" resource.
     *
     * Gets a list of workflow scripts.
     *
     * RESOURCE:
     *
     * workflow/script
     *
     * QUERY PARAMETERS:
     *
     * json => Return JSON representation if present, XML otherwise
     *
     * @return wcmRestResponse => S_OK on success
     */
    private function getScript()
    {
        // Get workflow scripts if any
        $scripts = $this->workflowManager->getWorkflowScripts();

        // Response is set of workflow scripts as JSON or XML
        if (isset($_REQUEST['json']))
        {
            $body = json_encode((array)$scripts);
            $headers = array(wcmRestResponse::H_CONTENT_TYPE =>
                             wcmRestResponse::H_CTCS_APPLICATION_JSON);
        }
        else
        {
            $body  = '<?xml version="1.0" encoding="utf-8"?>' . PHP_EOL;
            $body .= '<workflow>' . PHP_EOL;
            $body .= '<scripts>' . PHP_EOL;
            if ($scripts)
            {
                foreach ($scripts as $script)
                {
                    $body .= '<script>'
                        . $script
                        . '</script>' . PHP_EOL;
                }
            }
            $body .= '</scripts>' . PHP_EOL;
            $body .= '</workflow>' . PHP_EOL;

            $headers = array(wcmRestResponse::H_CONTENT_TYPE => wcmRestResponse::H_CTCS_TEXT_XML);
        }

        return new wcmRestResponse(wcmRestResponse::S_OK, $body, $headers);
    }

    /**
     * HTTP "PUT" method.
     *
     * Currently does exactly the same thing as the HTTP "POST"
     * method.
     *
     * @return wcmRestResponse => S_CREATED     on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on conflicting parameters
     *                         => S_BAD_REQUEST on missing object class name or ID
     *                         => S_BAD_REQUEST on missing transition ID or code
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid object class name or ID
     *                         => S_INTERNAL_SERVER_ERROR on transition execution failure
     *
     * @see wcmRestWorkflowController::post()
     */
    protected function put()
    {
        // TODO - remove this as PUT should be for resource update
        return $this->post();
    }

    /**
     * HTTP "POST" method.
     *
     * Executes a transition on a given object.
     *
     * Either the transition ID or the transition code must be
     * specified.
     *
     * RESOURCE:
     *
     * workflow/transition
     *
     * QUERY PARAMETERS:
     *
     * className => Object class name
     * objectId  => Object ID
     * id        => Transition ID (default is null)
     * code      => Transition code (default is null)
     *
     * @return wcmRestResponse => S_CREATED     on success
     *                         => S_BAD_REQUEST on missing, corrupt, or mismatched session ID
     *                         => S_BAD_REQUEST on conflicting parameters
     *                         => S_BAD_REQUEST on missing object class name or ID
     *                         => S_BAD_REQUEST on missing transition ID or code
     *                         => S_NOT_FOUND   on invalid session ID
     *                         => S_NOT_FOUND   on invalid object class name or ID
     *                         => S_INTERNAL_SERVER_ERROR on transition execution failure
     */
    protected function post()
    {
        // Initialize / validate session
        $response = $this->initSession();
        if ($response !== null)
        {
            return $response;
        }

        // Validate object class name
        $className = getArrayParameter($_REQUEST, 'className', null);
        if (!$className)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_CLASS);
        }
        if (!autoloadManager::classExists($className))
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_CLASS);
        }

        // Validate object ID
        $objectId = getArrayParameter($_REQUEST, 'objectId', null);
        if (!$objectId)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_INVALID_OBJECT_ID);
        }

        // Refresh object
        $wcmObject = wcmObjectFactory::create($className);
        $wcmObject->refresh($objectId);
        if (!$wcmObject->id)
        {
            return new wcmRestResponse(wcmRestResponse::S_NOT_FOUND, _REST_INVALID_OBJECT_ID);
        }

        // Complain if both a workflow transition ID and code were given
        $id = getArrayParameter($_REQUEST, 'id', null);
        $code = getArrayParameter($_REQUEST, 'code', null);
        if ($id && $code)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST, _REST_CONFLICTING_PARAMETERS);
        }
        if (!$id && !$code)
        {
            return new wcmRestResponse(wcmRestResponse::S_BAD_REQUEST,
                                       _REST_INVALID_WORKFLOW_TRANSITION_ID);
        }

        // Get the workflow transition
        if ($id)
        {
            $transition = $this->workflowManager->getWorkflowTransitionById($id);
        }
        else
        {
            $transition = $this->workflowManager->getWorkflowTransitionByCode($code);
        }

        // Execute transiition
        if (!$wcmObject->executeTransition($transition))
        {
            $message = sprintf('%s: %s', _REST_EXECUTE_TRANSITION_FAILED, $wcmObject->getErrorMsg());
            return new wcmRestResponse(wcmRestResponse::S_INTERNAL_SERVER_ERROR, $message);
        }

        return new wcmRestResponse(wcmRestResponse::S_CREATED);
    }
}
